/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVSender;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zum Testen der AQ-Befehle
 * 
 * @author Pittner
 * 
 */
public class SimulationAnzeigeinhaltSollA14 implements StandardApplication, ClientReceiverInterface,
    ClientSenderInterface
{

  /**
   * DebugLogger fr Debug-Ausgaben
   */
  private static Debug _debug;

  /**
   * Verbindung zum Datenverteiler
   */
  public ClientDavInterface _connection;

  private String _dav;

  /**
   * Konstruktor.<br>
   * ffnen der Log-Datei.
   */
  public SimulationAnzeigeinhaltSollA14()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  /**
   * berschriebene Methode von StandardApplication, die die speziellen Startparameter auswertet.<br>
   * Die Liste der Konfigurationsbereiche wird durch Aufspaltung des bergebenen Strings erstellt und die speziellen
   * Startparameter werden in die Log-Datei eingetragen.
   * 
   * @param argumentList
   *          siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList) throws Exception
  {
    _debug = Debug.getLogger();

    _debug.config("argumentList = " + argumentList);

    argumentList.fetchUnusedArguments();
  }

  /*
   * (non-Javadoc)
   * 
   * @see sys.funclib.application.StandardApplication#initialize(stauma.dav.clientside.ClientDavInterface)
   */
  /**
   * berschriebene Methode von StandardApplication, die die Initialisierung durchfhrt.<br>
   * Entsprechend dem Argument -layer wird die entsprechende Methode aufgerufen und danach die Log-Datei geschlossen.<br>
   * 
   * @param connection
   *          siehe sys.funclib.application.StandardApplication#initialize(stauma.dav.clientside.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection) throws Exception
  {

    _connection = connection;

    new Thread() {

      @Override
      public void run()
      {
        // TODO Auto-generated method stub
        List<SystemObject> anzeigen = _connection.getDataModel().getType("typ.anzeige").getElements();
        AttributeGroup atgCo = _connection.getDataModel().getAttributeGroup("atg.anzeige");

        Map<String, List<SystemObject>> map = new HashMap<String, List<SystemObject>>();

        for (SystemObject anzeige : anzeigen)
        {
          Data dataCo = anzeige.getConfigurationData(atgCo);

          SystemObject so = dataCo.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();

          if (so.getPid().equals("anzeigeTyp.uz.a14.TypA.1.links"))
          {
            List<SystemObject> list = new ArrayList<SystemObject>();

            SystemObject so1 = null;
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.Dunkel");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst130");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst120");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst100");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst80");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst60");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst50");
            list.add(so1);

            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst130");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst120");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst100");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst80");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst60");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst50");
            list.add(so1);

            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsStau");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeAllerStreckenverbote");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsFahrstreifenWechselnNachRechts");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsFahrstreifenGesperrt");
            list.add(so1);

            map.put(so.getPid(), list);
          }
          else if (so.getPid().equals("anzeigeTyp.uz.a14.TypA.1.rechts"))
          {
            List<SystemObject> list = new ArrayList<SystemObject>();

            SystemObject so1 = null;
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.Dunkel");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst130");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst120");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst100");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst80");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst60");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsZulaessigeHoechst50");
            list.add(so1);

            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst130");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst120");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst100");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst80");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst60");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeZulaessigeHoechst50");
            list.add(so1);

            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsStau");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeAllerStreckenverbote");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsFahrstreifenWechselnNachLinks");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsFahrstreifenGesperrt");
            list.add(so1);

            map.put(so.getPid(), list);
          }
          else if (so.getPid().equals("anzeigeTyp.uz.a14.TypB.1"))
          {
            List<SystemObject> list = new ArrayList<SystemObject>();

            SystemObject so1 = null;
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.Dunkel");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsUeberholverbotAllgemein");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeUeberholverbotAllgemein");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsUeberholverbotLkw");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsEndeUeberholverbotLkw");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsGefahrenstelle");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsBaustelle");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsStau");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsGlaettegefahr");
            list.add(so1);

            map.put(so.getPid(), list);
          }
          else if (so.getPid().equals("anzeigeTyp.uz.a14.TypC.1"))
          {
            List<SystemObject> list = new ArrayList<SystemObject>();

            SystemObject so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.Dunkel");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsNebel");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsUnfall");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls7Komma5Tonnen");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls500Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls1000Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls1500Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls2000Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls2500Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsAuf500Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsAuf1000Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsAuf1500Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsAuf2000Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsAuf2500Meter");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsNaesse");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls-asGlaette");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tls-asOelspur");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsStaugefahr");
            list.add(so1);
            so1 = _connection.getDataModel().getObject("wvzInhalt.uz.a14.tlsSicht");
            list.add(so1);

            map.put(so.getPid(), list);
          }
          else
          {
            continue;
          }
        }

        final AttributeGroup atg1 = _connection.getDataModel().getAttributeGroup("atg.anzeigeEigenschaftSoll");
        final Aspect asp1 = _connection.getDataModel().getAspect("asp.aqVorgabe");
        final Data data1 = _connection.createData(atg1);

        data1.getItem("Eigenschaft").getItem("Blinken").asTextValue().setText("Ja");
        data1.getItem("Eigenschaft").getItem("Blinktakt").asTextValue().setText("0,000");
        data1.getItem("Eigenschaft").getItem("Passiviert").asTextValue().setText("Ja");
        data1.getItem("Eigenschaft").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue()
            .setText("Linksbndig");
        
        data1.getItem("SollZeitpunkt").asTimeValue().setMillis(new Date().getTime());

        DaVSender sender = new DaVSender(_connection, SenderRole.source());

        while (true)
        {

          for (SystemObject anzeige : anzeigen)
          {
            Data dataCo = anzeige.getConfigurationData(atgCo);
            SystemObject so = dataCo.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();

            List<SystemObject> list = map.get(so.getPid());

            if (list == null)
              continue;

            int i = new Random().nextInt(list.size());

            SystemObject inhalt = list.get(i);
            
            data1.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(inhalt);

            sender.sende(anzeige, atg1, asp1, data1, (short)0);
          }

          try
          {
            this.sleep(10 * 1000);
          }
          catch (InterruptedException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }

        }
      }

    }.start();

  }// public void initialize(ClientDavInterface connection)

  /**
   * Programmeinstieg.<br>
   * 
   * @param arguments
   *          Kommandozeilenargumente
   */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new SimulationAnzeigeinhaltSollA14(), arguments);

  }

  @Override
  public void update(ResultData[] results)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
  {
    // TODO Auto-generated method stub
    return false;
  }

  /**
   * Zuflliges System-Objekt vom gewnschten Typ erzeugen
   * 
   * @param type
   *          Der gewnschte Typ
   * 
   * @return Zufllig erzeugtes System-Objekt
   * 
   * @throws Exception
   *           wenn Kommunikation mit der Konfiguration nicht klappt oder ein anderes Problem auftritt
   */
  private SystemObject getRandomSystemObjectOfType(SystemObjectType type) throws Exception
  {
    List elements = (List)type.getElements();
    if (elements.size() == 0)
    {
      return null;
    }
    else
    {
      return (SystemObject)elements.get((int)(Math.random() * elements.size()));
    }
  }

}
