package de.bsvrz.buv.plugin.anlagendarstellung.dav;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.PixelGrabber;
import java.awt.image.RescaleOp;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;

/**
 * Tools zum Umgang mit den Images.
 * 
 * @author HofmannJ
 * 
 */
public class ImageUtils
{

	/**
	 * Ldt ein {@link Image} aus den Ressourcen des Eclipse-Bundles.
	 * 
	 * @param resPath
	 * @return
	 */
	public static Image readImageFromBundle(String resPath)
	{
		Image re = null;

		if ((Activator.getDefault() != null)
				&& (Activator.getDefault().getBundle() != null))
		{
			URL resource = Activator.getDefault().getBundle().getResource(resPath);
			if (resource != null)
			{
				ImageIcon ic = new ImageIcon(resource);
				re = ic.getImage();
			}
		}
		return re;
	}

	public static Image readImageFromFile(String path)
	{
		Image img = null;

		if (path != null)
		{
			File file = new File(path);
			if ((file != null) && (file.exists()) && (file.isFile()))
			{
				ImageIcon ic = new ImageIcon(path);
				img = ic.getImage();
			}
		}

		return img;
	}

	/**
	 * Liefert die Bytes eines {@link Image} im PNG-Format in einem byte-Array.
	 * 
	 * @param image
	 *          {@link Image}.
	 * @return byte-Array mit dem {@link Image} im PNG-Format. Ist
	 *         <code>null</code>, falls es nicht erzeugt werden konnte.
	 */
	public static byte[] swingImageToBytes(Image image)
	{
		byte[] re = null;

		if (image != null)
		{
			BufferedImage bi = toBufferedImage(image);
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			try
			{
				if (ImageIO.write(bi, "png", baos))
				{
					re = baos.toByteArray();
				}
			} catch (IOException e)
			{
				e.printStackTrace();
			}

			if (baos != null)
			{
				try
				{
					baos.close();
				} catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}

		return re;
	}

	/**
	 * Baut ein SWT-Image aus den Daten in einem Byte-Array auf.
	 * 
	 * @param ba
	 *          Daten des SWT-Images.
	 * @return SWT-Image oder <code>null</code>, falls aus den Daten kein
	 *         SWT-Image erzeugbar ist.
	 */
	public static org.eclipse.swt.graphics.Image bytesToSwtImage(byte[] ba)
	{
		org.eclipse.swt.graphics.Image img = null;

		if ((ba != null) && (ba.length > 0))
		{
			ByteArrayInputStream bis = new ByteArrayInputStream(ba);
			img = new org.eclipse.swt.graphics.Image(Display.getDefault(), bis);
			try
			{
				bis.close();
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}

		return img;
	}

	/**
	 * Liefert ein Swing-Image als SWT-Image.
	 * 
	 * @param swing
	 *          Swing Image
	 * @return SWT-Image oder <code>null</code>, falls aus dem Swing-Image kein
	 *         SWT-Image erzeugte werden konnte.
	 */
	public static org.eclipse.swt.graphics.Image swingToSwtImage(Image swing)
	{
		org.eclipse.swt.graphics.Image re = null;

		if (swing != null)
		{
			byte[] png = swingImageToBytes(swing);
			re = bytesToSwtImage(png);
		}

		return re;
	}

	public static Image rotateImage(Image img, double degree)
	{
		BufferedImage bufImg = toBufferedImage(img);
		double angle = Math.toRadians(degree);

		return tilt(bufImg, angle);
	}

	public static BufferedImage tilt(BufferedImage image, double angle)
	{
		double sin = Math.abs(Math.sin(angle)), cos = Math.abs(Math.cos(angle));
		int w = image.getWidth(), h = image.getHeight();
		int neww = (int) Math.floor(w * cos + h * sin), newh = (int) Math.floor(h
				* cos + w * sin);
		GraphicsConfiguration gc = getDefaultConfiguration();
		BufferedImage result = gc.createCompatibleImage(neww, newh,
				Transparency.TRANSLUCENT);
		Graphics2D g = result.createGraphics();
		g.translate((neww - w) / 2, (newh - h) / 2);
		g.rotate(angle, w / 2, h / 2);
		g.drawRenderedImage(image, null);
		g.dispose();
		return result;
	}

	/**
	 * Korrigiert die Helligkeit des Images fr die blinkende Darstellung.
	 * 
	 * @param img
	 *          Zu modifizierendes Image.
	 * @return Modifiziertes Image oder <code>null</code>, falls das Image nicht
	 *         modifiziert werden konnte.
	 */
	public static Image brightenSwingImage1(Image img)
	{
		Image re = null;

		if (img != null)
		{
			BufferedImage bi = toBufferedImage(img);

			BufferedImage neu = toBufferedImage(img);
			Graphics2D g = neu.createGraphics();

			g.setColor(Color.BLACK);
			g.fillRect(0, 0, bi.getWidth(), bi.getHeight());

			for (int x = 0; x < bi.getWidth(); x++)
			{
				for (int y = 0; y < bi.getHeight(); y++)
				{
					Color col = null;

					int red = (new Color(bi.getRGB(x, y))).getRed();
					int green = (new Color(bi.getRGB(x, y))).getGreen();
					int blue = (new Color(bi.getRGB(x, y))).getBlue();
					int alpha = (new Color(bi.getRGB(x, y))).getAlpha();

					/*
					 * Wenn nicht schwarz, den Alpha-Kanal modifizieren.
					 */
					if ((red != 0) || (green != 0) || (blue != 0))
					{
						col = new Color(red, green, blue, 20);
					}

					if (col != null)
					{
						g.setColor(col);
						g.drawRect(x, y, 1, 1);
					}
				}
			}

			re = neu;

		}

		return re;
	}
	
	public static Image brightenSwingImage(Image img)
	{
		Image re = null;

		if (img != null)
		{
			BufferedImage bi = toBufferedImage(img);

			BufferedImage neu = toBufferedImage(img);
			Graphics2D g = neu.createGraphics();

			Color colorQuick = new Color(0, 0, 0, 128);
			g.setColor(colorQuick);
			g.fillRect(0, 0, bi.getWidth(), bi.getHeight());

			re = neu;

		}

		return re;
	}


	/**
	 * Erhht die Helligkeit eines Images.
	 * 
	 * @param bundlePath
	 *          Pfad zum Image.
	 * @return Helleres Image.
	 */
	private static Image brightenImageFromBundle(String bundlePath)
	{
		Image re = null;
		if (bundlePath != null)
		{
			/*
			 * Zu erst aus dem Bungle aktualisieren.
			 */
			Image img = readImageFromBundle(bundlePath);
			if (img == null)
			{
				/*
				 * Falls nicht mglich versuchen aus dem Dateisystem zu laden.
				 */
				img = readImageFromFile(bundlePath);
			}

			if (img != null)
			{
				BufferedImage bi = toBufferedImage(img);

				RescaleOp op = new RescaleOp(1.0f, 150.0f, null);
				bi = op.filter(bi, null);
				re = bi;
			}
		}

		return re;
	}

	/**
	 * Liefert ein erhelltes Image aus dem Bundle.
	 * 
	 * @param bundlePath
	 *          Pfad zum Image im Bundle.
	 * @return Erhelltes Image oder <code>null</code>, falls das Image nicht
	 *         geladen werden konnte.
	 */
	public static org.eclipse.swt.graphics.Image brightenImageFromBundleAsSwt(
			String bundlePath)
	{
		org.eclipse.swt.graphics.Image re = null;

		Image img = brightenImageFromBundle(bundlePath);
		if (img != null)
		{
			re = swingToSwtImage(img);
		}

		return re;
	}

	public static GraphicsConfiguration getDefaultConfiguration()
	{
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice gd = ge.getDefaultScreenDevice();
		return gd.getDefaultConfiguration();
	}

	// http://www.exampledepot.com/egs/java.awt.image/Image2Buf.html
	// An Image object cannot be converted to a BufferedImage object.
	// The closest equivalent is to create a buffered image and then draw the
	// image on the buffered image.
	// This example defines a method that does this.

	// This method returns a buffered image with the contents of an image
	public static BufferedImage toBufferedImage(Image image)
	{
		if (image instanceof BufferedImage)
		{
			return (BufferedImage) image;
		}

		// This code ensures that all the pixels in the image are loaded
		image = new ImageIcon(image).getImage();

		// Determine if the image has transparent pixels; for this method's
		// implementation, see e661 Determining If an Image Has Transparent Pixels
		boolean hasAlpha = hasAlpha(image);

		// Create a buffered image with a format that's compatible with the screen
		BufferedImage bimage = null;
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		try
		{
			// Determine the type of transparency of the new buffered image
			int transparency = Transparency.OPAQUE;
			if (hasAlpha)
			{
				transparency = Transparency.BITMASK;
			}

			// Create the buffered image
			GraphicsDevice gs = ge.getDefaultScreenDevice();
			GraphicsConfiguration gc = gs.getDefaultConfiguration();
			bimage = gc.createCompatibleImage(image.getWidth(null), image
					.getHeight(null), transparency);
		} catch (HeadlessException e)
		{
			// The system does not have a screen
		}

		if (bimage == null)
		{
			// Create a buffered image using the default color model
			int type = BufferedImage.TYPE_INT_RGB;
			if (hasAlpha)
			{
				type = BufferedImage.TYPE_INT_ARGB;
			}
			bimage = new BufferedImage(image.getWidth(null), image.getHeight(null),
					type);
		}

		// Copy image to buffered image
		Graphics g = bimage.createGraphics();

		// Paint the image onto the buffered image
		g.drawImage(image, 0, 0, null);
		g.dispose();

		return bimage;
	}

	// http://www.exampledepot.com/egs/java.awt.image/HasAlpha.html
	// This method returns true if the specified image has transparent pixels
	public static boolean hasAlpha(Image image)
	{
		// If buffered image, the color model is readily available
		if (image instanceof BufferedImage)
		{
			BufferedImage bimage = (BufferedImage) image;
			return bimage.getColorModel().hasAlpha();
		}

		// Use a pixel grabber to retrieve the image's color model;
		// grabbing a single pixel is usually sufficient
		PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
		try
		{
			pg.grabPixels();
		} catch (InterruptedException e)
		{
		}

		// Get the image's color model
		ColorModel cm = pg.getColorModel();
		return cm.hasAlpha();
	}

	public static void main(String[] args)
	{
		/*
		 * Wichtig: Erst heller machen fr das Blinken, dann rotieren!
		 */

		// String p =
		// "D:/entwicklung/eclipse/workspaces/workspace_koeln_loevenich/de.bsvrz.buv.plugin.anlagendarstellung/icons/c1_75.png";
		// String p =
		// "D:/entwicklung/eclipse/workspaces/workspace_koeln_loevenich/de.bsvrz.buv.plugin.anlagendarstellung/icons/b2_2.png";
		String p = "D:/entwicklung/eclipse/workspaces/workspace_koeln_loevenich/de.bsvrz.buv.plugin.anlagendarstellung/icons/a1_114.png";

		Image img = readImageFromFile(p);
		System.out.println("=>IMG: " + img);

		Image rotateImage = rotateImage(img, -90.0d);

		JFrame frame = new JFrame();
		frame.setSize(300, 300);

		ImageIcon ic = new ImageIcon(rotateImage);
		frame.setTitle("Test, Size: " + ic.getIconWidth() + "x"
				+ ic.getIconHeight() + " px");

		JLabel label = new JLabel(ic);
		frame.add(label, BorderLayout.CENTER);

		// Image imgBrighter = brightenImageFromBundle(p);
		// Image imgBrighter = ImageUtils.brightenSwingImage(rotateImage);
		Image imgBrighter = ImageUtils.brightenSwingImage(img);
		imgBrighter = rotateImage(imgBrighter, -90.0d);
		ImageIcon icBrighter = new ImageIcon(imgBrighter);
		JLabel lBrighter = new JLabel(icBrighter);
		frame.add(lBrighter, BorderLayout.SOUTH);

		frame.setVisible(true);

	}

}