/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;

import de.bsvrz.buv.plugin.anlagendarstellung.AQModelTypGeneric;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittTypGenericFigure;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AQModelTypGeneric} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AQTypGenericEditPart extends DoModelEditPart<AQModelTypGeneric, AnzeigeQuerschnittTypGenericFigure>
    implements ClientReceiverInterface
{

  private IFigure _contentPane;

  @Override
  protected void geheOnline()
  {
    super.geheOnline();
    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();

    DataDescription dd = new DataDescription(dav.getDataModel().getAttributeGroup(
        "atg.anzeigeQuerschnittEigenschaftIst"), dav.getDataModel().getAspect("asp.aqZustand"), (short)getModel()
        .getKontext());
  }

  @Override
  protected void geheOffline()
  {
    super.geheOffline();
    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataDescription dd = new DataDescription(dav.getDataModel().getAttributeGroup(
        "atg.anzeigeQuerschnittEigenschaftIst"), dav.getDataModel().getAspect("asp.aqZustand"));
    dav.unsubscribeReceiver(this, getModel().getSystemObject(), dd);     
  }

  @Override
  protected AnzeigeQuerschnittTypGenericFigure createFigure()
  {

    AnzeigeQuerschnittTypGenericFigure aq = new AnzeigeQuerschnittTypGenericFigure();
    aq.setOpaque(true);
    aq.setSize(100, 50);

    return aq;
  }

  @Override
  public void update(ResultData[] results)
  {
    for (ResultData result : results)
    {
      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    }
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.AQ_MODEL_TYP_GENERIC__AQ_NAME.equals(notification.getFeature()))
    {
      getFigure().repaint();
    }
    else if (AnlagendarstellungPackage.Literals.AQ_MODEL_TYP_GENERIC__KONTEXT.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.AQ_MODEL_TYP_GENERIC__ANZEIGEN.equals(notification.getFeature()))
    {
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
   */
  protected void refreshVisuals()
  {    
    Point pos = getModel().getLocation();
    Dimension size = getModel().getSize();

    getFigure().setLocation(pos);
    if (size != null)
      getFigure().setSize(size);

  }

  @Override
  public void zoomChanged(double zoom)
  {
    super.zoomChanged(zoom);
    getFigure().repaint();
  }

  public AQModelTypGeneric getAnzeigeQuerschnitt()
  {

    return (AQModelTypGeneric)getModel();

  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

}
