/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import com.sun.xml.internal.bind.v2.TODO;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeInhaltFreiTextDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeFonts;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText.EnumDynamischerWechseltextTextAusrichtung;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeTypDEditPart extends AnzeigeTypAEditPart
{

//  private static final String STR_BW_ZEICHEN_D_ZEILE_A8_MITTELSCHRIFT_PUNKT = "bw.zeichen.dZeile.A8.mittelschrift.punkt";

  private final Dimension DIM = new Dimension(220, 24);

//  private VisuAnzeigeKapsel _kapsel;

  @Override
  protected AnzeigeElementFigure createFigure()
  {
    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = dav.getDataModel();

    AnzeigeModelTypAbstract model = getCastedModel();

    setzteInhalteModell(model, dataModel);

    boolean isFrNord = getCastedModel().getFahrtrichtung() != 1 ? true : false;
    AnzeigeElementFigure aef = new AnzeigeElementFigure("AnzeigeD", AnzeigeElementTyp.FREITEXT,
        AnzeigeInhaltTyp.DUNKEL_TEXT, true, isFrNord);
    aef.setOpaque(true);
    aef.setSize(DIM.width, DIM.height);
    aef.setBorder(new LineBorder(ColorConstants.orange, 2));

    addToolTipListener(aef);

    return aef;
  }

  protected IFigure aktionToolTip(String s)
  {
    Figure fig = new Figure();
    fig.setOpaque(true);
    fig.setLayoutManager(new ToolbarLayout(false));
    Label label = new Label(s);
    fig.add(label);
   
    Object obj = getModel().eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
    
    if (obj instanceof VisuAnzeigeKapsel)
    {
      VisuAnzeigeKapsel _kapsel = (VisuAnzeigeKapsel)obj;
      processTooltip(fig, _kapsel);
    }
    else if (obj instanceof ResultData)
    {
      ResultData _result = (ResultData)obj;
      VisuAnzeigeKapsel _kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(_result);
      processTooltip(fig, _kapsel);
    }
    
    return fig;
  }

  private void processTooltip(Figure fig, VisuAnzeigeKapsel _kapsel)
  {
    boolean dynamisch;
    VisuAnzeigeFonts fonts = VisuAnzeigeFonts.getInstanz();
    String[] pidsZeichen = _kapsel.getPidsZeichen();
    String ausrichtung = _kapsel.getAusrichtung();
    String schriftart = _kapsel.getSchriftart();
    int abstand = _kapsel.getAbstand();

    Image image = null;

    if (!_kapsel.isInaktiv())
    {
      image = fonts.erstelleBitmapSWT(pidsZeichen, ausrichtung, schriftart, 432, 20, abstand);
      dynamisch = true;
    }
    else
    {
      ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
      SystemObject so = getModel().getSystemObject();       
      
      SystemObject[] objs = DaVHelper.holeFontsUeberAnzeige(connection, so);
      VisuAnzeigeKapsel kapselTmp = new VisuAnzeigeKapsel(new String[] {}, objs[0].getPid(),
          EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);
      
      image = VisuAnzeigeFonts.getInstanz().erstelleBitmapSWT(pidsZeichen, kapselTmp.getAusrichtung(), kapselTmp.getSchriftart(), DIM.width, DIM.height, kapselTmp.getAbstand());
      dynamisch = true;
//        image = ImageFactory.getImage("icons/zeile_dunkel.PNG");
    }

    if (image != null)
    {
      AnzeigeElementFigure aef = new AnzeigeElementFigure("AnzeigeD", AnzeigeElementTyp.FREITEXT,
          AnzeigeInhaltTyp.DUNKEL_TEXT, true, true);
      aef.setOpaque(true);
      aef.setSize(432, 20);
      aef.setForegroundColor(ColorConstants.tooltipBackground);
      aef.setBackgroundColor(ColorConstants.tooltipBackground);
      aef.setBorder(new LineBorder(ColorConstants.tooltipBackground, 1));
      
      if (aef.isImageDynamisch()){
        Image imageOld = aef.getImage();
        if (imageOld != null && !_kapsel.isInaktiv()){            
          imageOld.dispose();
          imageOld = null;
        }            
      }
      
      aef.setImageDynamisch(dynamisch);
      
      aef.setImage(image);
      fig.add(aef);             
    }
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification.getFeature()))
    {
      Object newValue = notification.getNewValue();
      Object oldValue = notification.getOldValue();
      
      if (oldValue instanceof AnzeigeInhaltTyp)
      {
        AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)oldValue;
        if (AnzeigeElementFigure.pruefeAufStatischenInhalt(inhaltAlt))
          ((AnzeigeElementFigure)getFigure()).setImageDynamisch(false);
      }
      
      
      DoModelMitVerbLinie model = getModel();
      
//      System.out.println("-------------------------------> notify!!! -> model1: " + model.getSystemObject() + " : " + newValue);

      if (newValue instanceof VisuAnzeigeKapsel)
      {
        VisuAnzeigeKapsel _kapsel = (VisuAnzeigeKapsel)newValue;
        kapselUpdateAnzeigeModelAbstract(_kapsel);
      }
      else if (newValue instanceof AnzeigeInhaltTyp)
      {
        AnzeigeInhaltTyp inhaltNeu = (AnzeigeInhaltTyp)newValue;
        int kontext = 0;
        if (getParent() != null)
        {
          Object parentModel = getParent().getModel();
          {
            if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parentModel;
              kontext = aq.getKontext();
            }
          }
        }        
        updateAnzeigeInhalt(kontext, newValue, oldValue, AnzeigeElementTyp.FREITEXT);
//        inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.FREITEXT, inhaltNeu);
      }
      else if (newValue instanceof ResultData)
      {
        int kontext = 0;
        if (getParent() != null)
        {
          Object parentModel = getParent().getModel();
          {
            if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parentModel;
              kontext = aq.getKontext();
            }
          }
        }

        ResultData result = (ResultData)newValue;
        SystemObject so = getModel().getSystemObject();
        if (so != null && so.equals(result.getObject())){
          VisuAnzeigeKapsel _kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
          kapselUpdateAnzeigeModelAbstract(_kapsel);                            
        }else{
          System.out.println(getModel().getSystemObject() + " else - > Result: " + newValue);          
        }
      }
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof String)
      {
        String s = (String)object;
        getFigure().setToolTip(aktionToolTip(s));
      }
    }
  }

  /**
   * {@link TODO}
   * 
   * @param kapsel
   */
  private void kapselUpdateAnzeigeModelAbstract(final VisuAnzeigeKapsel kapsel)
  {
    Image image = null;
    boolean dynamisch = false;
    
    if (kapsel != null)
    {
      VisuAnzeigeFonts fonts = VisuAnzeigeFonts.getInstanz();
      String[] pidsZeichen = kapsel.getPidsZeichen();
      String ausrichtung = kapsel.getAusrichtung();
      String schriftart = kapsel.getSchriftart();
      int abstand = kapsel.getAbstand();
      
      if (!kapsel.isInaktiv())
      {

        int len = fonts.berechneTextLaenge(pidsZeichen, schriftart, abstand);
        if (len > DIM.width - 4)
        {
          List<String> list = new ArrayList<String>();
          
          ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
          SystemObject so = getModel().getSystemObject();  
          if (connection != null && so != null){
            SystemObject[] objs = DaVHelper.holeFontsUeberAnzeige(connection, so);
            NonMutableSet set = ((ConfigurationObject)objs[0]).getNonMutableSet("Zeichen");
            List<SystemObject> elements = set.getElements();
            SystemObject soPunkt = null; 
            for (SystemObject so2 : elements)
            {
              if (so2.getNameOrPidOrId().contains("Punkt")){
                soPunkt = so2;
                break;
              }
            }
            if (soPunkt != null){
              list.add(soPunkt.getPid());
              list.add(soPunkt.getPid());
              list.add(soPunkt.getPid());
            }              
          }
                   
//          list.add(STR_BW_ZEICHEN_D_ZEILE_A8_MITTELSCHRIFT_PUNKT);
//          list.add(STR_BW_ZEICHEN_D_ZEILE_A8_MITTELSCHRIFT_PUNKT);
//          list.add(STR_BW_ZEICHEN_D_ZEILE_A8_MITTELSCHRIFT_PUNKT);
          for (int i = 0; i < pidsZeichen.length; i++)
          {

            String s = pidsZeichen[i];
            list.add(i, s);
            len = fonts.berechneTextLaenge(list.toArray(new String[list.size()]), schriftart, abstand);
            if (len > DIM.width - 4)
            {
              list.remove(i);
              break;
            }
          }

          pidsZeichen = list.toArray(new String[list.size()]);
        }

        image = fonts.erstelleBitmapSWT(pidsZeichen, ausrichtung, schriftart, DIM.width - 4, DIM.height - 3, abstand);
        dynamisch = true;
      }
      else
      {        
        ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
        SystemObject so = getModel().getSystemObject();       
        
        SystemObject[] objs = DaVHelper.holeFontsUeberAnzeige(connection, so);
        
        VisuAnzeigeKapsel kapselTmp = new VisuAnzeigeKapsel(new String[] {}, objs[0].getPid(),
            EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);
        
        image = VisuAnzeigeFonts.getInstanz().erstelleBitmapSWT(pidsZeichen, kapselTmp.getAusrichtung(), kapselTmp.getSchriftart(), DIM.width, DIM.height, kapselTmp.getAbstand());
        dynamisch = true;
        
//        System.out.println("----> Kapsel ist inaktiv!!!" + getModel().getSystemObject().getName());
        
//        image = ImageFactory.getImage("icons/zeile_dunkel.PNG");
      }
    }
    else
    {
      image = ImageFactory.getImage("icons/zeile_gestoert.PNG");
    }

    if (image != null && !image.isDisposed())
    {
      final AnzeigeElementFigure aef = (AnzeigeElementFigure)getFigure();
      final Image fimage = image;
      if (aef.isImageDynamisch()){
        Image imageOld = aef.getImage();
        if (imageOld != null && kapsel != null && !kapsel.isInaktiv())
        {
          imageOld.dispose();
          imageOld = null;
        }        
      }
      
      final boolean fdynamisch = dynamisch;
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
        @Override
        public void run()
        { 
          if (fimage != null && !fimage.isDisposed())
          {
//            System.out.println(getModel().getSystemObject());
            
            aef.setImage(fimage);            
            aef.setImageDynamisch(fdynamisch);
          }else{
            System.out.println("Image ist schon disposed11" + fimage);            
          }
        }
      });
    }else{
      System.out.println("Image ist schon disposed1" + image);
    }
  }
}
