package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.FigurePickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.model.KilometrierungZuordnung;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;

public class EditPartPickerParent extends DoMitVerbLinieEditPart
{

	/**
	 * Initialie Hhe
	 */
	public static final int INIT_BREITE = 25;

	/**
	 * Initiale Breite
	 */
	public static final int INIT_HOEHE = 400;
	
	
  @Override
  protected List getModelChildren()
  {
//    System.out.println("EditPartPickerParent.getModelChildren(): " + getCastedModel().getKomponenten().size());
    
    ConfiguratedDoTyp doTyp = DarstellungExtensionPoint.getDoTyp("PickerVertical");
    Darstellung darstellung = (Darstellung)getViewer().getProperty(Darstellung.class.toString());
    EList<DoTyp> doTypen = darstellung.getDoTypen();
    boolean vorhanden = false;
    for (DoTyp typ : doTypen)
    {
      if (typ instanceof ConfiguratedDoTyp)
      {
        ConfiguratedDoTyp co = (ConfiguratedDoTyp)typ;
        if (co.getId().equals(doTyp.getId()))
        {
          vorhanden = true;
          doTyp = co;
          break;
        }
      }
    }
        
    if (!vorhanden)
    {
      doTypen.add(doTyp);
    }
    
    return getCastedModel().getKomponenten();     
  }
	

	@Override
	protected IFigure createFigure()
	{
//	  Figure figure = new Figure();
	  FigurePickerParent figure = new FigurePickerParent();
		// getLayer(CONNECTION_LAYER).add(figurePickerVertical.getLinie());

		PickerParent castedModel = getCastedModel();
		if (castedModel != null)
		{

//			/*
//			 * Breite und Hhe bernehmen.
//			 */
			int breite = castedModel.getBreite();
			int hoehe = castedModel.getHoehe();
			int distanz = castedModel.getDistanz();
			int yPosPickerTop = castedModel.getYPosPickerTop();

			/*
			 * Bei Bedarf die initiale Breite setzen.
			 */
			if (breite == 0)
			{
				breite = INIT_BREITE;
				castedModel.setBreite(breite);
			}

			/*
			 * Bei Bedarf die initiale Hhe setzen.
			 */
			if (hoehe == 0)
			{
				hoehe = INIT_HOEHE;
				castedModel.setHoehe(hoehe);
			}
			figure.setSize(breite, hoehe);

			/*
			 * Bei Bedarf die Distanz auf die initiale Hhe - yPosPickerTop setzen.
			 */
			if ((distanz == 0) && (yPosPickerTop == 0))
			{
				distanz = castedModel.getHoehe();
				castedModel.setDistanz(distanz);
			}

		}

		return figure;

		// IFigure figure = new Figure();
		// figure.setSize(25, 150);
		// figure.setLayoutManager(new XYLayout());
		//		
		// figure.setBorder(new LineBorder(ColorConstants.red,2));
		//
		// return figure;
	}

	/**
	 * Liefert das Modell {@link PickerHorizontal} dieses EditParts.
	 * 
	 * @return
	 */
	public PickerParent getCastedModel()
	{
	  PickerParent pickerParent = null;

		DoModelMitVerbLinie model = getModel();
		if ((model != null) && (model instanceof PickerParent))
		{
		  pickerParent = (PickerParent) model;
		}

		return pickerParent;
	}

	public FigurePickerParent getCastedFigure()
	{
	  FigurePickerParent fig = null;

		if ((getFigure() != null) && (getFigure() instanceof FigurePickerParent))
		{
			fig = (FigurePickerParent) getFigure();
//			fig.setModelAndStack(getCastedModel(), getCommandStack());
		}

		return fig;
	}

	@Override
	public void notifyChanged(Notification notification)
	{
		super.notifyChanged(notification);
		
		
//		System.out.println("EditPartPickerParent.notifyChanged(): " + notification.getFeature() + " -> " + notification.getNewValue());

		if (AnlagendarstellungPackage.Literals.PICKER_PARENT__BREITE
				.equals(notification.getFeature()))
		{
			Object oNewBreite = notification.getNewValue();
			if ((oNewBreite != null) && (oNewBreite instanceof Integer))
			{
				IFigure figure = getFigure();
				if (figure != null)
				{
					Integer breite = (Integer) oNewBreite;
					Dimension sizeOld = figure.getSize();
					Dimension sizeNew = new Dimension(breite, sizeOld.height);
					figure.setSize(sizeNew);
					figure.repaint();
					refreshVisuals();
//					if (figure instanceof FigurePickerVertical)
//					{
//						((FigurePickerVertical) figure).rep();
//						refreshVisuals();
//					}
				}
			}

		} else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__HOEHE
				.equals(notification.getFeature()))
		{
			Object oNewHoehe = notification.getNewValue();
			if ((oNewHoehe != null) && (oNewHoehe instanceof Integer))
			{
				IFigure figure = getFigure();
				if (figure != null)
				{
					Integer hoehe = (Integer) oNewHoehe;
					Dimension sizeOld = figure.getSize();
					Dimension sizeNew = new Dimension(sizeOld.width, hoehe);
					figure.setSize(sizeNew);
					figure.repaint();
					refreshVisuals();
//					if (figure instanceof FigurePickerVertical)
//					{
//						((FigurePickerVertical) figure).rep();
//						refreshVisuals();
//					}
				}
			}
		} else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__YPOS_PICKER_TOP
				.equals(notification.getFeature()))
		{
//			int oYPosPickerTop = notification.getNewIntValue();
//			if (oYPosPickerTop != notification.getOldIntValue())
//			{
//				refreshVisuals();
//			}
		} else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__DISTANZ
				.equals(notification.getFeature()))
		{
//			int oDistanz = notification.getNewIntValue();
//			if (oDistanz != notification.getOldIntValue())
//			{
//				refreshVisuals();
//			}
		} else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__KILOMETRIERUNG_ZUORDNUNG
				.equals(notification.getFeature()))
		{
			refreshVisuals();
		} else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__TICKS_ANZEIGEN
				.equals(notification.getFeature()))
		{
			refreshVisuals();
		}
		 else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__OBJEKT_TYP
	        .equals(notification.getFeature()))
	    {
	      refreshVisuals();
	    }
		 else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__FARBE_VORDERGRUND
		     .equals(notification.getFeature()))
		 {
	      refreshVisuals();
		 }		
		 else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__FARBE_HINTERGRUND
		     .equals(notification.getFeature()))
		 {
		   Object oNewHoehe = notification.getNewValue();
		   
		   
		   System.out.println("PICKER_PARENT__FARBE_HINTERGRUND: " + oNewHoehe);
		   
		   if (oNewHoehe instanceof RGB)
		   {
        RGB rgb = (RGB)oNewHoehe;
        FigurePickerParent cf = getCastedFigure();                 
        cf.setFarbeHintergrund(rgb);        
		   }
		   
       refreshVisuals();   
		   
		}
		else if (AnlagendarstellungPackage.Literals.PICKER_PARENT__TRANSPARENZ
         .equals(notification.getFeature()))
     {
       Object oNewHoehe = notification.getNewValue();
       
       if (oNewHoehe instanceof Integer)
       {
         Integer alpha = (Integer)oNewHoehe;
         FigurePickerParent cf = getCastedFigure();                 
         cf.setTranparenz(alpha);                
       }             
       refreshVisuals();
     }
		else
		{
		  
		}
	}

//	@Override
//	protected void createEditPolicies()
//	{
//		super.createEditPolicies();
//	}

	protected void refreshVisuals()
	{
//	  System.out.println("EditPartPickerParent.refreshVisuals()");
	  
	  
		/*
		 * Position und Gre der FigurePickerVertical setzen.
		 */
		Point pos = getModel().getLocation();
		Dimension dim = getFigure().getSize();

		Rectangle constraint = new Rectangle(pos, dim);

		if (constraint != null)
			((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(),
					constraint);
		else
		{
			((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(),
					new Rectangle(pos.x, pos.y, -1, -1));
		}
		
    PickerParent castedModel = getCastedModel();
    if (castedModel != null)
    {
      int yPosPickerTop = castedModel.getYPosPickerTop();
      int distanz = castedModel.getDistanz();
      FigurePickerParent castedFigure = getCastedFigure();
      if (castedFigure != null)
      {
        /*
         * Pfeil obere Position und Distanz zur unteren Pfeilspitze.
         */
        // castedFigure.setPickerData(yPosPickerTop, distanz);

        String strKM = castedModel.getKilometrierungZuordnung();
        if (strKM != null)
        {
          /*
           * Tickpositionen setzen.
           */
          LinkedList<KilometrierungZuordnung> lTicks = KilometrierungZuordnung.deserFromMultipleString(strKM);
          if (lTicks != null)
          {
            LinkedList<Integer> list = new LinkedList<Integer>();
            for (KilometrierungZuordnung kilometrierungZuordnung : lTicks)
            {
              if (kilometrierungZuordnung.getyVon() != null)
              {
                list.add(kilometrierungZuordnung.getyVon());
              }

              if (kilometrierungZuordnung.getyBis() != null)
              {
                list.add(kilometrierungZuordnung.getyBis());
              }
            }

            castedFigure.setTicks(list);
          }
        }

        /*
         * Flag zum Zeichnen der Ticks
         */
        castedFigure.setPaintTicks(castedModel.isTicksAnzeigen());
        /*
         * Flag zum Zeichnen der Ticks
         */
        if (castedModel.getFarbeHintergrund() != null)
        {

//          System.out.println("EditPartPickerParent.refreshVisuals(Farbe(Htg)): " + castedModel.getFarbeHintergrund());

          if (!castedFigure.getBackgroundColor().getRGB().equals(castedModel.getFarbeHintergrund()))
          {
//            castedFigure.setBackgroundColor(new Color(null, castedModel.getFarbeHintergrund()));
            castedFigure.setFarbeHintergrund(castedModel.getFarbeHintergrund());
          }
        }
        if (castedModel.getTransparenz() >= 0)
        {          
//          System.out.println("EditPartPickerParent.refreshVisuals(alpha)): " + castedModel.getTransparenz());
          
          if (castedFigure.getTranparenz() != castedModel.getTransparenz())
          {
//            castedFigure.setBackgroundColor(new Color(null, castedModel.getFarbeHintergrund()));
            castedFigure.setTranparenz(castedModel.getTransparenz());
          }
        }
      }
    }
	}

	public void test()
	{
		CommandStack commandStack = getCommandStack();

	}
}
