/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.Date;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.LayerConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqZeilenFigure;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link DoModelMitVerbLinie} verwendet.
 * 
 * @author Pittner
 * 
 */
public class TabelleMqA14EditPart extends DoMitVerbLinieEditPart implements LayerConstants
{

  private final static Font _font = new Font(null, "Arial", 7, SWT.NORMAL);

  private final static Color colorGray = ColorConstants.gray;

  private final static Color colorBlack = ColorConstants.black;

  @Override
  protected void geheOnline()
  {
    super.geheOnline();
  }

  @Override
  protected void geheOffline()
  {
    super.geheOffline();
  }

  @Override
  public void deactivate()
  {    
    super.deactivate();
  }

  @Override
  protected IFigure createFigure()
  {
    Figure figure = new Figure();
    GridLayout layout = new GridLayout(4, true);
    layout.horizontalSpacing = 0;
    layout.verticalSpacing = 0;
    figure.setLayoutManager(layout);
    figure.setOpaque(true);

    for (int i = 0; i < 3; i++)
    {
      if (i == 0)
      {
        Label l1 = new Label("--- %");
        l1.setFont(_font);
        l1.setToolTip(new Label());
        l1.setForegroundColor(colorBlack);
        TabellenMqFigure fig1 = new TabellenMqFigure(l1, false);
        TabellenMqZeilenFigure cf1 = fig1.getListCompartments().get(0);
        TabellenMqZeilenFigure cf2 = fig1.getListCompartments().get(1);
        Label l2 = new Label(" Q ");
        l2.setFont(_font);
        l2.setToolTip(new Label());
        l2.setForegroundColor(colorGray);
        cf1.add(l2);
        Label l3 = new Label(" V ");
        l3.setFont(_font);
        l3.setToolTip(new Label());
        l3.setForegroundColor(colorGray);
        cf2.add(l3);
        figure.add(fig1);

        GridData gd = new GridData();
        gd.widthHint = 55;
        gd.horizontalAlignment = SWT.FILL;
        gd.grabExcessHorizontalSpace = true;
        figure.getLayoutManager().setConstraint(fig1, gd);
      }
      else
      {
        Label l1 = new Label();
        l1.setFont(_font);
        switch (i)
        {
          case 1:
            l1.setText("Pkw");
            break;
          case 2:
            l1.setText("Lkw");
            break;
          default:
            l1.setText("");
        }
        l1.setForegroundColor(ColorConstants.gray);
        l1.setToolTip(new Label());
        TabellenMqFigure fig1 = new TabellenMqFigure(l1, true);
        TabellenMqZeilenFigure cf1 = fig1.getListCompartments().get(0);
        TabellenMqZeilenFigure cf2 = fig1.getListCompartments().get(1);
        Label l2 = new Label("undefiniert");
        l2.setFont(_font);
        l2.setToolTip(new Label());
        cf1.add(l2);
        Label l3 = new Label("undefiniert");
        l3.setFont(_font);
        l3.setToolTip(new Label());
        cf2.add(l3);
        figure.add(fig1);

        GridData gd = new GridData();
        gd.widthHint = 55;
        gd.horizontalAlignment = SWT.FILL;
        gd.grabExcessHorizontalSpace = true;
        figure.getLayoutManager().setConstraint(fig1, gd);
      }
    }

    figure.setSize(180, 50);

    addToolTipListener(figure);

    return figure;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__BEZEICHNUNG.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__KONTEXT.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__DATA_UPDATE.equals(notification.getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdate(result);
    }
    else if (AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__DARSTELLUNG_UPDATE.equals(notification.getFeature()))
    {

      Object object = notification.getNewValue();

    }
    else if (DobjPackage.Literals.DO_MODEL__SYSTEM_OBJECT.equals(notification.getFeature()))
    {
      SystemObject so = (SystemObject)notification.getNewValue();
      // createFigureDynamisch(getFigure(), so);
      refresh();
    }
  }


  public void dataUpdate(ResultData result)
  {
    final IFigure figure = getFigure();

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    SystemObject so = result.getObject();
    if (atg.getPid().equals("atg.verkehrsDatenKurzZeitMq"))
    {
      Data data = result.getData();
      if (data != null)
      {
        if (getModel().getSystemObject().equals(result.getObject()))
        {
          NumberValue _qPkw = data.getItem("QPkw").getScaledValue("Wert");
          final String qPkw = _qPkw.getValueText() + (_qPkw.isNumber() ? " " : "");
          final String qPkwTooltip = _qPkw.getText() + "  (" + new Date(result.getDataTime()) + ")";

          NumberValue _qLkw = data.getItem("QLkw").getScaledValue("Wert");
          final String qLkw = _qLkw.getValueText() + (_qLkw.isNumber() ? " " : "");
          final String qLkwTooltip = _qLkw.getText() + "  (" + new Date(result.getDataTime()) + ")";

          NumberValue _vPkw = data.getItem("VPkw").getScaledValue("Wert");
          final String vPkw = _vPkw.getValueText() + (_vPkw.isNumber() ? " " : "");
          final String vPkwTooltip = _vPkw.getText() + "  (" + new Date(result.getDataTime()) + ")";

          NumberValue _vLkw = data.getItem("VLkw").getScaledValue("Wert");
          final String vLkw = _vLkw.getValueText() + (_vLkw.isNumber() ? " " : "");
          final String vLkwTooltip = _vLkw.getText() + "  (" + new Date(result.getDataTime()) + ")";

          final String belegung = data.getItem("B").getItem("Wert").asTextValue().getText();

          PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run()
            {
              ((TabellenMqFigure)figure.getChildren().get(0)).getName().setText(belegung);

              List children = figure.getChildren();
              List<TabellenMqZeilenFigure> list1 = ((TabellenMqFigure)children.get(1)).getListCompartments();
              Label l1 = (Label)((TabellenMqZeilenFigure)list1.get(0)).getChildren().get(0);
              l1.setText(qPkw);
              ((Label)l1.getToolTip()).setText(qPkwTooltip);
              List<TabellenMqZeilenFigure> list2 = ((TabellenMqFigure)children.get(2)).getListCompartments();
              Label l2 = (Label)((TabellenMqZeilenFigure)list2.get(0)).getChildren().get(0);
              l2.setText(qLkw);
              ((Label)l2.getToolTip()).setText(qLkwTooltip);
              List<TabellenMqZeilenFigure> list3 = ((TabellenMqFigure)children.get(1)).getListCompartments();
              Label l3 = (Label)((TabellenMqZeilenFigure)list3.get(1)).getChildren().get(0);
              l3.setText(vPkw);
              ((Label)l3.getToolTip()).setText(vPkwTooltip);
              List<TabellenMqZeilenFigure> list4 = ((TabellenMqFigure)children.get(2)).getListCompartments();
              Label l4 = (Label)((TabellenMqZeilenFigure)list4.get(1)).getChildren().get(0);
              l4.setText(vLkw);
              ((Label)l4.getToolTip()).setText(vLkwTooltip);
            }
          });
        }
      }
      else
      {
        if (getModel().getSystemObject().equals(result.getObject()))
        {
          String temp = "keine Quelle / keine Daten";

          if (result.isNoSourceAvailable())
          {
            temp = "keine Quelle";
          }
          else if (result.isNoDataAvailable())
          {
            temp = "keine Daten";
          }
          else if (result.isNoRightsAvailable())
          {
            temp = "keine Rechte";
          }
          else if (result.isNoValidSubscription())
          {
            temp = "fehl. Anmeldung";
          }

          final String s = temp;

          PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run()
            {
              ((TabellenMqFigure)figure.getChildren().get(0)).getName().setText("--- %");

              List children = figure.getChildren();
              List<TabellenMqZeilenFigure> list1 = ((TabellenMqFigure)children.get(1)).getListCompartments();
              Label l1 = (Label)((TabellenMqZeilenFigure)list1.get(0)).getChildren().get(0);
              l1.setText(s);
              ((Label)l1.getToolTip()).setText(s);
              List<TabellenMqZeilenFigure> list2 = ((TabellenMqFigure)children.get(2)).getListCompartments();
              Label l2 = (Label)((TabellenMqZeilenFigure)list2.get(0)).getChildren().get(0);
              l2.setText(s);
              ((Label)l2.getToolTip()).setText(s);
              List<TabellenMqZeilenFigure> list3 = ((TabellenMqFigure)children.get(1)).getListCompartments();
              Label l3 = (Label)((TabellenMqZeilenFigure)list3.get(1)).getChildren().get(0);
              l3.setText(s);
              ((Label)l3.getToolTip()).setText(s);
              List<TabellenMqZeilenFigure> list4 = ((TabellenMqFigure)children.get(2)).getListCompartments();
              Label l4 = (Label)((TabellenMqZeilenFigure)list4.get(1)).getChildren().get(0);
              l4.setText(s);
              ((Label)l4.getToolTip()).setText(s);
            }
          });
        }
      }
    }
  }
}
