/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqZeilenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UmfeldDatenBandFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionSbaUfdMessStelleNaesseStufe;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionUfdSensorSichtWeite;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.UmfeldDatenTypenPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.ToolTipHelper2;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link DoModelMitVerbLinie} verwendet.
 * 
 * @author Pittner
 * 
 */
public class UfdSbaMessStelleNSEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{
  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  private final Dimension DIM = new Dimension(107, 21);

  private final static Font _font1 = new Font(null, "Arial", 9, SWT.BOLD);

  private final static Font _font = new Font(null, "Arial", 7, SWT.NORMAL);

  private final static Font _font2 = new Font(null, "Arial", 7, SWT.ITALIC);

  private final static Color colorGray = ColorConstants.gray;

  private final static Color colorBlack = ColorConstants.black;

  private SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected IFigure createFigure()
  {
    UmfeldDatenBandFigure aef = new UmfeldDatenBandFigure("", ZusatzElementTyp.NIEDERSCHLAG,
        ZusatzElementTyp.NIEDERSCHLAG_TROCKEN);
    aef.setOpaque(true);
    aef.setSize(DIM.width, DIM.height);

    addToolTipListener(aef);

    return aef;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BEZEICHNUNG.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__KONTEXT.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      if (result != null)
      {
        dataUpdateUmfeldDaten(result);
      }
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      aktionToolTipNeu();
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension dim = getFigure().getSize();

    Rectangle constraint = new Rectangle(pos, dim);

    if (constraint != null)
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
    else
    {
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), new Rectangle(pos.x, pos.y, -1, -1));
    }
  }

  @Override
  public void zoomChanged(double newZoom)
  {
    // System.out.println("UfdSbaMessStelleNSEditPart.zoomChanged()");
  }

  @Override
  protected IFigure getToolTip()
  {   
    return null;
  }

  protected void aktionToolTipNeu()
  {

    Figure figure = new Figure();
    GridLayout layout = new GridLayout(3, true);
    layout.horizontalSpacing = 0;
    layout.verticalSpacing = 0;
    figure.setLayoutManager(layout);
    figure.setOpaque(true);

    GridData gd0 = new GridData();
    gd0.horizontalAlignment = SWT.LEFT;
    gd0.grabExcessHorizontalSpace = true;
    gd0.horizontalSpan = 4;

    Label label = new Label(getModel().getSystemObject().getName());
    label.setFont(_font1);
    figure.add(label);
    figure.getLayoutManager().setConstraint(label, gd0);

    // SystemObject ufdms = bestimmeUfdMessStelleVonUfdSensor(getModel().getSystemObject());

    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaUmfeldDatenMessStelleReal");
    Data cd = getModel().getSystemObject().getConfigurationData(atg);
    SystemObject ufdms = cd.getItem("UmfeldDatenMessStelleRealReferenz").asReferenceValue().getSystemObject();

    SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();
    NonMutableSet set = ((ConfigurationObject)ufdms).getNonMutableSet("UmfeldDatenSensoren");
    if (set != null)
    {
      List<SystemObject> sensors = set.getElements();
      for (SystemObject sensor : sensors)
      {
        sortedMap.put(sensor.getName(), sensor);
      }
    }

    int offset1 = 3;
    
    SystemObject sswUfdMs = getModel().getSystemObject();

    int offset2 = 0;

    if (sswUfdMs != null)
    {

      offset2 = 2;
    }

    TabellenMqFigure spalte1 = new TabellenMqFigure(false, offset1 + offset2 + sortedMap.size());
    TabellenMqFigure spalte2 = new TabellenMqFigure(true, offset1 + offset2 + sortedMap.size());
    TabellenMqFigure spalte3 = new TabellenMqFigure(true, offset1 + offset2 + sortedMap.size());

    int cnt = 0;
    for (SystemObject sensor : sortedMap.values())
    {
      TabellenMqZeilenFigure cf1 = spalte1.getListCompartments().get(cnt);
      String name = holeUmfeldDatenTypBezeichnung(sensor);
      Label l1 = new Label(name);
      l1.setFont(_font);
      l1.setForegroundColor(colorBlack);
      cf1.add(l1);

      TabellenMqZeilenFigure cf2 = spalte2.getListCompartments().get(cnt);
      String wert = holeWertOderStufe(sensor, false);
      Label l2 = new Label(wert);
      l2.setFont(_font);
      l2.setForegroundColor(colorGray);
      cf2.add(l2);

      TabellenMqZeilenFigure cf3 = spalte3.getListCompartments().get(cnt);
      String stufe = holeWertOderStufe(sensor, true);
      Label l3 = new Label(stufe);
      l3.setFont(_font);
      l3.setForegroundColor(colorGray);
      cf3.add(l3);

      cnt++;
    }

    if (offset1 > 0)
    {
      int cnt1 = 1;
      for (int i = sortedMap.size(); i < sortedMap.size() + offset1; i++)
      {
        TabellenMqZeilenFigure cf1 = spalte1.getListCompartments().get(i);
        String name = holeUmfeldDatenTypBezeichnungSswSba(getModel().getSystemObject().getType().getPid() + cnt1);
        if (i >= (sortedMap.size() + offset1 - 3))
        {
          atg = connection.getDataModel().getAttributeGroup("atg.sswSbaUmfeldDatenMessStelleReal");
          cd = getModel().getSystemObject().getConfigurationData(atg);
          if (cd != null)
          {
            SystemObject realUfd = cd.getItem("UmfeldDatenMessStelleRealReferenz").asReferenceValue().getSystemObject();
            name = holeUmfeldDatenTypBezeichnungSswSba(realUfd.getType().getPid() + cnt1);            
          }
        }

        Label l1 = new Label("*" + name);
        l1.setFont(_font2);
        l1.setForegroundColor(colorBlack);
        cf1.add(l1);

        TabellenMqZeilenFigure cf2 = spalte2.getListCompartments().get(i);
        String wert = holeWertOderStufeUfdMs(ufdms, cnt1, false);
        Label l2 = new Label(wert);
        l2.setFont(_font2);
        l2.setForegroundColor(colorGray);
        cf2.add(l2);

        TabellenMqZeilenFigure cf3 = spalte3.getListCompartments().get(i);
        String stufe = holeWertOderStufeUfdMs(ufdms, cnt1, true);
        Label l3 = new Label(stufe);
        l3.setFont(_font2);
        l3.setForegroundColor(colorGray);
        cf3.add(l3);

        cnt1++;
      }
    }

    if (offset2 > 0)
    {
      int cnt2 = 1;
      for (int i = sortedMap.size() + offset1; i < sortedMap.size() + offset1 + offset2; i++)
      {
        TabellenMqZeilenFigure cf1 = spalte1.getListCompartments().get(i);
        String name = holeUmfeldDatenTypBezeichnungSswSba(sswUfdMs.getType().getPid() + cnt2);
        Label l1 = new Label("*" + name);
        l1.setFont(_font2);
        l1.setForegroundColor(colorBlack);
        cf1.add(l1);

        TabellenMqZeilenFigure cf2 = spalte2.getListCompartments().get(i);
        String wert = holeWertOderStufeSswSbaUfdMs(sswUfdMs, cnt2, false);
        Label l2 = new Label(wert);
        l2.setFont(_font2);
        l2.setForegroundColor(colorGray);
        cf2.add(l2);

        TabellenMqZeilenFigure cf3 = spalte3.getListCompartments().get(i);
        String stufe = holeWertOderStufeSswSbaUfdMs(sswUfdMs, cnt2, true);
        Label l3 = new Label(stufe);
        l3.setFont(_font2);
        l3.setForegroundColor(colorGray);
        cf3.add(l3);

        cnt2++;
      }
    }

    GridData gd1 = new GridData();
    gd1.widthHint = 175;
    gd1.horizontalAlignment = SWT.BEGINNING;
    gd1.grabExcessHorizontalSpace = true;

    GridData gd2 = new GridData();
    gd2.widthHint = 175;
    gd2.horizontalAlignment = SWT.BEGINNING;
    gd2.grabExcessHorizontalSpace = true;

    GridData gd3 = new GridData();
    gd3.widthHint = 175;
    gd3.horizontalAlignment = SWT.BEGINNING;
    gd3.grabExcessHorizontalSpace = true;

    figure.add(spalte1);
    figure.getLayoutManager().setConstraint(spalte1, gd1);
    figure.add(spalte2);
    figure.getLayoutManager().setConstraint(spalte2, gd2);
    figure.add(spalte3);
    figure.getLayoutManager().setConstraint(spalte3, gd3);

    ToolTipHelper2 helper = new ToolTipHelper2(getViewer().getControl());
    helper.setDelay(10000);
    org.eclipse.swt.graphics.Point loc = Display.getDefault().getCursorLocation();
    helper.displayToolTipNear(getFigure(), figure, loc.x, loc.y);

  }

  private String holeUmfeldDatenTypBezeichnung(SystemObject sensor)
  {
    SystemObjectType type = sensor.getType();
    String[] ss = UmfeldDatenTypenPreference.getMapTypAtg().get(type.getPid());

    if (ss != null && ss.length > 0)
    {
      return ss[1];
    }
    return null;

  }

  private String holeUmfeldDatenTypBezeichnungSswSba(String typ)
  {
    String[] ss = UmfeldDatenTypenPreference.getMapTypAtg().get(typ);

    if (ss != null && ss.length > 0)
    {
      return ss[1];
    }
    return null;

  }

  private String holeWertOderStufe(SystemObject sensor, boolean b)
  {
    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    String re = "";

    SystemObjectType type = sensor.getType();
    String[] ss = UmfeldDatenTypenPreference.getMapTypAtg().get(type.getPid());

    if (ss != null && ss.length > 0)
    {
      if (!b)
      {
        AttributeGroup atg = connection.getDataModel().getAttributeGroup(ss[0]);
        Aspect asp = connection.getDataModel().getAspect("asp.externeErfassung");
        ResultData result = connection.getData(sensor, new DataDescription(atg, asp), 1000);

        String wert = "";

        if (result.getDataState().equals(DataState.NO_SOURCE))
        {
          wert = "keine Quelle";
        }
        else if (result.getDataState().equals(DataState.NO_DATA))
        {
          wert = "keine Daten";
        }
        else if (result.hasData())
        {
          Data data = result.getData();
          wert = data.getItem(ss[1]).getItem("Wert").asTextValue().getText();
        }
        else
        {

        }

        String wertZeit = _sdf.format(new Date(result.getDataTime()));
        re = wert + " (" + wertZeit + ")";
      }
      else
      {
        if (ss.length > 3)
        {
          AttributeGroup atg = connection.getDataModel().getAttributeGroup(ss[3]);
          Aspect asp = connection.getDataModel().getAspect("asp.klassifizierung");
          ResultData result = connection.getData(sensor, new DataDescription(atg, asp), 1000);
          String stufe = "";
          if (result.getDataState().equals(DataState.NO_SOURCE))
          {
            stufe = "keine Quelle";
          }
          else if (result.getDataState().equals(DataState.NO_DATA))
          {
            stufe = "keine Daten";
          }
          else if (result.hasData())
          {
            Data data = result.getData();
            stufe = data.getItem("Stufe").asTextValue().getText();
          }
          else
          {

          }
          String stufeZeit = _sdf.format(new Date(result.getDataTime()));
          re = stufe + " (" + stufeZeit + ")";
        }
      }
    }
    return re;
  }

  private String holeWertOderStufeSswSbaUfdMs(SystemObject so, int offset, boolean b)
  {
    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    String re = "";

    SystemObjectType type = so.getType();
    String[] ss = UmfeldDatenTypenPreference.getMapTypAtg().get(type.getPid() + offset);

    if (ss != null && ss.length > 0)
    {
      if (!b)
      {
        AttributeGroup atg = connection.getDataModel().getAttributeGroup(ss[0]);
        Aspect asp = connection.getDataModel().getAspect("asp.zustand");

        if (atg != null)
        {
          ResultData result = connection.getData(so, new DataDescription(atg, asp), 1000);

          String wert = "";

          if (result.getDataState().equals(DataState.NO_SOURCE))
          {
            wert = "keine Quelle";
          }
          else if (result.getDataState().equals(DataState.NO_DATA))
          {
            wert = "keine Daten";
          }
          else if (result.hasData())
          {
            Data data = result.getData();
            Array array = data.getItem("ErgebnisTagNacht").asArray();
            if (array != null && array.getLength() > 0)
            {
              wert = array.getItem(0).getItem("Status").asTextValue().getText();
            }
          }
          else
          {

          }

          String wertZeit = _sdf.format(new Date(result.getDataTime()));
          re = wert + " (" + wertZeit + ")";
        }
      }
      else
      {
        if (ss.length > 3)
        {
          AttributeGroup atg = connection.getDataModel().getAttributeGroup(ss[3]);

          Aspect asp = connection.getDataModel().getAspect("asp.klassifizierung");
          ResultData result = connection.getData(so, new DataDescription(atg, asp), 1000);
          String stufe = "";
          if (result.getDataState().equals(DataState.NO_SOURCE))
          {
            stufe = "keine Quelle";
          }
          else if (result.getDataState().equals(DataState.NO_DATA))
          {
            stufe = "keine Daten";
          }
          else if (result.hasData())
          {
            Data data = result.getData();
            stufe = data.getItem("NsseStufe").asTextValue().getText();
          }
          else
          {

          }
          String stufeZeit = _sdf.format(new Date(result.getDataTime()));
          re = stufe + " (" + stufeZeit + ")";
        }
      }
    }
    return re;
  }

  private String holeWertOderStufeUfdMs(SystemObject so, int offset, boolean b)
  {
    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    String re = "";

    SystemObjectType type = so.getType();
    String[] ss = UmfeldDatenTypenPreference.getMapTypAtg().get(type.getPid() + offset);

    if (ss != null && ss.length > 0)
    {
      if (!b)
      {
        AttributeGroup atg = connection.getDataModel().getAttributeGroup(ss[0]);
        Aspect asp = connection.getDataModel().getAspect("asp.analyse");

        if (atg != null)
        {
          ResultData result = connection.getData(so, new DataDescription(atg, asp), 1000);

          String wert = "";

          if (result.getDataState().equals(DataState.NO_SOURCE))
          {
            wert = "keine Quelle";
          }
          else if (result.getDataState().equals(DataState.NO_DATA))
          {
            wert = "keine Daten";
          }
          else if (result.hasData())
          {
            Data data = result.getData();
            wert = data.getItem(ss[1]).asTextValue().getText();
          }
          else
          {

          }

          String wertZeit = _sdf.format(new Date(result.getDataTime()));
          re = wert + " (" + wertZeit + ")";
        }
      }
      else
      {
        if (ss.length > 3)
        {
          AttributeGroup atg = connection.getDataModel().getAttributeGroup(ss[3]);

          Aspect asp = connection.getDataModel().getAspect("asp.klassifizierung");
          ResultData result = connection.getData(so, new DataDescription(atg, asp), 1000);
          String stufe = "";
          if (result.getDataState().equals(DataState.NO_SOURCE))
          {
            stufe = "keine Quelle";
          }
          else if (result.getDataState().equals(DataState.NO_DATA))
          {
            stufe = "keine Daten";
          }
          else if (result.hasData())
          {
            Data data = result.getData();
            stufe = data.getItem(ss[1]).asTextValue().getText();
          }
          else
          {

          }
          String stufeZeit = _sdf.format(new Date(result.getDataTime()));
          re = stufe + " (" + stufeZeit + ")";
        }
      }
    }
    return re;
  }

  private SystemObject bestimmeSswSbaUfdMsVonRealUfdms(SystemObject realUfdms)
  {

    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    SystemObjectType typeufdms2 = connection.getDataModel().getType("typ.sswSbaUmfeldDatenMessStelle");
    List<SystemObject> ufdmss2 = typeufdms2.getElements();

    for (SystemObject so : ufdmss2)
    {
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaUmfeldDatenMessStelleReal");
      Data data = so.getConfigurationData(atg);
      if (data != null)
      {
        Data item = data.getItem("UmfeldDatenMessStelleRealReferenz");
        if (item != null)
        {
          ReferenceValue rv = item.asReferenceValue();
          if (rv != null)
          {
            SystemObject ufdms = rv.getSystemObject();
            if (so.getPid().equals("sba.st.sba.uz.a14.ufds.A14.195.65.DD"))
            {
              ufdms = connection.getDataModel().getObject("st.sba.uz.a14.ufds.A14.195.65.DD");
            }

            if (ufdms != null && ufdms.equals(realUfdms))
            {
              return so;
            }
          }
        }
      }
    }
    return null;
  }

  private SystemObject bestimmeUfdMessStelleVonUfdSensor(SystemObject sensor)
  {

    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    SystemObjectType typeufdms = connection.getDataModel().getType("typ.umfeldDatenMessStelle");
    List<SystemObject> ufdmss = typeufdms.getElements();

    for (SystemObject so : ufdmss)
    {
      NonMutableSet set = ((ConfigurationObject)so).getNonMutableSet("UmfeldDatenSensoren");
      if (set != null)
      {
        List<SystemObject> elements = set.getElements();
        for (SystemObject so2 : elements)
        {
          if (so2.equals(sensor))
            return so;
        }
      }
    }
    return null;
  }
  
  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();     
      for (Object obj : getDefinition())
      {
        List<LegendeBaustein> bausteine = LegendeHelper.erzeugeLegendeBausteine(obj);        
        legendenBausteine.addAll(bausteine);        
      }
    }
    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
    return null;   
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return getModel().getDoTyp().getName();
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return getText();
  }

  @Override
  public List getDefinition()
  {
    List list = new ArrayList();
    list.add(LegendeDefinitionSbaUfdMessStelleNaesseStufe.class);
    return list;
  }

}
