/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnschlussStelleModel;
import de.bsvrz.buv.plugin.doeditor.model.LinienForm;

/**
 * Sektion zur Darstellung der Linienattribute einer {@link LinienForm} innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class LineAttributesSection extends AbstractDobjSection<AnschlussStelleModel>
{

  /** das Eingabeelement fr die Einstellung der Linienstrke. */
  private Spinner linienStaerke;


  @Override
  protected void aktualisiereDaten()
  {     
    final int newValue = Float.valueOf(getElement().getStaerke()).intValue();

    if (newValue != linienStaerke.getSelection())
    {
      linienStaerke.setSelection(Float.valueOf(getElement().getStaerke()).intValue());
    }
  }

  @Override
  public void createUI(final Composite container)
  {
    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    FormData data;

    final CLabel linienStaerkeLabel = factory.createCLabel(container, "Linienstrke:");
    data = new FormData();
    data.left = new FormAttachment(0);
    data.right = new FormAttachment(linienStaerkeLabel, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    linienStaerkeLabel.setLayoutData(data);

    linienStaerke = new Spinner(container, SWT.BORDER);
    data = new FormData();
    data.left = new FormAttachment(linienStaerkeLabel, 0);
    data.right = new FormAttachment(50, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(linienStaerkeLabel, 0, SWT.CENTER);
    linienStaerke.setLayoutData(data);
  }

  @Override
  protected void installListeners()
  {   
    linienStaerke.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        if (isAktiv())
        {
          int att = linienStaerke.getSelection();

          getElement().setStaerke(att);

          final SetCommand cmd = new SetCommand(getElement(),
              AnlagendarstellungPackage.Literals.ANSCHLUSS_STELLE_MODEL__STAERKE, att);
          getCommandStack().execute(cmd);
        }
      }
    });
  }
}
