/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.bitctrl.lib.eclipse.emf.gef.model.GefPackage;
import com.bitctrl.lib.eclipse.emf.gef.model.Located;

 
/**
 * Sektion zur Darstellung der Position eines Elements aus dem DOT-Editor innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class LocationSection extends AbstractDobjSection<Located>
{

  /** das Eingabefeld fr die Eingabe der X-Position. */
  private Spinner xPos;

  /** das Eingabefeld fr die Eingabe der Y-Position. */
  private Spinner yPos;

  @Override
  protected void aktualisiereDaten()
  {
    final Point location = getElement().getLocation();
    if (location != null)
    {
      if (location.x != xPos.getSelection())
      {
        xPos.setSelection(location.x);
      }
      if (location.y != yPos.getSelection())
      {
        yPos.setSelection(location.y);
      }
    }
    else
    {
      xPos.setSelection(0);
      yPos.setSelection(0);
    }
  }

  @Override
  public void createUI(final Composite parent)
  {
    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    FormData data;
    CLabel label;

    xPos = new Spinner(parent, SWT.BORDER);
    xPos.setMinimum(Integer.MIN_VALUE);
    xPos.setMaximum(Integer.MAX_VALUE);
    data = new FormData();
    data.left = new FormAttachment(0, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(50, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    xPos.setLayoutData(data);

    label = factory.createCLabel(parent, "X-Pos:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(xPos, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(xPos, 0, SWT.CENTER);
    label.setLayoutData(data);

    yPos = new Spinner(parent, SWT.BORDER);
    yPos.setMinimum(Integer.MIN_VALUE);
    yPos.setMaximum(Integer.MAX_VALUE);
    data = new FormData();
    data.left = new FormAttachment(xPos, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(xPos, 0, SWT.CENTER);
    yPos.setLayoutData(data);

    label = factory.createCLabel(parent, "Y-Pos:");
    data = new FormData();
    data.left = new FormAttachment(50, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(yPos, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(yPos, 0, SWT.CENTER);
    label.setLayoutData(data);
  }

  @Override
  protected void installListeners()
  {
    xPos.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        versendeDimension();
      }
    });
    yPos.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        versendeDimension();
      }
    });
  }

  /** versendet ein Kommando zur nderung der Position. */
  private void versendeDimension()
  {
    if (isAktiv())
    {
      final EStructuralFeature feature;
      feature = GefPackage.Literals.LOCATED__LOCATION;
      final Point location = new Point(xPos.getSelection(), yPos.getSelection());
      final SetCommand cmd = new SetCommand(getElement(), feature, location);
      getCommandStack().execute(cmd);
    }
  }
}
