/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.section;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.bitctrl.lib.eclipse.emf.util.EmfUtil;
import com.bitctrl.lib.eclipse.emf.viewers.EmfLabelProvider;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.dobj.model.DoKompositum;
import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;

/**
 * Sektion zur Darstellung eines DoModel mit Verbindungslinie in der Properties View.
 * 
 * @author Pittner
 * @param <T>
 */
public class DoModelMitVerbLinieSection extends AbstractSection<DoModelMitVerbLinie>
{
  private Text relativZuText;

  private Button doSelectButton;

  @Override
  public void createControls(final Composite parent, final TabbedPropertySheetPage aTabbedPropertySheetPage)
  {
    super.createControls(parent, aTabbedPropertySheetPage);

    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    final Composite container = factory.createFlatFormComposite(parent);

    FormData data;

    relativZuText = factory.createText(container, "");
    relativZuText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    relativZuText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Verbindung:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(relativZuText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(relativZuText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    doSelectButton = factory.createButton(container, "Ziel auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(relativZuText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(relativZuText, 0, SWT.CENTER);
    doSelectButton.setLayoutData(data);

    doSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        Ebene ebene = (Ebene)getElement().eContainer();
        Darstellung darstellung = ebene.getDarstellung();
        Ebene ebene0 = darstellung.getEbenen().get(0);

        final List<Object> contents = new ArrayList<Object>(ebene0.getDoObjekte());
        contents.remove(getElement());

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine anderen Darstellungsobjekte auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("Darstellungsobjekt auswhlen");
        dialog
            .setMessage("Bitte whlen ein anderes Darstellungsobjekt zu dem dieses relativ positioniert werden soll.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        DoModelMitVerbLinie targetOld = getElement().getTarget();

        if (targetOld != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(targetOld));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET;
          EStructuralFeature feature2 = AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET_OF;

          DoModelMitVerbLinie ziel = (DoModelMitVerbLinie)value;
          if (ziel != null)
          {            
              // Beim alten Ziel wird die Referenz auf die Quelle gelscht, da sich das Ziel bei der Quelle ndert.                
              getElement().setTarget(ziel);              
              final SetCommand cmd = new SetCommand(getElement(), feature, ziel);
              getCommandStack().execute(cmd);
              EList<DoModelMitVerbLinie> targetOf = ziel.getTargetOf();
              if (!targetOf.contains(getElement())){
                targetOf.add(getElement());
                final SetCommand cmd2 = new SetCommand(ziel, feature2, targetOf);
                getCommandStack().execute(cmd2);
              }
          }          
          else
          {
            // Ziel wird explizit gelscht, ohne dass ein neues Ziel angegeben wird
            getElement().setTarget(null);
            final SetCommand cmd = new SetCommand(getElement(), feature, null);
            getCommandStack().execute(cmd);            
            EList<DoModelMitVerbLinie> targetOf = targetOld.getTargetOf();
            if (targetOf.contains(getElement())){
              targetOf.remove(getElement());
              final SetCommand cmd2 = new SetCommand(targetOld, feature2, targetOf);
              getCommandStack().execute(cmd2);
            }
          }
        }
        refresh();
      }
    });
  }

  @Override
  public void refresh()
  {
    super.refresh();
    DoModelMitVerbLinie target = getElement().getTarget();
    if (target != null)
    {
      relativZuText.setText(EmfUtil.getText((DoModelMitVerbLinie)target));     
    }
    else
    {
      relativZuText.setText("");
    }
  }
}
