/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.section;

import java.util.ArrayList;
import java.util.Collections;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.bitctrl.lib.eclipse.emf.viewers.EmfLabelProvider;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;

/**
 * Sektion zur Darstellung eines KontextMenue in der Properties View.
 * 
 * @author Pittner
 * @param <T>
 */
public class KontextMenueSection extends AbstractSection<VerhaltenDoTyp>
{

  private Text actionSetIstZustandText;

  private Button istSelectButton;

  private Text actionSetSollZustandText;

  private Button sollSelectButton;

  private Text actionSetHandSchaltungText;

  private Button handSelectButton;

  private Text actionSetSimulationText;

  private Button simulationSelectButton;

  private Text actionSetTrichterText;

  private Button trichterSelectButton;

  private Text actionSetParameterText;

  private Button parameterSelectButton;

  @Override
  public void createControls(final Composite parent, final TabbedPropertySheetPage aTabbedPropertySheetPage)
  {
    super.createControls(parent, aTabbedPropertySheetPage);

    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    final Composite container = factory.createFlatFormComposite(parent);

    erzeugeGruppeIstZustand(factory, container);
    erzeugeGruppeSollZustand(factory, container);
    erzeugeGruppeHandSchaltung(factory, container);
    erzeugeGruppeSimulation(factory, container);
    erzeugeGruppeTrichter(factory, container);
    erzeugeGruppeParameter(factory, container);
  }

  private void erzeugeGruppeIstZustand(final TabbedPropertySheetWidgetFactory factory, final Composite container)
  {
    FormData data;
    actionSetIstZustandText = factory.createText(container, "");
    actionSetIstZustandText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    actionSetIstZustandText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Ist-Zustand:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(actionSetIstZustandText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(actionSetIstZustandText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    istSelectButton = factory.createButton(container, "Action auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(actionSetIstZustandText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(actionSetIstZustandText, 0, SWT.CENTER);
    istSelectButton.setLayoutData(data);

    istSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {

        VerhaltenDoTyp doTyp = getElement();
        String s = doTyp.getKontextMenueIdIstZustand();

        final java.util.List<Object> contents = new ArrayList<Object>();

        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] aConfigurationElementsFor = reg
            .getConfigurationElementsFor("org.eclipse.ui.actionSets");
        for (IConfigurationElement confEle : aConfigurationElementsFor)
        {
          String contributor = confEle.getContributor().getName();
          // if (contributor.equals(Activator.PLUGIN_ID))
          {
            String id = confEle.getAttribute("id");
            contents.add(id);
          }
        }

        contents.remove(s);

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine anderen ActionSets auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("ActionSet auswhlen");
        dialog.setMessage("Bitte whlen ein anderes ActionSet zur Auswahl des KontextMens.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.VERHALTEN_DO_TYP__KONTEXT_MENUE_ID_IST_ZUSTAND;
          getElement().setKontextMenueIdIstZustand((String)value);

          final SetCommand cmd = new SetCommand(getElement(), feature, (String)value);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
  }

  private void erzeugeGruppeSollZustand(final TabbedPropertySheetWidgetFactory factory, final Composite container)
  {
    FormData data;
    actionSetSollZustandText = factory.createText(container, "");
    actionSetSollZustandText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(actionSetIstZustandText, ITabbedPropertyConstants.VSPACE);
    actionSetSollZustandText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Soll-Zustand:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(actionSetSollZustandText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(actionSetSollZustandText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    sollSelectButton = factory.createButton(container, "Action auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(actionSetSollZustandText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(actionSetSollZustandText, 0, SWT.CENTER);
    sollSelectButton.setLayoutData(data);

    sollSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {

        VerhaltenDoTyp doTyp = getElement();
        String s = doTyp.getKontextMenueIdSollZustand();

        final java.util.List<Object> contents = new ArrayList<Object>();

        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] aConfigurationElementsFor = reg
            .getConfigurationElementsFor("org.eclipse.ui.actionSets");
        for (IConfigurationElement confEle : aConfigurationElementsFor)
        {
          String contributor = confEle.getContributor().getName();
          // if (contributor.equals(Activator.PLUGIN_ID))
          {
            String id = confEle.getAttribute("id");
            contents.add(id);
          }
        }

        contents.remove(s);

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine anderen ActionSets auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("ActionSet auswhlen");
        dialog.setMessage("Bitte whlen ein anderes ActionSet zur Auswahl des KontextMens.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.VERHALTEN_DO_TYP__KONTEXT_MENUE_ID_SOLL_ZUSTAND;
          getElement().setKontextMenueIdSollZustand((String)value);

          final SetCommand cmd = new SetCommand(getElement(), feature, (String)value);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
  }

  private void erzeugeGruppeHandSchaltung(final TabbedPropertySheetWidgetFactory factory, final Composite container)
  {
    FormData data;
    actionSetHandSchaltungText = factory.createText(container, "");
    actionSetHandSchaltungText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(actionSetSollZustandText, ITabbedPropertyConstants.VSPACE);
    actionSetHandSchaltungText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Handschaltung:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(actionSetHandSchaltungText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(actionSetHandSchaltungText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    handSelectButton = factory.createButton(container, "Action auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(actionSetHandSchaltungText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(actionSetHandSchaltungText, 0, SWT.CENTER);
    handSelectButton.setLayoutData(data);

    handSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {

        VerhaltenDoTyp doTyp = getElement();
        String s = doTyp.getKontextMenueIdHandSchaltung();

        final java.util.List<Object> contents = new ArrayList<Object>();

        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] aConfigurationElementsFor = reg
            .getConfigurationElementsFor("org.eclipse.ui.actionSets");
        for (IConfigurationElement confEle : aConfigurationElementsFor)
        {
          String contributor = confEle.getContributor().getName();
          // if (contributor.equals(Activator.PLUGIN_ID))
          {
            String id = confEle.getAttribute("id");
            contents.add(id);
          }
        }

        contents.remove(s);

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine anderen ActionSets auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("ActionSet auswhlen");
        dialog.setMessage("Bitte whlen ein anderes ActionSet zur Auswahl des KontextMens.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.VERHALTEN_DO_TYP__KONTEXT_MENUE_ID_HAND_SCHALTUNG;
          getElement().setKontextMenueIdHandSchaltung((String)value);

          final SetCommand cmd = new SetCommand(getElement(), feature, (String)value);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
  }

  private void erzeugeGruppeSimulation(final TabbedPropertySheetWidgetFactory factory, final Composite container)
  {
    FormData data;
    actionSetSimulationText = factory.createText(container, "");
    actionSetSimulationText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(actionSetHandSchaltungText, ITabbedPropertyConstants.VSPACE);
    actionSetSimulationText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Simulation:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(actionSetSimulationText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(actionSetSimulationText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    simulationSelectButton = factory.createButton(container, "Action auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(actionSetSimulationText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(actionSetSimulationText, 0, SWT.CENTER);
    simulationSelectButton.setLayoutData(data);

    simulationSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {

        VerhaltenDoTyp doTyp = getElement();
        String s = doTyp.getKontextMenueIdSimulation();

        final java.util.List<Object> contents = new ArrayList<Object>();

        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] aConfigurationElementsFor = reg
            .getConfigurationElementsFor("org.eclipse.ui.actionSets");
        for (IConfigurationElement confEle : aConfigurationElementsFor)
        {
          String contributor = confEle.getContributor().getName();
          // if (contributor.equals(Activator.PLUGIN_ID))
          {
            String id = confEle.getAttribute("id");
            contents.add(id);
          }
        }

        contents.remove(s);

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine anderen ActionSets auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("ActionSet auswhlen");
        dialog.setMessage("Bitte whlen ein anderes ActionSet zur Auswahl des KontextMens.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.VERHALTEN_DO_TYP__KONTEXT_MENUE_ID_SIMULATION;
          getElement().setKontextMenueIdSimulation((String)value);

          final SetCommand cmd = new SetCommand(getElement(), feature, (String)value);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
  }

  private void erzeugeGruppeTrichter(final TabbedPropertySheetWidgetFactory factory, final Composite container)
  {
    FormData data;
    actionSetTrichterText = factory.createText(container, "");
    actionSetTrichterText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(actionSetSimulationText, ITabbedPropertyConstants.VSPACE);
    actionSetTrichterText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Trichter:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(actionSetTrichterText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(actionSetTrichterText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    trichterSelectButton = factory.createButton(container, "Action auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(actionSetTrichterText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(actionSetTrichterText, 0, SWT.CENTER);
    trichterSelectButton.setLayoutData(data);

    trichterSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {

        VerhaltenDoTyp doTyp = getElement();
        String s = doTyp.getKontextMenueIdTrichter();

        final java.util.List<Object> contents = new ArrayList<Object>();

        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] aConfigurationElementsFor = reg
            .getConfigurationElementsFor("org.eclipse.ui.actionSets");
        for (IConfigurationElement confEle : aConfigurationElementsFor)
        {
          String contributor = confEle.getContributor().getName();
          // if (contributor.equals(Activator.PLUGIN_ID))
          {
            IConfigurationElement[] iceAction = confEle.getChildren("action");
            for (IConfigurationElement element : iceAction)
            {
              String id = element.getAttribute("id");
              contents.add(id);
            }
          }
        }

        contents.remove(s);

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine anderen ActionSets auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("ActionSet auswhlen");
        dialog.setMessage("Bitte whlen ein anderes ActionSet zur Auswahl des KontextMens.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.VERHALTEN_DO_TYP__KONTEXT_MENUE_ID_TRICHTER;
          getElement().setKontextMenueIdTrichter((String)value);

          final SetCommand cmd = new SetCommand(getElement(), feature, (String)value);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
  }

  private void erzeugeGruppeParameter(final TabbedPropertySheetWidgetFactory factory, final Composite container)
  {
    FormData data;
    actionSetParameterText = factory.createText(container, "");
    actionSetParameterText.setEditable(false);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(80, 0);
    data.top = new FormAttachment(actionSetTrichterText, ITabbedPropertyConstants.VSPACE);
    actionSetParameterText.setLayoutData(data);

    final CLabel label4 = factory.createCLabel(container, "Parameter:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(actionSetParameterText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(actionSetParameterText, 0, SWT.CENTER);
    label4.setLayoutData(data);

    parameterSelectButton = factory.createButton(container, "Action auswhlen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(actionSetParameterText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(actionSetParameterText, 0, SWT.CENTER);
    parameterSelectButton.setLayoutData(data);

    parameterSelectButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {

        VerhaltenDoTyp doTyp = getElement();
        String s = doTyp.getDoppelKlickIdParameter();

        final java.util.List<Object> contents = new ArrayList<Object>();

        IExtensionRegistry reg = Platform.getExtensionRegistry();
        IConfigurationElement[] aConfigurationElementsFor = reg
            .getConfigurationElementsFor("org.eclipse.ui.actionSets");
        for (IConfigurationElement confEle : aConfigurationElementsFor)
        {
          String contributor = confEle.getContributor().getName();
          // if (contributor.equals(Activator.PLUGIN_ID))
          {
            IConfigurationElement[] iceAction = confEle.getChildren("action");
            for (IConfigurationElement element : iceAction)
            {
              String id = element.getAttribute("id");
              contents.add(id);
            }
          }
        }

        contents.remove(s);

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
              "Es gibt keine andere Action auf die referenziert werden kann.");
          return;
        }

        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new EmfLabelProvider());
        dialog.setTitle("Action auswhlen");
        dialog.setMessage("Bitte whlen Sie eine andere Action zur Auswahl des KontextMens.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());

        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }

          EStructuralFeature feature = AnlagendarstellungPackage.Literals.VERHALTEN_DO_TYP__KONTEXT_MENUE_ID_PARAMETER;
//          getElement().setDoppelKlickIdParameter((String)value);
          getElement().setKontextMenueIdParameter((String)value);

          final SetCommand cmd = new SetCommand(getElement(), feature, (String)value);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
  }

  @Override
  public void refresh()
  {
    String s1 = getElement().getKontextMenueIdIstZustand();
    if (s1 != null)
    {
      actionSetIstZustandText.setText(s1);
    }
    else
    {
      actionSetIstZustandText.setText("");
    }

    String s2 = getElement().getKontextMenueIdSollZustand();
    if (s2 != null)
    {
      actionSetSollZustandText.setText(s2);
    }
    else
    {
      actionSetSollZustandText.setText("");
    }

    String s3 = getElement().getKontextMenueIdHandSchaltung();
    if (s3 != null)
    {
      actionSetHandSchaltungText.setText(s3);
    }
    else
    {
      actionSetHandSchaltungText.setText("");
    }

    String s4 = getElement().getKontextMenueIdSimulation();
    if (s4 != null)
    {
      actionSetSimulationText.setText(s4);
    }
    else
    {
      actionSetSimulationText.setText("");
    }

    String s5 = getElement().getKontextMenueIdTrichter();
    if (s5 != null)
    {
      actionSetTrichterText.setText(s5);
    }
    else
    {
      actionSetTrichterText.setText("");
    }

    String s6 = getElement().getKontextMenueIdParameter();
    if (s6 != null)
    {
      actionSetParameterText.setText(s6);
    }
    else
    {
      actionSetParameterText.setText("");
    }
  }

}
