package de.bsvrz.buv.plugin.anlagendarstellung.dobj.section;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Status;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.preference.ColorSelector;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ViewZuordnungKilometrierung;
import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Sektion zum Bearbeiten des {@link PickerVertical}-Daten.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class SectionPickerParent extends AbstractSection<PickerParent>
{
	/**
	 * Feld zur Erfassung der Breite.
	 */
	private Text _tBreite;

	/**
	 * Feld zur Erfassung der Hhe.
	 */
	private Text _tHoehe;

	/**
	 * Feld zur Erfassung der Distanz zwischen den Pfeilspitzen.
	 */
	private Text _tDistanz;

	/**
	 * Feld zur Erfassung der initialen Y-Position der oberen Pfeilspitze
	 */
	private Text _tYPosPickerTop;

	/**
	 * Feld zur Erfassung der Zuordnung von Kilometrierung zu Pixeln
	 */
	private Text _tKilometrierungZuordnung;

	/**
	 * Checkbox die festlegt, ob Ticks gezeichnet werden sollen oder nicht.
	 */
	private Button _chkPaintTicks = null;

	private boolean _modifiyListenerAktiv = true;

	private String sndId4ViewZuordnungKilometrierung = null;

  private CCombo fahrtrichtung = null;

  private ColorSelector _csPickerParent;

  private CCombo _comboObjektTyp;

  private ComboViewer _cvObjektTyp;

  private ColorSelector _csPickerParent2;

  private CCombo transparenz;

	@Override
	public void createControls(Composite parent,
			TabbedPropertySheetPage aTabbedPropertySheetPage)
	{
		super.createControls(parent, aTabbedPropertySheetPage);

		sndId4ViewZuordnungKilometrierung = "" + System.currentTimeMillis();

		final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
		final Composite container = factory.createFlatFormComposite(parent);

		/*
		 * GUI-Elemente erzeugen
		 */
		CLabel lBreite = factory.createCLabel(container, "Breite:");
		FormData fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(0, 0);
		lBreite.setLayoutData(fd);

		_tBreite = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lBreite, 0);
		fd.top = new FormAttachment(0, 0);
		fd.right = new FormAttachment(25, 0);
		_tBreite.setLayoutData(fd);

		CLabel lHoehe = factory.createCLabel(container, "Hhe:");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lBreite, 0);
		lHoehe.setLayoutData(fd);

		_tHoehe = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lHoehe, 0);
		fd.top = new FormAttachment(_tBreite, 0);
		fd.right = new FormAttachment(25, 0);
		_tHoehe.setLayoutData(fd);

		CLabel lDistanz = factory.createCLabel(container, "Distanz:");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lHoehe, 0);
		lDistanz.setLayoutData(fd);

		_tDistanz = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lDistanz, 0);
		fd.top = new FormAttachment(_tHoehe, 0);
		fd.right = new FormAttachment(25, 0);
		_tDistanz.setLayoutData(fd);

		CLabel lYPosPickerTop = factory
				.createCLabel(container, "Y-Pos Picker Top:");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lDistanz, 0);
		lYPosPickerTop.setLayoutData(fd);

		_tYPosPickerTop = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lYPosPickerTop, 0);
		fd.top = new FormAttachment(_tDistanz, 0);
		fd.right = new FormAttachment(25, 0);
		_tYPosPickerTop.setLayoutData(fd);

		CLabel lKilometrierung = factory.createCLabel(container, "Kilometrierung");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lYPosPickerTop, 0);
		lKilometrierung.setLayoutData(fd);

		_tKilometrierungZuordnung = factory.createText(container, "");
		_tKilometrierungZuordnung.setEditable(false);
		fd = new FormData();
		fd.left = new FormAttachment(lKilometrierung, 0);
		fd.top = new FormAttachment(_tYPosPickerTop, 0);
		fd.right = new FormAttachment(25, 0);
		_tKilometrierungZuordnung.setLayoutData(fd);

		Button bKM = factory.createButton(container, "Editieren", SWT.PUSH);
		fd = new FormData();
		fd.left = new FormAttachment(_tKilometrierungZuordnung, 0);
		fd.top = new FormAttachment(_tYPosPickerTop, 0);
		fd.right = new FormAttachment(40, 0);
		bKM.setLayoutData(fd);
		bKM.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				actionKilometrierungYPosEditieren();
			}

		});

		// Ticks (de-)aktivieren
		_chkPaintTicks = factory.createButton(container, "Ticks zeichnen?",
				SWT.CHECK);
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lKilometrierung, 0);
		_chkPaintTicks.setLayoutData(fd);
		

    final CLabel label3 = factory.createCLabel(container, "Fahrtrichtung:");
    FormData data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.top = new FormAttachment(_chkPaintTicks, 0);
    label3.setLayoutData(data);
    
    fahrtrichtung = factory.createCCombo(container);
    fahrtrichtung.add("Nord", 0);
    fahrtrichtung.add("Sd", 1);
    data = new FormData();
    data = new FormData();  
    data.left = new FormAttachment(label3, 0);
    data.top = new FormAttachment(_chkPaintTicks, 0);
    data.right = new FormAttachment(25, 0);
    fahrtrichtung.setLayoutData(data);

    fahrtrichtung.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
        if (fahrtrichtung.getSelectionIndex() != getElement().getFahrtrichtung())
        {
          // XXX: Der Wert wurde in der PropertyPage gendert, also
          // versenden wir ein neues Command an das Modell-Objekt
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__FAHRTRICHTUNG;

          int i = fahrtrichtung.getSelectionIndex();
          getElement().setFahrtrichtung(i);

          final SetCommand cmd = new SetCommand(getElement(), feature, i);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
    
    final CLabel label31 = factory.createCLabel(container, "ObjektTyp:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.top = new FormAttachment(fahrtrichtung, 0);
    label31.setLayoutData(data);
    
    _comboObjektTyp = factory.createCCombo(container);
    _cvObjektTyp = new ComboViewer(_comboObjektTyp);
    _cvObjektTyp.setLabelProvider(new LabelProvider());
    _cvObjektTyp.setContentProvider(new ArrayContentProvider());
    
    
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null){
      
      List<SystemObject> input = new ArrayList<SystemObject>();
      input.add(connection.getDataModel().getType("typ.baustelle"));
      input.add(connection.getDataModel().getType("typ.unfall"));
      input.add(connection.getDataModel().getType("typ.stau"));
      _cvObjektTyp.setInput(input);
    }
    
      
    data = new FormData();  
    data.left = new FormAttachment(label31, 0);
    data.top = new FormAttachment(fahrtrichtung, 0);
    data.right = new FormAttachment(25, 0);
    _comboObjektTyp.setLayoutData(data);
    
    _cvObjektTyp.addSelectionChangedListener(new ISelectionChangedListener() {
      
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        // TODO Auto-generated method stub
        if (isModifiyListenerAktiv())
        {
          ISelection sel = event.getSelection();
          if (sel instanceof StructuredSelection)
          {
            StructuredSelection ss = (StructuredSelection)sel;
            Object fe = ss.getFirstElement();
            if (fe instanceof SystemObject)
            {
              SystemObject so = (SystemObject)fe;
              // XXX: Der Wert wurde in der PropertyPage gendert, also
              // versenden wir ein neues Command an das Modell-Objekt
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__OBJEKT_TYP;            
              getElement().setObjektTyp(so.getPid());
              
              final SetCommand cmd = new SetCommand(getElement(), feature, so.getPid());
              getCommandStack().execute(cmd);              
            }            
          }          
        }        
      }
    });
    
    
    final CLabel label4 = factory.createCLabel(container, "Farbe (Vordergrund):");   
//    Composite compColor = factory.createComposite(container, SWT.BORDER);    
    Composite flatFormComposite = factory.createFlatFormComposite(container);    
    _csPickerParent = new ColorSelector(flatFormComposite);
    _csPickerParent.setColorValue(ColorConstants.black.getRGB());
    _csPickerParent.addListener(new IPropertyChangeListener() {
      
      @Override
      public void propertyChange(PropertyChangeEvent event)
      {
        System.out.println("SourceCS2: " + event);        
        if (isModifiyListenerAktiv())
        {
          // TODO Auto-generated method stub           
          
          Object newValue = event.getNewValue();
          Object oldValue = event.getOldValue();
          if (newValue instanceof RGB)
          {
            RGB rgb = (RGB)newValue;    
            // XXX: Der Wert wurde in der PropertyPage gendert, also
            // versenden wir ein neues Command an das Modell-Objekt
            if (!newValue.equals(oldValue)){
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__FARBE_VORDERGRUND;            
              getElement().setFarbeVordergrund(rgb);
              
              
              System.out.println("SectionPickerParent.{...}.propertyChange(rgb): "  + rgb);
              
              
              final SetCommand cmd = new SetCommand(getElement(), feature, rgb);
              getCommandStack().execute(cmd);            
            }
          }          
        }
      }
    });
     		
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.top = new FormAttachment(_comboObjektTyp, 0);
    label4.setLayoutData(data);

    data = new FormData();  
    data.left = new FormAttachment(label4, 0);
    data.top = new FormAttachment(_comboObjektTyp, 0);
    data.right = new FormAttachment(40, 0);
    flatFormComposite.setLayoutData(data);
    
    final CLabel label5 = factory.createCLabel(container, "Farbe (Hintergrund):");   
//    Composite compColor = factory.createComposite(container, SWT.BORDER);    
    Composite flatFormComposite2 = factory.createFlatFormComposite(container);    
    _csPickerParent2 = new ColorSelector(flatFormComposite2);
    _csPickerParent2.setColorValue(ColorConstants.lightGray.getRGB());
    _csPickerParent2.addListener(new IPropertyChangeListener() {
      
      @Override
      public void propertyChange(PropertyChangeEvent event)
      {
        System.out.println("SourceCS2: " + event);
        if (isModifiyListenerAktiv())
        {
          // TODO Auto-generated method stub           
          
          Object newValue = event.getNewValue();
          Object oldValue = event.getOldValue();
          if (newValue instanceof RGB)
          {
            RGB rgb = (RGB)newValue;    
            // XXX: Der Wert wurde in der PropertyPage gendert, also
            // versenden wir ein neues Command an das Modell-Objekt
            if (!newValue.equals(oldValue)){
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__FARBE_HINTERGRUND;            
              getElement().setFarbeHintergrund(rgb);
              
              
              System.out.println("SectionPickerParent.{...}.propertyChange(rgbHtg): "  + rgb);
              
              
              final SetCommand cmd = new SetCommand(getElement(), feature, rgb);
              getCommandStack().execute(cmd);            
            }
          }          
        }
      }
    });
    
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.top = new FormAttachment(flatFormComposite, 0);
    label5.setLayoutData(data);
    
    data = new FormData();  
    data.left = new FormAttachment(label5, 0);
    data.top = new FormAttachment(flatFormComposite, 0);
    data.right = new FormAttachment(40, 0);
    flatFormComposite2.setLayoutData(data);
    
    final CLabel label6 = factory.createCLabel(container, "Transparenz (Hintergrund):");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.top = new FormAttachment(flatFormComposite2, 0);
    label6.setLayoutData(data);
    
    transparenz = factory.createCCombo(container);
    for (int i = 0; i < 255; i++)
    {
      transparenz.add("" + i, i);      
    }
    
    
    data = new FormData();  
    data.left = new FormAttachment(label6, 0);
    data.top = new FormAttachment(flatFormComposite2, 0);
    data.right = new FormAttachment(25, 0);
    transparenz.setLayoutData(data);
    
    transparenz.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
        if (transparenz.getSelectionIndex() != getElement().getFahrtrichtung())
        {
          // XXX: Der Wert wurde in der PropertyPage gendert, also
          // versenden wir ein neues Command an das Modell-Objekt
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__TRANSPARENZ;

          String s = transparenz.getText();
          Integer i = new Integer(s);          
          getElement().setTransparenz(i);

          final SetCommand cmd = new SetCommand(getElement(), feature, i);
          getCommandStack().execute(cmd);
        }
        refresh();
      }
    });
   
		/*
		 * Verify-Listener anmelden.
		 */
		_tBreite.addListener(SWT.Verify, getNumberOnlyListener());
		_tHoehe.addListener(SWT.Verify, getNumberOnlyListener());
		_tDistanz.addListener(SWT.Verify, getNumberOnlyListener());
		_tYPosPickerTop.addListener(SWT.Verify, getNumberOnlyListener());

		/*
		 * Modify-Listener
		 */
		_tBreite.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerParent element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int asInt = getAsInt(str);

							if (asInt != element.getBreite())
							{
								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__BREITE;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										asInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_tHoehe.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerParent element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int asInt = getAsInt(str);

							if (asInt != element.getBreite())
							{
								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__HOEHE;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										asInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_tDistanz.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerParent element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int distanzAsInt = getAsInt(str);

							if (distanzAsInt != element.getDistanz())
							{

								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__DISTANZ;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										distanzAsInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_tYPosPickerTop.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerParent element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int asInt = getAsInt(str);

							if (asInt != element.getYPosPickerTop())
							{
								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__YPOS_PICKER_TOP;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										asInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_chkPaintTicks.addSelectionListener(new SelectionListener()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					if (_chkPaintTicks != null)
					{
						Boolean selection = new Boolean(_chkPaintTicks.getSelection());
						EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__TICKS_ANZEIGEN;
						final SetCommand cmd = new SetCommand(getElement(), feature,
								selection);
						getCommandStack().execute(cmd);
					}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
	}

	/**
	 * Validierung der eingegbenen Werte.
	 * 
	 * @param hoehe
	 *          Hhe der Figure
	 * @param yPosPickerTop
	 *          Position des oberen Pickers
	 * @param distance
	 *          Distance zum unteren Picker
	 * @return Liefert <code>true</code> wenn validiert, sonst <code>false</code>.
	 */
	public boolean validate(int hoehe, int yPosPickerTop, int distance)
	{
		boolean validated = false;

		if ((yPosPickerTop >= 0) && (yPosPickerTop <= hoehe))
		{
			if ((yPosPickerTop + distance) <= hoehe)
			{
				validated = true;
			}
		}

		return validated;
	}

	/**
	 * Liefert einen Listener fr {@link SWT#Verify}, der nur die Eingaben der
	 * Zahlen 0-9 zulsst.
	 * 
	 * @return
	 */
	public static Listener getNumberOnlyListener()
	{
		Listener numberOnlyListener = new Listener()
		{
			public void handleEvent(Event e)
			{
				String string = e.text;
				char[] chars = new char[string.length()];
				string.getChars(0, chars.length, chars, 0);
				for (int i = 0; i < chars.length; i++)
				{
					if (!('0' <= chars[i] && chars[i] <= '9'))
					{
						e.doit = false;
						return;
					}
				}
			}
		};

		return numberOnlyListener;
	}

	@Override
	public void refresh()
	{
		PickerParent pickerParent = getElement();
		if (pickerParent != null)
		{
			setModifiyListenerAktiv(false);
			if (_tBreite != null)
			{
				String breiteNeu = "" + pickerParent.getBreite();
				if (!_tBreite.getText().equals(breiteNeu))
				{
					_tBreite.setText(breiteNeu);
				}
			}

			if (_tHoehe != null)
			{
				String hoeheNeu = "" + pickerParent.getHoehe();
				if (!_tHoehe.getText().equals(hoeheNeu))
				{
					_tHoehe.setText(hoeheNeu);
				}
			}

			if (_tDistanz != null)
			{
				String distanzNeu = "" + pickerParent.getDistanz();
				if (!_tDistanz.getText().equals(distanzNeu))
				{
					_tDistanz.setText(distanzNeu);
				}
			}

			if (_tYPosPickerTop != null)
			{
				String yPosPickerTopNeu = "" + pickerParent.getYPosPickerTop();
				if (!_tYPosPickerTop.getText().equals(yPosPickerTopNeu))
				{
					_tYPosPickerTop.setText(yPosPickerTopNeu);
				}
			}

			if (_tKilometrierungZuordnung != null)
			{
				String kilometrierungZuordnung = pickerParent
						.getKilometrierungZuordnung();

				if ((kilometrierungZuordnung != null)
						&& (kilometrierungZuordnung.length() > 0))
				{
					_tKilometrierungZuordnung.setText("Gesetzt");
				} else
				{
					_tKilometrierungZuordnung.setText("Nicht Gesetzt");
				}

			}

			if (_chkPaintTicks != null)
			{
				_chkPaintTicks.setSelection(pickerParent.isTicksAnzeigen());
			}
			
			if (fahrtrichtung != null){
			  int fr = pickerParent.getFahrtrichtung();
			  fahrtrichtung.select(fr);			  
			}
			
			if (_cvObjektTyp != null){
			  String pid = pickerParent.getObjektTyp();
			  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
			  if (pid != null && connection != null){
			    SystemObject so = connection.getDataModel().getObject(pid);
			    _cvObjektTyp.setSelection(new StructuredSelection(so));			  
			  }
			}
			
			if (_csPickerParent != null){
			  RGB rgb = pickerParent.getFarbeVordergrund();
			  System.out.println("SectionPickerParent.refresh(rgb): " + rgb);
			  
			  if (rgb != null){			     
			    _csPickerParent.setColorValue(rgb);	  
			  }else{
			    _csPickerParent2.setColorValue(ColorConstants.lightGray.getRGB());   
			  }
			}
			
			if (_csPickerParent2 != null){
			  RGB rgb = pickerParent.getFarbeHintergrund();
			  System.out.println("SectionPickerParent.refresh(rgbHtg): " + rgb);
			  
			  if (rgb != null){			     
			    _csPickerParent2.setColorValue(rgb);	  
			  }else{
			    _csPickerParent2.setColorValue(ColorConstants.lightGray.getRGB());	  			    
			  }
			}
			
			if (transparenz != null){
			  int alpha = pickerParent.getTransparenz();
			  System.out.println("SectionPickerParent.refresh(alpha): " + alpha);
			  
			  if (alpha >= 0){
			    transparenz.select(alpha);
			  }else{
			    transparenz.select(255);	  			    
			  }
			}
			

			setModifiyListenerAktiv(true);
		}
	}
	private static int getAsInt(String str)
	{
		int re = 0;

		if ((str != null) && (str.length() > 0))
		{
			try
			{
				re = Integer.parseInt(str);
			} catch (NumberFormatException nfe)
			{
			}
		}

		return re;
	}
	private boolean isModifiyListenerAktiv()
	{
		return _modifiyListenerAktiv;
	}
	private void setModifiyListenerAktiv(boolean modifiyListenerAktiv)
	{
		_modifiyListenerAktiv = modifiyListenerAktiv;
	}

	private void actionKilometrierungYPosEditieren()
	{
		CommandStack commandStack = getCommandStack();
		PickerParent pickerVertical = getElement();

		IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();

		try
		{
			window.getActivePage().showView(ViewZuordnungKilometrierung.ID,
					sndId4ViewZuordnungKilometrierung, IWorkbenchPage.VIEW_VISIBLE);

			IViewReference reference = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage()
					.findViewReference(ViewZuordnungKilometrierung.ID,
							sndId4ViewZuordnungKilometrierung);
			IViewPart view = reference.getView(true);
			if (view instanceof ViewZuordnungKilometrierung)
			{
				ViewZuordnungKilometrierung re = (ViewZuordnungKilometrierung) view;
				re.setCommandStackAndModel2(commandStack, pickerVertical);
				window.getActivePage().activate(re);
			}

		} catch (PartInitException e)
		{
			String msg = "Die ViewAusgabefenster (ID: "
					+ ViewZuordnungKilometrierung.ID + ", SecId: "
					+ sndId4ViewZuordnungKilometrierung
					+ ") konnte nicht geffnet werden!.";

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, msg, e));
		}

	}

}
