/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.Timer;
import java.util.TreeMap;

import org.eclipse.emf.common.util.EList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Definitionsdialog zur Auswahl der verfgbaren Anzeigeinhalte. <br>
 * 
 * <br>
 * <br>
 * 
 * @version $Revision: 1.3 $ / $Date: 2014/10/30 13:01:14 $ / ($Author: Pittner $)
 * @author Pittner
 * 
 */
public class AnzeigeInhaltAuswahlDialog extends Dialog
{

  /**
   * Singleton-Instanz
   */
  private static AnzeigeInhaltAuswahlDialog _instanz = null;

  private Shell shell = null;

  /**
   * Speichert die in diesem Dialog erfassten Daten.
   */
  private List<AnzeigeInhaltTyp> _inhalte = null;

  private AnzeigeInhaltTyp _inhalt = null;

  private Label[] _labels;

  private MouseAdapter _mouseAdapter;

  private MouseTrackListener _mouseTrackListener;

  private FocusAdapter _focusAdapter;

  private boolean _schalten = true;

  /**
   * Das Timer Objekt
   */
  private Timer timer = new Timer();

  private AnzeigeElementTyp _typ;
 
  /**
   * Liste der gestrten Inhalte, wenn vorhanden
   */
  private List<String> m_listeGestoerteInhalte = null;
  /**
   * SystemObjekt der Anzeige, fr Zugriff auf gestrte Inhalte.
   */
  private SystemObject m_anzeigeObjekt = null;
  /**
   * Datenverteilerverbindung.
   */
  private ClientDavInterface m_connection = null;
  
  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link AnzeigeInhaltAuswahlDialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public AnzeigeInhaltAuswahlDialog(Shell parent)
  {
    super(parent);
    _instanz = this;
  }

  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link AnzeigeInhaltAuswahlDialog}.
   * 
   * @param parent
   * @param schalten
   */
  public AnzeigeInhaltAuswahlDialog(Shell parent, boolean schalten)
  {
    super(parent);
    _instanz = this;
    _schalten = schalten;
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Anzeigen der verfgbaren Inhalte
   * 
   * @return den Inhalt-Typ
   */
  public AnzeigeInhaltTyp oeffne(AnzeigeElementTyp typ, EList<?> bildInhalte)
  {
    _typ = typ;

    SortedMap<String, AnzeigeInhaltTyp> map = new TreeMap<String, AnzeigeInhaltTyp>();

    for (Object object : bildInhalte)
    {
      if (object instanceof String)
      {
        String s = (String)object;
        AnzeigeInhaltTyp inhalt = AnzeigeInhaltTyp.setBildInhalt(s, null);
        if (inhalt != null)
        {
          map.put(s, inhalt);
        }
      }
    }
    _inhalte = new ArrayList<AnzeigeInhaltTyp>();
    _inhalte.addAll(map.values());

    shell = new Shell(getParent(), SWT.NONE);    

    _focusAdapter = new FocusAdapter() {

      @Override
      public void focusLost(FocusEvent e)
      {
        dispose();
      }
    };

    shell.addFocusListener(_focusAdapter);

    _mouseTrackListener = new MouseTrackListener() {

      @Override
      public void mouseEnter(MouseEvent e)
      {
      }

      @Override
      public void mouseExit(MouseEvent e)
      {

        if (e.getSource() instanceof Shell)
        {

          Shell s = (Shell)e.getSource();
          Rectangle ca = s.getClientArea();

          if (true) // Dialog bei Verlassen mit Mauszeiger schliessen
          {
            if (e.x < 0 || e.x >= ca.width || e.y < 0 || e.y >= ca.height)
            // if (!s.getBounds().contains(e.x, e.y))
            {
              dispose();
            }
          }
          else
          // Mauszeiger am Verlassen des Dialogbereiches hindern (schlechtes Verhalten bei Eclipse !!!)
          {
            if (e.x < 0 || e.x >= ca.width || e.y < 0 || e.y >= ca.height)
            // if (!s.getBounds().contains(e.x, e.y))
            {
              int x = e.x;
              int y = e.y;

              if (e.x <= 0)
              {
                x = 2;
              }

              if (e.x >= ca.width)
              {
                x = ca.width - 3;
              }

              if (e.y <= 0)
              {
                y = 2;
              }

              if (e.y >= ca.height)
              {
                y = ca.height - 3;
              }

              Point p = shell.toDisplay(x, y);

              Display.getCurrent().setCursorLocation(p);
            }
          }

        }
        else if (e.getSource() instanceof Label)
        {          
          Point pt = shell.toDisplay(e.x, e.y);
          timer.schedule(new SchliesseZeitGesteuert(_instanz, pt), 100);

        }
      }

      @Override
      public void mouseHover(MouseEvent e)
      {
        
      }
    };

    shell.addMouseTrackListener(_mouseTrackListener);

    RowLayout rowLayout = new RowLayout();
    rowLayout.type = org.eclipse.swt.SWT.HORIZONTAL;

    rowLayout.wrap = true;

    shell.setLayout(rowLayout);

    createCompositeImage();

    Display display = shell.getDisplay();
    Point p = display.getCursorLocation();

    Rectangle rectangle = new Rectangle(p.x - 10, p.y - 10, 211, 107);
    if (typ != null
        && (typ.equals(AnzeigeElementTyp.WZG_C2) || typ.equals(AnzeigeElementTyp.PR_W) || typ
            .equals(AnzeigeElementTyp.PR_W1) || typ.equals(AnzeigeElementTyp.PR_W11)))
    {
      rectangle = new Rectangle(p.x - 10, p.y - 10, 211, 207);
    }

    shell.setBounds(rectangle);

    shell.open();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _inhalt;

  }

  /**
   * This method initializes compositeImage
   * 
   */
  private void createCompositeImage()
  {
    _mouseAdapter = new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        if (e.button == 1)
        {
          if (_schalten)
          {
            Label l = (Label)e.widget;
            AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)l.getData();
            _inhalt = inhalt;
          }
          dispose();
        }
      }
    };

    _labels = new Label[_inhalte.size()];

    int i = 0;

    for (AnzeigeInhaltTyp inhalt : _inhalte)
    {
      _labels[i] = new Label(shell, SWT.None);
      _labels[i].setToolTipText(inhalt.getType());
      _labels[i].addMouseListener(_mouseAdapter);
      _labels[i].addMouseTrackListener(_mouseTrackListener);

      String path = AnzeigeElementFigure.createImagePath(_typ, inhalt);
      
      if (path != null)
      {
        boolean gestoert = false;
        
        if (m_listeGestoerteInhalte != null)
        {
          for (String gestoerterInhalt : m_listeGestoerteInhalte)
          {
            if (gestoerterInhalt.equals(inhalt.getType()))
            {
              gestoert = true;
              
              break;
            }
          }
        }
        
        Image image = ImageFactory.getImage(path, gestoert); // 2. Argument legt fest, ob das Bild nicht darstellbar ist
        _labels[i].setData(inhalt);
        _labels[i].setImage(image);
        i++;
      }
    }
  }

  public void dispose()
  {
    if (!shell.isDisposed())
    {
      shell.removeFocusListener(_focusAdapter);
      shell.removeMouseTrackListener(_mouseTrackListener);
      for (Label l : _labels)
      {
        if (l != null)
        {
          l.removeMouseListener(_mouseAdapter);
          l.removeMouseTrackListener(_mouseTrackListener);
        }
      }
      shell.dispose();
    }
  }

  public Shell getShell()
  {
    return shell;
  }

  public void setShell(Shell shell)
  {
    this.shell = shell;
  }

//  /**
//   * Setzt das SystemObjekt der Anzeige, fr Zugriff auf gestrte Inhalte und die Datenverteilerverbindung.
//   * 
//   * @param anzeigeObjekt SystemObjekt der Anzeige.
//   * @param connection Datenverteilerverbindung.
//   */
//  public void setAnzeigeObjekt(SystemObject anzeigeObjekt, ClientDavInterface connection)
//  {
//    m_anzeigeObjekt = anzeigeObjekt;
//    m_connection = connection;
//  }

  /**
   * Setzt die Liste der gestrten Inhalte.
   * 
   * @param listeGestoerteInhalte Liste der gestrten Inhalte.
   */
  public void setListeGestoerteInhalte(List<String> listeGestoerteInhalte)
  {
    m_listeGestoerteInhalte = listeGestoerteInhalte;
  }
}
