/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Clickable;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.ImageFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.ScaledGraphics;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;

import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeQuerschnittTyp;

/**
 * Figure, welche einen AQ eines beliebigen Typs reprsentiert.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittLinksFigure extends ImageFigure implements ScalableFigure
{

  String _zeit = null;

  private Label _labelZeit;

  private Clickable _clickable;

  private static Font _fNormal = new Font(null, "Arial", 8, SWT.NORMAL);

  private static Font _fBold = new Font(null, "Arial", 8, SWT.BOLD);

  private AnzeigeQuerschnittLinksFigure _instanz;

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  private Label _labelName;

  private int _x;

  private AnzeigeQuerschnittTyp _type;

  /**
   * Legt fest ob die Border gendert werden darf.
   */
  private boolean _borderChangeable = true;

  /**
   * Sets the zoom level
   * 
   * @param newZoom
   *          The new zoom level
   */
  public void setScale(double newZoom)
  {
    if (scale == newZoom)
      return;
    scale = newZoom;
    // sorgt dafr, dass die Gre der Figur unabhngig von der Zoomstufe
    // gleich bleibt.
    // setSize((int) (DIM.width / scale), (int) (DIM.height / scale));
    // TODO: Die Skalierung des Textes anpassen.
    updateFigure();
    layout();
    repaint();
  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    setBounds(new Rectangle(getLocation(), getSize()));
  }

  @Override
  public double getScale()
  {
    return scale;
  }

  @Override
  protected void paintFigure(final Graphics graphics)
  {
    if (graphics instanceof ScaledGraphics)
    {
      final Dimension size = getSize();

      final Point location = getLocation();
      graphics.translate(location.x + size.width / 2, location.y + size.height / 2);
      graphics.scale(1 / scale);
      graphics.translate(-location.x - size.width / 2, -location.y - size.height / 2);
    }

    super.paintFigure(graphics);
  }

  public void switchLinksRechts(int fr)
  {

    if (fr != 1)
    {
      if (!(_type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ3) || _type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ4)))
      {
        if (!(_type.equals(AnzeigeQuerschnittTyp.WVZ_A14_Typ1) 
            || _type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ1)
            || _type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ2)
            || _type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ3)
            || _type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ4)
            || _type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ5)
            ))
        {
          setConstraint(_labelZeit, new Rectangle(3, 3, -1, -1));
          setConstraint(_labelName, new Rectangle(_x, 3, -1, -1));
        }       
        else
        {
          setConstraint(_labelZeit, new Rectangle(3, 19, -1, -1));
          setConstraint(_labelName, new Rectangle(_x, 3, -1, -1));
        }
      }
      else if (!_type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ4))
      {
        setConstraint(_labelZeit, new Rectangle(45, 3, -1, -1));
        setConstraint(_labelName, new Rectangle(136, 3, -1, -1));
      }
      else
      {
        setConstraint(_labelZeit, new Rectangle(45, 3, -1, -1));
        setConstraint(_labelName, new Rectangle(45, 19, -1, -1));
      }

    }
    else
    {
      if (!(_type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ3) || _type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ4)))
      {
        if (!(_type.equals(AnzeigeQuerschnittTyp.WVZ_A14_Typ1)))
        {
          if (!_type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ7))
          {

            if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ1) || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ2)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ3) || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ4)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ5))
            {
              setConstraint(_labelZeit, new Rectangle(_x - 3, 54, -1, -1));
              setConstraint(_labelName, new Rectangle(3, 54, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ16)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ17)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ17B))
            {
              setConstraint(_labelZeit, new Rectangle(_x - 3, 80, -1, -1));
              setConstraint(_labelName, new Rectangle(3, 80, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ15)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ18))
            {
              setConstraint(_labelZeit, new Rectangle(_x - 3, 70, -1, -1));
              setConstraint(_labelName, new Rectangle(3, 70, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ21)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ6D) || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ7D)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ10D)
                || _type.equals(AnzeigeQuerschnittTyp.WVZ_A8_Typ14D))
            {
              setConstraint(_labelZeit, new Rectangle(_x - 3, 76, -1, -1));
              setConstraint(_labelName, new Rectangle(3, 76, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ1))
            {
              setConstraint(_labelZeit, new Rectangle(3, 54-19, -1, -1));
              setConstraint(_labelName, new Rectangle(_x - 3, 54, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ2))
            {
              setConstraint(_labelZeit, new Rectangle(3, 54-19, -1, -1));
              setConstraint(_labelName, new Rectangle(_x - 3, 54, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ3)  )
            {
              setConstraint(_labelZeit, new Rectangle(3, 91-19, -1, -1));
              setConstraint(_labelName, new Rectangle(_x - 3, 91, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ4))
            {
              setConstraint(_labelZeit, new Rectangle(3, 72-19, -1, -1));
              setConstraint(_labelName, new Rectangle(_x - 3, 72, -1, -1));
            }
            else if (_type.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ5))
            {
              setConstraint(_labelZeit, new Rectangle(3, 81, -1, -1));
              setConstraint(_labelName, new Rectangle(_x - 3, 81 + 19, -1, -1));
            }
            else
            {
              setConstraint(_labelZeit, new Rectangle(_x - 8, 54, -1, -1));
              setConstraint(_labelName, new Rectangle(3, 54, -1, -1));
            }
          }
          else
          {
            setConstraint(_labelZeit, new Rectangle(_x + 30, 54, -1, -1));
            setConstraint(_labelName, new Rectangle(3 + 45, 54, -1, -1));
          }
        }
        else
        {
          setConstraint(_labelZeit, new Rectangle(_x, 39, -1, -1));
          setConstraint(_labelName, new Rectangle(3, 39 + 17, -1, -1));
        }
      }
      else if (!_type.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ4))
      {
        setConstraint(_labelZeit, new Rectangle(45, 85, -1, -1));
        setConstraint(_labelName, new Rectangle(136, 85, -1, -1));
      }
      else
      {
        setConstraint(_labelZeit, new Rectangle(45, 65, -1, -1));
        setConstraint(_labelName, new Rectangle(45, 84, -1, -1));
      }
    }
  }

  public AnzeigeQuerschnittLinksFigure(AnzeigeQuerschnittTyp typ, int x)
  {
    _instanz = this;
    _x = x;
    _type = typ;

    setBorder(new LineBorder(ColorConstants.orange, 2));
    setBackgroundColor(ColorConstants.gray);
    // setToolTip(new Label(typ.getType()));
    setOpaque(true);

    setLayoutManager(new XYLayout());

    _labelZeit = new Label("--:--");
    Font f2 = new Font(null, "Arial", 8, SWT.NORMAL);
    _labelZeit.setFont(f2);
    _labelZeit.setToolTip(new Label("--.--.---- --:--"));
    _labelZeit.setLabelAlignment(GridData.BEGINNING);
    add(_labelZeit);
    if (!(typ.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ3)) || typ.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ4))
    {
      if (!(typ.equals(AnzeigeQuerschnittTyp.WVZ_A14_Typ1) 
          || typ.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ1)
          || typ.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ2)
          || typ.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ3)
          || typ.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ4)
          || typ.equals(AnzeigeQuerschnittTyp.WVZ_A2_Typ5)
          ))
        setConstraint(_labelZeit, new Rectangle(3, 0, -1, -1));
      else
        setConstraint(_labelZeit, new Rectangle(0, 19, -1, -1));
    }
    else
      setConstraint(_labelZeit, new Rectangle(45, 0, -1, -1));

    _labelName = new Label(typ.getType());
    _labelName.setFont(_fBold);
    _labelName.setToolTip(new Label(typ.getType()));
    _labelName.setLabelAlignment(GridData.END);

    add(_labelName);
    if (!typ.equals(AnzeigeQuerschnittTyp.WVZ_B27_Typ4))
      setConstraint(_labelName, new Rectangle(0, 0, -1, -1));
    else
      setConstraint(_labelName, new Rectangle(0, 19, -1, -1));
  }

  public String getZeit()
  {
    return _zeit;
  }

  public void setZeit(String zeit)
  {
    if (_labelZeit != null)
    {
      _labelZeit.setText(zeit);
      updateFigure();
    }
  }

  public void setName(String name)
  {
    if (_labelName != null)
    {
      _labelName.setText(name);
      updateFigure();
    }
  }

  public Label getLabelName()
  {
    return _labelName;
  }

  public Label getLabelZeit()
  {
    return _labelZeit;
  }

  public void setLabelZeit(Label l)
  {
    _labelZeit = l;
  }

  public Clickable getClickable()
  {
    return _clickable;
  }

  public void setClickable(Clickable clickable)
  {
    _clickable = clickable;
  }

  /**
   * Prft ob die Border der Figure gendert werden darf.
   * 
   * @return
   */
  public boolean isBorderChangeable()
  {
    return _borderChangeable;
  }

  /**
   * Legt fest ob die Border der Figure gendert werden darf.
   * 
   * @param borderChangeable
   */
  public void setBorderChangeable(boolean borderChangeable)
  {
    _borderChangeable = borderChangeable;
  }

}
