/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import java.util.Arrays;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert die Datenstruktur der BuVWechseltextFontKombi. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @author Pittner
 * 
 */
public class BuvWechselTextKombi implements Cloneable
{
  private SystemObject _font = null;

  private String[] _textAusrichtungen = null;
  
  private SystemObject[] _zeilenZeichen = null;
  
  private String _beschreibung = null;
  
  public BuvWechselTextKombi(SystemObject font, String beschreibung, String[] textAusrichtungen, SystemObject[] zeilenZeichen)
  {   
    _font = font;
    _beschreibung = beschreibung;
    _textAusrichtungen = textAusrichtungen;
    _zeilenZeichen = zeilenZeichen;
  }

  public SystemObject getFont()
  {
    return _font;
  }

  public void setFont(SystemObject font)
  {
    _font = font;
  }
  
  public String[] getTextAusrichtungen()
  {
    return _textAusrichtungen;
  }

  public void setTextAusrichtungen(String[] textAusrichtungen)
  {
    _textAusrichtungen = textAusrichtungen;
  }

  public SystemObject[] getZeilenZeichen()
  {
    return _zeilenZeichen;
  }

  public void setZeilenZeichen(SystemObject[] zeilenZeichen)
  {
    _zeilenZeichen = zeilenZeichen;
  }
  
  public String getBeschreibung()
  {
    return _beschreibung;
  }

  public void setBeschreibung(String beschreibung)
  {
    _beschreibung = beschreibung;
  }

  @Override
  protected Object clone() throws CloneNotSupportedException
  {
    return new BuvWechselTextKombi(_font, _beschreibung, _textAusrichtungen, _zeilenZeichen);
  }

  @Override
  public int hashCode()
  {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((_font == null) ? 0 : _font.hashCode());
    result = prime * result + Arrays.hashCode(_textAusrichtungen);
    result = prime * result + Arrays.hashCode(_zeilenZeichen);
    return result;
  }

  @Override
  public boolean equals(Object obj)
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    BuvWechselTextKombi other = (BuvWechselTextKombi)obj;
    if (_font == null)
    {
      if (other._font != null)
        return false;
    }
    else if (!_font.equals(other._font))
      return false;
    if (!Arrays.equals(_textAusrichtungen, other._textAusrichtungen))
      return false;
    if (!Arrays.equals(_zeilenZeichen, other._zeilenZeichen))
      return false;
    return true;
  }
  
 

}
