/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;

/**
 * Figure, welche eine Ufd-Messstelle mit OberflaechenZustand-Sensor reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class UmfeldDatenSensorOberflaechenFigure extends Figure
{
  private String _pid;

  private Label _labelImage;

  public UmfeldDatenSensorOberflaechenFigure(String pid, ZusatzElementTyp typ, ZusatzElementTyp inhalt)
  {
    _pid = pid;

    setOpaque(false);
    setLayoutManager(new ToolbarLayout());

    _labelImage = new Label();

    setInhalt(inhalt);

    add(_labelImage);
  }

  public Label getLabelImage()
  {
    return _labelImage;
  }

  public void setLabelImage(Label l)
  {
    _labelImage = l;
  }

  public void setInhalt(ZusatzElementTyp inhalt)
  {
    if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_TROCKEN))
    {
      Image image1 = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image1);
    }
    else if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_NASS_1))
    {
      Image image2 = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image2);
    }
    else if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_NASS_2))
    {
      Image image3 = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image3);
    }
    else if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_NASS_3))
    {
      Image image4 = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image4);
    }
    else if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_SCHNEE))
    {
      Image image5 = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image5);
    }
    else if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_GESTOERT))
    {
      Image image = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image);
    }
    else if (inhalt.equals(ZusatzElementTyp.OBERFLAECHE_UNDEFINIERT))
    {
      Image image = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image);
    }
    else
    {
      Image image6 = ImageFactory.getImage("icons/band3.png");
      _labelImage.setIcon(image6);
    }

  }

  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

}
