/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Model Object of a Fahrstreifen, wird aus Kompatibilittsgrunden bentigt
 * 
 * @deprecated  
 * @author Pittner
 * 
 */
public class Fahrstreifen extends PropertyAwareObject
{

  private IFigure _figure;

  private String _pid;

  private String _name;

  private List _mqElemente = new ArrayList();

  private Rectangle _bounds;

  private List _sourceConnections = new ArrayList();

  private List _targetConnections = new ArrayList();

  private FahrstreifenTyp _typ;

  private SituationTyp _situation;

  private int _x = 0;

  private int _y = 0;

  private int[] _pts = null;

  private int _width = 0;

  private int _height = 0;

  private int _width2 = 0;

  private int _height2 = 0;

  private int _abzweig = 0;

  private String _viewId;

  public Fahrstreifen()
  {
    super();
  }

  public Fahrstreifen(FahrstreifenTyp typ, String pid, String name, int x, int y, int width, int height, String viewId)
  {

    this._typ = typ;
    this._pid = pid;
    this._name = name;
    this._x = x;
    this._y = y;
    this._width = width;
    this._height = height;
    this._viewId = viewId;
    this._viewId = viewId;

    _situation = SituationTyp.KEINE_AUSSAGE;

  }

  public Fahrstreifen(FahrstreifenTyp typ, String pid, String name, int x, int y, int width, int height, int abzweig,
      String viewId)
  {

    this._typ = typ;
    this._pid = pid;
    this._name = name;
    this._x = x;
    this._y = y;
    this._width = width;
    this._height = height;
    this._abzweig = abzweig;
    this._viewId = viewId;

    _situation = SituationTyp.KEINE_AUSSAGE;

  }

  public Fahrstreifen(FahrstreifenTyp typ, String pid, String name, List mqs, int x, int y, int width, int height,
      String viewId)
  {

    this._typ = typ;
    this._pid = pid;
    this._name = name;
    this._mqElemente = mqs;
    this._x = x;
    this._y = y;
    this._width = width;
    this._height = height;
    this._viewId = viewId;

    _situation = SituationTyp.KEINE_AUSSAGE;

  }

  public Fahrstreifen(FahrstreifenTyp typ, String pid, String name, List mqs, int x, int y, int width, int height,
      int abzweig, String viewId)
  {

    this._typ = typ;
    this._pid = pid;
    this._name = name;
    this._mqElemente = mqs;
    this._x = x;
    this._y = y;
    this._width = width;
    this._height = height;
    this._width2 = width;
    this._height2 = height;
    this._abzweig = abzweig;
    this._viewId = viewId;

    _situation = SituationTyp.KEINE_AUSSAGE;

  }

  public Fahrstreifen(FahrstreifenTyp typ, String pid, String name, List mqs, int x, int y, int[] pts, int abzweig,
      String viewId)
  {

    this._typ = typ;
    this._pid = pid;
    this._name = name;
    this._mqElemente = mqs;
    this._x = x;
    this._y = y;
    this._pts = pts;
    this._abzweig = abzweig;
    this._viewId = viewId;

    _situation = SituationTyp.KEINE_AUSSAGE;

  }

  /**
   * Returns the persistent position and size of the node. The edit part handling this node has the task to translate it
   * into a constraint that enables the draw2D system to place the figure of the node. Therefore, the coordinate system
   * of the rectangle is known only to the edit part (but in fact, in this example it is always the local coordinate
   * system of the figure of the node).
   */
  public Rectangle getConstraint()
  {
    return (Rectangle)getProperty("constraint");
  }

  /**
   * Sets the persistent position and size of the node.
   * 
   * @param constraint
   *          The data that enables the edit part handling this node to create a constraint that enables the draw2D
   *          system to place the figure of the node.
   */
  public void setConstraint(Rectangle constraint)
  {
    setProperty("constraint", constraint);
  }

  /**
   * Sets name without firing off any event notifications
   * 
   * @param name
   *          The name to set.
   */
  public void setPid(String pid)
  {
    this._pid = pid;
  }

  /**
   * Sets bounds without firing off any event notifications
   * 
   * @param bounds
   *          The bounds to set.
   */
  public void setBounds(Rectangle bounds)
  {
    this._bounds = bounds;
  }

  /**
   * Adds relationship where the current object is the foreign key table in a relationship
   * 
   * @param table
   *          the primary key relationship
   */
  public void addTargetConnections(LinkElement table)
  {
    _targetConnections.add(table);
    // firePropertyChange(OUTPUT, null, table);
  }

  /**
   * Adds relationship where the current object is the primary key table in a relationship
   * 
   * @param table
   *          the foreign key relationship
   */
  public void addSourceConnections(LinkElement table)
  {
    _sourceConnections.add(table);
    // firePropertyChange(INPUT, null, table);
  }

  /**
   * Removes relationship where the current object is the foreign key table in a relationship
   * 
   * @param table
   *          the primary key relationship
   */
  public void removeSourceConnections(LinkElement table)
  {
    _sourceConnections.remove(table);
    // firePropertyChange(OUTPUT, table, null);
  }

  /**
   * Removes relationship where the current object is the primary key table in a relationship
   * 
   * @param table
   *          the foreign key relationship
   */
  public void removeTargetConnections(LinkElement table)
  {
    _targetConnections.remove(table);
    // firePropertyChange(INPUT, table, null);
  }

  public void addMessQuerschnitt(AnzeigeElement ae)
  {
    if (_mqElemente.contains(ae))
    {
      throw new IllegalArgumentException("AnzeigeElement already present");
    }
    _mqElemente.add(ae);
  }

  public void addMessQuerschnitt(AnzeigeElement ae, int index)
  {
    if (_mqElemente.contains(ae))
    {
      throw new IllegalArgumentException("AnzeigeElement already present");
    }
    _mqElemente.add(index, ae);
  }

  public void removeMessQuerschnitt(AnzeigeElement ae)
  {
    _mqElemente.remove(ae);
    // firePropertyChange(CHILD, column, null);
  }

  public void switchMessQuerschnitt(AnzeigeElement ae, int index)
  {
    _mqElemente.remove(ae);
    _mqElemente.add(index, ae);
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param _pid
   *          The name to set.
   */
  public void situationChange(String status)
  {

    Fahrstreifen old = this;

    SituationTyp neu = _situation.setTyp(status);

    if (neu != null)
    {
      firePropertyChange(SITUATION, old, neu);
      // _status = neu;
    }
    else
      System.err.println(_pid + " : Situation konnte nicht gesetzt werden");
  }

  /**
   * If modified, sets bounds and fires off event notification
   * 
   * @param bounds
   *          The bounds to set.
   */
  public void modifyBounds(Rectangle bounds)
  {
    Rectangle oldBounds = this._bounds;
    if (!bounds.equals(oldBounds))
    {
      this._bounds = bounds;
      // firePropertyChange(BOUNDS, null, bounds);
    }
  }

  public String getPid()
  {
    return _pid;
  }

  public List getMessQuerschnittElemente()
  {
    return _mqElemente;
  }

  /**
   * @return Returns the foreignKeyRelationships.
   */
  public List getTargetConnections()
  {
    return _targetConnections;
  }

  /**
   * @return Returns the primaryKeyRelationships.
   */
  public List getSourceConnections()
  {
    return _sourceConnections;
  }

  /**
   * @return Returns the bounds.
   */
  public Rectangle getBounds()
  {
    return _bounds;
  }

  public String toString()
  {
    return _pid;
  }

  public SituationTyp getSituation()
  {
    return _situation;
  }

  public void setStatus(SituationTyp situation)
  {
    _situation = situation;
  }

  public int getX()
  {
    return _x;
  }

  public int getY()
  {
    return _y;
  }

  public String getName()
  {
    return _name;
  }

  public void setName(String name)
  {
    _name = name;
  }

  public int getWidth()
  {
    return _width;
  }

  public void setWidth(int width)
  {
    _width = width;
  }

  public int getHeight()
  {
    return _height;
  }

  public void setHeight(int height)
  {
    _height = height;
  }

  public int getAbzweig()
  {
    return _abzweig;
  }

  public void setAbzweig(int abzweig)
  {
    _abzweig = abzweig;
  }

  public int getWidth2()
  {
    return _width2;
  }

  public void setWidth2(int width2)
  {
    _width2 = width2;
  }

  public int getHeight2()
  {
    return _height2;
  }

  public void setHeight2(int height)
  {
    _height2 = height;
  }

  public int[] getPts()
  {
    return _pts;
  }

  public void setPts(int[] pts)
  {
    _pts = pts;
  }

  public FahrstreifenTyp getTyp()
  {
    return _typ;
  }

  public void setTyp(FahrstreifenTyp typ)
  {
    _typ = typ;
  }

  public String getViewId()
  {
    return _viewId;
  }

}
