/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.util.Iterator;
import java.util.NoSuchElementException;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class SonderProgrammTypDataHelper {
	
	/**
	 * Prft 2 Datenstze rekursiv auf Gleichheit.<br>
	 * Eingebettete Attributlisten mit dem Namen "Urlasser" werden bersprungen.
	 * 
	 * @param data1 1. zu prfendender Datensatz
	 * @param data2 2. zu prfendender Datensatz
	 * @return <code>true</code>, wenn die bergebenen Datenstze gleich sind,
	 * <code>false</code> sonst. 
	 */
	public static boolean isEqual(Data data1, Data data2) {
		
		if (data1 == null || data2 == null) {
			return data1 == data2;
		}
		
		if (data1.isPlain()) {
			if (data1.getAttributeType() instanceof ReferenceAttributeType) {
				if (data2.getAttributeType() instanceof ReferenceAttributeType) {
					SystemObject wert1 = data1.asReferenceValue().getSystemObject();
					SystemObject wert2 = data2.asReferenceValue().getSystemObject();
					if (wert1 == wert2) {
						return true;
					}
					else {
						return false;
					}
				}
				else {
					return false;
				}
			}
			else {
				String wert1 = data1.asTextValue().getText();
				String wert2 = data2.asTextValue().getText();
				if (wert1.equals(wert2)) {
					return true;
				}
				else {
					return false;
				}
			}
		}
		else if (data1.isArray()) {
			if (data2.isArray()) {
				Array array1 = data1.asArray();
				Array array2 = data2.asArray();
				int length = array1.getLength();
                if (length == array2.getLength()) {
                	for (int i = 0; i < length; i += 1) {
                		Data item1 = array1.getItem(i);
                		Data item2 = array2.getItem(i);
                		
        				if (!isEqual(item1, item2)) {
        					return false;
        				}
                	}
                	return true;
                }
                else {
                	return false;
                }
			}
			else {
				return false;
			}
		}
		else { // Attributliste
			Iterator iter = data1.iterator();
			while (iter.hasNext()) {
				Data item1 = (Data)iter.next();
				Data item2 = null;
				try {
					item2 = data2.getItem(item1.getName());	
				}
				catch (NoSuchElementException e)
				{
					return false;
				}
				
				// Attributliste 'Urlasser' wird ignoriert
				if (item1.isList() && item1.getName().equals("Urlasser") &&
					item2.isList() && item2.getName().equals("Urlasser")) {
					continue;
				}
				if (!isEqual(item1, item2)) {
					return false;
				}
			}
			return true;
		}
	}
}
