/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;

import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;

/**
 * Darstellung des Fensters zur Parameternderung des Schaltbildes
 * 
 * @author Pittner
 */
public class CompositeSperrungBereiche extends Composite
{
  /**
   * Der Sonderprogramm-Datensatz
   */
  private SonderProgramm _sonderProgrammDatensatz = null;

  /**
   * Composite mit den Reitern fr die Stufen
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
  private ArrayList<String> _list;

  /**
   * Composites mit den Darstellungen
   */
  private CompositeSperrungDarstellung[] _compositeParameterDarstellung;

  /**
   * Anzahl der zu versorgenden Fahrstreifen
   */
  private int _anzahlStufen;

  /**
   * Groups
   */
  private Composite _groupStufen;

  /**
   * Anzahl der Stufen
   */
  private int _indexPrg;

  private String _secId;

  /**
   * Auswahl der Programme
   */
  private Combo _combo;

  /**
   * Konstruktor der Klasse
   * 
   * @param parent
   *          Aufrufendes Composite
   * @param style
   *          Stil
   * @param resultData
   *          Ergebnis Stufen
   */
  public CompositeSperrungBereiche(Composite parent, int style, SonderProgramm sonderProgrammDatensatz, int index,
      String secId)
  {
    super(parent, style);
    this._sonderProgrammDatensatz = sonderProgrammDatensatz;
    this._indexPrg = index;
    this._secId = secId;
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = false;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.CENTER;

    GridLayout gl = new GridLayout(1, false);
    gl.marginWidth = 0;
    gl.marginLeft = 0;
    gl.marginRight = 0;

    setLayout(gl);
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.CENTER;

    _list = new ArrayList<String>();

    createGroupStufen();
    createCompositeTab();

    int anzahl = bestimmeAnzahlStufen();

    if (anzahl > 0)
      erzeugeBenoetigteReiter(anzahl);
    else
    {
      erzeugeBenoetigteReiter(1);
      CTabItem item = _tabFolder.getItems()[0];
      Composite cSel = (Composite)item.getControl();
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
        comp.setVisible(false);
      }
    }

    updateData();
  }

  /**
   * Methode bestimmt die Anzahl der Stufen aus dem Ergebnis des DaV
   * 
   * @return Anzahl Stufen
   */
  public int bestimmeAnzahlStufen()
  {
    int length = _tabFolder.getItems().length;
    if (length == 1)
    {
      CTabItem item = _tabFolder.getItem(0);
      if (item != null && !item.isDisposed())
      {
        Composite cSel = (Composite)item.getControl();
        if (((Composite)cSel).getChildren().length > 0)
        {
          CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
          if (!comp.isVisible())
            return 0;
        }
      }
    }

    return length;
  }

  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData()
  {
    // if (_sonderProgrammDatensatz != null)
    {
      updataDataAlgorithmus();
    }
  }

  /**
   * Methode legt die bentigte Anzahl der Reiter an
   * 
   * @param anzahl
   *          Anzahl bentigter Reiter (= Anzahl Stufen)
   */
  public void erzeugeBenoetigteReiter(int anzahl)
  {

    if (anzahl == _anzahlStufen)
      return;

    /*
     * Beim ersten Anlegen der Tabelle wird Reiter 0 markiert, bei nderung der Anzahl der Reiter ber den Spinner wird
     * der letzten Reiter markiert.
     */
    int indexAnwahl = 0;
    if (_compositeParameterDarstellung != null)
      indexAnwahl = anzahl - 1;

    /*
     * Namen der Reiter anpassen
     */
    _list.clear();
    for (int i = 0; i < anzahl; ++i)
      _list.add("Bereich " + (i + 1));

    /*
     * Bisherige Composite temp. speichern
     */
    Vector<CompositeSperrungDarstellung> tmp = new Vector<CompositeSperrungDarstellung>();
    if (_compositeParameterDarstellung != null)
      for (CompositeSperrungDarstellung cpa : _compositeParameterDarstellung)
        tmp.add(cpa);

    /*
     * Nicht bentigte Reiter entfernen
     */
    CTabItem[] items = _tabFolder.getItems();
    for (int i = items.length - 1; i >= anzahl; i--)
    {
      items[i].dispose();
    }

    /*
     * Feld mit den Composites erweitern bzw. reduzieren
     */
    _compositeParameterDarstellung = new CompositeSperrungDarstellung[_list.size()];
    for (int i = 0; i < tmp.size() && i < anzahl; ++i)
      _compositeParameterDarstellung[i] = tmp.get(i);

    /*
     * Reiter hinzufgen
     */
    if (anzahl > _anzahlStufen)
    {
      for (int i = _anzahlStufen; i < anzahl; ++i)
      {
        erzeugeTabItem(_list.get(i));
        updataDataAlgorithmus(i);
      }
    }

    _anzahlStufen = anzahl;

    if (indexAnwahl >= 0)
      _tabFolder.setSelection(indexAnwahl);

    _tabFolder.redraw();
  }

  /**
   * Updatefunktion fr die Stufen
   */
  private void updataDataAlgorithmus()
  {
    int indexStufe = _tabFolder.getSelectionIndex();

    for (int index = 0; index < _tabFolder.getItems().length; ++index)
    {
      if (indexStufe == index)
        updataDataAlgorithmus(index);
    }
  }

  /**
   * Updatefunktion fr die Stufen
   */
  private void updataDataAlgorithmus(int indexStufe)
  {
    // if (_sonderProgrammDatensatz != null)
    {

      CTabItem[] items = _tabFolder.getItems();
      CTabItem item = items[indexStufe];

      if (item != null && !item.isDisposed())
      {
        Composite c = (Composite)item.getControl();
        Control[] children = c.getChildren();
        CompositeSperrungDarstellung compPar = null;

        if (children.length == 0)
        {
          compPar = new CompositeSperrungDarstellung(c, SWT.NONE, indexStufe, _sonderProgrammDatensatz, _secId);
        }
        else
        {
          compPar = (CompositeSperrungDarstellung)children[0];
          compPar.setSonderProgrammDatensatz(_sonderProgrammDatensatz);
        }
        _compositeParameterDarstellung[indexStufe] = compPar;
        compPar.layout(true);
        c.layout(true);
      }
    }
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Messquerschnitte
   * 
   */
  private void createGroupStufen()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;
    gridData2.verticalAlignment = GridData.CENTER;
    gridData2.horizontalIndent = 0;

    GridLayout gl = new GridLayout(1, false);
    gl.marginLeft = 0;
    gl.marginRight = 0;

    _groupStufen = new Composite(this, SWT.NONE);
    _groupStufen.setLayoutData(gridData2);
    _groupStufen.setLayout(gl);
  }

  public Combo getCombo()
  {
    return _combo;
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = false;
    gridDataGroup.verticalAlignment = GridData.CENTER;

    GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = false;
    gridDataTabFolder.verticalAlignment = GridData.CENTER;

    _tabFolder = new CTabFolder(_groupStufen, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);

    erzeugeKontextMenue(_tabFolder);

  }

  private MenuManager _kontextMenuMgr;

  private ActionRegistry _actionRegistry;

  /**
   * Liefert das KontextMenue zum Baum
   */
  private void erzeugeKontextMenue(CTabFolder folder)
  {
    _kontextMenuMgr = new MenuManager("#SperrungBereichMenu");
    erzeugeAktionen();

    addtoKontextMenue((Action)_actionRegistry.getAction("Hinzufuegen"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Loeschen"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Kopieren"));

    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr.createContextMenu(folder);
    // Das Men dem Baum bekannt machen
    folder.setMenu(menu);
  }

  /**
   * Aktion im Kontextmen setzen
   * 
   * @param action
   *          Aktion die im Kontextmen erscheinen soll
   */
  public void addtoKontextMenue(Action action)
  {

    if (action != null)
      _kontextMenuMgr.add(action);
  }

  /**
   * Erzeugt die Aktionen zum Verwalten der Hierarchieobjekte
   */
  private void erzeugeAktionen()
  {
    _actionRegistry = new ActionRegistry();

    Action action1 = new Action() {
      public void run()
      {
        aktionHinzufuegen();
        aktualisiereKontextMenue();
      }
    };
    action1.setId("Hinzufuegen");
    action1.setText("Hinzufgen");
    action1.setToolTipText("Erstellt einen neuen Bereich");
    action1.setEnabled(false);
    _actionRegistry.registerAction(action1);

    Action actionLoeschen = new Action() {
      public void run()
      {
        aktionLoeschen();
        aktualisiereKontextMenue();
      }
    };
    actionLoeschen.setId("Loeschen");
    actionLoeschen.setText("Lschen");
    actionLoeschen.setToolTipText("Lscht eine bestehenden Bereich");
    actionLoeschen.setEnabled(false);
    _actionRegistry.registerAction(actionLoeschen);

    Action action5 = new Action() {
      public void run()
      {
        aktionKopieren();
        aktualisiereKontextMenue();
      }
    };
    action5.setId("Kopieren");
    action5.setText("Kopieren");
    action5.setToolTipText("Kopiert den Bereich und fgt diese am Ende der Bereiche ein");
    action5.setEnabled(false);
    _actionRegistry.registerAction(action5);
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems(List<String> list)
  {
    for (int i = 0; i < list.size(); ++i)
      erzeugeTabItem(list.get(i));

    _tabFolder.setSelection(0);
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  private void erzeugeTabItem(String name)
  {
    String so = name;

    CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);

    String tabName = so;
    tabItem.setText(tabName);
    tabItem.setData(tabName, so);

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite(_tabFolder, SWT.NONE);
    c1.setLayout(new GridLayout());
    c1.setLayoutData(gridData2);

    tabItem.setControl(c1);
  }

  /**
   * Methode liefert die einzelnen Composite fr die Stufen zurck
   * 
   * @return Composites
   */
  public CompositeSperrungDarstellung[] getCompositeParameterStauBeleg()
  {
    return _compositeParameterDarstellung;
  }

  /**
   * @return die aktuelle Stufe
   */
  public int getIndexStufe()
  {
    return _indexPrg;
  }

  /**
   * @return
   */
  public CompositeSperrungDarstellung[] getCompositeParameterDarstellung()
  {
    return _compositeParameterDarstellung;
  }

  public SonderProgramm getSonderProgrammDatensatz()
  {
    return _sonderProgrammDatensatz;
  }

  public void setSonderProgrammDatensatz(SonderProgramm sonderProgrammDatensatz)
  {
    _sonderProgrammDatensatz = sonderProgrammDatensatz;
    updateData();
    aktualisiereKontextMenue();
  }

  private void aktualisiereKontextMenue()
  {
    if (_sonderProgrammDatensatz != null && _sonderProgrammDatensatz.getTypBezeichnung() != null)
    {
      String typ = _sonderProgrammDatensatz.getTypBezeichnung();
      boolean b = DaVHelper.isProgrammTypSpurSperrung(typ);
      if (b)
      {
        _actionRegistry.getAction("Hinzufuegen").setEnabled(true);
        _actionRegistry.getAction("Loeschen").setEnabled(true);
        if (bestimmeAnzahlStufen() > 0)
          _actionRegistry.getAction("Kopieren").setEnabled(true);
        else
          _actionRegistry.getAction("Kopieren").setEnabled(false);
      }
      else
      {
        _actionRegistry.getAction("Hinzufuegen").setEnabled(false);
        _actionRegistry.getAction("Loeschen").setEnabled(false);
        _actionRegistry.getAction("Kopieren").setEnabled(false);
      }
    }
  }

  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  private void aktionLoeschen()
  {
    int anzahl = bestimmeAnzahlStufen();
    if (anzahl > 0)
      anzahl--;
    if (anzahl == 0)
    {
      CTabItem item = _tabFolder.getItems()[0];
      Composite cSel = (Composite)item.getControl();
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
        if (comp.isVisible())
        {
          comp.initialisiereControls();
          comp.setVisible(false);
        }
      }

    }
    else
    {
      int index = _tabFolder.getSelectionIndex();
      loescheReiter(index);
    }
  }
  
  private void aktionAlleLoeschen()
  {
    
  }
  
  

  private void loescheReiter(int index)
  {
    /*
     * Bisherige Composite temp. speichern
     */
    Vector<CompositeSperrungDarstellung> tmp = new Vector<CompositeSperrungDarstellung>();
    if (_compositeParameterDarstellung != null)
      for (CompositeSperrungDarstellung cpa : _compositeParameterDarstellung)
        tmp.add(cpa);

    if (index < tmp.size() - 1)
    {
      tmp.remove(index);
    }

    _list.clear();
    for (int i = 0; i < tmp.size(); i++)
    {
      _list.add("Bereich " + (i + 1));
    }

    /*
     * Nicht bentigte Reiter entfernen
     */
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < _compositeParameterDarstellung.length; i++)
    {
      if (i == index)
      {
        items[i].dispose();
      }

    }

    items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      if (items[i] != null && !items[i].isDisposed())
      {
        items[i].setText(_list.get(i));
      }
    }

    /*
     * Feld mit den Composites erweitern bzw. reduzieren
     */
    _compositeParameterDarstellung = new CompositeSperrungDarstellung[tmp.size()];
    for (int i = 0; i < tmp.size() && i < _compositeParameterDarstellung.length; i++)
      _compositeParameterDarstellung[i] = tmp.get(i);

    _anzahlStufen--;

    if (index >= 0)
      _tabFolder.setSelection(index);

    _tabFolder.redraw();

  }

  private void aktionKopieren()
  {
    String text2 = null;
    String text3 = null;
    Boolean[] bb = new Boolean[8];

    CTabItem item = _tabFolder.getSelection();
    Composite cSel = (Composite)item.getControl();
    if (((Composite)cSel).getChildren().length > 0)
    {
      CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
      text2 = comp.getTextStartKmSperrung().getText();
      text3 = comp.getTextStopKmSperrung().getText();
      Control[] children = comp.getGroupSperrung().getChildren();
      if (children != null)
      {
        for (int i = 0; i < children.length; i++)
        {
          Control control = children[i];
          if (control instanceof Button)
          {
            Button btn = (Button)control;
            bb[i] = btn.getSelection();
          }
        }
      }
    }

    aktionHinzufuegen();

    item = _tabFolder.getSelection();
    cSel = (Composite)item.getControl();
    if (((Composite)cSel).getChildren().length > 0)
    {
      CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
      comp.getTextStartKmSperrung().setText(text2);
      comp.getTextStopKmSperrung().setText(text3);
      Control[] children = comp.getGroupSperrung().getChildren();
      if (children != null)
      {
        for (int i = 0; i < children.length; i++)
        {
          Control control = children[i];
          if (control instanceof Button)
          {
            Button btn = (Button)control;
            btn.setSelection(bb[i]);
          }
        }
      }
    }

  }

  private void aktionHinzufuegen()
  {
    int anzahl = bestimmeAnzahlStufen();

    if (anzahl == 0)
    {
      CTabItem item = _tabFolder.getItems()[0];
      Composite cSel = (Composite)item.getControl();
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
        if (!comp.isVisible())
          comp.setVisible(true);
      }

    }
    else
    {
      anzahl++;
      // int index = _tabFolder.getSelectionIndex();
      // erzeugeReiter(index+1);
      erzeugeBenoetigteReiter(anzahl);
    }
  }

  /**
   * @param index
   */
  private void erzeugeReiter(int index)
  {
    /*
     * Bisherige Composite temp. speichern
     */
    Vector<CompositeSperrungDarstellung> tmp = new Vector<CompositeSperrungDarstellung>();
    if (_compositeParameterDarstellung != null)
      for (CompositeSperrungDarstellung cpa : _compositeParameterDarstellung)
        tmp.add(cpa);

    erzeugeTabItem("Bereich " + tmp.size() + 1);

    int position = 0;

    for (int i = 0; i < _tabFolder.getItemCount(); i++)
    {
      CTabItem item = _tabFolder.getItem(i);
      Composite c = (Composite)item.getControl();
      CompositeSperrungDarstellung comp = null;

      if (c.getChildren() != null && c.getChildren().length == 0)
      {

        System.out.println(i + " :::: " + index);

        comp = new CompositeSperrungDarstellung(c, SWT.NONE, i, _sonderProgrammDatensatz, _secId);
        position = i;
        tmp.add(comp);
      }
      else
      {
        comp = (CompositeSperrungDarstellung)c.getChildren()[0];
        comp.setIndexPrg(i);
      }

    }

    Collections.swap(Arrays.asList(_tabFolder.getItems()), index, position);

    _list.clear();
    for (int i = 0; i < tmp.size(); i++)
    {
      _list.add("Bereich " + (i + 1));
    }

    /*
     * Nicht bentigte Reiter entfernen
     */
    CTabItem[] items = _tabFolder.getItems();
    for (int i = position; i >= index; i--)
    {
      items[i].dispose();
    }

    /*
     * Feld mit den Composites erweitern bzw. reduzieren
     */
    _compositeParameterDarstellung = new CompositeSperrungDarstellung[tmp.size()];
    for (int i = 0; i < tmp.size() && i < _compositeParameterDarstellung.length; i++)
      _compositeParameterDarstellung[i] = tmp.get(i);

    /*
     * Reiter hinzufgen
     */
    // if (index > _anzahlStufen)
    {
      for (int i = index; i < position + 1; i++)
      {
        erzeugeTabItem(_list.get(i));
        Composite c = (Composite)_tabFolder.getItem(i).getControl();

        CompositeSperrungDarstellung comp = _compositeParameterDarstellung[i];

        CompositeSperrungDarstellung compPar = new CompositeSperrungDarstellung(c, SWT.NONE, comp.getIndexPrg(),
            _sonderProgrammDatensatz, _secId);
        compPar.getTextStartKmSperrung().setText(comp.getTextStartKmSperrung().getText());
        compPar.getTextStopKmSperrung().setText(comp.getTextStopKmSperrung().getText());
        Control[] children = comp.getGroupSperrung().getChildren();
        for (int j = 0; j < children.length; j++)
        {
          Control control = children[j];
          if (control instanceof Button)
          {
            Button btn = (Button)control;
            ((Button)compPar.getGroupSperrung().getChildren()[i]).setSelection(btn.getSelection());

          }
        }

        compPar.layout(true);
        c.layout(true);
      }
    }

    _anzahlStufen++;

    if (index >= 0)
      _tabFolder.setSelection(index);

    _tabFolder.redraw();

  }
}
