/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm.SswSbaProgrammTyp;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Dialog zum Kopieren von Sonderprogrammen.
 * Zustzlich kann auch noch der Programmtyp veraendert werden.
 * Der Dialog wird auch nur zum Aendern der Sonderprogramm-Typen verwendet, 
 * dann ohne die Eingabemglichkeit einer Bezeichnung fr das Sonderprogramm. 
 * 
 * @author Pittner
 * 
 */
public class SonderProgrammKopierenDialog extends Dialog
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.sonderProgrammKopieren";

  private Shell shell;

  private Composite _parent;

  private Composite _compositeButton;

  private Button _buttonSchalten;

  private Button _buttonAbbrechen;

  private ComboViewer _comboViewerProgrammTyp;

  private List<SswSbaProgrammTyp> _listProgrammTypen;
  
  private ClientDavInterface _connection;

  private Text _textName;

  private SonderProgramm _sonderProgramm;

  private boolean _nurTyp;

  private ControlDecoration _decorationTextName;
  
  /**
   * Vorsatz zum Bilden der Pid fr Ursacheneinheiten zum Schalten von Sonderprogramme.
   */
  private final String _praefixPid = "sba.sp";
  /**
   * Anhang zum Bilden der Pid fr Ursacheneinheiten zum Schalten von Sonderprogramme.
   */
  private String _postfixPid;

  private String _secondId;
  
  /**
   * Konstruktor, erzeugt die {@link Shell} sowie Verbindung zum Datenverteiler {@link ClientDavInterface}
   * @param parent
   */
  public SonderProgrammKopierenDialog(Shell parent, String secondId)
  {
    super(parent);
    _parent = parent;     
    _secondId = secondId;     
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    shell.setText("Sonderprogramm Kopieren");
    shell.setLayout(new GridLayout());
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    _postfixPid = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
  }

  /**
   * Oeffnet den Dialog. 
   * 
   * @param sonderProgramm
   *                der Datensatz
   * @return der geaenderte Datensatz, null bei Abbruch
   */
  public SonderProgramm oeffne(SonderProgramm sonderProgramm, boolean nurTyp)
  {
    _sonderProgramm = sonderProgramm;
    _nurTyp = nurTyp;
    
    createCompositeProgrammTypName();    
    createCompositeButton();
    
    uebernehmeDaten();

    shell.setSize(new Point(400, 200));

    shell.open();
    Display display = _parent.getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _sonderProgramm;
  }

  /**
   * bernimmt die Daten aus der bergebenen Datensruktur vom Typ {@link SonderProgramm}
   */
  private void uebernehmeDaten()
  {
    // Name
    if (!_nurTyp){
      if (!_textName.isDisposed())
      {
        String s = "Kopie von ";
        _textName.setText(s + _sonderProgramm.getBezeichnung());
        _textName.selectAll();
      }      
    }
    
    /*
     * Umsetzung des Sonderprogramm-Strings nach der passenden SswSbaProgrammTyp-Instanz.
     */
    String s2 = _sonderProgramm.getTypBezeichnung();
    if ((s2 != null) && (_listProgrammTypen != null))
    {
      Iterator<SswSbaProgrammTyp> it = _listProgrammTypen.iterator();
      while (it.hasNext())
      {
        SswSbaProgrammTyp sswSbaProgrammTyp = it.next();
        if ((sswSbaProgrammTyp != null) && (sswSbaProgrammTyp.getBezeichnung().equals(s2)))
        {
          _comboViewerProgrammTyp.setSelection(new StructuredSelection(sswSbaProgrammTyp));
          break;
        }
      }
    }
  }

  /**
   * Erzeugt die GUI-Komponenten
   */
  private void createCompositeProgrammTypName()
  {
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = false;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalSpan = 3;
    gridData11.verticalAlignment = GridData.FILL;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    Composite compPrgName = new Composite(shell, SWT.NONE);
    compPrgName.setLayoutData(gridData);
    compPrgName.setLayout(gridLayout);

    Label labelTyp = new Label(compPrgName, SWT.NONE);
    labelTyp.setText("Programm-Typ");
    Combo combo = new Combo(compPrgName, SWT.BORDER | SWT.READ_ONLY);
    combo.setLayoutData(gridData21);
    combo.pack();

    _comboViewerProgrammTyp = new ComboViewer(combo);
    _comboViewerProgrammTyp.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection ss = (IStructuredSelection)event.getSelection();
        if (ss != null && !ss.isEmpty())
        {
          Object fe = ss.getFirstElement();
          if (fe != null)
          {
            if (fe instanceof SswSbaProgrammTyp)
            {              
              SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)fe;
              _sonderProgramm.setTypBezeichnung(programmTyp.getBezeichnung());
            }
          }
        }
      }
    });
    _comboViewerProgrammTyp.setLabelProvider(new ILabelProvider() {
      @Override
      public Image getImage(Object element)
      {
        return null;
      }

      @Override
      public String getText(Object element)
      {
        String returnString = "";

        if (element instanceof String)
        {
          returnString = (String)element;
        }
        else if (element instanceof SswSbaProgrammTyp)
        {
          SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)element;
          if (programmTyp.getBezeichnung() != null)
          {
            returnString = programmTyp.getBezeichnung();
          }
        }

        return returnString;

      }

      @Override
      public void addListener(ILabelProviderListener listener)
      {
      }

      @Override
      public void dispose()
      {
      }

      @Override
      public boolean isLabelProperty(Object element, String property)
      {
        return false;
      }

      @Override
      public void removeListener(ILabelProviderListener listener)
      {
      }

    });
    _comboViewerProgrammTyp.setContentProvider(new ArrayContentProvider());
   
    _listProgrammTypen = ZustandsDarstellungSonderProgramm.bestimmeProgrammTypen(_connection);
    // Combo fuellen
    _comboViewerProgrammTyp.setInput(_listProgrammTypen);

    if (!_nurTyp)
    {
      Label labelName = new Label(compPrgName, SWT.NONE);
      labelName.setText("Name");
      _textName = new Text(compPrgName, SWT.BORDER);
      _textName.setLayoutData(gridData1);     
      _decorationTextName = new ControlDecoration(_textName, SWT.TOP | SWT.LEFT);
      
      Image image = FieldDecorationRegistry.getDefault().getFieldDecoration(FieldDecorationRegistry.DEC_ERROR).getImage();
      _decorationTextName.setImage(image);
      _decorationTextName.setShowOnlyOnFocus(true);
      _decorationTextName.hide();
      
      _textName.addModifyListener(new ModifyListener() {
        
        @Override
        public void modifyText(ModifyEvent e)
        {          
          if (!configHasPidUe(_textName.getText())){
            _decorationTextName.hide();
            _sonderProgramm.setBezeichnung(_textName.getText());
            _buttonSchalten.setEnabled(true);
          }else{
            //String pidUe = ZustandsDarstellungSonderProgramm.erzeugePidUrsachenEinheitSonderProgramm(_textName.getText(), _praefixPid, _postfixPid, _secondId);
            _decorationTextName.setDescriptionText("Es ist bereits ein Sonderprogramm mit dem Namen vorhanden!");
            _decorationTextName.show();
            _buttonSchalten.setEnabled(false);
          }
        }
      });
      _textName.setFocus();
    }
  }
  
  
  /**
   * Prft ob es schon ein {@link SystemObject} mit der Bezeichnung gibt.
   * Die Bezeichnung wird in eine Pid umgesetzt.
   * 
   * @param bezeichnung
   * @return true, wenn es schon ein Object gibt
   */
  private boolean configHasPidUe(String bezeichnung){
    
    String pidUe = ZustandsDarstellungSonderProgramm.erzeugePidUrsachenEinheitSonderProgramm(_textName.getText(), _praefixPid, _postfixPid, _secondId);    
    SystemObject object = null;
    if (pidUe != null && pidUe.length() > 0){
      object = _connection.getDataModel().getObject(pidUe);      
      if (object != null && object.isValid()){        
        return true;        
      }
    }
    
    return false;
  }
  

  /**
   * Erzeugt die Buttons
   */
  private void createCompositeButton()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton = new Composite(shell, SWT.NONE);
    _compositeButton.setLayoutData(gridData);
    _compositeButton.setLayout(rowLayout);

    _buttonSchalten = new Button(_compositeButton, SWT.NONE);
    _buttonSchalten.setText(" bernehmen ");
    _buttonSchalten.setEnabled(_nurTyp);
    
    _buttonSchalten.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        dispose();
      }
    });

    _buttonAbbrechen = new Button(_compositeButton, SWT.NONE);
    _buttonAbbrechen.setText(" Abbrechen ");
    _buttonAbbrechen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        _sonderProgramm = null;
        dispose();
      }
    });

  }
  
  /**
   * Schliesst den Dialog
   */
  public void dispose()
  {
    if (!shell.isDisposed())
    {
      shell.dispose();
    }
  }
}
