/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.util;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;

import com.bitctrl.util.Interval;
import com.sun.xml.internal.bind.v2.TODO;

import de.bsvrz.buv.plugin.anlagendarstellung.PickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.EditPartPickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.EditPartPickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.model.KilometrierungZuordnung;
import de.bsvrz.buv.plugin.anlagendarstellung.model.MathHelper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.BaustellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.SituationsWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.StauWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.UnfallWrapper;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Stau;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnitt;

/**
 * Helfer Klasse zum Verarbeitung der Situationen in der Anlagendarstellung (Bustelle, Unfall, Stau)
 * 
 * @author Pittner
 * 
 */
public class SituationUtil
{
  /**
   * Verarbeitet die Daten von Situationen fr die Anlagendarstellung
   * @param viewer
   *          der {@link GraphicalViewer}
   * @param result
   *          das DaV-Daten {@link ResultData}
   */
  public static void verarbeiteSituationUpdate(final GraphicalViewer viewer, final ResultData result)
  {
    if (viewer != null){
      
      AnsichtEditPart contents = (AnsichtEditPart)viewer.getContents();
      
      if (result.getObject().isValid() && result.hasData())
      {
        Data data = result.getData();
        
        Array array = data.getItem("StraenSegment").asArray();
        if (array.getLength() > 0)
        {
          ObjektFactory objektFactory = DObjPlugin.getDefault().getObjektFactory();
          final SystemObjekt systemObjekt = objektFactory.getModellobjekt(result.getObject());
          
          
          SituationsWrapper wrapper = null;
          
//          SituationsWrapper wrapper = SituationsWrapperProvider.getInstanz().getSituationsWrapper(result.getObject().getPid()); 
//          if (wrapper == null)
          {
            if (systemObjekt instanceof Baustelle)
            {
              try
              {
                Baustelle baustelle = (Baustelle)systemObjekt;
                wrapper = new BaustellenWrapper(baustelle);
              }
              catch (Exception e)
              {
                // TODO Auto-generated catch block
                e.printStackTrace();
              }
            }
            else if (systemObjekt instanceof Unfall)
            {
              try
              {
                Unfall unfall = (Unfall)systemObjekt;
                wrapper = new UnfallWrapper(unfall);
              }
              catch (Exception e)
              {
                // TODO Auto-generated catch block
                e.printStackTrace();
              }
            }
            else if (systemObjekt instanceof Stau)
            {
              try
              {
                Stau stau = (Stau)systemObjekt;             
                wrapper = new StauWrapper(stau);
              }
              catch (Exception e)
              {
                // TODO Auto-generated catch block
                e.printStackTrace();
              }
            }
          } 
          
          if (wrapper != null){
            SituationsWrapperProvider.getInstanz().addSituationsWrapper(result.getObject().getPid(), wrapper);
            if (!wrapper.getRichtung().equals(AttTmcRichtung.ZUSTAND_0_OHNE_RICHTUNG)
                && wrapper.getStreckenAbschnitt() != null)
              holeBzwErzeugeDarstellungsObjekte(viewer, result, wrapper);                
          }
        }
        
      }else if (!result.getObject().isValid()){
        
//      System.out.println("Loesche Situation: " + result.getObject());
        
        SystemObject so = result.getObject();
        PickerParent pp = DaVHelper.holePickerParentVonSystemObject2(contents, so);
        SituationsWrapper wrapper = SituationsWrapperProvider.getInstanz().getSituationsWrapper(so.getPid());
        if (pp != null){
          PickerVertical pv = null;
          for (Object object2 : pp.getKomponenten()){
            PickerVertical loc = (PickerVertical)object2;
            if (loc.getSystemObject().getPid().equals(result.getObject().getPid()))
            {
              //System.out.println("---> gefunden: " + loc.getSystemObject().getPid());
              loc.setSichtbar(false);
              pv = loc;
            }
          }
          
          
          if (pv != null){
            
            double bisKm = wrapper.getStreckenAbschnitt().getStartBetriebsKilometer();
            double vonKm = wrapper.getStreckenAbschnitt().getEndBetriebsKilometer();
            
            
            if (wrapper.getRichtung().equals(AttTmcRichtung.ZUSTAND_1N_NEGATIV))
            {      
              vonKm = wrapper.getStreckenAbschnitt().getStartBetriebsKilometer();
              bisKm = wrapper.getStreckenAbschnitt().getEndBetriebsKilometer();
            }
            
            Point pvLoc = pv.getLocation();
            int fr = pp.getFahrtrichtung();
            List<PickerVertical> list = pruefeUeberSchneidung(vonKm, bisKm, pv, pp);
            
            for (PickerVertical tmp : list)
            {
              
              Point loc = tmp.getLocation();
              
//            if (object2.isVerschiebbar() && !object2.isVerschoben()){ 
              if (fr == 0 && pvLoc.x > loc.x || fr == 1 && pvLoc.x < loc.x)
              {
                if (fr == 0)
                {
                  loc.x += 10;
                }
                else
                {
                  loc.x -= 10;
                }
                tmp.setLocation(loc);
              }
            }
            
            pp.getKomponenten().remove(pv);
            if (pp.getKomponenten().size() == 1)
              pp.getKomponenten().get(0).setVerschiebbar(false);
          }        
        }                   
      }
    }
    
    
  }

  /**
   * Holt bzw. Erzeugt die Darstellungsobjekte aus/fuer die Darstellung
   * 
   * @param viewer
   *          der {@link GraphicalViewer}
   * @param result
   *          die DaV-Daten {@link ResultData}
   * @param wrapper
   *          der {@link SituationsWrapper}, liefert BKM und Richtung
   */
  public static void holeBzwErzeugeDarstellungsObjekte(GraphicalViewer viewer, ResultData result, SituationsWrapper wrapper)
  {
    AttTmcRichtung richtung = wrapper.getRichtung();
    SystemObject systemObject = result.getObject();
    
    if (viewer != null){
      
    AnsichtEditPart contents = (AnsichtEditPart)viewer.getContents();
    PickerParent domParent = DaVHelper.holePickerParentVonSystemObject(contents, result.getObject(), richtung);          
    if (domParent != null)
    {
      PickerVertical d = null;                    
      EList<PickerVertical> komponenten = domParent.getKomponenten();
      for (PickerVertical pv : komponenten)
      {
        if (pv.getSystemObject().getPid().equals(result.getObject().getPid())){
          d = pv; 
          break;
        }
      }
      EditPartPickerVertical gep = null;
      
      boolean create = false;
      
      // Wenn kein darstellungsobjekt gefunden wurde, wird ein Neues erzeugt.
      if (result.getObject().isValid() && d == null)
      {              
        create = true;              
        ConfiguratedDoTyp doTyp = DarstellungExtensionPoint.getDoTyp("PickerVertical");
        d = (PickerVertical)doTyp.createModel();    
        
        domParent.getKomponenten().add(d);
        gep = (EditPartPickerVertical)doTyp.createController();            
        gep.setModel(d);
        d.setSystemObject(systemObject);
        d.setName(systemObject.getName());
        d.setLocation(new Point(domParent.getLocation()));
        d.setYPosPickerTop(0);
        d.setSichtbar(true);
        if (domParent.getKomponenten().size() != 1)
          d.setVerschiebbar(true);
        else 
          d.setVerschiebbar(false);
      }
      
      Map editPartRegistry = viewer.getEditPartRegistry();            
      if (domParent != null && editPartRegistry.containsKey(domParent))
      {
        Object object = editPartRegistry.get(domParent);
        if (object instanceof EditPartPickerParent)
        {
          EditPartPickerParent pep = (EditPartPickerParent)object;
          if (create){
            gep.setParent(pep);
            gep.addNotify();
            gep.activate();                  
          }else{
            List children = pep.getChildren();       
            for (Object object2 : children)
            {
              if (object2 instanceof EditPartPickerVertical)
              {
                EditPartPickerVertical tmp = (EditPartPickerVertical)object2;                  
                PickerVertical castedModel = tmp.getCastedModel();
                if (castedModel.getSystemObject().getPid().equals(systemObject.getPid())){
                  gep = tmp;
                }
              }
            }
          }
          Map.Entry<Double, Double> me = aktionStreckenAbschnitt(wrapper, gep, pep);
          if (me != null){
            adaptiereSituationFuerDarstellung(pep, gep, me.getKey(), me.getValue());
          }
        } 
      }            
    }
    
    }
    
  }

  
  /**
   * Sorgt fr die korrekt Reihenfolge
   *
   * {@link TODO} geht im Moment noch davon aus, dass die FR der AQ "nicht gedreht" -> NO bzw "gedreht" -> SW ist. 
   * 
   * @param wrapper
   *          der {@link SituationsWrapper}
   * @param gep
   *          der entsprechende {@link GraphicalEditPart} einer {@link Situation} aus der {@link Darstellung}
   * @param pep
   *          der Parent-{@link GraphicalEditPart}
   */
  private static Map.Entry<Double, Double> aktionStreckenAbschnitt(SituationsWrapper wrapper,
      EditPartPickerVertical gep, EditPartPickerParent pep)
  {
    StreckenAbschnitt streckenAbschnitt = wrapper.getStreckenAbschnitt();
    
    double bis = streckenAbschnitt.getStartBetriebsKilometer();
    double von = streckenAbschnitt.getEndBetriebsKilometer();
    
    
    if (wrapper.getRichtung().equals(AttTmcRichtung.ZUSTAND_1N_NEGATIV))
    {      
      von = streckenAbschnitt.getStartBetriebsKilometer();
      bis = streckenAbschnitt.getEndBetriebsKilometer();
    }
    return new AbstractMap.SimpleEntry<Double, Double>(von, bis);
  }
  
  
  
   
  
  /**
   * Berechnet die Position und Ausdehnung fr den Picker, sofern diese Darstellung
   * ber einen Picker verfgt.
   */
  /**
   * @param parent
   *          der Parent-{@link GraphicalEditPart}
   * @param editPart
   *          der entsprechende {@link GraphicalEditPart} einer {@link Situation} aus der {@link Darstellung}
   * @param vonKm
   *          Start-Kilometer
   * @param bisKm
   *          Ende-Kilometer
   */
  private static void adaptiereSituationFuerDarstellung(GraphicalEditPart parent, GraphicalEditPart editPart, double vonKm, double bisKm)
  {    
    /*
     * Zuordnung der bestimmten Kilometer zu den Bereichen
     * (KilometrierungZuordnungen) des Pickers bestimmen. Y-Start und Y-Ende
     * bestimmen.
     */
    if (editPart instanceof EditPartPickerVertical)
    {     
      EditPartPickerVertical pickerVerticalEditPart = (EditPartPickerVertical)editPart;
      PickerVertical castedModel = pickerVerticalEditPart.getCastedModel();
//      System.out.println("SituationUtil.adaptiereSituationFuerDarstellung(): " + castedModel.getSystemObject() + " -> " + vonKm + " : " + bisKm);
      if (castedModel != null)
      {
        if (parent instanceof EditPartPickerParent)
        {
          EditPartPickerParent pickerParentEditPart = (EditPartPickerParent)parent;
          PickerParent parentModel = pickerParentEditPart.getCastedModel();
          String strKilometrierungZuordnung = parentModel.getKilometrierungZuordnung();
          if (strKilometrierungZuordnung != null)
          {
            LinkedList<KilometrierungZuordnung> lKMZu = KilometrierungZuordnung
            .deserFromMultipleString(strKilometrierungZuordnung);
            
            KilometrierungZuordnung zuVon = KilometrierungZuordnung
            .bestimmeKilometrierungZuordnungFuerKm(lKMZu, vonKm);
            KilometrierungZuordnung zuBis = KilometrierungZuordnung
            .bestimmeKilometrierungZuordnungFuerKm(lKMZu, bisKm);
            
            double yStart = -1.0d;
            double yEnde = -1.0d;
            
            if (zuVon != null)
            {
              double prozentBereichVon = MathHelper.berechneFortschrittProzent(
                  zuVon.getKmVon(), zuVon.getKmBis(), vonKm);
              
              yStart = MathHelper.berechnePositionNachProzent(zuVon.getyVon(),
                  zuVon.getyBis(), prozentBereichVon);
            }
            
            if (zuBis != null)
            {
              double prozentBereichBis = MathHelper.berechneFortschrittProzent(
                  zuBis.getKmVon(), zuBis.getKmBis(), bisKm);
              
              yEnde = MathHelper.berechnePositionNachProzent(zuBis.getyVon(),
                  zuBis.getyBis(), prozentBereichBis);
            }
            
            if ((yStart >= 0.0d) && (yEnde >= 0.0d))
            {
              /*
               * Runden und nach int fr die absolute Bildschirmposition casten.
               */
              int iYStart = (int) (Math.round(yStart)); // Direkt an PickerModell
              int iYEnde = (int) (Math.round(yEnde)); // geht NICHT an das
                            
              if (iYEnde >= iYStart)
              {               
                // Distanz fr den Picker
                int dist = iYEnde - iYStart;
                if (castedModel.getLocation().y != iYStart)
                {
                    Point pt = new Point(castedModel.getLocation().x, iYStart); 
                    int fr = parentModel.getFahrtrichtung();
                    
                    // Die Darstellung in FR SW wird rechtsbndig vorgenommen, dies wir dann im Model gespeichert,
                    // damit nicht noch einmal versetzt wird.
                    if (fr == 0 && !castedModel.isVersetzt()){
                      pt.x += parentModel.getBreite()-4;
                      castedModel.setVersetzt(true);   
                    }                    
                    
                   
                    List<PickerVertical> list = pruefeUeberSchneidung(vonKm, bisKm, castedModel, parentModel);                    
                    for (PickerVertical tmp : list)
                    {                      
                      // Erste Komponente wird nicht bercksichtigt, wenn es Ueberschneidungen gibt.
                      // Zustzlich wird nur einmal verschoben und dann im Modell abgespeichert, damit nicht
                      // noch einmal verschoben wird. 
                      if (castedModel.isVerschiebbar() && !castedModel.isVerschoben()){ 
                        if (parentModel.getFahrtrichtung() == 0){
                          pt.x -= 10;                            
                        }else{
                          pt.x += 10;                                                        
                        }                                                                       
                      }                              
                    }
                    if (list.size() > 0)
                      castedModel.setVerschoben(true);                          
                    
                    castedModel.setLocation(pt);  
                    
                }                
                if (castedModel.getDistanz() != dist)
                {    
                  castedModel.setBreite(5);
                  castedModel.setHoehe(dist);
                  castedModel.setDistanz(dist);                  
                }
                if (castedModel.getVonKm() != vonKm)
                {
                  castedModel.setVonKm(vonKm);             
                }
                if (castedModel.getBisKm() != bisKm)
                {     
                  castedModel.setBisKm(bisKm);                   
                }
                if (castedModel.getFarbe() != parentModel.getFarbeVordergrund())
                {     
                  castedModel.setFarbe(parentModel.getFarbeVordergrund());                  
                }               
              }
            }
          }
        }        
      }
    }
  }
                                                       
  /**
   * Liefert eine Liste mit den sich berschneidenden Modell-Objekten. 
   * Wird fr die versetzte Darstellung bentigt.
   * 
   * @param vonKm
   *          Start-Kilometer   *          
   * @param bisKm
   *          Ende-Kilometer
   * @param castedModel
   *          das Darstellungs-Objekt der {@link Situation}
   * @param parentModel
   *          das Eltern-Darstellungs-Objekt der {@link Situation}
   * @return
   */
  private static List<PickerVertical> pruefeUeberSchneidung(double vonKm, double bisKm, PickerVertical castedModel, PickerParent parentModel)
  {
    List<PickerVertical> re = new ArrayList<PickerVertical>();                    
    
    long l1 = new Double(vonKm * 1000.0).longValue();
    long l2 = new Double(bisKm * 1000.0).longValue();
    
    Interval i1 = new Interval(l1, l2);
    
    for (PickerVertical loc : parentModel.getKomponenten())
    {
      if (!loc.equals(castedModel)){
        long l3 = new Double(loc.getVonKm() * 1000.0).longValue();
        long l4 = new Double(loc.getBisKm() * 1000.0).longValue();
        Interval i2 = new Interval(l3, l4);
        
        boolean intersect = i1.intersect(i2) || i1.equals(i2) || i1.contains(i2) || i2.contains(i1) ? true : false;
//        System.out.println(castedModel.getSystemObject().getPid() + " -> " + loc.getSystemObject().getPid() + " : " + intersect + " : " + castedModel.isVerschiebbar() + " : " + castedModel.isVerschoben());
       
        if (intersect && !re.contains(loc))
          re.add(loc);        
      }                  
    }
    
    return re;
  }

}
