/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Dialog zum ndern der Einstellungen von Anzeigen.<br>
 * Die Anzeigequerschnitte, deren Einstellungen gendert werden soll, werden als Textliste (;-getrennt) in der secondary
 * Viev-ID bergeben. Die Textliste kann PID von Objekten vom Typ Anzeige oder vom Typ SBAAnzeige (auch gemischt)
 * enthalten.
 * 
 * @author Pittner
 * 
 */
public class AnzeigenEinstellenDialog extends Dialog
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.anzeigenEinstellen";

  private Shell shell;

  public AnzeigenEinstellenDialog(Shell parent)
  {
    super(parent);
    _parent = parent;
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    shell.setText("Einstellung Anzeigen");
    shell.setLayout(new GridLayout());
  }

  private Composite _parent;

  private Composite _compositeAnzeigeQuerschnitte;

  private Group _gruppeTabelleAnzeigen;

  private TableViewer _tvAnzeigen;

  private Composite _compositeSchalten;

  private Composite _compositeButton;

  private Button _buttonSchalten;

  private Button _buttonAbbrechen;

//  private final static String _strAtgSswSbaMassnahmenVorschlagAnforderung = "atg.sswSbaManahmeVorschlagAnforderung";
//
//  private final static String _strAspSswSbaAnforderung = "asp.sswSbaAnforderung";

  private Map<String, ModelObject> _mapModel = new LinkedHashMap<String, ModelObject>();

  private Group _gruppeSchalten2;

  private String _viewId;

  private void uebernehmeDatenAusDarstellung()
  {
    List<AnzeigeModelTypAbstract> list = holeAnzeigeModelTypAbstractAusViewer();
    for (AnzeigeModelTypAbstract doModel : list)
    {
      doModel.setPrioritaet(_comboPrioritaet.getText());

      int index = _comboNotwendig.getSelectionIndex();

      switch (index)
      {
        case 0:
          doModel.setNotwendig(1);
          doModel.setAbsolutNotwendig(1);
          break;
        case 1:
          doModel.setNotwendig(1);
          doModel.setAbsolutNotwendig(0);
          break;
        case 2:
          doModel.setNotwendig(0);
          doModel.setAbsolutNotwendig(0);
          break;

        default:
          doModel.setNotwendig(0);
          doModel.setAbsolutNotwendig(0);
      }

      index = _comboBlinktakt.getSelectionIndex();
      switch (index)
      {
        case 0:
          doModel.setBlinken(0);
          doModel.setBlinktakt("Undefiniert");
          break;
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        case 7:
        case 8:
        case 9:
        case 10:
        case 11:
          doModel.setBlinken(1);
          Double d = 1.0;
          d += (index - 1) * 0.2;
          doModel.setBlinktakt(d.toString().substring(0, 3));
          break;

        default:
          doModel.setBlinken(0);
          doModel.setBlinktakt("Undefiniert");
      }
    }
  }

  private GraphicalViewer getViewer()
  {

    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    if (_viewId != null && _secId != null)
    {
      IViewReference ref = page.findViewReference(_viewId, _secId);
      if (ref != null)
      {
        IViewPart view = ref.getView(false);
        if (view != null)
          return (GraphicalViewer)view.getAdapter(GraphicalViewer.class);
      }
    }
    return null;
  }

  public int oeffne(String viewId, String secondaryId)
  {

    _viewId = viewId;
    init(secondaryId);

    createParent(shell);

    uebernehmeDatenAusModell();

    shell.setSize(new Point(500, 330));

    shell.open();
    Display display = _parent.getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return 1;
  }

  public void init(String secondaryId)
  {
    String pidsAqText = "";
    if (secondaryId != null)
    {
      String[] components = secondaryId.split("#");
      if (components.length > 1)
      {
        pidsAqText = components[0];
        _secId = components[1];
      }

    }

    String[] pidsAqArray = pidsAqText.split(";");

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();

      for (String pid : pidsAqArray)
      {
        SystemObject so = connection.getDataModel().getObject(pid);
        if (so != null)
          _mapModel.put(so.getPid(), new ModelObject(so.getName()));
        else
          System.out.println(pid + " ist null!!!");
      }

      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);

      SystemObject object = null;

      if (pidSba != null && pidSba.length() > 0)
      {
        object = dataModel.getObject(pidSba);
      }

      List<String> list = new ArrayList<String>();

      if (object != null)
      {
        list.add(object.getPid());
      }
    }
  }

  // @Override
  public void createParent(Composite parent)
  {
    _parent = parent;

    _parent.setLayout(new GridLayout());
    createCompositeTabelleAnzeigen();
    createCompositeSchalten();
    createCompositeButton();

    if (_mapModel != null)
    {
      _tvAnzeigen.setInput(_mapModel.values());
      _buttonSchalten.setEnabled(_mapModel.size() > 0);
    }
  }

  private Combo _comboPrioritaet;

  private Group _gruppeSchalten3;

  private Combo _comboBlinktakt;

  private Group _gruppeSchalten4;

  private Combo _comboNotwendig;

  private String _secId;

  private void createCompositeTabelleAnzeigen()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeAnzeigeQuerschnitte = new Composite(_parent, SWT.NONE);
    _compositeAnzeigeQuerschnitte.setLayout(new GridLayout());
    _compositeAnzeigeQuerschnitte.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;

    GridData gridDataGruppeAnzeigequerschnitte = new GridData();
    gridDataGruppeAnzeigequerschnitte.horizontalAlignment = GridData.FILL;
    gridDataGruppeAnzeigequerschnitte.grabExcessHorizontalSpace = true;
    gridDataGruppeAnzeigequerschnitte.grabExcessVerticalSpace = true;
    gridDataGruppeAnzeigequerschnitte.verticalAlignment = GridData.FILL;
    _gruppeTabelleAnzeigen = new Group(_compositeAnzeigeQuerschnitte, SWT.NONE);
    _gruppeTabelleAnzeigen.setLayout(new GridLayout());
    _gruppeTabelleAnzeigen.setLayoutData(gridDataGruppeAnzeigequerschnitte);
    _gruppeTabelleAnzeigen.setText("Ausgewhlte Anzeigen");

    _tvAnzeigen = new TableViewer(_gruppeTabelleAnzeigen, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeigen, SWT.LEFT);
    column1.getColumn().setWidth(200);
    column1.getColumn().setText("Name");

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeigen, SWT.LEFT);
    column2.getColumn().setWidth(80);
    column2.getColumn().setText("Prioritt");

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeigen, SWT.LEFT);
    column3.getColumn().setWidth(80);
    column3.getColumn().setText("Blinktakt");

    TableViewerColumn column4 = new TableViewerColumn(_tvAnzeigen, SWT.LEFT);
    column4.getColumn().setWidth(90);
    column4.getColumn().setText("Notwendigkeit");

    _tvAnzeigen.setContentProvider(new ContentProvider());
    _tvAnzeigen.setLabelProvider(new LabelProvider());
    _tvAnzeigen.getTable().setLayoutData(gridDataTabelle);
    _tvAnzeigen.getTable().setLinesVisible(true);
    _tvAnzeigen.getTable().setHeaderVisible(true);
  }

  private void createCompositeSchalten()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeSchalten = new Composite(_parent, SWT.NONE);
    _compositeSchalten.setLayout(new GridLayout(2, false));
    _compositeSchalten.setLayoutData(gridData);
    GridLayout layoutGrp = new GridLayout(2, false);
    GridData gridDataGrp = new GridData();
    gridDataGrp.horizontalAlignment = GridData.FILL;
    gridDataGrp.grabExcessHorizontalSpace = true;
    gridDataGrp.verticalAlignment = GridData.FILL;
    GridData gridDataGrp2 = new GridData();
    gridDataGrp2.horizontalAlignment = GridData.FILL;
    gridDataGrp2.grabExcessHorizontalSpace = true;
    gridDataGrp2.verticalAlignment = GridData.FILL;
    _gruppeSchalten2 = new Group(_compositeSchalten, SWT.NONE);
    _gruppeSchalten2.setLayoutData(gridDataGrp);
    _gruppeSchalten2.setLayout(layoutGrp);
    _gruppeSchalten2.setText("Zu schaltende Prioritt");

    _comboPrioritaet = new Combo(_gruppeSchalten2, SWT.NONE);
    _comboPrioritaet.add("undefiniert", 0);
    _comboPrioritaet.add("0", 1);
    _comboPrioritaet.add("60000", 2);
    _comboPrioritaet.setTextLimit(5);
    _comboPrioritaet.setLayout(GridLayoutFactory.fillDefaults().create());
    _comboPrioritaet.setLayoutData(gridDataGrp);

    _gruppeSchalten3 = new Group(_compositeSchalten, SWT.NONE);
    _gruppeSchalten3.setLayoutData(gridDataGrp);
    _gruppeSchalten3.setLayout(layoutGrp);
    _gruppeSchalten3.setText("Zu schaltender Blinktakt");

    _comboBlinktakt = new Combo(_gruppeSchalten3, SWT.READ_ONLY);
    _comboBlinktakt.add("Undefiniert", 0);
    _comboBlinktakt.add("1.000 s", 1);
    _comboBlinktakt.add("1.200 s", 2);
    _comboBlinktakt.add("1.400 s", 3);
    _comboBlinktakt.add("1.600 s", 4);
    _comboBlinktakt.add("1.800 s", 5);
    _comboBlinktakt.add("2.000 s", 6);
    _comboBlinktakt.add("2.200 s", 7);
    _comboBlinktakt.add("2.400 s", 8);
    _comboBlinktakt.add("2.600 s ", 9);
    _comboBlinktakt.add("2.800 s", 10);
    _comboBlinktakt.add("3.000 s", 11);
    _comboBlinktakt.setLayout(GridLayoutFactory.fillDefaults().create());
    _comboBlinktakt.setLayoutData(gridDataGrp);

    _gruppeSchalten4 = new Group(_compositeSchalten, SWT.NONE);
    _gruppeSchalten4.setLayoutData(gridDataGrp);
    _gruppeSchalten4.setLayout(layoutGrp);
    _gruppeSchalten4.setText("Zu schaltende Notwendigkeit");

    _comboNotwendig = new Combo(_gruppeSchalten4, SWT.READ_ONLY);
    _comboNotwendig.add("absolut notwendig", 0);
    _comboNotwendig.add("notwendig", 1);
    _comboNotwendig.add("nicht notwendig", 2);
    _comboNotwendig.setLayout(new GridLayout());
    _comboNotwendig.setLayoutData(gridDataGrp);
  }

  private void uebernehmeDatenAusModell()
  {

    List<AnzeigeModelTypAbstract> list = holeAnzeigeModelTypAbstractAusViewer();

    for (int i = 0; i < list.size(); i++)
    {
      AnzeigeModelTypAbstract doModel = list.get(i);
      
      SystemObject so = doModel.getSystemObject();
      if (so != null && _mapModel != null)
      {
        ModelObject modelObject = _mapModel.get(so.getPid());
        if (modelObject != null)
        {
          modelObject.setBlinken(doModel.getBlinktakt());

          boolean b1 = doModel.getNotwendig() > 0 ? true : false;
          boolean b2 = doModel.getAbsolutNotwendig() > 0 ? true : false;

          String s = "";
          if (b1 && b2)
          {
            s = "absolut notwendig";
          }
          else if (b1 && !b2)
          {
            s = "notwendig";
          }
          else
          {
            s = "nicht notwendig";
          }
          modelObject.setNotwendig(s);
          modelObject.setPrio(doModel.getPrioritaet());

        }
      }

      if (i == 0)
      {
        String[] items = _comboPrioritaet.getItems();
        List<String> listItems = Arrays.asList(items);
        String prio = doModel.getPrioritaet();

        if (prio != null && listItems.contains(prio))
        {
          _comboPrioritaet.select(listItems.indexOf(prio));
        }
        else
        {
          _comboPrioritaet.add(prio);
          items = _comboPrioritaet.getItems();
          Arrays.sort(items);
          listItems = Arrays.asList(items);

          _comboPrioritaet.removeAll();
          for (String s : listItems)
          {
            _comboPrioritaet.add(s);
          }

          int index = listItems.indexOf(prio);
          _comboPrioritaet.select(index);

        }

        boolean blinken = doModel.getBlinken() > 0 ? true : false;
        String blinktakt = doModel.getBlinktakt();
        
        
        //System.out.println(doModel.getSystemObject()  + "AnzeigenEinstellenDialog.uebernehmeDatenAusModell(): " + blinken + " -> " + blinktakt);

        if (blinken)
        {
          items = _comboBlinktakt.getItems();
          listItems = Arrays.asList(items);
          if (blinktakt != null)
          {
            int index = 0;
            for (int j = 0; j < items.length; j++)
            {
              String s = items[j];
              String tmp = blinktakt;
              if (blinktakt.contains(","))
                tmp = blinktakt.replace(",", ".");
              if (s.contains(tmp))
              {
                index = j;
                break;
              }
            }
            _comboBlinktakt.select(index);
          }
          else
          {
            _comboBlinktakt.select(0);
          }
        }
        else
        {
          _comboBlinktakt.select(0);
        }

        items = _comboNotwendig.getItems();
        listItems = Arrays.asList(items);

        ModelObject modelObject = _mapModel.get(doModel.getSystemObject().getPid());
        if (modelObject != null)
        {
          String notwendig = modelObject.getNotwendig();
          if (notwendig != null && listItems.contains(notwendig))
          {
            _comboNotwendig.select(listItems.indexOf(notwendig));
          }
        }
        else
        {
          _comboNotwendig.select(2);
        }
      }
    }

    if (_tvAnzeigen != null)
      _tvAnzeigen.refresh();

  }

  private void createCompositeButton()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton = new Composite(_parent, SWT.NONE);
    _compositeButton.setLayoutData(gridData);
    _compositeButton.setLayout(rowLayout);

    _buttonSchalten = new Button(_compositeButton, SWT.NONE);
    _buttonSchalten.setText(" bernehmen ");
    _buttonSchalten.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        uebernehmeDatenAusDarstellung();
        dispose();
      }
    });

    _buttonAbbrechen = new Button(_compositeButton, SWT.NONE);
    _buttonAbbrechen.setText(" Abbrechen ");
    _buttonAbbrechen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        dispose();
      }
    });

  }

  private static class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      return ((Collection)inputElement).toArray();
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  private static class LabelProvider implements ITableLabelProvider
  {
    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String result = null;
      ModelObject modelObject = (ModelObject)element;

      switch (columnIndex)
      {
        case 0:
          String name = modelObject.getName();
          result = name != null ? name : "";
          break;
        case 1:
          String prioritaet = modelObject.getPrio();
          result = prioritaet != null ? prioritaet : "";
          break;
        case 2:
          String blinktakt = modelObject.getBlinken();

          result = blinktakt != null ? blinktakt.replace(",", ".") : "";
          break;
        case 3:
          String notwendig = modelObject.getNotwendig();
          result = notwendig != null ? notwendig : "";
          break;
        default:
          result = "";
          break;
      }

      return result;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }

  private List<AnzeigeModelTypAbstract> holeAnzeigeModelTypAbstractAusViewer()
  {
    List<AnzeigeModelTypAbstract> re = new ArrayList<AnzeigeModelTypAbstract>();

    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection sel = viewer.getSelection();
      if (sel instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)sel;
        Iterator it = ss.iterator();
        while (it.hasNext())
        {
          Object obj = it.next();
          if (obj instanceof DoMitVerbLinieEditPart)
          {
            DoMitVerbLinieEditPart ep = (DoMitVerbLinieEditPart)obj;
            DoModelMitVerbLinie model = ep.getModel();
            if (model instanceof AnzeigeModelTypAbstract)
            {
              AnzeigeModelTypAbstract doModel = (AnzeigeModelTypAbstract)model;
              re.add(doModel);
            }
          }
        }
      }
    }
    return re;
  }

  public void dispose()
  {
    if (!shell.isDisposed())
    {
      shell.dispose();
    }
  }

  private static class ModelObject
  {

    private String _name;

    private String _prio = "undefiniert";

    private String _blinken = "Undefiniert";

    private String _notwendig = "nicht notwendig";

    public ModelObject(String name)
    {
      _name = name;
    }

    public String getName()
    {
      return _name;
    }

    public String getPrio()
    {
      return _prio;
    }

    public String getBlinken()
    {
      return _blinken;
    }

    public String getNotwendig()
    {
      return _notwendig;
    }

    public void setPrio(String prio)
    {
      _prio = prio;
    }

    public void setBlinken(String blinken)
    {
      _blinken = blinken;
    }

    public void setNotwendig(String notwendig)
    {
      _notwendig = notwendig;
    }
  }
}
