/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Sicht zur Anzeige von Steuermodul-Daten.
 * 
 * @author Pittner
 * 
 */ 
public class SteuermodulAnzeigeView extends ViewPart implements IDaVDataUpdateListener
{
  public static final String ID = "de.bsvrz.buv.plugin.anlagendarstellung.steuermodulanzeige";

  private CTabFolder _tabFolder = null;

  private Composite _parent;

  private Composite _composite = null;

  private IMemento _memento = null;

  private String _secondViewId;

  private ClientDavInterface _connection;

  private TableViewer _tv1;

  private TableViewer _tv2;

  private TableViewer _tv3;

  private Label _l1;

  private Label _l2;

  private Label _l3;

  private SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private int _selection = 0;

  private Label _l4;

  private TableViewer _tv4;

  private Label _l5;

  private TableViewer _tv5;

  private Map<String, Integer> _soMap;

  private Label _l21;

  private Label _l31;

  private Label _l41;

  private Label _l51;

  private Label _l11;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".zuflussParameter");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite(parent, SWT.NONE);
    _composite.setLayout(gridLayout);
    _composite.setLayoutData(gridData);

    restoreState();

    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    erzeugeCompositeTop();
    createCompositeTab();
    erzeugeTabItems();

    SystemObject so = _connection.getDataModel().getObject(_secondViewId);

    if (so != null)
      erzeugeDAVanmeldeDaten(so);

  }

  private void erzeugeCompositeTop()
  {
    SystemObject so = _connection.getDataModel().getObject(_secondViewId);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.gert");
    Data data = so.getConfigurationData(atg);
    if (data != null)
    {
      setPartName(so.getName());

      Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);

      Composite c01 = new Composite(_composite, SWT.NONE);
      c01.setLayout(new GridLayout(2, false));
      c01.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

      Label l11 = new Label(c01, SWT.NONE);
      l11.setText("KnNr : ");
      l11.setFont(boldFont);
      String nummer = "";

      long l = data.getItem("KnotenNummer").asUnscaledValue().longValue();
      if (l > 0)
      {
        Long l1 = l / 256L;
        Long l2 = l % 256L;

        nummer = l1 + " - " + l2;
      }

      Label l12 = new Label(c01, SWT.NONE);
      if (nummer != null)
      {
        l12.setText(nummer);
      }
    }
  }

  private void erzeugeTabelle1(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv1 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv1, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Attribut");

    TableViewerColumn column2 = new TableViewerColumn(_tv1, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Ein");

    _tv1.setContentProvider(new ArrayContentProvider());
    _tv1.setLabelProvider(new TableLabelProvider());
    _tv1.getTable().setLayoutData(gridData);
    _tv1.getTable().setLinesVisible(true);
    _tv1.getTable().setHeaderVisible(false);

  }

  private void erzeugeTabelle2(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv2 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv2, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("1");

    TableViewerColumn column2 = new TableViewerColumn(_tv2, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("2");

    _tv2.setContentProvider(new ArrayContentProvider());
    _tv2.setLabelProvider(new TableLabelProvider());
    _tv2.getTable().setLayoutData(gridData);
    _tv2.getTable().setLinesVisible(true);
    _tv2.getTable().setHeaderVisible(false);

  }

  private void erzeugeTabelle3(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv3 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv3, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Attribut");

    TableViewerColumn column2 = new TableViewerColumn(_tv3, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Ein");

    _tv3.setContentProvider(new ArrayContentProvider());
    _tv3.setLabelProvider(new TableLabelProvider());
    _tv3.getTable().setLayoutData(gridData);
    _tv3.getTable().setLinesVisible(true);
    _tv3.getTable().setHeaderVisible(false);

  }

  private void erzeugeTabelle4(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv4 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv4, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("DEKanalStatus");

    TableViewerColumn column2 = new TableViewerColumn(_tv4, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("aktiv");

    _tv4.setContentProvider(new ArrayContentProvider());
    _tv4.setLabelProvider(new TableLabelProvider());
    _tv4.getTable().setLayoutData(gridData);
    _tv4.getTable().setLinesVisible(true);
    _tv4.getTable().setHeaderVisible(false);
  }

  private void erzeugeTabelle5(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv5 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv5, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("DEKanalStatus");

    TableViewerColumn column2 = new TableViewerColumn(_tv5, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("aktiv");

    _tv5.setContentProvider(new ArrayContentProvider());
    _tv5.setLabelProvider(new TableLabelProvider());
    _tv5.getTable().setLayoutData(gridData);
    _tv5.getTable().setLinesVisible(true);
    _tv5.getTable().setHeaderVisible(false);
  }

  public class TableLabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {

      String returnString = "undefiniert";

      Data data = (Data)element;

      switch (columnIndex)
      {
        case 0:
          returnString = data.getName();
          break;
        case 1:
          returnString = data.asTextValue().getText();
          break;

        default:
      }
      return returnString;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }

    @Override
    public void dispose()
    {
      // TODO Auto-generated method stub

    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }
  }

  private void erzeugeDAVanmeldeDaten(SystemObject systemObject)
  {

    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);

    _soMap = new HashMap<String, Integer>();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
    ConfigurationObject co = (ConfigurationObject)systemObject;
    NonMutableSet set = co.getNonMutableSet("Eak");
    if (set != null)
    {
      for (SystemObject so2 : set.getElements())
      {
        ConfigurationObject co2 = (ConfigurationObject)so2;
        NonMutableSet set2 = co2.getNonMutableSet("De");
        if (set2 != null)
        {
          for (SystemObject so3 : set2.getElements())
          {
            if (so3.isOfType("typ.deVlt"))
            {
              Data data = so3.getConfigurationData(atg);
              int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
              if (deKanal != 255)
              {
                _soMap.put(so3.getPid(), deKanal);
              }
            }
          }
        }
      }
    }

    List<String> soList = new ArrayList<String>();
    soList.addAll(_soMap.keySet());

    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSm(soList, "atg.tlsVltHeizung", "asp.tlsAntwort");
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSm(soList, "atg.tlsVltLicht", "asp.tlsAntwort");
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSm(soList, "atg.tlsVltStromVersorung", "asp.tlsAntwort");
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSm(soList, "atg.tlsVltTemperaturberwachung", "asp.tlsAntwort");
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSm(soList, "atg.tlsVltTrKontakt", "asp.tlsAntwort");
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * This method initializes _composite
   * 
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);     
  }

  public void erzeugeTabItems()
  {
    CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);
    tabItem.setText("Heizung");
    tabItem.setData("atg.tlsVltHeizung");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;

    Composite c1 = new Composite(_tabFolder, SWT.NONE);
    c1.setLayout(new GridLayout());
    c1.setLayoutData(gridData);

    _l11 = new Label(c1, SWT.NONE);
    _l11.setLayoutData(gridData2);   
    _l1 = new Label(c1, SWT.NONE);
    _l1.setLayoutData(gridData2);
    _l1.setText("--.--.---- --:--:--");

    erzeugeTabelle1(c1);
    tabItem.setControl(c1);

    CTabItem tabItem2 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem2.setText("Licht");
    tabItem2.setData("atg.tlsVltLicht");

    Composite c2 = new Composite(_tabFolder, SWT.NONE);
    c2.setLayout(new GridLayout());
    c2.setLayoutData(gridData);

    _l21 = new Label(c2, SWT.NONE);
    _l21.setLayoutData(gridData2);
    _l2 = new Label(c2, SWT.NONE);
    _l2.setLayoutData(gridData2);
    _l2.setText("--.--.---- --:--:--");

    erzeugeTabelle2(c2);

    tabItem2.setControl(c2);

    CTabItem tabItem3 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem3.setText("Stromversorgung");
    tabItem3.setData("atg.tlsVltStromVersorung");

    Composite c3 = new Composite(_tabFolder, SWT.NONE);
    c3.setLayout(new GridLayout());
    c3.setLayoutData(gridData);

    _l31 = new Label(c3, SWT.NONE);
    _l31.setLayoutData(gridData2);    
    _l3 = new Label(c3, SWT.NONE);
    _l3.setLayoutData(gridData2);
    _l3.setText("--.--.---- --:--:--");

    erzeugeTabelle3(c3);
    tabItem3.setControl(c3);

    CTabItem tabItem4 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem4.setText("Temperaturberwachung");
    tabItem4.setData("atg.tlsVltTemperaturberwachung");

    Composite c4 = new Composite(_tabFolder, SWT.NONE);
    c4.setLayout(new GridLayout());
    c4.setLayoutData(gridData);

    _l41 = new Label(c4, SWT.NONE);
    _l41.setLayoutData(gridData2);    
    _l4 = new Label(c4, SWT.NONE);
    _l4.setLayoutData(gridData2);
    _l4.setText("--.--.---- --:--:--");

    erzeugeTabelle4(c4);
    tabItem4.setControl(c4);

    CTabItem tabItem5 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem5.setText("Trkontakt");
    tabItem5.setData("atg.tlsVltTrKontakt");

    Composite c5 = new Composite(_tabFolder, SWT.NONE);
    c5.setLayout(new GridLayout());
    c5.setLayoutData(gridData);

    _l51 = new Label(c5, SWT.NONE);
    _l51.setLayoutData(gridData2);
    _l5 = new Label(c5, SWT.NONE);
    _l5.setLayoutData(gridData2);
    _l5.setText("--.--.---- --:--:--");

    erzeugeTabelle5(c5);
    tabItem5.setControl(c5);

    _tabFolder.setSelection(0);
  }

  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init(site, memento);
  }

  private void restoreState()
  {
    if (_memento == null)
    {
      return;
    }
    _secondViewId = _memento.getString("pidSteuermodulAnzeige");
    _selection = _memento.getInteger("selektionSm");
  }

  @Override
  public void saveState(IMemento memento)
  {
    memento.putString("pidSteuermodulAnzeige", _secondViewId);
    memento.putInteger("selektionSm", _tabFolder.getSelectionIndex());
    super.saveState(memento);
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    if (e != null && _soMap.keySet().contains(e.getResult().getObject().getPid()))
    {
      verarbeiteDavDaten(e);
    }
  }

  /**
   * @param e
   */
  private void verarbeiteDavDaten(DaVDataUpdateEvent e)
  {   
    List<Data> list = new ArrayList<Data>();

    String pidAtg = e.getResult().getDataDescription().getAttributeGroup().getPid();

    String time = _sdf.format(new Date(e.getResult().getDataTime())) + " Uhr";

    SystemObject so = e.getResult().getObject();
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
    Data cdata = so.getConfigurationData(atg);

    int deKanal = 0;

    if (cdata != null)
    {
      deKanal = cdata.getItem("DEKanal").asUnscaledValue().intValue();
    }

    if (e.getResult().getDataState().equals(DataState.NO_SOURCE)
        || e.getResult().getDataState().equals(DataState.NO_DATA))
    {

      String s = "";
      if (e.getResult().getDataState().equals(DataState.NO_SOURCE))
        s = " (keine Quelle)";
      else if (e.getResult().getDataState().equals(DataState.NO_DATA))
        s = " (keine Daten)";

      if (pidAtg.equals("atg.tlsVltHeizung"))
      {        
        _l1.setText(time + s);
        _tv1.setInput(list);
      }
      else if (pidAtg.equals("atg.tlsVltLicht"))
      {       
        _l2.setText(time + s);
        _tv2.setInput(list);
      }
      else if (pidAtg.equals("atg.tlsVltStromVersorung"))
      {       
        _l3.setText(time + s);
        _tv3.setInput(list);
      }
      else if (pidAtg.equals("atg.tlsVltTemperaturberwachung"))
      {       
        _l4.setText(time + s);
        _tv4.setInput(list);
      }
      else if (pidAtg.equals("atg.tlsVltTrKontakt"))
      {      
        _l5.setText(time + s);
        _tv5.setInput(list);
      }
    }
    else
    {
      Data data = e.getResult().getData();
      if (data != null)
      {
        Iterator it = data.iterator();
        while (it.hasNext())
        {
          Data plainData = (Data)it.next();
          if (plainData.isPlain())
          {
            list.add(plainData);
          }
        }

        if (pidAtg.equals("atg.tlsVltHeizung"))
        {
          _l11.setText(e.getResult().getObject().getNameOrPidOrId() + " / " + deKanal);
          _l1.setText(time);
          _tv1.setInput(list);
          TableColumn[] columns = _tv1.getTable().getColumns();
          for (TableColumn tc : columns)
          {
            tc.pack();
          }
        }
        else if (pidAtg.equals("atg.tlsVltLicht"))
        {
          _l21.setText(e.getResult().getObject().getNameOrPidOrId() + " / " + deKanal);
          _l2.setText(time);
          _tv2.setInput(list);
          TableColumn[] columns = _tv2.getTable().getColumns();
          for (TableColumn tc : columns)
          {
            tc.pack();
          }
        }
        else if (pidAtg.equals("atg.tlsVltStromVersorung"))
        {
          _l31.setText(e.getResult().getObject().getNameOrPidOrId() + " / " + deKanal);
          _l3.setText(time);
          _tv3.setInput(list);
          TableColumn[] columns = _tv3.getTable().getColumns();
          for (TableColumn tc : columns)
          {
            tc.pack();
          }
        }
        else if (pidAtg.equals("atg.tlsVltTemperaturberwachung"))
        {
          _l41.setText(e.getResult().getObject().getNameOrPidOrId() + " / " + deKanal);
          _l4.setText(time);
          _tv4.setInput(list);
          TableColumn[] columns = _tv4.getTable().getColumns();
          for (TableColumn tc : columns)
          {
            tc.pack();
          }
        }
        else if (pidAtg.equals("atg.tlsVltTrKontakt"))
        {
          _l51.setText(e.getResult().getObject().getNameOrPidOrId() + " / " + deKanal);
          _l5.setText(time);
          _tv5.setInput(list);
          TableColumn[] columns = _tv5.getTable().getColumns();
          for (TableColumn tc : columns)
          {
            tc.pack();
          }
        }

      }
    }
  }

  @Override
  public void dispose()
  {
    // super.dispose();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if (page != null)
    {
      page.hideView(this);
    }
  }
}
