/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.UmfeldDatenTypenPreference;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Sicht zur Anzeige von Umfelddaten.
 * 
 * @author Pittner
 * 
 */ 
public class UmfeldDatenTabelleView extends ViewPart implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.umfelddatenTabelleView"; // @jve:decl-index=0:

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private Composite _parent;

  private Composite _composite;

  private Composite _compositeTop;

  private Group _gruppeUebersicht;

  private TableViewer _tableViewerKmdPar;

  private Composite compositeBottom;

  private Button buttonAbbrechen;

  private Button buttonHilfe;

  private ClientDavInterface _connection;

  private SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private Map<String, String[]> _mapTypAtg = null;

  private Map<String, List<SystemObject>> _mapMsSensor;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".umfeldDaten");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite(_parent, SWT.NONE);
    _composite.setLayoutData(gridData);
    _composite.setLayout(gridLayout);

    createCompositeTop();
    createCompositeBottom();
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * Erzeugt gruppeUebersicht
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeTop = new Composite(_composite, SWT.NONE);
    _compositeTop.setLayout(new GridLayout());
    _compositeTop.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridDataGruppeUebersicht = new GridData();
    gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
    gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
    gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
    gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
    _gruppeUebersicht = new Group(_compositeTop, SWT.NONE);
    _gruppeUebersicht.setLayout(new GridLayout());
    _gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
    _gruppeUebersicht.setText("UmfeldDaten");
    _tableViewerKmdPar = new TableViewer(_gruppeUebersicht, SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);

    _tableViewerKmdPar.setContentProvider(new ArrayContentProvider());

    ColumnViewerToolTipSupport.enableFor(_tableViewerKmdPar, ToolTip.RECREATE);

    TableViewerColumn tvc = new TableViewerColumn(_tableViewerKmdPar, SWT.LEFT);
    tvc.setLabelProvider(new ToolTipCellLabelProvider());
    TableColumn tc = tvc.getColumn();
    tc.setText("");
    tc.setWidth(170);
    tc.setMoveable(false);

    _tableViewerKmdPar.getTable().setHeaderVisible(true);
    _tableViewerKmdPar.getTable().setLayoutData(gridDataTabelle);
    _tableViewerKmdPar.getTable().setLinesVisible(true);

    Listener paintListener = new Listener() {
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
          {
            TableItem item = (TableItem)event.item;

            String text = getText(item, event.index);
            Point size = event.gc.textExtent(text);
            event.width = size.x;
            event.height = Math.max(event.height, size.y);
            break;
          }
          case SWT.PaintItem:
          {
            TableItem item = (TableItem)event.item;
            String text = getText(item, event.index);
            Point size = event.gc.textExtent(text);
            int offset2 = event.index == 0 ? Math.max(0, (event.height - size.y) / 2) : 0;
            event.gc.drawText(text, event.x, event.y + offset2, true);
            event.gc.drawText(text, event.x, event.y + offset2, true);
            break;
          }
          case SWT.EraseItem:
          {
            event.detail &= ~SWT.FOREGROUND;
            break;
          }
        }
      }

      String getText(TableItem item, int column)
      {
        String text = "";

        if (column != 0)
        {
          UmfeldDatenTabelleDaten daten = (UmfeldDatenTabelleDaten)item.getData(new Integer(column).toString());
          if (daten != null)
          {
            text = daten.toString();
          }
        }
        else
          text = item.getText(0);

        return text;
      }
    };
    _tableViewerKmdPar.getTable().addListener(SWT.MeasureItem, paintListener);
    _tableViewerKmdPar.getTable().addListener(SWT.PaintItem, paintListener);
    _tableViewerKmdPar.getTable().addListener(SWT.EraseItem, paintListener);

    erzeugeInhaltTabelle();
  }

  private List<String> bestimmeSpaltenTabelle()
  {
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    SystemObjectType typeufdms = _connection.getDataModel().getType("typ.umfeldDatenMessStelle");
    _mapMsSensor = new HashMap<String, List<SystemObject>>();

    List<SystemObject> ufdmss = typeufdms.getElements();

    for (SystemObject so : ufdmss)
    {
      ConfigurationObject co = (ConfigurationObject)so;
      NonMutableSet set = co.getNonMutableSet("UmfeldDatenSensoren");
      List<SystemObject> sensors = set.getElements();

      boolean nsNiederSchlagIntes = false;
      boolean nsWasserFilm = false;
      boolean nsNiederSchlagArt = false;
      boolean nsFahrBahnZustand = false;
      boolean tpLuftFeuchte = false;
      boolean tpFahrBahnTemp = false;
      boolean tpLuftTemp = false;

      List<SystemObject> hauptsensors = new ArrayList<SystemObject>();
      for (SystemObject systemObject : sensors)
      {
        SystemObjectType typeSensor = systemObject.getType();
        if (typeSensor.getPid().equals("typ.ufdsNiederschlagsIntensitt"))
        {
          nsNiederSchlagIntes = true;
        }
        if (typeSensor.getPid().equals("typ.ufdsWasserFilmDicke"))
        {
          nsWasserFilm = true;
        }
        if (typeSensor.getPid().equals("typ.ufdsNiederschlagsArt"))
        {
          nsNiederSchlagArt = true;
        }
        if (typeSensor.getPid().equals("typ.ufdsFahrBahnOberFlchenZustand"))
        {
          nsFahrBahnZustand = true;
        }
        if (typeSensor.getPid().equals("typ.ufdsRelativeLuftFeuchte"))
        {
          tpLuftFeuchte = true;
        }
        if (typeSensor.getPid().equals("typ.ufdsFahrBahnOberFlchenTemperatur"))
        {
          tpFahrBahnTemp = true;
        }
        if (typeSensor.getPid().equals("typ.ufdsLuftTemperatur"))
        {
          tpLuftTemp = true;
        }

        AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.umfeldDatenSensor");
        Data cd = systemObject.getConfigurationData(atg);
        if (cd != null)
        {
          String text = cd.getItem("Typ").asTextValue().getValueText();
          if (text.equals("HauptSensor"))
          {
            hauptsensors.add(systemObject);
          }
        }
      }

      // Wg. UmfelddatenMessStellen
      List<SystemObject> elements = new ArrayList<SystemObject>();
      elements.addAll(hauptsensors);

      if (((nsNiederSchlagIntes || nsWasserFilm) && nsNiederSchlagArt && nsFahrBahnZustand)
          || (tpLuftFeuchte && tpFahrBahnTemp) || (tpLuftFeuchte && tpLuftTemp))
        elements.add(so);

      _mapMsSensor.put(so.getName(), elements);
    }

    SystemObjectType typeufdms2 = _connection.getDataModel().getType("typ.sswSbaUmfeldDatenMessStelle");
    List<SystemObject> ufdmss2 = typeufdms2.getElements();

    for (SystemObject so : ufdmss2)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaUmfeldDatenMessStelleReal");
      Data data = so.getConfigurationData(atg);
      if (data != null)
      {
        Data item = data.getItem("UmfeldDatenMessStelleRealReferenz");
        if (item != null)
        {
          ReferenceValue rv = item.asReferenceValue();
          if (rv != null)
          {
            SystemObject ufdms = rv.getSystemObject();
            if (so.getPid().equals("sba.st.sba.uz.a14.ufds.A14.195.65.DD"))
            {
              ufdms = _connection.getDataModel().getObject("st.sba.uz.a14.ufds.A14.195.65.DD");
            }
            if (ufdms != null)
            {
              ConfigurationObject co = (ConfigurationObject)ufdms;
              NonMutableSet set = co.getNonMutableSet("UmfeldDatenSensoren");
              List<SystemObject> sensors = set.getElements();

              boolean nsNiederSchlagIntes = false;
              boolean nsWasserFilm = false;
              boolean nsNiederSchlagArt = false;
              boolean nsFahrBahnZustand = false;
              boolean hk = false;

              List<SystemObject> hauptsensors = new ArrayList<SystemObject>();
              for (SystemObject systemObject : sensors)
              {
                SystemObjectType typeSensor = systemObject.getType();
                if (typeSensor.getPid().equals("typ.ufdsNiederschlagsIntensitt"))
                {
                  nsNiederSchlagIntes = true;
                }
                if (typeSensor.getPid().equals("typ.ufdsWasserFilmDicke"))
                {
                  nsWasserFilm = true;
                }
                if (typeSensor.getPid().equals("typ.ufdsNiederschlagsArt"))
                {
                  nsNiederSchlagArt = true;
                }
                if (typeSensor.getPid().equals("typ.ufdsFahrBahnOberFlchenZustand"))
                {
                  nsFahrBahnZustand = true;
                }
                if (typeSensor.getPid().equals("typ.ufdsHelligkeit"))
                {
                  hk = true;
                }

                atg = _connection.getDataModel().getAttributeGroup("atg.umfeldDatenSensor");
                Data cd = systemObject.getConfigurationData(atg);
                if (cd != null)
                {
                  String text = cd.getItem("Typ").asTextValue().getValueText();
                  if (text.equals("HauptSensor"))
                  {
                    hauptsensors.add(systemObject);
                  }
                }
              }

              // Wg. UmfelddatenMessStellen
              List<SystemObject> elements = new ArrayList<SystemObject>();
              elements.addAll(hauptsensors);

              if (((nsNiederSchlagIntes || nsWasserFilm) && nsNiederSchlagArt && nsFahrBahnZustand) || (hk))
                elements.add(so);

              _mapMsSensor.put(so.getName(), elements);
            }
          }
        }
      }
    }

    SortedMap<Integer, String> tmpMap = new TreeMap<Integer, String>();

    IPreferenceStore preferences = Activator.getDefault().getPreferenceStore();
    assert preferences != null;

    for (Map.Entry<String, List<SystemObject>> me : _mapMsSensor.entrySet())
    {
      String key = Activator.PLUGIN_ID + ":" + me.getKey();
      int pos = preferences.getInt(key);
      if (pos > 0)
      {
        tmpMap.put(new Integer(pos - 1), key);
      }
    }

    return new ArrayList<String>(tmpMap.values());

  }

  public void erzeugeInhaltTabelle()
  {
    _tableViewerKmdPar.getTable().removeAll();

    TableColumn[] columns = _tableViewerKmdPar.getTable().getColumns();
    for (int i = 0; i < columns.length; i++)
    {
      TableColumn tc = columns[i];
      if (i != 0)
      {
        tc.dispose();
      }

    }

    List<String> listSpalten = bestimmeSpaltenTabelle();

    int cnt = 0;

    for (String s : listSpalten)
    {
      TableViewerColumn tvc = new TableViewerColumn(_tableViewerKmdPar, SWT.LEFT);
      tvc.setLabelProvider(new ToolTipCellLabelProvider());
      TableColumn tc = tvc.getColumn();

      String text = s;

      if (s.contains(":"))
      {
        String[] split = s.split(":");
        if (split.length > 1)
          text = split[1];
      }
      tc.setText(text);
      tc.setToolTipText(text);
      tc.setWidth(120);
      tc.setMoveable(false);

      cnt++;
    }

    _mapTypAtg = new HashMap<String, String[]>();

    IPreferenceStore preferences = Activator.getDefault().getPreferenceStore();

    assert preferences != null;

    _mapTypAtg = UmfeldDatenTypenPreference.getMapTypAtg();

    boolean stufeHelligkeit = preferences.getBoolean(Activator.PLUGIN_ID + ":" + "stufeHelligkeit");
    String[] strs = null;
    if (stufeHelligkeit)
      strs = new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H", "atg.ufdsStufeHelligkeit" };
    else
    {
      strs = new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H" };
    }
    _mapTypAtg.put("typ.ufdsHelligkeit", strs);

    SortedMap<Integer, String[]> tmpMap = new TreeMap<Integer, String[]>();

    for (Map.Entry<String, String[]> me : _mapTypAtg.entrySet())
    {
      String key = Activator.PLUGIN_ID + ":" + me.getKey();
      String[] ss = me.getValue();
      int pos = preferences.getInt(key);
      if (pos > 0)
      {
        tmpMap.put(new Integer(pos - 1), ss);
      }
    }

    cnt = 0;
    for (String[] ss : tmpMap.values())
    {
      TableItem ti = new TableItem(_tableViewerKmdPar.getTable(), SWT.NONE);
      ti.setText(0, ss[1]);
      cnt++;
    }

    DaVKommunikationDobj.getInstanz().anmeldenAufDatenUmfeld(_mapTypAtg);
  }

  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(_composite, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        close();
      }

    });

    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }
    });
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".umfeldDaten";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * Schliet diese View
   * 
   */
  private void close()
  {
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    IWorkbench wb = PlatformUI.getWorkbench();
    if (wb != null)
    {
      IWorkbenchWindow dw = wb.getActiveWorkbenchWindow();
      if (dw != null)
      {
        IWorkbenchPage page = dw.getActivePage();
        if (page != null)
          page.hideView(this);
      }
    }
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData result = e.getResult();
    if (result != null)
    {
      verarbeiteDaVDaten(result);
    }
  }

  private void verarbeiteDaVDaten(ResultData result)
  {
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    SystemObject so = result.getObject();
    SystemObjectType type = so.getType();
    String pidTyp = type.getPid();

    Data data = result.getData();

    if (atg.getPid().equals("atg.ufdsFahrBahnFeuchte") || atg.getPid().equals("atg.ufdsFahrBahnGltte")
        || atg.getPid().equals("atg.ufdsFahrBahnOberFlchenTemperatur")
        || atg.getPid().equals("atg.ufdsFahrBahnOberFlchenZustand")
        || atg.getPid().equals("atg.ufdsGefrierTemperatur") || atg.getPid().equals("atg.ufdsHelligkeit")
        || atg.getPid().equals("atg.ufdsLuftDruck") || atg.getPid().equals("atg.ufdsLuftTemperatur")
        || atg.getPid().equals("atg.ufdsNiederschlagsArt") || atg.getPid().equals("atg.ufdsNiederschlagsIntensitt")
        || atg.getPid().equals("atg.ufdsNiederschlagsMenge") || atg.getPid().equals("atg.ufdsRelativeLuftFeuchte")
        || atg.getPid().equals("atg.ufdsRestSalz") || atg.getPid().equals("atg.ufdsSchneeHhe")
        || atg.getPid().equals("atg.ufdsSichtWeite") || atg.getPid().equals("atg.ufdsTaupunktTemperatur")
        || atg.getPid().equals("atg.ufdsTemperaturInTiefe1") || atg.getPid().equals("atg.ufdsTemperaturInTiefe2")
        || atg.getPid().equals("atg.ufdsTemperaturInTiefe3") || atg.getPid().equals("atg.ufdsWasserFilmDicke")
        || atg.getPid().equals("atg.ufdsWindGeschwindigkeitMittelWert")
        || atg.getPid().equals("atg.ufdsWindGeschwindigkeitSpitzenWert") || atg.getPid().equals("atg.ufdsWindRichtung")

        || atg.getPid().equals("atg.ufdsStufeFahrBahnFeuchte") || atg.getPid().equals("atg.ufdsStufeFahrBahnGltte")
        || atg.getPid().equals("atg.ufdsStufeFahrBahnOberFlchenTemperatur")
        || atg.getPid().equals("atg.ufdsStufeFahrBahnOberFlchenZustand")
        || atg.getPid().equals("atg.ufdsStufeGefrierTemperatur") || atg.getPid().equals("atg.ufdsStufeHelligkeit")
        || atg.getPid().equals("atg.ufdsStufeLuftDruck") || atg.getPid().equals("atg.ufdsStufeLuftTemperatur")
        || atg.getPid().equals("atg.ufdsStufeNiederschlagsIntensitt")
        || atg.getPid().equals("atg.ufdsStufeNiederschlagsMenge")
        || atg.getPid().equals("atg.ufdsStufeRelativeLuftFeuchte") || atg.getPid().equals("atg.ufdsStufeRestSalz")
        || atg.getPid().equals("atg.ufdsStufeSchneeHhe") || atg.getPid().equals("atg.ufdsStufeSichtWeite")
        || atg.getPid().equals("atg.ufdsStufeTaupunktTemperatur")
        || atg.getPid().equals("atg.ufdsStufeTemperaturInTiefe1")
        || atg.getPid().equals("atg.ufdsStufeTemperaturInTiefe2")
        || atg.getPid().equals("atg.ufdsStufeTemperaturInTiefe3")
        || atg.getPid().equals("atg.ufdsStufeWasserFilmDicke")
        || atg.getPid().equals("atg.ufdsStufeWindGeschwindigkeitMittelWert")
        || atg.getPid().equals("atg.ufdsStufeWindGeschwindigkeitSpitzenWert")
        || atg.getPid().equals("atg.ufdsStufeWindRichtung")

        || atg.getPid().equals("atg.ufdmsNsseStufe") || atg.getPid().equals("atg.ufdmsTaupunktTemperaturFahrBahn")
        || atg.getPid().equals("atg.ufdmsTaupunktTemperaturLuft")

        || atg.getPid().equals("atg.sswSbaHelligkeitErgebnisTagNacht")
        || atg.getPid().equals("atg.sswSbaUfdmsNsseStufe")

    )

    {
      if (result.getDataState().equals(DataState.NO_SOURCE) || result.getDataState().equals(DataState.NO_DATA))
      {
        for (Map.Entry<String, List<SystemObject>> me : _mapMsSensor.entrySet())
        {
          if (me.getValue().contains(so))
          {
            pidTyp = createTypExtension(atg, pidTyp);
            String[] ss = _mapTypAtg.get(pidTyp);
            setNoSourceNoDataInTable(result, me, ss);
            // break;
          }
        }
      }
      else if (data != null)
      {
        for (Map.Entry<String, List<SystemObject>> me : _mapMsSensor.entrySet())
        {
          String text = me.getKey();
          text = sbaUfdInRealUfd(so, text);
          text = extractColumnName(me, text);

          if (me.getValue().contains(so))
          {
            pidTyp = createTypExtension(atg, pidTyp);
            String[] ss = _mapTypAtg.get(pidTyp);

            String s = "";

            if (atg.getPid().equals(ss[0]))
            {
              if (atg.getPid().equals("atg.ufdmsTaupunktTemperaturFahrBahn"))
              {
                s = data.getItem(ss[1]).asTextValue().getText();
              }
              else if (atg.getPid().equals("atg.ufdmsTaupunktTemperaturLuft"))
              {
                s = data.getItem(ss[1]).asTextValue().getText();
              }
              else if (atg.getPid().equals("atg.sswSbaHelligkeitErgebnisTagNacht"))
              {
                Array array = data.getItem("ErgebnisTagNacht").asArray();
                for (int i = 0; i < array.getLength(); i++)
                {
                  Data item = array.getItem(i);
                  SystemObject sensor = item.getItem("Sensor").asReferenceValue().getSystemObject();
                  String s1 = item.getItem("Status").asTextValue().getValueText();
                  List<SystemObject> value = me.getValue();
                  if (value.contains(sensor))
                  {
                    setDataInTable(result, me, text, ss, s1);
                    break;
                  }
                }
                continue;
              }
              else
                s = data.getItem(ss[1]).getItem("Wert").asTextValue().getText();
            }
            else if (atg.getPid().equals(ss[3]))
            {
              if (atg.getPid().equals("atg.ufdmsNsseStufe"))
              {
                s = data.getItem("NsseStufe").asTextValue().getText();
              }
              else if (atg.getPid().equals("atg.sswSbaUfdmsNsseStufe"))
              {
                s = data.getItem("NsseStufe").asTextValue().getText();
              }
              else
                s = data.getItem("Stufe").asTextValue().getText();
            }
            else
              System.out.println("Fehler ATG0: " + atg.getPid());

            if (s.length() > 0)
              setDataInTable(result, me, text, ss, s);
            // break;
          }
        }
      }    
    }
  }

  /**
   * Fllt die Tabelle mit keine Quelle und/oder keine Daten
   * 
   * @param result
   *          der DaV-Datensatz
   * @param me
   *          die Zuordnung MessStelle(Bezeichnung)-Sensoren
   * @param ss
   *          die Zuordnung ObjektTyp-Attributgruppe/Bezeichnung
   */
  private void setNoSourceNoDataInTable(ResultData result, Map.Entry<String, List<SystemObject>> me, String[] ss)
  {
    SystemObject so = result.getObject();
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    TableItem[] items = _tableViewerKmdPar.getTable().getItems();
    for (TableItem ti : items)
    {
      String first = ti.getText(0);
      if (first.equals(ss[1]))
      {
        TableColumn[] columns = _tableViewerKmdPar.getTable().getColumns();
        for (int i = 0; i < columns.length; i++)
        {
          TableColumn tc = columns[i];
          String column = tc.getText();

          String text = me.getKey();

          text = sbaUfdInRealUfd(so, text);

          text = extractColumnName(me, text);

          if (column.equals(text))
          {
            UmfeldDatenTabelleDaten daten = (UmfeldDatenTabelleDaten)ti.getData(new Integer(i).toString());

            if (daten == null)
              daten = new UmfeldDatenTabelleDaten();
            daten.setIndex(i);

            if (ss[0].length() == 0)
            {
              daten.setKeinWertVorhanden(true);
            }
            if (ss.length > 3)
            {
              daten.setStufeVorhanden(true);
            }

            if (!daten.isKeinWertVorhanden())
              daten.setWertZeit(result.getDataTime());

            if (daten.isStufeVorhanden())
              daten.setStufeZeit(result.getDataTime());

            if (result.getDataState().equals(DataState.NO_SOURCE))
            {
              if (!daten.isKeinWertVorhanden())
              {
                if (atg.getPid().equals(ss[0]))
                {
                  daten.setWert(UmfeldDatenTabelleDaten.KEINE_QUELLE);
                }
              }
              if (daten.isStufeVorhanden())
              {
                if (atg.getPid().equals(ss[3]))
                  daten.setStufe(UmfeldDatenTabelleDaten.KEINE_QUELLE);
              }
            }
            else if (result.getDataState().equals(DataState.NO_DATA))
            {
              if (!daten.isKeinWertVorhanden())
              {
                if (atg.getPid().equals(ss[0]))
                {
                  daten.setWert(UmfeldDatenTabelleDaten.KEINE_DATEN);
                }
              }
              if (daten.isStufeVorhanden())
              {
                if (atg.getPid().equals(ss[3]))
                  daten.setStufe(UmfeldDatenTabelleDaten.KEINE_DATEN);
              }
            }
            else
            {
              System.out.println("Fehler DataState: " + result.getDataState());
            }

            if (!daten.hasVollstaendigeDaten())
            {
              if (!daten.hasDaten())
              {
                ti.setBackground(i, ColorConstants.red);
              }
              else
              {
                ti.setBackground(i, ColorConstants.orange);
              }
            }
            else
            {
              ti.setBackground(i, ColorConstants.white);
            }

            ti.setText(i, daten.toString());
            ti.setData(new Integer(i).toString(), daten);

          }
        }
      }
    }
  }

  /**
   * Fllt die Tabelle mit konkreten Daten
   * 
   * @param result
   *          der DaV-Datensatz
   * @param me
   *          die Zuordnung MessStelle(Bezeichnung)-Sensoren
   * @param text
   *          der SpaltenText(Bezeichnung Messstelle)
   * @param ss
   *          die Zuordnung ObjektTyp-Attributgruppe/Bezeichnung
   * @param s
   *          die Daten als String
   */
  private void setDataInTable(ResultData result, Map.Entry<String, List<SystemObject>> me, String text, String[] ss,
      String s)
  {
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    TableItem[] items = _tableViewerKmdPar.getTable().getItems();
    for (TableItem ti : items)
    {
      String first = ti.getText(0);

      if (first.equals(ss[1]))
      {
        TableColumn[] columns = _tableViewerKmdPar.getTable().getColumns();
        for (int i = 0; i < columns.length; i++)
        {
          TableColumn tc = columns[i];
          String column = tc.getText();

          if (column.equals(text))
          {
            UmfeldDatenTabelleDaten daten = (UmfeldDatenTabelleDaten)ti.getData(new Integer(i).toString());

            if (daten == null)
              daten = new UmfeldDatenTabelleDaten();

            if (ss[0].length() == 0)
            {
              daten.setKeinWertVorhanden(true);
            }
            if (ss.length > 3)
            {
              daten.setStufeVorhanden(true);
            }

            if (!daten.isKeinWertVorhanden())
            {
              if (atg.getPid().equals(ss[0]))
              {
                daten.setWert(s);
                daten.setWertZeit(result.getDataTime());
              }
            }
            if (daten.isStufeVorhanden())
            {
              if (ss.length > 3)
              {
                if (atg.getPid().equals(ss[3]))
                {

                  daten.setStufe(s);
                  daten.setStufeZeit(result.getDataTime());
                }
              }
              else
              {
                System.out.println("else ss[3].length() > 3: " + me.getKey());
              }
            }

            if (daten != null)
            {
              if (!daten.hasVollstaendigeDaten())
              {
                if (!daten.hasDaten())
                {
                  ti.setBackground(i, ColorConstants.red);
                }
                else
                {
                  ti.setBackground(i, ColorConstants.orange);
                }
              }
              else
              {
                ti.setBackground(i, ColorConstants.white);
              }
              ti.setText(i, daten.toString());
              ti.setData(new Integer(i).toString(), daten);
            }
          }
        }
      }
    }
  }

  private String extractColumnName(Map.Entry<String, List<SystemObject>> me, String text)
  {
    if (me.getKey().contains(":"))
    {
      String[] split = me.getKey().split(":");
      if (split.length > 1)
      {
        text = split[1];
      }
    }
    return text;
  }

  private String createTypExtension(AttributeGroup atg, String pidTyp)
  {
    if (pidTyp.equals("typ.umfeldDatenMessStelle"))
    {

      String pidAtg = atg.getPid();
      if (pidAtg.equals("atg.ufdmsNsseStufe"))
      {
        pidTyp += "1";
      }
      else if (pidAtg.equals("atg.ufdmsTaupunktTemperaturFahrBahn"))
      {
        pidTyp += "2";
      }
      else if (pidAtg.equals("atg.ufdmsTaupunktTemperaturLuft"))
      {
        pidTyp += "3";
      }
    }
    else if (pidTyp.equals("typ.sswSbaUmfeldDatenMessStelle"))
    {

      String pidAtg = atg.getPid();

      if (pidAtg.equals("atg.sswSbaHelligkeitErgebnisTagNacht"))
      {
        pidTyp += "1";
      }
      else if (pidAtg.equals("atg.sswSbaUfdmsNsseStufe"))
      {
        pidTyp += "2";
      }
    }
    return pidTyp;
  }

  private String sbaUfdInRealUfd(SystemObject so, String text)
  {
    if (so.getType().getPid().equals("typ.sswSbaUmfeldDatenMessStelle"))
    {
      AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.sswSbaUmfeldDatenMessStelleReal");
      Data cdata = so.getConfigurationData(atg2);
      if (cdata != null)
      {
        Data item = cdata.getItem("UmfeldDatenMessStelleRealReferenz");
        if (item != null)
        {
          ReferenceValue rv = item.asReferenceValue();

          if (rv != null)
          {
            SystemObject ufdms = rv.getSystemObject();
            if (so.getPid().equals("sba.st.sba.uz.a14.ufds.A14.195.65.DD"))
            {
              ufdms = _connection.getDataModel().getObject("st.sba.uz.a14.ufds.A14.195.65.DD");
            }
            if (ufdms != null)
            {
              text = ufdms.getName();
            }
          }
        }
      }
    }
    return text;
  }

  @Override
  public void dispose()
  {
    close();
  }

  public class ToolTipCellLabelProvider extends CellLabelProvider
  {
    public String getToolTipText(Object element)
    {
      Table table = _tableViewerKmdPar.getTable();
      Point location = PlatformUI.getWorkbench().getDisplay().getCursorLocation();

      String s = "";

      location = _tableViewerKmdPar.getControl().toControl(location);
      TableItem item = table.getItem(location);

      if (item != null)
      {
        String text1 = "Typ       : " + item.getText(0);
        String text2 = "MessStelle: ";
        String text3 = "Wert      : ";
        String text4 = "Stufe     : ";

        // boolean gefunden = false;
        for (int i = 0; i < table.getColumnCount(); i++)
        {
          Rectangle rect = item.getBounds(i);
          if (rect.contains(location))
          {
            // In der 1. Spalte kein Tooltip!!!
            if (i == 0)
              return null;

            // Zellen ohne Inhalt kein Tooltip!!!
            String text = item.getText(i);
            if (text == null)
              return null;
            else if (text.length() < 1)
              return null;

            text2 += table.getColumn(i).getText();

            UmfeldDatenTabelleDaten daten = (UmfeldDatenTabelleDaten)item.getData(new Integer(i).toString());

            if (daten != null)
            {
              if (!daten.isKeinWertVorhanden())
              {
                text3 += daten.getWert();
                if (!daten.getWert().equals(UmfeldDatenTabelleDaten.UNDEFINERT))
                {
                  String z1 = _sdf.format(new Date(daten.getWertZeit()));
                  text3 += " (" + z1 + ")";
                }
              }
              else
                text3 = null;

              if (daten.isStufeVorhanden())
              {
                text4 += daten.getStufe();
                if (!daten.getStufe().equals(UmfeldDatenTabelleDaten.UNDEFINERT))
                {
                  String z2 = _sdf.format(new Date(daten.getStufeZeit()));
                  text4 += " (" + z2 + ")";
                }
              }
              else
              {
                text4 = null;
              }
            }
            break;
          }
        }

        if (text1 != null)
          s += text1 + "\n";
        if (text2 != null)
          s += text2 + "\n";
        if (text3 != null)
          s += text3;
        if (text4 != null)
          if (text3 != null)
            s += "\n" + text4;
          else
            s += text4;
      }
      return s;
    }

    @Override
    public boolean useNativeToolTip(Object object)
    {
      // TODO Auto-generated method stub
      return false;
    }

    public void update(ViewerCell cell)
    {
      // cell.setText(cell.getElement().toString());
    }
  }

  public void setPartName(String partName)
  {
    super.setPartName(partName);
  }
}
