package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.LinkedList;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.model.KilometrierungZuordnung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;

/**
 * Dialog zum Editieren der Kilometrierung.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class ViewZuordnungKilometrierung extends ViewPart
{
  public static final String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.ViewZuordnungKilometrierung";

  private static DecimalFormat _decimalFormat;

  private Composite _top;

  private Composite _compLinks;

  private Composite _compRechts;

  private LinkedList<KilometrierungZuordnung> _listeZuordnungen = new LinkedList<KilometrierungZuordnung>();

  private TableViewer _tv;

  private Button _bRemove;

  private Button _bMoveUp;

  private Button _bMoveDown;

  private Button _bSave;

  /**
   * Der {@link CommandStack} mit dem das Modell bearbeitet wird.
   */
  private CommandStack _commandStack = null;

  /**
   * Das zu bearbeitende Modell.
   */
  private DoModel _model = null;

  public static final String COL_Y_VON = "Y von";

  public static final String COL_KM_VON = "BKM von";

  private static final String[] PROPS_COL = new String[] { COL_Y_VON, COL_KM_VON };

  @Override
  public void createPartControl(Composite parent)
  {
    _top = new Composite(parent, SWT.NONE);

    _top.setLayout(new GridLayout(2, false));

    GridData gdLinks = new GridData();
    gdLinks.grabExcessHorizontalSpace = true;
    gdLinks.grabExcessVerticalSpace = true;
    gdLinks.horizontalAlignment = GridData.FILL;
    gdLinks.verticalAlignment = GridData.FILL;

    getCompLinks(_top).setLayoutData(gdLinks);

    GridData gdRechts = new GridData();
    gdRechts.grabExcessHorizontalSpace = false;
    gdRechts.grabExcessVerticalSpace = false;
    gdRechts.horizontalAlignment = GridData.BEGINNING;
    gdRechts.verticalAlignment = GridData.BEGINNING;

    getCompRechts(_top).setLayoutData(gdRechts);

    validateButtons();
  }

  private Composite getCompLinks(Composite parent)
  {
    if (_compLinks == null)
    {
      _compLinks = new Composite(parent, SWT.BORDER);

      _compLinks.setLayout(new FillLayout());

      _tv = new TableViewer(_compLinks, SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
      _tv.getTable().setLinesVisible(true);
      _tv.getTable().setHeaderVisible(true);

      int columnWidth = 70;

      TableViewerColumn yVon = new TableViewerColumn(_tv, SWT.NONE);
      yVon.getColumn().setWidth(columnWidth);
      yVon.getColumn().setText(COL_Y_VON);

      TableViewerColumn kmVon = new TableViewerColumn(_tv, SWT.NONE);
      kmVon.getColumn().setWidth(columnWidth);
      kmVon.getColumn().setText(COL_KM_VON);

      _tv.setColumnProperties(PROPS_COL);
      _tv.setContentProvider(new CP());
      _tv.setLabelProvider(new LP());
      _tv.setCellModifier(new Modifier(_tv));

      TextCellEditor textCellEditor = new TextCellEditor(_tv.getTable());

      textCellEditor.setValidator(new ICellEditorValidator() {
        @Override
        public String isValid(Object value)
        {
          String re = null;

          if (value instanceof String)
          {
            try
            {
              Double.parseDouble(((String)value).replaceAll(",", "."));
            }
            catch (NumberFormatException e)
            {
              re = "Nur Double ist erlaubt!";
            }
          }
          else
          {
            System.out.println("VALUE=>" + value);
            if (value != null)
            {
              System.out.println("\tValue Class=" + value.getClass().getName());
            }
          }

          return re;
        }

      });

      CellEditor[] editors = new CellEditor[] { new DoubleNumberCellEditor(_tv.getTable()),
          new DoubleNumberCellEditor(_tv.getTable()), new IntegerNumberCellEditor(_tv.getTable()),
          new IntegerNumberCellEditor(_tv.getTable()) };
      _tv.setCellEditors(editors);

      _tv.addSelectionChangedListener(new ISelectionChangedListener() {

        @Override
        public void selectionChanged(SelectionChangedEvent event)
        {
          validateButtons();
        }
      });

    }

    return _compLinks;
  }

  private Composite getCompRechts(Composite parent)
  {
    if (_compRechts == null)
    {
      _compRechts = new Composite(parent, SWT.NONE);

      Button bAdd = new Button(_compRechts, SWT.PUSH);
      bAdd.setText("&Hinzufgen");
      bAdd.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionAdd();
        }
      });

      _bRemove = new Button(_compRechts, SWT.PUSH);
      _bRemove.setText("&Entfernen");
      _bRemove.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionRemove();
        }
      });
      _bRemove.setEnabled(false);

      _bMoveUp = new Button(_compRechts, SWT.PUSH);
      _bMoveUp.setText("Nach &Oben verschieben");
      _bMoveUp.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionMoveUp();
        }
      });
      _bMoveUp.setEnabled(false);

      _bMoveDown = new Button(_compRechts, SWT.PUSH);
      _bMoveDown.setText("Nach &Unten verschieben");
      _bMoveDown.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionMoveDown();
        }
      });
      _bMoveDown.setEnabled(false);

      _bSave = new Button(_compRechts, SWT.PUSH);
      _bSave.setText("&bernehmen");
      _bSave.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionSave();
        }
      });

      Button bAbbrechen = new Button(_compRechts, SWT.PUSH);
      bAbbrechen.setText("&Abbrechen");
      bAbbrechen.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionCancel();
        }
      });

      Button bImport = new Button(_compRechts, SWT.PUSH);
      bImport.setText("&Import");
      bImport.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionImport();
        }
      });
      Button bExport = new Button(_compRechts, SWT.PUSH);
      bExport.setText("&Export");
      bExport.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          actionExport();
        }

      });

      RowLayout rl = new RowLayout(SWT.VERTICAL);
      rl.pack = false;
      _compRechts.setLayout(rl);
    }

    return _compRechts;
  }

  protected void actionImport()
  {
    // TODO Auto-generated method stub

    FileDialog fsd = new FileDialog(_top.getShell(), SWT.OPEN);
    fsd.setFilterExtensions(new String[] { "*.txt" });
    fsd.setText("Select File to import");
    String str = fsd.open();
    System.out.println("Import: " + str);

    // String fileName = fileDialog.open();
    if (str != null)
    {
      try
      {
        BufferedReader br = new BufferedReader(new FileReader(str));
        try
        {
          StringBuilder sb = new StringBuilder();
          String line = br.readLine();

          while (line != null)
          {
            sb.append(line);
//            sb.append("\n");
            line = br.readLine();
          }
          
          _listeZuordnungen = KilometrierungZuordnung.deserFromMultipleString(sb.toString());
          
//          for (KilometrierungZuordnung kzu : _listeZuordnungen)
//          {
//            System.out.println("kzu: " + kzu);
//          }
          
          _tv.setInput(_listeZuordnungen);
//          _tv.refresh();
        }
        finally
        {
          br.close();
        }
      }
      catch (FileNotFoundException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      catch (IOException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

    }
  }

  protected void actionExport()
  {
    // TODO Auto-generated method stub

    FileDialog fsd = new FileDialog(_top.getShell(), SWT.SAVE);
    fsd.setFilterExtensions(new String[] { "*.txt" });
    String str = fsd.open();

    File file = null;

    if (str != null)
    {
      FileOutputStream stream = null;
      PrintStream out = null;
      try
      {
        file = new File(str); // Create new File

        boolean success;
        try
        {
          success = file.createNewFile();
          if (success)
          {
            stream = new FileOutputStream(file);
            out = new PrintStream(stream);
            out.print(((PickerParent)_model).getKilometrierungZuordnung()); // This will overwrite existing contents
          }
        }
        catch (Exception e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
      catch (Exception ex)
      {
        // do something
      }
      finally
      {
        try
        {
          if (stream != null)
            stream.close();
          if (out != null)
            out.close();
        }
        catch (Exception ex)
        {
          // do something
        }
      }
    }

  }

  private void actionAdd()
  {
    KilometrierungZuordnung neu = new KilometrierungZuordnung(0.0D, 0.0D, 0, 0);
    _listeZuordnungen.add(neu);
    _tv.add(neu);
  }

  private void actionRemove()
  {
    ISelection selection = _tv.getSelection();
    if ((selection != null) && (selection instanceof StructuredSelection))
    {
      StructuredSelection ss = (StructuredSelection)selection;
      Object firstElement = ss.getFirstElement();
      if ((firstElement != null) && (firstElement instanceof KilometrierungZuordnung))
      {
        KilometrierungZuordnung removeZuordnung = (KilometrierungZuordnung)firstElement;
        _listeZuordnungen.remove(removeZuordnung);
        _tv.remove(removeZuordnung);
      }
    }
  }

  private void actionMoveUp()
  {
    ISelection selection = _tv.getSelection();
    if ((selection != null) && (selection instanceof StructuredSelection))
    {
      StructuredSelection ss = (StructuredSelection)selection;
      Object firstElement = ss.getFirstElement();
      if ((firstElement != null) && (firstElement instanceof KilometrierungZuordnung))
      {
        KilometrierungZuordnung moveUpZuordnung = (KilometrierungZuordnung)firstElement;

        int idx = _listeZuordnungen.indexOf(moveUpZuordnung);
        if (idx > 0)
        {
          moveUpZuordnung.setKmVon(123.456D);
          _listeZuordnungen.remove(moveUpZuordnung);
          _listeZuordnungen.add(idx - 1, moveUpZuordnung);

          _tv.setInput(_listeZuordnungen);

          _tv.setSelection(new StructuredSelection(moveUpZuordnung), true);
        }
      }
    }
  }

  private void actionMoveDown()
  {
    ISelection selection = _tv.getSelection();
    if ((selection != null) && (selection instanceof StructuredSelection))
    {
      StructuredSelection ss = (StructuredSelection)selection;
      Object firstElement = ss.getFirstElement();
      if ((firstElement != null) && (firstElement instanceof KilometrierungZuordnung))
      {
        KilometrierungZuordnung element = (KilometrierungZuordnung)firstElement;
        element.setYBis(123);

        int idx = _listeZuordnungen.indexOf(element);

        if ((idx + 1) < _listeZuordnungen.size())
        {
          KilometrierungZuordnung after = _listeZuordnungen.get(idx + 1);
          _listeZuordnungen.remove(element);
          int indexNew = _listeZuordnungen.indexOf(after) + 1;
          if (indexNew < _listeZuordnungen.size())
          {
            _listeZuordnungen.add(indexNew, element);
          }
          else
          {
            _listeZuordnungen.add(element);
          }
          _tv.setInput(_listeZuordnungen);
        }
      }
    }
  }

  /**
   * Diese Funktion stellt sicher dass der BIS-Wert (KM und Y-Pixel) eines Elmentes dem VON-Wert (KM und Y-Pixel) des
   * Nachfolgelementes entspricht!
   */
  private void processListEndActElementSetToBeginNxtElement()
  {
    if ((_listeZuordnungen != null) && (_listeZuordnungen.size() > 1))
    {
      for (int i = 0; i < _listeZuordnungen.size(); i++)
      {
        KilometrierungZuordnung akt = _listeZuordnungen.get(i);
        if ((i + 1) < _listeZuordnungen.size())
        {
          KilometrierungZuordnung nxt = _listeZuordnungen.get(i + 1);

          if (nxt.getyVon() != null)
          {
            akt.setYBis(nxt.getyVon().intValue());
          }

          if (nxt.getKmVon() != null)
          {
            akt.setKmBis(nxt.getKmVon().doubleValue());
          }

        }
      }
    }
  }

  private void actionSave()
  {
    if ((_commandStack != null) && (_model != null))
    {
      processListEndActElementSetToBeginNxtElement();
      String serKmZu = KilometrierungZuordnung.serToString(_listeZuordnungen);

      if (serKmZu != null)
      {
        EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__KILOMETRIERUNG_ZUORDNUNG;
        if (_model instanceof PickerParent)
        {
          feature = AnlagendarstellungPackage.Literals.PICKER_PARENT__KILOMETRIERUNG_ZUORDNUNG;
        }

        System.out.println("ViewZuordnungKilometrierung.actionSave(): " + _model.getClass().getSimpleName() + " -> "
            + feature.getName());

        final SetCommand cmd = new SetCommand(_model, feature, serKmZu);
        _commandStack.execute(cmd);
        System.out.println("Serialisierte Daten ins Modell geschriebwen: " + serKmZu);
      }
      else
      {
        System.err.println("Die Daten konnten nicht serialisiert werden!");
      }
    }
    else
    {
      System.err.println("CommandStack oder Modell ist null!");
    }

    actionCancel();
  }

  private void actionCancel()
  {
    ViewZuordnungKilometrierung.this.getViewSite().getPage().hideView(ViewZuordnungKilometrierung.this);
    ViewZuordnungKilometrierung.this.dispose();
  }

  /**
   * De/Aktiviert in Abhngigkeit von der Selektion in der Tabelle die Buttons.
   */
  private void validateButtons()
  {
    ISelection selection = _tv.getSelection();
    if (selection instanceof StructuredSelection)
    {
      StructuredSelection ss = (StructuredSelection)selection;
      if (!ss.isEmpty())
      {
        _bRemove.setEnabled(true);
        _bMoveUp.setEnabled(true);
        _bMoveDown.setEnabled(true);
      }
      else
      {
        _bRemove.setEnabled(false);
        _bMoveUp.setEnabled(false);
        _bMoveDown.setEnabled(false);
      }
    }

    if ((_commandStack != null) && (_model != null))
    {
      _bSave.setEnabled(true);
    }
    else
    {
      _bSave.setEnabled(false);
    }

  }

  @Override
  public void setFocus()
  {
    _top.setFocus();
  }

  private class CP implements IStructuredContentProvider
  {
    @SuppressWarnings("unchecked")
    @Override
    public Object[] getElements(Object inputElement)
    {
      if ((inputElement != null) && (inputElement instanceof LinkedList))
      {
        Object ele = ((LinkedList)inputElement).getFirst();
        if (ele instanceof KilometrierungZuordnung)
        {
          LinkedList<KilometrierungZuordnung> liste = (LinkedList<KilometrierungZuordnung>)inputElement;
          return liste.toArray(new KilometrierungZuordnung[liste.size()]);
        }
      }

      return null;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  private class LP implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String re = "";

      if ((element != null) && (element instanceof KilometrierungZuordnung))
      {
        KilometrierungZuordnung zu = (KilometrierungZuordnung)element;

        switch (columnIndex)
        {

          case 0:
            Integer getyVon = zu.getyVon();
            if (getyVon != null)
            {
              re = getyVon.toString();
            }
            break;
          case 1:
            Double kmVon = zu.getKmVon();
            if (kmVon != null)
            {
              re = getDecimalFormat().format(kmVon);
            }
            break;

          default:
            break;
        }

      }

      return re;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {

    }

  }

  private class EditKm extends EditingSupport
  {
    private TableViewer _viewer;

    private CellEditor _editor;

    private int _column = -1;

    public EditKm(TableViewer viewer, int column)
    {
      super(viewer);

      this._viewer = viewer;
      this._editor = new TextCellEditor();
      this._column = column;
    }

    @Override
    protected boolean canEdit(Object element)
    {
      if (element != null)
        System.out.println("ViewZurodnungKilometrierung.Edit.canEdit() ELEMENT=>" + element.getClass().getName());
      return true;
    }

    @Override
    protected CellEditor getCellEditor(Object element)
    {
      return _editor;
    }

    @Override
    protected Object getValue(Object element)
    {
      Object re = null;

      if ((element != null) && (element instanceof KilometrierungZuordnung))
      {
        KilometrierungZuordnung zu = (KilometrierungZuordnung)element;

        if (_column >= 0)
        {
          switch (_column)
          {
            case 0:
              re = zu.getKmVon().toString();
              break;
            case 1:
              re = zu.getKmBis().toString();
              break;
          }
        }
      }

      return re;
    }

    @Override
    protected void setValue(Object element, Object value)
    {
      System.out.println("ViewZurodnungKilometrierung.Edit.setValue()");
      if (element != null)
      {
        System.out.println("\tElement=>" + element.getClass().getName());
      }

      if (value != null)
      {
        System.out.println("\tValue=>" + value.getClass().getName());
      }
    }

  }

  private class Modifier implements ICellModifier
  {
    private Viewer _viewer = null;

    public Modifier(Viewer viewer)
    {
      this._viewer = viewer;
    }

    @Override
    public boolean canModify(Object element, String property)
    {
      System.out.println("...canModify() PROP=" + property + " ELEMENT=" + element);

      boolean canModify = false;

      if ((element != null) && (element instanceof KilometrierungZuordnung) && (property != null))
      {
        KilometrierungZuordnung zu = (KilometrierungZuordnung)element;

        if (property.equals(COL_KM_VON))
        {
          canModify = true;
        }
        else if (property.equals(COL_Y_VON))
        {
          canModify = true;
        }

      }

      System.out.println("\t-> " + canModify);

      return canModify;
    }

    @Override
    public Object getValue(Object element, String property)
    {
      Object re = null;

      if ((element != null) && (element instanceof KilometrierungZuordnung) && (property != null))
      {
        KilometrierungZuordnung zu = (KilometrierungZuordnung)element;

        if (property.equals(COL_KM_VON))
        {
          re = zu.getKmVon().toString();
        }
        else if (property.equals(COL_Y_VON))
        {
          re = zu.getyVon().toString();
        }
      }

      return re;
    }

    @Override
    public void modify(Object element, String property, Object value)
    {
      System.out.println("=>> Modify Prop=" + property + " Value=" + value + " Element=" + element);

      if ((element != null) && (element instanceof TableItem))
      {
        TableItem ti = (TableItem)element;
        Object data = ti.getData();
        if ((data != null) && (data instanceof KilometrierungZuordnung))
        {
          KilometrierungZuordnung zu = (KilometrierungZuordnung)data;

          if ((property != null) && (value != null))
          {
            System.out.println("Value TYPE =>" + value.getClass().getName());

            if (value instanceof String)
            {
              String strVal = (String)value;

              try
              {
                if (property.equals(COL_KM_VON))
                {
                  double kmVon = Double.parseDouble(strVal.replaceAll(",", "."));
                  zu.setKmVon(kmVon);
                }
                else if (property.equals(COL_Y_VON))
                {
                  int yVon = Integer.parseInt(strVal);
                  zu.setYVon(yVon);
                }

                // _viewer.refresh();
                aktTableViewer();
              }
              catch (NumberFormatException nfe)
              {
                nfe.printStackTrace();
              }
            }
          }
        }
      }
    }
  }

  public static DecimalFormat getDecimalFormat()
  {
    if (_decimalFormat == null)
    {
      DecimalFormatSymbols dfs = DecimalFormatSymbols.getInstance();
      dfs.setDecimalSeparator(',');
      _decimalFormat = new DecimalFormat("0.000", dfs);
    }

    return _decimalFormat;
  }

  private class DoubleNumberCellEditor extends TextCellEditor
  {

    public DoubleNumberCellEditor()
    {
      super();
      installValidator();
    }

    public DoubleNumberCellEditor(Composite parent, int style)
    {
      super(parent, style);
      installValidator();
    }

    public DoubleNumberCellEditor(Composite parent)
    {
      super(parent);
      installValidator();
    }

    private void installValidator()
    {
      this.setValidator(new ICellEditorValidator() {
        @Override
        public String isValid(Object value)
        {
          String re = null;

          if (value instanceof String)
          {
            try
            {
              Double.parseDouble(((String)value).replaceAll(",", "."));
            }
            catch (NumberFormatException e)
            {
              re = "Nur Double ist erlaubt!";
            }
          }
          else
          {
            System.out.println("VALUE=>" + value);
            if (value != null)
            {
              System.out.println("\tValue Class=" + value.getClass().getName());
            }
          }

          return re;
        }

      });
    }

    @Override
    protected Control createControl(Composite parent)
    {
      Control ctrl = super.createControl(parent);

      if ((ctrl != null) && (ctrl instanceof Text))
      {
        Text txt = (Text)ctrl;

        txt.addListener(SWT.Verify, new Listener() {
          public void handleEvent(Event e)
          {
            String string = e.text;
            char[] chars = new char[string.length()];
            string.getChars(0, chars.length, chars, 0);

            for (int i = 0; i < chars.length; i++)
            {
              if (!(('0' <= chars[i] && chars[i] <= '9') || (chars[i] == ',')))
              {
                e.doit = false;
                return;
              }
            }
          }
        });

      }

      return ctrl;
    }
  }

  private class IntegerNumberCellEditor extends TextCellEditor
  {

    public IntegerNumberCellEditor()
    {
      super();
      installValidator();
    }

    public IntegerNumberCellEditor(Composite parent, int style)
    {
      super(parent, style);
      installValidator();
    }

    public IntegerNumberCellEditor(Composite parent)
    {
      super(parent);
      installValidator();
    }

    private void installValidator()
    {
      this.setValidator(new ICellEditorValidator() {
        @Override
        public String isValid(Object value)
        {
          String re = null;

          if (value instanceof String)
          {
            try
            {
              Integer.parseInt((String)value);
            }
            catch (NumberFormatException e)
            {
              re = "Nur Integer ist erlaubt!";
            }
          }
          else
          {
            System.out.println("VALUE=>" + value);
            if (value != null)
            {
              System.out.println("\tValue Class=" + value.getClass().getName());
            }
          }

          return re;
        }

      });
    }

    @Override
    protected Control createControl(Composite parent)
    {
      Control ctrl = super.createControl(parent);

      if ((ctrl != null) && (ctrl instanceof Text))
      {
        Text txt = (Text)ctrl;

        txt.addListener(SWT.Verify, new Listener() {
          public void handleEvent(Event e)
          {
            String string = e.text;
            char[] chars = new char[string.length()];
            string.getChars(0, chars.length, chars, 0);

            for (int i = 0; i < chars.length; i++)
            {
              if (!('0' <= chars[i] && chars[i] <= '9'))
              {
                e.doit = false;
                return;
              }
            }
          }
        });

      }

      return ctrl;
    }
  }

  /**
   * Aktualisiert den TableViewer an Hand der Liste.
   */
  private void aktTableViewer()
  {
    if ((_tv != null) && (_listeZuordnungen != null))
    {
      /*
       * Selektiertes Element zur Wiederherstellung merken.
       */
      Object selElement = null;
      if ((_tv.getSelection() != null) && (_tv.getSelection() instanceof StructuredSelection))
      {
        StructuredSelection ss = (StructuredSelection)_tv.getSelection();
        selElement = ss.getFirstElement();
      }

      _tv.setInput(_listeZuordnungen);

      /*
       * Gemerkte Selektion wiederherstellen
       */
      if (selElement != null)
      {
        StructuredSelection ss = new StructuredSelection(selElement);
        _tv.setSelection(ss);
      }
    }
  }

  public void setCommandStackAndModel(CommandStack commandStack, PickerVertical model)
  {
    this._commandStack = commandStack;
    this._model = model;

    _listeZuordnungen.clear();
    String strKilometrierungZuordnung = model.getKilometrierungZuordnung();
    if (strKilometrierungZuordnung != null)
    {
      LinkedList<KilometrierungZuordnung> liste = KilometrierungZuordnung
          .deserFromMultipleString(strKilometrierungZuordnung);
      if (liste != null)
      {
        _listeZuordnungen.addAll(liste);
        aktTableViewer();
      }
    }

    validateButtons();
  }

  public void setCommandStackAndModel2(CommandStack commandStack, PickerParent model)
  {
    this._commandStack = commandStack;
    this._model = model;

    System.out.println("ViewZuordnungKilometrierung.setCommandStackAndModel2()" + _model.getClass().getSimpleName());

    _listeZuordnungen.clear();
    String strKilometrierungZuordnung = model.getKilometrierungZuordnung();
    if (strKilometrierungZuordnung != null)
    {
      LinkedList<KilometrierungZuordnung> liste = KilometrierungZuordnung
          .deserFromMultipleString(strKilometrierungZuordnung);
      if (liste != null)
      {
        _listeZuordnungen.addAll(liste);
        aktTableViewer();
      }
    }

    validateButtons();
  }
}
