/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.PrintAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModel;
import de.bsvrz.buv.plugin.anlagendarstellung.TabelleMqModel;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.DefaultEditDomainViewPart;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegende;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorAktion;
import de.bsvrz.buv.rw.rw.menu.deskriptoren.DeskriptorElement;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCacheExtended;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnittCache;

/**
 * View-Klasse, welche die Sicht der Zustandsdarstellung (Soll-Darstellung) darstellt
 * 
 * @author Pittner
 * 
 */
public class ZustandsDarstellungSollView extends GraphicalEditorToView implements IDaVDataUpdateListener
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.zustandsdarstellungSollView";

  private Composite _parent;

//  private GraphicalViewer graphicalViewer;

  private ActionRegistry _registry;

  private ISelectionChangedListener _selListener;

  private MenuManager _kontextMenuMgr;

  private ActionRegistry _actionRegistry;

//  private IMemento _memento = null;

  private ClientDavInterface _connection;


  public ZustandsDarstellungSollView()
  {
    DefaultEditDomainViewPart defaultEditDomain = new DefaultEditDomainViewPart(this);
    setEditDomain(defaultEditDomain);
    _registry = new ActionRegistry();
    IAction action;
    action = new PrintAction(this); // this is your IEditorPart-derived
    _registry.registerAction(action);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".automatik");
    // Ansicht in einen extra Viewer packen.
    graphicalViewer = new ScrollingGraphicalViewer();
    graphicalViewer.createControl(_parent);
    // XXX: Wichtig - Damit wei der Viewer welchen EditPart er fr ein
    // Modell Objekt benutzen soll.
    graphicalViewer.setEditDomain(new EditDomain());
    graphicalViewer.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewer.setEditPartFactory(new DarstellungEditPartFactory());
    
    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(DefsPreference.P_DARSTELLUNG_ZUSTAND_SOLL);
    if (ansicht != null)
      graphicalViewer.setContents(ansicht);    

    restoreFromPrefs();

    getCommandStack().setUndoLimit(1);

    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    if (_connection != null)
    {
      List<SystemObject> list = DaVHelper.holeSystemObjekteAusDarstellung(getGraphicalViewer());
      DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
      DaVKommunikationDobj.getInstanz().anmeldenAufDaten(list);
      
      anmeldenSituationen(_connection);
      
    }

    registriereActions();
    erzeugeKontextMenue();

    graphicalViewer.getControl().addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        // TODO Auto-generated method stub
        ISelection selection = graphicalViewer.getSelection();
        IStructuredSelection ss = (IStructuredSelection)selection;
        if (ss != null && !ss.isEmpty())
        {
          Object obj = ss.getFirstElement();
          // System.out.println("DoubleClick:" + obj);

          if (obj instanceof DoModelEditPart)
          {
            DoModelEditPart part = (DoModelEditPart)obj;
            DoModel model = (DoModel)part.getModel();
            DoTyp doTyp = model.getDoTyp();
            if (doTyp instanceof VerhaltenDoTyp)
            {
              VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

              int kontext = -1;

              if (model instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
                kontext = aq.getKontext();
              }
              else if (model instanceof AnzeigeModelTypAbstract)
              {
                AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
                EObject parent = a.eContainer();
                if (parent != null)
                {
                  if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
                  {
                    AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
                    kontext = aq.getKontext();
                  }
                }
              }
              else if (model instanceof StreckenStationModel)
              {
                StreckenStationModel sst = (StreckenStationModel)model;
                kontext = sst.getKontext();
              }
              else if (model instanceof DetektorModelTypAbstract)
              {
                DetektorModelTypAbstract de = (DetektorModelTypAbstract)model;
                kontext = de.getKontext();
              }
              else if (model instanceof TabelleMqModel)
              {
                TabelleMqModel tab = (TabelleMqModel)model;
                kontext = 1;
              }

              String id = null;

              switch (kontext)
              {
                case 0:
                  id = coDoTyp.getDoppelKlickIdIstZustand();
                  break;
                case 1:
                  id = coDoTyp.getDoppelKlickIdSollZustand();
                  break;
                case 2:
                  id = coDoTyp.getDoppelKlickIdHandSchaltung();
                  break;
                case 3:
                  id = coDoTyp.getDoppelKlickIdSimulation();
                  break;
                case 4:
                  id = coDoTyp.getDoppelKlickIdTrichter();
                  break;

                default:
                  break;
              }

              if (id != null)
                leseActions(id);
            }
          }

        }
      }
    });

    graphicalViewer.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection ss = (IStructuredSelection)event.getSelection();

        Object obj = ss.getFirstElement();

        _kontextMenuMgr.removeAll();

        if (obj instanceof DoModelEditPart)
        {
          DoModelEditPart part = (DoModelEditPart)obj;
          DoModel model = (DoModel)part.getModel();
          DoTyp doTyp = model.getDoTyp();
          if (doTyp instanceof VerhaltenDoTyp)
          {
            VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

            int kontext = -1;

            if (model instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
              kontext = aq.getKontext();
            }
            else if (model instanceof StreckenStationModel)
            {
              StreckenStationModel sst = (StreckenStationModel)model;
              kontext = sst.getKontext();
            }
            else if (model instanceof AnzeigeModelTypAbstract)
            {
              AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
              kontext = a.getKontext();
            }

            String id = null;

            switch (kontext)
            {
              case 0:
                id = coDoTyp.getKontextMenueIdIstZustand();
                break;
              case 1:
                id = coDoTyp.getKontextMenueIdSollZustand();
                break;
              case 2:
                id = coDoTyp.getKontextMenueIdHandSchaltung();
                break;
              case 3:
                id = coDoTyp.getKontextMenueIdSimulation();
                break;
              case 4:
                id = coDoTyp.getKontextMenueIdTrichter();
                break;

              default:
                break;
            }

            if (id != null)
              erzeugeKontextMenueAusActionSetId(id);
          }
        }
      }
    });
    getSite().setSelectionProvider(this.getGraphicalViewer());
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

    IWorkbenchPage page = getSite().getPage();

    if (page != null)
    {
      IViewReference fr = page.findViewReference(AuswahlEbenenView.ID);

      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        AuswahlEbenenView auswahl = (AuswahlEbenenView)view;
        auswahl.setContent((AnsichtEditPart)graphicalViewer.getContents());
      }

      fr = page.findViewReference(AuswahlSpaltenView.ID);

      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        AuswahlSpaltenView auswahl = (AuswahlSpaltenView)view;
        auswahl.setContent((AnsichtEditPart)graphicalViewer.getContents());
      }
    }
  }

  @Override
  public void dispose()
  {
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    IWorkbenchPage page = getSite().getPage();
    if (page != null)
    {
      IViewReference fr = page.findViewReference(ZustandsDarstellungOutlineView.ID);
      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        if (view != null)
        {
          ZustandsDarstellungOutlineView outline = (ZustandsDarstellungOutlineView)view;
          if (outline != null)
          {
            outline.setOutlineContent(new ScalableRootEditPart());
          }
        }
      }

      fr = page.findViewReference(AuswahlSpaltenView.ID);
      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        if (view != null)
        {
          AuswahlSpaltenView auswahl = (AuswahlSpaltenView)view;
          if (auswahl != null)
          {
            auswahl.setContent(null);
          }
        }
      }

    }   
    graphicalViewer.removeSelectionChangedListener(_selListener);
  }

  /**
   * Adaptable implementation for Editor
   */
  public Object getAdapter(Class adapter)
  {
    // we need to handle common GEF elements we created
    if (adapter == GraphicalViewer.class || adapter == EditPartViewer.class)
      return getGraphicalViewer();
    else if (adapter == CommandStack.class)
      return getCommandStack();
    else if (adapter == EditDomain.class)
      return getEditDomain();
    else if (adapter == ActionRegistry.class)
      return getActionRegistry();
    if (adapter == ILegende.class) {
      return new Legende();
    }

    // the super implementation handles the rest
    return super.getAdapter(adapter);
  }

  public GraphicalViewer getGraphicalViewer()
  {
    return graphicalViewer;
  }

  @Override
  protected void initializeGraphicalViewer()
  {
    // TODO Auto-generated method stub
  }

  /**
   * Registriert die Actions zum Aufruf der Outline-View
   */
  private void registriereActions()
  {
    IMenuManager menuManager = this.getViewSite().getActionBars().getMenuManager();
    IToolBarManager toolBarManager = this.getViewSite().getActionBars().getToolBarManager();

    Action oeffneAction = new Action() {
      public void run()
      {
        oeffneZustandsDarstellungOutlineView();         
      }

    };
    oeffneAction.setText("ffne Picker");

    ImageDescriptor id = Activator.getImageDescriptor("icons/editor.gif");

    oeffneAction.setImageDescriptor(id);
    menuManager.add(oeffneAction);
    toolBarManager.add(oeffneAction);

    AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
    Ansicht ansicht = contents.getModel();
    Darstellung darstellung = ansicht.getDarstellung();
    EList<Double> stufen = darstellung.getZoomStufen();

    final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewer.getRootEditPart();
    final ZoomManager zoomManager = rootEditPart.getZoomManager();

    double[] dd = new double[stufen.size()];

    for (int i = 0; i < dd.length; i++)
    {
      dd[i] = stufen.get(i);
    }

    zoomManager.setZoomLevels(dd);

    for (final Double stufe : stufen)
    {
      Action zoomAction = new Action() {
        public void run()
        {
          zoomManager.setZoom(stufe);
        }

      };
      zoomAction.setText("Zoomstufe " + stufe);
      menuManager.add(zoomAction);
    }

    Action oeffneAction2 = new Action() {
      public void run()
      {
        oeffneAuswahlSpaltenView();
      }

    };
    oeffneAction2.setText("ffne Spalten-Auswahl");

    ImageDescriptor id2 = ImageDescriptor.createFromImage(ImageFactory.getImage("icons/th_horizontal.png"));
    oeffneAction2.setImageDescriptor(id2);
    menuManager.add(oeffneAction2);
    toolBarManager.add(oeffneAction2);
    
    IAction aktionLegende = holeAction("de.bsvrz.buv.rw.bitctrl.eclipse.legende.oeffneLegende");
    if (aktionLegende != null){
      menuManager.add(aktionLegende);
      toolBarManager.add(aktionLegende);            
    }
  }

  private void oeffneAuswahlSpaltenView()
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    try
    {
      page.showView(AuswahlSpaltenView.ID);
      IViewReference ref = page.findViewReference(AuswahlSpaltenView.ID);
      if (ref != null)
      {
        AuswahlSpaltenView view = (AuswahlSpaltenView)ref.getView(true);
        if (view != null)
        {
          view.setContent((AnsichtEditPart)graphicalViewer.getContents());
        }
      }
    }
    catch (PartInitException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  private void oeffneZustandsDarstellungOutlineView()
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    try
    {
      page.showView(ZustandsDarstellungOutlineView.ID, null, IWorkbenchPage.VIEW_ACTIVATE);
      IViewReference ref = page.findViewReference(ZustandsDarstellungOutlineView.ID);
      if (ref != null)
      {
        ZustandsDarstellungOutlineView view = (ZustandsDarstellungOutlineView)ref.getView(true);
        if (view != null)
        {
          view.setOutlineContent(graphicalViewer.getRootEditPart());
        }
      }
    }
    catch (PartInitException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData result = e.getResult();
    if (result != null)
    {
      Aspect asp = result.getDataDescription().getAspect();
      if (!asp.getPid().equals("asp.sswSbaSonderprogrammSimu"))
        DaVHelper.verarbeiteDaVDaten(_connection, getGraphicalViewer(), result, 1);
    }
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {

    CommandStack stack = graphicalViewer.getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  public AnsichtEditPart getContents()
  {

    AnsichtEditPart contents = null;

    if (graphicalViewer != null)
      contents = (AnsichtEditPart)graphicalViewer.getContents();

    return contents;
  }

  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenue()
  {
    _kontextMenuMgr = new MenuManager("#SollZustandMenu");
    erzeugeAktionen();
    Menu menu = _kontextMenuMgr.createContextMenu(graphicalViewer.getControl());
    graphicalViewer.getControl().setMenu(menu);
  }

  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenueAusActionSetId(String id)
  {
    leseActionSets(id);
  }

  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActionSets(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String att = confEle.getAttribute("id");
      String contributor = confEle.getContributor().getName();
      if (Activator.PLUGIN_ID.equals(contributor) && att.equals(id))
      {
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {
          boolean pullDownStyle = false;
          String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
          if (style != null)
          {
            pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
          }
          else
          {
            String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
            pullDownStyle = pulldown != null && pulldown.equals("true");
          }

          ActionDescriptor desc = null;
          if (pullDownStyle)
          {
            desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
          }
          else
          {
            desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
          }
          PluginAction action = desc.getAction();
          _kontextMenuMgr.add(action);
        }

      }
    }
  }

  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActions(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();
      if (Activator.PLUGIN_ID.equals(contributor))
      {
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {
          String att = element.getAttribute("id");
          if (att.equals(id))
          {
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            }
            else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }

            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            }
            else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }
            PluginAction action = desc.getAction();
            action.run();
          }
        }
      }
    }
  }

  /**
   * Aktion im Kontextmen setzen
   * 
   * @param action
   *          Aktion die im Kontextmen erscheinen soll
   */
  public void addtoKontextMenue(Action action)
  {

    if (action != null)
      _kontextMenuMgr.add(action);
  }

  /**
   * Erzeugt die Aktionen zum Verwalten der Hierarchieobjekte
   */
  private void erzeugeAktionen()
  {
    _actionRegistry = new ActionRegistry();
    Action action1 = new Action() {
      public void run()
      {
        System.out.println("Aktion1");
      }
    };
    action1.setId("Aktion1");
    action1.setText("Aktion1");
    action1.setToolTipText("Aktion1");
    _actionRegistry.registerAction(action1);

    Action action2 = new Action() {
      public void run()
      {
        System.out.println("Aktion1");
      }
    };
    action2.setId("Aktion2");
    action2.setText("Aktion2");
    action2.setToolTipText("Aktion2");
    _actionRegistry.registerAction(action2);
  }
}
