/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;

import de.bsvrz.buv.plugin.baueditor.sts.actions.EngstellenVerwaltungPluginAktion;
import de.bsvrz.buv.rw.rw.menu.ElementRegistrierung;

/**
 * <b>Workaround:</b> Da Aktionen (<code>IActionDelegate</code>), die via
 * Rahmenwerk-Menueeditor in einen Toolbar oder in ein Menue appliziert worden
 * nicht mehr mit den an der Workbenchseite angemeldeten
 * <code>ISelectionProvidern</code> verbunden sind, wird hier diese Verbindung
 * nochmal explizit hergestellt.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: RahmenwerkSelektionsVerteiler.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public final class RahmenwerkSelektionsVerteiler {

	/** Menge aller registrierten Aktionen. */
	static final Set<EngstellenVerwaltungPluginAktion> GLOBAL_RAHMENWERK_AKTIONS_SET = new HashSet<EngstellenVerwaltungPluginAktion>();

	/**
	 * Statischer Listener an allen Selections, die ueber saemtliche
	 * registrierte <code>ISelectionProvider</code> verteilt werden.
	 */
	private static final ISelectionChangedListener LISTENER = new ISelectionChangedListener() {

		/**
		 * Verteilt alle empfangenen Selections an die angemeldeten Aktionen und
		 * aktualisiert ggf. die Menueleiste sowie die Toolbar.
		 * 
		 * @param event
		 *            empfangener Selektions-Event.
		 */
		public void selectionChanged(final SelectionChangedEvent event) {

			boolean updateNecessary = false;
			synchronized (RahmenwerkSelektionsVerteiler.GLOBAL_RAHMENWERK_AKTIONS_SET) {
				for (final EngstellenVerwaltungPluginAktion aktion : RahmenwerkSelektionsVerteiler.GLOBAL_RAHMENWERK_AKTIONS_SET) {
					final boolean vorher = aktion.isEnabled();
					aktion.selectionChanged(null, event.getSelection());
					final boolean nachher = aktion.isEnabled();
					updateNecessary |= vorher ^ nachher;
				}
			}
			if (updateNecessary) {
				ElementRegistrierung.getInstanz().getCoolBarManager().update(
						true);
				ElementRegistrierung.getInstanz().getMenuManager().update(true);
			}
		}
	};

	/**
	 * Fuegt diesem Verteiler eine neue Aktion hinzu.
	 * 
	 * @param aktion
	 *            eine neue Aktion.
	 */
	public static void addRahmenwerkAktion(
			final EngstellenVerwaltungPluginAktion aktion) {
		synchronized (RahmenwerkSelektionsVerteiler.GLOBAL_RAHMENWERK_AKTIONS_SET) {
			RahmenwerkSelektionsVerteiler.GLOBAL_RAHMENWERK_AKTIONS_SET
					.add(aktion);
		}
	}

	/**
	 * Fuegt diesem Verteiler einen neuen <code>ISelectionProvider</code> hinzu
	 * und verteilt dessen Selections an alle registrierten Aktionen.
	 * 
	 * @param selectionProvider
	 *            ein neuer <code>ISelectionProvider</code>.
	 */
	public static void addSelectionProvider(
			final ISelectionProvider selectionProvider) {
		selectionProvider
				.addSelectionChangedListener(RahmenwerkSelektionsVerteiler.LISTENER);
	}

	/** privater Konstruktor verhindert das Anlegen von zustzlichen Instanzen. */
	private RahmenwerkSelektionsVerteiler() {
		super();
	}
}
