/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.EngstellenVerwaltungPluginFunktion;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.EngstellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.BaustellenHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.RichtungHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.StrassenHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.UnfallHelper;
import de.bsvrz.buv.plugin.selektion.SelektionsTransfer;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.InneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCache;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCacheExtended;
import de.bsvrz.sys.funclib.bitctrl.modell.util.comparator.InnereSegmenteComparator;

/**
 * Abstrakte Aktion zum Anlegen einer neuen Engstelle.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: AnlegenAktion.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public abstract class AnlegenAktion extends VerkehrsModellNetzAktion {

	/**
	 * Der Konstruktor.
	 * 
	 * @param name
	 *            der Name der Aktion
	 * @param imageDescriptor
	 *            das Symbol der Aktion
	 * @param funktion
	 *            die dazugehrige Funktion, siehe
	 *            EngstellenVerwaltungPluginFunktion.*
	 * 
	 * @see EngstellenVerwaltungPluginFunktion
	 */
	public AnlegenAktion(final String name,
			final ImageDescriptor imageDescriptor,
			final EngstellenVerwaltungPluginFunktion funktion) {
		super(name, imageDescriptor, funktion);
	}

	/**
	 * Liefert die aktuell selektierte Lage zurck.
	 * 
	 * @return die aktuell selektierte Lage zurck
	 */
	protected LageInfo getSelektierteLage() {

		Strasse strasse = null;
		AttTmcRichtung richtung = null;
		StrassenSegment segment = null;

		Object first = null;
		final SelektionsTransfer transfer = SelektionsTransfer.getInstanz();
		final IStructuredSelection aktuelleSelection = transfer
				.getAktuelleSelection();
		if (aktuelleSelection != null && !aktuelleSelection.isEmpty()) {
			first = aktuelleSelection.getFirstElement();
		}

		if (first instanceof BaustellenHelper || first instanceof UnfallHelper) {
			final IStructuredSelection vorherigeSelection = transfer
					.getVorherigeSelection();
			if (vorherigeSelection != null && !vorherigeSelection.isEmpty()) {
				first = vorherigeSelection.getFirstElement();
			}
		}

		if (first != null) {

			if (first instanceof StrassenHelper) {
				strasse = ((StrassenHelper) first).getStrasse();

			} else if (first instanceof RichtungHelper) {
				final RichtungHelper richtungHelper = (RichtungHelper) first;
				strasse = richtungHelper.getStrasse();
				richtung = richtungHelper.getRichtung();

			} else if (first instanceof EngstellenWrapper) {
				final EngstellenWrapper wrapper = (EngstellenWrapper) first;
				strasse = wrapper.getStrasse();
				richtung = wrapper.getRichtung();

			} else if (!(first instanceof SystemObject)
					&& !(first instanceof SystemObjekt)) {

				final SystemObjekt systemObjekt = (SystemObjekt) Platform
						.getAdapterManager().getAdapter(first,
								SystemObjekt.class);
				if (systemObjekt != null) {
					first = systemObjekt;
				} else {
					final SystemObject systemObject = (SystemObject) Platform
							.getAdapterManager().getAdapter(first,
									SystemObject.class);
					if (systemObject != null) {
						first = systemObject;
					}
				}
			}

			if (first instanceof SystemObject) {
				first = BauEditorPlugin.getDefault().getObjektFactory()
						.getModellobjekt(((SystemObject) first).getPid());
			}

			final NetzCache netzCache = BauEditorPlugin.getDefault()
					.getNetzCache();
			final NetzCacheExtended netzCacheExtended = StartupPlugin
					.getDefault().getDefaultNetzCacheExtended();

			// Bestimme Lage-Info anhand eines selektierten Straenknotens.
			if (first instanceof StrassenKnoten) {
				final StrassenKnoten knoten = (StrassenKnoten) first;

				final InnereSegmenteComparator comparator = new InnereSegmenteComparator();
				final List<InneresStrassenSegment> innereSegmente = new ArrayList<InneresStrassenSegment>(
						netzCache.getInneresStrassenSegmente(knoten));

				if (innereSegmente != null && !innereSegmente.isEmpty()) {
					Collections.sort(innereSegmente,
							new Comparator<InneresStrassenSegment>() {
								@Override
								public int compare(
										final InneresStrassenSegment o1,
										final InneresStrassenSegment o2) {

									// (1) berprfe zunchste die Strae...
									final Strasse s1 = netzCacheExtended
											.getStrasse(o1);
									final Strasse s2 = netzCacheExtended
											.getStrasse(o2);
									int compare = netzCacheExtended
											.compareStrassen(s1, s2);
									if (compare != 0) {
										return compare;
									}

									// (2) ...danach ggf. die Richtung...
									final AttTmcRichtung r1 = netzCacheExtended
											.getRichtung(o1);
									final AttTmcRichtung r2 = netzCacheExtended
											.getRichtung(o2);
									compare = r1.getValue().compareTo(
											r2.getValue());
									if (compare != 0) {
										return compare;
									}

									// (3) ...und abschlieend ggf. die
									// Hauptfahrbeziehung und Lnge.
									return comparator.compare(o1, o2);
								}
							});

					first = innereSegmente.get(0);

				} else {

					final List<AeusseresStrassenSegment> segmente = netzCacheExtended
							.getAeussereStrassenSegmente(knoten);

					boolean gefunden = false;
					for (final AeusseresStrassenSegment ass : segmente) {
						if (ass.getKdAeusseresStrassenSegment().getDatum()
								.getVonKnoten().getPid()
								.equals(knoten.getPid())) {
							first = ass;
							gefunden = true;
							break;
						}
					}
					if (!gefunden && segmente != null && !segmente.isEmpty()) {
						first = segmente.get(0);
					}
				}
			}

			// Bestimme Lage-Info anhand eines selektierten inneren
			// Straensegments.
			if (first instanceof InneresStrassenSegment) {
				final InneresStrassenSegment iss = (InneresStrassenSegment) first;

				strasse = netzCacheExtended.getStrasse(iss);
				richtung = netzCacheExtended.getRichtung(iss);
				segment = iss;
			}

			// Bestimme Lage-Info anhand eines selektierten ueren
			// Straensegments.
			if (first instanceof AeusseresStrassenSegment) {
				final AeusseresStrassenSegment ass = (AeusseresStrassenSegment) first;

				final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdStrassenSegment.Daten kdSegment = ass
						.getKdStrassenSegment().getDatum();
				final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdAeusseresStrassenSegment.Daten kdAeuesseres = ass
						.getKdAeusseresStrassenSegment().getDatum();

				strasse = (kdSegment != null) ? kdSegment.getGehoertZuStrasse()
						: null;
				richtung = (kdAeuesseres != null) ? kdAeuesseres
						.getTmcRichtung() : null;
				segment = ass;
			}
		}
		return new LageInfo(strasse, richtung, segment);
	}

	/**
	 * Hilfsklasse, welche eine selektierte Lage (Strae, Richtung,
	 * Straensegment) beschreibt.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: AnlegenAktion.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
	 */
	protected class LageInfo {

		/** Die Strasse. */
		private final Strasse strasse;

		/** Die Richtung. */
		private final AttTmcRichtung richtung;

		/** Das Straensegment. */
		private final StrassenSegment segment;

		/**
		 * Der Konstruktor.
		 * 
		 * @param strasse
		 *            die aktuelle Strae
		 * @param richtung
		 *            die Straenrichtung
		 * @param segment
		 *            das Straensegment
		 */
		public LageInfo(final Strasse strasse, final AttTmcRichtung richtung,
				final StrassenSegment segment) {
			this.strasse = strasse;
			this.richtung = richtung;
			this.segment = segment;
		}

		/**
		 * Liefert die Strae zurck.
		 * 
		 * @return die Strae
		 */
		public Strasse getStrasse() {
			return strasse;
		}

		/**
		 * Liefert die Richtung zurck.
		 * 
		 * @return die Richtung
		 */
		public AttTmcRichtung getRichtung() {
			return richtung;
		}

		/**
		 * Liefert den Startstraenknoten zurck.
		 * 
		 * @return den Startstraenknoten
		 */
		public StrassenSegment getStrassenSegment() {
			return segment;
		}
	}
}
