/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.util.wrapper;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorException;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttBaustellenVeranlasser;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttStatusBaustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;

/**
 * Wrapper, welcher fr ein Baustellen-SystemObjekt smtliche Daten vorhlt.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: BaustellenWrapper.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 * 
 * @see Baustelle
 */
public class BaustellenWrapper extends EngstellenWrapper {

	/** Der Status der Baustelle. */
	private AttStatusBaustelle status;

	/** Der Veranlasser der Baustelle. */
	private AttBaustellenVeranlasser veranlasser;

	/** Die verantwortliche Firma. */
	private String verantwortlicheFirma;

	/** Die Telefonnummer der verantwortlichen Firma. */
	private String telefonFirma;

	/** Der Baustellenverantwortliche. */
	private String baustellenVerantwortlicher;

	/** Die Telefonnummer des Baustellenverantwortlichen. */
	private String telefonBaustellenVerantwortlicher;

	/** Die Mobiltelefonnummer des Baustellenverantwortlichen. */
	private String mobilTelefonBaustellenVerantwortlicher;

	/**
	 * Der Konstruktor.
	 * 
	 * @param strasse
	 *            die Strae, kann <code>null</code> sein
	 * @param richtung
	 *            die Richtung, kann <code>null</code> sein
	 * @param segment
	 *            das Straensegment, kann <code>null</code> sein
	 */
	public BaustellenWrapper(final Strasse strasse,
			final AttTmcRichtung richtung, final StrassenSegment segment) {
		super(strasse, richtung, segment);

		setDefaultEigenschaften();
		setDefaultVerantwortlicher();
	}

	/**
	 * Setzt die Standardwerte fr Status und Veranlasser.
	 */
	private void setDefaultEigenschaften() {
		status = AttStatusBaustelle.ZUSTAND_0_ENTWORFEN;
		veranlasser = AttBaustellenVeranlasser.ZUSTAND_2_VRZ_INTERN;
	}

	/**
	 * Setzt die Standardwerte fr den Baustellenverantwortlichen.
	 */
	private void setDefaultVerantwortlicher() {
		verantwortlicheFirma = "";
		telefonFirma = "";
		baustellenVerantwortlicher = "";
		telefonBaustellenVerantwortlicher = "";
		mobilTelefonBaustellenVerantwortlicher = "";
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param baustelle
	 *            ein Baustellen-SystemObjekt
	 * 
	 * @throws BauEditorException
	 *             wird geworfen, wenn die gegebene Baustelle nicht geladen
	 *             werden konnte
	 */
	public BaustellenWrapper(final Baustelle baustelle)
			throws BauEditorException {
		super(baustelle);

		// Lese Parameterdatensatz Eigenschaften
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenEigenschaften.Daten pdEigenschaften = baustelle
				.getPdBaustellenEigenschaften().getDatum();
		if (pdEigenschaften != null) {
			setZeitStempel(pdEigenschaften.dGetZeitstempel());
			status = pdEigenschaften.getStatus();
			if (status == null) {
				status = AttStatusBaustelle.ZUSTAND_0_ENTWORFEN;
			}
			veranlasser = pdEigenschaften.getVeranlasser();
			if (veranlasser == null) {
				veranlasser = AttBaustellenVeranlasser.ZUSTAND_2_VRZ_INTERN;
			}
		} else {
			setDefaultEigenschaften();
		}

		// Lese Parameterdatensatz Verantworlicher
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenVerantwortlicher.Daten pdVerantwortlicher = baustelle
				.getPdBaustellenVerantwortlicher().getDatum();
		if (pdVerantwortlicher != null) {
			setZeitStempel(pdVerantwortlicher.dGetZeitstempel());
			verantwortlicheFirma = pdVerantwortlicher.getFirma();
			if (verantwortlicheFirma == null) {
				verantwortlicheFirma = "";
			}
			telefonFirma = pdVerantwortlicher.getTelefonFirma();
			if (telefonFirma == null) {
				telefonFirma = "";
			}
			baustellenVerantwortlicher = pdVerantwortlicher
					.getNameBaustellenVerantwortlicher();
			if (baustellenVerantwortlicher == null) {
				baustellenVerantwortlicher = "";
			}
			telefonBaustellenVerantwortlicher = pdVerantwortlicher
					.getTelefonBaustellenVerantwortlicher();
			if (telefonBaustellenVerantwortlicher == null) {
				telefonBaustellenVerantwortlicher = "";
			}
			mobilTelefonBaustellenVerantwortlicher = pdVerantwortlicher
					.getTelefonMobilBaustellenVerantwortlicher();
			if (mobilTelefonBaustellenVerantwortlicher == null) {
				mobilTelefonBaustellenVerantwortlicher = "";
			}
		} else {
			setDefaultVerantwortlicher();
		}
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            ein anderer BaustellenWrapper
	 */
	public BaustellenWrapper(final BaustellenWrapper wrapper) {
		this(wrapper, false);
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            ein anderer BaustellenWrapper
	 * @param neuesElement
	 *            <code>true</code>, wenn es sich bei dem neu erstellten
	 *            BaustellenWrapper, um eine neue Baustelle handelt. false, wenn
	 *            ein Klon erstellt werden soll.
	 */
	public BaustellenWrapper(final BaustellenWrapper wrapper,
			final boolean neuesElement) {
		super(wrapper, neuesElement);

		status = new AttStatusBaustelle(wrapper.status.getValue());
		veranlasser = new AttBaustellenVeranlasser(
				wrapper.veranlasser.getValue());
		verantwortlicheFirma = new String(wrapper.verantwortlicheFirma);
		telefonFirma = new String(wrapper.telefonFirma);
		baustellenVerantwortlicher = new String(
				wrapper.baustellenVerantwortlicher);
		telefonBaustellenVerantwortlicher = new String(
				wrapper.telefonBaustellenVerantwortlicher);
		mobilTelefonBaustellenVerantwortlicher = new String(
				wrapper.mobilTelefonBaustellenVerantwortlicher);
	}

	/**
	 * Liefert den aktuellen Status der Baustelle zurck.
	 * 
	 * @return den aktuellen Status der Baustelle
	 */
	public AttStatusBaustelle getStatus() {
		return status;
	}

	/**
	 * Setzt den Status fr diese Baustelle.
	 * 
	 * @param neuerWert
	 *            der neue Status
	 */
	public void setStatus(final AttStatusBaustelle neuerWert) {
		final AttStatusBaustelle alterWert = status;
		status = neuerWert;
		firePropertyChange("status", alterWert, neuerWert);

		if (status.getValue() == AttStatusBaustelle.ZUSTAND_2_GUELTIG
				.getValue()) {
			setVerknuepfeEreignis(true);
		} else if (getStartZeit().getTime() > System.currentTimeMillis()) {
			setVerknuepfeEreignis(false);
		}
	}

	/**
	 * Liefert den Veranlasser dieser Baustelle zurck.
	 * 
	 * @return den Veranlasser dieser Baustelle
	 */
	public AttBaustellenVeranlasser getVeranlasser() {
		return veranlasser;
	}

	/**
	 * Setzt den neuen Veranlasser fr diese Baustelle.
	 * 
	 * @param neuerWert
	 *            der neue Veranlasser
	 */
	public void setVeranlasser(final AttBaustellenVeranlasser neuerWert) {
		final AttBaustellenVeranlasser alterWert = veranlasser;
		veranlasser = neuerWert;
		firePropertyChange("veranlasser", alterWert, neuerWert);
	}

	/**
	 * Liefert die verantwortliche Firma zurck.
	 * 
	 * @return die verantwortliche Firma
	 */
	public String getVerantwortlicheFirma() {
		return verantwortlicheFirma;
	}

	/**
	 * Setzt die verantwortliche Firma.
	 * 
	 * @param neuerWert
	 *            die neue verantwortliche Firma
	 */
	public void setVerantwortlicheFirma(final String neuerWert) {
		final String alterWert = verantwortlicheFirma;
		verantwortlicheFirma = neuerWert;
		firePropertyChange("verantwortlicheFirma", alterWert, neuerWert);
	}

	/**
	 * Liefert die Telefonnummer der verantwortlichen Firma zurck.
	 * 
	 * @return die Telefonnummer der verantwortlichen Firma
	 */
	public String getTelefonFirma() {
		return telefonFirma;
	}

	/**
	 * Setzt die Telefonnummer der verantwortlichen Firma.
	 * 
	 * @param neuerWert
	 *            die neue Telefonnummer
	 */
	public void setTelefonFirma(final String neuerWert) {
		final String alterWert = telefonFirma;
		telefonFirma = neuerWert;
		firePropertyChange("telefonFirma", alterWert, neuerWert);
	}

	/**
	 * Liefert den Baustellenverantwortlichen zurck.
	 * 
	 * @return den Baustellenverantwortlichen
	 */
	public String getBaustellenVerantwortlicher() {
		return baustellenVerantwortlicher;
	}

	/**
	 * Setzt den Baustellenverantwortlichen.
	 * 
	 * @param neuerWert
	 *            der neue Baustellenverantwortlichen
	 */
	public void setBaustellenVerantwortlicher(final String neuerWert) {
		final String alterWert = baustellenVerantwortlicher;
		baustellenVerantwortlicher = neuerWert;
		firePropertyChange("baustellenVerantwortlicher", alterWert, neuerWert);
	}

	/**
	 * Liefert die Telefonnummer des Baustellenverantwortlichen zurck.
	 * 
	 * @return die Telefonnummer des Baustellenverantwortlichen
	 */
	public String getTelefonBaustellenVerantwortlicher() {
		return telefonBaustellenVerantwortlicher;
	}

	/**
	 * Setzt die Telefonnummer des Baustellenverantwortlichen.
	 * 
	 * @param neuerWert
	 *            die neue Telefonnnummer
	 */
	public void setTelefonBaustellenVerantwortlicher(final String neuerWert) {
		final String alterWert = telefonBaustellenVerantwortlicher;
		telefonBaustellenVerantwortlicher = neuerWert;
		firePropertyChange("telefonBaustellenVerantwortlicher", alterWert,
				neuerWert);
	}

	/**
	 * Liefert die Mobiltelefonnummer des Baustellenverantwortlichen zurck.
	 * 
	 * @return die Mobiltelefonnummer des Baustellenverantwortlichen
	 */
	public String getMobilTelefonBaustellenVerantwortlicher() {
		return mobilTelefonBaustellenVerantwortlicher;
	}

	/**
	 * Setzt die Mobiltelefonnummer des Baustellenverantwortlichen.
	 * 
	 * @param neuerWert
	 *            die neue Telefonnummer
	 */
	public void setMobilTelefonBaustellenVerantwortlicher(final String neuerWert) {
		final String alterWert = mobilTelefonBaustellenVerantwortlicher;
		mobilTelefonBaustellenVerantwortlicher = neuerWert;
		firePropertyChange("mobilTelefonBaustellenVerantwortlicher", alterWert,
				neuerWert);
	}

	@Override
	public String toString() {
		return getName() + " (Baustelle)";
	}

	@Override
	public String getInfo() {
		final StringBuffer buf = new StringBuffer(super.getInfo());

		buf.append("\nStatus: ");
		buf.append(status);
		buf.append("\nVeranlasser: ");
		buf.append(veranlasser);
		buf.append("\nVerantwortliche Firma: ");
		buf.append(verantwortlicheFirma);
		buf.append("\nTelefon Firma: ");
		buf.append(telefonFirma);
		buf.append("\nBaustellenverantwortlicher: ");
		buf.append(baustellenVerantwortlicher);
		buf.append("\nTelefon Baustellenverantwortlicher: ");
		buf.append(telefonBaustellenVerantwortlicher);
		buf.append("\nMobiltelefon Baustellenverantwortlicher: ");
		buf.append(mobilTelefonBaustellenVerantwortlicher);

		return buf.toString();
	}
}
