/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.paperclips.BigPrint;
import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridColumn;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.ImagePrint;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.TextPrint;
import net.sf.paperclips.TextStyle;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.dialogs.FilteredTree;
import org.eclipse.ui.dialogs.PatternFilter;
import org.eclipse.ui.progress.UIJob;

import com.bitctrl.lib.eclipse.actions.CollapseAllAction;
import com.bitctrl.lib.eclipse.actions.ExpandAllAction;
import com.bitctrl.lib.eclipse.actions.ShowTextFilterAction;
import com.bitctrl.lib.eclipse.help.DefaultHelpContext;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.BildDateiPfade;
import de.bsvrz.buv.plugin.baueditor.sts.Konstanten;
import de.bsvrz.buv.plugin.baueditor.sts.Zeichenketten;
import de.bsvrz.buv.plugin.baueditor.sts.actions.BaustelleBearbeitenAktion;
import de.bsvrz.buv.plugin.baueditor.sts.actions.EngstellenVerwaltungPluginAktion;
import de.bsvrz.buv.plugin.baueditor.sts.actions.StauAnzeigenAktion;
import de.bsvrz.buv.plugin.baueditor.sts.actions.UnfallBearbeitenAktion;
import de.bsvrz.buv.plugin.baueditor.sts.editors.EngstellenEditor;
import de.bsvrz.buv.plugin.baueditor.sts.editors.EngstellenEditorInput;
import de.bsvrz.buv.plugin.baueditor.sts.util.EngstellenCache;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.BaustellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.EngstellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.StauWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.UnfallWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.BaustellenHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.Helper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.RichtungHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.StauHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.StrassenHelper;
import de.bsvrz.buv.plugin.baueditor.sts.views.helper.UnfallHelper;
import de.bsvrz.buv.plugin.selektion.SelektionsTransfer;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.eclipse.PatternOrSubstringFilter;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.java.JavaModellEinstellungen;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCacheExtended;

/**
 * Die Ansicht "Engstellen-Navigator".
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenNavigatorViewPart.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public class EngstellenNavigatorViewPart extends AbstractBauEditorViewPart {

	/** Die eindeutige ID dieser Ansicht/View. */
	public static final String ID = "de.bsvrz.buv.plugin.baueditor.view.engstellenNavigator"; //$NON-NLS-1$

	/** Das Root-Objekt des TreeViewers. */
	private String root;

	/** Die Einstellungen dieser Ansicht. */
	private EngstellenNavigatorViewPartEinstellungen einstellungen;

	/**
	 * Die Aktion zum Bearbeiten einer Baustelle (also zum ffnen des
	 * Baustellen-Editors).
	 */
	private BaustelleBearbeitenAktion baustelleBearbeitenAktion;

	/**
	 * Die Aktion zum Bearbeiten eines Unfalls (also zum ffnen des
	 * Baustellen-Editors).
	 */
	private UnfallBearbeitenAktion unfallBearbeitenAktion;

	/**
	 * Die Aktion zum Anzeigen der Daten eines Staus (also zum ffnen des
	 * Stau-Editors).
	 */
	private StauAnzeigenAktion stauAnzeigenAktion;

	/** Der FilteredTree. */
	private FilteredTree filteredTree;

	/** Der TreeViewer. */
	private TreeViewer viewer;

	/** Der PatternFilter des FilterTrees. */
	private PatternFilter patternFiter;

	/** Aktion zum Aktualisieren der Ansicht. */
	private IAction aktualisierenAction;

	/** Aktion zum Ein- bzw. Ausblenden des Text-Filters. */
	private ShowTextFilterAction zeigeTextFilterAction;

	/** Aktion zum Aktivieren bzw. Deaktivieren des Straenfilters. */
	private IAction filterAction;

	/**
	 * Aktion zum Aktivieren bzw. Deaktivieren der allgemeinen
	 * Selektions-Verknpfung.
	 */
	private IAction verknuepfeAction;

	/** Aktion zum Aktivieren bzw. Deaktivieren der Verknpfung mit dem Editor. */
	private IAction editorAction;

	/** Map, welche UIJobs zum Aktualisieren dieser Ansicht speichert. */
	private Map<String, UpdateViewJob> updateViewJobMap;

	/** Der ContentProvider des TreeViewers dieser Ansicht. */
	private final ITreeContentProvider contentProvider = new ITreeContentProvider() {

		public Object[] getChildren(final Object parentElement) {

			final NetzCacheExtended strassenCache = StartupPlugin.getDefault()
					.getDefaultNetzCacheExtended();

			if (parentElement == root) {
				return new Object[] { BaustellenHelper.getInstanz(root),
						UnfallHelper.getInstanz(root),
						StauHelper.getInstanz(root) };

			} else if (parentElement instanceof BaustellenHelper
					|| parentElement instanceof UnfallHelper
					|| parentElement instanceof StauHelper) {

				int i = 0;
				final List<Strasse> strassen = strassenCache
						.getStrassenMitSegmenten();
				final StrassenHelper[] strassenHelper = new StrassenHelper[strassen
						.size()];
				for (final Strasse strasse : strassen) {
					strassenHelper[i] = StrassenHelper.getInstanz(
							parentElement, strasse);
					i++;
				}
				return strassenHelper;

			} else if (parentElement instanceof StrassenHelper) {

				final StrassenHelper strassenHelper = (StrassenHelper) parentElement;

				if (BauEditorPlugin.getDefault().isStrasseEindeutig()) {
					if (strassenHelper.getParent() instanceof BaustellenHelper) {
						final List<BaustellenWrapper> baustellenWrapper = BauEditorPlugin
								.getDefault()
								.getEngstellenCache()
								.getBaustellenWrapper(
										strassenHelper.getStrasse());
						if (baustellenWrapper != null) {
							return baustellenWrapper.toArray();
						}
					} else if (strassenHelper.getParent() instanceof UnfallHelper) {
						final List<UnfallWrapper> unfallWrapper = BauEditorPlugin
								.getDefault().getEngstellenCache()
								.getUnfallWrapper(strassenHelper.getStrasse());
						if (unfallWrapper != null) {
							return unfallWrapper.toArray();
						}
					} else if (strassenHelper.getParent() instanceof StauHelper) {
						final List<StauWrapper> stauWrapper = BauEditorPlugin
								.getDefault().getEngstellenCache()
								.getStauWrapper(strassenHelper.getStrasse());
						if (stauWrapper != null) {
							return stauWrapper.toArray();
						}
					}
				} else {
					final List<RichtungHelper> richtungsHelper = new ArrayList<RichtungHelper>();
					final List<StrassenKnoten> positiveKnoten = strassenCache
							.getStrassenKnoten(strassenHelper.getStrasse(),
									AttTmcRichtung.ZUSTAND_1_POSITIV);
					if (positiveKnoten != null && !positiveKnoten.isEmpty()) {
						richtungsHelper.add(RichtungHelper.getInstanz(
								strassenHelper,
								AttTmcRichtung.ZUSTAND_1_POSITIV));
					}
					final List<StrassenKnoten> negativeKnoten = strassenCache
							.getStrassenKnoten(strassenHelper.getStrasse(),
									AttTmcRichtung.ZUSTAND_1N_NEGATIV);
					if (negativeKnoten != null && !negativeKnoten.isEmpty()) {
						richtungsHelper.add(RichtungHelper.getInstanz(
								strassenHelper,
								AttTmcRichtung.ZUSTAND_1N_NEGATIV));
					}
					return richtungsHelper.toArray();
				}

			} else if (parentElement instanceof RichtungHelper) {

				final RichtungHelper richtungHelper = (RichtungHelper) parentElement;
				final StrassenHelper grandParentElement = (StrassenHelper) richtungHelper
						.getParent();
				if (grandParentElement.getParent() instanceof BaustellenHelper) {
					final List<BaustellenWrapper> baustellenWrapper = BauEditorPlugin
							.getDefault()
							.getEngstellenCache()
							.getBaustellenWrapper(richtungHelper.getStrasse(),
									richtungHelper.getRichtung());
					if (baustellenWrapper != null) {
						return baustellenWrapper.toArray();
					}
				} else if (grandParentElement.getParent() instanceof UnfallHelper) {
					final List<UnfallWrapper> unfallWrapper = BauEditorPlugin
							.getDefault()
							.getEngstellenCache()
							.getUnfallWrapper(richtungHelper.getStrasse(),
									richtungHelper.getRichtung());
					if (unfallWrapper != null) {
						return unfallWrapper.toArray();
					}
				} else if (grandParentElement.getParent() instanceof StauHelper) {
					final List<StauWrapper> stauWrapper = BauEditorPlugin
							.getDefault()
							.getEngstellenCache()
							.getStauWrapper(richtungHelper.getStrasse(),
									richtungHelper.getRichtung());
					if (stauWrapper != null) {
						return stauWrapper.toArray();
					}
				}
			}
			return null;
		}

		public Object getParent(final Object element) {
			if (element instanceof Helper) {
				return ((Helper) element).getParent();
			} else if (element instanceof EngstellenWrapper
					&& !BauEditorPlugin.getDefault().isStrasseEindeutig()) {
				final EngstellenWrapper wrapper = (EngstellenWrapper) element;
				return RichtungHelper.getInstanz(
						wrapper instanceof BaustellenWrapper,
						wrapper.getStrasse(), wrapper.getRichtung());
			} else if (element instanceof EngstellenWrapper
					&& BauEditorPlugin.getDefault().isStrasseEindeutig()) {
				final EngstellenWrapper wrapper = (EngstellenWrapper) element;
				return wrapper.getStrasse();
			}
			return null;
		}

		public boolean hasChildren(final Object element) {
			if (element == root || element instanceof BaustellenHelper
					|| element instanceof UnfallHelper
					|| element instanceof StauHelper
					|| element instanceof StrassenHelper) {
				return true;
			} else if (element instanceof RichtungHelper) {
				final RichtungHelper richtungHelper = (RichtungHelper) element;
				final StrassenHelper grandParentElement = (StrassenHelper) richtungHelper
						.getParent();
				if (grandParentElement.getParent() instanceof BaustellenHelper) {
					final List<BaustellenWrapper> baustellenWrapper = BauEditorPlugin
							.getDefault()
							.getEngstellenCache()
							.getBaustellenWrapper(richtungHelper.getStrasse(),
									richtungHelper.getRichtung());
					return baustellenWrapper != null
							&& !baustellenWrapper.isEmpty();
				} else if (grandParentElement.getParent() instanceof UnfallHelper) {
					final List<UnfallWrapper> unfallWrapper = BauEditorPlugin
							.getDefault()
							.getEngstellenCache()
							.getUnfallWrapper(richtungHelper.getStrasse(),
									richtungHelper.getRichtung());
					return unfallWrapper != null && !unfallWrapper.isEmpty();
				} else if (grandParentElement.getParent() instanceof StauHelper) {
					final List<StauWrapper> stauWrapper = BauEditorPlugin
							.getDefault()
							.getEngstellenCache()
							.getStauWrapper(richtungHelper.getStrasse(),
									richtungHelper.getRichtung());
					return stauWrapper != null && !stauWrapper.isEmpty();
				}
			}
			return false;
		}

		public Object[] getElements(final Object inputElement) {
			return getChildren(inputElement);
		}

		public void dispose() {
			// do nothing
		}

		public void inputChanged(final Viewer v, final Object oldInput,
				final Object newInput) {
			// do nothing
		}
	};

	/** Der LabelProvider des TreeViewers dieser Ansicht. */
	private final LabelProvider labelProvider = new LabelProvider() {

		@Override
		public String getText(final Object element) {
			if (element instanceof Helper) {
				return ((Helper) element).getName();
			} else if (element instanceof EngstellenWrapper) {
				final EngstellenWrapper wrapper = (EngstellenWrapper) element;
				final StringBuffer text = new StringBuffer(wrapper.getName());
				// TODO herausnehmen: nur zu Testzwecken eingebaut
				if (wrapper.getSituation() != null
						&& !wrapper.getSituation().getSystemObject().isValid()) {
					text.append(" (invalid)");
				}
				return text.toString();
			}
			return super.getText(element);
		}

		@Override
		public Image getImage(final Object element) {
			String imagePath = null;
			if (element instanceof Helper) {
				imagePath = ((Helper) element).getImagePath();
			} else if (element instanceof BaustellenWrapper) {
				imagePath = BildDateiPfade.BAUSTELLE;
			} else if (element instanceof UnfallWrapper) {
				imagePath = BildDateiPfade.UNFALL;
			} else if (element instanceof StauWrapper) {
				imagePath = BildDateiPfade.STAU;
			}
			if (imagePath != null) {
				return BauEditorPlugin.getDefault().getImage(imagePath);
			}
			return super.getImage(element);
		}
	};

	/** Der Straenfilter dieser Ansicht. */
	private final ViewerFilter filter = new ViewerFilter() {

		@Override
		public boolean select(final Viewer v, final Object parentElement,
				final Object element) {

			if (element instanceof StrassenHelper
					|| element instanceof RichtungHelper) {

				boolean ueberpruefeBaustellen = false;
				boolean ueberpruefeStaus = false;
				Strasse strasse = null;
				AttTmcRichtung richtung = null;

				if (element instanceof StrassenHelper) {
					ueberpruefeBaustellen = parentElement instanceof BaustellenHelper;
					ueberpruefeStaus = parentElement instanceof StauHelper;
					strasse = ((StrassenHelper) element).getStrasse();
				} else if (element instanceof RichtungHelper) {
					ueberpruefeBaustellen = ((StrassenHelper) parentElement)
							.getParent() instanceof BaustellenHelper;
					ueberpruefeStaus = ((StrassenHelper) parentElement)
							.getParent() instanceof StauHelper;
					final RichtungHelper richtungHelper = (RichtungHelper) element;
					strasse = richtungHelper.getStrasse();
					richtung = richtungHelper.getRichtung();
				}

				final EngstellenCache cache = BauEditorPlugin.getDefault()
						.getEngstellenCache();
				if (ueberpruefeBaustellen) {
					List<BaustellenWrapper> baustellenWrapper = null;
					if (richtung != null) {
						baustellenWrapper = cache.getBaustellenWrapper(strasse,
								richtung);
					} else {
						baustellenWrapper = cache.getBaustellenWrapper(strasse);
					}
					return baustellenWrapper != null
							&& !baustellenWrapper.isEmpty();
				} else if (ueberpruefeStaus) {
					List<StauWrapper> stauWrapper = null;
					if (richtung != null) {
						stauWrapper = cache.getStauWrapper(strasse, richtung);
					} else {
						stauWrapper = cache.getStauWrapper(strasse);
					}
					return stauWrapper != null && !stauWrapper.isEmpty();
				} else {
					List<UnfallWrapper> unfallWrapper = null;
					if (richtung != null) {
						unfallWrapper = cache.getUnfallWrapper(strasse,
								richtung);
					} else {
						unfallWrapper = cache.getUnfallWrapper(strasse);
					}
					return unfallWrapper != null && !unfallWrapper.isEmpty();
				}
			}
			return true;
		}
	};

	/** Der DoubleClickListener des TreeViewers dieser Ansicht. */
	private final IDoubleClickListener doubleClickListener = new IDoubleClickListener() {

		public void doubleClick(final DoubleClickEvent event) {
			final ISelection sel = viewer.getSelection();
			if (sel instanceof StructuredSelection) {
				final Object first = ((StructuredSelection) sel)
						.getFirstElement();

				if (first != null) {

					EngstellenVerwaltungPluginAktion aktion = null;
					if (first instanceof BaustellenWrapper) {
						aktion = baustelleBearbeitenAktion;
					} else if (first instanceof UnfallWrapper) {
						aktion = unfallBearbeitenAktion;
					} else if (first instanceof StauWrapper) {
						aktion = stauAnzeigenAktion;
					} else if (first instanceof Helper) {
						viewer.setExpandedState(first,
								!viewer.getExpandedState(first));
						return;
					}

					if (aktion != null && aktion.isEnabled()) {
						aktion.runWithEvent(null, first,
								EngstellenNavigatorViewPart.this);
					}
				}
			}
		}
	};

	@Override
	public void init(final IViewSite site, final IMemento mem)
			throws PartInitException {
		super.init(site, mem);
		root = "ROOT";
	}

	@Override
	public void erzeugePartControl(final Composite parent) {

		patternFiter = new PatternOrSubstringFilter();
		filteredTree = new FilteredTree(parent, SWT.SINGLE | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.BORDER, patternFiter, true);

		viewer = filteredTree.getViewer();
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(labelProvider);
		viewer.addDoubleClickListener(doubleClickListener);
		viewer.setSorter(new ViewerSorter() {
			@Override
			public int compare(final Viewer v, final Object e1, final Object e2) {
				if (e1 instanceof StrassenHelper
						&& e2 instanceof StrassenHelper) {
					return StartupPlugin
							.getDefault()
							.getDefaultNetzCacheExtended()
							.compareStrassen(
									((StrassenHelper) e1).getStrasse(),
									((StrassenHelper) e2).getStrasse());
				}
				if (e1 instanceof RichtungHelper
						&& e2 instanceof RichtungHelper) {
					final AttTmcRichtung r1 = ((RichtungHelper) e1)
							.getRichtung();
					final AttTmcRichtung r2 = ((RichtungHelper) e2)
							.getRichtung();
					return -1 * r1.compareTo(r2);
				}
				return super.compare(v, e1, e2);
			}
		});

		// setze ISelectionChangedListener
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (baustelleBearbeitenAktion != null) {
					baustelleBearbeitenAktion.selectionChanged(null, selection);
				}
				if (unfallBearbeitenAktion != null) {
					unfallBearbeitenAktion.selectionChanged(null, selection);
				}
				if (stauAnzeigenAktion != null) {
					stauAnzeigenAktion.selectionChanged(null, selection);
				}
			}
		});

		// finales Initialisieren der UI
		getSite().setSelectionProvider(viewer);
		SelektionsTransfer.getInstanz().addSelektionsListener(this);

		// Setzen des Hilfe-Kontexts
		DefaultHelpContext.registerDefaultHelpContext(viewer.getControl(),
				EngstellenNavigatorViewPart.class);
	}

	@Override
	public void preInitInUIThread() {
		setEnablement(false);
	}

	@Override
	public void initInUIThread() {
		if (getViewSite().getActionBars().getMenuManager().isEmpty()) {
			fuelleActionBars();
			erzeugeKontextMenu();
		}
		baustelleBearbeitenAktion = new BaustelleBearbeitenAktion();
		unfallBearbeitenAktion = new UnfallBearbeitenAktion();
		stauAnzeigenAktion = new StauAnzeigenAktion();
		if (viewer != null && viewer.getControl() != null
				&& !viewer.getControl().isDisposed()) {
			viewer.setInput(root);
		}
		setEnablement(true);
		setContentDescription();
		if (getCache() != null) {
			getCache().addEngstellenListener(this);
		}
	}

	/**
	 * Fllt das Men und die Symbolleiste dieser Ansicht mit Aktionen.
	 */
	private void fuelleActionBars() {

		aktualisierenAction = new AktualisierenAction();
		filterAction = new FilterLeereStrassenAction();
		zeigeTextFilterAction = new ShowTextFilterAction(filteredTree);
		final CollapseAllAction collapseAllAction = new CollapseAllAction(
				viewer);
		final ExpandAllAction expandAllAction = new ExpandAllAction(viewer);
		editorAction = new VerknuepfeMitEditorAction();
		editorAction.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(final PropertyChangeEvent event) {
				if (event.getSource() == editorAction
						&& event.getProperty() == IAction.CHECKED
						&& editorAction.isChecked()) {
					final IWorkbenchPage activePage = getSite()
							.getWorkbenchWindow().getActivePage();
					if (activePage == null) {
						return;
					}

					final IEditorPart editor = activePage.getActiveEditor();
					if (editor != null
							&& editor.getEditorInput() instanceof EngstellenEditorInput) {
						final EngstellenEditorInput input = (EngstellenEditorInput) editor
								.getEditorInput();
						final EngstellenWrapper element = input.getWrapper();
						if (element.isNeuesElement()) {
							return;
						}
						expandElement(element);
					}
				}
			}
		});
		verknuepfeAction = new VerknuepfeAction();

		final IActionBars bars = getViewSite().getActionBars();
		final IToolBarManager toolbar = bars.getToolBarManager();
		final IMenuManager menu = bars.getMenuManager();

		toolbar.add(aktualisierenAction);
		toolbar.add(new Separator());
		toolbar.add(expandAllAction);
		toolbar.add(collapseAllAction);
		toolbar.add(new Separator());
		toolbar.add(filterAction);
		toolbar.add(verknuepfeAction);
		toolbar.add(editorAction);

		menu.add(aktualisierenAction);
		menu.add(new Separator());
		menu.add(expandAllAction);
		menu.add(collapseAllAction);
		menu.add(new Separator());
		menu.add(filterAction);
		menu.add(zeigeTextFilterAction);
		menu.add(new Separator());
		menu.add(verknuepfeAction);
		menu.add(editorAction);

		// Initialisiere Check-State der Actions mit Hilfe des Mementos
		if (einstellungen != null) {
			filterAction.setChecked(einstellungen.isFilterActionChecked());
			zeigeTextFilterAction.setChecked(einstellungen
					.isTextFilterActionChecked());
			editorAction.setChecked(einstellungen.isEditorActionChecked());
			verknuepfeAction.setChecked(einstellungen
					.isVerknuepfeActionChecked());
		} else {
			filterAction.setChecked(true);
			zeigeTextFilterAction.setChecked(true);
			editorAction.setChecked(true);
			verknuepfeAction.setChecked(true);
		}
		filterAction.run();
		zeigeTextFilterAction.run();

		bars.updateActionBars();
	}

	/**
	 * Erzeugt das Kontextmen.
	 */
	private void erzeugeKontextMenu() {

		// create menu manager
		final MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(final IMenuManager mgr) {
				mgr.add(new Separator("gruppe_oben")); //$NON-NLS-1$
				mgr.add(new Separator("anlegen_gruppe"));
				mgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
				mgr.add(new Separator("gruppe_unten"));
			}
		});

		// create menu.
		final Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// register menu for extension.
		getSite().registerContextMenu(menuMgr, viewer);
	}

	@Override
	protected void clearViewer() {
		if (filteredTree != null && filteredTree.getViewer() != null
				&& filteredTree.getViewer().getControl() != null
				&& !filteredTree.getViewer().getControl().isDisposed()) {
			filteredTree.getViewer().setInput(null);
		}
	}

	@Override
	protected void setEnablement(final boolean enabled) {
		if (viewer != null && viewer.getControl() != null
				&& !viewer.getControl().isDisposed()) {
			viewer.getControl().setEnabled(enabled);
		}
		if (aktualisierenAction != null) {
			aktualisierenAction.setEnabled(enabled);
		}
	}

	/**
	 * Selektiert die gegebene Engstelle in der Ansicht. Hierfr wird ggf. die
	 * Baumstruktur ausgeklappt.
	 * 
	 * @param wrapper
	 *            die Engstelle
	 */
	public void expandElement(final EngstellenWrapper wrapper) {
		if (viewer != null && viewer.getControl() != null
				&& !viewer.getControl().isDisposed() && wrapper != null) {

			final RichtungHelper richtungElement = RichtungHelper.getInstanz(
					wrapper instanceof BaustellenWrapper, wrapper.getStrasse(),
					wrapper.getRichtung());

			if (richtungElement != null) {

				final StrassenHelper strassenElement = (StrassenHelper) richtungElement
						.getParent();
				final Helper helper = (Helper) strassenElement.getParent();
				final TreePath path = new TreePath(new Object[] { helper,
						strassenElement, richtungElement });

				viewer.setExpandedState(path, true);
			}
			viewer.setSelection(new StructuredSelection(wrapper));
			viewer.reveal(wrapper);
		}
	}

	/**
	 * Aktualisiert die Content-Description dieser Ansicht.
	 */
	private void setContentDescription() {
		if (getCache() == null || !getCache().isInitialisiert()) {
			setContentDescription(Zeichenketten.LADE_ENGSTELLEN + "...");
			return;
		}

		if (!BauEditorPlugin.getDefault().isAusfuehrbar()) {
			setContentDescription(Zeichenketten.FEHLERMELDUNG_NICHT_AUSFUEHRBAR);
			return;
		}

		final StringBuffer buf = new StringBuffer();
		final int anzahlBaustellen = getCache().getBaustellenWrapper().size();
		buf.append(anzahlBaustellen);
		buf.append(" Baustelle");
		if (anzahlBaustellen != 1) {
			buf.append("n");
		}
		buf.append(" und ");
		final int anzahlUnfaelle = getCache().getUnfallWrapper().size();
		buf.append(anzahlUnfaelle);
		if (anzahlUnfaelle == 1) {
			buf.append(" Unfall");
		} else {
			buf.append(" Unflle");
		}
		setContentDescription(buf.toString());
	}

	@Override
	public void setFocus() {
		if (viewer != null && viewer.getControl() != null
				&& !viewer.getControl().isDisposed()) {
			viewer.getControl().setFocus();
		}
	}

	/**
	 * Liefert die Selection des TreeViewers zurck.
	 * 
	 * @return die Selection des TreeViewers
	 */
	public ISelection getSelection() {
		if (viewer != null && !viewer.getTree().isDisposed()) {
			return viewer.getSelection();
		}
		return null;
	}

	/**
	 * Aktion zum Aktualisieren der Ansicht.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenNavigatorViewPart.java 24859 2010-07-19 15:12:41Z
	 *          meissner $
	 */
	private class AktualisierenAction extends Action {

		/**
		 * Der Konstruktor.
		 */
		public AktualisierenAction() {
			setText("Aktualisieren");
			setToolTipText("Aktualisieren");
			setImageDescriptor(BauEditorPlugin.getDefault().getImageDescriptor(
					BildDateiPfade.AKTUALISIEREN));
		}

		@Override
		public void run() {

			preInitInUIThread();
			setContentDescription(Zeichenketten.LADE_ENGSTELLEN + "...");
			if (viewer != null && viewer.getControl() != null
					&& !viewer.getControl().isDisposed()) {
				viewer.setInput(null);
			}

			final Job ladeJob = new Job(Zeichenketten.LADE_ENGSTELLEN) {
				@Override
				protected IStatus run(final IProgressMonitor monitor) {
					if (getCache() != null) {
						getCache().reInit();
					}
					return Status.OK_STATUS;
				}
			};
			ladeJob.schedule();
		}
	}

	/**
	 * Aktion zum Aktivieren bzw. Deaktivieren der Verknpfung dieser Ansicht
	 * mit Selektionen.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenNavigatorViewPart.java 24859 2010-07-19 15:12:41Z
	 *          meissner $
	 */
	private class VerknuepfeAction extends Action {

		/**
		 * Der Konstruktor.
		 */
		public VerknuepfeAction() {
			setText("Verknpfe Selektionen");
			setToolTipText("Verknpfe Selektionen");
			setImageDescriptor(BauEditorPlugin.getDefault().getImageDescriptor(
					BildDateiPfade.VERKNUEPFEN));
		}

		@Override
		public int getStyle() {
			return AS_CHECK_BOX;
		}
	}

	/**
	 * Aktion zum Aktivieren bzw. Deaktivieren der Verknpfung dieser Ansicht
	 * mit dem Editor.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenNavigatorViewPart.java 24859 2010-07-19 15:12:41Z
	 *          meissner $
	 */
	private class VerknuepfeMitEditorAction extends Action {

		/**
		 * Der Konstruktor.
		 */
		public VerknuepfeMitEditorAction() {
			setText("Verknpfe mit Editor");
			setToolTipText("Verknpfe mit Editor");
			setImageDescriptor(BauEditorPlugin.getDefault().getImageDescriptor(
					BildDateiPfade.VERKNUEPFE_MIT_EDITOR));
		}

		@Override
		public int getStyle() {
			return AS_CHECK_BOX;
		}
	}

	/**
	 * Aktion zum Aktivieren bzw. Deaktivieren eines Filters, welche smtliche
	 * leeren Straen(richtungs)-Knoten ausblendet
	 * 
	 * @author meissner
	 */
	private class FilterLeereStrassenAction extends Action {

		/**
		 * Der Konstruktor.
		 */
		public FilterLeereStrassenAction() {
			setText("Filter leere Straen");
			setToolTipText("Filter leere Straen");
			setImageDescriptor(BauEditorPlugin.getDefault().getImageDescriptor(
					BildDateiPfade.FILTER));

		}

		@Override
		public void run() {
			setFilters(isChecked());
		}

		@Override
		public int getStyle() {
			return AS_CHECK_BOX;
		}
	}

	@Override
	public void engstellenAdded(final Object quelle,
			final List<EngstellenWrapper> engstellen) {
		if (viewer != null && !viewer.getControl().isDisposed()) {
			for (final EngstellenWrapper wrapper : engstellen) {
				addEngstellenWrapper(wrapper);
			}
		}
	}

	@Override
	public void engstellenRemoved(final Object quelle,
			final List<EngstellenWrapper> engstellen) {
		if (viewer != null && !viewer.getControl().isDisposed()) {
			for (final EngstellenWrapper wrapper : engstellen) {
				removeEngstellenWrapper(wrapper);
			}
		}
	}

	@Override
	public void engstelleChanged(final Object quelle,
			final EngstellenWrapper neuerWert, final EngstellenWrapper alterWert) {

		// Die View wird nicht sofort aktualisiert. Es wird zunchst eine
		// bestimmte Zeit gewartet, um fr mglicherweise nacheinander
		// eintreffende Datensatz-Updates nur eine Aktualisierung durchzufhren.
		final Display display = getSite().getShell().getDisplay();
		if (display != null && !display.isDisposed()) {
			if (updateViewJobMap == null) {
				updateViewJobMap = new HashMap<String, UpdateViewJob>();
			}
			synchronized (updateViewJobMap) {
				UpdateViewJob updateViewJob = updateViewJobMap.get(neuerWert
						.getSituation().getPid());
				if (updateViewJob == null || updateViewJob.isDone()) {
					updateViewJob = new UpdateViewJob(display, alterWert,
							neuerWert);
					updateViewJobMap.put(neuerWert.getSituation().getPid(),
							updateViewJob);
					updateViewJob.schedule(Konstanten.UPDATE_TIMEOUT);
				} else {
					updateViewJob.setNeuerWrapper(neuerWert);
				}
			}
		}
	}

	/**
	 * UIJob, welcher den Editor aktualisiert.
	 */
	private final class UpdateViewJob extends UIJob {

		/**
		 * Die alte Engstelle, welcher durch die neue Engstelle ersetzt werden
		 * soll.
		 */
		private final EngstellenWrapper alterWert;

		/** Die aktuelle Engstelle. */
		private EngstellenWrapper neuerWert;

		/** Flag, welches angibt, ob der Job bereits ausgefhrt wurde. */
		private boolean done;

		/**
		 * Der Konstruktor.
		 * 
		 * @param jobDisplay
		 *            das Display
		 * @param alterWert
		 *            die alte Engstelle, welcher durch die neue Engstelle
		 *            ersetzt werden soll
		 * @param neuerWert
		 *            die neue Engstelle
		 */
		public UpdateViewJob(final Display jobDisplay,
				final EngstellenWrapper alterWert,
				final EngstellenWrapper neuerWert) {
			super(jobDisplay, "Aktualisiere Engstellen-Navigator");
			this.alterWert = alterWert;
			this.neuerWert = neuerWert;
			done = false;
		}

		/**
		 * Liefert <code>true</code> zurck, wenn der Job bereits ausgefhrt
		 * wurde, ansonsten <code>false</code>.
		 * 
		 * @return <code>true</code>, wenn der Job bereits ausgefhrt wurde,
		 *         ansonsten <code>false</code>
		 */
		public boolean isDone() {
			return done;
		}

		/**
		 * Setzt die neue Engstelle, welche dieser Editor anzeigen soll.
		 * 
		 * @param neuerWrapper
		 *            die Engstelle
		 */
		public void setNeuerWrapper(final EngstellenWrapper neuerWrapper) {
			neuerWert = neuerWrapper;
		}

		@Override
		public IStatus runInUIThread(final IProgressMonitor monitor) {
			if (monitor.isCanceled()) {
				return Status.CANCEL_STATUS;
			}

			if (viewer != null && !viewer.getControl().isDisposed()) {
				removeEngstellenWrapper(alterWert);
				addEngstellenWrapper(neuerWert);
			}
			done = true;
			return Status.OK_STATUS;
		}
	};

	/**
	 * Fgt die gegebenen Engstelle der Ansicht hinzu.
	 * 
	 * @param wrapper
	 *            die Engstelle
	 */
	private void addEngstellenWrapper(final EngstellenWrapper wrapper) {

		if (wrapper == null) {
			return;
		}

		final RichtungHelper parentElement = RichtungHelper.getInstanz(
				wrapper instanceof BaustellenWrapper, wrapper.getStrasse(),
				wrapper.getRichtung());
		final Display display = Display.getDefault();
		if (display != null && !display.isDisposed()) {
			display.syncExec(new AddWrapperToViewerRunnable(wrapper,
					parentElement));
		}
	}

	/**
	 * Runnable zum Hinzufgen von Engstellen zu dieser Ansicht.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenNavigatorViewPart.java 24859 2010-07-19 15:12:41Z
	 *          meissner $
	 */
	private final class AddWrapperToViewerRunnable implements Runnable {

		/** Die einzufgende Engstelle. */
		private final EngstellenWrapper wrapper;

		/** Das Parent-Objekt der einzufgenden Engstelle. */
		private final RichtungHelper parentElement;

		/**
		 * Der Konstruktor.
		 * 
		 * @param wrapper
		 *            die einzufgenden Engstelle
		 * @param parentElement
		 *            das Parent-Objekt der einzufgenden Engstelle
		 */
		private AddWrapperToViewerRunnable(final EngstellenWrapper wrapper,
				final RichtungHelper parentElement) {
			this.wrapper = wrapper;
			this.parentElement = parentElement;
		}

		public void run() {

			if ((viewer != null) && (!viewer.getControl().isDisposed())) {
				final boolean updateContentDescription = viewer
						.testFindItem(wrapper) == null;
				final TreePath[] expandedTreePaths = viewer
						.getExpandedTreePaths();
				viewer.add(parentElement, wrapper);
				viewer.refresh(parentElement);
				setFilters(filterAction.isChecked());
				if (updateContentDescription) {
					setContentDescription();
				}
				viewer.setExpandedTreePaths(expandedTreePaths);
			}
		}
	}

	/**
	 * Entfernt die gegebene Engstelle aus der Ansicht.
	 * 
	 * @param wrapper
	 *            die Engstelle
	 */
	private void removeEngstellenWrapper(final EngstellenWrapper wrapper) {

		if (wrapper == null) {
			return;
		}

		final RichtungHelper parentElement = RichtungHelper.getInstanz(
				wrapper instanceof BaustellenWrapper, wrapper.getStrasse(),
				wrapper.getRichtung());
		final Display display = Display.getDefault();
		if (display != null && !display.isDisposed()) {
			display.syncExec(new RemoveWrapperFromViewerRunnable(parentElement,
					wrapper));
		}
	}

	/**
	 * Runnable zum Entfernen von Engstellen aus der Ansicht.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenNavigatorViewPart.java 24859 2010-07-19 15:12:41Z
	 *          meissner $
	 */
	private final class RemoveWrapperFromViewerRunnable implements Runnable {

		/** Das Parent-Objekt der zu entfernenden Engstelle. */
		private final RichtungHelper parentElement;

		/** Die zu entfernende Engstelle. */
		private final EngstellenWrapper wrapper;

		/**
		 * Der Konstruktor.
		 * 
		 * @param parentElement
		 *            das Parent-Objekt der zu entfernenden Engstelle
		 * @param wrapper
		 *            die zu entfernende Engstelle
		 */
		private RemoveWrapperFromViewerRunnable(
				final RichtungHelper parentElement,
				final EngstellenWrapper wrapper) {

			this.parentElement = parentElement;
			this.wrapper = wrapper;
		}

		public void run() {
			final boolean updateContentDescription = viewer
					.testFindItem(wrapper) != null;
			final Object[] segments = { root, parentElement.getParent(),
					parentElement, wrapper };
			final TreePath path = new TreePath(segments);
			final TreePath[] expandedTreePaths = viewer.getExpandedTreePaths();
			viewer.remove(path);
			viewer.refresh(parentElement);
			setFilters(filterAction.isChecked());
			if (updateContentDescription) {
				setContentDescription();
			}
			viewer.setExpandedTreePaths(expandedTreePaths);
		}
	}

	/**
	 * Setzt die {@link ViewerFilter} dieser View.
	 * 
	 * @param filterStrassen
	 *            <code>true</code>, wenn leere Straen gefiltert werden sollen,
	 *            ansonsten <code>false</code>
	 */
	private void setFilters(final boolean filterStrassen) {
		if (filterStrassen) {
			viewer.setFilters(new ViewerFilter[] { patternFiter, filter });
		} else {
			viewer.setFilters(new ViewerFilter[] { patternFiter });
		}
	}

	@Override
	public PagePrint getDruckAuftrag() {
		final String header = getContentDescription();

		final GridPrint grid = new GridPrint();
		grid.addColumn(new GridColumn(GridColumn.DEFAULT_ALIGN,
				GridColumn.DEFAULT_SIZE, GridColumn.DEFAULT_WEIGHT));
		grid.add(new TextPrint(header, new FontData("Arial", 14, SWT.BOLD)));
		grid.add(new TextPrint("", new FontData("Arial", 10, SWT.NORMAL)));

		final GridPrint tabelle = new GridPrint();
		final DefaultGridLook look = new DefaultGridLook();
		look.setCellSpacing(DefaultGridLook.BORDER_OVERLAP, 3);
		tabelle.setLook(look);

		TextStyle textStyle = new TextStyle();
		textStyle = textStyle.font("Arial", 10, SWT.NORMAL);

		final GridColumn gridColumn = new GridColumn(SWT.LEFT, SWT.DEFAULT, 1);
		tabelle.addColumn(gridColumn);

		final List<Object> expandedElements = Arrays.asList(viewer
				.getExpandedElements());

		final BaustellenHelper baustellenHelper = BaustellenHelper
				.getInstanz(root);
		final UnfallHelper unfallHelper = UnfallHelper.getInstanz(root);
		printSubTree(tabelle, textStyle, expandedElements, baustellenHelper);
		printSubTree(tabelle, textStyle, expandedElements, unfallHelper);
		grid.add(tabelle);

		return new PagePrint(new BigPrint(grid));
	}

	/**
	 * Hilfsmethode zum Drucken eines Teilbaumes der Ansicht.
	 * 
	 * @param tabelle
	 *            die Tabelle
	 * @param textStyle
	 *            der TextStyle
	 * @param expandedElements
	 *            die aktuelle ausgeklappten Elemente der Baumansicht
	 * @param helper
	 *            der Knoten, welcher den Teilbaum aufspannt
	 */
	private void printSubTree(final GridPrint tabelle,
			final TextStyle textStyle, final List<Object> expandedElements,
			final Helper helper) {

		// Baustellen- bzw. Unflle-Knoten
		if (!filteredTree.getPatternFilter().isElementVisible(viewer, helper)) {
			return;
		}
		printTreeItem(tabelle, textStyle, helper);
		if (!expandedElements.contains(helper)) {
			return;
		}

		for (final Object strasse : contentProvider.getChildren(helper)) {

			// Straenknoten
			if (!filteredTree.getPatternFilter().isElementVisible(viewer,
					strasse)
					|| (filterAction.isChecked() && !filter.select(viewer,
							helper, strasse))) {
				continue;
			}
			printTreeItem(tabelle, textStyle, strasse);
			if (!expandedElements.contains(strasse)) {
				continue;
			}

			for (final Object richtung : contentProvider.getChildren(strasse)) {

				// Richtungsknoten
				if (!filteredTree.getPatternFilter().isElementVisible(viewer,
						richtung)
						|| (filterAction.isChecked() && !filter.select(viewer,
								strasse, richtung))) {
					continue;
				}
				printTreeItem(tabelle, textStyle, richtung);
				if (!expandedElements.contains(richtung)) {
					continue;
				}

				for (final Object engstelle : contentProvider
						.getChildren(richtung)) {

					// Baustellen- bzw. Unflle
					if (!filteredTree.getPatternFilter().isElementVisible(
							viewer, engstelle)) {
						continue;
					}
					printTreeItem(tabelle, textStyle, engstelle);
				}
			}
		}
	}

	/**
	 * Hilfsmethode zum Drucken eines einzelnen Knotens.
	 * 
	 * @param tabelle
	 *            die Tabelle
	 * @param textStyle
	 *            der TextStyle
	 * @param object
	 *            der Knoten
	 */
	private void printTreeItem(final GridPrint tabelle,
			final TextStyle textStyle, final Object object) {
		String indent = "";
		String text = object.toString();
		Image image = null;
		if (object instanceof BaustellenHelper) {
			image = BauEditorPlugin.getDefault().getImage(
					BildDateiPfade.BAUSTELLE);
		} else if (object instanceof UnfallHelper) {
			image = BauEditorPlugin.getDefault()
					.getImage(BildDateiPfade.UNFALL);
		} else if (object instanceof StrassenHelper) {
			indent = "        --  ";
			image = BauEditorPlugin.getDefault().getImage(
					BildDateiPfade.STRASSE);
		} else if (object instanceof RichtungHelper) {
			indent = "              --  ";
			image = BauEditorPlugin.getDefault().getImage(
					BildDateiPfade.RICHTUNG_NEGATIV);
		} else if (object instanceof EngstellenWrapper) {
			indent = "                    --  ";
			text = ((EngstellenWrapper) object).getName();
			if (object instanceof BaustellenWrapper) {
				image = BauEditorPlugin.getDefault().getImage(
						BildDateiPfade.BAUSTELLE);
			} else if (object instanceof UnfallWrapper) {
				image = BauEditorPlugin.getDefault().getImage(
						BildDateiPfade.UNFALL);
			}
		}
		tabelle.add(createCell(textStyle, text, indent, image));
	}

	/**
	 * Hilfsmethode zum Drucken einer einzelnen Zelle mit Bild und Text.
	 * 
	 * @param textStyle
	 *            der TextStyle
	 * @param text
	 *            der Text
	 * @param indent
	 *            der Einschub
	 * @param image
	 *            das Bild
	 * 
	 * @return das druckbare Element
	 */
	private Print createCell(final TextStyle textStyle, final String text,
			final String indent, final Image image) {
		if (image == null) {
			return textStyle.create(text);
		}

		final GridPrint grid1 = new GridPrint("p, d");
		grid1.add(textStyle.create(indent));
		final GridPrint grid2 = new GridPrint("p, d");
		grid2.add(SWT.LEFT, SWT.CENTER, new ImagePrint(image.getImageData()));
		grid2.add(SWT.LEFT, SWT.BOTTOM, textStyle.create(" " + text));
		grid1.add(grid2);
		return grid1;
	}

	@Override
	public String getTitel() {
		return "Ansicht Engstellen-Navigator";
	}

	@Override
	public void selectionChanged(final IWorkbenchPart part,
			final ISelection selection) {

		if (part == this) {
			return;
		}
		if (part instanceof EngstellenEditor) {
			if (editorAction == null || !editorAction.isChecked()) {
				return;
			}
		} else if (verknuepfeAction == null || !verknuepfeAction.isChecked()) {
			return;
		}

		Object selectedElement = null;
		if (viewer != null && viewer.getControl() != null
				&& !viewer.getControl().isDisposed()) {
			final ISelection currentSelection = viewer.getSelection();
			if (currentSelection != null && !currentSelection.isEmpty()
					&& currentSelection instanceof IStructuredSelection) {
				selectedElement = ((IStructuredSelection) currentSelection)
						.getFirstElement();
			}
		}

		final EngstellenWrapper wrapperToSelect = getEngstellenWrapper(selection);
		if (wrapperToSelect != selectedElement) {
			new ExpandElementRunnable(wrapperToSelect).run();
		}
	}

	/**
	 * Runnable zum Selektieren einer Engstelle im Navigator.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenNavigatorViewPart.java 24861 2010-07-19 17:37:17Z
	 *          meissner $
	 */
	private class ExpandElementRunnable implements Runnable {

		/** Die zu selektierende Engstelle. */
		private final EngstellenWrapper wrapperToSelect;

		/**
		 * Der Konstruktor.
		 * 
		 * @param wrapper
		 *            der EngstellenWrapper, welcher selektiert werden soll
		 */
		public ExpandElementRunnable(final EngstellenWrapper wrapper) {
			wrapperToSelect = wrapper;
		}

		@Override
		public void run() {
			final Display display = Display.getDefault();
			if (display != null && !display.isDisposed()) {
				display.syncExec(new Runnable() {
					@Override
					public void run() {
						expandElement(wrapperToSelect);
					}
				});
			}
		}
	}

	@Override
	protected void ladeEinstellungen() {
		if (BauEditorPlugin.getDefault().isAusfuehrbar()) {
			final JavaModellEinstellungen<EngstellenNavigatorViewPartEinstellungen> e = new JavaModellEinstellungen<EngstellenNavigatorViewPartEinstellungen>(
					EngstellenNavigatorViewPartEinstellungen.class);
			einstellungen = e.getModellEinstellungen(
					EinstellungsArt.benutzerNetzweit, ID);
			if (einstellungen == null) {
				einstellungen = e.getModellEinstellungen(
						EinstellungsArt.benutzerLokal, ID);
			}
		}
		if (einstellungen == null) {
			einstellungen = new EngstellenNavigatorViewPartEinstellungen();
		}
	}

	@Override
	protected void speicherEinstellungen() {
		if (BauEditorPlugin.getDefault().isAusfuehrbar()
				&& editorAction != null && verknuepfeAction != null
				&& filterAction != null && zeigeTextFilterAction != null) {

			final EngstellenNavigatorViewPartEinstellungen einstell = new EngstellenNavigatorViewPartEinstellungen();
			einstell.setEditorActionChecked(editorAction.isChecked());
			einstell.setVerknuepfeActionChecked(verknuepfeAction.isChecked());
			einstell.setFilterActionChecked(filterAction.isChecked());
			einstell.setTextFilterActionChecked(zeigeTextFilterAction
					.isChecked());

			final JavaModellEinstellungen<EngstellenNavigatorViewPartEinstellungen> e = new JavaModellEinstellungen<EngstellenNavigatorViewPartEinstellungen>(
					EngstellenNavigatorViewPartEinstellungen.class);
			e.setModellEinstellungen(EinstellungsArt.benutzerLokal, ID,
					einstell);
			e.setModellEinstellungen(EinstellungsArt.benutzerNetzweit, ID,
					einstell);
		}
	}

	@Override
	public void dispose() {
		if (updateViewJobMap != null) {
			synchronized (updateViewJobMap) {
				for (final String key : updateViewJobMap.keySet()) {
					UpdateViewJob job = updateViewJobMap.get(key);
					if (job != null) {
						job.done(Status.CANCEL_STATUS);
						job = null;
					}
					updateViewJobMap.remove(key);
				}
			}
		}
		super.dispose();
	}
}
