/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.ablauf;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.IViewSite;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;

/**
 * Darstellung des Fensters zur Parameternderung der Parameter Ablauf-TSF (Stufe)
 * @author Pittner
 */
/**
 * @author Pittner
 *
 */
public class CompositeParameterStufen extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = true;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe TSF-Ablauf
   */
  private ResultData _resultData;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit dem Spinner fr die Anzahl Stufen
   */
  private Composite _compositeSpinnerStufen = null;

  /**
   * Composite mit den Reitern fr die Stufen
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Composites mit den Darstellungen
	 */
	private CompositeParameterDarstellung[] _compositeParameterDarstellung;


  /**
	 * Anzahl der zu versorgenden Fahrstreifen
	 */
	private int _anzahlStufen;

	/**
	 * Spinner Anzahl Fahrstreifen
	 */
	private Spinner _spinnerAnzahlStufen;

	
	/**
	 * Groups
	 */
	private Group groupStufen;
  private Group groupProgramm;

  /**
   * Anzahl der Stufen
   */
  private int _indexPrg;

  /**
   * Auswahl der Programme
   */
  private Combo _combo;

  private IViewSite _viewSite;
	
	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultData Ergebnis Stufen 
	 */
  public CompositeParameterStufen(IViewSite viewSite, Composite parent, int style, ResultData resultData, int index)
  {
    super(parent, style);   
    this._viewSite = viewSite;
    this._resultData = resultData;
    this._indexPrg = index;
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    _list = new ArrayList<String>();

    createComboProgramm();
    createSpinnerAnzahlStufen();

    createCompositeTab();

    int anzahl = bestimmeAnzahlStufen();
    erzeugeBenoetigteReiter(anzahl);
        
    updateData ();    
  }

  /**
   * Methode bestimmt die Anzahl der Stufen aus dem Ergebnis des DaV
   * @return Anzahl Stufen
   */
  private int bestimmeAnzahlStufen ()
  {
    int anzahl = 0;
    if (_resultData != null && _resultData.hasData())
    {
    	Data data = _resultData.getData();
    	Array array = data.getArray("Programme");    	 
    	if (array.getLength() > _indexPrg)
      {
    	  anzahl = array.getItem(_indexPrg).getItem("Stufen").asArray().getLength();    	  
      }
    }
    
    return anzahl;
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultData != null){
    	updataDataAlgorithmus ();
    	updateDataProgramme();
    }
  }

  /**
   * Aktualisiert die ausgewaehlten Programme
   */
  private void updateDataProgramme()
  {
    if (_resultData.hasData()){
      Data data = _resultData.getData();
      
      Array array = data.getArray("Programme");      
      if (array.getLength() > _indexPrg)
      {
        String strPrg = array.getItem(_indexPrg).getItem("Programm").asTextValue().getValueText();                
        String[] items = _combo.getItems();
        int index = 0;
        for (int i = 0; i < items.length; i++)
        {
          String s = items[i];
          if (s.equals(strPrg)){
            _combo.setText(s);
            break;
          }        
        }
      }            
    }
  }

  /**
   * Methode legt die bentigte Anzahl der Reiter an
   * @param anzahl Anzahl bentigter Reiter (= Anzahl Stufen)
   */
  private void erzeugeBenoetigteReiter (int anzahl)
  {
  	if (anzahl == _anzahlStufen)
  		return;
  	
  	/*
  	 * Beim ersten Anlegen der Tabelle wird Reiter 0 markiert, bei nderung der Anzahl der
  	 * Reiter ber den Spinner wird der letzten Reiter markiert.
  	 */
  	int indexAnwahl = 0;
  	if (_compositeParameterDarstellung != null)
  		indexAnwahl = anzahl - 1;

		if (_spinnerAnzahlStufen != null)
			_spinnerAnzahlStufen.setSelection(anzahl);
		
		/*
		 * Namen der Reiter anpassen
		 */
		_list.clear();
    for (int i=0; i<anzahl; ++i)
      _list.add ("Stufe " + (i+1));
    
    /*
     * Bisherige Composite temp. speichern
     */
		Vector<CompositeParameterDarstellung> tmp = new Vector<CompositeParameterDarstellung>();
		if (_compositeParameterDarstellung != null)
			for (CompositeParameterDarstellung cpa : _compositeParameterDarstellung)
				tmp.add(cpa);

		/*
		 * Nicht bentigte Reiter entfernen
		 */
	  CTabItem[] items = _tabFolder.getItems();
	  for (int i=items.length - 1; i >= anzahl; i--)
		  items [i].dispose();

		/*
		 * Feld mit den Composites erweitern bzw. reduzieren
		 */
  	_compositeParameterDarstellung = new CompositeParameterDarstellung [_list.size()];
  	for (int i=0; i<tmp.size() && i<anzahl; ++i)
  		_compositeParameterDarstellung [i] = tmp.get(i);
		
		/*
		 * Reiter hinzufgen
		 */
		if (anzahl > _anzahlStufen)
		{
		  for (int i=_anzahlStufen; i<anzahl; ++i)
		  {
		  	erzeugeTabItem(_list.get(i));
		  	updataDataAlgorithmus (i);
		  }
		}
    
	  _anzahlStufen = anzahl;

	  if (indexAnwahl >= 0)
  	  _tabFolder.setSelection( indexAnwahl );

    _tabFolder.redraw();
  }

  /**
   * Updatefunktion fr die Stufen
   */
  private void updataDataAlgorithmus()
  {
//    int indexStufe = _tabFolder.getSelectionIndex();
    
    for (int index = 0; index < _tabFolder.getItems().length; ++index){
//      if (indexStufe == index)
        updataDataAlgorithmus(index);
    }
  }

  /**
   * Updatefunktion fr die Stufen
   */
  private void updataDataAlgorithmus (int indexStufe)
  {
    Data data = null;
    if (_resultData != null && _resultData.hasData())
    {
      data = _resultData.getData();

			CTabItem[] items = _tabFolder.getItems();
	    CTabItem item = items [indexStufe];
	    
	    if (item != null )
	    {
        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterDarstellung compPar = null;

        Data aktuell;

        Array array = data.getArray("Programme");      
        if (array.getLength() > _indexPrg)
        {
          Array a = array.getItem(_indexPrg).getItem("Stufen").asArray();
          if (a.getLength() > indexStufe)
            aktuell = array.getItem(_indexPrg).getItem("Stufen").asArray().getItem(indexStufe);
          else
            aktuell = null;          
        }
        
        if ( children.length == 0 )
        {
          compPar = new CompositeParameterDarstellung(_viewSite, c, SWT.NONE, _indexPrg, indexStufe, _resultData);
          _compositeParameterDarstellung [indexStufe] = compPar;
          compPar.layout( true );
          c.layout( true );         
        }
        else
        {
          compPar = (CompositeParameterDarstellung) children[0]; 
          compPar.layout( true );
          c.layout( true );
        }
      }
    }
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Messquerschnitte
   * 
   */
  private void createSpinnerAnzahlStufen()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    groupStufen = new Group(_composite, SWT.NONE);
    groupStufen.setText("Anzahl Stufen");
    groupStufen.setLayout(new GridLayout(1, false));
    groupStufen.setLayoutData(gridData2);

    _compositeSpinnerStufen = new Composite(groupStufen, SWT.NONE);
    _compositeSpinnerStufen.setLayout(new GridLayout(1, false));

  	_spinnerAnzahlStufen = new Spinner(_compositeSpinnerStufen, SWT.BORDER);
  	_spinnerAnzahlStufen.setSelection(_anzahlStufen);
  	
  	_spinnerAnzahlStufen.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (e.getSource() instanceof Spinner)
				{
					Spinner sp = (Spinner) e.getSource();
					int selection = sp.getSelection();					
					erzeugeBenoetigteReiter (selection);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
  }
  
  private void createComboProgramm()
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;
    
    groupProgramm = new Group(_composite, SWT.NONE);
    groupProgramm.setText("Programm");
    groupProgramm.setLayout(new GridLayout(1, false));
    groupProgramm.setLayoutData(gdTableItem);
    
    
    _combo = new Combo( groupProgramm, SWT.BORDER | SWT.READ_ONLY ); 
    _combo.setLayoutData( gdTableItem );

    // Moegliche Zustaende
    _combo.add("Keine");
    _combo.add("Freigabe");
    _combo.add("Sperrung");
    _combo.add("Schnellsperrung");
    _combo.add("FreigabeArgoscan");
    _combo.add("Abbrechen");
  }
  

  public Combo getCombo()
  {
    return _combo;
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 300;

    _tabFolder = new CTabFolder(groupStufen, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktualisiereViewer();
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  		erzeugeTabItem(list.get(i));

  	_tabFolder.setSelection( 0 );
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  private void erzeugeTabItem (String name)
  { 	
    String so = name;
    
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

    String tabName = so;
    tabItem.setText( tabName );
    tabItem.setData( tabName, so );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );
  }

  /**
   * Methode liefert die einzelnen Composite fr die Stufen zurck
   * @return Composites
   */
  public CompositeParameterDarstellung[] getCompositeParameterStauBeleg ()
  {
  	return _compositeParameterDarstellung;
  }
  
  /**
   * @return die aktuelle Stufe
   */
  public int getIndexStufe()
  {
    return _indexPrg;
  }

//  /**
//   * Setzt das aktuelle Programm
//   * @param indexPrg
//   */
//  public void setIndexPrg(int indexPrg)
//  {
//    _indexPrg = indexPrg;
//    updataDataAlgorithmus();
//  }

  /**
   * Aktualisiert die Viewer der Darstellung
   */
  public void aktualisiereViewer()
  {
//    CTabItem itemSel = _tabFolder.getSelection();        
//    int indexStufe = _tabFolder.getSelectionIndex();
    
    
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem itemSel = items[i];
      Composite cSel = (Composite)itemSel.getControl();
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeParameterDarstellung compParSel = (CompositeParameterDarstellung)cSel.getChildren()[0];
//        compParSel.setIndexStufe(i);
      }
    }
    
//    if (itemSel != null){      
//      Composite cSel = (Composite)itemSel.getControl();
//      if (((Composite)cSel).getChildren().length > 0)
//      {
//        CompositeParameterDarstellung compParSel = (CompositeParameterDarstellung)cSel.getChildren()[0];
//        compParSel.setIndexStufe(indexStufe);
//      }
//    }
    
  }
  
  /**
   * @return
   */
  public CompositeParameterDarstellung[] getCompositeParameterDarstellung()
  {
    return _compositeParameterDarstellung;
  }
  
  public ResultData getResultData()
  {
    return _resultData;
  }

  public void setResultData(ResultData resultData)
  {
    _resultData = resultData;
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem item = items[i];
      Composite cSel = (Composite)item.getControl();
      CompositeParameterDarstellung tabComp = (CompositeParameterDarstellung)cSel.getChildren()[0];
      tabComp.setResultData(_resultData);          
    }
  }

  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  public void setTabFolder(CTabFolder tabFolder)
  {
    _tabFolder = tabFolder;
  }
  
  
  @Override
  public void dispose()
  {
    super.dispose();
    System.err.println("---> CompositeParameterStufen.dispose()");
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem item = items[i];
      if (!item.isDisposed()){
        Composite cSel = (Composite)item.getControl();
        CompositeParameterDarstellung tabComp = (CompositeParameterDarstellung)cSel.getChildren()[0];
        if (!tabComp.isDisposed())
          tabComp.dispose();
        tabComp = null;
        item.dispose();
      }
      item = null;
    }
  }
  
}
