/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung;



import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.parameter.tsf.Activator;
import de.bsvrz.buv.plugin.parameter.tsf.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.tsf.common.DataHelper;
import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;


/**
 * Fenster zur nderung der Parameter fr MessQuerschnitte zur Harmonisierung.
 * 
 * @author Krause
 * 
 */
public class MqZuweisungViewParameter3 extends ViewPart implements IDaVDataUpdateListener
{

  public static final String ID = "de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung.viewParameter3";

  private static final boolean _isDebug = false;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaUeEngstelle";
  
    
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  private Composite _parent;
  
  private CTabFolder _tabFolder = null;

  private Composite _composite = null;

  private Composite _compositeBottom;

  private Button _buttonOk;

  private Button _buttonUebernehemen;

  private Button _buttonAbbrechen;

  private Button _buttonHilfe;

  private IMemento _memento = null;

  private String _secondViewId;

  private ClientDavInterface _connection;

  // private Map<String, Map<String, ResultData>> _mapRcvNeu = new
  // HashMap<String, Map<String, ResultData>>();

  private boolean _sperreFrNeueDaten = false;

  private int _tabCnt = 0;

  /**
   * alle Messquerschnitte
   */
  private List<String> _listObjekte = null;

  /**
   * alle noch mglichen Messquerschnitte je Pid
   */
  private Map<String, List<String>> _mapPidObjList = null;

  @Override
  public void createPartControl( Composite parent )
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: createPartControl" );

    _parent = parent;

//    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".mqZuordnungParameter" );

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );

    // Darstellung aufbauen
    createCompositeTab();
    createCompositeBottom();

    restoreState();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    List<SystemObject> list = new ArrayList<SystemObject>();

    if ( _secondViewId.contains( ";" ) )
    {

      String[] split = _secondViewId.split( ";" );

      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject( s );
        list.add( so );
      }

    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject( _secondViewId );
      list.add( so );
    }

    if ( _isDebug )
      System.out.println( "Listener anmelden!" );

    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener( this );

    if ( _isDebug )
    {
      System.out.println( "Anzahl Tabs:" + list.size() );

      for (SystemObject so : list)
      {
        System.out.println( "\tPids: " + so.getPid() );
      }
    }
    DaVKommunikation.getInstanz().anmeldeDaten( list, _attributgruppe, "asp.parameterSoll" );

    // Reiter erstellen
    erzeugeTabItems( list );
  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * This method initializes compositeTab (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {
    _mapPidObjList = new HashMap<String, List<String>>();
    
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);
    
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {
    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = so.getName();
      tabItem.setText( tabName );
      tabItem.setData( tabName, so.getPid() );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }

  

  /**
   * This method initializes compositeBottom (ButtonLeiste)
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

    _buttonOk = new Button( _compositeBottom, SWT.NONE );
    _buttonOk.setText( "Ok" );
    _buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    _buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    _buttonUebernehemen.setText( "bernehmen" );
    _buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    _buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    _buttonAbbrechen.setText( "Abbrechen" );
    _buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    _buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    _buttonHilfe.setText( "Hilfe" );
    _buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }


	  protected void aktionHilfe()
	  {
		String contextId = Activator.PLUGIN_ID + ".mqZuordnungParameter";  
	    _debug.fine("ContextID: " + contextId);
	    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
	  }

  protected void aktionSchalten()
  {
    if ( schaltenUndParametrieren() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  protected void aktionUebernehmen()
  {
    if ( schaltenUndParametrieren() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {

    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento2" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento2", _secondViewId );
    super.saveState( memento );
  }

 
  
  /**
   * Schalten und Parametrieren im DAV durchfhren
   * 
   * @return
   */
  private Boolean schaltenUndParametrieren()
  {
    DataModel dataModel = _connection.getDataModel();

    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );
    AttributeGroup atg = dataModel.getAttributeGroup( _attributgruppe );
    
//    List<SystemObject> elements = dataModel.getType( "typ.sswSbaUeSeitenstreifen" ).getElements();

    Map<MqZuweisungCompositeParameter3, ResultData> map = new LinkedHashMap<MqZuweisungCompositeParameter3, ResultData>();

    CTabItem[] items = _tabFolder.getItems();

    for (CTabItem ti : items)
    {
      Composite c = (Composite) ti.getControl();
    	
      String pid = (String) ti.getData( ti.getText() );
   // Objekt zur Pid ermitteln
      SystemObject so = dataModel.getObject( pid );
      
      Data data = _connection.createData( atg );
      MqZuweisungCompositeParameter3 compPar = (MqZuweisungCompositeParameter3) c.getChildren()[0];
      
      EngstelleWrapper engstelleWrapper = compPar.getWrapper();
      List<SystemObject> itemsSo = engstelleWrapper.getListStrasseSegment();
      
      Array array = data.getItem( "StraenSegment" ).asArray();
      array.setLength( itemsSo.size() );
      for (int j = 0; j < itemsSo.size(); j++)
      {
        SystemObject soSs = itemsSo.get(j);
        array.getItem(j).asReferenceValue().setSystemObject(soSs);
      }
      
      System.out.println("Wrapper: " + engstelleWrapper.toString());
      data.getItem("StartOffset").asScaledValue().set(engstelleWrapper.getStartOffset());                              
      data.getItem("EndOffset").asScaledValue().set(engstelleWrapper.getEndOffset());
      data.getItem("Lnge").asScaledValue().set(engstelleWrapper.getLaenge());                              
      
      System.out.println("---> Data: " + data.toString());
      
      if (!DataHelper.isEqual(data, compPar.getResultData().getData())) {
	      ResultData rd = new ResultData( so, new DataDescription( atg, asp ), new Date().getTime(), data );
	      map.put( compPar, rd );
      }    
    }

//    UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
//    Boolean schlieenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();
    
    if (!map.isEmpty()) {
    	UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
    	        .getDisplay().getActiveShell());
    	            
	    UrlasserDialog.UrlasserDialogInfo anzeigen = uld.anzeigen(map.values());
	    boolean eingabeOk = anzeigen.getEingabeOk();
	        
	    if (eingabeOk && (anzeigen.getFehlerInfo() == null)) {
	    	for (Map.Entry<MqZuweisungCompositeParameter3, ResultData> entry: map.entrySet()) {
	    		entry.getKey().setResultData(entry.getValue());
	    	}
	    	return true;
	    }
	    else {
	    	return false;
	    }
    }
    else {
    	return true;
    }

//    return schlieenOk;

  }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.buv.plugin.automatikprogramme.daVDataUpdateListener.IDaVDataUpdateListener#daVDataUpdate(de.bsvrz.buv.plugin.automatikprogramme.daVDataUpdateListener.DaVDataUpdateEvent)
   */
  @Override
  public void daVDataUpdate( DaVDataUpdateEvent e )
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: daVDataUpdate" );

    if ( e != null )
    {
      if ( !_sperreFrNeueDaten )
        verarbeiteDavDaten( e );
      else
      {
        if ( _isDebug )
          System.out.println( "Neue Daten -> kein Update, da Sperre fr neue Daten aktiv" );
      }
    }
  }

  private void verarbeiteDavDaten( DaVDataUpdateEvent e )
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: VerarbeiteDavDaten" );

    if (e.getResult() != null && e.getResult().getDataDescription().getAttributeGroup().getPid().equals(_attributgruppe)){
      SystemObject so = e.getResult().getObject();
      
      // einmalig alle mglichen MessQuerschnitte ermitteln
      if ( _listObjekte == null )
      {
        List<SystemObject> elements = _connection.getDataModel().getType( "typ.sswSbaUeSeitenstreifen" ).getElements();
        _listObjekte = new ArrayList<String>();
//      for (SystemObject soMQ : elements)
//      {
//        _listObjekte.add( soMQ.getName() );
//      }
      }
      
      // Reiter zur Pid ermitteln
      CTabItem[] items = _tabFolder.getItems();
      CTabItem itemPid = null;
      for (CTabItem ti : items)
      {
        String pid = (String) ti.getData( ti.getText() );
        
        if ( pid == so.getPid() )
        {
          itemPid = ti;
          break;
        }
      }
      
      if ( _isDebug )
        System.out.println( "Anzeige Item: " + itemPid + "\nPidAtg: "
            + e.getResult().getDataDescription().getAttributeGroup().getPid() );
      
      if ( itemPid != null )
      {
        Composite c = (Composite) itemPid.getControl();
        Control[] children = c.getChildren();
        MqZuweisungCompositeParameter3 compPar;
        
        _tabCnt++;
        
        if ( children.length == 0 )
        {
          compPar = new MqZuweisungCompositeParameter3(c, SWT.None, _listObjekte, e.getResult());
          compPar.layout( true );
          c.layout( true );
        }
        else
        {
          compPar = (MqZuweisungCompositeParameter3) children[0];
          // compPar.layout( true );
        }
        
        ResultData rd = e.getResult();
        EngstelleWrapper wrapper = compPar.getWrapper(); 
        
        if (rd.hasData())
        {
          Data data = rd.getData();
          Array array = data.getArray("StraenSegment");
          for (int i = 0; i < array.getLength(); i += 1)
          {
            ReferenceValue ref = array.getItem(i).asReferenceValue();
            // ReferenceValue ref = array.getReferenceValue(i);
            SystemObject refSo = ref.getSystemObject();
            if (refSo != null)
            {
              if (!wrapper.getListStrasseSegment().contains(refSo))
                wrapper.getListStrasseSegment().add(refSo);
            }
          }
          wrapper.setStartOffset(data.getItem("StartOffset").asUnscaledValue().longValue());
          wrapper.setEndOffset(data.getItem("EndOffset").asUnscaledValue().longValue());
          wrapper.setLaenge(data.getItem("Lnge").asUnscaledValue().longValue());
          compPar.updateData(wrapper);
        }
        
        // Wenn alle Reiter versorgt sind, Sperre fr Neue Daten setzen
        if ( _tabCnt == _tabFolder.getItemCount() )
        {
          _sperreFrNeueDaten = true;
          if ( _isDebug )
            System.out.println( "Sperre fr neue Daten ist aktiv" );
        }
        
      }
      else
      {
        System.out.println( "nicht gefunden: " + so.getPid() );
      }
      
    }
    

  }

  @Override
  public void dispose()
  {
    super.dispose();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener( this );
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }

}
