/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.parameter.ufd.dav;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Klasse enthlt allgemeine Methoden
 * @author ObertM
 */
public class Tools
{
	/**
	 * Text fr Zustand "undefiniert"
	 */
	public static String _undefiniert = "undefiniert";
	
	/**
	 * Methode liefert eine Liste mit allen Systemobjekttypen zu einem bergebenen Basistyp.
	 * Dabei werden automatisch die mglichen Subtypen bestimmt.
	 * @param pidTyp Pid des Basistyps
	 * @return Liste mit Typen (Basistyp + Subtyp)
	 */
  public static List<SystemObjectType> getSystemObjektTypen (String pidTyp)
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    
    List<SystemObjectType> liste = new ArrayList<SystemObjectType>();
    
    SystemObjectType sot = connection.getDataModel().getType(pidTyp);
    bestimmeObjektTypen(sot, liste);
    
//    for (SystemObjectType s : liste)
//    	System.out.println("Tools.getSystemObjektTypen() " + s.getPid());
    
    return liste;
  }
  
  /**
   * Methode bestimmt rekursiv die vorhandenen Objekttypen
   * @param sot ausgehendes Objekt
   * @param liste Liste in die die Objekte eingetragen werden
   */
  private static void bestimmeObjektTypen(SystemObjectType sot, List<SystemObjectType> liste)
  {
    if ( sot.getSubTypes().size() == 0 )
    {
    	liste.add(sot);
    }
    else
    {
    	liste.add(sot);
    	
      for (SystemObjectType subTyp : sot.getSubTypes())
      	bestimmeObjektTypen(subTyp, liste);
    }
  }
  
  /**
   * Methode liefert alle Objekte die zu einem Systemobjekttyp gehren. Dabei werden auch
   * eventuelle Subtypen des bergebenen Typs bercksichtigt.
   * @param pidTyp Pid des Objekttyps
   * @return Liste mit Objekten
   */
  public static List<SystemObject> getSystemObjekte (String pidTyp)
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    
    List<SystemObjectType> typen = getSystemObjektTypen(pidTyp);
    
    List<SystemObject> objekte = new ArrayList<SystemObject>();
    
    for (SystemObjectType sot : typen)
    {
  	  List<SystemObject> elemente = connection.getDataModel().getType( sot.getPid() ).getElements();
  	  for (SystemObject so : elemente)
  	  {
  	  	if (!so.getPid().toUpperCase().contains("PUA"))
	  	    if (!objekte.contains(so))
	  	    	objekte.add(so);
  	  }
    }
    
    return objekte;
  }
  
  /**
   * Methode bestimmt eine Liste mit den alphabetisch sortieren Namen der bergebenen Objekte.
   * Zustzlich kann der Text "undefiniert" mit einsortiert werden (bei optionalen Referenzen)
   * @param objekte Liste mit Systemobjekten
   * @param mitUndefiniert Merker ob der Text undefiniert mit ausgegeben werden soll
   * @return Liste mit alphabetisch sortierten Namen
   */
  public static List<String> bestimmeObjektNamen (List<SystemObject> objekte, boolean mitUndefiniert)
  {
  	List<String> l  = new ArrayList<String>();
  	
    for (SystemObject so: objekte)
    	l.add( getName(so) );

    Collections.sort( l );
    if (mitUndefiniert)
      l.add(0, _undefiniert);
    
    return l;
  }

  /**
   * Methode bestimmt eine Liste mit den alphabetisch sortieren Namen der bergebenen Objekte.
   * @param objekte Liste mit Systemobjekten
   * @return Liste mit alphabetisch sortierten Namen
   */
  public static List<String> bestimmeObjektNamen (List<SystemObject> objekte)
  {
  	return bestimmeObjektNamen(objekte, false);
  }
  
  /**
   * Methode liefert eine Liste mit SystemObjekten aus einer Liste von Namen
   * @param objekte mgliche Objekte
   * @param bezeichnungen Bezeichnungen der gesuchten Objekte
   * @return Liste mit den Systemobjekten
   */
  public static List<SystemObject> getSystemObjekte (List<SystemObject> objekte, List<String> bezeichnungen)
  {
  	List<SystemObject> l = new ArrayList<SystemObject>();
  	
  	for (String s : bezeichnungen)
  	{
  		for (SystemObject so : objekte)
  		{
  			if (getName(so).equals(s))
  			{
  				l.add(so);
  				break;
  			}
  		}
  	}
  	
  	return l;
  }
  
  /**
   * Methode liefert die Bezeichnung fr ein Systemobjekt. Hierber kann einheitlich erreicht werden,
   * ob nur der Namen oder der Name oder die Pid oder die Id ausgegeben werden
   * @param systemObject SystemObjekt
   * @return Bezeichnung
   */
  public static String getName (SystemObject systemObject)
  {
  	return systemObject.getNameOrPidOrId();
  }
}
