/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.parameter.ufd.haqparam;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.plugin.parameter.ufd.Activator;
import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractDataContainerComposite;
import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractViewParameter;

/**
 * View zum Bearbeiten der Parameter Helligkeit
 * 
 * @author Sans
 * 
 */
public class HAQViewParameter extends AbstractViewParameter
{

  public static final String ID = "de.bsvrz.buv.plugin.parameter.ufd.haqparamView";
  
  private final static List<String> PARAMETER_ATG_PIDS = new ArrayList<String>();
  static {
    PARAMETER_ATG_PIDS.add("atg.sswSbaHelligkeitZuordnungSensor");
    PARAMETER_ATG_PIDS.add("atg.sswSbaHelligkeitSteuerungStufe");
    PARAMETER_ATG_PIDS.add("atg.sswSbaHelligkeitSensorstrung");
  }
  
  @Override
  protected void createLayoutAndChildren(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    
    parent.setLayout( new GridLayout(1, false) );
    parent.setLayoutData( gridData );
    
    gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
//    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
//    gridData.grabExcessVerticalSpace = true;
    Composite composite = new HAQZuordnungSensorComposite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    composite.setLayoutData(gridData);
          
    gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    composite = new HAQSteuerungStufeComposite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    composite.setLayoutData(gridData);
    
    gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    composite = new HAQSensorStoerungComposite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    composite.setLayoutData(gridData);
  }

  @Override
  protected Class<? extends AbstractDataContainerComposite> getDataContainerComposite(String atgPid)
  {
    switch (getAtgIndex(atgPid)) {
      default:
        return null;
      case 0:
        return HAQZuordnungSensorComposite.class;
      case 1:
        return HAQSteuerungStufeComposite.class;
      case 2:
        return HAQSensorStoerungComposite.class;
    }
  }

  @Override
  protected String getHelpId()
  {
    return Activator.PLUGIN_ID + ".parameterHAQ";
  }

  @Override
  protected List<String> getParameterAtgPids()
  {
    return Collections.unmodifiableList(PARAMETER_ATG_PIDS);
  }

}
