/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.parameter.ufd.hstnparam;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractDataContainerComposite;
import de.bsvrz.buv.plugin.parameter.ufd.common.MinMaxColumnEditingSupport;
import de.bsvrz.buv.plugin.parameter.ufd.common.SpinnerColumnEditingSupport;
import de.bsvrz.buv.plugin.parameter.ufd.dav.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.ufd.dav.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;


/**
 * Darstellung des Fensters fr die Bearbeitung der Parameter "Helligkeitstuerung Tag/Nacht"
 * 
 * @author Sans
 *
 */
public class HSTNComposite extends AbstractDataContainerComposite
{ 
  public final static int DEFAULT_HW_HELL_TAG = 500;
  public final static int DEFAULT_HW_HELL_NACHT = 400;
  
  private List<SystemObject> _sensoren = new ArrayList<SystemObject>();
  private List<HSTNEntry> _hstnDaten = new ArrayList<HSTNEntry>();
  
  private TableViewer _tvSensoren;
  private TableViewer _tvHSTN;
  
  private Button _btnAdd;
  private Button _btnRemove;
    
  public HSTNComposite(Composite parent, int style) {
    super(parent, style);
  }

  @Override
  public void initialize() {
    
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    if (connection != null) {
      DataModel dataModel = connection.getDataModel();
      
      SystemObjectType type = dataModel.getType("typ.umfeldDatenSensor");
      
      if (type != null) {
        for (SystemObject so : type.getElements())
        {
          if (so.isValid() && !so.getPid().startsWith("swarco.pua")) {
            _sensoren.add(so);
          }  
        }  
        
        Collections.sort(_sensoren, new Comparator<SystemObject>() {

          @Override
          public int compare(SystemObject arg0, SystemObject arg1)
          {
            String name0 = Tools.getName(arg0);
            String name1 = Tools.getName(arg1);
            
            return name0.compareTo(name1);
          }
          
        });
      }
      
    }
      
    super.initialize();
  }
  
  public void setUpdate(Data data) {
    if (!isInitialized())
      return;
    
    Array array = data.getArray("ParameterTagNacht");
    _hstnDaten.clear();
    
    for (int i = 0; i < array.getLength(); i++) {
      Data item = array.getItem(i);
      SystemObject sensor = item.getItem("Sensor").asReferenceValue().getSystemObject();
      int hwHellTag = item.getItem("HelligkeitswertHellTag").asUnscaledValue().intValue();
      int hwHellNacht = item.getItem("HelligkeitswertHellNacht").asUnscaledValue().intValue();
      _hstnDaten.add(new HSTNEntry(sensor, hwHellTag, hwHellNacht));
    }
    
    Collections.sort(_hstnDaten, HSTN_ENTRY_COMPARATOR);
    _tvSensoren.refresh();
    _tvHSTN.refresh();
    
    enableDisableButtons();
  }
  
  public Data getUpdate() {
    if (!isInitialized())
      return null;
    
    Data result = _data.createModifiableCopy();
    Array array = result.getArray("ParameterTagNacht");
    array.setLength(0);
    for (int i = 0; i < _hstnDaten.size(); i++) {
      HSTNEntry entry = _hstnDaten.get(i);
      int index = array.getLength();
      array.setLength(index + 1);
      Data item = array.getItem(index);
      item.getItem("Sensor").asReferenceValue().setSystemObject(entry.sensor);
      item.getItem("HelligkeitswertHellTag").asUnscaledValue().set(entry.hwHellTag);
      item.getItem("HelligkeitswertHellNacht").asUnscaledValue().set(entry.hwHellNacht);
    }
    return result;
  }

  /**
   * This method initializes createGroupDefinition
   * 
   */
  protected void createControls()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
  
    Group groupParameter = new Group(this, SWT.NONE);
    groupParameter.setLayout(new GridLayout(3, false));
    groupParameter.setLayoutData(gridData);
    groupParameter.setText("Helligkeitsteuerung Tag/Nacht");

    createTableSensoren(groupParameter);
    createCompositeButton(groupParameter);
    createTableHSTN(groupParameter);
    
    enableDisableButtons();
  }
  
  private void createTableSensoren(Composite parent) {
    _tvSensoren = new TableViewer(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvSensoren.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvSensoren, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof SystemObject) {
          return (String.valueOf(element));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(200);
    column.getColumn().setText("Zuordbare Umfelddatensensoren");
    column.getColumn().setMoveable(false);
    column.getColumn().setResizable(true);
    
    Table table = _tvSensoren.getTable();
    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    
    GridData gridData = new GridData(SWT.DEFAULT, 
        10 * table.getItemHeight() + table.getHeaderHeight());
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    table.setLayoutData(gridData);
    
    _tvSensoren.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        enableDisableButtons();
      }
    });
    
    _tvSensoren.addFilter(new ViewerFilter() {

      @Override
      public boolean select(Viewer viewer, Object parentElement, Object element)
      {
        if (element instanceof SystemObject) {
          
          for (HSTNEntry entry: _hstnDaten) {
            if (entry.sensor.equals(element)) {
              return false;
            }
          }
          return true;
        }
        return false;
      }
      
    });
    
    _tvSensoren.setInput(_sensoren);
   
  }
  
   
  private void createCompositeButton(Composite parent) {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    composite.setLayoutData(gridData);
      
    GridData gridDataButton = new GridData();
    gridDataButton.horizontalAlignment = GridData.CENTER;
    
    _btnAdd = new Button(composite, SWT.PUSH);
    _btnAdd.setLayoutData(gridDataButton);
    _btnAdd.setText("Einfgen >>");
    _btnAdd.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionAddHSTNEntries();
      }
      
    });
    
    _btnRemove = new Button(composite, SWT.PUSH);
    _btnRemove.setLayoutData(gridDataButton);
    _btnRemove.setText("Entfernen <<");
    _btnRemove.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionRemoveHSTNEntries();
      }
      
    });
  }
  
  private void createTableHSTN(Composite parent) {
    _tvHSTN = new TableViewer(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvHSTN.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvHSTN, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof HSTNEntry) {
          return (String.valueOf(((HSTNEntry)element).sensor));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(200);
    column.getColumn().setText("Sensor");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
    column = new TableViewerColumn(_tvHSTN, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof HSTNEntry) {
          return (String.valueOf(((HSTNEntry)element).hwHellTag));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(130);
    column.getColumn().setText("HelligkeitswertHellTag");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
//    column.setEditingSupport(new MinMaxColumnEditingSupport(_tvHSTN, 0, 60000) {
    column.setEditingSupport(new SpinnerColumnEditingSupport(_tvHSTN, 0, 60000) {

      @Override
      protected Object getValue(Object element)
      {
//        if (element instanceof HSTNEntry) {
//          return String.valueOf(((HSTNEntry)element).hwHellTag);
//        }
        if (element instanceof HSTNEntry) {
          return Integer.valueOf(((HSTNEntry)element).hwHellTag);
        }
        return null;
      }

      @Override
      protected void setValue(Object element, Object value)
      {
        if (element instanceof HSTNEntry && value instanceof Integer) {
          ((HSTNEntry)element).hwHellTag = ((Integer)value).intValue();
          _tvHSTN.update(element, null);
        }
//        if (element instanceof HSTNEntry) {
//          try {
//            ((HSTNEntry)element).hwHellTag = value != null ? Integer.parseInt(value.toString()) : 0;
//            _tvHSTN.update(element, null);  
//          }
//          catch(NumberFormatException e) {
//            // tue nichts
//          }
//          
//        }
        
      }
      
    });
    

    column = new TableViewerColumn(_tvHSTN, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof HSTNEntry) {
          return (String.valueOf(((HSTNEntry)element).hwHellNacht));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(130);
    column.getColumn().setText("HelligkeitswertHellNacht");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
//    column.setEditingSupport(new MinMaxColumnEditingSupport(_tvHSTN, 0, 60000) {
    column.setEditingSupport(new SpinnerColumnEditingSupport(_tvHSTN, 0, 60000) {
      

      @Override
      protected Object getValue(Object element)
      {
//        if (element instanceof HSTNEntry) {
//          return String.valueOf(((HSTNEntry)element).hwHellNacht);
//        }
        if (element instanceof HSTNEntry) {
          return Integer.valueOf(((HSTNEntry)element).hwHellNacht);
        }
        return null;
      }

      @Override
      protected void setValue(Object element, Object value)
      {
        if (element instanceof HSTNEntry && value instanceof Integer) {
          ((HSTNEntry)element).hwHellNacht = ((Integer)value).intValue();
          _tvHSTN.update(element, null);
        }
//        if (element instanceof HSTNEntry) {
//          try {
//            ((HSTNEntry)element).hwHellNacht = value != null ? Integer.parseInt(value.toString()) : 0;
//            _tvHSTN.update(element, null);  
//          }
//          catch(NumberFormatException e) {
//            // tue nichts
//          }
//          
//        }
        
      }
      
    });
    
    Table table = _tvHSTN.getTable();
    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    
    GridData gridData = new GridData(SWT.DEFAULT, 
        10 * table.getItemHeight() + table.getHeaderHeight());
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    table.setLayoutData(gridData);
    
    _tvHSTN.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        enableDisableButtons();
      }
    });
    
    _tvHSTN.setInput(_hstnDaten);
  }
  
  private void actionAddHSTNEntries() {
    ISelection sel = _tvSensoren.getSelection();
    if (!sel.isEmpty() && sel instanceof StructuredSelection) {
      Object[] array = ((StructuredSelection)sel).toArray();
      
      for (int i = 0; i < array.length; i++) {
        if (array[i] instanceof SystemObject) {
          _hstnDaten.add(new HSTNEntry((SystemObject)array[i], DEFAULT_HW_HELL_TAG, DEFAULT_HW_HELL_NACHT));
        }
      }
      
      Collections.sort(_hstnDaten, HSTN_ENTRY_COMPARATOR);
      _tvHSTN.refresh();
      _tvSensoren.refresh();
      
      enableDisableButtons();
      
    }
  }
  
  private void actionRemoveHSTNEntries() {
    ISelection sel = _tvHSTN.getSelection();
    if (!sel.isEmpty() && sel instanceof StructuredSelection) {
      
      Object[] array = ((StructuredSelection)sel).toArray();
      ListIterator<HSTNEntry> it = _hstnDaten.listIterator();
      while(it.hasNext()) {
        HSTNEntry entry = it.next();
        for (int i = 0; i < array.length; i++) {
          if (entry.equals(array[i])) {
            it.remove();
            break;
          }
        }
      }
      _tvSensoren.refresh();
      _tvHSTN.refresh();
      
      enableDisableButtons();
    }
  }
  
  private void enableDisableButtons() {
    _btnAdd.setEnabled(_tvSensoren.getTable().getSelectionCount() > 0);
    _btnRemove.setEnabled(_tvHSTN.getTable().getSelectionCount() > 0);
  }
  
  private class HSTNEntry {
    public SystemObject sensor;
    public int hwHellTag;
    public int hwHellNacht;
    
    public HSTNEntry(SystemObject object, int hwHellTag, int hwHellNacht)
    {
      this.sensor = object;
      this.hwHellTag = hwHellTag;
      this.hwHellNacht = hwHellNacht;
    }
  }
  
  private final static Comparator HSTN_ENTRY_COMPARATOR = new Comparator<HSTNEntry>() {

    @Override
    public int compare(HSTNEntry arg0, HSTNEntry arg1)
    {
      String name0 = Tools.getName(arg0.sensor);
      String name1 = Tools.getName(arg1.sensor);
      
      return name0.compareTo(name1);
    }
  };

  
private class TableColumnEditingSupport extends EditingSupport {
    
    private int _index;
    private TextCellEditor _cellEditor;

    public TableColumnEditingSupport(int index) {
      super(_tvHSTN);
      _index = index;
      _cellEditor = new TextCellEditor((Composite)_tvHSTN.getTable(), SWT.NONE);
      
      ((Text)_cellEditor.getControl()).addVerifyListener(new VerifyListener() {
        public void verifyText(VerifyEvent e) {
        
          String text = ((Text)_cellEditor.getControl()).getText();
//          System.out.println("TEXT: \"" + text + "\"");
          String leading = text.substring(0, e.start);
//          System.out.println("LEADING: \"" + leading + "\"");
          String trailing = text.substring(e.end);
//          System.out.println("TRAILING: \"" + trailing + "\"");
         
          String result = leading + e.text + trailing;
//          System.out.println("RESULT: \"" + result + "\"");

          if (!"".equals(result)) {
            try {
              long value = Long.parseLong(result);
              if (value < 0 || value > 60000) {
                // Nur Ganzzahlen >= 0 und <= 60000
                throw new NumberFormatException();
              }
//              if (!String.valueOf(value).equals(result)) {
//                // Keine fhrenden Nullen etc.
//                throw new NumberFormatException();
//              }
            }
            catch (NumberFormatException ex) {
              e.doit = false;
//              System.out.println("DO IT: " + e.doit);
            }
          }
         }
        });
    }

    @Override
    protected CellEditor getCellEditor(Object element) {
            
      return _cellEditor;
    }

    @Override
    protected boolean canEdit(Object element) {
        return true;  
    }

    @Override
    protected Object getValue(Object element) {
      if (element instanceof HSTNEntry) {
        if (_index == 1) {
            return "" + ((HSTNEntry)element).hwHellTag;
        }
        if (_index == 2) {
          return "" + ((HSTNEntry)element).hwHellNacht;
        }
      }
      return null;
    }

    @Override
    protected void setValue(Object element, Object value) {
      if (element instanceof HSTNEntry) {
        try {
          if (_index == 1) {
            ((HSTNEntry)element).hwHellTag = value != null ? Integer.parseInt((String)value) : 0;
            _tvHSTN.update(element, null);
          }
          if (_index == 2) {
            ((HSTNEntry)element).hwHellNacht = value != null ? Integer.parseInt((String)value) : 0;
            _tvHSTN.update(element, null);

          }  
        }
        catch (NumberFormatException e) {
          // tue nichts
        }
          
      }
    }
  } 
  
}
