/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import org.eclipse.swt.widgets.TableColumn;

import de.bsvrz.buv.plugin.pua.basis.csv.CSVPrinter;
import de.bsvrz.buv.plugin.pua.basis.dav.AnfrageDaten;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStrukturZeitGruppe;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse ermglicht den Export einer Tabelle in eine CSV-Datei
 * 
 * @author Swarco Traffic Systems GmbH
 */
public class ExportCSV2
{
  /**
   * Tabellenmodell
   */
  private AbstractTableModel m_model;

  private LinkedList<String[]> _daten;

  private TableColumn[] _tableColumns;

  /**
   * Konstruktor der Klasse
   * 
   * @param model
   *          der Tabelle zugrundeliegendes Tabellenmodell
   */
  public ExportCSV2(AbstractTableModel model)
  {
    m_model = model;
  }

  public ExportCSV2(LinkedList<String[]> exportDaten)
  {
    _daten = exportDaten;
  }

  /**
   * Methode speichert die Tabelle in eine Datei
   * 
   * @param selectedFile
   *          ausgewhlte Datei
   * @param info
   *          Protokollinformation
   */
  public void speichereCsv(String name, AnfrageDaten info)
  {
    FileOutputStream out = null;
    CSVPrinter csvp = null;
    try
    {
      out = new FileOutputStream(name);
      csvp = new CSVPrinter(out);

      // Allgemeine Informationen

      if (info != null)
      {
        List<SystemObject> objekte = info.getObjekteAnzeige();
        for (SystemObject objekt : objekte)
          csvp.writeln(new String[] { "Objekt", objekt.getName(), objekt.getPid() });

        List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = info.getZeitgruppen();

        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
        Date d1 = new Date();
        Date d2 = new Date();

        if (zeitgruppen.size() > 0)
        {
          for (KalenderZeitFilterDatenStrukturZeitGruppe zg : zeitgruppen)
          {
            d1.setTime(zg.getStartzeitpunkt());
            d2.setTime(zg.getEndzeitpunkt());

            csvp.writeln(new String[] { "Zeibereich", sdf.format(d1), sdf.format(d2) });
          }
        }
      }

      // berschriften
      if (_tableColumns != null)
      {
        String daten[] = new String[_tableColumns.length];
        for (int i = 0; i < _tableColumns.length; ++i)
          daten[i] = _tableColumns[i].getText();
        csvp.writeln(daten);
      }

      // Daten
      if (_daten != null)
      {
        for (String[] d : _daten)
          csvp.writeln(d);
      }

      out.close();
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public void setTableColumns(TableColumn[] c)
  {
    _tableColumns = c;

  }
}
