/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.actions;

import javax.swing.SwingUtilities;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;

import com.swarco.sts.chart.control.TAxis;
import com.swarco.sts.chart.control.TChart;
import com.swarco.sts.chart.control.event.TActionEvent;
import com.swarco.sts.chart.control.event.TActionListener;

import de.bsvrz.buv.plugin.pua.basis.ImageFactory;

public class GitterlinienAktion implements IViewActionDelegate, IMenuCreator
{
  public final static int GA_AUS = 0; 
  public final static int GA_AU_LINKS = 1; 
  public final static int GA_LINKS = 2; 
  public final static int GA_RECHTS = 3; 
  public final static int GA_AU_RECHTS = 4;

  private ChartAktionenZiel chartAktionenZiel = null;
  private IAction aktion = null;
  private Menu menu = null;
  private UnterGitterlinienAktion ausAktion = null;
  private UnterGitterlinienAktion auLinksAktion = null;
  private UnterGitterlinienAktion linksAktion = null;
  private UnterGitterlinienAktion rechtsAktion = null;
  private UnterGitterlinienAktion auRechtsAktion = null;
  private boolean auLinksSelected = false;
  private boolean linksSelected = false;
  private boolean rechtsSelected = false;
  private boolean auRechtsSelected = false;
  
  /**
   * Die Singleton-Instanz der Image-Verwaltung
   */
  private ImageFactory _imageFactory = ImageFactory.getInstanz();
  
  public GitterlinienAktion()
  {
    super();
  }
  
  public GitterlinienAktion(ChartAktionenZiel chartAktionenZiel)
  {
    super();
    
    this.chartAktionenZiel = chartAktionenZiel;
  }
  
  @Override
  public void init(IViewPart sicht)
  {
    if(sicht instanceof ChartAktionenZiel)
    {
      this.chartAktionenZiel = (ChartAktionenZiel)sicht;
      this.chartAktionenZiel.getChart().addPaintListener(new PaintActionListener());
    }
  }

  @Override
  public void run(IAction aktion)
  {
  }

  @Override
  public void selectionChanged(IAction aktion, ISelection auswahl)
  {
    if(aktion != this.aktion)
    {
      aktion.setMenuCreator(this);
      this.aktion = aktion;
    }
  }

  public void dispose()
  {
    if(menu != null)
    {
      menu.dispose();
      menu = null;
    }
//    if(ausAktion != null)
//    {
//      ausAktion.dispose();
//      ausAktion = null;
//    }
//    if(auLinksAktion != null)
//    {
//      auLinksAktion.dispose();
//      auLinksAktion = null;
//    }
//    if(linksAktion != null)
//    {
//      linksAktion.dispose();
//      linksAktion = null;
//    }
//    if(rechtsAktion != null)
//    {
//      rechtsAktion.dispose();
//      rechtsAktion = null;
//    }
//    if(auRechtsAktion != null)
//    {
//      auRechtsAktion.dispose();
//      auRechtsAktion = null;
//    }
  }

  private class PaintActionListener implements TActionListener
  {
    public void actionPerformed(TActionEvent actionEvent)
    {
      if(actionEvent.isAction(TChart.CHART_POST_PAINT))
      {
        TChart chart = chartAktionenZiel.getChart();
        if(chart != null)
        {
          boolean auLinksVisible = chart.getAxis(TAxis.AC_Y0).isVisible();
          boolean linksVisible = chart.getAxis(TAxis.AC_Y1).isVisible();
          boolean rechtsVisible = chart.getAxis(TAxis.AC_Y2).isVisible();
          boolean auRechtsVisible = chart.getAxis(TAxis.AC_Y3).isVisible();

          if(auLinksSelected && !auLinksVisible ||
             linksSelected && !linksVisible ||
             rechtsSelected && !rechtsVisible ||
             auRechtsSelected && !auRechtsVisible)
          {
            doGitterlinienAktion(GitterlinienAktion.GA_AUS);
            auLinksSelected = false;
            linksSelected = false;
            rechtsSelected = false;
            auRechtsSelected = false;
          }
        }
      }
    }
  }
  
  public Menu getMenu(Control vorgaenger)
  {
    dispose();
    menu = new Menu(vorgaenger);

    if(chartAktionenZiel != null)
    {
      TChart chart = chartAktionenZiel.getChart();
      if(chart != null)
      {
        boolean auLinksGitter = chart.getAxis(TAxis.AC_Y0).isGridEnabled();
        boolean auLinksVisible = chart.getAxis(TAxis.AC_Y0).isVisible();
        boolean linksGitter = chart.getAxis(TAxis.AC_Y1).isGridEnabled();
        boolean linksVisible = chart.getAxis(TAxis.AC_Y1).isVisible();
        boolean rechtsGitter = chart.getAxis(TAxis.AC_Y2).isGridEnabled();
        boolean rechtsVisible = chart.getAxis(TAxis.AC_Y2).isVisible();
        boolean auRechtsGitter = chart.getAxis(TAxis.AC_Y3).isGridEnabled();
        boolean auRechtsVisible = chart.getAxis(TAxis.AC_Y3).isVisible();
        
        ausAktion = new UnterGitterlinienAktion(vorgaenger.getDisplay(), "Gitter aus", GitterlinienAktion.GA_AUS);
        ausAktion.setEnabled(auLinksGitter || linksGitter || rechtsGitter || auRechtsGitter); 
        if(ausAktion.isEnabled())
        {
          if(auLinksGitter && !auLinksVisible ||
             linksGitter && !linksVisible ||
             rechtsGitter && !rechtsVisible ||
             auRechtsGitter && !auRechtsVisible)
          {
            doGitterlinienAktion(GitterlinienAktion.GA_AUS);
            ausAktion.setEnabled(false);
            auLinksGitter = false;
            linksGitter = false;
            rechtsGitter = false;
            auRechtsGitter = false;
          }
        }
        auLinksAktion = new UnterGitterlinienAktion(vorgaenger.getDisplay(), "Gitter (links aussen)", GitterlinienAktion.GA_AU_LINKS);
        auLinksAktion.setEnabled(!auLinksGitter && auLinksVisible);
        linksAktion = new UnterGitterlinienAktion(vorgaenger.getDisplay(), "Gitter (links)", GitterlinienAktion.GA_LINKS);
        linksAktion.setEnabled(!linksGitter && linksVisible);
        rechtsAktion = new UnterGitterlinienAktion(vorgaenger.getDisplay(), "Gitter (rechts)", GitterlinienAktion.GA_RECHTS);
        rechtsAktion.setEnabled(!rechtsGitter && rechtsVisible);
        auRechtsAktion = new UnterGitterlinienAktion(vorgaenger.getDisplay(), "Gitter (rechts aussen)", GitterlinienAktion.GA_AU_RECHTS);
        auRechtsAktion.setEnabled(!auRechtsGitter && auRechtsVisible);
        hinzufuegenAktionZuMenu(menu, ausAktion);
        hinzufuegenAktionZuMenu(menu, auLinksAktion);
        hinzufuegenAktionZuMenu(menu, linksAktion);
        hinzufuegenAktionZuMenu(menu, rechtsAktion);
        hinzufuegenAktionZuMenu(menu, auRechtsAktion);
      }
    }

    return (menu);
  }


  public Menu getMenu(Menu vorgaenger)
  {
    return (null);
  }

  private void hinzufuegenAktionZuMenu(Menu menu, IAction aktion)
  {
    ActionContributionItem element = new ActionContributionItem(aktion);
    element.fill(menu, -1);
  }
  
  public void doGitterlinienAktion(final int type)
  {
    final TChart chart = chartAktionenZiel.getChart();
    if(chart != null)
    {
      SwingUtilities.invokeLater(new Runnable()
      {
        public void run()
        {
          chart.getAxis(TAxis.AC_X).setGridEnabled(false);
          chart.getAxis(TAxis.AC_Y0).setGridEnabled(false);
          chart.getAxis(TAxis.AC_Y1).setGridEnabled(false);
          chart.getAxis(TAxis.AC_Y2).setGridEnabled(false);
          chart.getAxis(TAxis.AC_Y3).setGridEnabled(false);
          auLinksSelected = false;
          linksSelected = false;
          rechtsSelected = false;
          auRechtsSelected = false;

          switch(type)
          {
            case GitterlinienAktion.GA_AU_LINKS:
            {
              if(chart.getAxis(TAxis.AC_Y0).isVisible())
              {
                chart.getAxis(TAxis.AC_X).setGridEnabled(true);
                chart.getAxis(TAxis.AC_Y0).setGridEnabled(true);
                auLinksSelected = true;
              }
              break;
            }
            case GitterlinienAktion.GA_LINKS:
            {
              if(chart.getAxis(TAxis.AC_Y1).isVisible())
              {
                chart.getAxis(TAxis.AC_X).setGridEnabled(true);
                chart.getAxis(TAxis.AC_Y1).setGridEnabled(true);
                linksSelected = true;
              }
              break;
            }
            case GitterlinienAktion.GA_RECHTS:
            {
              if(chart.getAxis(TAxis.AC_Y2).isVisible())
              {
                chart.getAxis(TAxis.AC_X).setGridEnabled(true);
                chart.getAxis(TAxis.AC_Y2).setGridEnabled(true);
                rechtsSelected = true;
              }
              break;
            }
            case GitterlinienAktion.GA_AU_RECHTS:
            {
              if(chart.getAxis(TAxis.AC_Y3).isVisible())
              {
                chart.getAxis(TAxis.AC_X).setGridEnabled(true);
                chart.getAxis(TAxis.AC_Y3).setGridEnabled(true);
                auRechtsSelected = true;
              }
              break;
            }
          }
          
          chart.repaint(false);
        }
      });
    }
  }
  
  private class UnterGitterlinienAktion extends Action
  {
    private Image symbol = null;
    private int type = -1;
    
    public UnterGitterlinienAktion(Display display, String text, int type)
    {
      super(text, Action.AS_PUSH_BUTTON);
      this.type = type;
      
      switch(type)
      {
        case GitterlinienAktion.GA_AUS:
        {
//          imageName = "grid_off.gif";
          symbol = _imageFactory.getImage("GRID_OFF");
          break;
        }
        case GitterlinienAktion.GA_AU_LINKS:
        {
//          imageName = "grid_osleft.gif";
          symbol = _imageFactory.getImage("GRID_OSLEFT");
          break;
        }
        case GitterlinienAktion.GA_LINKS:
        {
//          imageName = "grid_left.gif";
          symbol = _imageFactory.getImage("GRID_LEFT");
          break;
        }
        case GitterlinienAktion.GA_RECHTS:
        {
//          imageName = "grid_right.gif";
          symbol = _imageFactory.getImage("GRID_RIGHT");
          break;
        }
        case GitterlinienAktion.GA_AU_RECHTS:
        {
//          imageName = "grid_osright.gif";
          symbol = _imageFactory.getImage("GRID_OSRIGHT");
          break;
        }
      }
      
//      if(imageName != null)
//      {
//        try
//        {
//          Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
//          Path pfad = new Path("icons/" + imageName);
//          URL fileURL = FileLocator.find(bundle, pfad, null);
//          InputStream in = fileURL.openStream();
//          symbol = new Image(display, in);
//          in.close();
//          setImageDescriptor(ImageDescriptor.createFromImage(symbol));
//        }
//        catch(Exception e)
//        {
//          e.printStackTrace();
//        }
//      }
      
      if (symbol != null){
        setImageDescriptor(ImageDescriptor.createFromImage(symbol));
      }
    }
    
//    public void dispose()
//    {
//      if(symbol != null)
//      {
//        symbol.dispose();
//        symbol = null;
//      }
//    }
    
    public void run()
    {
      chartAktionenZiel.setGitterlinienAktion(type);
      doGitterlinienAktion(type);
    }
  }
}
