package de.bsvrz.buv.plugin.pua.basis.combinedChart;

import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.bsvrz.buv.plugin.pua.basis.views.GanglinieSicht;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class GanglinieAuswahlComposite extends Composite
{
  private static Comparator<SystemObject> SO_COMPARATOR = new Comparator<SystemObject>() {
    @Override
    public int compare(SystemObject arg0, SystemObject arg1) {
      return arg0.getNameOrPidOrId().compareTo(arg1.getNameOrPidOrId());
    }
  };
  
  private List<IStateChangeListener> _stateChangeListeners = new ArrayList<IStateChangeListener>();
    
  private List<SystemObject> _objects = new ArrayList<SystemObject>();
  
  private List<SystemObject> _selectedObjects = Collections.emptyList();

  private List<String> _availableColumns = new ArrayList<String>();

  private List<String> _notAvailableColumns = new ArrayList<String>();

  private List<String> _selectedColumns = new ArrayList<String>();
    
  private TableViewer _tvObjects;
  private TableViewer _tvAvailableColumns;
  private TableViewer _tvNotAvailableColumns;
  private TableViewer _tvSelectedColumns;
  
  private TableColumn _colAvailableColumns;
  private TableColumn _colNotAvailableColumns;
  private TableColumn _colSelectedColumns;
     
  private Button _buttonAdd;
  private Button _buttonRemove;

  public GanglinieAuswahlComposite(Composite parent, IProtokollAusgabe pa, int style)
  {
    super(parent, style);
        
    setLayout( new GridLayout(3, false) );
    
    _objects.addAll(pa.getObjekte());
    Collections.sort(_objects, SO_COMPARATOR);
    _availableColumns.addAll(pa.getProjektInfo().getAusgabeTabelleSpalten());
    Collections.sort(_availableColumns);
    
    createObjectsComposite();
    createColumnComposite();
    createButtonComposite();
    createSelectedComposite();
    
    updateAvailableColumns(getSelectedUnits());
  }
  
 
  public List<SystemObject> getObjects()
  {
    return Collections.unmodifiableList(_objects);
  }

  public List<String> getAvailableColumns()
  {
    return Collections.unmodifiableList(_availableColumns);
  }

  public List<String> getNotAvailableColumns()
  {
    return Collections.unmodifiableList(_notAvailableColumns);
  }

  public List<String> getSelectedColumns()
  {
    return Collections.unmodifiableList(_selectedColumns);
  }
  
  private void createButtonComposite() {
    Composite composite = new Composite(this, SWT.NONE);
//    composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true));
    composite.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
    composite.setLayout(new GridLayout());
    
    _buttonAdd = new Button(composite, SWT.PUSH);
    _buttonAdd.setEnabled(false);
    _buttonAdd.setText(">");
//    _buttonAdd.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
    _buttonAdd.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e) {
      }

      @Override
      public void widgetSelected(SelectionEvent e) {
        actionButtonAdd();
      }
      
    });
    
    _buttonRemove = new Button(composite, SWT.PUSH);
    _buttonRemove.setEnabled(false);
    _buttonRemove.setText("<");
//    _buttonRemove.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
    _buttonRemove.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e) {
      }

      @Override
      public void widgetSelected(SelectionEvent e) {
        actionButtonRemove();
      }
      
    });
    
  }
  
  private void createObjectsComposite() {
    Composite composite = new Composite(this, SWT.NONE);
    GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
    gridData.horizontalSpan = 3;
    composite.setLayoutData(gridData);
    composite.setLayout(new GridLayout());
    
    Label label = new Label(composite, SWT.NONE);
    label.setText("Objektauswahl");
    
    _tvObjects = new TableViewer(composite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvObjects.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvObjects, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof SystemObject) {
          return ((SystemObject)element).getName();
        }
        return null;
      }
      
    });
    
    column.getColumn().setMoveable(false);
    column.getColumn().setResizable(false);
       
    Table table = _tvObjects.getTable();
    table.setLinesVisible(false);
    table.setHeaderVisible(false);
    
    GridData gdObjects = new GridData(SWT.FILL, SWT.FILL, true, true);
    gdObjects.heightHint = 4 * table.getItemHeight() + table.getHeaderHeight(); 
    table.setLayoutData(gdObjects);
    
    _tvObjects.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        List<SystemObject> selectedObjects = new ArrayList<SystemObject>();
        
        ISelection selection = event.getSelection();
        if (selection instanceof IStructuredSelection)
        {
          Iterator<?> it = ((IStructuredSelection)selection).iterator();
          while (it.hasNext()) {
            selectedObjects.add((SystemObject)it.next());
          }
        }
        
        boolean changed = !selectedObjects.equals(_selectedObjects);
        _selectedObjects = selectedObjects;
        
        if (changed) {
          fireStateChange();  
        }
      }
    });
        
    _tvObjects.setInput(_objects);
    
    column.getColumn().pack();
    
  }
  
  /**
   * 
   * @return the width of the single column of both tables - after filling with data and pack();
   */
  private void createColumnComposite() {
    Composite composite = new Composite(this, SWT.NONE);
    composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
    composite.setLayout(new GridLayout());
    
    Label labelAvailable = new Label(composite, SWT.NONE);
    labelAvailable.setText("Verfgbare Spalten");
    
    _tvAvailableColumns = new TableViewer(composite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAvailableColumns.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn columnAvailable = new TableViewerColumn(_tvObjects, SWT.NONE);
    columnAvailable.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof String) {
          return (String)element;
        }
        return null;
      }
      
    });
    
    _colAvailableColumns = columnAvailable.getColumn();
    _colAvailableColumns.setMoveable(false);
    _colAvailableColumns.setResizable(false);
       
    Table tableAvailable = _tvAvailableColumns.getTable();
    tableAvailable.setLinesVisible(false);
    tableAvailable.setHeaderVisible(false);
    
    GridData gdAvailable = new GridData(SWT.FILL, SWT.FILL, true, true);
    gdAvailable.heightHint = 6 * tableAvailable.getItemHeight() + tableAvailable.getHeaderHeight(); 
    tableAvailable.setLayoutData(gdAvailable);

    _tvAvailableColumns.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        _buttonAdd.setEnabled(!selection.isEmpty());
      }
    });
    
    _tvAvailableColumns.setInput(_availableColumns);
    _colAvailableColumns.pack();
    
           
    Label labelNotAvailable = new Label(composite, SWT.NONE);
    labelNotAvailable.setText("Nicht mehr verfgbare Spalten");
    GridData gd = new GridData();
    gd.verticalIndent = 8;
    labelNotAvailable.setLayoutData(gd);
    
    _tvNotAvailableColumns = new TableViewer(composite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvNotAvailableColumns.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn columnNotAvailable = new TableViewerColumn(_tvObjects, SWT.NONE);
    columnNotAvailable.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof String) {
          return (String)element;
        }
        return null;
      }
      
    });
    
    _colNotAvailableColumns = columnAvailable.getColumn();
    _colNotAvailableColumns.setMoveable(false);
    _colNotAvailableColumns.setResizable(false);
       
    Table tableNotAvailable = _tvNotAvailableColumns.getTable();
    tableNotAvailable.setLinesVisible(false);
    tableNotAvailable.setHeaderVisible(false);
    
    GridData gdNotAvailable = new GridData(SWT.FILL, SWT.FILL, true, true);
    gdNotAvailable.heightHint = 6 * tableNotAvailable.getItemHeight() + tableNotAvailable.getHeaderHeight(); 
    tableNotAvailable.setLayoutData(gdNotAvailable);
      
    _tvNotAvailableColumns.setInput(_notAvailableColumns);
    _colNotAvailableColumns.pack();
  }
  
  private void createSelectedComposite() {
    Composite composite = new Composite(this, SWT.NONE);
    composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
    composite.setLayout(new GridLayout());
    
    Label label = new Label(composite, SWT.NONE);
    label.setText("Ausgewhlte Spalten");
    
    _tvSelectedColumns = new TableViewer(composite, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvSelectedColumns.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvSelectedColumns, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof String) {
          return (String)element;
        }
        return null;
      }
      
    });
    
    _colSelectedColumns = column.getColumn();
    _colSelectedColumns.setMoveable(false);
    _colSelectedColumns.setResizable(false);
             
    Table table = _tvSelectedColumns.getTable();
    table.setLinesVisible(false);
    table.setHeaderVisible(false);
    
    table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
    
    _tvSelectedColumns.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        _buttonRemove.setEnabled(!selection.isEmpty());
      }
    });
    
    _tvSelectedColumns.setInput(_selectedColumns);
    _colSelectedColumns.pack();
  }
  
  private List<GanglinieAuswahlComposite> getAllGanglinieAuswahlComposite() {
    List<GanglinieAuswahlComposite> result = new ArrayList<GanglinieAuswahlComposite>();
    Composite parent = getParent();
    if (parent instanceof CTabFolder) {
      for (CTabItem item: ((CTabFolder)parent).getItems()) {
        Control c = item.getControl();
        if (c instanceof GanglinieAuswahlComposite) {
          result.add((GanglinieAuswahlComposite)c);
        }
      }
    }
    return result;
  }
  
  private HashSet<String> getSelectedUnits() {
    HashSet<String> result = new HashSet<String>();
    
    for (GanglinieAuswahlComposite gac: getAllGanglinieAuswahlComposite()) {
      for (String col: gac.getSelectedColumns()) {
          result.add(getUnit(col));
      }
    }
  
    return result;
  }
  
  private void actionButtonAdd() {
    ISelection selection = _tvAvailableColumns.getSelection();
    if (selection instanceof IStructuredSelection)
    {
      Object[] selectedValues = ((IStructuredSelection)selection).toArray();
      if (selectedValues.length > 0)
      {
        // bisher ausgewaehlte einheiten bestimmen
        HashSet<String> selectedUnits = getSelectedUnits();

        boolean added = false;
        for (Object selectedValue : selectedValues)
        {
          String selectedItem = (String)selectedValue;
          if (selectedUnits.contains(getUnit(selectedItem))) {
            // hinzuzufuegendes element hat eine bereits ausgewaehlte einheit
            _selectedColumns.add(selectedItem);
            _availableColumns.remove(selectedItem);
            added = true;
          }
          else if (selectedUnits.size() < GanglinieSicht.MAX_Y_UNITS_COUNT) {
            // noch nicht das maximum aller auswaehlbaren einheiten erreicht
            selectedUnits.add(getUnit(selectedItem));
            _selectedColumns.add(selectedItem);
            _availableColumns.remove(selectedItem);
            added = true;
          }
        }

//        updateOkButton();

        if (added)
        {
          // zur tabelle der ausgewaehlten elemente wurde etwas hinzugefuegt -> inhalt sortieren
          Collections.sort(_selectedColumns);
          _tvSelectedColumns.refresh();
          _colSelectedColumns.pack();
          _tvAvailableColumns.refresh();
          _colAvailableColumns.pack();
          
          for (GanglinieAuswahlComposite gac: getAllGanglinieAuswahlComposite()) {
            gac.updateAvailableColumns(selectedUnits);
          }
          
          fireStateChange();
        }
      }
    }
  }
  
  private void actionButtonRemove() {
    ISelection selection = _tvSelectedColumns.getSelection();
    if (selection instanceof IStructuredSelection)
    {
      Object[] selectedValues = ((IStructuredSelection)selection).toArray();
      if (selectedValues.length > 0)
      {
        

        boolean added = false;
        for (Object selectedValue : selectedValues)
        {
          String selectedItem = (String)selectedValue;
          _availableColumns.add(selectedItem);
          _selectedColumns.remove(selectedItem);
          added = true;
        }

        if (added)
        {
          Collections.sort(_availableColumns);
          _tvAvailableColumns.refresh();
          _colAvailableColumns.pack();
          _tvSelectedColumns.refresh();
          _colSelectedColumns.pack();
          
          HashSet<String> selectedUnits = getSelectedUnits();
          
          for (GanglinieAuswahlComposite gac: getAllGanglinieAuswahlComposite()) {
            gac.updateNotAvailableColumns(selectedUnits);
          }
          
          fireStateChange();
        }
      }
    }  
  }
  
  private void updateAvailableColumns(HashSet<String> selectedUnits) {
    if (selectedUnits.size() >= GanglinieSicht.MAX_Y_UNITS_COUNT)
    {
      // maximale anzahl einheiten ausgewaehlt -> evtl. nicht mehr passende elemente von
      // auwaehlbare elemente nach nicht auswaehlbare elemente verschieben
      boolean changed = false;
      ListIterator<String> it = _availableColumns.listIterator();
      while (it.hasNext()) {
        String col = it.next();
        if (!selectedUnits.contains(getUnit(col))) {
          it.remove();
          _notAvailableColumns.add(col);
          changed = true;  
        }
      }
      if (changed) {
        Collections.sort(_availableColumns);
        _tvAvailableColumns.refresh();
        _colAvailableColumns.pack();
        Collections.sort(_notAvailableColumns);
        _tvNotAvailableColumns.refresh();
        _colNotAvailableColumns.pack();
      }
    }
  }
  
  private void updateNotAvailableColumns(HashSet<String> selectedUnits) {
    if (selectedUnits.size() < GanglinieSicht.MAX_Y_UNITS_COUNT)
    {
      // maximale anzahl einheiten ausgewaehlt -> evtl. nicht mehr passende elemente von
      // auwaehlbare elemente nach nicht auswaehlbare elemente verschieben
      boolean changed = false;
      ListIterator<String> it = _notAvailableColumns.listIterator();
      while (it.hasNext()) {
        String col = it.next();
        it.remove();
        _availableColumns.add(col);
        changed = true;  
      }
      if (changed) {
        Collections.sort(_availableColumns);
        _tvAvailableColumns.refresh();
        _colAvailableColumns.pack();
        Collections.sort(_notAvailableColumns);
        _tvNotAvailableColumns.refresh();
        _colNotAvailableColumns.pack();
      }
    }
  }
    
  private String getUnit(String text)
  {
    String result = null;

    int kla = text.indexOf("[");
    int klz = text.indexOf("]");
    if (kla >= 0 && klz >= kla)
    {
      result = text.substring(kla, klz + 1);
    }

    return result;
  }
  
  public void addStateChangeListener(IStateChangeListener listener) {
    if (!_stateChangeListeners.contains(listener)) {
      _stateChangeListeners.add(listener);
    }
  }

  public void removeStateChangeListener(IStateChangeListener listener) {
    if (_stateChangeListeners.contains(listener)) {
      _stateChangeListeners.remove(listener);
    }
  }
  
  private void fireStateChange() {
    for (IStateChangeListener listener: _stateChangeListeners) {
      listener.stateChange();
    }
  }
  
  public List<SystemObject> getSelectedObjects() {
    return Collections.unmodifiableList(_selectedObjects);
  }
   
}
