package de.bsvrz.buv.plugin.pua.basis.combinedChart;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.pua.basis.expose.IPIGanglinienKonvertierung;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;
import de.bsvrz.buv.plugin.pua.basis.views.GanglinieSicht;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.losb.util.Tuple;

public class KombinierteGanglinieAuswahlComposite extends Composite implements IProtokollAusgabeListener,
  IStateChangeListener {

  private CTabFolder _tabFolder;
  
  private List<IChartAvailableListener> _chartAvailableListeners = new ArrayList<IChartAvailableListener>();

  public KombinierteGanglinieAuswahlComposite(Composite parent, int style) {
    super(parent, style);

    setLayout(new GridLayout());

    _tabFolder = new CTabFolder(this, SWT.NONE);
    _tabFolder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

  }

  @Override
  public void updateProtokollAusgaben(IProtokollAusgabe[] protokollAusgaben) {
    boolean changed = false;
    for (CTabItem item : _tabFolder.getItems()) {
      IProtokollAusgabe data = (IProtokollAusgabe)item.getData();
      boolean found = false;
      for (IProtokollAusgabe pa : protokollAusgaben) {
        if (data.getId().equals(pa.getId())) {
          found = true;
          break;
        }
      }
      if (!found) {
        Control c = item.getControl();
        if (c instanceof GanglinieAuswahlComposite) {
          ((GanglinieAuswahlComposite)c).removeStateChangeListener(this);
        }
        if (c != null) {
          c.dispose();  
        }
        item.dispose();
        changed = true;
      }
    }
    CTabItem sel = _tabFolder.getSelection();
    for (IProtokollAusgabe pa : protokollAusgaben) {
      boolean found = false;
      for (CTabItem item : _tabFolder.getItems()) {
        IProtokollAusgabe data = (IProtokollAusgabe)item.getData();
        if (data.getId().equals(pa.getId())) {
          found = true;
          break;
        }
      }
      if (!found) {
        CTabItem item = new CTabItem(_tabFolder, SWT.NONE);
        item.setText(pa.getName());
        item.setData(pa);

        GanglinieAuswahlComposite composite = new GanglinieAuswahlComposite(_tabFolder, pa, SWT.NONE);
        composite.addStateChangeListener(this);
        composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        item.setControl(composite);
        changed = true;
      }
    }
    if (changed) {
      fireChartAvailable(isChartAvailable());
    }
    if (sel == null && _tabFolder.getItemCount() > 0) {
      _tabFolder.setSelection(0);
    }

  }

  @Override
  public void stateChange()
  {
    fireChartAvailable(this.isChartAvailable());
  }
  
  public void addChartAvailableListener(IChartAvailableListener listener) {
    if (!_chartAvailableListeners.contains(listener)) {
      _chartAvailableListeners.add(listener);
    }
  }

  public void removeChartAvailableListener(IChartAvailableListener listener) {
    if (_chartAvailableListeners.contains(listener)) {
      _chartAvailableListeners.remove(listener);
    }
  }
  
  private void fireChartAvailable(boolean available) {
    for (IChartAvailableListener listener: _chartAvailableListeners) {
      listener.chartAvailable(available);
    }
  }
  
  private boolean isChartAvailable() {
    boolean result = false;
    
    for (CTabItem item: _tabFolder.getItems()) {
      Control c = item.getControl();
      if (c instanceof GanglinieAuswahlComposite) {
        if (((GanglinieAuswahlComposite)c).getSelectedObjects().size() > 0 &&
            ((GanglinieAuswahlComposite)c).getSelectedColumns().size() > 0) {
          result = true;
          break;
        }
      }
    }
    
    return result;
  }
  
  boolean checkSelection() {
    boolean result = true;
    
    int ganglinien = 0;
    String text = "";
    
    for (CTabItem item: _tabFolder.getItems()) {
      Control c = item.getControl();
      if (c instanceof GanglinieAuswahlComposite) {
        int objekte = ((GanglinieAuswahlComposite)c).getSelectedObjects().size();
        int spalten = ((GanglinieAuswahlComposite)c).getSelectedColumns().size();
        if (objekte > 0 && spalten > 0) {
          ganglinien += objekte * spalten;
          text += "\t" + objekte + "Objekte x " + spalten + " Spalten\n";
        }
      }
    }
    
    if (ganglinien > GanglinieSicht.MAX_GANGLINIEN)
    {
      result = false;
    
      MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.ICON_WARNING
          | SWT.OK);
      mb.setText("Zuviele Objekte und Spalten selektiert");
      mb.setMessage("Es knnen Objekte und Spalten fr maximal " + GanglinieSicht.MAX_GANGLINIEN + 
          " Ganglinien selektiert werden. Aktuell sind\n" + text + " fr " + ganglinien + " Ganglinien selektiert.");
      mb.open();
    }
    
    return result;
  }
  
  Map<String, Map<String, SortedMap<Long, String>>>  erzeugeChartInput() {
    Map<String, Map<String, SortedMap<Long, String>>> result = 
      new TreeMap<String, Map<String, SortedMap<Long, String>>>();
    
    Map<String, Integer> attributIndexMap = new HashMap<String, Integer>();

    for (CTabItem item: _tabFolder.getItems()) {
      Control c = item.getControl();
      if (c instanceof GanglinieAuswahlComposite) {
        IProtokollAusgabe pa = (IProtokollAusgabe)item.getData();
        IProjektInfo projektInfo = pa.getProjektInfo();
        Table table = pa.getTabelle();
        Collection<String> selektierteSpalten = ((GanglinieAuswahlComposite)c).getSelectedColumns();
        Collection<SystemObject> selektierteObjekte = ((GanglinieAuswahlComposite)c).getSelectedObjects();
        Tuple<Long, Long> gesamtIntervall = pa.getGesamtIntervall();

        if (selektierteObjekte.isEmpty()) {
          continue;
        }
        
        // map fuer alle attribute mit spalte in der tabelle anlegen
        Integer platzhalter = new Integer(0);
        SortedMap<String, Integer> spaltenNachschlagewerk = new TreeMap<String, Integer>();
        for (String spalte: selektierteSpalten)
        {
          spaltenNachschlagewerk.put(spalte, platzhalter);
          
          Integer attributIndex = attributIndexMap.get(spalte);
          attributIndexMap.put(spalte, attributIndex != null ? attributIndex.intValue() + 1 : platzhalter);
        }
        
        // spalteninformationen in der attribute-map setzen
        TableColumn[] spalten = table.getColumns();
        for (int i = 0; i < spalten.length; i++)
        {
          if (spaltenNachschlagewerk.containsKey(spalten[i].getText()))
          {
            spaltenNachschlagewerk.put(spalten[i].getText(), new Integer(i));
          }
        }
        
        // ueber alle attribute laufen
        for (Map.Entry<String, Integer> mapEntry: spaltenNachschlagewerk.entrySet())
        {
          // attribut und spalte ermitteln
          String attribut = mapEntry.getKey();
                    
          int spalte = mapEntry.getValue().intValue();
          Map<String, SortedMap<Long, String>> objektNachschlagewerk = new TreeMap<String, SortedMap<Long, String>>();
          
          // ueber alle zeilen der tabelle laufen
          int itemCount = table.getItemCount();
          for (int i = 0; i < itemCount; i++)
          {
            TableItem tableItem = table.getItem(i);
            // objekt ermittlen
            String objektName = tableItem.getText(1);
            SystemObject object = null;
            
            for (SystemObject so: selektierteObjekte) {
              if (objektName.equals(so.getName())) {
                object = so;
              }
            }
            
            if (object == null) {
              continue;
            }

            // zeitstempel ermitteln
            Long zeitstempel = null;
//            try
//            {
//              zeitstempel = new Long(GanglinieSicht.SDF.parse(tableItem.getText(0)).getTime());
//            }
//            catch (Exception e)
//            {
//              continue;
//            }
            // Hole Zeitstempel aus Daten beim TableItem mit Key TIMESTAMP_KEY
            Object data = tableItem.getData(ProtokollAusgabeSicht.TIMESTAMP_KEY);
            if (data instanceof Long) {
              zeitstempel = (Long)data;
            }
            else {
              continue;
            }

            // wert ermitteln
            String wert = tableItem.getText(spalte);
            
            if (projektInfo instanceof IPIGanglinienKonvertierung && ((IPIGanglinienKonvertierung)projektInfo).isAttributKonvertierbar(attribut)) {
              if (((IPIGanglinienKonvertierung)projektInfo).getZuIgnorierendeWerte().contains(wert)) {
                continue;
              }
              String konvWert = ((IPIGanglinienKonvertierung)projektInfo).getWertKonvertiert(attribut, wert, object.getName());
              if (konvWert != null) {
                wert = konvWert;  
              }
            }

            wert.replace(',', '.');  
                        
            // die liste der zeitstempel und werte ermitteln
            SortedMap<Long, String> zeitstempelWertListe = objektNachschlagewerk.get(objektName);
            if (zeitstempelWertListe == null)
            {
              // liste noch nicht vorhanden -> neu anlegen und im objekt nachschlagewerk ablegen
              zeitstempelWertListe = new TreeMap<Long, String>();
              objektNachschlagewerk.put(objektName, zeitstempelWertListe);
            }
            // zeitstempel und wert in die liste eintragen
            zeitstempelWertListe.put(zeitstempel, wert);
            
//            System.out.println("ZEITSTEMPEL: " + new SimpleDateFormat("dd.MM.yyyy HH:mm:ss:SSS").format(new Date(zeitstempel))
//                + "\tWERT: " + wert);
          }

          if (projektInfo instanceof IPIGanglinienKonvertierung && ((IPIGanglinienKonvertierung)projektInfo).isAttributKonvertierbar(attribut)) {
            
            long milliesTreppenfunktionErgaenzen = ((IPIGanglinienKonvertierung)projektInfo).getMilliesTreppenfunktionErgaenzen();
            if (milliesTreppenfunktionErgaenzen > 0) {
              // Werte fr 'Treppenfunktion' ergnzen
              for ( SortedMap<Long, String> zeitstempelWertListe: objektNachschlagewerk.values()) {
                
                Long letzterZeitstempel = null;
                String letzterWert = null;
                String letzerGeaenderterWert = null;
                               
                Map<Long, String> ergaenzungsMap = new HashMap<Long, String>();
                
                for (Long zeitstempel: zeitstempelWertListe.keySet()) {
                  
                  String wert = zeitstempelWertListe.get(zeitstempel);
                  
                  if (letzerGeaenderterWert == null || !letzerGeaenderterWert.equals(wert)) {
                    if (letzerGeaenderterWert != null && letzterZeitstempel != null && letzterWert != null) {
                      if (zeitstempel - letzterZeitstempel > milliesTreppenfunktionErgaenzen) {
                        ergaenzungsMap.put(zeitstempel - milliesTreppenfunktionErgaenzen, letzerGeaenderterWert);
                      }
                    }
                    letzerGeaenderterWert = wert;
                  }
                  letzterWert = wert;
                  letzterZeitstempel = zeitstempel;
                }
                
                // Ergnze Werte fr Treppenfunktion
                zeitstempelWertListe.putAll(ergaenzungsMap);
                
                // Anfang auffllen
                if (((IPIGanglinienKonvertierung)projektInfo).anfangAuffuellen()) {
                  Integer letzterWertVorAnfang = null;
                  Collection<Long> zeitstempelVorAnfang = new LinkedList<Long>();
                  
                  for (Long zeitstempel: zeitstempelWertListe.keySet()) {
                    
                    if (gesamtIntervall == null ||  zeitstempel >= gesamtIntervall.first) {
                      break;
                    }
                    
                    String wert = zeitstempelWertListe.get(zeitstempel);
                    zeitstempelVorAnfang.add(zeitstempel);
                    try {
                      letzterWertVorAnfang = Integer.parseInt(wert);
                    }
                    catch (Exception e) {
                      // tue nichts
                    }
                  }
                  
                  for (Long zeitstempel: zeitstempelVorAnfang) {
                    zeitstempelWertListe.remove(zeitstempel);
                  }
                  
                  if (letzterWertVorAnfang != null && !zeitstempelWertListe.containsKey(gesamtIntervall.first)) {
                    zeitstempelWertListe.put(gesamtIntervall.first, "" + letzterWertVorAnfang);
                  }
                }
                
                // Ende auffllen
                if (((IPIGanglinienKonvertierung)projektInfo).endeAuffuellen()) {
                  Long zeitstempelEnde = null;
                  Integer letzterNumerischerWert = null;
                  
                  for (Long zeitstempel: zeitstempelWertListe.keySet()) {
                    
                    String wert = zeitstempelWertListe.get(zeitstempel);
                    zeitstempelEnde = zeitstempel;
                    try {
                      letzterNumerischerWert = Integer.parseInt(wert);
                    }
                    catch (Exception e) {
                      // tue nichts
                    }
                  }
                                    
                  if (gesamtIntervall != null && zeitstempelEnde != null && gesamtIntervall.last > zeitstempelEnde) {
                    if (letzterNumerischerWert != null /* && !zeitstempelWertListe.containsKey(gesamtIntervall.last) */) {
                      zeitstempelWertListe.put(gesamtIntervall.last, "" + letzterNumerischerWert);  
                    }
                  }  
                }
              }  
            }
          }
                    
          String key = attribut;
          if (projektInfo instanceof IPIGanglinienKonvertierung) {
            key = ((IPIGanglinienKonvertierung)projektInfo).getAttributKonvertiert(attribut);
          }
          
          Integer attributIndex = attributIndexMap.get(attribut);
          if (attributIndex != null) {
            int index = attributIndex.intValue();
            if (index > 0) {
              int kla = key.indexOf("[");
              int klz = key.indexOf("]"); 
              if (kla >= 0 && klz >= kla)
              {
                String prefix = key.substring(0, kla).trim();
                key = key.replace(prefix, prefix + "/" + index);
              }
              else {
                key += "/" + index;
              }
            }
          }
          
          result.put(key, objektNachschlagewerk);
        }
      }
    }
    
    return result;
  }

}
