/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.preferences;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import com.swarco.sts.chart.control.TChart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;

/**
 * {@link PreferencePage} des Plug-Ins Zustandsdarstellung. Es kann:
 * <ul>
 * <li>einstellen, unter welcher StreckenBeeinflussungsAnlage die Zustandsdarstellung
 * arbeitet.</li>
 * </ul>
 * 
 * @author Pittner
 * 
 */
public class PreferencePageChart extends PreferencePage implements
		IWorkbenchPreferencePage
{
	/**
	 * ID dieser PreferencePage
	 */
	public static final String ID = "de.bsvrz.buv.plugin.pua.basis.pageChart";

	/**
	 * Gruppe zur Auswahl des Konfigurationsbereiches
	 */
	private Group _groupConfArea;

	/**
	 * Gruppe fr sonstige Einstellungen
	 */
	private Group _groupMisc;


	/**
	 * Checkbox zum Anzeigen, ob Benutzer ohne Berechtigungsklasse dargestellt
	 * werden sollen oder nicht
	 */
	private Button _checkUserWithoutClass;
	

	/**
	 * Listener, der auf die DaVVerbindung hrt. Ist die Verbindung inaktiv, wird
	 * der Button zur Auswahl des Konfigurationsbereiches deaktiviert, ist sie
	 * jedoch aktiv, so wird der Button aktiviert.
	 */
	private IDaVVerbindungZuhoerer _verbindungZuhoerer;

  

	/**
	 * Konstruktor. Setzt {@link PreferenceStore} und Titel dieser
	 * {@link PreferencePage}.
	 */
	public PreferencePageChart()
	{
		super();
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("Prferenzen fr Ganglinien");
	}

	@Override
	protected Control createContents(Composite parent)
	{
	  Composite top = new Composite(parent, SWT.NONE);
	  
  	  //Fr die F1 Taste
	  PlatformUI.getWorkbench().getHelpSystem().setHelp( top, Activator.PLUGIN_ID + ".chartEinstellungen" );
		
	  createGroupMiscellaneous(top);
	
		actionLoad();

		/*
		 * TabList
		 */
		Control[] tabList = new Control[] { _groupMisc };
		top.setTabList(tabList);

	  /*
     * Layout
     */
    GridLayout glTop = new GridLayout(1, false);
    top.setLayout(glTop);

    GridData gdTop = new GridData();
    gdTop.grabExcessHorizontalSpace = true;
    gdTop.horizontalAlignment = GridData.FILL;
    gdTop.grabExcessVerticalSpace = true;
    gdTop.verticalAlignment = GridData.FILL;
    top.setLayoutData(gdTop);


		GridData gdGroupMisc = new GridData();
		gdGroupMisc.grabExcessHorizontalSpace = true;
		gdGroupMisc.horizontalAlignment = GridData.FILL;
		gdGroupMisc.grabExcessVerticalSpace = true;
		gdGroupMisc.verticalAlignment = GridData.FILL;
		_groupMisc.setLayoutData(gdGroupMisc);
					

		return null;
	}
	
  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow
   * ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des
   * aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen
   * bercksichtigt werden. Dies ist fr ein Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  private IAction leseActions(String id, Text text)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg
    .getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench()
    .getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();
      
      if (Activator.PLUGIN_ID.equals(contributor))
      {
//        System.out.println("PreferencePageAnlagenDarstellung.leseActions()-1");
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {          
          String att = element.getAttribute("id");  
//          System.out.println(att + " >-<" + id);
          if (att.equals(id)){            
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            } else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }
            
            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element,
                  ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            } else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }           
            PluginAction action = desc.getAction();   
            
            System.out.println("Action: " + action.getText());
            
            if (text != null)
              text.setText(action.getText());    
            
            return action;
            
          }  
        }
        
      }      
    }
    
    return null;
    
  }
	

	private Group createGroupMiscellaneous(Composite parent)
	{
		if (_groupMisc == null)
		{
			_groupMisc = new Group(parent, SWT.NONE);
			_groupMisc.setText("Chart:");

			_checkUserWithoutClass = new Button(_groupMisc, SWT.CHECK);
			_checkUserWithoutClass.setText("Aktiviere Workaround");

			/*
			 * TabList
			 */
			Control[] tabList = new Control[] { _checkUserWithoutClass };
			_groupMisc.setTabList(tabList);

			/*
			 * Layout
			 */
			GridLayout glMisc = new GridLayout(1, false);
			_groupMisc.setLayout(glMisc);
		}

		return _groupMisc;
	}
	
	/**
	 * Ldt die Einstellungen aus dem {@link PreferenceStore}.
	 */
	private void actionLoad()
	{
 		/*
		 * Benutzer ohne Berechtigungsklasse
		 */
		boolean userWithoutClass = getPreferenceStore().getBoolean(
				DefsPreference.P_USE_WORK_AROUND_CHART);
		
//		System.out.println("------------------> USE WORKAROUND: " + userWithoutClass);
		
		_checkUserWithoutClass.setSelection(userWithoutClass);
		
	}

	/**
	 * Ldt die Default-Einstellungen aus dem {@link PreferenceStore}.
	 */
	private void actionRestoreDefaults()
	{
		/*
		 * Benutzer ohne Berechtigungsklasse (Default-Wert)
		 */
		boolean userWithoutClass = getPreferenceStore().getDefaultBoolean(
		    DefsPreference.P_USE_WORK_AROUND_CHART);
		_checkUserWithoutClass.setSelection(userWithoutClass);	
	
	}

	/**
	 * Speichert die Einstellungen im {@link PreferenceStore}.
	 */
	private boolean actionSave()
	{
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    boolean showUserWithoutClass = _checkUserWithoutClass.getSelection();
    
    getPreferenceStore().setValue(DefsPreference.P_USE_WORK_AROUND_CHART, showUserWithoutClass);
 
    System.setProperty(TChart.BUGFIX_GRAPHICS_RENDER_LABEL, "" + showUserWithoutClass);
    
    return true;
	}

	@Override
	protected void performApply()
	{
	  actionSave();
	}

	@Override
	public boolean performOk()
	{		
		if (actionSave())
		  return super.performOk();
		return false;
	}

	@Override
	protected void performDefaults()
	{
		actionRestoreDefaults();
		updateApplyButton();
	}

	@Override
	public boolean performCancel()
	{
		return true;
	}

	@Override
	public void performHelp()
	{
		String helpContextId = Activator.PLUGIN_ID + ".chartEinstellungen";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	@Override
	public void init(IWorkbench workbench)
	{
	}

	@Override
	public void dispose()
	{
		super.dispose();

		/*
		 * Deregistrieren des Verbindungszuhrers
		 */
		if (_verbindungZuhoerer != null)
		{
			DaVVerbindung.getInstanz().entferneZuhoerer(_verbindungZuhoerer);
		}
	}
	
}
