/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridColumn;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.ImagePrint;
import net.sf.paperclips.LineBorder;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.PrintJob;
import net.sf.paperclips.ScalePrint;
import net.sf.paperclips.TextPrint;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.ExportCSV;
import de.bsvrz.buv.plugin.pua.basis.ExportXml;
import de.bsvrz.buv.plugin.pua.basis.ProtokollInfo;
import de.bsvrz.buv.plugin.pua.basis.combinedChart.IProtokollAusgabe;
import de.bsvrz.buv.plugin.pua.basis.combinedChart.ProtokollAusgabeManager;
import de.bsvrz.buv.plugin.pua.basis.expose.INachbereitung;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;
import de.bsvrz.buv.plugin.pua.basis.expose.ProjektInfoDefault;
import de.bsvrz.buv.plugin.pua.basis.extension.Bindung;
import de.bsvrz.buv.plugin.pua.basis.extension.ExtensionVerwaltung;
import de.bsvrz.buv.plugin.pua.basis.extension.SkriptInfo;
import de.bsvrz.buv.plugin.pua.basis.pua.ProtokolleUndAuswertungen;
import de.bsvrz.buv.plugin.pua.basis.pua.ZugriffPua;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.puaDataUpdateListener.IPuADataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.puaDataUpdateListener.PuADataUpdateEvent;
import de.bsvrz.buv.plugin.pua.basis.puaDataUpdateListener.PuADataUpdateListener;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.DefsSeiteneinstellungen;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog.SeitenDekoration;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.Column;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAdministrationData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAggregationData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAggregationResultData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolNoChanges;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResult;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.client.dataobject.Column.ColumnType;
import de.bsvrz.pua.prot.data.ProtocolData;
import de.bsvrz.pua.prot.processing.util.DataInformation;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.util.Tuple;

/**
 * /** Klasse zeigt das Ergebnis einer Auswertung in einer View an. Zustzlich werden Funktionalitten zum Drucken,
 * Abbrechen, Exportieren, etc. zur Verfgung gestellt.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-8</li>
 * </ul>
 * 
 * @version $Revision: 1.41 $ / $Date: 2014/02/18 15:31:14 $ / ($Author: Sans $)
 * @author HofmannJ
 * 
 */
public class ProtokollAusgabeSicht extends ViewPart implements IPuADataUpdateListener, IProtokollAusgabe
{
  /**
   * Die ID der View
   */
  public static final String ID = "de.bsvrz.buv.plugin.pua.protokollAusgabeSicht"; // TODO

  /**
   * Anzahl der zusaetzlichen Spalten (Datum + Uhrzeit, Objektbezeichner, usw.)
   */
  public final static int ZUSATZ_INFO_ANZAHL = 2;
  
  /**
   * Index zusaetzliche Spalte Datum + Uhrzeit
   */
  public final static int ZUSATZ_INFO_DATUMZEIT_INDEX = 0;

  /**
   * Index zusaetzliche Spalte Objektbezeichner
   */
  public final static int ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX = 1;

  public final static int INTERVALL_ENDE_ANZAHL_TRENNZEILEN = 2;
  
  public final static String TIMESTAMP_KEY = "zeitstempel";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * TableViewer
   */
  private TableViewer _tvAusgabe = null;
  
  /**
   * Tabellen-Daten
   */
  private List<String[]> _datenAusgabe;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private int[] _tvSpaltenSortierung = null;

  private Composite _parent;

  private Composite _top = null;

  private Composite compositeTop = null;
  
  private Composite composite = null;

  private Composite composite3 = null;

  private Button buttonOk = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Button buttonExport;

  private ProtokollInformationen _protokollInformationen; // @jve:decl-index=0:

  private ClientDavInterface _connection; // @jve:decl-index=0:

  private ProtokollInfo _protokollInfo; // @jve:decl-index=0:

  private Button buttonGanglinie;

  private ProtocolAdministrationData adminData; // @jve:decl-index=0:

  private ProgressBar progressBar = null;

  private Label infoTextNeu = null;
  
  private Button oeffentlichNeu = null;
  
  private Button aenderungsProtokollNeu = null;

  private Label bindungNeu = null;

  private Label archivdatenNeu = null;

  private Label simvarNeu = null;

  private Label objektNeu = null;

  private Label zeitbereichNeu = null;

  private String _secondaryId;
  
  private List<String[]> _letzteZellenwerteListe = null;
  
  private boolean _intervallEnde = false;

  private boolean _trennzeilenEinfuegen = false;
  
  private String[] _trennzeileZellenwerte = null;

  private List<String> _aggregationsdaten = null;
  
  private SkriptInfo _skriptInfo;
  
  private List<SystemObject> _anfrageObjekte = null;
  
  private int[] _objektZuordnung = null;
  
//  private Set _skriptAtg = null;
  
  private boolean _anfrageAktiv = false;
  
  private IProjektInfo _projektInfo = null;
  
  private INachbereitung _nachbereitung = null;
  

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".anzeige");

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _top = new Composite(_parent, SWT.NONE);
    createCompositeTop();
    _top.setLayout(gridLayout);
    createCompositeTable();
    createCompositeBottom();
    
    buttonOk.setEnabled(false);
    buttonExport.setEnabled(false);
    buttonGanglinie.setEnabled(false);

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    PuADataUpdateListener.getInstanz().addPuADataUpdateListener(this);

    _secondaryId = getViewSite().getSecondaryId();
  }

  /**
   * Zeigt die Hilfe zur View an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".anzeige";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * Schliet die View.
   * 
   */
  private void aktionSchliessen()
  {
    _debug.fine("Schliessen");
    spaltenSortierungSpeichern();
    
    getViewSite().getPage().hideView(ProtokollAusgabeSicht.this);
  }

  /**
   * Speichert die Spaltenreihenfolge ab.
   * 
   */
  private void spaltenSortierungSpeichern()
  {

    String ordnung = "(";
    // Auslesen der Spaltenordnung
    _tvSpaltenSortierung = _tvAusgabe.getTable().getColumnOrder();
    for (int i = 0; i < _tvSpaltenSortierung.length; i++)
    {
      ordnung = ordnung + "[" + _tvSpaltenSortierung[i] + "] ";
    }
    ordnung = ordnung + ")";
    _debug.fine("Spaltensortierung: " + ordnung);
  }

  /**
   * Ld die Sortierung der Spalten.
   * 
   */
  private void spaltenSortierungLaden()
  {
    // Setzen der Spaltenordnung
    _tvAusgabe.getTable().setColumnOrder(_tvSpaltenSortierung);
  }

  /**
   * This method initializes compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridLayout gridLayoutTop = new GridLayout();
    gridLayoutTop.numColumns = 1;
    gridLayoutTop.verticalSpacing = 0;
    GridData gridDataTop = new GridData();
    gridDataTop.grabExcessHorizontalSpace = true;
    gridDataTop.verticalAlignment = GridData.CENTER;
    gridDataTop.horizontalAlignment = GridData.FILL;
    compositeTop = new Composite(_top, SWT.NONE);
    compositeTop.setLayoutData(gridDataTop);
    compositeTop.setLayout(gridLayoutTop);
    
    GridLayout gridLayoutTopOben = new GridLayout();
    gridLayoutTopOben.numColumns = 3;
//    gridLayoutTopOben.marginHeight = 0;
    GridData gridDataTopOben = new GridData();
    gridDataTopOben.grabExcessHorizontalSpace = true;
    gridDataTopOben.verticalAlignment = GridData.CENTER;
    gridDataTopOben.horizontalAlignment = GridData.FILL;
    Composite compositeTopOben = new Composite(compositeTop, SWT.NONE);
    compositeTopOben.setLayoutData(gridDataTopOben);
    compositeTopOben.setLayout(gridLayoutTopOben);
    
    GridLayout gridLayoutTopUnten = new GridLayout();
    gridLayoutTopUnten.numColumns = 2;
    GridData gridDataTopUnten = new GridData();
    gridDataTopUnten.grabExcessHorizontalSpace = true;
    gridDataTopUnten.verticalAlignment = GridData.CENTER;
    gridDataTopUnten.horizontalAlignment = GridData.FILL;
    Composite compositeTopUnten = new Composite(compositeTop, SWT.NONE);
    compositeTopUnten.setLayoutData(gridDataTopUnten);
    compositeTopUnten.setLayout(gridLayoutTopUnten);
    
    Label infoText = new Label(compositeTopOben, SWT.NONE);
    infoText.setText("Informationstext:");
    
    GridData gridDataInfoTextNeu = new GridData();
    gridDataInfoTextNeu.grabExcessHorizontalSpace = true;
    gridDataInfoTextNeu.verticalAlignment = GridData.CENTER;
    gridDataInfoTextNeu.horizontalAlignment = GridData.FILL;
    infoTextNeu = new Label(compositeTopOben, SWT.NONE);
    infoTextNeu.setText("");
    infoTextNeu.setLayoutData(gridDataInfoTextNeu);
    
    GridLayout gridLayoutOeffentlich = new GridLayout();
    gridLayoutOeffentlich.numColumns = 1;
    gridLayoutOeffentlich.verticalSpacing = 0;
    gridLayoutOeffentlich.marginHeight = 0;
    gridLayoutOeffentlich.marginTop = 0;
    gridLayoutOeffentlich.marginWidth = 0;
    gridLayoutOeffentlich.marginBottom = 0;
    GridData gridDataOeffentlich = new GridData();
    gridDataOeffentlich.grabExcessHorizontalSpace = false;
    gridDataOeffentlich.verticalAlignment = GridData.CENTER;
    Composite compositeOeffentlich = new Composite(compositeTopOben, SWT.NONE);
    compositeOeffentlich.setLayoutData(gridDataOeffentlich);
    compositeOeffentlich.setLayout(gridLayoutOeffentlich);
    compositeOeffentlich.setEnabled(false);
    oeffentlichNeu = new Button(compositeOeffentlich, SWT.CHECK);
    oeffentlichNeu.setText("ffentlich");
    oeffentlichNeu.setEnabled(false);
    
    Label archivdaten = new Label(compositeTopOben, SWT.NONE);
    archivdaten.setText("Archivdatenart:");
    
    GridData gridDataArchivdatenNeu = new GridData();
    gridDataArchivdatenNeu.grabExcessHorizontalSpace = true;
    gridDataArchivdatenNeu.verticalAlignment = GridData.CENTER;
    gridDataArchivdatenNeu.horizontalAlignment = GridData.FILL;
    archivdatenNeu = new Label(compositeTopOben, SWT.NONE);
    archivdatenNeu.setText("");
    archivdatenNeu.setLayoutData(gridDataArchivdatenNeu);
    
    GridLayout gridLayoutAenderungsProtokoll = new GridLayout();
    gridLayoutAenderungsProtokoll.numColumns = 1;
    gridLayoutAenderungsProtokoll.verticalSpacing = 0;
    gridLayoutAenderungsProtokoll.marginHeight = 0;
    gridLayoutAenderungsProtokoll.marginTop = 0;
    gridLayoutAenderungsProtokoll.marginWidth = 0;
    gridLayoutAenderungsProtokoll.marginBottom = 0;
    GridData gridDataAenderungsProtokoll = new GridData();
    gridDataAenderungsProtokoll.grabExcessHorizontalSpace = false;
    gridDataAenderungsProtokoll.verticalAlignment = GridData.CENTER;
    Composite compositeAenderungsProtokoll = new Composite(compositeTopOben, SWT.NONE);
    compositeAenderungsProtokoll.setLayoutData(gridDataAenderungsProtokoll);
    compositeAenderungsProtokoll.setLayout(gridLayoutAenderungsProtokoll);
    compositeAenderungsProtokoll.setEnabled(false);
    aenderungsProtokollNeu = new Button(compositeAenderungsProtokoll, SWT.CHECK);
    aenderungsProtokollNeu.setText("nderungsprotokoll");
    aenderungsProtokollNeu.setEnabled(false);

    Label bindung = new Label(compositeTopOben, SWT.NONE);
    bindung.setText("Bindung:");
    
    GridData gridBindungNeu = new GridData();
    gridBindungNeu.grabExcessHorizontalSpace = true;
    gridBindungNeu.verticalAlignment = GridData.CENTER;
    gridBindungNeu.horizontalAlignment = GridData.FILL;
    bindungNeu = new Label(compositeTopOben, SWT.NONE);
    bindungNeu.setText("");
    bindungNeu.setLayoutData(gridBindungNeu);
    
    GridLayout gridLayoutSimvar = new GridLayout();
    gridLayoutSimvar.numColumns = 2;
    gridLayoutSimvar.verticalSpacing = 0;
    gridLayoutSimvar.marginHeight = 0;
    gridLayoutSimvar.marginTop = 0;
    gridLayoutSimvar.marginWidth = 0;
    gridLayoutSimvar.marginBottom = 0;
    GridData gridDataSimvar = new GridData();
    gridDataSimvar.grabExcessHorizontalSpace = false;
    gridDataSimvar.verticalAlignment = GridData.CENTER;
    Composite compositeSimvar = new Composite(compositeTopOben, SWT.NONE);
    compositeSimvar.setLayoutData(gridDataSimvar);
    compositeSimvar.setLayout(gridLayoutSimvar);
    
    Label simvar = new Label(compositeSimvar, SWT.NONE);
    simvar.setText("Simulationsvariante:");
    
//    GridData gridDataSimvarNeu = new GridData();
//    gridDataSimvarNeu.grabExcessHorizontalSpace = true;
//    gridDataSimvarNeu.verticalAlignment = GridData.CENTER;
//    gridDataSimvarNeu.horizontalAlignment = GridData.FILL;
    simvarNeu = new Label(compositeSimvar, SWT.NONE);
    simvarNeu.setText("      ");
//    simvarNeu.setLayoutData(gridDataSimvarNeu);
//    
    Label objekt = new Label(compositeTopUnten, SWT.NONE);
    objekt.setText("Objekte: ");
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = true;
    gridData10.verticalAlignment = GridData.CENTER;
    objektNeu = new Label(compositeTopUnten, SWT.NONE);
    objektNeu.setText("");
    objektNeu.setLayoutData(gridData10);
    
    Label zeitbereich = new Label(compositeTopUnten, SWT.NONE);
    zeitbereich.setText("Zeitbereiche: ");
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalAlignment = GridData.CENTER;
    zeitbereichNeu = new Label(compositeTopUnten, SWT.NONE);
    zeitbereichNeu.setText("");
    zeitbereichNeu.setLayoutData(gridData11);
    
    Label status = new Label(compositeTopUnten, SWT.NONE);
    status.setText("Status: ");
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    progressBar = new ProgressBar(compositeTopUnten, SWT.SMOOTH | SWT.HORIZONTAL);
    progressBar.setLayoutData(gridData12);
    // progressBar.setMinimum(0);
    progressBar.setMaximum(100);
    progressBar.addPaintListener(new PaintListener() {
      public void paintControl(PaintEvent e)
      {
        String string = "";

        if (progressBar.getSelection() == 0 && !_anfrageAktiv) {
        	string = "Warten auf Antwort vom Server...";
        	e.gc.setForeground(compositeTop.getDisplay().getSystemColor(SWT.COLOR_BLACK));
        }
        else {
        	if (progressBar.getSelection() != 100)
        		string = progressBar.getSelection() + "%";
            else
            	string = "Ausgabe abgeschlossen!";
        	e.gc.setForeground(compositeTop.getDisplay().getSystemColor(SWT.COLOR_WHITE));
        }

        Point point = progressBar.getSize();
        // Font font = new Font(compositeTop.getDisplay(),"Courier",10,SWT.BOLD);
        // e.gc.setFont(font);

//    	e.gc.setForeground(compositeTop.getDisplay().getSystemColor(SWT.COLOR_WHITE));
        //        
        FontMetrics fontMetrics = e.gc.getFontMetrics();
        int stringWidth = fontMetrics.getAverageCharWidth() * string.length();
        int stringHeight = fontMetrics.getHeight();

        e.gc.drawString(string, (point.x - stringWidth) / 2, (point.y - stringHeight) / 2, true);
        // font.dispose();
      }
    });
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTable()
  {
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    composite = new Composite(_top, SWT.NONE);
    composite.setLayout(new FillLayout());
    composite.setLayoutData(gridData1);

    // _tvBetriebsMeldungen = new TableViewer(composite, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAusgabe = new TableViewer(composite, SWT.FULL_SELECTION | SWT.BORDER);
    _tvAusgabe.setContentProvider(new TabelleInhaltsVersorger());
    _tvAusgabe.setLabelProvider(new TabelleDarstellungsVersorger());
    
    
  }

  /**
   * This method initializes composite3
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    rowLayout1.justify = false;
    rowLayout1.fill = false;
    rowLayout1.wrap = false;
    GridData gridData2 = new GridData();
    gridData2.grabExcessHorizontalSpace = false;
    gridData2.verticalAlignment = GridData.CENTER;
    gridData2.horizontalAlignment = GridData.END;
    composite3 = new Composite(_top, SWT.NONE);
    composite3.setLayoutData(gridData2);
    composite3.setLayout(rowLayout1);
    buttonExport = new Button(composite3, SWT.NONE);
    buttonExport.setText("Export");
    buttonExport.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        FileDialog fld = new FileDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.SAVE);
        fld.setText("CSV/XML Datei exportieren");
        fld.setFilterNames(new String[] { "CSV Datei", "XML Datei" });
        fld.setFilterExtensions(new String[] { "*.csv", "*.xml" });

        String filePath = fld.open();

        if (filePath != null && filePath.length() > 0)
        {
          File file = new File(filePath);

          if (file.getAbsoluteFile().toString().endsWith(".csv"))
          {
            ExportCSV exportCSV = new ExportCSV(_tvAusgabe.getTable());
            exportCSV.speichereCsv(file, _protokollInfo, getObjekte());
          }

          if (file.getAbsoluteFile().toString().endsWith(".xml"))
          {
            ExportXml exportXml = new ExportXml(_tvAusgabe.getTable());
            exportXml.speichereXml(file, _protokollInfo, getObjekte());
          }
        }
      }
    });

    buttonOk = new Button(composite3, SWT.NONE);
    buttonOk.setText("Drucken");
    buttonOk.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
//        Print print = createPrint(_tvBetriebsMeldungen.getTable());
//        PrintJob printJob = new PrintJob("abc", print);
        Print print = Tools.createPrint(_tvAusgabe.getTable(), getPartName());
        PrintJob printJob = new PrintJob(getPartName(), print);        
        DruckvorschauDialog dvd = new DruckvorschauDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
        dvd.oeffne(printJob);

      }

    });

    buttonGanglinie = new Button(composite3, SWT.NONE);
    buttonGanglinie.setText("Ganglinie");
    buttonGanglinie.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionGanglinieAuswahl();
      }
    });

    buttonSchliessen = new Button(composite3, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionSchliessen();
      }
    });

    buttonHilfe = new Button(composite3, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {

        aktionHilfe();

      }
    });
  }

  private void aktionGanglinieAuswahl()
  {
    // TODO Auto-generated method stub
    GanglinieAuswahlDialog fvd = new GanglinieAuswahlDialog(Display.getDefault().getActiveShell());

    Collection<SystemObject> objekte = getObjekte(); 
    
    if (adminData != null && _tvAusgabe != null && objekte != null) {
   	
    	String zusatzInfo = null;
    
        String informationsText = adminData.infoText;
        int beginIndex = informationsText.indexOf('[');
        if (beginIndex == 0)
        {
          int endIndex = informationsText.indexOf(']');
          String verborgenerText = informationsText.substring(beginIndex + 1, endIndex);
          String[] zerlegung = verborgenerText.split(":");
          if (zerlegung.length > 0)
          {
            String aktionsId = zerlegung[0];
            if (aktionsId != null) {
            	zusatzInfo = ExtensionVerwaltung.getInstanz().getAktionsText(aktionsId);
            }
          }
       }
   	
       fvd.oeffne(_projektInfo, _tvAusgabe.getTable(), objekte, zusatzInfo, getGesamtIntervall());
    }
  }

  public void erzeugeTabelle(List<String> listeSpalten)
  {
    TableColumn tc0 = new TableColumn(_tvAusgabe.getTable(), SWT.LEFT | SWT.Move);
    tc0.setText("Zeitstempel");
    // tc0.setWidth(100);
    tc0.setMoveable(true);
    tc0.addListener(SWT.Selection, new Listener() 
    {
      private int colPos = 0;
      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvAusgabe.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);
          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new TimeComparator(strings, i, colPos, item.getData(TIMESTAMP_KEY));
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }
    });

    TableColumn tc1 = new TableColumn(_tvAusgabe.getTable(), SWT.LEFT | SWT.Move);
    tc1.setText("Objekt");
    // tc1.setWidth(100);
    tc1.setMoveable(true);
    tc1.addListener(SWT.Selection, new Listener() 
    {
      private int colPos = 1;
      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvAusgabe.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);
          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos, item.getData(TIMESTAMP_KEY));
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }
    });

    for (int k = 0; k < listeSpalten.size(); k++)
    {
      TableColumn tc = new TableColumn(_tvAusgabe.getTable(), SWT.LEFT | SWT.Move);
      String s = listeSpalten.get(k);
      Point pt = new GC(_tvAusgabe.getTable()).textExtent(s);
      tc.setText(s);
      // tc.setWidth(pt.x);
      tc.setMoveable(true);

      final int fk = k + 2;

      tc.addListener(SWT.Selection, new Listener() {
        private int colPos = fk;

        private boolean sortFlag = false;

        public void handleEvent(Event event)
        {
          sortFlag = !sortFlag;
          Table table = _tvAusgabe.getTable();

          BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

          for (int i = 0; i < table.getItemCount(); i++)
          {
            TableItem item = table.getItem(i);

            String[] strings = new String[table.getColumnCount()];
            for (int j = 0; j < table.getColumnCount(); j++)
            {
              String text = item.getText(j);
              strings[j] = text;
            }

            comparables[colPos][i] = new StringComparator(strings, i, colPos, item.getData(TIMESTAMP_KEY));
          }

          sortTable(table, table.getItems(), colPos, comparables, sortFlag);
        }

      });
    }

    _tvAusgabe.getTable().setHeaderVisible(true);
    _tvAusgabe.getTable().setLinesVisible(true);

    TableColumn[] columns = _tvAusgabe.getTable().getColumns();

    for (TableColumn tc : columns)
    {
      tc.pack();
    }
    tc0.setWidth(115);
  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      String[] strings = (String[])object;

      if (columnIndex < strings.length)
        returnString = strings[columnIndex];
      else
        returnString = "";

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
//    String[] textBuffer = null;
//    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);

      // Sort timestamps contained in tableitem's dara
      tableItems[rowNum].setData(TIMESTAMP_KEY,  comparables[colPos][rowNum]._data);
      tableItems[i].setData(TIMESTAMP_KEY,  comparables[colPos][i]._data);
            
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */

  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;
    
    Object _data;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos, Object data)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
      _data = data;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt.
   */

  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer
     *          : Der Text der Zeile
     * @param rowNum
     *          : Die Zeilennr der Tabellenzeile
     * @param sortFlag
     *          : Aufsteigend false, Absteigend true
     * @param colPos
     *          : Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos, Object data)
    {
      super(textBuffer, rowNum, colPos, data);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * NumberComparator kmmert sich um das Vergleichen von Numerischen Werten.
   * 
   */

  class NumberComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public NumberComparator(String[] textBuffer, int rowNum, int colPos, Object data)
    {
      super(textBuffer, rowNum, colPos, data);
      try
      {
        _longValue = Long.parseLong(textBuffer[colPos]);
      }
      catch (Exception e)
      {
        // TODO: handle exception
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((NumberComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * TimeComparator kmmert sich um das Vergleichen von Datumsangaben.
   * 
   */

  class TimeComparator extends BaseComapator implements Comparable
  {

//    long _longValue;

    public TimeComparator(String[] textBuffer, int rowNum, int colPos, Object data)
    {
      super(textBuffer, rowNum, colPos, data);
//      try
//      {
////        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
//    	SimpleDateFormat sdf = INachbereitung.SDF;   
//        if (textBuffer[colPos].length() > 0)
//        {
//          Date d = sdf.parse(textBuffer[colPos]);
//          _longValue = d.getTime();
//        }
//      }
//      catch (Exception e)
//      {
//        // TODO: handle exception
//        e.printStackTrace();
//      }
    }

    public final int compareTo(Object o)
    {

      Long longValue = (Long)_data;
      Long longValToCompare = (Long)((TimeComparator)o)._data;
      int ret = -1;
      
      if (longValue != null && longValToCompare != null) {
        ret = longValue < longValToCompare ? -1 : (longValue == longValToCompare ? 0 : 1);
      }
      else if (longValue == null && longValToCompare == null) {
        ret = 0;
      }
      else if (longValToCompare == null) {
        ret = 1;
      }
      
//      long _longValToCompare = ((TimeComparator)o)._longValue;
//      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  @Override
  public void dispose()
  {
    ProtokollAusgabeManager.getInstance().unregister(this);
    
    PuADataUpdateListener.getInstanz().removePuADataUpdateListener(this);
    
    // Ggf. Pua-Verarbeitung abbrechen, wenn View geschlossen wird
    if (_anfrageAktiv && _protokollInfo != null && _connection != null) {
    	
    	long id = _protokollInfo.getProtokollId();
       	
       	try {
       		String msg;
       		ZugriffPua zugriffPua = ZugriffPua.getInstanz(_connection);
       		ProtocolRequestResult result = zugriffPua.abbruchProtokollerstellung(id);
       		if (result.isRequestSuccessful()) {
        		msg = "Pua-Protokoll " + id + " Ausgabe abgebrochen";
        		Activator.getDefault().getLog().log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
        	}
        	else {
        		msg = "Abbruch der Ausgabe von Pua-Protokoll " + id + " NICHT erfolgreich: " + result.getErrorMsg();
        		Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg));
        	}
	      	System.err.println(msg);
       	}
       	catch (RuntimeException e) {
       		String msg = "Fehler beim Abbruch der Ausgabe von Pua-Protokoll " + id;
	      	Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
	      	System.err.println(msg);
	      	e.printStackTrace(System.err);
       	}
    }
    
    super.dispose();
  }

  @Override
  public void puADataUpdate(PuADataUpdateEvent e)
  {
////    System.out.println("puaUpdate: " + e.getViewId() + " - " + getViewSite().getSecondaryId());
//    
//    String viewId = e.getViewId().toString();
//    String pid = e.getInfo().getSkript().getPid();
//    String s = pid + ";" + viewId;    
////    if (e.getViewId().toString().equals(getViewSite().getSecondaryId()))
//    if (s.equals(getViewSite().getSecondaryId()))
//    {
//      verarbeiteDatenVonPuA(e);
//    }
    
//  	System.out.println("puaUpdate: " + e.getViewId() + " - " + getViewSite().getSecondaryId());
  	
    String viewId = e.getViewId().toString();
    String aktionsId = null;
    String informationsText = e.getInfo().getInformationsText();
//    System.out.println("INFO_TEXT: " + informationsText);
  	int beginIndex = informationsText.indexOf('[');
  	if (beginIndex == 0) {
  		int endIndex = informationsText.indexOf(']');
  		String verborgenerText = informationsText.substring(beginIndex + 1, endIndex);
  		String[] zerlegung = verborgenerText.split(":");
  		if (zerlegung.length > 0) {
  			aktionsId = zerlegung[0];
  		}
  	}
  	if (aktionsId != null) {
  		viewId = aktionsId + ";" + viewId;
  	}
//  	System.out.println("AKTIONS_ID: " + aktionsId);
//  	System.out.println("SEC_ID: " + getViewSite().getSecondaryId());
  	
  	if (viewId.equals(getViewSite().getSecondaryId()))
    {
      verarbeiteDatenVonPuA(e);
    }
    
  }

  private String getEinheit(Data dataZeiger)
  {
    String ret = "";

    AttributeType attributTyp = dataZeiger.getAttributeType();

    if (attributTyp instanceof IntegerAttributeType)
    {
      IntegerValueRange integerValueRange = ((IntegerAttributeType)attributTyp).getRange();

      if (integerValueRange != null)
      {
        ret = integerValueRange.getUnit();
      }
    }
    else if (attributTyp instanceof DoubleAttributeType)
    {
      ret = ((DoubleAttributeType)attributTyp).getUnit();
    }

    return (ret);
  }
  
  @SuppressWarnings("unchecked")
  private void doAttributhierarchieAufloesen(Data dataZeiger, String kopfzeileBisher, List<String> kopfzeilen)
  {
    if (dataZeiger.isPlain())
    {
      String einheit = getEinheit(dataZeiger);
      if (einheit.length() > 0)
      {
        kopfzeilen.add(kopfzeileBisher + " [" + einheit + "]");
      }
      else
      {
        kopfzeilen.add(kopfzeileBisher);
      }
    }
    else
    {
      if (dataZeiger.isList())
      {
        Iterator<Data> it = dataZeiger.iterator();
        if (it.hasNext())
        {
          while (it.hasNext())
          {
            Data data = it.next();
            doAttributhierarchieAufloesen(data, kopfzeileBisher + "." + data.getName(), kopfzeilen);
          }
        }
        else
        {
          // hor9tr: Bugfix - Liste enthaelt keine weiteren Elemente -> kopfzeileBisher verwenden
          String einheit = getEinheit(dataZeiger);
          if (einheit.length() > 0)
          {
            kopfzeilen.add(kopfzeileBisher + " [" + einheit + "]");
          }
          else
          {
            kopfzeilen.add(kopfzeileBisher);
          }
        }
      }
      else if (dataZeiger.isArray())
      {
        Array array = dataZeiger.asArray();
        if(array.getLength() > 0)
        {
          for (int i = 0; i < array.getLength(); i++)
          {
            Data item = array.getItem(i);
            doAttributhierarchieAufloesen(item, kopfzeileBisher + "." + item.getName(), kopfzeilen);          
          }
        }
        else
        {
          // hor9tr: Bugfix - Array enthaelt keine weiteren Elemente -> kopfzeileBisher verwenden
          String einheit = getEinheit(dataZeiger);
          if (einheit.length() > 0)
          {
            kopfzeilen.add(kopfzeileBisher + " [" + einheit + "]");
          }
          else
          {
            kopfzeilen.add(kopfzeileBisher);
          }
        }
      }
      else
      {
        throw new RuntimeException("nicht behandelte Auspraegung von Administrationsdaten"); 
      }
    }
  }
  
  
  
  private void doZellenwerteAufloesen(ProtocolData protocolData, List<String> zellenwerteFlach)
  {
    if (protocolData.isPlain())
    {
      zellenwerteFlach.add(Tools.toText(protocolData));
    }
    else
    {
      if (protocolData.isList())
      {
        Iterator<Data> it = protocolData.iterator();
        while (it.hasNext())
        {
          doZellenwerteAufloesen((ProtocolData)it.next(), zellenwerteFlach);
        }
      }
      else if (protocolData.isArray())
      {
        Iterator<Data> it = protocolData.iterator();
        while (it.hasNext())
        {
          doZellenwerteAufloesen((ProtocolData)it.next(), zellenwerteFlach);
        }
      }
      else
      {
        throw new RuntimeException("Nicht behandelte Ausprgung von Zellendaten"); 
      }
    }
  }

  private void verarbeiteDatenVonPuA(PuADataUpdateEvent e)
  {
    ProtocolResult res = e.getResult();
        
//    System.out.println("Result: " + res);
    
    if (_protokollInformationen == null && _connection != null && res != null)
    {
      _protokollInformationen = e.getInfo();
      
      
      _skriptInfo = ExtensionVerwaltung.getInstanz().getSkriptInfo(_protokollInformationen);
      _projektInfo = _skriptInfo.createProjektInfo();
      _nachbereitung = _skriptInfo.createNachbereitung();
      
//      buttonGanglinie.setEnabled(_skriptInfo.isGanglinien());
      
      _protokollInfo = new ProtokollInfo(_protokollInformationen, new ProtokolleUndAuswertungen(_connection));
      _protokollInfo.setProtokollId(e.getProtokollId());
      
//      System.out.println("PROTOKOLL ID: " + _protokollInfo.getProtokollId());
      
      _anfrageAktiv = true;
      
      String msg = "Pua-Protokoll " + e.getProtokollId() + " Ausgabe gestartet";
      Activator.getDefault().getLog().log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
      System.err.println(msg);
    }
 
    if (res instanceof ProtocolAdministrationData)
    {
      adminData = (ProtocolAdministrationData)res;
      
      long[][] indices = adminData.dids;
      
      _anfrageObjekte = new ArrayList<SystemObject>();
      for (int i = 0; i < indices.length; i +=1)
      {
		long idObj = indices[i][0];
		_anfrageObjekte.add(_connection.getDataModel().getObject(idObj));
      }	
      
//      _skriptAtg = new HashSet<SystemObject>();
//      for (int i = 0; i < indices.length; i +=1)
//      {
//		long idObj = indices[i][1];
//		_skriptAtg.add(_connection.getDataModel().getObject(idObj));
//      }	
            
      _objektZuordnung = adminData.columnToDataId;
      
      _protokollInfo.setScriptName(adminData.scriptName);

      setzeAuswertungInfo();
      
      int k = 0;
      if (_secondaryId.contains(";"))
      {
        String[] split = _secondaryId.split(";");
        if (split.length > 1)
          k = new Integer(split[1]);      
      }else{
        k = new Integer(_secondaryId);            
      }
      k += 1;

      String partName = ExtensionVerwaltung.getInstanz().getAktionsText(_skriptInfo.getAktionsId());
      if (partName == null) {
    	  partName = getPartName();
      }
      else {
      	partName = "Ausgabe " + partName;
      }
      String infoText = _protokollInfo.getInformationsTextBereinigt();
      if (infoText != null && !infoText.isEmpty()) {
    	  partName += " - " + infoText;
      }
      setPartName(partName + " - " + k);
      
      if (_projektInfo == null){
    	  List<String> listeSpalten = new ArrayList<String>();
    	  String objekteKey = e.getInfo().getPseudoObjekte().keySet().toArray(new String[0])[0];
          String[] objekte = e.getInfo().getPseudoObjekte().get(objekteKey);

          for (int i = 0; i < adminData.columns.length; i += objekte.length)
          {
            String caption = adminData.columns[i].getCaption();
            Column col = adminData.columns[i];
            int index = adminData.columnToDataId[i];
            long idAtg = adminData.dids[index][1];
            SystemObject so = _connection.getDataModel().getObject(idAtg);
            String attr = col.getAttributeName();
            AttributeGroup atg = (AttributeGroup)so;
            Data data = _connection.createData(atg);

            switch(col.getType())
            {
              case REAL_ATTRIBUTE:
              case ATTRIBUTE_GROUP:
              // case TEMP_ATTRIBUTE:
                break;
              default:
                throw new RuntimeException("unbehandelter Attributtyp in der Spalte");
            }
            
            Data dataZeiger = data;
            if(col.getType() == ColumnType.REAL_ATTRIBUTE /*|| col.getType() == ColumnType.TEMP_ATTRIBUTE*/)
            {
              StringTokenizer zerleger = new StringTokenizer(attr, ".");
              while (zerleger.hasMoreTokens())
              {
                String attributEbene = zerleger.nextToken();
                
                if (attributEbene.contains("[")){
                  attributEbene = attributEbene.split("\\[")[0];
                }
//                String attributEbene = zerleger.nextToken(); // .split("\\[")[0]; // bei Feldern, den Index vernachlssigen
                dataZeiger = dataZeiger.getItem(attributEbene);
                if (zerleger.hasMoreTokens() && dataZeiger.isArray()) { 
              	// Falls dataZeiger auf ein Array zeigt und dieses Array noch 
              	// durch ein weiteres Attribut qualifiziert ist: 
                // erzeuge 1 Array-Element zur weitern Auswertung	   
                	dataZeiger.asArray().setLength(1);
                	dataZeiger = dataZeiger.asArray().getItem(0);
                }
              }
            }
            doAttributhierarchieAufloesen(dataZeiger, caption, listeSpalten);
          }
          _projektInfo = new ProjektInfoDefault(listeSpalten);
      }
    	  
//      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
//      if (dw != null)
//      {
//        IWorkbenchPage page = dw.getActivePage();
//        if (page != null)
//        {     
//          
//          String pid = _protokollInformationen.getSkript().getPid();          
//          String s = pid + ";" + e.getViewId();      
//          
////          System.out.println("suche Sec: " + s);
//          
//          IViewReference ref = page.findViewReference(ProtokollAusgabeSicht.ID, s);
//          if (ref != null)
//          {
//            final ProtokollAusgabeSicht view = (ProtokollAusgabeSicht)ref.getView(true);
//            if (view != null)
//            {
////              Display.getDefault().asyncExec(new Runnable() {
//            	Display.getDefault().syncExec(new Runnable() {
//                public void run()
//                {
//                  view.erzeugeTabelle(_projektInfo.getAusgabeTabelleSpalten());
//                }
//              });
//            }
//          }
//        }
//      }
      	Display.getDefault().syncExec(new Runnable() {
		      public void run()
		      {
		        erzeugeTabelle(_projektInfo.getAusgabeTabelleSpalten());
		      }
		    });

      _letzteZellenwerteListe = null;
    }
    
    if (res instanceof ProtocolAggregationData)
    {
      _aggregationsdaten = new ArrayList<String>();
      _aggregationsdaten.add("Aggregationsdaten erhalten");
      try
      {
        ProtocolAggregationData pad = (ProtocolAggregationData)res;
        String objekteKey = e.getInfo().getPseudoObjekte().keySet().toArray(new String[0])[0];
        String[] objekte = e.getInfo().getPseudoObjekte().get(objekteKey);
        List<List<String>> aggregationen = pad.getAggregations();
        _aggregationsdaten.add("  " + objekte.length + " Objekte");
        for (int i = 0; i < objekte.length; i++)
        {
          _aggregationsdaten.add("  Aggregationsdaten fr " + objekte[i].toString() + ":");
          if(aggregationen != null)
          {
            for(int j = i; j < aggregationen.size(); j += objekte.length)
            {
              _aggregationsdaten.add("    " + aggregationen.get(j).toString());
            }
          }
        }
      }
      catch(Exception ex)
      {
        _aggregationsdaten.add("  Fehler bei der Auswertung: " + ex.getLocalizedMessage());
      }
      _aggregationsdaten.add("");
      _aggregationsdaten.add("");
      _aggregationsdaten.add("");
    }
    
    if (res instanceof ProtocolAggregationResultData)
    {
      if (_aggregationsdaten != null)
      {
//        for (int i = 0; i < _aggregationsdaten.size(); i++)
//        {
//          System.out.println(_aggregationsdaten.get(i));
//        }
        _aggregationsdaten = null;
      }
      
//      System.out.println("Aggregationsergebnisdaten erhalten:");
      try
      {
        ProtocolAggregationResultData pard = (ProtocolAggregationResultData)res;
        String objekteKey = e.getInfo().getPseudoObjekte().keySet().toArray(new String[0])[0];
        String[] objekte = e.getInfo().getPseudoObjekte().get(objekteKey);
        List<ProtocolData> dataList = pard.getDataList(_connection.getDataModel());
//        System.out.println("  " + objekte.length + " Objekte");
//        for (int i = 0; i < objekte.length; i++)
//        {
//          System.out.println("  Aggregationsergebnisdaten fr " + objekte[i].toString() + ":");
//          for (int j = i; j < dataList.size(); j += objekte.length)
//          {
//            System.out.println("    " + dataList.get(j).toString());
//          }
//        }
      }
      catch(Exception ex)
      {
//        System.out.println("  Fehler bei der Auswertung: " + ex.getLocalizedMessage());
      }
//      System.out.println("");
//      System.out.println("");
//      System.out.println("");

    
      /* ProtocolAggregationResultData pard = (ProtocolAggregationResultData)res;
      String objekteKey = e.getInfo().getPseudoObjekte().keySet().toArray(new String[0])[0];
      String[] objekte = e.getInfo().getPseudoObjekte().get(objekteKey);
      List<ProtocolData> dataList = pard.getDataList(_connection.getDataModel());
      int anzahlAttribute = _tvBetriebsMeldungen.getTable().getColumnCount();
      if(anzahlAttribute >= ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL)
      {
        List<String[]> zellenwerteListe = new ArrayList<String[]>();
        for (int i = 0; i < objekte.length; i++)
        {
          String[] zellenwerte = new String[anzahlAttribute];

          // Datum + Objektbezeichner
          zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = "-";
          String pid = objekte[i].toString();
          if (_connection != null)
          {
            zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = _connection.getDataModel().getObject(pid).getName();
          }
          else
          {
            zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = pid;
          }

          // Attributwerte
          int position = ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL;
          for (int j = i; j < dataList.size() && position < anzahlAttribute; j += objekte.length)
          {
            ProtocolData protocolData = dataList.get(j);
            if (protocolData.isPlain())
            {
              zellenwerte[position] = toZellentext(protocolData);
              position++;
            }
            else
            {
              List<String> zellenwerteFlach = new ArrayList<String>();
              doZellenwerteAufloesen(protocolData, zellenwerteFlach);
              for (int k = 0; k < zellenwerteFlach.size() && position < anzahlAttribute; k++)
              {
                zellenwerte[position] = zellenwerteFlach.get(k);
                position++;
              }
            }
          }
          zellenwerteListe.add(zellenwerte);
        }

        for (int i = 0; i < zellenwerteListe.size(); i++)
        {
          TableItem tabellenzeile = new TableItem(_tvBetriebsMeldungen.getTable(), SWT.NONE);
          tabellenzeile.setText(zellenwerteListe.get(i));
        }
        
        updateAuswertungStatus(new Integer(pard.status));
      } */
    }
    
    if (res instanceof ProtocolNoChanges)
    {
      if (!_skriptInfo.isKeinNoChange()) {
          if (_letzteZellenwerteListe != null)
          {
            ProtocolNoChanges pnc = (ProtocolNoChanges)res;
//            System.out.println("NOCHANGE TS: " + INachbereitung.SDF.format(new Date(pnc.timestamp)));
            for (int i = 0; i < _letzteZellenwerteListe.size(); i++)
            {
              String[] letzteZellenwerte = _letzteZellenwerteListe.get(i);
              letzteZellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = INachbereitung.SDF.format(new Date(pnc.timestamp));
              TableItem tabellenzeile = new TableItem(_tvAusgabe.getTable(), SWT.NONE);
              tabellenzeile.setText(letzteZellenwerte);
              tabellenzeile.setData(TIMESTAMP_KEY, pnc.timestamp);
            }
          }
      }
    }

    if (res instanceof ProtocolResultData)
    {
      ProtocolResultData prd = (ProtocolResultData)res;
      
      int anzahlSpalten = _tvAusgabe.getTable().getColumnCount();
      if(anzahlSpalten >= ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL)
      {
        boolean intervallEnde = false;
        {
          if(prd.dataInformations != null && prd.dataInformations.length > 0)
          {
            intervallEnde = true;
            for(int i = 0; i < prd.dataInformations.length && intervallEnde; i++)
            {
              if(prd.dataInformations[i] == null)
              {
                intervallEnde = false;
              }
              else
              {
                if((prd.dataInformations[i].tag & DataInformation.END_OF_INTERVAL) == 0)
                {
                  intervallEnde = false;
                }
              }
            }
          }
        }

        if(_trennzeilenEinfuegen && !intervallEnde)
        {
          _trennzeilenEinfuegen = false;
          for(int i = 0; i < ProtokollAusgabeSicht.INTERVALL_ENDE_ANZAHL_TRENNZEILEN; i++)
          {
            TableItem tabellenzeile = new TableItem(_tvAusgabe.getTable(), SWT.NONE);
            tabellenzeile.setText(_trennzeileZellenwerte);
          }
        }
        
        if(intervallEnde)
        {
          if(_trennzeileZellenwerte == null)
          {
            _trennzeileZellenwerte = new String[anzahlSpalten];
            for(int i = 0; i < anzahlSpalten; i++)
            {
              _trennzeileZellenwerte[i] = "";
            }
          }
          _trennzeilenEinfuegen = true;
          _letzteZellenwerteListe = null;
        }
        
        if(!intervallEnde || !_intervallEnde)
        {
          _intervallEnde = intervallEnde;
                    
          List<String[]> zellenwerteListe = new ArrayList<String[]>();
          
          if (_nachbereitung != null) {
        	  zellenwerteListe.addAll(_nachbereitung.getZellenInhalte(_anfrageObjekte, 
        			  _objektZuordnung, prd, _projektInfo, _protokollInformationen));
          }
          else {
//        	  if (_skriptAtg != null && _skriptAtg.size() > 1) {
        		  List<ProtocolData> dataList = prd.getDataList(_connection.getDataModel());
            	  String zeitStempel = INachbereitung.SDF.format(new Date(prd.getTimeStamp()));
            	  Map<String, String[]> zellenwerteMap = new LinkedHashMap<String, String[]>();
            	  Map<String, Integer> positionsMap = new HashMap<String, Integer>();
            	  
            	  for (int i = 0; i < dataList.size(); i++)
                  {
            		  int objektIndex = _objektZuordnung[i];
            		  SystemObject anfrageObj = _anfrageObjekte.get(objektIndex);
            		  String pid = anfrageObj.getPid();
            		  String[] zellenwerte = zellenwerteMap.get(pid);
            		  
            		  if (zellenwerte == null) {
            			zellenwerte = new String[anzahlSpalten];
       					zellenwerteMap.put(pid, zellenwerte);
       					// Datum + Objektbezeichner
       	                zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
       	                String name = anfrageObj.getNameOrPidOrId();
       	                zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = name;
       	                positionsMap.put(pid,ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL);
            		  }
            		  
            		  int position = positionsMap.get(pid);
            		  ProtocolData protocolData = dataList.get(i);
            		  
            		  if (protocolData.isPlain())
                      {
                        zellenwerte[position] = Tools.toText(protocolData);
                        if (position < zellenwerte.length) {
                        	position++;	
                        }
                        else {
                        	System.err.println("Spaltenberlauf bei Spalte " + i + " Position " + 
                        			position + " fr Objekt " + pid);
                        }
                      }
                      else
                      {
                    	  List<String> zellenwerteFlach = new ArrayList<String>();
                          doZellenwerteAufloesen(protocolData, zellenwerteFlach);
                          // hor9tr: Bugfix - Array und Liste haben nur einen Header aber viele Werte -> Liste von Werten
                          //                  in einen flachen String kopieren und der Spalte des Headers hinzufuegen
                          if((anzahlSpalten - ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL) == 1 && zellenwerteFlach.size() > 1)
                          {
                            String zellenwert = zellenwerteFlach.get(0);
                            for(int m = 1; m < zellenwerteFlach.size(); m++)
                            {
                              zellenwert = zellenwert + ", " + zellenwerteFlach.get(m);
                            }
                            zellenwerteFlach.clear();
                            zellenwerteFlach.add(zellenwert);
                          }
                          for (int k = 0; k < zellenwerteFlach.size() && position < anzahlSpalten; k++)
                          {
                            zellenwerte[position] = zellenwerteFlach.get(k);
                            if (position < zellenwerte.length) {
                            	position++;	
                            }
                            else {
                            	System.err.println("Spaltenberlauf bei Spalte " + i + " Position " + 
                            			position + " fr Objekt " + pid);
                            	break;
                            }
                          }
                          
                          if (zellenwerteFlach.size() == 0) // Leere Zelle mit Leerstring belegen und dadurch berspringen
                          {
                            zellenwerte[position] = "";
                            
                            if (position < zellenwerte.length) 
                            {
                              position++;
                            }
                          }
                      }
            		  positionsMap.put(pid, position);
                  }	
            	  zellenwerteListe.addAll(zellenwerteMap.values());  
        	  }
//        	  else { // ORIGINAL CODE
//        		  
//        		  List<ProtocolData> dataList = prd.getDataList(_connection.getDataModel());
//            	  String zeitStempel = INachbereitung.SDF.format(new Date(prd.getTimeStamp()));
//              
//            	  for (int i = 0; i < _anfrageObjekte.size(); i++)
//                  {
//                    String[] zellenwerte = new String[anzahlAttribute];
//
//                    // Datum + Objektbezeichner
//                    zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
//                    String name = _anfrageObjekte.get(i).getNameOrPidOrId();
//                    zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = name;
//
//                    // Attributwerte
//                    int position = ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL;
//                    for (int j = i; j < dataList.size() && position < anzahlAttribute; j += _anfrageObjekte.size())
//                    {
//                      ProtocolData protocolData = dataList.get(j);
//                      if (protocolData.isPlain())
//                      {
//                        zellenwerte[position] = toZellentext(protocolData);
//                        position++;
//                      }
//                      else
//                      {
//                        List<String> zellenwerteFlach = new ArrayList<String>();
//                        doZellenwerteAufloesen(protocolData, zellenwerteFlach);
//                        // hor9tr: Bugfix - Array und Liste haben nur einen Header aber viele Werte -> Liste von Werten
//                        //                  in einen flachen String kopieren und der Spalte des Headers hinzufuegen
//                        if((anzahlAttribute - ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL) == 1 && zellenwerteFlach.size() > 1)
//                        {
//                          String zellenwert = zellenwerteFlach.get(0);
//                          for(int m = 1; m < zellenwerteFlach.size(); m++)
//                          {
//                            zellenwert = zellenwert + ", " + zellenwerteFlach.get(m);
//                          }
//                          zellenwerteFlach.clear();
//                          zellenwerteFlach.add(zellenwert);
//                        }
//                        for (int k = 0; k < zellenwerteFlach.size() && position < anzahlAttribute; k++)
//                        {
//                          zellenwerte[position] = zellenwerteFlach.get(k);
//                          position++;
//                        }
//                      }
//                    }
//                    zellenwerteListe.add(zellenwerte);
//                  }
//        	  }
//          }
          
          _letzteZellenwerteListe = zellenwerteListe;
          for (int i = 0; i < zellenwerteListe.size(); i++)
          {
            TableItem tabellenzeile = new TableItem(_tvAusgabe.getTable(), SWT.NONE);
            tabellenzeile.setText(zellenwerteListe.get(i));
            tabellenzeile.setData(TIMESTAMP_KEY, prd.getTimeStamp());
          }
          
          updateAuswertungStatus(prd.status);
        }
      }
      else
      {
        _letzteZellenwerteListe = null;
      }
    }

    if (res == null)
    {
      updateAuswertungStatus(100);
    }
  }

  private void updateAuswertungStatus(int soll)
  {
    int ist = progressBar.getSelection();

    if (soll != ist)
    {
        progressBar.setSelection(soll);
    }
    
    _anfrageAktiv = !(soll >= 100);
    
    buttonOk.setEnabled(soll >= 100);
    buttonExport.setEnabled(soll >= 100);
    buttonGanglinie.setEnabled(soll >= 100 && (_skriptInfo != null && _skriptInfo.isGanglinien()));
    
    if (soll >= 100 && (_skriptInfo != null && _skriptInfo.isGanglinien())) {
      ProtokollAusgabeManager.getInstance().register(this);
    }
  }

  private void setzeAuswertungInfo()
  {
    // System.out.println("ProtokollAusgabeSicht.setzeAuswertungInfo()");

    int i = 0;
    
    if (_secondaryId.contains(";"))
    {
      String[] split = _secondaryId.split(";");
      if (split.length > 1)
        i = new Integer(split[1]);      
    }else{
      i = new Integer(_secondaryId);            
    }
    
    i += 1;

//    setPartName(getPartName() + " " + _protokollInfo.getScriptName() + " " + i);
    
    infoTextNeu.setText(_protokollInfo.getInformationsTextBereinigt());
    infoTextNeu.setToolTipText(infoTextNeu.getText());
    
    oeffentlichNeu.setSelection(_protokollInfo.isOeffentlich());
    
    archivdatenNeu.setText(_protokollInfo.getArchivDatenArtAsString());
    
    aenderungsProtokollNeu.setSelection(_protokollInfo.isAenderungsProtokoll());
    
    bindungNeu.setText(bindungenAlsText());

    simvarNeu.setText("" + _protokollInfo.getSimulationsVariante());

    
    String s1 = getObjekteAlsString();
        
    String s21 = s1;
    if (s1.length() > 60){
      s21 = s1.substring(0, 59) + " ...";
    }
    
    objektNeu.setText(s21);
    objektNeu.setToolTipText(s1);

    zeitbereichNeu.setText(_protokollInfo.getZeitbereicheAsString());

//    compositeTop.layout(true);
  }

  String bindungenAlsText() {
	  String result = "";	  
	  Map<String, String> bindungsMap = _protokollInfo.getBindungen();
	  List<Bindung> bindungen = _skriptInfo.getBindungen();
	  for (String key: bindungsMap.keySet()) {
		  String wert = bindungsMap.get(key);
		  String bezeichner = key.startsWith("?") ? key.substring(1) : key;
		  Bindung bindung = null;
		  for (Bindung lauf: bindungen) {
			  if (bezeichner.equals(lauf.getBezeichner())) {
				  bindung = lauf;
				  break;
			  }
			  
		  }
		  if (bindung != null) {
			  Collection<String> namen = bindung.getAuswahlNamen();
			  for (String name: namen) {
				  if (wert.equals(bindung.getWertZuNamen(name))) {
					  if (!result.isEmpty()) {
						  result += ", ";
					  }
					  result += name;
				  }
			  }
		  }
	  }
	  
	  return result.isEmpty() ? "-" : result;
  }
  
  public static Print createPrint(Table table)
  {
    // Create GridPrint with all columns at default size.
    DefaultGridLook look = new DefaultGridLook();
    look.setCellBorder(new LineBorder());
    RGB background = table.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND).getRGB();
    look.setHeaderBackground(background);
    // look.setFooterBackground( background );

    GridPrint grid = new GridPrint(look);

    // Add header and footer to match table column names.
    TableColumn[] columns = table.getColumns();
    for (int i = 0; i < columns.length; i++)
    {
      TableColumn col = columns[i];

      // Add the column to the grid with alignment applied, default width, no
      // weight
      // if (i==0 || i == 1)
      grid.addColumn(new GridColumn(col.getAlignment(), SWT.DEFAULT, 0));
      // else
      // grid.addColumn( new GridColumn( col.getAlignment(), 30, 0 ) );

      Print cell = createCell(col.getImage(), col.getText(), SWT.CENTER);
      grid.addHeader(cell);
      // grid.addFooter( cell );
    }

    // Add content rows
    TableItem[] items = table.getItems();
    for (int i = 0; i < items.length; i++)
    {
      TableItem item = items[i];
      for (int j = 0; j < columns.length; j++)
        grid.add(createCell(item.getImage(j), item.getText(j), columns[j].getAlignment()));
    }

    ScalePrint scalePrint = new ScalePrint(grid);

    PagePrint page = new PagePrint(scalePrint);

    de.bsvrz.buv.rw.basislib.Activator activator = de.bsvrz.buv.rw.basislib.Activator.getDefault();

    IPreferenceStore preferenceStore = activator.getPreferenceStore();

    page.setHeader(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileLinks),
        preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileMitte), preferenceStore
            .getString(DefsSeiteneinstellungen.kopfzeileRechts), true, "Eine Auswertung"));
    page.setHeaderGap(5);

    page.setFooter(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks),
        preferenceStore.getString(DefsSeiteneinstellungen.fusszeileMitte), preferenceStore
            .getString(DefsSeiteneinstellungen.fusszeileRechts), false));
    page.setFooterGap(5);

    return page;
  }

  public static Print createCell(Image image, String text, int align)
  {
    if (image == null)
      return new TextPrint(text, align);

    GridPrint grid = new GridPrint("p, d");
    grid.add(new ImagePrint(image.getImageData(), image.getDevice().getDPI()));
    grid.add(new TextPrint(text, align));
    return grid;
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }
  
  public String getSecondaryId()
  {
    return _secondaryId;
  }
  
  private String getObjekteAlsString() {
	  
	  String result = "";;
	  
	  if (_nachbereitung != null) {
		  for (SystemObject so: _nachbereitung.getObjekte(_protokollInfo.getObjekte(), _protokollInformationen)) {
			  String name = so.getNameOrPidOrId();
			  result += result.isEmpty() ? name : ", " + name;	
		  }
	  }
	  else {
		  result = _protokollInfo.getObjekteAsString(); 
	  }
	  
	  return result;
  }
  
  public Collection<SystemObject> getObjekte() {
	  
	  if (_nachbereitung != null) {
		  return _nachbereitung.getObjekte(_protokollInfo.getObjekte(), _protokollInformationen);
	  }	  
	  else {
		 return _protokollInfo.getObjekte(); 
	  }
  }
  
  public String getId() {
    return "" + _protokollInfo.getProtokollId();
  }
  
  public String getName() {
    return getPartName();
  }
  
  public Table getTabelle() {
    return _tvAusgabe.getTable();
  }
  
  public IProjektInfo getProjektInfo() {
    return _projektInfo;
  }
  
  public Tuple<Long, Long> getGesamtIntervall() {
    
    Long anfang = null;
    Long ende = null;
    
    for (Tuple<Long, Long> intervall: _protokollInformationen.getIntervalle()) {
      if (anfang == null || intervall.first < anfang) {
        anfang = intervall.first;
      }
      if (ende == null || intervall.last > ende) {
        ende = intervall.last;
      }
    }
    
    return anfang != null && ende != null ? new Tuple<Long,Long>(anfang, ende) : null;
  }
 
  
} // @jve:decl-index=0:visual-constraint="10,10,847,295"
