/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridColumn;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.ImagePrint;
import net.sf.paperclips.LineBorder;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.ScalePrint;
import net.sf.paperclips.TextPrint;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.AnfrageThread;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.ProtokollInfo;
import de.bsvrz.buv.plugin.pua.basis.protokolleUpdateListener.IProtokolleUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.protokolleUpdateListener.ProtokolleUpdateEvent;
import de.bsvrz.buv.plugin.pua.basis.pua.ProtokolleUndAuswertungen;
import de.bsvrz.buv.plugin.pua.basis.pua.ZugriffPua;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.puaDataUpdateListener.PuADataUpdateEvent;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.DefsSeiteneinstellungen;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog.SeitenDekoration;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.pua.prot.client.ProtocolListener;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAdministrationData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAggregationData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolNoChanges;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResult;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.processing.util.DataInformation;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.datk.AtlBase;
import de.bsvrz.sys.funclib.losb.datk.AtlMeta;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;
import de.bsvrz.sys.funclib.losb.util.Util;

/**
 * Nach Auswahl des Menpunkts "Protokolle und Auswertungen/Ungelesene Protokolle" in der Rahmenapplikation muss sich 
 * die View zur Anzeige der ungelesenen Protokolle ffnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-8</li>
 * </ul>
 * 
 * @version $Revision: 1.5 $ / $Date: 2012/12/21 09:56:27 $ / ($Author: Pittner $)
 * @author HofmannJ
 * 
 */
public class UngeleseneProtokolleSicht extends ViewPart implements IProtokolleUpdateListener, ProtocolListener
{
  /**
   * Die ID der View
   */
  public static final String ID = "de.bsvrz.buv.plugin.pua.ungeleseneProtokolleSicht"; // TODO

  private static final long AKTUALISIERE_ZEIT_MINIMUM = 5000;
  
  private final Object AKTUALISIERE_SYNC = new Object();

  private final Object BERVORSTEHENDE_AKTUALISIERUNGEN_SYNC = new Object();

  private volatile int _bevorstehendeAktualisierungen = 0;
  
  private volatile boolean _disposed = false;

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  /**
   * TableViewer
   */
  private TableViewer _tvBetriebsMeldungen = null;

  /**
   * Gibt die Sortierreihenfolge der Spalten an.
   */
  private static int[] _tvSpaltenSortierung = null;
  
  private Composite _parent;
  
  private Composite _top = null;

  private Composite compositeTop = null;

  private Composite composite = null;

  private Composite composite3 = null;

  private Button buttonOk = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Label zeitpunkt = null;

  private SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private Button buttonExport;

  private ProtokollInformationen _protokollInformationen; // @jve:decl-index=0:

  private ClientDavInterface _connection; // @jve:decl-index=0:

  private ProtokollInfo _protokollInfo; // @jve:decl-index=0:

  private int _anzahlSpalten;

  // private ProtokolleUndAuswertungen _protokolleUndAuswertungen; // @jve:decl-index=0:

  private ZugriffPua _zugriffPuA; // @jve:decl-index=0:

  private Integer _numberInstances = 0;

  private ProtokollFertiggestelltVerarbeiter protokollFertiggestelltVerarbeiter = null; 

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".ungeleseneProtokolle" );

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _top = new Composite(_parent, SWT.NONE);
    createCompositeTop();
    _top.setLayout(gridLayout);
    createCompositeTable();
    createCompositeBottom();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    if (_connection != null)
    {
      _zugriffPuA = ZugriffPua.getInstanz(_connection);
      protokollFertiggestelltVerarbeiter = new ProtokollFertiggestelltVerarbeiter();
      protokollFertiggestelltVerarbeiter.start();
      addProtokollListener(this);
      aktualisiereUngeleseneProtokolle(BenutzerAngemeldet.getInstanz().getName());
    }
    else
      System.err.println("Connection ist null!");
  }

  /**
   * Klasse zum Aktualisieren der Tabelle. Die Zusammenfhrung der Zellendaten geschieht ausserhalb 
   * des SWT-Thread. Die reine zeitunkritische Aktualisierung im SWT-Thread.
   * <br>
   * 
   * @version $Revision:
   * 
   * @author SWARCO TRAFFIC SYSTEMS GmbH
   * @author Ralf Horst
   * 
   */
  private class AktualisiereUngeleseneProtokolle extends Thread
  {
    private UngeleseneProtokolleSicht m_ungeleseneProtokolleSicht = null;
    private String m_benutzer = null;
    
    private AktualisiereUngeleseneProtokolle(UngeleseneProtokolleSicht ungeleseneProtokolleSicht, String benutzer)
    {
      m_ungeleseneProtokolleSicht = ungeleseneProtokolleSicht;
      m_benutzer = benutzer;
    }
    
    public void run()
    {
      synchronized(AKTUALISIERE_SYNC)
      {
        synchronized(BERVORSTEHENDE_AKTUALISIERUNGEN_SYNC)
        {
          _bevorstehendeAktualisierungen--;
        }

        if(!_disposed)
        {
          boolean success = false;
          if(m_ungeleseneProtokolleSicht != null)
          {
            try
            {
              success = _zugriffPuA.aktualisiereListeUngeleseneProtokolle(m_benutzer);
            }
            catch(Exception e)
            {
              _debug.error("Fehler beim Aktualisieren der Liste der ungelesenen Protokolle.");
            }
          }
          
          if(!_disposed && success)
          {
            System.out.println("Aktualisiere jetzt");      
            String stand = "Stand: " + _sdf.format(new Date());
            Vector<ProtokollInformationen> vProtokollInformationen = _zugriffPuA.getListeUngeleseneProtokolle();
            Vector<Object[]> vTabellenInhalt = new Vector<Object[]>();
            for(ProtokollInformationen protokollInformationen : vProtokollInformationen)
            {
              Object[] zeile = new Object[6];

              ProtokollInfo info = new ProtokollInfo(protokollInformationen, new ProtokolleUndAuswertungen(_connection));
              zeile[0] = info.getZeitbereicheAsString();
              zeile[1] = protokollInformationen.getErsteller();
              zeile[2] = protokollInformationen.getInformationsTextBereinigt();
              zeile[3] = protokollInformationen.isOeffentlich() ? "ja" : "nein";
              if(!_disposed)
              {
                AtlMeta meta = new AtlMeta();
                ProtocolRequestResult requestResult = _zugriffPuA.holeSkriptInformationen((DynamicObject)protokollInformationen.getSkript(), new StringBuffer(), meta);
                if(requestResult != null)
                {
                  zeile[4] = meta.atlBase.description;
                }
                else
                {
                  zeile[4] = "";
                }
              }
              else
              {
                System.out.println("Aktualisierungsabbruch wegen dispose");      
                break;
              }
              zeile[5] = protokollInformationen;
              
              vTabellenInhalt.add(zeile);
            }
            if(!_disposed)
            {
              final Object[] objects = new Object[]{vTabellenInhalt, stand};
              Display.getDefault().syncExec(new Runnable()
              {
                public void run()
                {
                  try
                  {
                    System.out.println("Aktualisiere Tabelleninhalt");      
                    aktualisiereView(objects);
                  }
                  catch(Exception e)
                  {
                  }
                }
              });
              for(int i = 0; i < 20 && !_disposed; i++)
              {
                try
                {
                  Thread.sleep(50);
                }
                catch(Exception e)
                {
                }
              }
            }
          }
        }
        
        System.out.println("Aktualisierung beendet: " + _disposed);      
      }
      
      m_ungeleseneProtokolleSicht = null;
      m_benutzer = null;
    }
  }

  /**
   * Stsst das aktualisieren an. Wird sowohl vom SWT (Aktualisieren-Knopf) als auch vom ProtocolListener
   * aufgerufen. Es kann immer nur ein Aktualisierungsthread gerade laufen und ein weiterer gestartet sein.
   * Mehr als diese zwei Threads sollten nicht laufen.
   * 
   * @param benutzer Benutzer, der die ungelesenen Protokolle abruft.
   */
  private void aktualisiereUngeleseneProtokolle(String benutzer)
  {
    synchronized(BERVORSTEHENDE_AKTUALISIERUNGEN_SYNC)
    {
      System.out.println("Bevorstehende Aktualisierungen: " + (_bevorstehendeAktualisierungen + 1));      
      if(_bevorstehendeAktualisierungen <= 0)
      {
        _bevorstehendeAktualisierungen++;
        (new AktualisiereUngeleseneProtokolle(this, benutzer)).start();
      }
    }
  }

  /**
   * Zeigt die Hilfe zur View an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".ungeleseneProtokolle";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * Schliet die View.
   * 
   */
  private void aktionSchliessen()
  {
    _debug.fine("Schliessen");
    spaltenSortierungSpeichern();
    getViewSite().getPage().hideView(UngeleseneProtokolleSicht.this);
  }

  /**
   * Speichert die Spaltenreihenfolge ab.
   * 
   */
  private void spaltenSortierungSpeichern()
  {

    String ordnung = "(";
    // Auslesen der Spaltenordnung
    _tvSpaltenSortierung = _tvBetriebsMeldungen.getTable().getColumnOrder();
    for (int i = 0; i < _tvSpaltenSortierung.length; i++)
    {
      ordnung = ordnung + "[" + _tvSpaltenSortierung[i] + "] ";
    }
    ordnung = ordnung + ")";
    _debug.fine("Spaltensortierung: " + ordnung);
  }

  /**
   * This method initializes compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 1;
    GridData gridData = new GridData();
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    gridData.horizontalAlignment = GridData.FILL;
    compositeTop = new Composite(_top, SWT.NONE);
    compositeTop.setLayoutData(gridData);
    compositeTop.setLayout(gridLayout1);
    zeitpunkt = new Label(compositeTop, SWT.NONE);
    zeitpunkt.setText("Stand:");
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTable()
  {
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    composite = new Composite(_top, SWT.NONE);
    composite.setLayout(new FillLayout());
    composite.setLayoutData(gridData1);

    erzeugeTabelle();
  }

  /**
   * This method initializes composite3
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    rowLayout1.justify = false;
    rowLayout1.fill = false;
    rowLayout1.wrap = false;
    GridData gridData2 = new GridData();
    gridData2.grabExcessHorizontalSpace = false;
    gridData2.verticalAlignment = GridData.CENTER;
    gridData2.horizontalAlignment = GridData.END;
    composite3 = new Composite(_top, SWT.NONE);
    composite3.setLayoutData(gridData2);
    composite3.setLayout(rowLayout1);
    buttonExport = new Button(composite3, SWT.NONE);
    buttonExport.setText("Aktualisieren");
    buttonExport.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktualisiereUngeleseneProtokolle(BenutzerAngemeldet.getInstanz().getName());
      }
    });

    buttonOk = new Button(composite3, SWT.NONE);
    buttonOk.setText("Ausgeben");
    buttonOk.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      @SuppressWarnings("unchecked")
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        IStructuredSelection selection = (IStructuredSelection)_tvBetriebsMeldungen.getSelection();

        if(selection.size() > 0)
        {
          Iterator it = selection.iterator();
          while(it.hasNext())
          {
            Object object = it.next();
            if(object instanceof Object[] && ((Object[])object).length == 6 && ((Object[])object)[5] instanceof ProtokollInformationen)
            {
              ProtokollInformationen protokollInformationen = (ProtokollInformationen)(((Object[])object)[5]);
              protokollInformationen.setProtokollSpeichern(false);

              IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
              IWorkbenchPage page = dw.getActivePage();
              try
              {
                String pid = protokollInformationen.getSkript().getPid();
                String s = pid + ";" + _numberInstances.toString();
                page.showView(ProtokollAusgabeSicht.ID, s, IWorkbenchPage.VIEW_VISIBLE);
                _numberInstances++;
              }
              catch(PartInitException pie)
              {
                pie.printStackTrace();
              }
              new AnfrageThread(protokollInformationen, _numberInstances - 1);
            }
          }
        }
        else
        {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
          mb.setText("Fehler Ausgeben Protokolle");
          mb.setMessage("Es mssen Protokolle in der Tabelle ausgewhlt werden");
          mb.open();
        }
      }
    });

    buttonSchliessen = new Button(composite3, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionSchliessen();
      }
    });

    buttonHilfe = new Button(composite3, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionHilfe();
      }
    });
  }

  public void erzeugeTabelle()
  {
    _tvBetriebsMeldungen = new TableViewer(composite, SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);

    TableColumn tc0 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc0.setText("Zeitbereich:");
    tc0.setWidth(200);
    tc0.setMoveable(true);
    tc0.addListener(SWT.Selection, new Listener() {
      private int colPos = 0;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;

          }

          comparables[colPos][i] = new TimeComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc1 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc1.setText("Ersteller:");
    tc1.setWidth(200);
    tc1.setMoveable(true);
    tc1.addListener(SWT.Selection, new Listener() {
      private int colPos = 1;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc2 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc2.setText("Informationstext:");
    tc2.setWidth(400);
    tc2.setMoveable(true);
    tc2.addListener(SWT.Selection, new Listener() {
      private int colPos = 2;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc3 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc3.setText("ffentlich:");
    tc3.setWidth(100);
    tc3.setMoveable(true);
    tc3.addListener(SWT.Selection, new Listener() {
      private int colPos = 3;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    TableColumn tc4 = new TableColumn(_tvBetriebsMeldungen.getTable(), SWT.LEFT | SWT.Move);
    tc4.setText("Beschreibung:");
    tc4.setWidth(200);
    tc4.setMoveable(true);
    tc4.addListener(SWT.Selection, new Listener() {
      private int colPos = 4;

      private boolean sortFlag = false;

      public void handleEvent(Event event)
      {
        sortFlag = !sortFlag;
        Table table = _tvBetriebsMeldungen.getTable();

        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem(i);

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText(j);
            strings[j] = text;
          }

          comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    });

    _tvBetriebsMeldungen.getTable().setHeaderVisible(true);
    _tvBetriebsMeldungen.getTable().setLinesVisible(true);
    _tvBetriebsMeldungen.setContentProvider(new TabelleInhaltsVersorger());
    _tvBetriebsMeldungen.setLabelProvider(new TabelleDarstellungsVersorger());
    _tvBetriebsMeldungen.addSelectionChangedListener(new ISelectionChangedListener()
    {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
      }
    });
  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String ret = null;

      try
      {
        ret = (String)(((Object[])object)[columnIndex]);
        if(ret == null)
        {
          ret = "";
          throw new RuntimeException();
        }
      }
      catch(Exception e)
      {
        _debug.error("Fehler beim Fuellen der Tabelle (" + object + "; " + columnIndex + ") in UngeleseneProtokolleSicht.");
      }
      
      return ret;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */

  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt.
   */

  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer :
     *          Der Text der Zeile
     * @param rowNum :
     *          Die Zeilennr der Tabellenzeile
     * @param sortFlag :
     *          Aufsteigend false, Absteigend true
     * @param colPos :
     *          Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * NumberComparator kmmert sich um das Vergleichen von Numerischen Werten.
   * 
   */

  class NumberComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public NumberComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        _longValue = Long.parseLong(textBuffer[colPos]);
      }
      catch (Exception e)
      {
        // TODO: handle exception
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((NumberComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * TimeComparator kmmert sich um das Vergleichen von Datumsangaben.
   * 
   */

  class TimeComparator extends BaseComapator implements Comparable
  {

    long _longValue;

    public TimeComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
      try
      {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
        if (textBuffer[colPos].length() > 0)
        {
          Date d = sdf.parse(textBuffer[colPos]);
          _longValue = d.getTime();
        }
      }
      catch (Exception e)
      {
        // TODO: handle exception
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o)
    {

      long _longValToCompare = ((TimeComparator)o)._longValue;
      int ret = _longValue < _longValToCompare ? -1 : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  @Override
  public void dispose()
  {
    System.out.println("dispose ungelesene protokolle sicht");
    removeProtokollListener(this);
    if(protokollFertiggestelltVerarbeiter != null)
    {
      protokollFertiggestelltVerarbeiter.beenden();
    }
    _disposed = true;
    synchronized(AKTUALISIERE_SYNC)
    {
    }
    super.dispose();
    // _aktualisiereThread.stop();
    // ProtokolleUpdateListener.getInstanz().removeProtokolleUpdateListener(this);
  }

  public static Print createPrint(Table table)
  {
    // Create GridPrint with all columns at default size.
    DefaultGridLook look = new DefaultGridLook();
    look.setCellBorder(new LineBorder());
    RGB background = table.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND).getRGB();
    look.setHeaderBackground(background);
    // look.setFooterBackground( background );

    GridPrint grid = new GridPrint(look);

    // Add header and footer to match table column names.
    TableColumn[] columns = table.getColumns();
    for (int i = 0; i < columns.length; i++)
    {
      TableColumn col = columns[i];

      grid.addColumn(new GridColumn(col.getAlignment(), SWT.DEFAULT, 0));

      Print cell = createCell(col.getImage(), col.getText(), SWT.CENTER);
      grid.addHeader(cell);
    }

    // Add content rows
    TableItem[] items = table.getItems();
    for (int i = 0; i < items.length; i++)
    {
      TableItem item = items[i];
      for (int j = 0; j < columns.length; j++)
        grid.add(createCell(item.getImage(j), item.getText(j), columns[j].getAlignment()));
    }

    ScalePrint scalePrint = new ScalePrint(grid);

    PagePrint page = new PagePrint(scalePrint);

    Activator activator = Activator.getDefault();

    IPreferenceStore preferenceStore = activator.getPreferenceStore();

    page.setHeader(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileLinks),
        preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileMitte), preferenceStore
            .getString(DefsSeiteneinstellungen.kopfzeileRechts), true, "Eine Auswertung"));
    page.setHeaderGap(5);

    page.setFooter(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks),
        preferenceStore.getString(DefsSeiteneinstellungen.fusszeileMitte), preferenceStore
            .getString(DefsSeiteneinstellungen.fusszeileRechts), false));
    page.setFooterGap(5);

    return page;
  }

  public static Print createCell(Image image, String text, int align)
  {
    if (image == null)
      return new TextPrint(text, align);

    GridPrint grid = new GridPrint("p, d");
    grid.add(new ImagePrint(image.getImageData(), image.getDevice().getDPI()));
    grid.add(new TextPrint(text, align));
    return grid;
  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  
  }

  @Override
  public void protokolleDataUpdate(ProtokolleUpdateEvent e)
  {
    final Object[] objects = (Object[])e.getResult();

    Display.getDefault().asyncExec(new Runnable() {
      public void run()
      {
        aktualisiereView(objects);
      }
    });

  }

  private void aktualisiereView(Object[] objects)
  {
    _tvBetriebsMeldungen.setInput(objects[0]);
    zeitpunkt.setText((String)objects[1]);
    compositeTop.layout(true);
  }

  private boolean addProtokollListener(final ProtocolListener listener)
  {
    boolean ret = false;
    
    try
    {
      ProtocolRequestResult prr = _zugriffPuA.addProtokollListener(listener);
      ret = prr.isRequestSuccessful();
    }
    catch(FailureException fe)
    {
      _debug.error("Fehler beim Anmelden als ProtocolListener fr ungelesene Protokolle.");
    }
    
    return ret; 
  }

  public boolean removeProtokollListener(final ProtocolListener listener)
  {
    boolean ret = false;
    
    try
    {
      ProtocolRequestResult prr = _zugriffPuA.removeProtokollListener(listener);
      ret = prr.isRequestSuccessful();
    }
    catch(FailureException fe)
    {
      _debug.error("Fehler beim Abmelden als ProtocolListener fr ungelesene Protokolle.");
    }
    
    return ret; 
  }

  /**
   * Klasse die auf ProtocolListener-Ereignisse wartet und dann das Aktualisieren anstoesst.
   * <br>
   * 
   * @version $Revision:
   * 
   * @author SWARCO TRAFFIC SYSTEMS GmbH
   * @author Ralf Horst
   * 
   */
  private class ProtokollFertiggestelltVerarbeiter extends Thread
  {
    private volatile boolean gestartet = false;
    private volatile Object startenSync = new Object();
    private volatile boolean beenden = false;
    private volatile Object beendenSync = new Object();
    private volatile int ausloeser = 0;
    private volatile Object ausloeserSync = new Object();
    
    private ProtokollFertiggestelltVerarbeiter()
    {
    }

    private void beenden()
    {
      System.out.println("starte verarbeiter beenden");
      beenden = true;
      synchronized(ausloeserSync)
      {
        ausloeserSync.notify();
      }
      synchronized(beendenSync)
      {
      }
      System.out.println("verarbeiter beendet");
    }
    
    private void ausloesen()
    {
      synchronized(ausloeserSync)
      {
        ausloeser++;
        ausloeserSync.notify();
      }
    }
    
    public void run()
    {
      synchronized(beendenSync)
      {
        while(!beenden)
        {
          System.out.println("verarbeiter starte zyklus");
          synchronized(ausloeserSync)
          {
            System.out.println("verarbeiter warte auf erstes ausloesen");
            while(!beenden && ausloeser <= 0)
            {
              try
              {
                ausloeserSync.wait();
              }
              catch(InterruptedException ie)
              {
              }
            }
            
            System.out.println("verarbeiter warte bis ausloesen ende");
            long letzterAusloeser = 0;
            do
            {
              if(!beenden)
              {
                letzterAusloeser = ausloeser;
                try
                {
                  ausloeserSync.wait(UngeleseneProtokolleSicht.AKTUALISIERE_ZEIT_MINIMUM);
                }
                catch(InterruptedException ie)
                {
                }
              }
            }
            while(!beenden && letzterAusloeser != ausloeser);
            
            ausloeser = 0;
          }
          
          try
          {
            if(!beenden)
            {
              System.out.println("verarbeiter aktualisiere tabelle");
              aktualisiereUngeleseneProtokolle(BenutzerAngemeldet.getInstanz().getName());
            }
          }
          catch(Exception e)
          {
          }
        }
        System.out.println("verarbeiter thread beendet");
      }
    }
  }
  
  @Override
  public void protocolFinished(long protocolId)
  {
//    System.out.println("protocol finished: " + protocolId);
    String msg = "Ungelesene Protokolle: protocolFinished("+ protocolId + ")";
    Activator.getDefault().getLog().log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
    System.err.println(msg);

    protokollFertiggestelltVerarbeiter.ausloesen();
  }
} // @jve:decl-index=0:visual-constraint="10,10,847,295"
