/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.graphics.RGB;

import de.bsvrz.buv.plugin.selektion.SelektionsTransfer;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AchsenBeschreibung;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.DiagrammEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.RealerStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Strecke;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenEintrag;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenKnoten;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenTeilAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenZug;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilFactory;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypDatenQuellen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypVerkehrsDaten;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.util.LoadModel;
import de.bsvrz.buv.rw.bitctrl.BitCtrlRahmenwerkConstants;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.att.WerteBereich;
import de.bsvrz.sys.funclib.bitctrl.modell.fachmodellglobal.attribute.AttProzent;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttBemessungsFahrzeugDichte;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttBemessungsVerkehrsStaerkeStunde;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrzeugDichte;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttGeschwindigkeit;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttVerkehrsStaerkeStunde;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.AqCache;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCacheExtended;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnittCache;

/**
 * Wrapper fr das {@link StreckenprofilEditorModel}, der einige ntzliche
 * Methoden bereit stellt, um zum Beispiel den ersten oder letzten
 * {@link StreckenKnoten} zu ermitteln.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: Modell.java,v 1.5 2014/04/17 15:51:18 Pittner Exp $
 */
public class Modell {

	/**
	 * Default Wert fr die maximale Verkehrsstrke pro Stunde.
	 */
	private static final int MAX_VERKEHRSSTAERKESTUNDE = 6000;

	/**
	 * Default Wert fr die maximale Bemessungsverkehrsstrke.
	 */
	private static final int MAX_BEMESSUNGSVERKEHRSSTAERKE = 6000;

	/**
	 * Der erweiterte NetzCache.
	 */
	private final NetzCacheExtended netzCacheExtended;

	/**
	 * Der Strecken - Cache.
	 */
	private final StreckenAbschnittCache streckenCache;

	/**
	 * Das {@link StreckenprofilEditorModel}, das gewrappt wird.
	 */
	private final StreckenprofilEditorModel model;

	/**
	 * Der DataBinding {@link Realm}.
	 */
	private final Realm realm;

	/**
	 * Konsturktor, dem ein schon vorhandenes {@link StreckenprofilEditorModel}
	 * bergeben wird.
	 * 
	 * @param realm
	 *            der Databinding {@link Realm}.
	 * @param streckenprofil
	 *            das {@link StreckenprofilEditorModel}.
	 */
	public Modell(final Realm realm,
			final StreckenprofilEditorModel streckenprofil) {
		Assert.isNotNull(streckenprofil,
				"Das bergebene Streckenprofil darf nicht null sein.");
		this.realm = realm;
		model = streckenprofil;
		if (model.getStreckenZug() == null) {
			model.setStreckenZug(StreckenprofilFactory.eINSTANCE
					.createStreckenZug());
		}

		// hier wird defaultmig ein LineProperties Objekt angelegt
		if (getLineProperties().isEmpty()) {
			final LinienEigenschaften defaultLineProperties = StreckenprofilFactory.eINSTANCE
					.createLinienEigenschaften();
			defaultLineProperties.setTypDatenQuelle(TypDatenQuellen.MQ);
			final RGB farbe = StreckenprofilUtil.getNaechsteFarbe();
			defaultLineProperties.setRgb(farbe);
			defaultLineProperties.setKnotenFarbe(farbe);
			defaultLineProperties.setTypVerkehrsDaten(TypVerkehrsDaten.QKFZ);
			getLineProperties().add(defaultLineProperties);
		}

		// Default - ChartProperties
		if (getChartProperties() == null) {
			final DiagrammEigenschaften chartP = StreckenprofilFactory.eINSTANCE
					.createDiagrammEigenschaften();
			AchsenBeschreibung achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			chartP.setXAxisType(achseBeschreibung);

			achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			WerteBereich w = WerteBereich
					.getWerteBereich(AttBemessungsFahrzeugDichte.class);
			achseBeschreibung.setMinimum(w.getMinimum());
			achseBeschreibung.setMaximum(w.getMaximum());
			chartP.setYAxisBemessungsverkehrsdichte(achseBeschreibung);

			achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			w = WerteBereich
					.getWerteBereich(AttBemessungsVerkehrsStaerkeStunde.class);
			achseBeschreibung.setMinimum(w.getMinimum());
			achseBeschreibung.setMaximum(MAX_BEMESSUNGSVERKEHRSSTAERKE);
			chartP.setYAxisBemessungsverkehrsstaerke(achseBeschreibung);

			achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			w = WerteBereich.getWerteBereich(AttFahrzeugDichte.class);
			achseBeschreibung.setMinimum(w.getMinimum());
			achseBeschreibung.setMaximum(w.getMaximum());
			chartP.setYAxisFahrezeugdichte(achseBeschreibung);

			achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			w = WerteBereich.getWerteBereich(AttProzent.class);
			achseBeschreibung.setMinimum(w.getMinimum());
			achseBeschreibung.setMaximum(w.getMaximum());
			chartP.setYAxisLKWAnteil(achseBeschreibung);

			achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			w = WerteBereich.getWerteBereich(AttVerkehrsStaerkeStunde.class);
			achseBeschreibung.setMinimum(w.getMinimum());
			achseBeschreibung.setMaximum(MAX_VERKEHRSSTAERKESTUNDE);
			chartP.setYAxisTypAnzahl(achseBeschreibung);

			achseBeschreibung = StreckenprofilFactory.eINSTANCE
					.createAchsenBeschreibung();
			w = WerteBereich.getWerteBereich(AttGeschwindigkeit.class);
			achseBeschreibung.setMinimum(w.getMinimum());
			achseBeschreibung.setMaximum(w.getMaximum());
			chartP.setYAxisTypeGeschwindigkeit(achseBeschreibung);

			model.setChartProperties(chartP);
		}

		final StartupPlugin startupPlugin = StartupPlugin.getDefault();
		// MqCache initialisieren
		startupPlugin.getDefaultMqCache();
		// NetzCache inititialisieren
		startupPlugin.getDefaultNetzCache();
		netzCacheExtended = startupPlugin.getDefaultNetzCacheExtended();
		streckenCache = startupPlugin.getDefaultStreckenAbschnittCache();
		// AqCache initialisieren
		AqCache aqCache = startupPlugin.getDefaultAqCache();
		
		
//		System.out.println("---------------> AQs: " + getAlleAnzeigequerschnitte().size());
		

//    getStreckenZug().eAdapters().add(new EContentAdapter() {
//
//      /*
//       * (non-Javadoc)
//       * 
//       * @see org.eclipse.emf.ecore.util.EContentAdapter#notifyChanged(org. eclipse.emf.common.notify.Notification)
//       */
//      @Override
//      public void notifyChanged(final Notification notification)
//      {
//        super.notifyChanged(notification);
//        final Object feature = notification.getFeature();
//        if (StreckenprofilPackage.Literals.ACTIVATEABLE__STATE.equals(feature))
//        {
//          final List<StrassenSegment> stsListe = new ArrayList<StrassenSegment>();
//          for (final RealerStreckenAbschnitt<?> sta : getAlleRealenStreckenAbschnitte())
//          {
//            stsListe.add(sta.getModellObjekt());
//          }
//
//          SelektionsTransfer.getInstanz().setDefinierteSelektion(BitCtrlRahmenwerkConstants.SELECTION_STRECKEN_ZUG,
//              true, new StructuredSelection(stsListe));
//        }
//      }
//    });
		
		if (getAnzeigeEigenschaften() == null) {
      AnzeigeEigenschaften anzeigeEigenschaften = StreckenprofilFactory.eINSTANCE.createAnzeigeEigenschaften();       
      model.setAnzeigeEigenschaften(anzeigeEigenschaften);
		}
	}

	/**
	 * Konstruktor zur Erzeugung eines neuen {@link StreckenprofilEditorModel}.
	 * 
	 * @param realm
	 *            der Databinding {@link Realm}.
	 */
	public Modell(final Realm realm) {
		this(realm, StreckenprofilFactory.eINSTANCE
				.createStreckenprofilEditorModel());
	}

	/**
	 * Fgt dem {@link Modell} ein neues {@link StrassenSegment} hinzu.
	 * 
	 * @param strassenSegment
	 *            das {@link StrassenSegment}, das dem
	 *            {@link StreckenprofilEditorModel} hinzugefgt werden soll.
	 */
	public void addSegment(final StrassenSegment strassenSegment) {
		LoadModel.addSegment(getStreckenZug(), strassenSegment);
	}

	/**
	 * Fgt der Strecke alle Knoten bzw. Straenteilsegmente bis zu dem
	 * bergebenen Knoten hinzu.
	 * 
	 * @param knoten
	 *            der dann letzte und erste Knoten der Strecke
	 * @param strecke
	 *            die {@link Strecke}.
	 */
	public void addFirstStreckenKnoten(final StreckenKnoten knoten,
			final Strecke strecke) {

		if (!getStrecken().isEmpty() || getLetzteStrecke() != null
				|| getLetzterStreckenKnoten() != null) {
			removeAlleStrecken();
		}
		// der neu Knoten ist also der erste Knoten
		final Strecke s = LoadModel.createStrecke(getStreckenZug(), strecke
				.getStrasse(), strecke.getRichtung());
		// ersten Knoten anlegen
		LoadModel.createKnoten(s, knoten.getStrassenKnoten());
	}

	/**
	 * Fgt der Strecke alle Knoten bzw. Straenteilsegmente bis zu dem
	 * bergebenen Knoten hinzu.
	 * 
	 * @param knoten
	 *            der dann letzte Knoten der {@link Strecke}
	 * @param neueStrecke
	 *            die {@link Strecke}, zu der der {@link StreckenKnoten} gehrt.
	 */
	public void addStreckenKnoten(final Strecke neueStrecke,
			final StreckenKnoten knoten) {
		if (null == getLetzteStrecke() && null == getErsterStreckenKnoten()) {
			addFirstStreckenKnoten(knoten, neueStrecke);
			return;
		}

		final StreckenKnoten streckenKnoten = getLetzterStreckenKnoten();
		Assert.isTrue(streckenKnoten != null, "Doppelte Initializierung!");
		final List<StrassenKnoten> nachfolgerStrassenKnoten = streckenCache
				.getNachfolgerStrassenKnoten(neueStrecke.getStrasse(),
						neueStrecke.getRichtung(), streckenKnoten
								.getStrassenKnoten());
		for (final StrassenKnoten strassenKnoten : nachfolgerStrassenKnoten) {
			final List<AeusseresStrassenSegment> aeussere = netzCacheExtended
					.getAeussereStrassenSegmenteVorStrassenKnoten(
							strassenKnoten, neueStrecke.getStrasse()
									.getKdStrasse().getDatum(), neueStrecke
									.getRichtung());
			// damit wir nicht so viele Events gefeuert bekommen, wird hier der
			// EventMechanismus vorbergehend ausgeschaltet.
			final Iterator<AeusseresStrassenSegment> iter = aeussere.iterator();
			getStreckenZug().eSetDeliver(false);
			while (iter.hasNext()) {
				addSegment(iter.next());

				if (!iter.hasNext()) {
					getStreckenZug().eSetDeliver(true);
				}
			}
			if (strassenKnoten.equals(knoten.getStrassenKnoten())) {
				break;
			}
		}
	}

	/**
	 * Setzt den Startknoten des Streckenzuges neu.
	 * 
	 * @param vorhandeneStrecke
	 *            die {@link Strecke} deren Startknoten neu bestimmt werden
	 *            soll.
	 * @param neuerStartKnoten
	 *            der neue Startknoten der {@link Strecke}.
	 */
	public void setStartKnoten(final Strecke vorhandeneStrecke,
			final StreckenKnoten neuerStartKnoten) {
		if (null == getLetzteStrecke() && null == getErsterStreckenKnoten()) {
			addFirstStreckenKnoten(neuerStartKnoten, vorhandeneStrecke);
			return;
		}

		final StreckenKnoten streckenKnoten = getLetzterStreckenKnoten();
		Assert.isTrue(streckenKnoten != null, "Doppelte Initializierung!");
		final List<StrassenKnoten> nachfolgerStrassenKnoten = streckenCache
				.getNachfolgerStrassenKnoten(vorhandeneStrecke.getStrasse(),
						vorhandeneStrecke.getRichtung(), neuerStartKnoten
								.getStrassenKnoten());
		for (final StrassenKnoten strassenKnoten : nachfolgerStrassenKnoten) {
			final List<AeusseresStrassenSegment> aeussere = netzCacheExtended
					.getAeussereStrassenSegmenteVorStrassenKnoten(
							strassenKnoten, vorhandeneStrecke.getStrasse(),
							vorhandeneStrecke.getRichtung());
			for (final AeusseresStrassenSegment aeusseresStrassenSegment : aeussere) {
				addSegment(aeusseresStrassenSegment);
			}
			if (strassenKnoten.equals(streckenKnoten.getStrassenKnoten())) {
				break;
			}
		}
	}

	/**
	 * Entfernt einen {@link StreckenKnoten} und alle weiteren
	 * {@link StreckenKnoten} bis zum momentanen Ende des
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZug}s.
	 * 
	 * @param knoten
	 *            der {@link StreckenKnoten}, der aus dem
	 *            {@link StreckenprofilEditorModel} entfernt werden soll.
	 */
	public void removeStreckenKnoten(final StreckenKnoten knoten) {
		Assert.isNotNull(knoten);
		Assert.isNotNull(getLetzterStreckenKnoten());
		LoadModel.removeActivateable(knoten);
	}

	/**
	 * Entfernt alle Strecken aus dem Streckenzug.
	 */
	public void removeAlleStrecken() {
		getStreckenZug().eSetDeliver(false);
		if (getErsterStreckenKnoten() != null) {
			removeStreckenKnoten(getErsterStreckenKnoten());
		}
		getStreckenZug().eSetDeliver(true);
		getStreckenZug().getStreckenZugEintraege().clear();
	}

	/**
	 * Liefert den gesamten Streckenzug.
	 * 
	 * @return der {@link StreckenZug} des {@link StreckenprofilEditorModel}s.
	 */
	public StreckenZug getStreckenZug() {
		return model.getStreckenZug();
	}

	/**
	 * Liefert alle Strecken des Streckenzuges.
	 * 
	 * @return die Menge mit allen {@link Strecke}n des
	 *         {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZug}es.
	 */
	public EList<Strecke> getStrecken() {
		return getStreckenZug().getStrecken();
	}

	/**
	 * Liefert die letzte Strecke des Streckenzuges.
	 * 
	 * @return die letzte {@link Strecke} des
	 *         {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZug}es.
	 */
	public Strecke getLetzteStrecke() {
		Strecke result = null;
		if (!getStrecken().isEmpty()) {
			result = getStrecken().get(getStrecken().size() - 1);
		}
		return result;
	}

	/**
	 * Liefert die Streckeneintrge der letzten {@link Strecke}.
	 * 
	 * @return die Menge der
	 *         {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZugEintrag}
	 *         e der letzten {@link Strecke}.
	 */
	public List<StreckenEintrag> getLetzteStreckenEintraege() {
		List<StreckenEintrag> result = new ArrayList<StreckenEintrag>();
		if (getLetzteStrecke() != null) {
			result = getLetzteStrecke().getStreckenEintraege();
		}
		return result;
	}

	/**
	 * Liefert den letzten {@link StreckenKnoten} des
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZug}es.
	 * 
	 * @return den letzten {@link StreckenKnoten} des
	 *         {@link StreckenprofilEditorModel}s.
	 */
	public StreckenKnoten getLetzterStreckenKnoten() {
		StreckenKnoten knoten = null;
		for (final StreckenEintrag e : getLetzteStreckenEintraege()) {
			if (e instanceof StreckenKnoten) {
				knoten = (StreckenKnoten) e;
			}
		}
		return knoten;
	}

	/**
	 * Liefert den {@link Realm}.
	 * 
	 * @return den Databinding {@link Realm}.
	 */
	public Realm getRealm() {
		return realm;
	}

	/**
	 * Liefert den ersten {@link StreckenKnoten} des
	 * {@link StreckenprofilEditorModel}s.
	 * 
	 * @return the ersterStreckenKnoten
	 */
	public final StreckenKnoten getErsterStreckenKnoten() {
		StreckenKnoten knoten = null;
		if (getErsteStrecke() != null) {
			for (final StreckenEintrag e : getErsteStrecke()
					.getStreckenEintraege()) {
				if (e instanceof StreckenKnoten) {
					knoten = (StreckenKnoten) e;
					break;
				}
			}
		}
		return knoten;
	}

	/**
	 * Liefert alle {@link LinienEigenschaften} des
	 * {@link StreckenprofilEditorModel} s.
	 * 
	 * @return the lineProperties
	 */
	public final AnzeigeEigenschaften getAnzeigeEigenschaften() {
	  return model.getAnzeigeEigenschaften();
	}
	
	/**
	 * Liefert alle {@link LinienEigenschaften} des
	 * {@link StreckenprofilEditorModel} s.
	 * 
	 * @return the lineProperties
	 */
	public final EList<LinienEigenschaften> getLineProperties() {
		return model.getLineProperties();
	}

	/**
	 * Liefert das {@link StreckenprofilEditorModel}.
	 * 
	 * @return das {@link StreckenprofilEditorModel}.
	 */
	public final StreckenprofilEditorModel getEMFModell() {
		return model;
	}

	/**
	 * Liefert alle {@link DiagrammEigenschaften} des
	 * {@link StreckenprofilEditorModel}.
	 * 
	 * @return the chartProperties
	 */
	public final DiagrammEigenschaften getChartProperties() {
		return model.getChartProperties();
	}

	/**
	 * Liefert die erste {@link Strecke} des {@link StreckenprofilEditorModel}s.
	 * 
	 * @return the ersteStrecke
	 */
	public final Strecke getErsteStrecke() {
		Strecke result = null;
		if (!getStrecken().isEmpty()) {
			result = getStrecken().get(0);
		}
		return result;
	}

	/**
	 * Liefert alle {@link MessQuerschnitt}e des Streckenzuges.
	 * 
	 * @param activeOnesOnly
	 *            Wenn <code>true</code>, schrnkt die Menge der
	 *            {@link MessQuerschnitt}es auf die ein, die als aktive markiert
	 *            sind.
	 * @return die Menge aller {@link MessQuerschnitt}e des
	 *         {@link StreckenprofilEditorModel}s
	 */
	public List<MessQuerschnitt> getAlleMessquerschnitte(
			final boolean activeOnesOnly) {
		final List<MessQuerschnitt> result = new ArrayList<MessQuerschnitt>();

		final Iterator<EObject> iter = getStreckenZug().eAllContents();
		while (iter.hasNext()) {
			final EObject obj = iter.next();
			if (obj instanceof MessQuerschnitt) {
				final MessQuerschnitt mq = (MessQuerschnitt) obj;
				if (!activeOnesOnly
						|| (activeOnesOnly && ActivationState.ACTIVE.equals((mq
								.getState())))) {
					result.add(mq);
				}
			}
		}
		return result;

	}

	/**
	 * Liefert die Menge aller {@link StreckenKnoten} des
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZug}es.
	 * 
	 * @return Menge aller {@link StreckenKnoten}.
	 */
	public List<StreckenKnoten> getAlleStreckenKnoten() {
		final List<StreckenKnoten> result = new ArrayList<StreckenKnoten>();

		final Iterator<EObject> iter = getStreckenZug().eAllContents();
		while (iter.hasNext()) {
			final EObject obj = iter.next();
			if (obj instanceof StreckenKnoten) {
				result.add((StreckenKnoten) obj);
			}
		}
		return result;
	}

	/**
	 * Liefert alle {@link StreckenTeilAbschnitt}e des Streckenprofils.
	 * 
	 * @return eine {@link List} mit allen {@link StreckenTeilAbschnitt}en.
	 */
	public List<StreckenTeilAbschnitt> getAlleStreckenTeilAbschnitte() {
		final List<StreckenTeilAbschnitt> result = new ArrayList<StreckenTeilAbschnitt>();

		final Iterator<EObject> iter = getStreckenZug().eAllContents();
		while (iter.hasNext()) {
			final EObject obj = iter.next();
			if (obj instanceof StreckenTeilAbschnitt) {
				result.add((StreckenTeilAbschnitt) obj);
			}
		}
		return result;

	}

	/**
	 * Liefert alle {@link RealerStreckenAbschnitt}e des Streckenprofils.
	 * 
	 * @return eine {@link List} mit allen {@link RealerStreckenAbschnitt}en.
	 */
	public List<RealerStreckenAbschnitt<?>> getAlleRealenStreckenAbschnitte() {
		final List<RealerStreckenAbschnitt<?>> result = new ArrayList<RealerStreckenAbschnitt<?>>();

		final Iterator<EObject> iter = getStreckenZug().eAllContents();
		while (iter.hasNext()) {
			final EObject obj = iter.next();
			if (obj instanceof RealerStreckenAbschnitt<?>) {
				result.add((RealerStreckenAbschnitt<?>) obj);
			}
		}
		return result;

	}

	/**
	 * Liefert die Menge aller {@link AnzeigeQuerschnitt} der
	 * {@link StreckenZug}es.
	 * 
	 * @return Menge aller {@link AnzeigeQuerschnitt}e.
	 */
	public List<AnzeigeQuerschnitt> getAlleAnzeigequerschnitte() {
		final List<AnzeigeQuerschnitt> result = new ArrayList<AnzeigeQuerschnitt>();

		final Iterator<EObject> iter = getStreckenZug().eAllContents();
		while (iter.hasNext()) {
			final EObject obj = iter.next();
			if (obj instanceof AnzeigeQuerschnitt) {
				result.add((AnzeigeQuerschnitt) obj);
			}
		}
		return result;
	}
}
