/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import org.eclipse.birt.chart.device.swt.SwtRendererImpl;
import org.eclipse.birt.chart.event.LineRenderEvent;
import org.eclipse.birt.chart.exception.ChartException;
import org.eclipse.birt.chart.model.attribute.Location;

/**
 * Renderer, der die Position der X - Achse eines Birt-Charts ermittelt.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilDeviceRenderer.java,v 1.3 2014/04/17 15:51:18 Pittner Exp $
 */
public class StreckenprofilDeviceRenderer extends SwtRendererImpl {

	/**
	 * Die minimal erlaubte Lnge der X-Achse.
	 */
	private static final int MINIMALE_LAENGE_XACHSE = 100;

	/**
	 * Die ermittelte {@link Location} des Startpunktes der X-Achse.
	 */
	private Location xAchseStart;

	/**
	 * Die ermittelte {@link Location} des Endpunktes der X-Achse.
	 */
	private Location xAchseEnde;

	@Override
	public void drawLine(final LineRenderEvent lre) throws ChartException {
		super.drawLine(lre);
		if (lre.getStart().getY() == lre.getEnd().getY()) {
			// nur waagerechte Linien
			if (xAchseStart == null || xAchseEnde == null) {
				xAchseStart = lre.getStart();
				xAchseEnde = lre.getEnd();
			} else {
				final double neueLaenge = lre.getEnd().getX()
						- lre.getStart().getX();
				if (neueLaenge >= MINIMALE_LAENGE_XACHSE
						&& xAchseStart.getY() <= lre.getStart().getY()) {
					xAchseStart = lre.getStart();
					xAchseEnde = lre.getEnd();
				}

			}

		}
	}

	/**
	 * Setzt die Position der X-Achse auf nicht ermittelbar zurck.
	 */
	public void resetXAchse() {
		xAchseEnde = null;
		xAchseStart = null;
	}

	/**
	 * Liefert die aktuelle Startposition der X-Achse.
	 * 
	 * @return the xAchseStart
	 */
	public Location getxAchseStart() {
		return xAchseStart;
	}

	/**
	 * Liefert die aktuelle Endposition der X-Achse.
	 * 
	 * @return the xAchseEnde
	 */
	public Location getxAchseEnde() {
		return xAchseEnde;
	}

}
