/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.EventListenerList;

import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Definition der Funktionen, fr die Berechtigungen innerhalb des
 * Rahmenwerkes festgelegt werden knnen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * 
 * @version $Id: StreckenprofilPluginFunktion.java,v 1.3 2014/04/17 15:51:18 Pittner Exp $
 */
public enum StreckenprofilPluginFunktion {

	/**
	 * ID-Suffix, fr die Oberfchenberechtigung des Plugins Streckenprofil.
	 */
	HISTORISCHES_STRECKENPROFIL("historischeStreckenprofile");

	/**
	 * liefert die Funktion, die mit der bergebenen Id definiert ist. Wenn
	 * keine entsprechende Funktion gefunden werden konnte liefert die Funktion
	 * den Wert <code>null</code>.
	 * 
	 * @param id
	 *            die Id
	 * @return die Funktion oder <code>null</code>
	 */
	public static StreckenprofilPluginFunktion getFunktion(final String id) {
		StreckenprofilPluginFunktion result = null;

		for (final StreckenprofilPluginFunktion item : StreckenprofilPluginFunktion
				.values()) {
			if (item.funktion.getFunktionsName().equals(id)) {
				result = item;
				break;
			}
		}

		return result;
	}

	/**
	 * Debug-Logger zur Beobachtung der Rahmenwerks-Callbacks.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/** definiert den Freigabezustand der Funktion. */
	private boolean freigegeben;

	/** die zugeordnete Rahmenwerks-Oberflchenfunktion. */
	private OberflaechenFunktion funktion;

	/**
	 * die Liste der zu benachrichtigenden Listener, wenn sich der
	 * Freigabezustand der Funktion ndert.
	 */
	private final EventListenerList listeners = new EventListenerList();

	/**
	 * Konstruktor, erzeugt eine Funktionsdefinition basierend auf dem
	 * bergebenen Name.
	 * 
	 * Innerhalb des Rahmenwerks wird die Funktion mit der Dialog-ID, die dem
	 * Name des {@link PluginSystemKalender} entspricht und dem Klassenname der
	 * {@link StreckenprofilPluginFunktion} erweitert um den bergebenen Name
	 * registriert.
	 * 
	 * @param name
	 *            der Name
	 */
	private StreckenprofilPluginFunktion(final String name) {
		freigegeben = false;
		funktion = new OberflaechenFunktion(StreckenprofilPluginFunktion.class
				.getName()
				+ "." + name, StreckenprofilBerechtigungsVerwaltung
				.getInstanz(), Activator.class.getName(), name);
		OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(
				funktion);
	}

	/**
	 * fgt einen Listener zur Beobachtung des Berechtigungszustands der
	 * Funktion hinzu.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addFunktionsBerechtigungsListener(
			final FunktionsBerechtigungsListener listener) {
		listeners.add(FunktionsBerechtigungsListener.class, listener);
	}

	/**
	 * liefert den Freigabezustand der Funktion in der aktuellen Umgebung.
	 * 
	 * @return den Zustand
	 */
	public boolean isFreigegeben() {
		if (!freigegeben) {
			DEBUG
					.finer("Streckenprofil-Plugin: Funktion ist nicht freigegeben, prfe Freigabe..."
							+ this);
			final List<SystemObject> objekt = new ArrayList<SystemObject>();
			objekt.add(DaVVerbindung.getInstanz().getDaVVerbindung()
					.getLocalApplicationObject());
			OberflaechenBerechtigung.getInstanz()
					.pruefeBerechtigungElementGruppe(objekt,
							Activator.class.getName());
			DEBUG.finer("Freigabe=", new Object[] { this, freigegeben });
		}
		return freigegeben;
	}

	/**
	 * entfernt einen Listener zur Beobachtung des Berechtigungszustands der
	 * Funktion.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removeFunktionsBerechtigungsListener(
			final FunktionsBerechtigungsListener listener) {
		listeners.remove(FunktionsBerechtigungsListener.class, listener);
	}

	/**
	 * setzt den Freigabezustand der Funktion.
	 * 
	 * @param freigegeben
	 *            der Zustand
	 */
	void setFreigegeben(final boolean freigegeben) {
		if (freigegeben != this.freigegeben) {
			this.freigegeben = freigegeben;
			final ObjektFactory factory = Activator.getDefault()
					.getObjektFactory();
			if (factory.isVerbunden()) {
				for (final FunktionsBerechtigungsListener listener : listeners
						.getListeners(FunktionsBerechtigungsListener.class)) {
					listener.berechtigungsAenderung();
				}
			}
		}
	}
}
