/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilEinstellungen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditorInput;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.emf.EMFModellLoadAction;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCache;

/**
 * Action zum Laden eines {@link StreckenprofilEditorModel}.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilLadenAction.java,v 1.3 2014/04/17 15:51:11 Pittner Exp $
 */
public class StreckenprofilLadenAction extends
		EMFModellLoadAction<StreckenprofilEditorModel> implements
		IWorkbenchWindowActionDelegate {

	/**
	 * {@link IWorkbenchWindow} zum ffnen eines {@link StreckenprofilEditor}s.
	 */
	private IWorkbenchWindow windowLokal;
	private NetzCache cache;

	/**
	 * Erzeugt eine neue Aktion zum Laden eines Streckenprofils in den
	 * {@link StreckenprofilEditor}.
	 */
	public StreckenprofilLadenAction() {
		super(StreckenprofilEditorModel.class);
	}

	@Override
	protected StreckenprofilEinstellungen createModelEinstellungen() {
		return StreckenprofilEinstellungen.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
	 */
	@Override
	public void dispose() {
		// nichts
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.
	 * IWorkbenchWindow)
	 */
	@Override
	public void init(final IWorkbenchWindow window) {
		windowLokal = window;
		final StartupPlugin startupPlugin = StartupPlugin.getDefault();
		cache = startupPlugin.getDefaultNetzCache();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action
	 * .IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(final IAction action,
			final ISelection selection) {
		// ncihts
	}

	@Override
	protected void doLoad(final StreckenprofilEditorModel object) {
		try {

			final StreckenprofilEditorInput input = new StreckenprofilEditorInput();
			input.setModel(object);
			windowLokal.getActivePage().openEditor(input,
					StreckenprofilEditor.EDITOR_ID);
		} catch (final PartInitException e) {
			e.printStackTrace();
		}

	}

	@Override
	public void run(final IAction action) {
		final Job j = new Job("Cache initialisieren...") {

			@Override
			protected IStatus run(final IProgressMonitor monitor) {
				while (!cache.isInitialisiert()) {
					try {
						Thread.sleep(100);
					} catch (final InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				return Status.OK_STATUS;
			}
		};
		j.addJobChangeListener(new JobChangeAdapter() {

			@Override
			public void done(final IJobChangeEvent event) {
				windowLokal.getShell().getDisplay().asyncExec(new Runnable() {

					@Override
					public void run() {
						StreckenprofilLadenAction.this.run();
					}
				});
			}
		});
		j.setPriority(Job.INTERACTIVE);
		j.schedule();

	}

}
