/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart;

import java.util.ArrayList;
import java.util.List;

/**
 * Wrapper um ein AttVerkehrsBeschraenkung - Objekt, um Zustnde fr die
 * Aufhebung der Beschrnkungen zu ergnzen.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: AttVerkehrsBeschraenkungWrapper.java,v 1.3 2014/04/17 15:51:12 Pittner Exp $
 */
public final class AttVerkehrsBeschraenkungWrapper implements
		Comparable<AttVerkehrsBeschraenkungWrapper> {

	/**
	 * Aufhebung Geschwindigkeitsbeschrnkung.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_16_ENDE_GESCHWINDIGKEITSBESCHRAENKUNG = new AttVerkehrsBeschraenkungWrapper(
			"Ende Geschwindigkeitsbeschrnkung", Byte.valueOf("16"));

	/**
	 * Aufhebung der allgemeinen Verkehrsbeschrnkung.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_17_ENDE_VERKEHRSBESCHRAENKUNG = new AttVerkehrsBeschraenkungWrapper(
			"Ende Verkehrsbeschrnkung", Byte.valueOf("17"));

	/**
	 * Nicht nher spezifizierte Einschrnkung.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_0_SONSTIGE = new AttVerkehrsBeschraenkungWrapper(
			"sonstige", Byte.valueOf("0"));

	/**
	 * berholverbot allgemein.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_1_UEBERHOLVERBOT = new AttVerkehrsBeschraenkungWrapper(
			"berholverbot", Byte.valueOf("1"));

	/**
	 * berholverbot fr LKW.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_2_UEBERHOLVERBOT_FUER_LKW = new AttVerkehrsBeschraenkungWrapper(
			"berholverbot fr LKW", Byte.valueOf("2"));

	/**
	 * berholverbot bei Nsse.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_3_UEBERHOLVERBOT_BEI_NAESSE = new AttVerkehrsBeschraenkungWrapper(
			"berholverbot bei Nsse", Byte.valueOf("3"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_4_HOECHSTGEWINDIGKEIT_ALLGEMEIN = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgewindigkeit Allgemein", Byte.valueOf("4"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 10 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_13_HOECHSTGESCHWINDIGKEIT_10_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 10 km/h", Byte.valueOf("13"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 20 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_14_HOECHSTGESCHWINDIGKEIT_20_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 20 km/h", Byte.valueOf("14"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 30 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_15_HOECHSTGESCHWINDIGKEIT_30_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 30 km/h", Byte.valueOf("15"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 40 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_5_HOECHSTGESCHWINDIGKEIT_40_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 40 km/h", Byte.valueOf("5"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 50 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_6_HOECHSTGESCHWINDIGKEIT_50_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 50 km/h", Byte.valueOf("6"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 60 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_7_HOECHSTGESCHWINDIGKEIT_60_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 60 km/h", Byte.valueOf("7"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 70 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_8_HOECHSTGESCHWINDIGKEIT_70_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 70 km/h", Byte.valueOf("8"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 80 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_9_HOECHSTGESCHWINDIGKEIT_80_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 80 km/h", Byte.valueOf("9"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 100 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_10_HOECHSTGESCHWINDIGKEIT_100_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 100 km/h", Byte.valueOf("10"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 120 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_11_HOECHSTGESCHWINDIGKEIT_120_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 120 km/h", Byte.valueOf("11"));

	/**
	 * Allgemeine statische Geschwindigkeitsbeschrnkung auf 130 km/h.
	 * <p>
	 */
	public static final AttVerkehrsBeschraenkungWrapper ZUSTAND_12_HOECHSTGESCHWINDIGKEIT_130_KM_H = new AttVerkehrsBeschraenkungWrapper(
			"Hchstgeschwindigkeit 130 km/h", Byte.valueOf("12"));

	/**
	 * Name des Zustandes.
	 */
	private final String name;

	/**
	 * Der Wert des Zustandes.
	 */
	private final byte value;

	/**
	 * Erzeugt einen neuen {@link AttVerkehrsBeschraenkungWrapper}.
	 * 
	 * @param zustandsName
	 *            Zustandsname.
	 * @param zustandsWert
	 *            Zustandswert.
	 */
	private AttVerkehrsBeschraenkungWrapper(final String zustandsName,
			final Byte zustandsWert) {
		name = zustandsName;
		value = zustandsWert;
	}

	/**
	 * Liefert alle mglichen {@link AttVerkehrsBeschraenkungWrapper}.
	 * 
	 * @return Liste aller {@link AttVerkehrsBeschraenkungWrapper}.
	 */
	public static List<AttVerkehrsBeschraenkungWrapper> getZustaende() {
		final List<AttVerkehrsBeschraenkungWrapper> list = new ArrayList<AttVerkehrsBeschraenkungWrapper>();
		list.add(ZUSTAND_0_SONSTIGE);
		list.add(ZUSTAND_1_UEBERHOLVERBOT);
		list.add(ZUSTAND_2_UEBERHOLVERBOT_FUER_LKW);
		list.add(ZUSTAND_3_UEBERHOLVERBOT_BEI_NAESSE);
		list.add(ZUSTAND_4_HOECHSTGEWINDIGKEIT_ALLGEMEIN);
		list.add(ZUSTAND_13_HOECHSTGESCHWINDIGKEIT_10_KM_H);
		list.add(ZUSTAND_14_HOECHSTGESCHWINDIGKEIT_20_KM_H);
		list.add(ZUSTAND_15_HOECHSTGESCHWINDIGKEIT_30_KM_H);
		list.add(ZUSTAND_5_HOECHSTGESCHWINDIGKEIT_40_KM_H);
		list.add(ZUSTAND_6_HOECHSTGESCHWINDIGKEIT_50_KM_H);
		list.add(ZUSTAND_7_HOECHSTGESCHWINDIGKEIT_60_KM_H);
		list.add(ZUSTAND_8_HOECHSTGESCHWINDIGKEIT_70_KM_H);
		list.add(ZUSTAND_9_HOECHSTGESCHWINDIGKEIT_80_KM_H);
		list.add(ZUSTAND_10_HOECHSTGESCHWINDIGKEIT_100_KM_H);
		list.add(ZUSTAND_11_HOECHSTGESCHWINDIGKEIT_120_KM_H);
		list.add(ZUSTAND_12_HOECHSTGESCHWINDIGKEIT_130_KM_H);
		list.add(ZUSTAND_16_ENDE_GESCHWINDIGKEITSBESCHRAENKUNG);
		list.add(ZUSTAND_17_ENDE_VERKEHRSBESCHRAENKUNG);
		return list;
	}

	/**
	 * Gibt den Zustand zu einer Zahl zurck oder <code>null</code>, wenn die
	 * Zahl keinen Zustand reprsentiert.
	 * 
	 * @param value
	 *            ein Zahlenwert
	 * @return der Zustand zu dem Zahlenwert oder <code>null</code>, wenn die
	 *         Zahl keine Entsprechnung als Zustand besitzt.
	 */
	public static AttVerkehrsBeschraenkungWrapper getZustand(final Byte value) {
		for (final AttVerkehrsBeschraenkungWrapper e : getZustaende()) {
			if (e.getValue().equals(value)) {
				return e;
			}
		}
		return null;
	}

	/**
	 * Gibt den Wert der Zahl zurck.
	 * 
	 * @return der Zahlenwert.
	 */
	public Byte getValue() {
		return value;
	}

	@Override
	public String toString() {
		return name;
	}

	/**
	 * Gibt den Zustand zu einem Zustandsnamen zurck oder <code>null</code>,
	 * wenn kein Zustand einen solchen Namen hat.
	 * 
	 * @param value
	 *            der Zustandsname
	 * @return der Zustand zu dem Zustandsnamen oder <code>null</code>, wenn
	 *         kein Zustand einen solchen Namen hat.
	 */
	public static AttVerkehrsBeschraenkungWrapper getZustand(final String value) {
		for (final AttVerkehrsBeschraenkungWrapper e : getZustaende()) {
			if (e.toString().equals(value)) {
				return e;
			}
		}
		return null;
	}

	@Override
	public int compareTo(final AttVerkehrsBeschraenkungWrapper o) {
		return Double.compare(getValue(), o.getValue());
	}

	@Override
	public int hashCode() {
		return getValue().hashCode();
	}

	@Override
	public boolean equals(final Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj instanceof AttVerkehrsBeschraenkungWrapper) {
			return getValue().equals(
					((AttVerkehrsBeschraenkungWrapper) obj).getValue());
		}
		return false;
	}

	/**
	 * Ermittelt die Menge aller Beschrnkungen, die allgemeiner Natur sind.
	 * 
	 * @return Liste aller allgemeinen Verkehrsbeschrnkungen.
	 */
	public static List<AttVerkehrsBeschraenkungWrapper> getZustaendeAllgemeineVerkehrsBeschraenkung() {
		final List<AttVerkehrsBeschraenkungWrapper> list = new ArrayList<AttVerkehrsBeschraenkungWrapper>();
		list.add(ZUSTAND_0_SONSTIGE);
		list.add(ZUSTAND_1_UEBERHOLVERBOT);
		list.add(ZUSTAND_2_UEBERHOLVERBOT_FUER_LKW);
		list.add(ZUSTAND_3_UEBERHOLVERBOT_BEI_NAESSE);

		return list;
	}

	/**
	 * Ermittelt die Menge aller Geschwindigkeitsbeschnkungen.
	 * 
	 * @return Liste aller Geschwindigkeitsbeschrnkungen.
	 */
	public static List<AttVerkehrsBeschraenkungWrapper> getZustaendeGeschwindigkeitsbeschaenkung() {
		final List<AttVerkehrsBeschraenkungWrapper> list = new ArrayList<AttVerkehrsBeschraenkungWrapper>();
		list.add(ZUSTAND_4_HOECHSTGEWINDIGKEIT_ALLGEMEIN);
		list.add(ZUSTAND_13_HOECHSTGESCHWINDIGKEIT_10_KM_H);
		list.add(ZUSTAND_14_HOECHSTGESCHWINDIGKEIT_20_KM_H);
		list.add(ZUSTAND_15_HOECHSTGESCHWINDIGKEIT_30_KM_H);
		list.add(ZUSTAND_5_HOECHSTGESCHWINDIGKEIT_40_KM_H);
		list.add(ZUSTAND_6_HOECHSTGESCHWINDIGKEIT_50_KM_H);
		list.add(ZUSTAND_7_HOECHSTGESCHWINDIGKEIT_60_KM_H);
		list.add(ZUSTAND_8_HOECHSTGESCHWINDIGKEIT_70_KM_H);
		list.add(ZUSTAND_9_HOECHSTGESCHWINDIGKEIT_80_KM_H);
		list.add(ZUSTAND_10_HOECHSTGESCHWINDIGKEIT_100_KM_H);
		list.add(ZUSTAND_11_HOECHSTGESCHWINDIGKEIT_120_KM_H);
		list.add(ZUSTAND_12_HOECHSTGESCHWINDIGKEIT_130_KM_H);
		return list;
	}

}
