/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart;

import java.text.DateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Logger;

import org.eclipse.birt.chart.model.Chart;
import org.eclipse.birt.chart.model.ChartWithAxes;
import org.eclipse.birt.chart.model.attribute.ActionType;
import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.DataPoint;
import org.eclipse.birt.chart.model.attribute.DataPointComponent;
import org.eclipse.birt.chart.model.attribute.DataPointComponentType;
import org.eclipse.birt.chart.model.attribute.IntersectionType;
import org.eclipse.birt.chart.model.attribute.LineStyle;
import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.birt.chart.model.attribute.TickStyle;
import org.eclipse.birt.chart.model.attribute.TriggerCondition;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.attribute.impl.DataPointComponentImpl;
import org.eclipse.birt.chart.model.attribute.impl.StringFormatSpecifierImpl;
import org.eclipse.birt.chart.model.attribute.impl.TooltipValueImpl;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.component.Scale;
import org.eclipse.birt.chart.model.component.Series;
import org.eclipse.birt.chart.model.component.impl.AxisImpl;
import org.eclipse.birt.chart.model.component.impl.SeriesImpl;
import org.eclipse.birt.chart.model.data.DataSet;
import org.eclipse.birt.chart.model.data.NumberDataSet;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.eclipse.birt.chart.model.data.Trigger;
import org.eclipse.birt.chart.model.data.impl.ActionImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataElementImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.SeriesDefinitionImpl;
import org.eclipse.birt.chart.model.data.impl.TriggerImpl;
import org.eclipse.birt.chart.model.impl.ChartWithAxesImpl;
import org.eclipse.birt.chart.model.layout.Legend;
import org.eclipse.birt.chart.model.type.BarSeries;
import org.eclipse.birt.chart.model.type.LineSeries;
import org.eclipse.birt.chart.model.type.impl.BarSeriesImpl;
import org.eclipse.birt.chart.model.type.impl.LineSeriesImpl;
import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilUtil;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages.PrognoseTypLabelProvider;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessquerschnittNameFormatSpecifier;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.SeriesType;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Strecke;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilFactory;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypVerkehrsDaten;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.Anzeige;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeEigenschaftIst.Daten;

/**
 * ChartBuilder fr das Streckenprofil.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilChartBuilder.java,v 1.4 2014/04/28 13:02:07 Pittner Exp $
 */
public class StreckenprofilChartBuilder extends AbstractStreckenprofilBuilder {

	/**
	 * Standard Font fr alle Titel und Labels.
	 */
	protected static final String FONT_NAME = "MS Sans Serif";

	/**
	 * Default Farbdefinition der Hintergrundfarbe eines {@link Chart}s.
	 */
	private static final ColorDefinition MIDDLE_GREY = ColorDefinitionImpl
			.create(160, 160, 160);

	/**
	 * Default Farbdefinition der Farbe der Hilfslinien eines {@link Chart}s.
	 */
	private static final ColorDefinition DARK_GREY = ColorDefinitionImpl
			.create(127, 127, 127);

	/**
	 * Reprsentiert den aktuellen Titel des {@link Chart}s.
	 */
	private String title;

	/**
	 * Die X Achse.
	 */
	private Axis xAxis;

	/**
	 * Die Y Achse Geschwindigkeit.
	 */
	private Axis yAxisGeschwindigkeit;
	/**
	 * Die Y Achse Anzahl Fahrzeuge.
	 */
	private Axis yAxisAnzahlFz;

	/**
	 * Die Y Achse Bemessungsverkehrsstaerke.
	 */
	private Axis yAxisBemessungsverkehrsstaerke;

	/**
	 * Die Y Achse Bemessungsverkehrsdichte (kB).
	 */
	private Axis yAxisBemessungsverkehrsdichte;

	/**
	 * Die Y Achse LKW - Anteil (aLKW).
	 */
	private Axis yAxisLKWAnteil;

	/**
	 * Die Y Achse Fahrzeugdichte.
	 */
	private Axis yAxisFahrezeugdichte;

	/**
	 * Die Instanz des BIRT Charts.
	 */
	private Chart chart;

	/**
	 * {@link SeriesDefinition}, in der alle Datenreihen organisiert werden, die
	 * Fz/Intervall abbilden.
	 */
	private SeriesDefinition anzahlFzSeriesDefinition;

	/**
	 * {@link SeriesDefinition}, in der alle Datenreihen organisiert werden, die
	 * Geschwindigkeiten pro Stunde abbilden.
	 */

	private SeriesDefinition geschwindigkeitSeriesDefinition;

	/**
	 * {@link SeriesDefinition}, in der alle Datenreihen organisiert werden, die
	 * die Bemessungsverkehrsdichte abbilden.
	 */

	private SeriesDefinition bemessungsverkehrsdichteSeriesDefinition;

	/**
	 * {@link SeriesDefinition}, in der alle Datenreihen organisiert werden, die
	 * die Bemessungsverkehrsstrke abbilden.
	 */

	private SeriesDefinition bemessungsverkehrsstaerkeSeriesDefinition;

	/**
	 * {@link SeriesDefinition}, in der alle Datenreihen organisiert werden, die
	 * die Fahrzeugdichte abbilden.
	 */

	private SeriesDefinition fahrzeugdichteSeriesDefinition;

	/**
	 * {@link SeriesDefinition}, in der alle Datenreihen organisiert werden, die
	 * den LKW Anteil abbilden.
	 */

	private SeriesDefinition lkwAnteilSeriesDefinition;

	/**
	 * Konstruktor.
	 * 
	 * @param model
	 *            Das Streckenprofil, das innerhalb des Charts angezeigt werden
	 *            soll.
	 * 
	 */
	public StreckenprofilChartBuilder(final Modell model) {
		super(model);
		buildChart();
	}

	/**
	 * Erzeugt einen neuen BIRT - Chart.
	 */
	public void buildChart() {
		title = new String("Streckenprofil ");
		getStreckenprofilPositionenMap().clear();
		createChart();
		buildPlot();
		buildLegend();
		buildTitle();

		buildXAxis();
		buildYAxis();
		buildXSeries();
		if (getModel().getChartProperties().isGeschwindigkeitsLimitLinieDyn()) {
			if (yAxisGeschwindigkeit == null) {
				yAxisGeschwindigkeit = AxisImpl.create(Axis.ORTHOGONAL);
				konfiguriereYAxisGeschwindigkeit(yAxisGeschwindigkeit);
				xAxis.getAssociatedAxes().add(yAxisGeschwindigkeit);
			}
			erzeugeGeschwindigkeitsBegrenzungsLinien(yAxisGeschwindigkeit);
		}

		if (getModel().getChartProperties().isGeschwindigkeitsLimitIcons()) {
			erzeugeStatischeGeschwindigkeitsBegrenzungsIcons(
					yAxisGeschwindigkeit, getStreckenprofilPositionenMap());
		}
	}

	/**
	 * Neu laden eines Charts.
	 */
	public void reloadChart() {
		buildYAxis();
	}

	/**
	 * Liefert das Chart-Objekt.
	 * 
	 * @return den aktuell verwendeten BIRT Chart.
	 */
	@Override
	public Chart getChart() {
		return chart;
	}

	/**
	 * Creates chart instance.
	 */
	private void createChart() {
		chart = ChartWithAxesImpl.create();
		// ((ChartWithAxes) chart).setUnitSpacing(3);
	}

	/**
	 * Builds plot.
	 */
	private void buildPlot() {
		chart.getPlot().getClientArea()
				.setBackground(ColorDefinitionImpl.create(192, 192, 192));
	}

	/**
	 * Builds Y axis.
	 */
	private void buildYAxis() {

		for (final LinienEigenschaften lp : getModel().getLineProperties()) {
			final TypVerkehrsDaten datenQuelle = lp.getTypVerkehrsDaten();
			if (datenQuelle != null) {
				Axis neueAchse = null;
				if (yAxisAnzahlFz == null && yAxisGeschwindigkeit == null
						&& yAxisBemessungsverkehrsdichte == null
						&& yAxisBemessungsverkehrsstaerke == null
						&& yAxisLKWAnteil == null
						&& yAxisFahrezeugdichte == null) {
					// erste Achse
					neueAchse = ((ChartWithAxes) chart)
							.getPrimaryOrthogonalAxis(xAxis);
					// xAxis.getAssociatedAxes().add(neueAchse);
				} else {
					neueAchse = AxisImpl.create(Axis.ORTHOGONAL);
				}

				if (TypVerkehrsDaten.QKFZ.equals(datenQuelle)
						|| TypVerkehrsDaten.QLKW.equals(datenQuelle)
						|| TypVerkehrsDaten.QPKW.equals(datenQuelle)) {
					konfiguriereYAxisAnzahlFz(neueAchse);
					if (yAxisAnzahlFz == null) {
						// Anzahl Fahrzeuge
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisAnzahlFz = neueAchse;
					}
				} else if (TypVerkehrsDaten.VKFZ.equals(datenQuelle)
						|| TypVerkehrsDaten.VLKW.equals(datenQuelle)
						|| TypVerkehrsDaten.VPKW.equals(datenQuelle)) {
					konfiguriereYAxisGeschwindigkeit(neueAchse);
					if (yAxisGeschwindigkeit == null) {
						// Geschwindigkeit Fahrzeuge
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisGeschwindigkeit = neueAchse;
					}
				} else if ((TypVerkehrsDaten.KKFZ.equals(datenQuelle)
						|| TypVerkehrsDaten.KLKW.equals(datenQuelle) || TypVerkehrsDaten.KPKW
						.equals(datenQuelle))) {
					konfiguriereYAxisFahrezeugdichte(neueAchse);
					if (yAxisFahrezeugdichte == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisFahrezeugdichte = neueAchse;
					}
				} else if (TypVerkehrsDaten.ALKW.equals(datenQuelle)) {
					konfiguriereYAxisLKWAnteil(neueAchse);
					// Anteil LKW
					if (yAxisLKWAnteil == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisLKWAnteil = neueAchse;
					}

				} else if (TypVerkehrsDaten.KB.equals(datenQuelle)) {
					konfiguriereYAxisBemessungsverkehrsdichte(neueAchse);
					// Bemessungsverkehrsdichte
					if (yAxisBemessungsverkehrsdichte == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisBemessungsverkehrsdichte = neueAchse;
					}
				} else if (TypVerkehrsDaten.QB.equals(datenQuelle)) {
					konfiguriereYAxisBemessungsverkehrsstaerke(neueAchse);
					// Bemessungsverkehrsstaerke
					if (yAxisBemessungsverkehrsstaerke == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisBemessungsverkehrsstaerke = neueAchse;
					}
				} else {
					throw new IllegalArgumentException("Unbekannter Achsentyp "
							+ datenQuelle);
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #konfiguriereYAxisAnzahlFz(org.eclipse.birt.chart.model.component.Axis)
	 */
	@Override
	protected void konfiguriereYAxisAnzahlFz(final Axis neueAchse) {
		super.konfiguriereYAxisAnzahlFz(neueAchse);
		setSichtbarkeitAchse(neueAchse, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #konfiguriereYAxisBemessungsverkehrsdichte
	 * (org.eclipse.birt.chart.model.component.Axis)
	 */
	@Override
	protected void konfiguriereYAxisBemessungsverkehrsdichte(
			final Axis neueAchse) {
		super.konfiguriereYAxisBemessungsverkehrsdichte(neueAchse);
		setSichtbarkeitAchse(neueAchse, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #konfiguriereYAxisBemessungsverkehrsstaerke
	 * (org.eclipse.birt.chart.model.component.Axis)
	 */
	@Override
	protected void konfiguriereYAxisBemessungsverkehrsstaerke(
			final Axis neueAchse) {
		super.konfiguriereYAxisBemessungsverkehrsstaerke(neueAchse);
		setSichtbarkeitAchse(neueAchse, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #konfiguriereYAxisFahrezeugdichte
	 * (org.eclipse.birt.chart.model.component.Axis)
	 */
	@Override
	protected void konfiguriereYAxisFahrezeugdichte(final Axis neueAchse) {
		super.konfiguriereYAxisFahrezeugdichte(neueAchse);
		setSichtbarkeitAchse(neueAchse, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #konfiguriereYAxisGeschwindigkeit
	 * (org.eclipse.birt.chart.model.component.Axis)
	 */
	@Override
	protected void konfiguriereYAxisGeschwindigkeit(final Axis neueAchse) {
		super.konfiguriereYAxisGeschwindigkeit(neueAchse);
		setSichtbarkeitAchse(neueAchse, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * 
	 * #konfiguriereYAxisLKWAnteil(org.eclipse.birt.chart.model.component.Axis)
	 */
	@Override
	protected void konfiguriereYAxisLKWAnteil(final Axis neueAchse) {
		super.konfiguriereYAxisLKWAnteil(neueAchse);
		setSichtbarkeitAchse(neueAchse, false);
	}

	/**
	 * Builds the chart title.
	 */
	private void buildTitle() {
		chart.getTitle().getLabel().getCaption().setValue(title);
		chart.getTitle().getLabel().getCaption().getFont().setSize(14);
		chart.getTitle().getLabel().getCaption().getFont().setName(FONT_NAME);
		chart.getTitle().getLabel().getCaption().getFont().setWordWrap(false);
	}

	/**
	 * Builds legend.
	 * 
	 */
	private void buildLegend() {
		final Legend lg = chart.getLegend();
		lg.setVisible(false);
		lg.setPosition(Position.RIGHT_LITERAL);
		// lg.getText().setColor(ColorDefinitionImpl.TRANSPARENT());
		// lg.getTitle().getCaption().setColor(ColorDefinitionImpl.TRANSPARENT());
		// lg.getSeparator().setColor(ColorDefinitionImpl.TRANSPARENT());

		// chart.getLegend().getTitle().getCaption().setValue("Legende");
		// chart.getLegend().getTitle().setVisible(true);
		// chart.getLegend().setTitlePosition(Position.ABOVE_LITERAL);
		//
		// final Block b = BlockImpl.create();
		// b.setBackground(ColorDefinitionImpl.BLACK());
		// b.setAnchor(Anchor.EAST_LITERAL);
		// b.setStretch(Stretch.BOTH_LITERAL);
		// chart.getBlock().setVisible(true);
		// chart.getLegend().add(b);

	}

	/**
	 * Builds X axis.
	 */
	private void buildXAxis() {
		xAxis = ((ChartWithAxes) chart).getPrimaryBaseAxes()[0];

		xAxis.getTitle().setVisible(true);
		xAxis.getTitle().getCaption().setValue("Kilometrierung [km]");
		xAxis.getTitle().getCaption().getFont().setBold(false);
		xAxis.getTitle().getCaption().getFont().setSize(11);
		xAxis.getTitle().getCaption().getFont().setName(FONT_NAME);

		xAxis.getLabel().setVisible(true);
		// xAxis.getLabel().getCaption().getFont().setRotation(90);
		xAxis.getLabel().getCaption().getFont()
				.setSize(DEFAULT_AXIS_LABEL_TEXT_SIZE);
		xAxis.getLabel().getCaption().getFont().setName(FONT_NAME);

		xAxis.getMajorGrid().setTickStyle(TickStyle.ACROSS_LITERAL);
		xAxis.getMajorGrid().getLineAttributes().setVisible(true);
		xAxis.getMajorGrid().getLineAttributes()
				.setStyle(LineStyle.SOLID_LITERAL);
		xAxis.getMajorGrid().getLineAttributes()
				.setColor(MIDDLE_GREY.copyInstance());
		xAxis.getMajorGrid().getTickAttributes()
				.setColor(DARK_GREY.copyInstance());

		xAxis.getMinorGrid().setTickStyle(TickStyle.BELOW_LITERAL);
		xAxis.getMinorGrid().getTickAttributes()
				.setColor(DARK_GREY.copyInstance());
		xAxis.getMinorGrid().getTickAttributes().setVisible(true);
		final Scale scale = xAxis.getScale();
		xAxis.setFormatSpecifier(StreckenprofilFactory.eINSTANCE
				.createKilometrierungJavaNumberFormatSpecifier());

		scale.setMin(NumberDataElementImpl.create(0D));
		scale.setMax(NumberDataElementImpl.create(getModel()
				.getChartProperties().getXAxisType().getMaximum() + 100));

		// scale.setMin(NumberDataElementImpl.create(glmq.getIntervall().getStart()));
		// scale.setMax(NumberDataElementImpl.create(glmq.getIntervall().getEnd()));
		// scale.setStepNumber(24);
		// xAxis.getScale().setStepNumber(4);
		// scale.setMinorGridsPerUnit(4);
		xAxis.setType(getModel().getChartProperties().getXAxisType()
				.getAchsenTyp());
		xAxis.getOrigin().setType(IntersectionType.MIN_LITERAL);

		// xAxis.setFormatSpecifier(ModelFactory.eINSTANCE
		// .createMyJavaNumberFormatSpecifier());
	}

	/**
	 * Skalierung der X - Achse.
	 */
	private void buildXSeries() {
		xAxis.getSeriesDefinitions().clear();
		getStreckenprofilPositionenMap().putAll(ermittleMQWegeValues());
		final DataSet categoryValues = NumberDataSetImpl
				.create(getStreckenprofilPositionenMap().keySet());

		final Series seCategory = SeriesImpl.create();
		seCategory.setDataSet(categoryValues);

		final Scale scale = xAxis.getScale();
		scale.setShowOutside(false);
		if (!getStreckenprofilPositionenMap().isEmpty()) {
			// scale.setMin(NumberDataElementImpl.create(werteMap.keySet()
			// .iterator().next()));
			scale.setMax(NumberDataElementImpl
					.create(getStreckenprofilPositionenMap().keySet().toArray(
							new Double[0])[getStreckenprofilPositionenMap()
							.size() - 1]));
		}
		// Apply the color palette
		final SeriesDefinition sdX = SeriesDefinitionImpl.create();

		sdX.getSeries().add(seCategory);
		xAxis.getSeriesDefinitions().add(sdX);
	}

	/**
	 * Ermittelt und setzt die den Zeitstempel, den Strassennamen und den
	 * Verkehrsdatentyp im Titel des Charts.
	 * 
	 * @param values
	 *            Menge von {@link OnlineDatum}s, von denen der Zeitstempel des
	 *            neuesten fr den Titel verwendet wird.
	 */
	private void bestimmeTitleMitZeit(final Collection<OnlineDatum> values) {
		long timestamp = 0;
		for (final OnlineDatum od : values) {
			if (od.dGetZeitstempel().getTime() > timestamp
					&& OnlineDatum.Status.DATEN.equals(od.dGetDatenStatus())) {
				timestamp = od.dGetZeitstempel().getTime();
			}
		}
		final Strecke ersteStrecke = getModel().getStrecken().iterator().next();
		title = "Streckenprofil "
				+ (ersteStrecke != null ? ersteStrecke.getName() : "")
				+ " "
				+ new PrognoseTypLabelProvider().getText(getModel()
						.getChartProperties().getTypPrognoseTyp())
				+ " ["
				+ (timestamp != 0 ? DateFormat.getInstance().format(
						new Date(timestamp)) : "-") + "]";
	}

	/**
	 * Liefert den Titel des Charts.
	 * 
	 * @return the title der Titeltext des BIRT Charts.
	 */
	@Override
	public final String getTitle() {
		return title;
	}

	/**
	 * Aktualisieren eines ganz bestimmten Daten-Reihe des Charts.
	 * 
	 * @param lp
	 *            Die grafischen Eigenschatften der Daten-Reihe in der
	 *            Chart-Darstellung.
	 * @param daten
	 *            Ein Map mit SystemObjekten und den dazugehrigen OnlineDaten.
	 */
	@Override
	public void setLineData(final LinienEigenschaften lp,
			final Map<SystemObjekt, OnlineDatum> daten) {
		Assert.isNotNull(lp);
		Assert.isNotNull(daten);

		if (!lp.isVisible()) {
			return;
		}

		bestimmeTitleMitZeit(daten.values());
		chart.getTitle().getLabel().getCaption().setValue(getTitle());

		final Map<Double, Double> result = new TreeMap<Double, Double>();
		result.putAll(getStreckenprofilPositionenMap());

		for (final MessQuerschnitt mq : getModel()
				.getAlleMessquerschnitte(true)) {
			switch (lp.getTypDatenQuelle()) {
			case MQ:
				if (ActivationState.ACTIVE.equals(mq.getState())) {
					final Double d = StreckenprofilUtil.getMQValue(lp,
							mq.getMessQuerschnitt(),
							daten.get(mq.getModellObjekt()));
					if (d != null && d < 0) {
						result.put(mq.getWegBisher(), null);
					} else {
						result.put(mq.getWegBisher(), d);
					}
				}
				break;
			case HFS:
			case UEFS1:
			case UEFS2:
			case UEFS3:
			case UEFS4:
			case UEFS5:
			case UEFS6:
				if (ActivationState.ACTIVE.equals(mq.getState())) {
					final Double d = StreckenprofilUtil.getFSValue(lp,
							mq.getMessQuerschnitt(), daten);
					if (d != null && d < 0) {
						result.put(mq.getWegBisher(), null);
					} else {
						result.put(mq.getWegBisher(), d);
					}
				}
				break;
			default:
				throw new IllegalArgumentException("Unbekannte Datenquelle "
						+ lp.getTypDatenQuelle());
			}
		}

		final String seriesID = lp.getTypDatenQuelle().getName() + " - "
				+ lp.getTypVerkehrsDaten().getName();

		SeriesDefinition aktuelleSeriesDefinition = null;
		if (yAxisAnzahlFz != null
				&& (TypVerkehrsDaten.QKFZ.equals(lp.getTypVerkehrsDaten())
						|| TypVerkehrsDaten.QLKW.equals(lp
								.getTypVerkehrsDaten()) || TypVerkehrsDaten.QPKW
						.equals(lp.getTypVerkehrsDaten()))) {
			if (anzahlFzSeriesDefinition == null) {
				anzahlFzSeriesDefinition = SeriesDefinitionImpl.create();
				anzahlFzSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisAnzahlFz.getSeriesDefinitions().add(
						anzahlFzSeriesDefinition);
			}
			aktuelleSeriesDefinition = anzahlFzSeriesDefinition;
		} else if (yAxisGeschwindigkeit != null
				&& (TypVerkehrsDaten.VKFZ.equals(lp.getTypVerkehrsDaten())
						|| TypVerkehrsDaten.VLKW.equals(lp
								.getTypVerkehrsDaten()) || TypVerkehrsDaten.VPKW
						.equals(lp.getTypVerkehrsDaten()))) {
			if (geschwindigkeitSeriesDefinition == null) {
				geschwindigkeitSeriesDefinition = SeriesDefinitionImpl.create();
				geschwindigkeitSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisGeschwindigkeit.getSeriesDefinitions().add(
						geschwindigkeitSeriesDefinition);
			}
			aktuelleSeriesDefinition = geschwindigkeitSeriesDefinition;
		} else if (yAxisBemessungsverkehrsdichte != null
				&& TypVerkehrsDaten.KB.equals(lp.getTypVerkehrsDaten())) {

			if (bemessungsverkehrsdichteSeriesDefinition == null) {
				bemessungsverkehrsdichteSeriesDefinition = SeriesDefinitionImpl
						.create();
				bemessungsverkehrsdichteSeriesDefinition.getSeriesPalette()
						.getEntries().clear();
				yAxisBemessungsverkehrsdichte.getSeriesDefinitions().add(
						bemessungsverkehrsdichteSeriesDefinition);
			}
			aktuelleSeriesDefinition = bemessungsverkehrsdichteSeriesDefinition;
		} else if (yAxisBemessungsverkehrsstaerke != null
				&& TypVerkehrsDaten.QB.equals(lp.getTypVerkehrsDaten())) {
			if (bemessungsverkehrsstaerkeSeriesDefinition == null) {
				bemessungsverkehrsstaerkeSeriesDefinition = SeriesDefinitionImpl
						.create();
				bemessungsverkehrsstaerkeSeriesDefinition.getSeriesPalette()
						.getEntries().clear();
				yAxisBemessungsverkehrsstaerke.getSeriesDefinitions().add(
						bemessungsverkehrsstaerkeSeriesDefinition);
			}
			aktuelleSeriesDefinition = bemessungsverkehrsstaerkeSeriesDefinition;
		} else if (yAxisFahrezeugdichte != null
				&& (TypVerkehrsDaten.KKFZ.equals(lp.getTypVerkehrsDaten())
						|| TypVerkehrsDaten.KLKW.equals(lp
								.getTypVerkehrsDaten()) || TypVerkehrsDaten.KPKW
						.equals(lp.getTypVerkehrsDaten()))) {
			if (fahrzeugdichteSeriesDefinition == null) {
				fahrzeugdichteSeriesDefinition = SeriesDefinitionImpl.create();
				fahrzeugdichteSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisFahrezeugdichte.getSeriesDefinitions().add(
						fahrzeugdichteSeriesDefinition);
			}
			aktuelleSeriesDefinition = fahrzeugdichteSeriesDefinition;
		} else if (yAxisLKWAnteil != null
				&& TypVerkehrsDaten.ALKW.equals(lp.getTypVerkehrsDaten())) {
			if (lkwAnteilSeriesDefinition == null) {
				lkwAnteilSeriesDefinition = SeriesDefinitionImpl.create();
				lkwAnteilSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisLKWAnteil.getSeriesDefinitions().add(
						lkwAnteilSeriesDefinition);
			}
			aktuelleSeriesDefinition = lkwAnteilSeriesDefinition;
		} else {
			Logger.getLogger(getClass().getName()).warning(
					"Unbekannter Verkehrsdatentyp " + lp.getTypVerkehrsDaten()
							+ " fr Linieneigenschaft " + lp);
			return;
		}

		final NumberDataSet orthoValuesDataSet1 = NumberDataSetImpl
				.create(result.values());
		Series s = null;
		if (SeriesType.LINE.equals(lp.getSeriesType())) {
			final LineSeries ls = (LineSeries) LineSeriesImpl.create();
			ls.setSeriesIdentifier(seriesID);
			ls.setDataSet(orthoValuesDataSet1);
			initLineColor(lp, ls);
			initLineStyle(lp, ls);
			initLineThickness(lp, ls);
			s = ls;
		} else if (SeriesType.BAR.equals(lp.getSeriesType())) {
			final BarSeries bs = (BarSeries) BarSeriesImpl.create();
			bs.setSeriesIdentifier(seriesID);
			bs.setDataSet(orthoValuesDataSet1);
			bs.setRiserOutline(ColorDefinitionImpl.BLACK());
			initBarColor(lp, bs);
			s = bs;
		} else if (SeriesType.BAR_STACKED.equals(lp.getSeriesType())) {
			final BarSeries bs = (BarSeries) BarSeriesImpl.create();
			bs.setStacked(true);
			bs.setSeriesIdentifier(seriesID);
			bs.setDataSet(orthoValuesDataSet1);
			bs.setRiserOutline(ColorDefinitionImpl.BLACK());
			initBarColor(lp, bs);
			s = bs;
		}

		if (aktuelleSeriesDefinition != null
				&& aktuelleSeriesDefinition.getSeries().size() == 0) {
			aktuelleSeriesDefinition.getSeriesPalette().getEntries().clear();
		}

		for (int i = 0; i < aktuelleSeriesDefinition.getSeries().size(); i++) {
			final Series alteSeries = aktuelleSeriesDefinition.getSeries().get(
					i);
			if (alteSeries.getSeriesIdentifier().equals(seriesID)) {

				aktuelleSeriesDefinition.getSeriesPalette().getEntries()
						.remove(i);
				aktuelleSeriesDefinition.getRunTimeSeries().remove(i);
				aktuelleSeriesDefinition.getSeries().remove(i);
				break;
			}
		}

		initSeriesVisibility(lp, s);
		aktuelleSeriesDefinition.getSeries().add(s);
		aktuelleSeriesDefinition
				.getSeriesPalette()
				.getEntries()
				.add(StreckenprofilUtil.createColorDefinition(lp
						.getKnotenFarbe()));

		// ls.setConnectMissingValue(false);
		final MessquerschnittNameFormatSpecifier mqSpec = StreckenprofilFactory.eINSTANCE
				.createMessquerschnittNameFormatSpecifier();
		mqSpec.setStreckenprofil(getModel().getEMFModell());

		final DataPoint dp = s.getDataPoint();
		final EList<DataPointComponent> components = dp.getComponents();
		components.clear();
		components.add(DataPointComponentImpl.create(
				DataPointComponentType.SERIES_VALUE_LITERAL, null));
		switch (lp.getTypVerkehrsDaten()) {
		case VKFZ:
		case VLKW:
		case VPKW:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL,
					StringFormatSpecifierImpl.create(" km/h")));
			break;
		case ALKW:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL,
					StringFormatSpecifierImpl.create(" %")));
			break;
		case KKFZ:
		case KLKW:
		case KPKW:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL,
					StringFormatSpecifierImpl.create(" Fz/km")));
			break;
		case QKFZ:
		case QLKW:
		case QPKW:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL,
					StringFormatSpecifierImpl.create(" Fz/Intervall")));
			break;
		case KB:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL,
					StringFormatSpecifierImpl.create(" Pkw/km")));
			break;
		case QB:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL,
					StringFormatSpecifierImpl.create(" Pkw/h")));
			break;
		default:
			components.add(DataPointComponentImpl.create(
					DataPointComponentType.ORTHOGONAL_VALUE_LITERAL, null));
		}

		components.add(DataPointComponentImpl.create(
				DataPointComponentType.BASE_VALUE_LITERAL, mqSpec));

		final Trigger trigger = TriggerImpl.create(
				TriggerCondition.ONMOUSEOVER_LITERAL, ActionImpl.create(
						ActionType.SHOW_TOOLTIP_LITERAL,
						TooltipValueImpl.create(500, null)));
		s.getTriggers().add(trigger);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #setAnzeigenDaten(java.util.Map)
	 */
	@Override
	public void erzeugeDynamischeAnzeigen(
			final Map<AnzeigeQuerschnitt, Map<Anzeige, Daten>> anzeigeDaten) {
		super.erzeugeDynamischeAnzeigen(anzeigeDaten);
		if (getModel().getChartProperties().isGeschwindigkeitsLimitLinieDyn()) {
			erzeugeGeschwindigkeitsBegrenzungsLinien(yAxisGeschwindigkeit);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #dispose()
	 */
	@Override
	public void dispose() {
		anzahlFzSeriesDefinition = null;
		bemessungsverkehrsdichteSeriesDefinition = null;
		bemessungsverkehrsstaerkeSeriesDefinition = null;
		fahrzeugdichteSeriesDefinition = null;
		geschwindigkeitSeriesDefinition = null;
		lkwAnteilSeriesDefinition = null;
		yAxisAnzahlFz = null;
		yAxisBemessungsverkehrsdichte = null;
		yAxisBemessungsverkehrsstaerke = null;
		yAxisFahrezeugdichte = null;
		yAxisGeschwindigkeit = null;
		yAxisLKWAnteil = null;
		chart = null;
		super.dispose();
	}

	/**
	 * Setzt die Sichtbarkeit aller {@link Axis}.
	 * 
	 * @param visible
	 *            <code>true</code>, wenn die Achsen sichtbar sein sollen, sonst
	 *            <code>false</code>
	 */
	@Override
	public void setAlleAchsenSichtbarkeit(final boolean visible) {
		setSichtbarkeitAchse(yAxisAnzahlFz, visible);
		setSichtbarkeitAchse(yAxisBemessungsverkehrsdichte, visible);
		setSichtbarkeitAchse(yAxisBemessungsverkehrsstaerke, visible);
		setSichtbarkeitAchse(yAxisFahrezeugdichte, visible);
		setSichtbarkeitAchse(yAxisGeschwindigkeit, visible);
		setSichtbarkeitAchse(yAxisLKWAnteil, visible);
	}
}
