/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.birt.chart.model.Chart;
import org.eclipse.birt.chart.model.ChartWithAxes;
import org.eclipse.birt.chart.model.attribute.Anchor;
import org.eclipse.birt.chart.model.attribute.DataPoint;
import org.eclipse.birt.chart.model.attribute.DataPointComponent;
import org.eclipse.birt.chart.model.attribute.DataPointComponentType;
import org.eclipse.birt.chart.model.attribute.MarkerType;
import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.birt.chart.model.attribute.Stretch;
import org.eclipse.birt.chart.model.attribute.impl.BoundsImpl;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.attribute.impl.DataPointComponentImpl;
import org.eclipse.birt.chart.model.attribute.impl.ImageImpl;
import org.eclipse.birt.chart.model.attribute.impl.SizeImpl;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.component.Scale;
import org.eclipse.birt.chart.model.component.Series;
import org.eclipse.birt.chart.model.component.impl.AxisImpl;
import org.eclipse.birt.chart.model.component.impl.SeriesImpl;
import org.eclipse.birt.chart.model.data.DataSet;
import org.eclipse.birt.chart.model.data.NumberDataSet;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.eclipse.birt.chart.model.data.impl.NumberDataElementImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.SeriesDefinitionImpl;
import org.eclipse.birt.chart.model.impl.ChartWithAxesImpl;
import org.eclipse.birt.chart.model.layout.Legend;
import org.eclipse.birt.chart.model.type.BarSeries;
import org.eclipse.birt.chart.model.type.LineSeries;
import org.eclipse.birt.chart.model.type.ScatterSeries;
import org.eclipse.birt.chart.model.type.impl.BarSeriesImpl;
import org.eclipse.birt.chart.model.type.impl.LineSeriesImpl;
import org.eclipse.birt.chart.model.type.impl.ScatterSeriesImpl;
import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilUtil;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.SeriesType;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypVerkehrsDaten;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.Anzeige;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeEigenschaftIst.Daten;

/**
 * Builder zum Erzeugen der Legende eines BIRT - Charts.
 * 
 * @author BitCtrl Systems GmbH, christian
 * @version $Id: StreckenprofilLegendeChartBuilder.java,v 1.4 2014/04/28 13:02:07 Pittner Exp $
 */
public class StreckenprofilLegendeChartBuilder extends
		AbstractStreckenprofilBuilder {

	/**
	 * Das BIRT Chart.
	 */
	private Chart chart;

	/**
	 * Die X Achse.
	 */
	private Axis xAxis;

	/**
	 * Die Achse, an der die Datenreihen ausgerichtet werden, mit denen die
	 * Beschilderung im Chart angezeigt wird. Im Gegensatz zu einem normalen
	 * Chart, wird in der Legende nur eine dynamische Anzeige, einmal statische
	 * Beschilderung angezeigt.
	 */
	private Axis schilderAchse;

	/**
	 * Das {@link Modell} das das Streckneprofil abbildet.
	 */
	private final Modell model;

	/**
	 * Die {@link Axis} fr die Anzahl der Fahrzeuge pro Intervall.
	 */
	private Axis yAxisAnzahlFz;

	/**
	 * Die {@link Axis} fr die Geschwindigkeiten.
	 */
	private Axis yAxisGeschwindigkeit;

	/**
	 * Die {@link Axis} fr die Bemessungsverkehrsdichte.
	 */
	private Axis yAxisBemessungsverkehrsdichte;

	/**
	 * Die {@link Axis} fr die Bemessungsverkehrsstrke.
	 */
	private Axis yAxisBemessungsverkehrsstaerke;

	/**
	 * Die {@link Axis} fr den prozentualen Anteil der LKW.
	 */
	private Axis yAxisLKWAnteil;

	/**
	 * Die {@link Axis} fr die Fahrzeugdichte.
	 */
	private Axis yAxisFahrezeugdichte;

	/**
	 * Beschreibung der Darstellung der Fahrzeuge pro Intervall im {@link Chart}
	 * .
	 */
	private SeriesDefinition anzahlFzSeriesDefinition;

	/**
	 * Beschreibung der Darstellung der Geschwindigkeiten im {@link Chart}.
	 */
	private SeriesDefinition geschwindigkeitSeriesDefinition;

	/**
	 * Beschreibt die Darstellung der Bemessungsverkehrsdichte im {@link Chart}.
	 */
	private SeriesDefinition bemessungsverkehrsdichteSeriesDefinition;

	/**
	 * Beschreibt die Darstellung der Bemessungsverkehrsstrke im {@link Chart}.
	 */
	private SeriesDefinition bemessungsverkehrsstaerkeSeriesDefinition;

	/**
	 * Beschreibt die Darstellung der Farhzeugdichte im {@link Chart}.
	 */
	private SeriesDefinition fahrzeugdichteSeriesDefinition;

	/**
	 * Beschreibt die Darstellung des LKW Anteils im {@link Chart}.
	 */
	private SeriesDefinition lkwAnteilSeriesDefinition;

	/**
	 * {@link SeriesDefinition} mit der die Verkehrsbeschrnkungen mit
	 * Schildsymbolen dargestellt werden.
	 */
	private SeriesDefinition verkehrsBeschreankungSchilderSeriesDefinition;

	/**
	 * Erzeugt einen neuen {@link StreckenprofilLegendeChartBuilder}.
	 * 
	 * @param model
	 *            das Streckenprofil.
	 */
	public StreckenprofilLegendeChartBuilder(final Modell model) {
		super(model);
		this.model = model;
		buildChart();
	}

	/**
	 * Erzeugt ein neues {@link Chart}.
	 */
	public void buildChart() {

		chart = ChartWithAxesImpl.create();
		xAxis = ((ChartWithAxes) chart).getPrimaryBaseAxes()[0];

		chart.getPlot().setVisible(false);
		chart.getPlot().setHeightHint(1);
		chart.getPlot().setWidthHint(1);
		chart.getPlot().setBounds(BoundsImpl.create(0, 0, 0, 0));
		chart.getPlot().setMinSize(SizeImpl.create(0, 0));
		chart.getTitle().setWidthHint(0);
		chart.getTitle().setHeightHint(0);
		chart.getTitle().setVisible(false);
		final Legend lg = chart.getLegend();
		lg.setVisible(true);
		lg.setMaxPercent(1d);
		// Schriftgroesse
		// lg.getText().getFont().setSize(17);
		lg.setAnchor(Anchor.NORTH_LITERAL);
		lg.setPosition(Position.LEFT_LITERAL);
		lg.setStretch(Stretch.BOTH_LITERAL);

		xAxis.getSeriesDefinitions().clear();

		final List<Double> xWerte = new ArrayList<Double>();
		xWerte.add(0d);
		final DataSet categoryValues = NumberDataSetImpl.create(xWerte);

		// final Scale scale = xAxis.getScale();
		// scale.setShowOutside(false);

		final Series seCategory = SeriesImpl.create();
		seCategory.setDataSet(categoryValues);

		// Apply the color palette
		final SeriesDefinition sdX = SeriesDefinitionImpl.create();

		sdX.getSeries().add(seCategory);
		xAxis.getSeriesDefinitions().add(sdX);

		buildYAxis();

		for (final LinienEigenschaften le : model.getLineProperties()) {
			setLineData(le, null);
		}

		erzeugeGeschwindigkeitsBeschraenkungenLegende();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #getChart()
	 */
	@Override
	public Chart getChart() {
		return chart;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #getTitle()
	 */
	@Override
	public String getTitle() {
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #setLineData(de.bsvrz.buv.plugin.streckenprofil.model.LineProperties,
	 * java.util.Map)
	 */
	@Override
	public void setLineData(final LinienEigenschaften lp,
			final Map<SystemObjekt, OnlineDatum> daten) {
		Assert.isNotNull(lp);
		// Assert.isNotNull(daten);

		if (!lp.isVisible()) {
			return;
		}

		final String seriesID = lp.getTypDatenQuelle().getName() + " - "
				+ lp.getTypVerkehrsDaten().getName();

		SeriesDefinition aktuelleSeriesDefinition = null;
		if (yAxisAnzahlFz != null
				&& (TypVerkehrsDaten.QKFZ.equals(lp.getTypVerkehrsDaten())
						|| TypVerkehrsDaten.QLKW.equals(lp
								.getTypVerkehrsDaten()) || TypVerkehrsDaten.QPKW
						.equals(lp.getTypVerkehrsDaten()))) {
			if (anzahlFzSeriesDefinition == null) {
				anzahlFzSeriesDefinition = SeriesDefinitionImpl.create();
				anzahlFzSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisAnzahlFz.getSeriesDefinitions().add(
						anzahlFzSeriesDefinition);
			}
			aktuelleSeriesDefinition = anzahlFzSeriesDefinition;
		} else if (yAxisGeschwindigkeit != null
				&& (TypVerkehrsDaten.VKFZ.equals(lp.getTypVerkehrsDaten())
						|| TypVerkehrsDaten.VLKW.equals(lp
								.getTypVerkehrsDaten()) || TypVerkehrsDaten.VPKW
						.equals(lp.getTypVerkehrsDaten()))) {
			if (geschwindigkeitSeriesDefinition == null) {
				geschwindigkeitSeriesDefinition = SeriesDefinitionImpl.create();
				geschwindigkeitSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisGeschwindigkeit.getSeriesDefinitions().add(
						geschwindigkeitSeriesDefinition);
			}
			aktuelleSeriesDefinition = geschwindigkeitSeriesDefinition;
		} else if (yAxisBemessungsverkehrsdichte != null
				&& TypVerkehrsDaten.KB.equals(lp.getTypVerkehrsDaten())) {

			if (bemessungsverkehrsdichteSeriesDefinition == null) {
				bemessungsverkehrsdichteSeriesDefinition = SeriesDefinitionImpl
						.create();
				bemessungsverkehrsdichteSeriesDefinition.getSeriesPalette()
						.getEntries().clear();
				yAxisBemessungsverkehrsdichte.getSeriesDefinitions().add(
						bemessungsverkehrsdichteSeriesDefinition);
			}
			aktuelleSeriesDefinition = bemessungsverkehrsdichteSeriesDefinition;
		} else if (yAxisBemessungsverkehrsstaerke != null
				&& TypVerkehrsDaten.QB.equals(lp.getTypVerkehrsDaten())) {
			if (bemessungsverkehrsstaerkeSeriesDefinition == null) {
				bemessungsverkehrsstaerkeSeriesDefinition = SeriesDefinitionImpl
						.create();
				bemessungsverkehrsstaerkeSeriesDefinition.getSeriesPalette()
						.getEntries().clear();
				yAxisBemessungsverkehrsstaerke.getSeriesDefinitions().add(
						bemessungsverkehrsstaerkeSeriesDefinition);
			}
			aktuelleSeriesDefinition = bemessungsverkehrsstaerkeSeriesDefinition;
		} else if (yAxisFahrezeugdichte != null
				&& (TypVerkehrsDaten.KKFZ.equals(lp.getTypVerkehrsDaten())
						|| TypVerkehrsDaten.KLKW.equals(lp
								.getTypVerkehrsDaten()) || TypVerkehrsDaten.KPKW
						.equals(lp.getTypVerkehrsDaten()))) {
			if (fahrzeugdichteSeriesDefinition == null) {
				fahrzeugdichteSeriesDefinition = SeriesDefinitionImpl.create();
				fahrzeugdichteSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisFahrezeugdichte.getSeriesDefinitions().add(
						fahrzeugdichteSeriesDefinition);
			}
			aktuelleSeriesDefinition = fahrzeugdichteSeriesDefinition;
		} else if (yAxisLKWAnteil != null
				&& TypVerkehrsDaten.ALKW.equals(lp.getTypVerkehrsDaten())) {
			if (lkwAnteilSeriesDefinition == null) {
				lkwAnteilSeriesDefinition = SeriesDefinitionImpl.create();
				lkwAnteilSeriesDefinition.getSeriesPalette().getEntries()
						.clear();
				yAxisLKWAnteil.getSeriesDefinitions().add(
						lkwAnteilSeriesDefinition);
			}
			aktuelleSeriesDefinition = lkwAnteilSeriesDefinition;
		}

		final List<Double> result = new ArrayList<Double>();
		result.add(0d);
		final NumberDataSet orthoValuesDataSet1 = NumberDataSetImpl
				.create(result);
		Series s = null;
		if (SeriesType.LINE.equals(lp.getSeriesType())) {
			final LineSeries ls = (LineSeries) LineSeriesImpl.create();

			ls.setSeriesIdentifier(seriesID);
			ls.setDataSet(orthoValuesDataSet1);
			initLineColor(lp, ls);
			initLineStyle(lp, ls);
			initLineThickness(lp, ls);
			s = ls;
		} else if (SeriesType.BAR.equals(lp.getSeriesType())) {
			final BarSeries bs = (BarSeries) BarSeriesImpl.create();

			bs.setSeriesIdentifier(seriesID);
			bs.setDataSet(orthoValuesDataSet1);
			bs.setRiserOutline(ColorDefinitionImpl.BLACK());
			initBarColor(lp, bs);
			s = bs;
		} else if (SeriesType.BAR_STACKED.equals(lp.getSeriesType())) {
			final BarSeries bs = (BarSeries) BarSeriesImpl.create();
			bs.setStacked(true);
			bs.setSeriesIdentifier(seriesID);
			bs.setDataSet(orthoValuesDataSet1);
			bs.setRiserOutline(ColorDefinitionImpl.BLACK());
			initBarColor(lp, bs);
			s = bs;
		}

		if (aktuelleSeriesDefinition.getSeries().size() == 0) {
			aktuelleSeriesDefinition.getSeriesPalette().getEntries().clear();
		}

		for (int i = 0; i < aktuelleSeriesDefinition.getSeries().size(); i++) {
			final Series alteSeries = aktuelleSeriesDefinition.getSeries().get(
					i);
			if (alteSeries.getSeriesIdentifier().equals(seriesID)) {

				aktuelleSeriesDefinition.getSeriesPalette().getEntries()
						.remove(i);
				aktuelleSeriesDefinition.getSeries().remove(i);
				break;
			}
		}

		initSeriesVisibility(lp, s);
		aktuelleSeriesDefinition.getSeries().add(s);
		aktuelleSeriesDefinition.getSeriesPalette().getEntries().add(
				StreckenprofilUtil.createColorDefinition(lp.getKnotenFarbe()));

		// ls.setConnectMissingValue(false);
		final DataPoint dp = s.getDataPoint();
		final EList<DataPointComponent> components = dp.getComponents();
		components.clear();
		components.add(DataPointComponentImpl.create(
				DataPointComponentType.SERIES_VALUE_LITERAL, null));
		components.add(DataPointComponentImpl.create(
				DataPointComponentType.ORTHOGONAL_VALUE_LITERAL, null));
		components.add(DataPointComponentImpl.create(
				DataPointComponentType.BASE_VALUE_LITERAL, null));

	}

	/**
	 * Builds Y axis.
	 */
	private void buildYAxis() {
		for (final LinienEigenschaften lp : model.getLineProperties()) {
			final TypVerkehrsDaten datenQuelle = lp.getTypVerkehrsDaten();
			if (datenQuelle != null) {
				Axis neueAchse = null;
				if (yAxisAnzahlFz == null && yAxisGeschwindigkeit == null
						&& yAxisBemessungsverkehrsdichte == null
						&& yAxisBemessungsverkehrsstaerke == null
						&& yAxisLKWAnteil == null
						&& yAxisFahrezeugdichte == null) {
					// erste Achse
					neueAchse = ((ChartWithAxes) chart)
							.getPrimaryOrthogonalAxis(xAxis);
					xAxis.getAssociatedAxes().add(neueAchse);
				} else {
					neueAchse = AxisImpl.create(Axis.ORTHOGONAL);
				}

				if (TypVerkehrsDaten.QKFZ.equals(datenQuelle)
						|| TypVerkehrsDaten.QLKW.equals(datenQuelle)
						|| TypVerkehrsDaten.QPKW.equals(datenQuelle)) {
					konfiguriereYAxisAnzahlFz(neueAchse);
					if (yAxisAnzahlFz == null) {
						// Anzahl Fahrzeuge
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisAnzahlFz = neueAchse;
					}
				} else if (TypVerkehrsDaten.VKFZ.equals(datenQuelle)
						|| TypVerkehrsDaten.VLKW.equals(datenQuelle)
						|| TypVerkehrsDaten.VPKW.equals(datenQuelle)) {
					konfiguriereYAxisGeschwindigkeit(neueAchse);
					if (yAxisGeschwindigkeit == null) {
						// Geschwindigkeit Fahrzeuge
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisGeschwindigkeit = neueAchse;
					}
				} else if ((TypVerkehrsDaten.KKFZ.equals(datenQuelle)
						|| TypVerkehrsDaten.KLKW.equals(datenQuelle) || TypVerkehrsDaten.KPKW
						.equals(datenQuelle))) {
					konfiguriereYAxisFahrezeugdichte(neueAchse);
					if (yAxisFahrezeugdichte == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisFahrezeugdichte = neueAchse;
					}
				} else if (TypVerkehrsDaten.ALKW.equals(datenQuelle)) {
					konfiguriereYAxisLKWAnteil(neueAchse);
					// Anteil LKW
					if (yAxisLKWAnteil == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisLKWAnteil = neueAchse;
					}

				} else if (TypVerkehrsDaten.KB.equals(datenQuelle)) {
					konfiguriereYAxisBemessungsverkehrsdichte(neueAchse);
					// Bemessungsverkehrsdichte
					if (yAxisBemessungsverkehrsdichte == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisBemessungsverkehrsdichte = neueAchse;
					}
				} else if (TypVerkehrsDaten.QB.equals(datenQuelle)) {
					konfiguriereYAxisBemessungsverkehrsstaerke(neueAchse);
					// Bemessungsverkehrsstaerke
					if (yAxisBemessungsverkehrsstaerke == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisBemessungsverkehrsstaerke = neueAchse;
					}
				} else {
					throw new IllegalArgumentException("Unbekannter Achsentyp "
							+ datenQuelle);
				}
			}
		}
		// yAxisQ = ((ChartWithAxes) chart).getPrimaryOrthogonalAxis(xAxis);
		// yAxisQ.setTitle("Y - Achse");
	}

	/**
	 * Erzeugt den Teil der Legende, der die dynamische und statische
	 * Schilderung darstellt.
	 */
	private void erzeugeGeschwindigkeitsBeschraenkungenLegende() {
		if (verkehrsBeschreankungSchilderSeriesDefinition == null) {
			verkehrsBeschreankungSchilderSeriesDefinition = SeriesDefinitionImpl
					.create();

			schilderAchse = AxisImpl.create(Axis.ORTHOGONAL);
			schilderAchse.getLineAttributes().setVisible(false);
			schilderAchse.getLabel().setVisible(false);
			final Scale scale = schilderAchse.getScale();
			scale.setMin(NumberDataElementImpl.create(0));
			scale.setMax(NumberDataElementImpl.create(1));
			scale.setStepNumber(1);
			((ChartWithAxes) getChart()).getPrimaryBaseAxes()[0]
					.getAssociatedAxes().add(schilderAchse);
			schilderAchse.getSeriesDefinitions().add(
					verkehrsBeschreankungSchilderSeriesDefinition);
		}

		if (model != null
				&& model.getChartProperties().isAnzeigeQuerschnitteIcons()) {
			final NumberDataSet orthoValuesDataSet1 = NumberDataSetImpl
					.create(1);
			final String seriesID = "dyn. Anzeigen";
			final ScatterSeries ls = (ScatterSeries) ScatterSeriesImpl.create();
			ls.setSeriesIdentifier(seriesID);
			ls.setSeriesIdentifier("dyn. Anzeigen");

			ls.setDataSet(orthoValuesDataSet1);

			for (int i = 0; i < ls.getMarkers().size(); i++) {
				ls.getMarkers().get(i).setType(MarkerType.ICON_LITERAL);
				ls.getMarkers().get(i).setSize(20);
				ls
						.getMarkers()
						.get(i)
						.setFill(
								ImageImpl
										.create(
												Activator.DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_100_ICON_MARKER_URL)
										.copyInstance());
			}

			verkehrsBeschreankungSchilderSeriesDefinition.getSeries().add(ls);

		}

		if (model != null
				&& model.getChartProperties().isGeschwindigkeitsLimitIcons()) {
			final NumberDataSet orthoValuesDataSet1 = NumberDataSetImpl
					.create(1);
			final String seriesID = "stat. Beschilderung";
			final ScatterSeries ls = (ScatterSeries) ScatterSeriesImpl.create();
			ls.setSeriesIdentifier(seriesID);
			ls.setSeriesIdentifier("stat. Beschilderung");

			ls.setDataSet(orthoValuesDataSet1);

			for (int i = 0; i < ls.getMarkers().size(); i++) {
				ls.getMarkers().get(i).setType(MarkerType.ICON_LITERAL);
				ls.getMarkers().get(i).setSize(20);
				ls
						.getMarkers()
						.get(i)
						.setFill(
								ImageImpl
										.create(
												Activator.GESCHWINDIGKEITSBESCHRAENKUNG_100_ICON_MARKER_URL)
										.copyInstance());
			}

			verkehrsBeschreankungSchilderSeriesDefinition.getSeries().add(ls);

		}

		if (model != null
				&& model.getChartProperties().isGeschwindigkeitsLimitLinieDyn()) {
			final NumberDataSet orthoValuesDataSet1 = NumberDataSetImpl
					.create(1);
			final String seriesID = "Geschw - Max";
			final LineSeries ls = (LineSeries) LineSeriesImpl.create();
			ls.setSeriesIdentifier(seriesID);

			ls.setDataSet(orthoValuesDataSet1);

			for (int i = 0; i < ls.getMarkers().size(); i++) {
				ls.getMarkers().get(i).setVisible(false);
			}

			ls.setConnectMissingValue(false);
			ls.getLineAttributes().setColor(
					ColorDefinitionImpl.create(255, 0, 0));
			ls.getLineAttributes().setThickness(10);
			verkehrsBeschreankungSchilderSeriesDefinition.getSeries().add(ls);

		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #erzeugeDynamischeAnzeigen(java.util.Map)
	 */
	@Override
	public void erzeugeDynamischeAnzeigen(
			final Map<AnzeigeQuerschnitt, Map<Anzeige, Daten>> anzeigeDaten) {
		// die Legende ist unabhngig von aktuellen Anzeigen.
	}

	@Override
	public void setAlleAchsenSichtbarkeit(final boolean visible) {
		setSichtbarkeitAchse(yAxisAnzahlFz, visible);
		setSichtbarkeitAchse(yAxisBemessungsverkehrsdichte, visible);
		setSichtbarkeitAchse(yAxisBemessungsverkehrsstaerke, visible);
		setSichtbarkeitAchse(yAxisFahrezeugdichte, visible);
		setSichtbarkeitAchse(yAxisGeschwindigkeit, visible);
		setSichtbarkeitAchse(yAxisLKWAnteil, visible);
	}

}
