/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart;

import java.util.Map;

import org.eclipse.birt.chart.model.Chart;
import org.eclipse.birt.chart.model.ChartWithAxes;
import org.eclipse.birt.chart.model.attribute.IntersectionType;
import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.component.impl.AxisImpl;
import org.eclipse.birt.chart.model.impl.ChartWithAxesImpl;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AchsenBeschreibung;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypVerkehrsDaten;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.Anzeige;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeEigenschaftIst.Daten;

/**
 * ChartBuilder fr die Darstellung der Y-Achsen getrennt vom eigentlichen
 * Chart.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilYAxisRightChartBuilder.java,v 1.4 2014/04/28 13:02:07 Pittner Exp $
 */
public class StreckenprofilYAxisRightChartBuilder extends
		AbstractStreckenprofilBuilder {

	/**
	 * Das BIRT Chart.
	 */
	private ChartWithAxes chart;

	/**
	 * Die X Achse.
	 */
	private Axis xAxis;

	/**
	 * Die {@link Axis} fr die Anzahl der Fahrzeuge pro Intervall.
	 */
	private Axis yAxisAnzahlFz;

	/**
	 * Die {@link Axis} fr die Geschwindigkeiten.
	 */
	private Axis yAxisGeschwindigkeit;

	/**
	 * Die {@link Axis} fr die Bemessungsverkehrsdichte.
	 */
	private Axis yAxisBemessungsverkehrsdichte;

	/**
	 * Die {@link Axis} fr die Bemessungsverkehrsstrke.
	 */
	private Axis yAxisBemessungsverkehrsstaerke;

	/**
	 * Die {@link Axis} fr den prozentualen Anteil der LKW.
	 */
	private Axis yAxisLKWAnteil;

	/**
	 * Die {@link Axis} fr die Fahrzeugdichte.
	 */
	private Axis yAxisFahrezeugdichte;

	/**
	 * Erzeugt einen neuen {@link StreckenprofilYAxisRightChartBuilder}.
	 * 
	 * @param model
	 *            das Streckenprofil.
	 */
	public StreckenprofilYAxisRightChartBuilder(final Modell model) {
		super(model);
		buildChart();
	}

	/**
	 * Erzeugt ein neues {@link Chart}.
	 */
	public void buildChart() {

		chart = ChartWithAxesImpl.create();
		xAxis = (chart).getPrimaryBaseAxes()[0];

		chart.getPlot().setVisible(true);
		chart.getTitle().getLabel().getCaption().setValue("");
		chart.getTitle().getLabel().getCaption().getFont().setSize(14);
		chart.getTitle().getLabel().getCaption().getFont().setName(FONT_NAME);
		chart.getTitle().getLabel().getCaption().getFont().setWordWrap(false);

		xAxis.getSeriesDefinitions().clear();
		xAxis.getLineAttributes().setVisible(false);
		xAxis.getTitle().setVisible(true);
		xAxis.getTitle().getCaption().setValue("");
		xAxis.getLabel().getCaption().setColor(ColorDefinitionImpl.WHITE());
		xAxis.getMajorGrid().getLineAttributes().setVisible(false);
		xAxis.getMajorGrid().getTickAttributes().setVisible(false);

		buildYAxis();
		if (yAxisGeschwindigkeit == null) {
			if ((getModel().getChartProperties().isGeschwindigkeitsLimitLinieDyn())
					&& Position.RIGHT_LITERAL.equals(getModel()
							.getChartProperties().getYAxisTypeGeschwindigkeit()
							.getAchsenposition())) {
				if (yAxisAnzahlFz == null
						&& yAxisBemessungsverkehrsdichte == null
						&& yAxisBemessungsverkehrsstaerke == null
						&& yAxisLKWAnteil == null
						&& yAxisFahrezeugdichte == null) {
					// erste Achse
					yAxisGeschwindigkeit = chart
							.getPrimaryOrthogonalAxis(xAxis);
					// xAxis.getAssociatedAxes().add(neueAchse);
				} else {
					yAxisGeschwindigkeit = AxisImpl.create(Axis.ORTHOGONAL);
				}

				konfiguriereYAxisGeschwindigkeit(yAxisGeschwindigkeit);
				xAxis.getAssociatedAxes().add(yAxisGeschwindigkeit);
			}
		}

		if (yAxisAnzahlFz == null && yAxisGeschwindigkeit == null
				&& yAxisBemessungsverkehrsdichte == null
				&& yAxisBemessungsverkehrsstaerke == null
				&& yAxisLKWAnteil == null && yAxisFahrezeugdichte == null) {
			// wenn keine Achse sichtbar ist, dann wird das ganze Chart
			// unsichtbar
			chart.getPlot().setVisible(false);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #getChart()
	 */
	@Override
	public Chart getChart() {
		return chart;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #getTitle()
	 */
	@Override
	public String getTitle() {
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #setLineData(de.bsvrz.buv.plugin.streckenprofil.model.LineProperties,
	 * java.util.Map)
	 */
	@Override
	public void setLineData(final LinienEigenschaften lp,
			final Map<SystemObjekt, OnlineDatum> daten) {
		// nix tun
	}

	/**
	 * Builds Y axis.
	 */
	private void buildYAxis() {

		for (final LinienEigenschaften lp : getModel().getLineProperties()) {
			final TypVerkehrsDaten datenQuelle = lp.getTypVerkehrsDaten();
			if (datenQuelle != null) {
				Axis neueAchse = null;

				if (yAxisAnzahlFz == null && yAxisGeschwindigkeit == null
						&& yAxisBemessungsverkehrsdichte == null
						&& yAxisBemessungsverkehrsstaerke == null
						&& yAxisLKWAnteil == null
						&& yAxisFahrezeugdichte == null) {
					// erste Achse
					neueAchse = chart.getPrimaryOrthogonalAxis(xAxis);
					// xAxis.getAssociatedAxes().add(neueAchse);
				} else {
					neueAchse = AxisImpl.create(Axis.ORTHOGONAL);
				}

				if (Position.RIGHT_LITERAL.equals(getModel()
						.getChartProperties().getYAxisTypAnzahl()
						.getAchsenposition())
						&& (TypVerkehrsDaten.QKFZ.equals(datenQuelle)
								|| TypVerkehrsDaten.QLKW.equals(datenQuelle) || TypVerkehrsDaten.QPKW
								.equals(datenQuelle))) {
					konfiguriereYAxisAnzahlFz(neueAchse);
					if (yAxisAnzahlFz == null) {
						// Anzahl Fahrzeuge
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisAnzahlFz = neueAchse;
					}
				} else if (Position.RIGHT_LITERAL.equals(getModel()
						.getChartProperties().getYAxisTypeGeschwindigkeit()
						.getAchsenposition())
						&& (TypVerkehrsDaten.VKFZ.equals(datenQuelle)
								|| TypVerkehrsDaten.VLKW.equals(datenQuelle) || TypVerkehrsDaten.VPKW
								.equals(datenQuelle))) {
					konfiguriereYAxisGeschwindigkeit(neueAchse);
					if (yAxisGeschwindigkeit == null) {
						// Geschwindigkeit Fahrzeuge
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisGeschwindigkeit = neueAchse;
					}
				} else if (Position.RIGHT_LITERAL.equals(getModel()
						.getChartProperties().getYAxisFahrezeugdichte()
						.getAchsenposition())
						&& (TypVerkehrsDaten.KKFZ.equals(datenQuelle)
								|| TypVerkehrsDaten.KLKW.equals(datenQuelle) || TypVerkehrsDaten.KPKW
								.equals(datenQuelle))) {
					konfiguriereYAxisFahrezeugdichte(neueAchse);
					if (yAxisFahrezeugdichte == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisFahrezeugdichte = neueAchse;
					}
				} else if (Position.RIGHT_LITERAL.equals(getModel()
						.getChartProperties().getYAxisLKWAnteil()
						.getAchsenposition())
						&& TypVerkehrsDaten.ALKW.equals(datenQuelle)) {
					konfiguriereYAxisLKWAnteil(neueAchse);
					// Anteil LKW
					if (yAxisLKWAnteil == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisLKWAnteil = neueAchse;
					}

				} else if (Position.RIGHT_LITERAL
						.equals(getModel().getChartProperties()
								.getYAxisBemessungsverkehrsdichte()
								.getAchsenposition())
						&& TypVerkehrsDaten.KB.equals(datenQuelle)) {
					konfiguriereYAxisBemessungsverkehrsdichte(neueAchse);
					// Bemessungsverkehrsdichte
					if (yAxisBemessungsverkehrsdichte == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisBemessungsverkehrsdichte = neueAchse;
					}
				} else if (Position.RIGHT_LITERAL.equals(getModel()
						.getChartProperties()
						.getYAxisBemessungsverkehrsstaerke()
						.getAchsenposition())
						&& TypVerkehrsDaten.QB.equals(datenQuelle)) {
					konfiguriereYAxisBemessungsverkehrsstaerke(neueAchse);
					// Bemessungsverkehrsstaerke
					if (yAxisBemessungsverkehrsstaerke == null) {
						xAxis.getAssociatedAxes().add(neueAchse);
						yAxisBemessungsverkehrsstaerke = neueAchse;
					}
				}
				neueAchse.getMajorGrid().getLineAttributes().setVisible(false);
				neueAchse.getMinorGrid().getLineAttributes().setVisible(false);
			}

		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #erzeugeDynamischeAnzeigen(java.util.Map)
	 */
	@Override
	public void erzeugeDynamischeAnzeigen(
			final Map<AnzeigeQuerschnitt, Map<Anzeige, Daten>> anzeigeDaten) {
		// die Legende ist unabhngig von aktuellen Anzeigen.
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	 * #bestimmeSkalen(org.eclipse.birt.chart.model.component.Axis,
	 * de.bsvrz.buv.plugin.streckenprofil.model.AchsenBeschreibung)
	 */
	@Override
	protected void bestimmeSkalen(final Axis neueAchse,
			final AchsenBeschreibung axisProps) {
		super.bestimmeSkalen(neueAchse, axisProps);
		neueAchse.getOrigin().setType(IntersectionType.MIN_LITERAL);
	}

	@Override
	public void setAlleAchsenSichtbarkeit(final boolean visible) {
		setSichtbarkeitAchse(yAxisAnzahlFz, visible);
		setSichtbarkeitAchse(yAxisBemessungsverkehrsdichte, visible);
		setSichtbarkeitAchse(yAxisBemessungsverkehrsstaerke, visible);
		setSichtbarkeitAchse(yAxisFahrezeugdichte, visible);
		setSichtbarkeitAchse(yAxisGeschwindigkeit, visible);
		setSichtbarkeitAchse(yAxisLKWAnteil, visible);

	}

	// /**
	// * Testet, ob die betreffende {@link AchsenBeschreibung} in diesem Chart
	// * angezeigt werden soll. <code>true</code>, wenn "ja", sonst
	// * <code>false</code>.
	// *
	// * @param b
	// * die zu berprfende {@link AchsenBeschreibung}
	// * @return <code>true</code>, wenn die betreffende
	// * {@link AchsenBeschreibung} angezeigt werden soll, sonst
	// * <code>false</code>
	// */
	// private boolean testeYAchsenPosition(final AchsenBeschreibung b) {
	// boolean result = false;
	// if (b != null && Position.RIGHT_LITERAL.equals(b.getAchsenposition())) {
	// result = true;
	// }
	// return result;
	// }
	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see
	// * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	// * #konfiguriereYAxisAnzahlFz(org.eclipse.birt.chart.model.component.Axis)
	// */
	// @Override
	// protected void konfiguriereYAxisAnzahlFz(final Axis neueAchse) {
	// if (testeYAchsenPosition(getModel().getChartProperties()
	// .getYAxisTypAnzahl())) {
	// super.konfiguriereYAxisAnzahlFz(neueAchse);
	// } else {
	// xAxis.getAssociatedAxes().remove(neueAchse);
	// yAxisAnzahlFz = null;
	// }
	// }
	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see
	// * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	// * #konfiguriereYAxisBemessungsverkehrsdichte
	// * (org.eclipse.birt.chart.model.component.Axis)
	// */
	// @Override
	// protected void konfiguriereYAxisBemessungsverkehrsdichte(
	// final Axis neueAchse) {
	// if (testeYAchsenPosition(getModel().getChartProperties()
	// .getYAxisBemessungsverkehrsdichte())) {
	// super.konfiguriereYAxisBemessungsverkehrsdichte(neueAchse);
	// } else {
	// xAxis.getAssociatedAxes().remove(neueAchse);
	// yAxisBemessungsverkehrsdichte = null;
	// }
	//
	// }
	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see
	// * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	// * #konfiguriereYAxisBemessungsverkehrsstaerke
	// * (org.eclipse.birt.chart.model.component.Axis)
	// */
	// @Override
	// protected void konfiguriereYAxisBemessungsverkehrsstaerke(
	// final Axis neueAchse) {
	// if (testeYAchsenPosition(getModel().getChartProperties()
	// .getYAxisBemessungsverkehrsstaerke())) {
	// super.konfiguriereYAxisBemessungsverkehrsstaerke(neueAchse);
	// } else {
	// xAxis.getAssociatedAxes().remove(neueAchse);
	// yAxisBemessungsverkehrsstaerke = null;
	// }
	//
	// }
	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see
	// * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	// * #konfiguriereYAxisFahrezeugdichte
	// * (org.eclipse.birt.chart.model.component.Axis)
	// */
	// @Override
	// protected void konfiguriereYAxisFahrezeugdichte(final Axis neueAchse) {
	// if (testeYAchsenPosition(getModel().getChartProperties()
	// .getYAxisFahrezeugdichte())) {
	// super.konfiguriereYAxisFahrezeugdichte(neueAchse);
	// } else {
	// xAxis.getAssociatedAxes().remove(neueAchse);
	// yAxisFahrezeugdichte = null;
	// }
	//
	// }
	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see
	// * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	// * #konfiguriereYAxisGeschwindigkeit
	// * (org.eclipse.birt.chart.model.component.Axis)
	// */
	// @Override
	// protected void konfiguriereYAxisGeschwindigkeit(final Axis neueAchse) {
	// if (testeYAchsenPosition(getModel().getChartProperties()
	// .getYAxisTypeGeschwindigkeit())) {
	// super.konfiguriereYAxisGeschwindigkeit(neueAchse);
	// } else {
	// xAxis.getAssociatedAxes().remove(neueAchse);
	// yAxisGeschwindigkeit = null;
	// }
	//
	// }
	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see
	// * de.bsvrz.buv.plugin.streckenprofil.chart.AbstractStreckenprofilBuilder
	// *
	// #konfiguriereYAxisLKWAnteil(org.eclipse.birt.chart.model.component.Axis)
	// */
	// @Override
	// protected void konfiguriereYAxisLKWAnteil(final Axis neueAchse) {
	// if (testeYAchsenPosition(getModel().getChartProperties()
	// .getYAxisLKWAnteil())) {
	// super.konfiguriereYAxisLKWAnteil(neueAchse);
	// } else {
	// xAxis.getAssociatedAxes().remove(neueAchse);
	// yAxisLKWAnteil = null;
	// }
	//
	// }

}
