/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import com.bitctrl.util.TwoDimensionalMap;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilCanvas;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart.AbstractStreckenprofilBuilder;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.simulation.SingleJobOfTypeSchedulingRule;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Activateable;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AeussererStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.RealerStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenKnoten;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypDatenQuellen;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.Anzeige;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeEigenschaftIst;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrStreifenLage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdFahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.FahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.OfflineSimulation;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstrakte Basisklasse fr alle Composites, die ein Streckenprofil - Canvas
 * anzeigen mchten.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: AbstractStreckenProfilComposite.java 29377 2011-05-17 14:00:51Z
 *          hoesel $
 */
public abstract class AbstractStreckenProfilComposite extends Composite
		implements DatensatzUpdateListener {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void dispose() {
		if (modell != null && updateAdapter != null) {
			modell.getEMFModell().eAdapters().remove(updateAdapter);
			modell.getChartProperties().eAdapters().remove(updateAdapter);
		}
		super.dispose();
	}

	/**
	 * Interner {@link EContentAdapter}, der sich auf alle mglichen nderungen
	 * innerhalb des EMF Modells anmeldet.
	 */
	private final EContentAdapter updateAdapter = new EContentAdapter() {

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.ecore.util.EContentAdapter#notifyChanged
		 * (org.eclipse.emf.common.notify.Notification)
		 */
		@Override
		public void notifyChanged(final Notification notification) {
			super.notifyChanged(notification);
			final Object feature = notification.getFeature();
			if ((StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__LINE_STYLE
					.equals(feature))
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__LINE_THICKNESS
							.equals(feature))
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__RGB
							.equals(feature))
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__LINE_STYLE
							.equals(feature))
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__VISIBLE
							.equals(feature))) {
				// chartCanvas.regenerateChart();
				reloadChart();
			} else if (StreckenprofilPackage.Literals.ACTIVATEABLE__STATE
					.equals(feature)
					|| StreckenprofilPackage.Literals.ACHSEN_BESCHREIBUNG__ACHSENPOSITION
							.equals(feature)
					|| StreckenprofilPackage.Literals.ACHSEN_BESCHREIBUNG__HILFSLINIEN
							.equals(feature)
					|| StreckenprofilPackage.Literals.ACHSEN_BESCHREIBUNG__MAXIMUM
							.equals(feature)
					|| StreckenprofilPackage.Literals.ACHSEN_BESCHREIBUNG__MINIMUM
							.equals(feature)
					|| StreckenprofilPackage.Literals.ACHSEN_BESCHREIBUNG__SCHRITTE
							.equals(feature)
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__KNOTEN_ART
							.equals(feature))
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__KNOTEN_FARBE
							.equals(feature))
					|| (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__SERIES_TYPE
							.equals(feature))
					|| (StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG
							.equals(feature))
					|| (StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__ANZEIGE_QUERSCHNITTE_ICONS
							.equals(feature))
					|| (StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__GESCHWINDIGKEITS_LIMIT_ICONS
							.equals(feature))
					|| (StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__GESCHWINDIGKEITS_LIMIT_LINIE_STAT
							.equals(feature))
							|| (StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__GESCHWINDIGKEITS_LIMIT_LINIE_DYN
							    .equals(feature))
					|| (StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__TYP_PROGNOSE_TYP
							.equals(feature))) {
				reloadChart();
			} else if (StreckenprofilPackage.Literals.STRECKENPROFIL_EDITOR_MODEL__LINE_PROPERTIES
					.equals(feature)) {
				onlineDaten.clear();
				for (final LinienEigenschaften lp : modell.getLineProperties()) {
					onlineDaten
							.put(lp,
									Collections
											.synchronizedMap(new HashMap<SystemObjekt, OnlineDatum>()));
				}
				reloadChart();
			} else if (StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__SKALIERUNG_XACHSE
					.equals(feature)) {
				chartCanvas.setCanvasBreite(notification.getNewFloatValue());
				reloadChart();
			}
		}
	};
	/**
	 * Cach mit allen Rckwrtsreferenzen von DAV MQ nach EMF Modell MQ.
	 */
	private final Map<MessQuerschnittAllgemein, MessQuerschnitt> mqModelCache = new HashMap<MessQuerschnittAllgemein, MessQuerschnitt>();

	/**
	 * Map fr die Darstellung der Daten in einem
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.export.MatrixGrid}.
	 */
	private final TwoDimensionalMap<SystemObjekt, LinienEigenschaften, OnlineDatum> map = new TwoDimensionalMap<SystemObjekt, LinienEigenschaften, OnlineDatum>(
			new DavMessquerschnittComparator(), new LinePropertiesComparator());

	/**
	 * Liste mit allen angemeldeten Listenern.
	 */
	private final ListenerList updateListener = new ListenerList();

	/**
	 * Der ChartBuilder zum erstellen des BIRT-Charts.
	 */
	private AbstractStreckenprofilBuilder chartBuilder;

	/**
	 * Das verwendete {@link StreckenprofilCanvas} zum malen des Chart.
	 */
	private final StreckenprofilCanvas chartCanvas;

	/**
	 * Dieses Flag, zeigt an, ob die Darstellung gerade aktualisiert wird oder
	 * nicht.
	 */
	private boolean diagrammAktualisieren;

	/**
	 * Flag das anzeigt, ob versucht wurde die Darstellung zu akualisieren,
	 * jedoch war bereits das
	 * {@link AbstractStreckenProfilComposite#diagrammAktualisieren} Flag auf
	 * <code>true</code>.
	 */
	private boolean diagrammEventWaehrendAktualisierung;

	/**
	 * Dieses Flag zeigt an, ob der Chart momentan neu geladen wird.
	 */
	private boolean diagrammNeuLaden;

	/**
	 * Flag das anzeigt, ob versucht wurde die Darstellung neu zu laden, jedoch
	 * war bereits das {@link AbstractStreckenProfilComposite#diagrammNeuLaden}
	 * Flag auf <code>true</code>.
	 */
	private boolean diagrammEventWaehrendNeuLaden;

	/**
	 * Die momentane
	 * {@link de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.Simulation}
	 * , die innerhalb des Streckenprofils verwendet wird, oder
	 * <code>null</code>, wenn keine
	 * {@link de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.Simulation}
	 * luft.
	 */
	private OfflineSimulation chartSimulation;

	/**
	 * Map, in der wir die Onlinedaten cachen. Es wird dabei fr jede
	 * {@link LinienEigenschaften} eine Map mit {@link SystemObjekt}en und den
	 * momentan fr dieses {@link SystemObjekt} aktuellen {@link OnlineDatum}s
	 * gespeichert.
	 */
	private final Map<LinienEigenschaften, Map<SystemObjekt, OnlineDatum>> onlineDaten = new TreeMap<LinienEigenschaften, Map<SystemObjekt, OnlineDatum>>(
			new LinePropertiesComparator());

	/**
	 * Map mit allen {@link AnzeigeQuerschnitt}en. Und den aktuellen
	 * {@link OdAnzeigeEigenschaftIst.Daten} pro {@link Anzeige}.
	 */
	private final Map<AnzeigeQuerschnitt, Map<Anzeige, OdAnzeigeEigenschaftIst.Daten>> anzeigeDaten = new HashMap<AnzeigeQuerschnitt, Map<Anzeige, OdAnzeigeEigenschaftIst.Daten>>();

	/**
	 * Das {@link Modell}, das alle Informationen zum Streckenprofil enthlt.
	 */
	private Modell modell;

	/**
	 * {@link Comparator} mit dem wir {@link MessQuerschnitt} vergleichen
	 * knnen. Der auf dem
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenZug} weiter vorn
	 * liegenden {@link MessQuerschnitt} ist dabei kleiner als ein weiter hinten
	 * liegender. Es wird als das Ergebnis der Methode
	 * {@link MessQuerschnitt#getWegBisher()} zugrunde gelegt.
	 * 
	 * @author BitCtrl Systems GmbH, hoesel
	 * @version $Id: AbstractStreckenProfilComposite.java 29377 2011-05-17
	 *          14:00:51Z hoesel $
	 */
	protected class DavMessquerschnittComparator implements
			Comparator<SystemObjekt> {

		@Override
		public int compare(final SystemObjekt o1, final SystemObjekt o2) {
			if (o1 instanceof MessQuerschnittAllgemein
					&& o2 instanceof MessQuerschnittAllgemein) {
				final MessQuerschnitt mq1 = mqModelCache.get(o1);
				final MessQuerschnitt mq2 = mqModelCache.get(o2);
				if (mq1 != null && mq2 != null) {
					return Double.compare(mq1.getWegBisher(),
							mq2.getWegBisher());
				}
			}
			return 0;
		}
	}

	/**
	 * {@link Comparator} fr {@link LinienEigenschaften}. Es wird hierbei
	 * zuchst {@link LinienEigenschaften#getTypDatenQuelle()} verglichen. Bei
	 * gleichheit wird noch {@link LinienEigenschaften#getTypVerkehrsDaten()}
	 * als Vergleichswert bemht.
	 * 
	 * @author BitCtrl Systems GmbH, hoesel
	 * @version $Id: AbstractStreckenProfilComposite.java 29377 2011-05-17
	 *          14:00:51Z hoesel $
	 */
	protected class LinePropertiesComparator implements
			Comparator<LinienEigenschaften> {

		@Override
		public int compare(final LinienEigenschaften o1,
				final LinienEigenschaften o2) {
			int result = o1.getTypDatenQuelle().compareTo(
					o2.getTypDatenQuelle());
			if (result == 0) {
				result = o1.getTypVerkehrsDaten().compareTo(
						o2.getTypVerkehrsDaten());
			}
			return result;
		}

	}

	/**
	 * Konstruktor zum erzeugen eines Streckenprofil - Compositums. Hier wird
	 * das Layout des {@link Composite}s festgelegt und ein
	 * {@link StreckenprofilCanvas} diesem hinzugefgt.
	 * 
	 * @param parent
	 *            das Parent - Objekt
	 * @param style
	 *            der SWT Style
	 */
	protected AbstractStreckenProfilComposite(final Composite parent,
			final int style) {
		super(parent, style);
		setLayout(new FillLayout());
		chartCanvas = new StreckenprofilCanvas(this, style);
	}

	/**
	 * Liefert eine Map mit allen Rckwrtsreferenzen von
	 * {@link MessQuerschnittAllgemein} nach {@link MessQuerschnitt}.
	 * 
	 * @return the mqModelCache
	 */
	public final Map<MessQuerschnittAllgemein, MessQuerschnitt> getMqModelCache() {
		return mqModelCache;
	}

	/**
	 * Fllt eine {@link TwoDimensionalMap} mit der dann ein
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.export.MatrixGrid} gefllt
	 * werden kann.
	 * 
	 * @param chartDaten
	 *            die Daten fr die {@link TwoDimensionalMap}.
	 */
	protected final void fuelleTwoDimensionalMap(
			final Map<LinienEigenschaften, Map<SystemObjekt, OnlineDatum>> chartDaten) {
		map.clear();
		for (final LinienEigenschaften lp : chartDaten.keySet()) {
			final Map<SystemObjekt, OnlineDatum> daten = chartDaten.get(lp);
			for (final SystemObjekt sobj : daten.keySet()) {
				if (sobj instanceof FahrStreifen
						&& !lp.getTypDatenQuelle().equals(TypDatenQuellen.MQ)) {
					final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt mq = StartupPlugin
							.getDefault().getDefaultMqCache()
							.getMessquerschnitt((FahrStreifen) sobj);
					final FahrStreifen fs = (FahrStreifen) sobj;
					final KdFahrStreifen.Daten fsDaten = fs.getKdFahrStreifen()
							.getDatum();
					if (fsDaten != null) {
						final AttFahrStreifenLage lage = fsDaten.getLage();
						if ((TypDatenQuellen.HFS.equals(lp.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_0_HFS
								.equals(lage))
								|| (TypDatenQuellen.UEFS1.equals(lp
										.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_1__1UEFS
										.equals(lage))
								|| (TypDatenQuellen.UEFS2.equals(lp
										.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_2__2UEFS
										.equals(lage))
								|| (TypDatenQuellen.UEFS3.equals(lp
										.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_3__3UEFS
										.equals(lage))
								|| (TypDatenQuellen.UEFS4.equals(lp
										.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_4__4UEFS
										.equals(lage))
								|| (TypDatenQuellen.UEFS5.equals(lp
										.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_5__5UEFS
										.equals(lage))
								|| (TypDatenQuellen.UEFS6.equals(lp
										.getTypDatenQuelle()) && AttFahrStreifenLage.ZUSTAND_6__6UEFS
										.equals(lage))) {
							map.put(mq, lp, daten.get(sobj));
						}
					}

				} else if (sobj instanceof MessQuerschnittAllgemein
						&& lp.getTypDatenQuelle().equals(TypDatenQuellen.MQ)) {
					map.put(sobj, lp, daten.get(sobj));
				}
			}
		}
		for (final Object obj : updateListener.getListeners()) {
			if (obj instanceof IChartUpdateListener) {
				((IChartUpdateListener) obj).datensatzAktualisiert(map);
			}
		}

	}

	/**
	 * Abmeldung am Dav und aufrumen der verschiedenen Caches.
	 */
	protected void abmeldung() {
		for (final Map<SystemObjekt, OnlineDatum> m : onlineDaten.values()) {
			m.clear();
		}

		if (chartSimulation != null) {
			final int simuVariante = chartSimulation
					.getKdSimulationsEigenschaften().getDatum()
					.getSimulationsVariante().intValue();
			for (final AnzeigeQuerschnitt aq : anzeigeDaten.keySet()) {
				for (final Anzeige anz : anzeigeDaten.get(aq).keySet()) {
					anz.getOdAnzeigeEigenschaftIst().removeUpdateListener(
							OdAnzeigeEigenschaftIst.Aspekte.AqZustand,
							(short) simuVariante,
							AbstractStreckenProfilComposite.this);
				}
			}

		} else {

			for (final AnzeigeQuerschnitt aq : anzeigeDaten.keySet()) {
				for (final Anzeige anz : anzeigeDaten.get(aq).keySet()) {
					anz.getOdAnzeigeEigenschaftIst().removeUpdateListener(
							OdAnzeigeEigenschaftIst.Aspekte.AqZustand,
							AbstractStreckenProfilComposite.this);
				}
			}
		}
		anzeigeDaten.clear();
		mqModelCache.clear();

		map.clear();
	}

	/**
	 * Liefert eine {@link TwoDimensionalMap} fr die Darstellung in einem
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.export.MatrixGrid}.
	 * 
	 * @return {@link TwoDimensionalMap} zum Fllen eines
	 *         {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.export.MatrixGrid} mit
	 *         den aktuellen Verkehrsdaten.
	 */
	public final TwoDimensionalMap<SystemObjekt, LinienEigenschaften, OnlineDatum> getGridMap() {
		return map;
	}

	/**
	 * 
	 * 
	 * Anmeldung fr einen {@link IChartUpdateListener}, der von diesem
	 * {@link AbstractStreckenProfilComposite} bei nderungen im Streckenprofil
	 * - Chart benachrichtigt werden soll.
	 * 
	 * @param listener
	 *            der {@link IChartUpdateListener}.
	 */
	public void addUpdateListener(final IChartUpdateListener listener) {
		updateListener.add(listener);
	}

	/**
	 * Entfernt einen {@link IChartUpdateListener} aus der Menge zu
	 * benachrichtigender Objekt.
	 * 
	 * @param listener
	 *            der {@link IChartUpdateListener}, der nicht mehr
	 *            benachrichtigt werden mchte.
	 */
	public void removeUpdateListener(final IChartUpdateListener listener) {
		updateListener.remove(listener);
	}

	/**
	 * Anmelden fr Verkehrsdaten eines {@link MessQuerschnittAllgemein} und
	 * dessen {@link FahrStreifen}.
	 * 
	 * @param mq
	 *            der {@link MessQuerschnitt} dessen {@link SystemObjekt} -
	 *            {@link MessQuerschnittAllgemein} ( und dessen
	 *            {@link FahrStreifen}) sich fr {@link OnlineDatum} anmelden
	 *            mchten.
	 */
	protected void mQundFsAnmelden(final MessQuerschnitt mq) {
		if (ActivationState.ACTIVE.equals(mq.getState())) {
			getMqModelCache().put(mq.getMessQuerschnitt(), mq);
		}
	}

	/**
	 * Liefert das {@link Modell} des Streckenprofils.
	 * 
	 * @return das {@link Modell} des Streckenprofils.
	 */
	public Modell getStreckenEditorModell() {
		return modell;
	}

	/**
	 * Setzt das {@link Modell}, dessen
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a81.model.StreckenprofilEditorModel}
	 * in einem Chart angezeigt werden soll.
	 * 
	 * @param neuesModell
	 *            das neue {@link Modell} des Streckenprofils.
	 */
	public void setStreckenEditorModell(final Modell neuesModell) {

		abmeldung();
		if (modell != null) {
			modell.getEMFModell().eAdapters().remove(updateAdapter);
			modell.getChartProperties().eAdapters().remove(updateAdapter);
		}
		modell = neuesModell;
		for (final Object obj : neuesModell.getLineProperties()) {
			final LinienEigenschaften lp = (LinienEigenschaften) obj;
			onlineDaten.put(lp, new HashMap<SystemObjekt, OnlineDatum>());
		}
		if (chartBuilder != null) {
			chartBuilder.dispose();
			chartBuilder = null;
		}
		chartBuilder = createStreckenprofilBuilder(neuesModell);
		chartCanvas.setChart(chartBuilder.getChart());

		chartCanvas.setCanvasBreite(modell.getChartProperties()
				.getSkalierungXAchse());

		final List<Activateable> objListe = new ArrayList<Activateable>();
		objListe.addAll(modell.getAlleMessquerschnitte(true));
		objListe.addAll(modell.getAlleStreckenKnoten());

		chartCanvas.fuelleStreckenObjektListe(objListe);
		anmeldung();

		modell.getEMFModell().eAdapters().add(updateAdapter);
		modell.getChartProperties().eAdapters().add(updateAdapter);

	}

	/**
	 * Erzeugt mit jedem Aufruf eine neue Instanz eines
	 * {@link AbstractStreckenprofilBuilder}s zum erzeugen eines BIRT-Charts.
	 * 
	 * @param model
	 *            das {@link Modell} des Streckenprofils.
	 * @return ein neuer {@link AbstractStreckenprofilBuilder} zum erzeugen
	 *         eines BIRT - Charts.
	 */
	public abstract AbstractStreckenprofilBuilder createStreckenprofilBuilder(
			Modell model);

	/**
	 * Setzt die Offline - Simulation, die von dem Streckenprofil in der
	 * Funktionalitt "Film abspielen" verwendet wird.
	 * 
	 * @param simulation
	 *            die {@link OfflineSimulation}, die vom Streckenprofil
	 *            angezeigt werden soll, oder <code>null</code>, wenn live -
	 *            Daten angezeigt werden sollen.
	 */
	public void setSimulation(final OfflineSimulation simulation) {

		if (simulation == null) {
			// es geht live weiter
			abmeldung();
			chartSimulation = null;
			reloadChart();
		} else if (!simulation.equals(chartSimulation)) {
			// von einer Simulation zur nchsten.
			abmeldung();
			chartSimulation = simulation;
			reloadChart();
		}

	}

	/**
	 * Liefert die momentan genutzte {@link OfflineSimulation} oder
	 * <code>null</code>, wenn keine Simulation luft.
	 * 
	 * @return die aktuell vom Streckenprofil genutzte {@link OfflineSimulation}
	 *         , oder <code>null</code>, wenn nicht simuliert wird.
	 */
	public OfflineSimulation getSimulation() {
		return chartSimulation;
	}

	/**
	 * Liefert den Titeltext des Streckenprofils.
	 * 
	 * @return der Titeltext des Streckenprofils.
	 */
	public String getTitle() {
		return chartBuilder.getTitle();
	}

	/**
	 * Erstellt das Chart neu, inkl. An- und Abmeldung am DAV.
	 */
	private void reloadChart() {

		if (diagrammNeuLaden) {
			// chart wird schon aktualisiert
			diagrammEventWaehrendNeuLaden = true;
		} else {
			diagrammNeuLaden = true;

			final Job j = new Job("Diagramm neu aufbauen...") {

				@Override
				protected IStatus run(final IProgressMonitor monitor) {
					modell.getRealm().exec(new Runnable() {

						@Override
						public void run() {
							abmeldung();
							do {
								diagrammEventWaehrendNeuLaden = false;
								if (chartBuilder != null) {
									chartBuilder.dispose();
									chartBuilder = null;
								}

								chartBuilder = createStreckenprofilBuilder(modell);
								chartCanvas.setChart(chartBuilder.getChart());
								anmeldung();
								final List<Activateable> objListe = new ArrayList<Activateable>();
								objListe.addAll(modell
										.getAlleMessquerschnitte(true));
								objListe.addAll(modell.getAlleStreckenKnoten());

								chartCanvas.fuelleStreckenObjektListe(objListe);
								chartCanvas.regenerateChart();
								layout(true, true);
							} while (diagrammEventWaehrendNeuLaden);

							diagrammNeuLaden = false;
						}
					});
					return Status.OK_STATUS;
				}
			};
			j.setRule(new SingleJobOfTypeSchedulingRule());
			j.schedule(500);
		}
	}

	/**
	 * Aktualisiert den Chart - kein komplettes neu Malen des Charts.
	 */
	protected void chartAktualisieren() {

		if (diagrammAktualisieren) {
			diagrammEventWaehrendAktualisierung = true;
		} else {
			diagrammAktualisieren = true;

			final Job j = new Job("Update Chart") {

				@Override
				protected IStatus run(final IProgressMonitor monitor) {
					if (chartCanvas != null && !chartCanvas.isDisposed()) {
						chartCanvas.getDisplay().syncExec(new Runnable() {
							@Override
							public void run() {
								do {
									diagrammEventWaehrendAktualisierung = false;
									diagrammAktualisieren = true;
									if (chartCanvas != null
											&& !chartCanvas.isDisposed()) {
										try {
											for (final LinienEigenschaften lp : onlineDaten
													.keySet()) {
												updateChartData(lp,
														onlineDaten.get(lp));
											}
											fuelleTwoDimensionalMap(onlineDaten);
										} catch (final ConcurrentModificationException e) {
											// es wurde sich whrend eines
											// Updates schon
											// wieder fr ein anderes
											// Streckenprofil
											// angemeldet.
											Debug.getLogger()
													.fine("Whrend der Aktualisierung des Streckenprofils wurde das Streckenprofil gendert.",
															e);
										}
										final List<Activateable> objListe = new ArrayList<Activateable>();
										objListe.addAll(modell
												.getAlleMessquerschnitte(true));
										objListe.addAll(modell
												.getAlleStreckenKnoten());

										chartCanvas
												.fuelleStreckenObjektListe(objListe);
										chartCanvas.regenerateChart();
										layout(true, true);

									}
								} while (diagrammEventWaehrendAktualisierung);
								diagrammAktualisieren = false;
							}
						});
					}
					return Status.OK_STATUS;
				}

			};

			j.setSystem(true);
			j.schedule(1000);

		}
	}

	/**
	 * Aktualisiert eine Linie des Charts. Wenn sich ein einziger Wert ndert,
	 * mu immer die gesamte Linie neu gemalt werde.
	 * 
	 * @param lp
	 *            die Linieneigenschaften, des zu aktualisierenden Datums.
	 * @param daten
	 *            eine Map mit {@link SystemObjekt}en und den dazugehrigen
	 *            {@link OnlineDatum}s.
	 */
	protected void updateChartData(final LinienEigenschaften lp,
			final Map<SystemObjekt, OnlineDatum> daten) {
		chartBuilder.setLineData(lp, daten);
	}

	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
	  
	  
	  System.out.println("AbstractStreckenProfilComposite.datensatzAktualisiert(): " + event.getObjekt());
	  
		if (event.getObjekt() instanceof MessQuerschnittAllgemein
				|| event.getObjekt() instanceof FahrStreifen) {
			for (final Map<SystemObjekt, OnlineDatum> odMaps : onlineDaten
					.values()) {
				odMaps.put(event.getObjekt(), event.getDatum());
			}
			chartAktualisieren();
		} else if (event.getObjekt() instanceof Anzeige) {
			if (modell.getChartProperties().isAnzeigeQuerschnitteIcons()) {
				for (final Map<Anzeige, OdAnzeigeEigenschaftIst.Daten> daten : anzeigeDaten
						.values()) {
					if (daten.keySet().contains(event.getObjekt())) {
						daten.put((Anzeige) event.getObjekt(),
								(OdAnzeigeEigenschaftIst.Daten) event
										.getDatum());
					}
				}
				chartBuilder.erzeugeDynamischeAnzeigen(anzeigeDaten);
				if (chartCanvas != null && !chartCanvas.isDisposed()) {
					chartCanvas.getDisplay().asyncExec(new Runnable() {

						@Override
						public void run() {
							if (chartCanvas != null
									&& !chartCanvas.isDisposed()) {
								chartCanvas.regenerateChart();
							}
						}
					});
				}
			}
		}
	}

	/**
	 * Meldet sich als {@link DatensatzUpdateListener} auf alle MQ's und
	 * Fahrstreifen des Streckenprofils an.
	 */
	protected void anmeldung() {
		// vorher abmelden, zur sicherheit!!!
		abmeldung();

		for (final MessQuerschnitt mq : modell.getAlleMessquerschnitte(true)) {
			mQundFsAnmelden(mq);
		}
		anmeldungAnzeigen();
	}

	/**
	 * Anmeldung fr {@link OdAnzeigeEigenschaftIst} aller Anzeigen, die
	 * innerhalb des Streckenprofils existieren.
	 */
	private void anmeldungAnzeigen() {
		StreckenKnoten knoten = modell.getErsterStreckenKnoten();
		while (knoten != null && knoten.getNachStrecke() != null) {
			RealerStreckenAbschnitt<?> abschnitt = knoten
					.getAktiverStreckenAbschnitt();

			abschnitt = knoten.getNachStrecke().getAktiverStreckenAbschnitt();
			if (abschnitt instanceof AeussererStreckenAbschnitt) {
				for (final AnzeigeQuerschnitt emfAq : abschnitt
						.getAnzeigeQuerschnitt()) {
					final de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.AnzeigeQuerschnitt aquerschnitt = emfAq
							.getModellObjekt();
					if (aquerschnitt != null) {
						final Collection<Anzeige> anzeigen = aquerschnitt
								.getAnzeigen();
						if (anzeigen == null) {
							Debug.getLogger()
									.fine("Anzeigequerschnitt "
											+ aquerschnitt.getPid()
											+ " hat keine Anzeigen in der Menge Anzeigen versorgt.");
						} else {
							for (final Anzeige anz : anzeigen) {
								for (final FahrStreifen f : anz
										.getFahrStreifen()) {
									final KdFahrStreifen.Daten lage = f
											.getKdFahrStreifen().getDatum();
									if (lage != null
											&& AttFahrStreifenLage.ZUSTAND_0_HFS
													.equals(lage.getLage())) {
										final OdAnzeigeEigenschaftIst datenAnzeigeEigenschaftIst = anz
												.getOdAnzeigeEigenschaftIst();

										if (!anzeigeDaten.containsKey(emfAq)) {
											anzeigeDaten
													.put(emfAq,
															new HashMap<Anzeige, OdAnzeigeEigenschaftIst.Daten>());
										}
										anzeigeDaten.get(emfAq).put(anz, null);
										if (chartSimulation != null) {
											final int simuVariante = chartSimulation
													.getKdSimulationsEigenschaften()
													.getDatum()
													.getSimulationsVariante()
													.intValue();
											datenAnzeigeEigenschaftIst
													.addUpdateListener(
															OdAnzeigeEigenschaftIst.Aspekte.AqZustand,
															(short) simuVariante,
															AbstractStreckenProfilComposite.this);

										} else {
											datenAnzeigeEigenschaftIst
													.addUpdateListener(
															OdAnzeigeEigenschaftIst.Aspekte.AqZustand,
															AbstractStreckenProfilComposite.this);
										}
										// es kann pro AQ nur eine HFB
										// geben
										break;
									}
								}
							}
						}
					}
				}
				knoten = knoten.getNachStrecke().getNachKnoten();
			}
		}

	}

	/**
	 * Liefert das {@link StreckenprofilCanvas} in dem das Streckenprofil gemalt
	 * wird.
	 * 
	 * @return the chartCanvas
	 */
	public final StreckenprofilCanvas getChartCanvas() {
		return chartCanvas;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#getSize()
	 */
	@Override
	public Point getSize() {
		Point result = super.getSize();
		if (chartCanvas != null && !chartCanvas.isDisposed()) {
			result = chartCanvas.getSize();
			// result.x += (chartCanvas.getMaxWortbreite() / 1.3);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#setSize(int, int)
	 */
	@Override
	public void setSize(final int width, final int height) {
		// super.setSize(width, height);
		if (chartCanvas != null && !chartCanvas.isDisposed()) {
			chartCanvas.setSize(width, height);
		} else {
			super.setSize(width, height);
		}
	}

	/**
	 * Setzt die Sichtbarkeit aller {@link Axis}.
	 * 
	 * @param value
	 *            <code>true</code>, wenn die Achsen sichtbar sein sollen, sonst
	 *            <code>false</code>
	 */
	public void setAlleAchsenSichtbarkeit(final boolean value) {
		chartBuilder.setAlleAchsenSichtbarkeit(value);
	}
}
