/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor;

import org.eclipse.birt.chart.model.attribute.LineStyle;
import org.eclipse.birt.chart.model.attribute.MarkerType;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.list.IListChangeListener;
import org.eclipse.core.databinding.observable.list.ListChangeEvent;
import org.eclipse.core.databinding.observable.value.ComputedValue;
import org.eclipse.core.databinding.observable.value.IValueChangeListener;
import org.eclipse.core.databinding.observable.value.ValueChangeEvent;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.databinding.EMFObservables;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ViewersObservables;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.nebula.jface.tablecomboviewer.TableComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.bitctrl.lib.eclipse.databinding.observables.ObservablesUtil;
import com.bitctrl.lib.eclipse.help.DefaultHelpContext;
import com.bitctrl.lib.eclipse.resources.WidgetImageRegistry;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienDicke;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.SeriesType;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilFactory;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;
import de.bsvrz.buv.rw.bitctrl.eclipse.widgets.ColorPicker;

/**
 * Erstellt UI-Komponenten fuer die Anzeige einer Tabelle von Linien-Einstellungen wie Farbe, Strichart und -dicke.
 * 
 * @author BitCtrl Systems GmbH, Enrico Schnepel
 * @version $Id: LinePropertiesTable.java,v 1.5 2014/04/17 15:51:08 Pittner Exp $
 */
public final class LinePropertiesTable
{

  /**
   * Konstruktor verstecken.
   */
  private LinePropertiesTable()
  {

  }

  /**
   * {@link LabelProvider} zur Darstellung eines {@link SeriesType} mit deutschem Text und Icons.
   * 
   * @author BitCtrl Systems GmbH, Hoesel
   * @version $Id: LinePropertiesTable.java,v 1.5 2014/04/17 15:51:08 Pittner Exp $
   */
  private static class SeriesTypeLableProvider extends LabelProvider
  {

    /**
     * Das {@link Widget}, das den {@link LabelProvider} verwendet.
     */
    private final Widget widget;

    /**
     * Erzeugt einen neuen {@link LabelProvider} fr Objekte vom Typ {@link SeriesType}.
     * 
     * @param widget
     *          das Widget das den {@link LabelProvider} verwendet.
     */
    public SeriesTypeLableProvider(final Widget widget)
    {
      super();
      this.widget = widget;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
     */
    @Override
    public Image getImage(final Object element)
    {
      ImageDescriptor result = null;
      if (element instanceof SeriesType)
      {
        final SeriesType type = (SeriesType)element;
        switch (type)
        {
          case BAR:
            result = Activator.getDefault().getImageDescriptor(Activator.SERIESTYP_BALKEN_ICON_MARKER_URL);
            break;
          case BAR_STACKED:
            result = Activator.getDefault().getImageDescriptor(Activator.SERIESTYP_BALKEN_GESPAPELT_ICON_MARKER_URL);
            break;
          case LINE:
            result = Activator.getDefault().getImageDescriptor(Activator.SERIESTYP_LINIE_ICON_MARKER_URL);
            break;
          default:
            result = null;
        }
      }
      final Image img = WidgetImageRegistry.getInstance(widget).getImage(result);
      return img;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */
    @Override
    public String getText(final Object element)
    {
      String result = super.getText(element);
      if (element instanceof SeriesType)
      {
        final SeriesType s = (SeriesType)element;
        switch (s)
        {
          case BAR:
            result = "Balken";
            break;
          case BAR_STACKED:
            result = "gestapelte Balken";
            break;
          case LINE:
            result = "Linie";
            break;
          default:
            result = super.getText(element);
        }
      }
      return result;
    }
  }

  /**
   * {@link LabelProvider} for {@link LineStyle} Objekte.
   * 
   * @author BitCtrl Systems GmbH, Hoesel
   * @version $Id: LinePropertiesTable.java,v 1.5 2014/04/17 15:51:08 Pittner Exp $
   */
  private static class LineStyleLableProvider extends LabelProvider
  {

    /**
     * Das {@link Widget}, das den {@link LabelProvider} verwendet.
     */
    private final Widget widget;

    /**
     * Erzeugt einen neuen {@link LabelProvider} fr Objekte vom Typ {@link LineStyle}.
     * 
     * @param widget
     *          das Widget das den {@link LabelProvider} verwendet.
     */
    public LineStyleLableProvider(final Widget widget)
    {
      super();
      this.widget = widget;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
     */
    @Override
    public Image getImage(final Object element)
    {
      ImageDescriptor result = null;
      if (element instanceof LineStyle)
      {
        final LineStyle type = (LineStyle)element;
        switch (type)
        {
          case DASH_DOTTED_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.LINESTYLE_DASH_DOTTED_ICON_MARKER_URL);
            break;
          case DASHED_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.LINESTYLE_DASHED_MARKER_URL);
            break;
          case DOTTED_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.LINESTYLE_DOTTED_ICON_MARKER_URL);
            break;
          case SOLID_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.LINESTYLE_SOLID_ICON_MARKER_URL);
            break;
          default:
            result = null;
        }
      }
      final Image img = WidgetImageRegistry.getInstance(widget).getImage(result);
      return img;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */
    @Override
    public String getText(final Object element)
    {
      String result = null;
      if (element instanceof LineStyle)
      {
        final LineStyle type = (LineStyle)element;
        switch (type)
        {
          case DASH_DOTTED_LITERAL:
            result = "Punkt - Strich";
            break;
          case DASHED_LITERAL:
            result = "Striche";
            break;
          case DOTTED_LITERAL:
            result = "Punkte";
            break;
          case SOLID_LITERAL:
            result = "Linie";
            break;
          default:
            result = super.getText(element);
        }
      }
      return result;
    }

  }

  /**
   * 
   * {@link LabelProvider} fr {@link MarkerType} Objekte.
   * 
   * @author BitCtrl Systems GmbH, Hoesel
   * @version $Id: LinePropertiesTable.java,v 1.5 2014/04/17 15:51:08 Pittner Exp $
   */
  private static class KnotenArtLableProvider extends LabelProvider
  {

    /**
     * Das {@link Widget}, das den {@link LabelProvider} verwendet.
     */
    private final Widget widget;

    /**
     * Erzeugt einen neuen {@link LabelProvider} fr Objekte vom Typ {@link MarkerType}.
     * 
     * @param widget
     *          das Widget das den {@link LabelProvider} verwendet.
     */
    public KnotenArtLableProvider(final Widget widget)
    {
      super();
      this.widget = widget;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object )
     */
    @Override
    public Image getImage(final Object element)
    {
      ImageDescriptor result = null;
      if (element instanceof MarkerType)
      {
        final MarkerType type = (MarkerType)element;
        switch (type)
        {
          case BOX_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_BOX_ICON_MARKER_URL);
            break;
          case CIRCLE_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_CIRCLE_ICON_MARKER_URL);
            break;
          case COLUMN_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYP_COLUMN_ICON_MARKER_URL);
            break;
          case CROSS_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_CROSS_ICON_MARKER_URL);
            break;
          case CROSSHAIR_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_CROSSHAIR_ICON_MARKER_URL);
            break;
          case DIAMOND_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_DIAMOND_ICON_MARKER_URL);
            break;
          case ELLIPSE_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_ELLIPSE_ICON_MARKER_URL);
            break;
          case FOUR_DIAMONDS_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_FOUR_DIAMONDS_ICON_MARKER_URL);
            break;
          case HEXAGON_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_HEXAGON_ICON_MARKER_URL);
            break;
          case ICON_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_ICON_ICON_MARKER_URL);
            break;
          case NABLA_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_NABLA_ICON_MARKER_URL);
            break;
          case RECTANGLE_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_RECTANGLE_ICON_MARKER_URL);
            break;
          case SEMI_CIRCLE_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_SEMI_CIRCLE_ICON_MARKER_URL);
            break;
          case STAR_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_STAR_ICON_MARKER_URL);
            break;
          case TRIANGLE_LITERAL:
            result = Activator.getDefault().getImageDescriptor(Activator.MARKERTYPE_TRIANGLE_ICON_MARKER_URL);
            break;
          default:
            result = null;
        }
      }
      final Image img = WidgetImageRegistry.getInstance(widget).getImage(result);
      return img;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */
    @Override
    public String getText(final Object element)
    {
      String result = super.getText(element);
      if (element instanceof MarkerType)
      {
        final MarkerType s = (MarkerType)element;
        switch (s)
        {
          case BOX_LITERAL:
            result = "Quadrat";
            break;
          case CIRCLE_LITERAL:
            result = "Kreis";
            break;
          case COLUMN_LITERAL:
            result = "Rechteck 2";
            break;
          case CROSS_LITERAL:
            result = "Kreuz";
            break;
          case CROSSHAIR_LITERAL:
            result = "Haarkreuz";
            break;
          case DIAMOND_LITERAL:
            result = "Diamant";
            break;
          case ELLIPSE_LITERAL:
            result = "Ellipse";
            break;
          case FOUR_DIAMONDS_LITERAL:
            result = "Vier Diamanten";
            break;
          case HEXAGON_LITERAL:
            result = "Hexagon";
            break;
          case ICON_LITERAL:
            result = "Ohne";
            break;
          case NABLA_LITERAL:
            result = "Dreieck 2";
            break;
          case RECTANGLE_LITERAL:
            result = "Rechteck 1";
            break;
          case SEMI_CIRCLE_LITERAL:
            result = "Halbkreis";
            break;
          case STAR_LITERAL:
            result = "Stern";
            break;
          case TRIANGLE_LITERAL:
            result = "Dreieck 1";
            break;
          default:
            result = super.getText(element);
        }
      }
      return result;
    }
  }

  /**
   * {@link LabelProvider} fr {@link LinienDicke} Objekte.
   * 
   * @author BitCtrl Systems GmbH, Hoesel
   * @version $Id: LinePropertiesTable.java,v 1.5 2014/04/17 15:51:08 Pittner Exp $
   */
  private static class LineThicknessLabelProvider extends LabelProvider
  {

    /**
     * Das {@link Widget}, das den {@link LabelProvider} verwendet.
     */
    private final Widget widget;

    /**
     * Erzeugt einen neuen {@link LabelProvider} fr Objekte vom Typ {@link LinienDicke}.
     * 
     * @param widget
     *          das Widget das den {@link LabelProvider} verwendet.
     */
    public LineThicknessLabelProvider(final Widget widget)
    {
      super();
      this.widget = widget;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
     */
    @Override
    public Image getImage(final Object element)
    {
      ImageDescriptor result = null;
      if (element instanceof LinienDicke)
      {
        final LinienDicke type = (LinienDicke)element;
        switch (type)
        {
          case NORMAL:
            result = Activator.getDefault().getImageDescriptor(Activator.LINETHICKNESS_NORMAL_ICON_MARKER_URL);
            break;
          case THICK:
            result = Activator.getDefault().getImageDescriptor(Activator.LINETHICKNESS_THICK_MARKER_URL);
            break;
          case THICKER:
            result = Activator.getDefault().getImageDescriptor(Activator.LINETHICKNESS_THICKER_ICON_MARKER_URL);
            break;
          case THIN:
            result = Activator.getDefault().getImageDescriptor(Activator.LINETHICKNESS_THIN_ICON_MARKER_URL);
            break;
          default:
            result = null;
        }
      }
      final Image img = WidgetImageRegistry.getInstance(widget).getImage(result);
      return img;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */
    @Override
    public String getText(final Object element)
    {
      String result = super.getText(element);
      if (element instanceof LinienDicke)
      {
        final LinienDicke s = (LinienDicke)element;
        switch (s)
        {
          case NORMAL:
            result = "Normal";
            break;
          case THICK:
            result = "Dick";
            break;
          case THICKER:
            result = "Dicker";
            break;
          case THIN:
            result = "Dnn";
            break;
          default:
            result = super.getText(element);
        }
      }
      return result;
    }

  }

  /**
   * Erzeugt eine neue {@link Section}, in der die Linieneigenschaften bearbeitet werden knnen.
   * 
   * @param comp
   *          das parent Composite.
   * @param toolkit
   *          das {@link FormToolkit}.
   * @param model
   *          das Streckenprofil
   * @param allProperties
   *          Flag, das anzeigt, ob alle Eigenschaften angezeigt werden sollen.
   * @return eine {@link Section} in der sich die Eigenschaften der {@link LinienEigenschaften} bearbeiten lassen.
   */
  public static Section createDiagrammComponents(final Composite comp, final FormToolkit toolkit, final Modell model,
      final boolean allProperties)
  {
    final Composite formBody = comp;

    final DataBindingContext dbc = new DataBindingContext();
//    
//    /**
//     * Lokaler {@link EContentAdapter}, der auf alle nderungen des Linientyps
//     * der {@link LinienEigenschaften} reagiert.
//     */
//    final EContentAdapter linienEigenschaftenAdapter = new EContentAdapter() {
//      /*
//       * (non-Javadoc)
//       * 
//       * @see org.eclipse.emf.ecore.util.EContentAdapter#notifyChanged
//       * (org.eclipse.emf.common.notify.Notification)
//       */
//      @Override
//      public void notifyChanged(final Notification notification) {
//        super.notifyChanged(notification);
//        final Object feature = notification.getFeature();
//        if ((StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__KNOTEN_ART.equals(feature)) ||
//            (StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__KNOTEN_FARBE.equals(feature))) {
//          System.out
//              .println("{...}.notifyChanged()" + notification.getNewValue() + " -> " + notification.getOldValue());
//        }
//      }
//    };
//    
//    
//    model.getEMFModell().eAdapters().add(linienEigenschaftenAdapter);
//    model.getChartProperties().eAdapters().add(linienEigenschaftenAdapter);
    

    final Section diagramSection = toolkit.createSection(formBody, ExpandableComposite.TITLE_BAR);
    diagramSection.setText("Dateneinstellungen");
    DefaultHelpContext.registerDefaultHelpContext(diagramSection, LinePropertiesTable.class, allProperties ? "gross"
        : "klein");

    final Composite diagramParent = toolkit.createComposite(diagramSection);
    diagramParent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    diagramParent.setLayout(new RowLayout(SWT.HORIZONTAL));

    final TableViewer axesTable = new TableViewer(diagramParent);
    axesTable.getTable().setLayoutData(new RowData(235, 80));
    // axesTable.getTable().setLayoutData(
    // new GridData(SWT.FILL, SWT.FILL, true, false, 1, 2));
    axesTable.setContentProvider(new ArrayContentProvider());
    axesTable.setLabelProvider(new LabelProvider() {

      /*
       * (non-Javadoc)
       * 
       * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
       */
      @Override
      public String getText(final Object element)
      {
        String result = "unbekannt";
        if (element instanceof LinienEigenschaften)
        {
          result = ((LinienEigenschaften)element).getTypDatenQuelle() + " - "
              + ((LinienEigenschaften)element).getTypVerkehrsDaten();
        }
        return result;
      }

    });
    axesTable.setInput(model.getLineProperties());

    EMFObservables.observeList(model.getEMFModell(),
        StreckenprofilPackage.Literals.STRECKENPROFIL_EDITOR_MODEL__LINE_PROPERTIES).addListChangeListener(
        new IListChangeListener() {

          @Override
          public void handleListChange(final ListChangeEvent event)
          {
            axesTable.refresh();

          }
        });

    final Composite diagramSectionClient = toolkit.createComposite(diagramParent);
    // diagramSectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL,
    // true, true));
    diagramSectionClient.setLayout(new GridLayout(1, false));
    createLineComponents(diagramSectionClient, toolkit, dbc, null);
    axesTable.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(final SelectionChangedEvent event)
      {
        final ISelection s = event.getSelection();
        if (s instanceof StructuredSelection)
        {
          final Object firstObject = ((StructuredSelection)s).getFirstElement();
          if (firstObject instanceof LinienEigenschaften)
          {
            for (final Control c : diagramSectionClient.getChildren())
            {
              c.dispose();
            }
            createLineComponents(diagramSectionClient, toolkit, dbc, (LinienEigenschaften)firstObject);
            diagramSectionClient.layout(true);
          }
        }
      }
    });

    // Default Selektion
    if (!model.getLineProperties().isEmpty())
    {
      axesTable.setSelection(new StructuredSelection(model.getLineProperties().get(0)));
    }

    diagramSection.setClient(diagramParent);

    return diagramSection;
  }

  /**
   * Fllt die {@link Section} zur Einstellung der {@link LinienEigenschaften} mit einer spezifischen
   * {@link LinienEigenschaften} und deren Eigenschaften.
   * 
   * @param client
   *          das parent {@link Composite}.
   * @param toolkit
   *          das {@link FormToolkit}-
   * @param dbc
   *          der {@link DataBindingContext}.
   * @param line
   *          die {@link LinienEigenschaften}
   */
  private static void createLineComponents(final Composite client, final FormToolkit toolkit,
      final DataBindingContext dbc, final LinienEigenschaften line)
  {

    final Composite obereZeile = toolkit.createComposite(client);
    obereZeile.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
    obereZeile.setLayout(new GridLayout(7, false));

    final LinienEigenschaften neueLinienEigenschaften;

    if (line == null)
    {
      neueLinienEigenschaften = StreckenprofilFactory.eINSTANCE.createLinienEigenschaften();
    }
    else
    {
      neueLinienEigenschaften = line;
    }

    final GridData centered = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
    final GridData spanned = new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1);

//    toolkit.createLabel(obereZeile, "Sichtbar").setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, true));

    toolkit.createLabel(obereZeile, "Typ").setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
    toolkit.createLabel(obereZeile, "Farbe").setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
    toolkit.createLabel(obereZeile, "Linienart").setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, true, 2, 1));
    toolkit.createLabel(obereZeile, "Liniendicke").setLayoutData(
        new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));

    toolkit.createLabel(obereZeile, "Darstellung").setLayoutData(centered);

//    final Button sichtbarkeitsButton = toolkit.createButton(obereZeile, "", SWT.CHECK);
//    sichtbarkeitsButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
//    dbc.bindValue(SWTObservables.observeSelection(sichtbarkeitsButton), EMFObservables.observeValue(
//        neueLinienEigenschaften, StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__VISIBLE));

    // Chart Series Typ
    final TableComboViewer chartStyle = new TableComboViewer(obereZeile);
    chartStyle.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
    chartStyle.setContentProvider(new ArrayContentProvider());
    chartStyle.setLabelProvider(new SeriesTypeLableProvider(chartStyle.getTableCombo()));
    chartStyle.setInput(SeriesType.values());
    dbc.bindValue(ViewersObservables.observeSingleSelection(chartStyle), EMFObservables.observeValue(
        neueLinienEigenschaften, StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__SERIES_TYPE));

    final ColorPicker colorPicker = new ColorPicker(obereZeile, SWT.FLAT);
    dbc.bindValue(colorPicker.getColor(), EMFObservables.observeValue(neueLinienEigenschaften,
        StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__RGB));

    final TableComboViewer lineStyle = new TableComboViewer(obereZeile);
    lineStyle.getControl().setLayoutData(spanned);
    lineStyle.setContentProvider(new ArrayContentProvider());
    lineStyle.setLabelProvider(new LineStyleLableProvider(lineStyle.getTableCombo()));

    lineStyle.setInput(LineStyle.VALUES.toArray());
    dbc.bindValue(ViewersObservables.observeSingleSelection(lineStyle), EMFObservables.observeValue(
        neueLinienEigenschaften, StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__LINE_STYLE));

    // wenn der Series-Typ nicht LINE ist, dann wird das Widget
    // "Linienart" auf Enabled "false" gesetzt. Klingt komisch, is aber
    // so.
    dbc.bindValue(SWTObservables.observeEnabled(lineStyle.getControl()), new ComputedValue() {
      @Override
      protected Object calculate()
      {
        EMFObservables.observeValue(neueLinienEigenschaften,
            StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__SERIES_TYPE);
        if (SeriesType.LINE.equals(neueLinienEigenschaften.getSeriesType()))
        {
          return true;
        }
        return false;
      }
    });

    final TableComboViewer lineThick = new TableComboViewer(obereZeile);
    lineThick.getControl().setLayoutData(centered);
    lineThick.setContentProvider(new ArrayContentProvider());
    lineThick.setInput(LinienDicke.values());
    lineThick.setLabelProvider(new LineThicknessLabelProvider(lineThick.getTableCombo()));
    dbc.bindValue(ViewersObservables.observeSingleSelection(lineThick), EMFObservables.observeValue(
        neueLinienEigenschaften, StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__LINE_THICKNESS));

    final ShowLineCanvas showLine = new ShowLineCanvas(obereZeile, line);
    final ComputedValue combinedLineProps = new ComputedValue(LinienEigenschaften.class) {
      @Override
      protected LinienEigenschaften calculate()
      {
        neueLinienEigenschaften.getLineStyle();
        neueLinienEigenschaften.getLineThickness();
        neueLinienEigenschaften.getRgb();
        return neueLinienEigenschaften;
      }
    };
    final IValueChangeListener redrawer = new IValueChangeListener() {
      public void handleValueChange(final ValueChangeEvent event)
      {
        showLine.redraw();
        combinedLineProps.getValue();
      }
    };
    ObservablesUtil.addAndDisposeListener(showLine, redrawer, combinedLineProps);
    showLine.setLayoutData(new GridData(60, 17));

    final Composite untereZeile = toolkit.createComposite(client);
    untereZeile.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    untereZeile.setLayout(new GridLayout(3, false));

    toolkit.createLabel(untereZeile, "Knotenart").setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1));
    toolkit.createLabel(untereZeile, "Knotenfllfarbe").setLayoutData(
        new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));

    final TableComboViewer knotenArtViewer = new TableComboViewer(untereZeile);
    knotenArtViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1));
    knotenArtViewer.setContentProvider(new ArrayContentProvider());
    knotenArtViewer.setInput(MarkerType.values());
    knotenArtViewer.setLabelProvider(new KnotenArtLableProvider(knotenArtViewer.getTableCombo()));
    dbc.bindValue(ViewersObservables.observeSingleSelection(knotenArtViewer), EMFObservables.observeValue(
        neueLinienEigenschaften, StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__KNOTEN_ART));

    final ColorPicker colorPicker2 = new ColorPicker(untereZeile, SWT.FLAT);
    dbc.bindValue(colorPicker2.getColor(), EMFObservables.observeValue(neueLinienEigenschaften,
        StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__KNOTEN_FARBE));
  }
}
