/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart.AbstractStreckenprofilBuilder;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart.StreckenprofilChartBuilder;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypDatenQuellen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypPrognoseTyp;
import de.bsvrz.sys.funclib.bitctrl.modell.Aspekt;
import de.bsvrz.sys.funclib.bitctrl.modell.impl.AbstractOnlineDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrStreifenLage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.FahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitIntervall;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationMq;

/**
 * Dieses {@link org.eclipse.swt.widgets.Canvas} wird mit einem {@link Modell}
 * versorgt. Wird fr das Streckenprofil verwendet.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilChartComposite.java 29377 2011-05-17 14:00:51Z
 *          hoesel $
 */
public class StreckenprofilChartComposite extends
		AbstractStreckenProfilComposite {

	/**
	 * Map mit allen {@link MessQuerschnittAllgemein} und deren aktuellen
	 * {@link AbstractOnlineDatensatz}en, fr die wir uns am DAV angemeldet
	 * haben.
	 */
	private final Map<MessQuerschnittAllgemein, AbstractOnlineDatensatz<?>> angemeldeteMQs = new HashMap<MessQuerschnittAllgemein, AbstractOnlineDatensatz<?>>();

	/**
	 * Map mit allen {@link FahrStreifen} und deren
	 * {@link AbstractOnlineDatensatz}en, fr die wir uns angemeldet haben.
	 */
	private final Map<FahrStreifen, AbstractOnlineDatensatz<?>> angemeldeteFSs = new HashMap<FahrStreifen, AbstractOnlineDatensatz<?>>();

	/**
	 * Erzeugt ein neues {@link StreckenprofilChartComposite}.
	 * 
	 * @param parent
	 *            das Eltern Objekt
	 * @param style
	 *            der SWT Style
	 */
	public StreckenprofilChartComposite(final Composite parent, final int style) {
		super(parent, style);

	}

	/**
	 * Meldet MQs und FS fr die im Modell vereinbarten Attributgruppen und
	 * Aspekte an.
	 * 
	 * @param mq
	 *            Der Messquerschnitt, fr dessen Onlinedaten (auch die
	 *            Onlinedaten der Fahrstreifen) sich angemeldet wird.
	 */
	@Override
	protected void mQundFsAnmelden(final MessQuerschnitt mq) {
		super.mQundFsAnmelden(mq);
		final TypPrognoseTyp aktuellerPrognoseTyp = getStreckenEditorModell()
				.getChartProperties().getTypPrognoseTyp();
		final MessQuerschnittAllgemein mqallgemein = mq.getMessQuerschnitt();
		if (ActivationState.ACTIVE.equals(mq.getState())) {
			if (mqallgemein instanceof de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt) {
				for (final FahrStreifen fs : ((de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt) mq
						.getMessQuerschnitt()).getFahrStreifen()) {
					final AttFahrStreifenLage fsLage = fs.getKdFahrStreifen()
							.getDatum().getLage();
					for (final LinienEigenschaften lp : getStreckenEditorModell()
							.getLineProperties()) {
						if ((AttFahrStreifenLage.ZUSTAND_0_HFS.equals(fsLage) && TypDatenQuellen.HFS
								.equals(lp.getTypDatenQuelle()))
								|| (AttFahrStreifenLage.ZUSTAND_1__1UEFS
										.equals(fsLage) && TypDatenQuellen.UEFS1
										.equals(lp.getTypDatenQuelle()))
								|| (AttFahrStreifenLage.ZUSTAND_2__2UEFS
										.equals(fsLage) && TypDatenQuellen.UEFS2
										.equals(lp.getTypDatenQuelle()))
								|| (AttFahrStreifenLage.ZUSTAND_3__3UEFS
										.equals(fsLage) && TypDatenQuellen.UEFS3
										.equals(lp.getTypDatenQuelle()))
								|| (AttFahrStreifenLage.ZUSTAND_4__4UEFS
										.equals(fsLage) && TypDatenQuellen.UEFS4
										.equals(lp.getTypDatenQuelle()))
								|| (AttFahrStreifenLage.ZUSTAND_5__5UEFS
										.equals(fsLage) && TypDatenQuellen.UEFS5
										.equals(lp.getTypDatenQuelle()))
								|| (AttFahrStreifenLage.ZUSTAND_6__6UEFS
										.equals(fsLage) && TypDatenQuellen.UEFS6
										.equals(lp.getTypDatenQuelle()))) {

							AbstractOnlineDatensatz<?> fsDatensatz = null;
							Aspekt fsaspekt = null;
							switch (aktuellerPrognoseTyp) {
							case ANALYSE:
								fsDatensatz = fs.getOdVerkehrsDatenKurzZeitFs();
								fsaspekt = OdVerkehrsDatenKurzZeitFs.Aspekte.Analyse;
								break;
							case FLINK_EXTRA_POLATION:
								fsDatensatz = fs
										.getOdVerkehrsDatenKurzZeitTrendExtraPolationFs();
								fsaspekt = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseFlink;
								break;
							case FLINK_GEGLAETTET:
								fsDatensatz = fs
										.getOdVerkehrsDatenKurzZeitGeglaettetFs();
								fsaspekt = OdVerkehrsDatenKurzZeitGeglaettetFs.Aspekte.PrognoseFlink;
								break;
							case NORMAL_EXTRA_POLATION:
								fsDatensatz = fs
										.getOdVerkehrsDatenKurzZeitTrendExtraPolationFs();
								fsaspekt = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseNormal;
								break;
							case NORMAL_GEGLAETTET:
								fsDatensatz = fs
										.getOdVerkehrsDatenKurzZeitGeglaettetFs();
								fsaspekt = OdVerkehrsDatenKurzZeitGeglaettetFs.Aspekte.PrognoseNormal;
								break;
							case TRAEGE_EXTRA_POLATION:
								fsDatensatz = fs
										.getOdVerkehrsDatenKurzZeitTrendExtraPolationFs();
								fsaspekt = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseTraege;
								break;
							case TRAEGE_GEGLAETTET:
								fsDatensatz = fs
										.getOdVerkehrsDatenKurzZeitGeglaettetFs();
								fsaspekt = OdVerkehrsDatenKurzZeitGeglaettetFs.Aspekte.PrognoseTraege;
								break;
							default:
								throw new IllegalArgumentException(
										"Es wurde ein unbekannter Typ fr das erweiterte Streckenprofil bergeben.");
							}

							if (fsDatensatz != null && fsaspekt != null
									&& !angemeldeteFSs.containsKey(fs)) {
								if (getSimulation() != null) {
									final int simvariante = getSimulation()
											.getKdSimulationsEigenschaften()
											.getDatum()
											.getSimulationsVariante()
											.intValue();
									fsDatensatz.addUpdateListener(fsaspekt,
											(short) simvariante, this);
								} else {
									fsDatensatz.addUpdateListener(fsaspekt,
											this);
								}
								angemeldeteFSs.put(fs, fsDatensatz);
							}

							break;
						}
					}
				}
			}
			AbstractOnlineDatensatz<?> mqDatensatz = null;
			Aspekt mqaspekt = null;
			switch (aktuellerPrognoseTyp) {
			case ANALYSE:
				mqDatensatz = mqallgemein.getOdVerkehrsDatenKurzZeitMq();
				mqaspekt = OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse;
				break;
			case FLINK_EXTRA_POLATION:
				mqDatensatz = mqallgemein
						.getOdVerkehrsDatenKurzZeitTrendExtraPolationMq();
				mqaspekt = OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Aspekte.PrognoseFlink;
				break;
			case FLINK_GEGLAETTET:
				mqDatensatz = mqallgemein
						.getOdVerkehrsDatenKurzZeitGeglaettetMq();
				mqaspekt = OdVerkehrsDatenKurzZeitGeglaettetMq.Aspekte.PrognoseFlink;
				break;
			case NORMAL_EXTRA_POLATION:
				mqDatensatz = mqallgemein
						.getOdVerkehrsDatenKurzZeitTrendExtraPolationMq();
				mqaspekt = OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Aspekte.PrognoseNormal;
				break;
			case NORMAL_GEGLAETTET:
				mqDatensatz = mqallgemein
						.getOdVerkehrsDatenKurzZeitGeglaettetMq();
				mqaspekt = OdVerkehrsDatenKurzZeitGeglaettetMq.Aspekte.PrognoseNormal;
				break;
			case TRAEGE_EXTRA_POLATION:
				mqDatensatz = mqallgemein
						.getOdVerkehrsDatenKurzZeitTrendExtraPolationMq();
				mqaspekt = OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Aspekte.PrognoseTraege;
				break;
			case TRAEGE_GEGLAETTET:
				mqDatensatz = mqallgemein
						.getOdVerkehrsDatenKurzZeitGeglaettetMq();
				mqaspekt = OdVerkehrsDatenKurzZeitGeglaettetMq.Aspekte.PrognoseTraege;
				break;
			default:
				throw new IllegalArgumentException(
						"Es wurde ein unbekannter Typ fr das erweiterte Streckenprofil bergeben.");
			}
			if (mqaspekt != null && mqDatensatz != null
					&& !angemeldeteMQs.containsKey(mq)) {
				if (getSimulation() != null) {
					final int simvariante = getSimulation()
							.getKdSimulationsEigenschaften().getDatum()
							.getSimulationsVariante().intValue();
					mqDatensatz.addUpdateListener(mqaspekt,
							(short) simvariante, this);
				} else {
					mqDatensatz.addUpdateListener(mqaspekt, this);
				}

				angemeldeteMQs.put(mqallgemein, mqDatensatz);
			} else {
				Activator
						.getDefault()
						.getLogger()
						.info(mq.getName()
								+ " ist schon fr Onlinedaten angemeldet.");
			}
		}
	}

	/**
	 * Abmeldung aller MQs und FS.
	 */
	@Override
	protected void abmeldung() {
		super.abmeldung();
		if (getSimulation() != null) {
			final int simvariante = getSimulation()
					.getKdSimulationsEigenschaften().getDatum()
					.getSimulationsVariante().intValue();
			for (final AbstractOnlineDatensatz<?> od : angemeldeteMQs.values()) {
				od.removeUpdateListener(
						OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse,
						(short) simvariante, this);
			}
		}
		for (final AbstractOnlineDatensatz<?> od : angemeldeteMQs.values()) {
			od.removeUpdateListener(OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse,
					StreckenprofilChartComposite.this);
		}
		angemeldeteMQs.clear();

		if (getSimulation() != null) {
			final int simvariante = getSimulation()
					.getKdSimulationsEigenschaften().getDatum()
					.getSimulationsVariante().intValue();
			for (final AbstractOnlineDatensatz<?> od : angemeldeteFSs.values()) {
				od.removeUpdateListener(
						OdVerkehrsDatenKurzZeitIntervall.Aspekte.ExterneErfassung,
						(short) simvariante, this);
			}
		}
		for (final AbstractOnlineDatensatz<?> od : angemeldeteFSs.values()) {
			od.removeUpdateListener(OdVerkehrsDatenKurzZeitFs.Aspekte.Analyse,
					StreckenprofilChartComposite.this);
		}
		angemeldeteFSs.clear();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void dispose() {
		abmeldung();
		super.dispose();
	}

	@Override
	public AbstractStreckenprofilBuilder createStreckenprofilBuilder(
			final Modell model) {
		return new StreckenprofilChartBuilder(model);
	}

}
