/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.preferences;

import java.util.List;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.preferences.SystemObjectEditor.ListLabelProvider;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * {@link PreferencePage} des Plug-Ins Steckenprofil. Es kann:
 * <ul>
 * </ul>
 * 
 * @author Pittner
 * 
 */
public class PreferencePageStreckenProfil extends PreferencePage implements IWorkbenchPreferencePage
{
  /**
   * ID dieser PreferencePage
   */
  public static final String ID = "de.bsvrz.buv.plugin.streckenprofil.sts.a8.pageStreckenprofil";

  private IDaVVerbindungZuhoerer _verbindungZuhoerer;


  private Group _groupAnzeigeTypAAuswahl;
  private Text _textAnzeigeTypAAuswahl;
  private Button _buttonAnzeigeTypAAuswahl;


  private Group _groupAnzeigeTypBAuswahl;
  private Text _textAnzeigeTypBAuswahl;
  private Button _buttonAnzeigeTypBAuswahl;


  private Group _groupAnzeigeTypCAuswahl;
  private Text _textAnzeigeTypCAuswahl;
  private Button _buttonAnzeigeTypCAuswahl;


  /**
   * Konstruktor. Setzt {@link PreferenceStore} und Titel dieser {@link PreferencePage}.
   */
  public PreferencePageStreckenProfil()
  {
    super();
    setPreferenceStore(Activator.getDefault().getPreferenceStore());
    setDescription("Prferenzen des Streckenprofils");
  }

  @Override
  protected Control createContents(Composite parent)
  {
    Composite top = new Composite(parent, SWT.NONE);
    
//	PlatformUI.getWorkbench().getHelpSystem().setHelp(top,
//			Activator.PLUGIN_ID + ".streckenprofilEinstellungen");

	  createGroupAnzeigeTypAAuswahl(top);
	  createGroupAnzeigeTypBAuswahl(top);
	  createGroupAnzeigeTypCAuswahl(top);
 
    actionLoad();

    /*
     * TabList
     */
    Control[] tabList = new Control[] { _groupAnzeigeTypAAuswahl, _groupAnzeigeTypBAuswahl, _groupAnzeigeTypCAuswahl };
    top.setTabList(tabList);

    /*
     * Layout
     */
    GridLayout glTop = new GridLayout(1, false);
    top.setLayout(glTop);

    GridData gdTop = new GridData();
    gdTop.grabExcessHorizontalSpace = true;
    gdTop.horizontalAlignment = GridData.FILL;
    gdTop.grabExcessVerticalSpace = true;
    gdTop.verticalAlignment = GridData.FILL;
    top.setLayoutData(gdTop);

    GridData gdConfArea0 = new GridData();
    gdConfArea0.grabExcessHorizontalSpace = true;
    gdConfArea0.horizontalAlignment = GridData.FILL;
    _groupAnzeigeTypAAuswahl.setLayoutData(gdConfArea0);
    
    GridData gdConfArea1 = new GridData();
    gdConfArea1.grabExcessHorizontalSpace = true;
    gdConfArea1.horizontalAlignment = GridData.FILL;
    _groupAnzeigeTypBAuswahl.setLayoutData(gdConfArea1);
    
    GridData gdConfArea2 = new GridData();
    gdConfArea2.grabExcessHorizontalSpace = true;
    gdConfArea2.horizontalAlignment = GridData.FILL;
    _groupAnzeigeTypCAuswahl.setLayoutData(gdConfArea2);
   
    DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
    if (!daVVerbindung.istDaVVerbindungAktiv())
    {
      _buttonAnzeigeTypAAuswahl.setEnabled(false);
      _buttonAnzeigeTypBAuswahl.setEnabled(false);
      _buttonAnzeigeTypCAuswahl.setEnabled(false);
    }

    _verbindungZuhoerer = new IDaVVerbindungZuhoerer() {

      @Override
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _buttonAnzeigeTypAAuswahl.setEnabled(false);
        _buttonAnzeigeTypBAuswahl.setEnabled(false);
        _buttonAnzeigeTypCAuswahl.setEnabled(false);
      }

      @Override
      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        _buttonAnzeigeTypAAuswahl.setEnabled(true);
        _buttonAnzeigeTypBAuswahl.setEnabled(true);
        _buttonAnzeigeTypCAuswahl.setEnabled(true);
      }

    };
    daVVerbindung.hinzufuegenZuhoerer(_verbindungZuhoerer);

    return null;
  }

  
  private Group createGroupAnzeigeTypAAuswahl(Composite parent)
  {
    if (_groupAnzeigeTypAAuswahl == null)
    {
      _groupAnzeigeTypAAuswahl = new Group(parent, SWT.NONE);
      _groupAnzeigeTypAAuswahl.setText("Auswahl AnzeigeTyp-A:");
      
      _textAnzeigeTypAAuswahl = new Text(_groupAnzeigeTypAAuswahl, SWT.BORDER);
      _buttonAnzeigeTypAAuswahl = new Button(_groupAnzeigeTypAAuswahl, SWT.PUSH);
      _buttonAnzeigeTypAAuswahl.setText("whlen");
      _buttonAnzeigeTypAAuswahl.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectAnzeigeTyp(_textAnzeigeTypAAuswahl);
        }
      });
      
      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textAnzeigeTypAAuswahl };
      _groupAnzeigeTypAAuswahl.setTabList(tabList);
      
      /*
       * Layout
       */
      GridLayout glConfArea = new GridLayout(2, false);
      _groupAnzeigeTypAAuswahl.setLayout(glConfArea);
      
      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textAnzeigeTypAAuswahl.setLayoutData(gdTextConfArea);
      
    }
    
    return _groupAnzeigeTypAAuswahl;
  }
  
  private Group createGroupAnzeigeTypBAuswahl(Composite parent)
  {
    if (_groupAnzeigeTypBAuswahl == null)
    {
      _groupAnzeigeTypBAuswahl = new Group(parent, SWT.NONE);
      _groupAnzeigeTypBAuswahl.setText("Auswahl AnzeigeTyp-B:");
      
      _textAnzeigeTypBAuswahl = new Text(_groupAnzeigeTypBAuswahl, SWT.BORDER);
      _buttonAnzeigeTypBAuswahl = new Button(_groupAnzeigeTypBAuswahl, SWT.PUSH);
      _buttonAnzeigeTypBAuswahl.setText("whlen");
      _buttonAnzeigeTypBAuswahl.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectAnzeigeTyp(_textAnzeigeTypBAuswahl);
        }
      });
      
      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textAnzeigeTypBAuswahl };
      _groupAnzeigeTypBAuswahl.setTabList(tabList);
      
      /*
       * Layout
       */
      GridLayout glConfArea = new GridLayout(2, false);
      _groupAnzeigeTypBAuswahl.setLayout(glConfArea);
      
      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textAnzeigeTypBAuswahl.setLayoutData(gdTextConfArea);
      
    }
    
    return _groupAnzeigeTypBAuswahl;
  }
  private Group createGroupAnzeigeTypCAuswahl(Composite parent)
  {
    if (_groupAnzeigeTypCAuswahl == null)
    {
      _groupAnzeigeTypCAuswahl = new Group(parent, SWT.NONE);
      _groupAnzeigeTypCAuswahl.setText("Auswahl AnzeigeTyp-C:");
      
      _textAnzeigeTypCAuswahl = new Text(_groupAnzeigeTypCAuswahl, SWT.BORDER);
      _buttonAnzeigeTypCAuswahl = new Button(_groupAnzeigeTypCAuswahl, SWT.PUSH);
      _buttonAnzeigeTypCAuswahl.setText("whlen");
      _buttonAnzeigeTypCAuswahl.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectAnzeigeTyp(_textAnzeigeTypCAuswahl);
        }
      });
      
      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textAnzeigeTypCAuswahl };
      _groupAnzeigeTypCAuswahl.setTabList(tabList);
      
      /*
       * Layout
       */
      GridLayout glConfArea = new GridLayout(2, false);
      _groupAnzeigeTypCAuswahl.setLayout(glConfArea);
      
      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textAnzeigeTypCAuswahl.setLayoutData(gdTextConfArea);
      
    }
    
    return _groupAnzeigeTypCAuswahl;
  }
 
  /**
   * Aktion hinter dem Button auswhlen. Es wird ein Dialog zur Auswahl der StreckenBeeinflussungsAnlage dargestellt. 
   */
  private void actionSelectAnzeigeTyp(Text text)
  {
    
    ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
      protected int getTableStyle()
      {
        return SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
      }
      
      @Override
      protected void configureShell(Shell shell) {
        // Fr die F1 Taste
//        PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".streckenprofilEinstellungen");        
        super.configureShell(shell);
      }
    };
    listDialog.setTitle("Auswahl Anzeige-Typ");
    listDialog.setMessage("Folgende Anzeige-Typen stehen zur Auswahl:");
    listDialog.setContentProvider(new ArrayContentProvider());
    listDialog.setLabelProvider(new ListLabelProvider());
    
    SystemObjectType type = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel()
    .getType("typ.anzeigeTyp");
    
    if (type != null)
    {
      List<SystemObject> anlagen = type.getElements();
      if (anlagen != null)
        listDialog.setInput(anlagen);
    }
    
    if (listDialog.open() == ListDialog.OK)
    {
      Object[] results = listDialog.getResult();
      if ((results != null) && (results.length > 0))
      {
        text.setText(((SystemObject)results[0]).getPid());
      }
    }
  }
 
  /**
   * Ldt die Einstellungen aus dem {@link PreferenceStore}.
   */
  private void actionLoad()
  {
    /*
     * Konfigurationsbereich
     */
    String pidConfArea0 = getPreferenceStore().getString(DefsPreference.P_ANZEIGE_TYP_A);
    _textAnzeigeTypAAuswahl.setText(pidConfArea0);
    
    /*
     * Konfigurationsbereich
     */
    String pidConfArea1 = getPreferenceStore().getString(DefsPreference.P_ANZEIGE_TYP_B);
    _textAnzeigeTypBAuswahl.setText(pidConfArea1);
    
    /*
     * Konfigurationsbereich
     */
    String pidConfArea2 = getPreferenceStore().getString(DefsPreference.P_ANZEIGE_TYP_C);
    _textAnzeigeTypCAuswahl.setText(pidConfArea2);
   }

  /**
   * Ldt die Default-Einstellungen aus dem {@link PreferenceStore}.
   */
  private void actionRestoreDefaults()
  {
    /*
     * Konfigurationsbereich
     */
    String pidConfArea0 = getPreferenceStore().getDefaultString(DefsPreference.P_ANZEIGE_TYP_A);
    _textAnzeigeTypAAuswahl.setText(pidConfArea0);
    /*
     * Konfigurationsbereich
     */
    String pidConfArea1 = getPreferenceStore().getDefaultString(DefsPreference.P_ANZEIGE_TYP_B);
    _textAnzeigeTypBAuswahl.setText(pidConfArea1);
    /*
     * Konfigurationsbereich
     */
    String pidConfArea2 = getPreferenceStore().getDefaultString(DefsPreference.P_ANZEIGE_TYP_C);
    _textAnzeigeTypCAuswahl.setText(pidConfArea2);   
  }

  /**
   * Speichert die Einstellungen im {@link PreferenceStore}.
   */
  private boolean actionSave()
  {
    /*
     * Konfigurationsbereich
     */
    String pidConfArea0 = _textAnzeigeTypAAuswahl.getText();
 
    /*
     * Konfigurationsbereich
     */
    String pidConfArea1 = _textAnzeigeTypBAuswahl.getText();
    
    /*
     * Konfigurationsbereich
     */
    String pidConfArea2 = _textAnzeigeTypCAuswahl.getText();
    
    boolean gueltig = true;

    if (!((pidConfArea0 != null) && (pidConfArea0.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea1 != null) && (pidConfArea1.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea2 != null) && (pidConfArea2.length() > 0)))
    {
      gueltig = false;
    }
 
    if (gueltig)
    {
      getPreferenceStore().setValue(DefsPreference.P_ANZEIGE_TYP_A, pidConfArea0);
      getPreferenceStore().setValue(DefsPreference.P_ANZEIGE_TYP_B, pidConfArea1);
      getPreferenceStore().setValue(DefsPreference.P_ANZEIGE_TYP_C, pidConfArea2);
    }
    return gueltig;
  }

  @Override
  protected void performApply()
  {
    actionSave();
  }

  @Override
  public boolean performOk()
  {
    if (actionSave())
      return super.performOk();
    return false;
  }

  @Override
  protected void performDefaults()
  {
    actionRestoreDefaults();
    updateApplyButton();
  }

  @Override
  public boolean performCancel()
  {
    return true;
  }

  @Override
  public void performHelp()
  {
    String helpContextId = Activator.PLUGIN_ID + ".streckenprofilEinstellungen";
//    PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
  }

  @Override
  public void init(IWorkbench workbench)
  {
  }

  @Override
  public void dispose()
  {
    super.dispose();

    /*
     * Deregistrieren des Verbindungszuhrers
     */
    if (_verbindungZuhoerer != null)
    {
      DaVVerbindung.getInstanz().entferneZuhoerer(_verbindungZuhoerer);
    }
  }
}
