package de.bsvrz.buv.plugin.streckenprofil.sts.a8.preferences;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.jface.preference.ListEditor;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * A field editor to edit directory paths.
 */
public class SystemObjectEditor extends ListEditor
{

  /**
   * Creates a new path field editor
   */
  protected SystemObjectEditor()
  {
  }

  /**
   * Creates a path field editor.
   * 
   * @param name
   *          the name of the preference this field editor works on
   * @param labelText
   *          the label text of the field editor
   * @param dirChooserLabelText
   *          the label text displayed for the directory chooser
   * @param parent
   *          the parent of the field editor's control
   */
  public SystemObjectEditor(String name, String labelText, Composite parent)
  {
    init(name, labelText);
    createControl(parent);
  }

  /*
   * (non-Javadoc) Method declared on ListEditor. Creates a single string from the given array by separating each string
   * with the appropriate OS-specific path separator.
   */
  protected String createList(String[] items)
  {
    StringBuffer path = new StringBuffer("");//$NON-NLS-1$

    for (int i = 0; i < items.length; i++)
    {
      path.append(items[i]);
      path.append(File.pathSeparator);
    }
    return path.toString();
  }

  /*
   * (non-Javadoc) Method declared on ListEditor. Creates a new path element by means of a directory dialog.
   */
  protected String getNewInputObject()
  {

    return createDialog();

  }

  /*
   * (non-Javadoc) Method declared on ListEditor.
   */
  protected String[] parseString(String stringList)
  {
    StringTokenizer st = new StringTokenizer(stringList, File.pathSeparator + "\n\r");//$NON-NLS-1$
    ArrayList v = new ArrayList();
    while (st.hasMoreElements())
    {
      v.add(st.nextElement());
    }
    return (String[])v.toArray(new String[v.size()]);
  }

  /**
   * Aktion hinter dem Button auswhlen. Es wird ein Dialog zur Auswahl der StreckenBeeinflussungsAnlage dargestellt.
   */
  private String createDialog()
  {

    ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
      protected int getTableStyle()
      {
        return SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
      }

      @Override
      protected void configureShell(Shell shell)
      {
        // Fr die F1 Taste
        // PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID +
        // ".streckenprofilEinstellungen");
        super.configureShell(shell);
      }
    };
    listDialog.setTitle("Auswahl AnzeigeTyp");
    listDialog.setMessage("Folgende Typen stehen zur Auswahl:");
    listDialog.setContentProvider(new ArrayContentProvider());
    listDialog.setLabelProvider(new ListLabelProvider());

    SystemObjectType type = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel().getType("typ.anzeigeTyp");

    if (type != null)
    {
      List<SystemObject> anlagen = type.getElements();
      if (anlagen != null)
        listDialog.setInput(anlagen);
    }

    if (listDialog.open() == ListDialog.OK)
    {
      Object[] results = listDialog.getResult();
      if ((results != null) && (results.length > 0))
      {
        return ((SystemObject)results[0]).getPid();
      }
    }

    return null;
  }

  /**
   * Innere Klasse, liefert den Namen eines {@link SystemObject}
   * 
   * @author Pittner
   * 
   */
  public static class ListLabelProvider implements ILabelProvider
  {
    private static ListLabelProvider instance;

    public static ListLabelProvider getInstance()
    {
      synchronized (ListLabelProvider.class)
      {
        if (instance == null)
        {
          instance = new ListLabelProvider();
        }
        return instance;
      }
    }

    @Override
    public Image getImage(Object element)
    {
      return null;
    }

    @Override
    public String getText(Object element)
    {
      return ((SystemObject)element).getName();
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }

  @Override
  protected Button getAddButton()
  {
    // TODO Auto-generated method stub
    return super.getAddButton();
  }

}
