/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.wizardpages;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilEinstellungen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.views.StreckenprofilLabelProvider;
import de.bsvrz.buv.rw.bitctrl.eclipse.viewers.EinstellungenContentProvider;

/**
 * {@link WizardPage} um die Einstellungen fr den Export von
 * {@link StreckenprofilEditorModel} festzulegen.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilDateiExportwizardPage.java,v 1.3 2014/04/17 15:51:22 Pittner Exp $
 */
public class StreckenprofilDateiExportwizardPage extends WizardPage {

	private StreckenprofilEditorModel streckenprofil;
	private Text fileName;

	/**
	 * Create the wizard.
	 * 
	 * @wbp.parser.constructor
	 */
	public StreckenprofilDateiExportwizardPage() {
		super("StreckenprofilDateiExportwizardPage");
		setTitle("Streckenprofil exportieren");
		setDescription("Whlen Sie das zu exportierenden Streckenprofil\nund das Zielverzeichnis aus!");
		setImageDescriptor(Activator.getDefault().getImageDescriptor(
				"/icons/export_wiz.png"));
	}

	public StreckenprofilDateiExportwizardPage(
			final StreckenprofilEditorModel streckenprofil) {
		this();
		this.streckenprofil = streckenprofil;
	}

	/**
	 * Create contents of the wizard.
	 * 
	 * @param parent
	 */
	public void createControl(final Composite parent) {
		final Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new GridLayout(2, false));

		final Label lblStreckenprofile = new Label(container, SWT.NONE);
		lblStreckenprofile.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP,
				false, false, 1, 1));
		lblStreckenprofile.setText("Streckenprofile:");

		final TreeViewer treeViewer = new TreeViewer(container, SWT.BORDER);
		treeViewer.setUseHashlookup(true);
		final Tree treeStreckenprofile = treeViewer.getTree();
		treeStreckenprofile.setLayoutData(new GridData(SWT.FILL, SWT.FILL,
				true, true, 1, 1));
		treeViewer.setLabelProvider(new StreckenprofilLabelProvider());
		treeViewer.setContentProvider(new EinstellungenContentProvider());
		treeViewer.setInput(StreckenprofilEinstellungen.getInstance());
		treeViewer.expandAll();
		if (streckenprofil != null) {
			treeViewer.setSelection(new StructuredSelection(streckenprofil));
		}
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection isel = event.getSelection();
				if (isel instanceof StructuredSelection) {
					final Object obj = ((StructuredSelection) isel)
							.getFirstElement();
					if (obj instanceof StreckenprofilEditorModel) {
						streckenprofil = (StreckenprofilEditorModel) obj;

					} else {
						streckenprofil = null;
					}
					updatePageCompletion();
				}

			}
		});

		final Label label = new Label(container, SWT.NONE);
		label.setText("Ziel:");
		GridDataFactory.fillDefaults().applyTo(label);

		fileName = new Text(container, SWT.BORDER);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(fileName);

		final Button fileNameButton = new Button(container, SWT.PUSH);
		fileNameButton.setText("Auswahl ...");
		GridDataFactory.fillDefaults().applyTo(fileNameButton);
		fileNameButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				final FileDialog dialog = new FileDialog(PlatformUI
						.getWorkbench().getActiveWorkbenchWindow().getShell(),
						SWT.SAVE);
				dialog.setFilterExtensions(new String[] { "*.streckenprofil" });
				String name = dialog.open();
				if (name != null) {
					if (!name.endsWith(".streckenprofil")) {
						name = name + ".streckenprofil";
					}
					fileName.setText(name);
					updatePageCompletion();
				}
			}

		});
		updatePageCompletion();
	}

	private void updatePageCompletion() {
		boolean state = false;
		final String destination = getDestination();
		if ((destination != null) && (!destination.isEmpty())
				&& (getStreckenprofil() instanceof StreckenprofilEditorModel)) {
			state = true;
		}
		setPageComplete(state);
	}

	public StreckenprofilEditorModel getStreckenprofil() {
		return streckenprofil;
	}

	public String getDestination() {
		String result = null;
		if ((fileName != null) && (!fileName.isDisposed())) {
			result = fileName.getText();
		}
		return result;
	};

}
