/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit;


import java.math.RoundingMode;
import java.text.NumberFormat;

import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;

/**
 * Spezielle AddOn-Methoden zur Spinnerklasse
 * 
 * @author Krause
 *
 */
public final class SpinnerAddOns
{

  /**
   * Zu dieser Klasse darf keine Instanz angelegt werden.
   */
  private SpinnerAddOns()
  {
  }

  /**
   * Minimum- und Maximum-Werte und Digits eines Spinners mit den Daten aus dem DAV setzen
   * 
   * @param attG
   *          AttributeGroup zum ResultData
   *          (result.getDataDescription().getAttributeGroup())
   * @param groups
   *          Strings der Gruppen, zu dem das Item gehrt, oder null falls keine
   *          Gruppe existiert
   * @param item
   *          String des Items zu dessen Attribut die Werte ermittelt werden
   *          sollen
   * @param spinner
   *          Spinner bei dem die Werte gesetzt werden sollen
   * 
   */
  public static void setSpinnerMinMax( AttributeGroup attG, String[] groups, String item, Spinner spinner )
  {
    //Attributliste zu der Gruppe ermitteln
    IntegerAttributeType iat = null;
    if ( groups != null && groups.length > 0)
    {
      AttributeListDefinition ald = (AttributeListDefinition) attG.getAttribute( groups[0] ).getAttributeType(); 
//      System.out.println("Group: " + groups[0] + " -> ald : " + ald);
      for (int i = 1; i < groups.length; i++)
      {
        ald = (AttributeListDefinition) ald.getAttribute( groups[i] ).getAttributeType();       
//          System.out.println("Group: " + groups[i] + " -> ald[" + i + "] : " + ald);
      }
      iat = (IntegerAttributeType) ald.getAttribute( item ).getAttributeType();
    }
    else
      iat = (IntegerAttributeType) attG.getAttribute( item ).getAttributeType();
 
    setSpinnerMinMax( iat, spinner );
  }

  /**
   * Minimum- und Maximum-Werte und Digits eines Spinners mit den Daten aus dem DAV setzen
   * 
   * @param attG
   *          AttributeGroup zum ResultData
   *          (result.getDataDescription().getAttributeGroup())
   * @param group
   *          String der Gruppe, zu dem das Item gehrt, oder null falls keine
   *          Gruppe existiert
   * @param item
   *          String des Items zu dessen Attribut die Werte ermittelt werden
   *          sollen
   * @param spinner
   *          Spinner bei dem die Werte gesetzt werden sollen
   * 
   */
  public static void setSpinnerMinMax( AttributeGroup attG, String group, String item, Spinner spinner )
  {
    //Attributliste zu der Gruppe ermitteln
    IntegerAttributeType iat = null;
    if ( group != null )
    {
      AttributeListDefinition ald = (AttributeListDefinition) attG.getAttribute( group ).getAttributeType();
      iat = (IntegerAttributeType) ald.getAttribute( item ).getAttributeType();
    }
    else
      iat = (IntegerAttributeType) attG.getAttribute( item ).getAttributeType();
    
    setSpinnerMinMax( iat, spinner );
  }

  /**
   * Minimum-, Maximum-Werte, Digits und Data eines Spinners mit den Daten aus dem DAV setzen
   * 
   * @param iat
   *          IntegerAttributeType aus dem DAV
   * @param spinner
   *          Spinner bei dem die Werte gesetzt werden sollen
   * 
   */
  public static void setSpinnerMinMax( IntegerAttributeType iat, Spinner spinner )
  {
    IntegerValueRange range1 = iat.getRange();

    double rangeFactor = range1.getConversionFactor();
    long rangeMin = range1.getMinimum();
    long rangeMax = range1.getMaximum();

//  System.out.println("RangeFactor: " + rangeFactor + " Log10: " + (-Math.log10( rangeFactor ))
//  + " Exponent: " + Math.getExponent( rangeFactor ) );
    //Digits ermitteln (log10(0.0001) = -4; log10(0.01) = -2)
    Double digits = -Math.log10( rangeFactor );
   
    spinner.setDigits( digits.intValue() );
    spinner.setData( "rangeFactor", rangeFactor );

    //Min- Max-Integer-Werte fr den Spinner setzen
    int iMin = (int) rangeMin;
    int iMax = Integer.MAX_VALUE;
    if ( rangeMax < (long) iMax )
      iMax = (int) rangeMax;

    spinner.setMinimum( iMin );
    spinner.setMaximum( iMax );

    //    System.out.println(item + " ==> Min: " + range1.getMinimum() + "->"+ iMin 
    //        + " Max: " + range1.getMaximum() +"->"+ iMax + " Digits: " + rangeFactor + "->" + spinner.getDigits());

  }

  /**
   * Setzen des Tooltips beim Spinner und dem zugehrigen Label
   * 
   * @param spinner Spinner
   * @param label   Label
   */
  public static void setSpinnerTooltip( Spinner spinner, Label label )
  {
    int digits = spinner.getDigits();
    Object obj = spinner.getData("rangeFactor");
    Double rangeFactor = null;
    if ( obj instanceof Double )
      rangeFactor = (Double) spinner.getData("rangeFactor");      
    else
    {
      rangeFactor = 1 / Math.pow( 10, digits );
      spinner.setData( "rangeFactor", rangeFactor );
    }
    double minSpin = (double) spinner.getMinimum() * rangeFactor;
    double maxSpin = (double) spinner.getMaximum() * rangeFactor;

    NumberFormat f1 = NumberFormat.getInstance();
    f1.setRoundingMode( RoundingMode.HALF_UP );
    f1.setMaximumFractionDigits( digits );
    f1.setMinimumFractionDigits( digits );
    f1.setGroupingUsed( true );

    String tt = "Wertebereich: " + f1.format( minSpin ) + " bis " + f1.format( maxSpin );
    spinner.setToolTipText( tt );
    if ( label != null )
      label.setToolTipText( tt );

    //  System.out.println("Tooltip: Max: " + spinner.getMaximum() + " Digits: " + spinner.getDigits() + " Faktor: " + spinner.getData("rangeFactor"));

  }

}
