/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.autarkprogramm;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen.EinzelmassnahmenViewParameterComposite;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.DaVKommunikation;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegende;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog.UrlasserDialogInfo;


/**
 * View zum Versorgen der Parameter einer Ursacheneinheit vom Typ "typ.sswSbaUeSchaltbildVersorgungAutarkProgramm"
 * @author ObertM
 */
public class ViewSchaltbildVersorgungAutarkProgramm extends GraphicalEditorToView
{
  public static String ID = "de.bsvrz.buv.plugin.ursacheneinheit.autarkprogramm.ViewschaltbildVersorgungAutarkProgramm";

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaAutarkprogramm";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspektEmpfangen = "asp.parameterSoll";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspektVersenden = "asp.parameterVorgabe";

  /**
   * Composites
   */
  private Composite _composite;

  private Composite _composite3;

  private Composite _compositeButton3;

  /**
   * Schaltflaechen
   */
  private Button _buttonSendeAutarkSchaltbilder;
  
  private Button _buttonAlleUbernehmen;

  private Button _buttonUbernehmen;

  private Button _buttonOk;

  private Button _buttonAbbrechen;

  private Button _buttonHilfe;

  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Das Register fr die einzelnen Reiter
   */
  private CTabFolder _tabFolder;

  /**
   * Die Secondary View-ID
   */
  private String _secondViewId;

  /**
   * Das Speicherobjekt zum Wiederherstellen des Zustandes nach dem Start der View
   */
  private IMemento _memento;

  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init(site, memento);
  }

  /**
   * Stellt den Zustand vor dem Beenden das Sicht wieder her
   */
  private void restoreState()
  {
    if (_memento == null)
    {
      return;
    }
    _secondViewId = _memento.getString("pidMemento");
  }

  @Override
  public void saveState(IMemento memento)
  {
    memento.putString("pidMemento", _secondViewId);
    super.saveState(memento);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _composite = new Composite(parent, SWT.NONE);
    _composite.setLayout(new GridLayout());
    _composite.setLayoutData(new GridData());

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_composite,
        de.bsvrz.buv.plugin.ursacheneinheit.Activator.PLUGIN_ID + ".schaltbildVersorgungAutarkProgramm");

    createCompositeTab();
    createCompositeButton();
    restoreState();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    if (_connection != null && _secondViewId != null)
    {
      List<SystemObject> list = holeSystemObjekteAusSecondId();
      if (list.size() > 0)
      {
        erzeugeTabItems(list);
        if (list.size() > 1)
          _buttonAlleUbernehmen.setEnabled(true);
      }
    }
    
    registriereActions();
    
  }

  /**
   * Holt die {@code SystemObject} der Ursacheneinheiten aus der SecondaryId.
   * 
   * @return
   */
  private List<SystemObject> holeSystemObjekteAusSecondId()
  {
    List<SystemObject> list = new ArrayList<SystemObject>();
    if (_secondViewId.contains(";"))
    {
      String[] split = _secondViewId.split(";");
      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject(s);
        if (so != null)
          list.add(so);
      }
    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject(_secondViewId);
      if (so != null)
        list.add(so);
    }
    return list;
  }

  /**
   * This method initializes _tabFolder (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData);
    _tabFolder.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();
        CTabItem itemSel = folder.getSelection();
        SystemObject so = (SystemObject)itemSel.getData(itemSel.getText());

        Composite cSel = (Composite)itemSel.getControl();
        Control[] children = cSel.getChildren();

        CompositeSchaltbildVersorgungAutarkProgramm tabComp = null;

        if (children.length == 0)
        {
          GridData gridData = new GridData();
          gridData.horizontalAlignment = GridData.FILL;
          gridData.grabExcessHorizontalSpace = true;
          gridData.grabExcessVerticalSpace = true;
          gridData.verticalAlignment = GridData.FILL;

          tabComp = new CompositeSchaltbildVersorgungAutarkProgramm(cSel, SWT.NONE, so);
          tabComp.setLayout(new GridLayout());
          tabComp.setLayoutData(gridData);
        }
        else
        {
          tabComp = (CompositeSchaltbildVersorgungAutarkProgramm)cSel.getChildren()[0];
        }

        cSel.layout();
        if (tabComp != null)
        {
          tabComp.layout();
          // Wenn Alle setzen bettigt wurde, muss der Baum neu initialisiert werden.
          tabComp.aktualisiereBaum(so);
        }
      }
    });
  }
  
  /**
   * Registriert die Actions zum Aufruf der Outline-View
   */
  private void registriereActions()
  {
      IMenuManager menuManager = this.getViewSite().getActionBars().getMenuManager();    
      IToolBarManager toolBarManager = this.getViewSite().getActionBars().getToolBarManager();
     
      AnsichtEditPart contents = (AnsichtEditPart)getGraphicalViewer().getContents();
      Ansicht ansicht = contents.getModel();
      Darstellung darstellung = ansicht.getDarstellung();
      EList<Double> stufen = darstellung.getZoomStufen();
      
      final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)getGraphicalViewer().getRootEditPart();
      final ZoomManager zoomManager = rootEditPart.getZoomManager();
      
      double[] dd = new double[stufen.size()];
      
      for (int i = 0; i < dd.length; i++)
      {
        dd[i] = stufen.get(i);      
      }
      
      zoomManager.setZoomLevels(dd);
      
      for (final Double stufe : stufen)
      {
        Action zoomAction = new Action() {
          public void run()
          { 
            CTabItem item = _tabFolder.getSelection();          
            Composite cSel = (Composite)item.getControl();
            CompositeSchaltbildVersorgungAutarkProgramm tabComp = (CompositeSchaltbildVersorgungAutarkProgramm)cSel.getChildren()[0];            
            GraphicalViewer graphicalViewer = tabComp.getGraphicalViewer();
            final ZoomManager zoomManager = ((ScalableRootEditPart)graphicalViewer.getRootEditPart()).getZoomManager();            
            zoomManager.setZoom(stufe );
          }

        };
        zoomAction.setText("Zoomstufe " + stufe);
        menuManager.add(zoomAction);
      }
      
      IAction aktionLegende = holeAction("de.bsvrz.buv.rw.bitctrl.eclipse.legende.oeffneLegende");
      if (aktionLegende != null){
        menuManager.add(aktionLegende);
        toolBarManager.add(aktionLegende);            
      }
      
    }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems(List<SystemObject> list)
  {

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);

      String tabName = Tools.getName(so);
      tabItem.setText(tabName);
      tabItem.setData(tabName, so);

      Composite c1 = new Composite(_tabFolder, SWT.NONE);
      c1.setLayout(new GridLayout());
      c1.setLayoutData(gridData2);

      tabItem.setControl(c1);

    }
    _tabFolder.setSelection(0);

    // da durch setSelection in diesem Fall der SelectionListener nicht aktiv wird, muss er selektierte Tab hier gesetzt
    // werden
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite)itemSel.getControl();
    Control[] children = cSel.getChildren();

    if (children.length == 0)
    {
      String soName = itemSel.getText();
      SystemObject so = (SystemObject)itemSel.getData(soName);
      CompositeSchaltbildVersorgungAutarkProgramm tabComp = new CompositeSchaltbildVersorgungAutarkProgramm(cSel, SWT.NONE, so);
      tabComp.setLayout(new GridLayout());
      tabComp.setLayoutData(gridData2);
    }

  }

  /**
   * This method initializes _composite3
   * 
   */
  private void createCompositeButton()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    _composite3 = new Composite(_composite, SWT.NONE);
    _composite3.setLayout(new GridLayout());
    _composite3.setLayoutData(gridData);

    createCompositeButton3();
  }

  /**
   * This method initializes _compositeButton3
   * 
   */
  private void createCompositeButton3()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton3 = new Composite(_composite3, SWT.NONE);
    _compositeButton3.setLayout(rowLayout);
    _compositeButton3.setLayoutData(gridData);
    
    _buttonSendeAutarkSchaltbilder = new Button(_compositeButton3, SWT.NONE);
    _buttonSendeAutarkSchaltbilder.setText("Sende Autark");
    _buttonSendeAutarkSchaltbilder.setToolTipText("Senden der Autarkschaltbilder fr alle Anzeigen dieser Darstellung");
    _buttonSendeAutarkSchaltbilder.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionSendeAutarkSchaltbilder();
      }
    });
    _buttonAlleUbernehmen = new Button(_compositeButton3, SWT.NONE);
    _buttonAlleUbernehmen.setText("Alle setzen");
    _buttonAlleUbernehmen.setEnabled(false);
    _buttonAlleUbernehmen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAlleUebernehmen();
      }
    });
    _buttonOk = new Button(_compositeButton3, SWT.NONE);
    _buttonOk.setText("Ok");
    _buttonOk.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionSchalten();
      }
    });

    _buttonUbernehmen = new Button(_compositeButton3, SWT.NONE);
    _buttonUbernehmen.setText("bernehmen");
    _buttonUbernehmen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionUebernehmen();
      }
    });

    _buttonAbbrechen = new Button(_compositeButton3, SWT.NONE);
    _buttonAbbrechen.setText("Abbrechen");
    _buttonAbbrechen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }
    });

    _buttonHilfe = new Button(_compositeButton3, SWT.NONE);
    _buttonHilfe.setText("Hilfe");
    _buttonHilfe.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }
    });
  }

  /**
   * Daten an DaV senden, danach schliesen
   */
  protected void aktionSchalten()
  {
    if (schaltenUndParametrieren())
    {
      aktionAbbrechen();
    }
  }

  /**
   * Daten an DaV senden
   */
  protected void aktionUebernehmen()
  {
    schaltenUndParametrieren();
  }
  
  /**
   * Daten an DaV senden
   */
  protected void aktionSendeAutarkSchaltbilder()
  {
    sendeAutarkSchaltbilder();
  }

  /**
   * Fr alle Reiter die Daten des aktiven Reiters an den DaV senden
   */
  protected void aktionAlleUebernehmen()
  {
    if (schaltenUndParametrierenAlle())
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht parametriert werden: " + "\nEventuell keinen AQ ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Methode bernimmt das Versenden der Daten
   * 
   * @return true wenn erfolgreich
   */
  private boolean schaltenUndParametrieren()
  {
    if (_connection != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = _connection.getDataModel().getAspect(_aspektVersenden);
//      Data data = _connection.createData(atg);

      Data data = kopiereDatenAusDarstellung();

      CTabItem item = _tabFolder.getSelection();
      List<ResultData> list = new ArrayList<ResultData>();
      String tabName = item.getText();
      SystemObject so = (SystemObject)item.getData(tabName);
      long l = new Date().getTime();
      ResultData rd = new ResultData(so, new DataDescription(atg, asp), l, data);
      list.add(rd);

      UrlasserDialog urlasserDialog = new UrlasserDialog(Display.getCurrent().getActiveShell());
      UrlasserDialogInfo info = urlasserDialog.anzeigen(list);
      boolean result = info.getEingabeOk();
      
      if (!result) {
        String fehler = info.getFehlerInfo();
        if (fehler == null || fehler.isEmpty()) {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_WARNING | SWT.OK);
          mb.setText("Urlasser");
          mb.setMessage("Es sind nicht alle Parameter an den DaV versendet worden");
          mb.open();
        }
        else {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Urlasser");
          mb.setMessage("Beim Versenden der Parameter an den DaV sind Fehler aufgetreten"/*:\n" + fehler*/);
          mb.open();
        }
      }
      
      return result;
    }
    return false;
  }

  /**
   * Methode bernimmt das Versenden der Daten
   * 
   * @return true wenn erfolgreich
   */
  private boolean schaltenUndParametrierenAlle()
  {
    if (_connection != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = _connection.getDataModel().getAspect(_aspektVersenden);
//      Data data = _connection.createData(atg);

      Data data = kopiereDatenAusDarstellung();

      List<ResultData> list = new ArrayList<ResultData>();
      long l = new Date().getTime();

      List<SystemObject> listSo = holeSystemObjekteAusSecondId();
      for (SystemObject so : listSo)
      {
        ResultData rd = new ResultData(so, new DataDescription(atg, asp), l, data);
        list.add(rd);
      }

      UrlasserDialog urlasserDialog = new UrlasserDialog(Display.getCurrent().getActiveShell());
      UrlasserDialogInfo info = urlasserDialog.anzeigen(list);
      boolean result = info.getEingabeOk();
      
      if (!result) {
        String fehler = info.getFehlerInfo();
        if (fehler == null || fehler.isEmpty()) {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_WARNING | SWT.OK);
          mb.setText("Urlasser");
          mb.setMessage("Es sind nicht alle Parameter an den DaV versendet worden");
          mb.open();
        }
        else {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Urlasser");
          mb.setMessage("Beim Versenden der Parameter an den DaV sind Fehler aufgetreten"/*:\n" + fehler*/);
          mb.open();
        }
      }
      
      return result;
    }
    return false;
  }
  
  private boolean sendeAutarkSchaltbilder() {
    if (_connection != null) {
      Map<String, Data> mapVersorgteAnzeigen = new HashMap<String, Data>();
      Data data = kopiereDatenAusDarstellung();
      Array array = data.getArray("Anzeige");
      for (int i = 0; i < array.getLength(); i++) {
        Data item = array.getItem(i);
        String pid = item.getReferenceValue("Anzeige").getSystemObjectPid();
        mapVersorgteAnzeigen.put(pid, item);
      }
      
      List<SystemObject> listAnzeigen = new ArrayList<SystemObject>();
      
      AnsichtEditPart contents = (AnsichtEditPart)getGraphicalViewer().getContents();
      Ansicht ansicht = contents.getModel();
      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

      EList<Ebene> ebenen = darstellung.getEbenen();
      for (Ebene ebene : ebenen) {
        EList<DoModel> dos = ebene.getDoObjekte();
        for (DoModel dom : dos) {
          if (dom instanceof AnzeigeQuerschnittModelTypAbstract) {
            AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
            EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
            for (AnzeigeModelTypAbstract anzeige : komponenten) {
              SystemObject so = anzeige.getSystemObject();
              if (so.isOfType("typ.anzeige")) {
                listAnzeigen.add(so);
              }
            }
          }
        }
      }
      
      DataModel dm = _connection.getDataModel();
//      for (SystemObject aq: dm.getType("typ.anzeigeQuerschnitt").getElements()) {
//        if (aq.isValid()) {
//          NonMutableSet set = ((ConfigurationObject)aq).getNonMutableSet("Anzeigen");
//          if (set != null) {
//            for (SystemObject anzeige: set.getElements()) {
//              listAnzeigen.add(anzeige);
//            }
//          }  
//        }
//      }
      
      AttributeGroup atg = dm.getAttributeGroup("atg.anzeigeAutarkSoll");
      Aspect asp = dm.getAspect("asp.aqVorgabe");
      long ts = _connection.getTime();
//      long ts = new Date().getTime();
      List<ResultData> list = new ArrayList<ResultData>();
      for (SystemObject anzeige: listAnzeigen) {
        Data item = _connection.createData(atg);
        Data autark = item.getItem("Autark");
        Data itemVersorgt =  mapVersorgteAnzeigen.get(anzeige.getPid());
        if (itemVersorgt != null) {
          Data schaltbild = itemVersorgt.getItem("Schaltbild");
          
          SystemObject anzeigeinhalt = schaltbild.getReferenceValue("AnzeigeInhalt").getSystemObject();
          autark.getReferenceValue("AnzeigeInhalt").setSystemObject(anzeigeinhalt);
          
          String blinken = schaltbild.getTextValue("Blinken").getText();
          autark.getTextValue("Blinken").setText(blinken);
          
          String blinktakt = schaltbild.getTextValue("Blinktakt").getText();
          autark.getTextValue("Blinktakt").setText(blinktakt);
          
          ReferenceArray gd = schaltbild.getReferenceArray("GrafikDarstellungen");
          for (int i = 0; i < gd.getLength(); i++) {
            ReferenceArray grafikDarstellungen = autark.getReferenceArray("GrafikDarstellungen");
            grafikDarstellungen.setLength(i + 1);
            SystemObject so = gd.getReferenceValue(i).getSystemObject();
            grafikDarstellungen.getReferenceValue(i).setSystemObject(so);
          }
          
          SystemObject wechseltext = schaltbild.getReferenceValue("Wechseltext").getSystemObject();
          autark.getReferenceValue("Wechseltext").setSystemObject(wechseltext);
          
          Data dwtSchaltbild = schaltbild.getItem("DynamischerWechseltext");
          Data dwtAutark = autark.getItem("DynamischerWechseltext");
          
          SystemObject font = dwtSchaltbild.getReferenceValue("Font").getSystemObject();
          dwtAutark.getReferenceValue("Font").setSystemObject(font);
          
          String textAusrichtung = dwtSchaltbild.getTextValue("TextAusrichtung").getText();
          dwtAutark.getTextValue("TextAusrichtung").setText(textAusrichtung);
          
          ReferenceArray td = dwtSchaltbild.getReferenceArray("Textdefinition");
          for (int i = 0; i < td.getLength(); i++) {
            ReferenceArray textdefintion = dwtAutark.getReferenceArray("Textdefinition");
            textdefintion.setLength(i + 1);
            SystemObject so = td.getReferenceValue(i).getSystemObject();
            textdefintion.getReferenceValue(i).setSystemObject(so);
          }
        }
        else {
          item.setToDefault();
          autark.getTextValue("Blinken").setText("Nein");
          autark.getTextValue("Blinktakt").setText("Undefiniert");
          autark.getItem("DynamischerWechseltext").getTextValue("TextAusrichtung").setText("Undefiniert");          
        }
                
        ResultData rd = new ResultData(anzeige, new DataDescription(atg, asp), ts, item);
        list.add(rd);
      }
      
      UrlasserDialog urlasserDialog = new UrlasserDialog(Display.getCurrent().getActiveShell(), UrlasserDialog.NEW_URLASSER_1);
      UrlasserDialogInfo info = urlasserDialog.anzeigen(list);
      boolean result = info.getEingabeOk();
      
      if (!result) {
        String fehler = info.getFehlerInfo();
        if (fehler == null || fehler.isEmpty()) {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_WARNING | SWT.OK);
          mb.setText("Urlasser");
          mb.setMessage("Es sind nicht alle Daten an den DaV versendet worden");
          mb.open();
        }
        else {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Urlasser");
          mb.setMessage("Beim Versenden der Daten an den DaV sind Fehler aufgetreten"/*:\n" + fehler*/);
          mb.open();
        }
      }
      
      return result;
    }
    return false;
  }
  
  /**
   * Kopiert die Daten aus der Darstellung in ein Daten-Objekt
   * @return Daten-Objekt
   */
  private Data kopiereDatenAusDarstellung ()
  {
  	/*
  	 * Schaltbilder pro Anzeige
  	 */
    CTabItem item = _tabFolder.getSelection();
    Composite itemComp = (Composite)item.getControl();
    CompositeSchaltbildVersorgungAutarkProgramm comp = (CompositeSchaltbildVersorgungAutarkProgramm)itemComp.getChildren()[0];
    
    Data data = comp.getData();
    
    /*
     * Defaultschaltbid der Anlage
     * als Defaultbild fr die Anlage wird Dunkel eingetragen
     */
    SystemObject wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, AnzeigeInhaltTyp.DUNKEL_BI.getType());

    Data dataAnlage = data.getItem("Schaltbild");

    dataAnlage.getReferenceValue( "AnzeigeInhalt" ).setSystemObject( wvzInhalt );
    dataAnlage.getTextValue( "Blinken" ).setText( "Nein" );
    dataAnlage.getUnscaledValue( "Blinktakt" ).set( 0 );
    
    Array grafikDarstellung = dataAnlage.getArray( "GrafikDarstellungen" );
    grafikDarstellung.setLength( 0 );
    
    dataAnlage.getReferenceValue( "Wechseltext" ).setSystemObject( null );
    
    Data dynamischerWechseltext = dataAnlage.getItem( "DynamischerWechseltext" );
    
    dynamischerWechseltext.getReferenceValue( "Font" ).setSystemObject( null );
    dynamischerWechseltext.getUnscaledValue( "TextAusrichtung" ).set( -1 );
    
    Array textDefinition = dynamischerWechseltext.getArray( "Textdefinition" );
    textDefinition.setLength( 0 );

    return data;
  }

  /**
   * Sicht wird geschlossen
   */
  protected void aktionAbbrechen()
  {
    dispose();
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  protected void aktionHilfe()
  {
    String contextId = de.bsvrz.buv.plugin.anlagendarstellung.Activator.PLUGIN_ID + ".schaltbildVersorgungAutarkProgramm";
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  @Override
  public void dispose()
  {
    getViewSite().getPage().hideView(ViewSchaltbildVersorgungAutarkProgramm.this);
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_composite != null)
      _composite.setFocus();
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.buv.plugin.zustandsdarstellung.editor.GraphicalEditorToView#getGraphicalViewer()
   */
  public GraphicalViewer getGraphicalViewer()
  {
    // Die Anfrage wird an den Viewer des selektierten Reiters weitergeleitet, dieser liefert dann den Viewer
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite)itemSel.getControl();
    CompositeSchaltbildVersorgungAutarkProgramm tabComp = (CompositeSchaltbildVersorgungAutarkProgramm)cSel.getChildren()[0];
    return tabComp.getGraphicalViewer();
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * de.bsvrz.buv.plugin.zustandsdarstellung.editor.GraphicalEditorToView#setGraphicalViewer(org.eclipse.gef.GraphicalViewer
   * )
   */
  public void setGraphicalViewer(GraphicalViewer graphicalViewer)
  {
    // Die Anfrage wird an den Viewer des selektierten Reiters weitergeleitet
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite)itemSel.getControl();
    CompositeSchaltbildVersorgungAutarkProgramm tabComp = (CompositeSchaltbildVersorgungAutarkProgramm)cSel.getChildren()[0];
    tabComp.setGraphicalViewer(graphicalViewer);
  }

  /**
   * Adaptable implementation for {@code GraphicalEditorToView}
   */
  public Object getAdapter(Class adapter)
  {
    // we need to handle common GEF elements we created
    if (adapter == GraphicalViewer.class || adapter == EditPartViewer.class)
      return getGraphicalViewer();
    else if (adapter == CommandStack.class)
      return getCommandStack();
    else if (adapter == EditDomain.class)
      return getEditDomain();
    else if (adapter == ActionRegistry.class)
      return getActionRegistry();
    else if (adapter == ILegende.class)
      return new Legende();
    // else if (adapter == IContentOutlinePage.class)
    // return getOverviewOutlinePage();
    // else if (adapter == ZoomManager.class)
    // return
    // ((ScalableFreeformRootEditPart)getGraphicalViewer().getRootEditPart()).getZoomManager();

    // the super implementation handles the rest
    return super.getAdapter(adapter);
  }

  @Override
  protected void initializeGraphicalViewer()
  {
    // TODO Auto-generated method stub
    /**
     * Hier passiert nichts, da die View keinen GEF-Viewer besitzt.
     */
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    // Die Anfrage wird an den Viewer des selektierten Reiters weitergeleitet, dieser liefert dann den Stack
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite)itemSel.getControl();
    CompositeSchaltbildVersorgungAutarkProgramm tabComp = (CompositeSchaltbildVersorgungAutarkProgramm)cSel.getChildren()[0];
    CommandStack stack = tabComp.getGraphicalViewer().getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }
}
