/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Formatter;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.SortedMap;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVSender;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Sperrung;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DataHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DoModelHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.SbaScalableRootEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm.ListLabelProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm.SswSbaProgrammTyp;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.ursacheneinheit.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.ursacheneinheit.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen.SonderProgramm.EnumSonderProgrammAktion;
import de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen.SonderProgramm.EnumSonderProgrammRichtung;
import de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen.SonderProgramm.EnumSonderProgrammZustand;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.DaVKommunikation;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse realisert die Darstellung und Bearbeitung des Schaltbild-Parameters (Einzelmassnahme) einer
 * Ursacheneinheit. Diese wird in einer Registerkarte der Sicht {@code EinzelmassnahmenViewParameter} verwendet.
 * 
 * @author Pittner
 * 
 */
public class EinzelmassnahmenViewParameterComposite extends Composite implements IDaVDataUpdateListener
{
  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Die geteilte Ansicht
   */
  private SashForm _sf1;

  /**
   * Der Baum
   */
  private Tree _tree;

  /**
   * Composites
   */
  private Composite _composite1;

  private Composite _composite12;

  private Composite _composite2;

  private Composite _composite21;

  private Composite _compositeButton1;

  /**
   * Schaltflaechen
   */
  private Button _buttonHinzufuegen;

  private Button _buttonLoeschen;

  private Button _buttonHoch;

  private Button _buttonRunter;

  private Button _buttonKopieren;

  /**
   * Der GEF-Viewer
   */
  private GraphicalViewer graphicalViewer;

  private MenuManager _kontextMenuMgr;

  private ActionRegistry _actionRegistry;

  private Combo _comboProgrammTyp;

  private ComboViewer _comboViewerProgrammTyp;

  private List<SswSbaProgrammTyp> _listProgrammTypen;

  private Button _buttonOk;

  private Button _buttonResetFrNo;

  private Button _radioButtonAufhebung;

  private Button _radioButtonQuerLaengsabgleich;

  private Text textStartKm;

  private Text textStopKm;

  private Label labelStartKm;

  private Label labelStopKm;

  private SystemObject _so;

//  private boolean angemeldet;
  
  private String _anfrageZeitstempelFrNo;
  private String _anfrageZeitstempelFrSw;

  private Text textStartKm2;

  private Label labelStartKm2;

  private Text textStopKm2;

  private Label labelStopKm2;
  
  private EnumSonderProgrammRichtung _richtung = null;
  

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaEinzelMassnahme";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspekt = "asp.parameterSoll";
  
  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe2 = "atg.sswSbaManahmeVorschlagAntwort";
  
  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspekt2 = "asp.sswSbaAntwort";

  private Section _sectionStreckeFrSw;

  private Section _sectionStreckeFrNo;

  private Button _buttonResetFrSw;

  private Composite _compositeTree;

  private TableViewer _tvFahrtrichtungNord;

  private TableViewer _tvFahrtrichtungSued;
  
  private LinkedList<SystemObject> _listSelectionFahrtrichtungNord = new LinkedList<SystemObject>();
  private LinkedList<SystemObject> _listSelectionFahrtrichtungSued = new LinkedList<SystemObject>();

  private EnumSonderProgrammRichtung _richtungVonUntenNachOben;

  private EnumSonderProgrammRichtung _richtungVonObenNachUnten;
   

  /**
   * Konstruktor
   * 
   * @param parent
   *          das Elternobjekt
   * @param style
   *          der Stil wie das {@code Composite} dargestellt werden soll
   * @param so
   *          die Ursacheneinheit
   */
  public EinzelmassnahmenViewParameterComposite(Composite parent, int style, SystemObject so)
  {
    super(parent, style);
    
    
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    _so = so;
     
    
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    final String strFr1 = store.getString(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_UNTEN_NACH_OBEN);
    final String strFr2 = store.getString(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_OBEN_NACH_UNTEN);
    if (strFr1 != null && strFr1.length() > 0)
    {
      _richtungVonUntenNachOben = EnumSonderProgrammRichtung.valueOf(strFr1);
      _richtungVonObenNachUnten = EnumSonderProgrammRichtung.valueOf(strFr2);
    }
    
    
    createInhaltTab(parent, so);
    anmeldenDaten();
    
    
    
  }



  private void pruefeFahrtRichtung(Darstellung model)
  {
//    EditPart contents = getGraphicalViewer().getContents();
//    if (contents instanceof AnsichtEditPart)
    {
//      AnsichtEditPart aep = (AnsichtEditPart)contents;
//      Ansicht ansicht = aep.getModel();
//      Darstellung model = ansicht.getDarstellung();
      if (model instanceof StilisierteDarstellung)
      {
        StilisierteDarstellung darstellung = (StilisierteDarstellung)model;
        EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
        
        boolean bFrNord = false;
        boolean bFrSued = false;
        for (DarstellungsSpalte spalte : spalten)
        {
          EList<DoModel> doObjekte = spalte.getDoObjekte();
          for (DoModel doModel : doObjekte)
          {
              if (doModel instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)doModel;
                int fahrtrichtung = aq.getFahrtrichtung();
                if (fahrtrichtung == 0){
                  bFrNord = true;                 
                }
                else{
                  bFrSued = true;                  
                }
              }
          }
        }        
        if (_sectionStreckeFrNo != null && _buttonResetFrNo != null){
          _sectionStreckeFrNo.setEnabled(bFrNord); 
          _buttonResetFrNo.setEnabled(bFrNord);          
        }
        if (_sectionStreckeFrSw != null && _buttonResetFrSw != null){
          _sectionStreckeFrSw.setEnabled(bFrSued);        
          _buttonResetFrSw.setEnabled(bFrSued);          
        }
      }      
    }
  }



  private void anmeldenDaten()
  {
    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        DaVKommunikation.getInstanz().anmeldeLeseDaV(object.getPid(), _attributgruppe2, _aspekt2, ReceiverRole.receiver());        
      }
    }
  }
  
  
  
  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
    super.dispose();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
  }

  /**
   * Initialisiert den Inhalt des {@code TabItem}
   * 
   * @param tabItemComposite
   *          das Eltern Objekt
   * @param so
   *          die Ursacheneinheit
   */
  private void createInhaltTab(Composite tabItemComposite, SystemObject so)
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;

    _sf1 = new SashForm(tabItemComposite, SWT.HORIZONTAL | SWT.BORDER);
    _sf1.setLayoutData(gridData1);

    // rechtes und linkes Composite in der Sash-Form
    createComposite12();
    createComposite2();

    _sf1.setWeights(new int[] { 60, 140 });

    
    setzeAlleAnzeigen(AnzeigeInhaltTyp.INAKTIV_BI);
    aktualisiereBaum(so);
  }

  /**
   * This method initializes _composite1
   * 
   */
  private void createComposite12()
  {
//    GridData gridData1 = new GridData();
//    gridData1.horizontalAlignment = GridData.FILL;
//    _composite1 = new Composite(_sf1, SWT.NONE);
//    _composite1.setLayout(new GridLayout());
//    _composite1.setLayoutData(gridData1);

    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    _composite12 = new Composite(_sf1, SWT.NONE);
    _composite12.setLayout(new GridLayout(2, false));
    _composite12.setLayoutData(gridData3);

    createGroupTree();
    createCompositeButton();
//    createCompositeGenerator();
    
    createForm(_composite12);

  }

  /**
   * This method initializes _tree
   */
  private void createGroupTree()
  {
    
 // Sets up the toolkit.
    FormToolkit toolkit = new FormToolkit(_composite12.getDisplay());

    // Creates a form instance.
    // Form form = toolkit.createForm(composite);
    ScrolledForm form = toolkit.createScrolledForm(_composite12);
    form.setLayoutData(new GridData(GridData.FILL_BOTH));

    // Sets title.
    // form.setText("Definition Sonder- und Handprogramm");

    // Adds body contents.
    form.getBody().setLayout(new GridLayout(1, false));
    
    /*
     * Section 
     */
    Section sectionTree = toolkit.createSection(form.getBody(), Section.TITLE_BAR 
        | Section.EXPANDED);
    sectionTree.setText("Alternativen fr Einzelmassnahmen");
    _compositeTree = toolkit.createComposite(sectionTree, SWT.WRAP);
    _compositeTree.setLayout(new GridLayout(2, false));
    sectionTree.setClient(_compositeTree);
    sectionTree.setLayoutData(new GridData(GridData.FILL_BOTH));
    
    
    
//    Group grp = new Group(_composite12, SWT.None);
//    grp.setText("Alternativen fr Einzelmassnahmen");
//
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;
//
//    grp.setLayout(new GridLayout());
//    grp.setLayoutData(gd);

    _tree = new Tree(_compositeTree, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    _tree.setHeaderVisible(false);
    _tree.setLinesVisible(false);
    _tree.setLayoutData(gd);

    _tree.addListener(SWT.Selection, new Listener() {

      @Override
      public void handleEvent(Event event)
      {
        // TODO Auto-generated method stub
        Widget widget = event.item;
        if (widget instanceof TreeItem)
        {
          updateButtons();
          updateKontextMenu();
          
          TreeItem treeitem = (TreeItem)widget;
          
          String text = treeitem.getText();
          AnzeigeInhaltTyp inhalt = null;
          
          // nur wenn auch eine Alternative ausgewhlt ist, knnen die Anzeigen gepixelt werden
          if (text.equals("Aktiv") || text.equals("Inaktiv")){
            inhalt = AnzeigeInhaltTyp.INAKTIV_BI;
          }else{
            inhalt = AnzeigeInhaltTyp.BELIEBIG_BI;            
          }
          setzeAlleAnzeigen(inhalt);
                    
          
          
          Data item = (Data)treeitem.getData();
          System.out.println("Selektion-> Data: " + item);
          
          aktualisiereDarstellung(item);
        }
      }
    });
  }
  
  
  private void createForm(Composite parent)
  {
    // Sets up the toolkit.
    FormToolkit toolkit = new FormToolkit(parent.getDisplay());

    // Creates a form instance.
    // Form form = toolkit.createForm(composite);
    ScrolledForm form = toolkit.createScrolledForm(parent);
    GridData gd = new GridData(GridData.FILL_BOTH);
    gd.horizontalSpan = 2;
    form.setLayoutData(gd);

    // Sets title.
    // form.setText("Definition Sonder- und Handprogramm");

    // Adds body contents.
    form.getBody().setLayout(new GridLayout());
    
    
    /*
     * Section 
     */
    Section sectionGenerieren = toolkit.createSection(form.getBody(), Section.TITLE_BAR 
        | Section.EXPANDED);        
    sectionGenerieren.setText("Generierung Einzelmassnahmen");
    Composite clientGenerieren = toolkit.createComposite(sectionGenerieren, SWT.WRAP);
    clientGenerieren.setLayout(new GridLayout());  
    sectionGenerieren.setClient(clientGenerieren);
    sectionGenerieren.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

    /*
     * Section Programmtyp-/Namensauswahl
     */
    Section sectionProgrammtypNamensauswahl = toolkit.createSection(clientGenerieren, Section.TITLE_BAR
        | Section.EXPANDED);
    sectionProgrammtypNamensauswahl.setText("Programm-Typ");
    Composite clientProgrammtypNamensauswahl = toolkit.createComposite(sectionProgrammtypNamensauswahl, SWT.WRAP);
    clientProgrammtypNamensauswahl.setLayout(new GridLayout());
    sectionProgrammtypNamensauswahl.setClient(clientProgrammtypNamensauswahl);
    sectionProgrammtypNamensauswahl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createCompositeProgrammTypName(clientProgrammtypNamensauswahl);
     
    
    /*
     * Section Streckenbereich von unten nach oben
     */
    _sectionStreckeFrNo = toolkit.createSection(clientGenerieren, Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    _sectionStreckeFrNo.addExpansionListener(new IExpansionListener() {
      
      @Override
      public void expansionStateChanging(ExpansionEvent e)
      {
        if (_tvFahrtrichtungNord.getInput() == null){
          setInputTabelleAq(_richtungVonUntenNachOben, _tvFahrtrichtungNord, false);
        }
      }
      
      @Override
      public void expansionStateChanged(ExpansionEvent e)
      {
        
      }
    });
    _sectionStreckeFrNo.setText("Streckenbereich Fahrtrichtung von unten nach oben");
    Composite clientStrecke = toolkit.createComposite(_sectionStreckeFrNo, SWT.WRAP);
    clientStrecke.setLayout(new GridLayout());
    _sectionStreckeFrNo.setClient(clientStrecke);
    _sectionStreckeFrNo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createCompositeStreckenbereichNeu(clientStrecke);
    
    /*
     * Section Streckenbereich von oben nach unten
     */
    _sectionStreckeFrSw = toolkit.createSection(clientGenerieren, Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    _sectionStreckeFrSw.addExpansionListener(new IExpansionListener() {
      
      @Override
      public void expansionStateChanging(ExpansionEvent e)
      {
        if (_tvFahrtrichtungSued.getInput() == null){
          setInputTabelleAq(_richtungVonUntenNachOben, _tvFahrtrichtungSued, true);
        }
      }
      
      @Override
      public void expansionStateChanged(ExpansionEvent e)
      {
        
      }
    });
    
    _sectionStreckeFrSw.setText("Streckenbereich Fahrtrichtung von oben nach unten");
    Composite clientStrecke2 = toolkit.createComposite(_sectionStreckeFrSw, SWT.WRAP);
    clientStrecke2.setLayout(new GridLayout());
    _sectionStreckeFrSw.setClient(clientStrecke2);
    _sectionStreckeFrSw.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createCompositeStreckenbereich2Neu(clientStrecke2);

    /*
     * Section Ctrl
     */
    Section sectionCtrl = toolkit.createSection(clientGenerieren, Section.TITLE_BAR | Section.EXPANDED);
    sectionCtrl.setText("Aktionen");
    Composite clientCtrl = toolkit.createComposite(sectionCtrl, SWT.WRAP);
    clientCtrl.setLayout(new GridLayout());
    sectionCtrl.setClient(clientCtrl);
    sectionCtrl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createCompositeCtrl(clientCtrl);

    /*
     * Section Zusatzprfung
     */
    Section sectionZusatzpruefung = toolkit.createSection(clientGenerieren, Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    sectionZusatzpruefung.setText("Zusatzprfung");
    Composite clientZusatzpruefung = toolkit.createComposite(sectionZusatzpruefung, SWT.WRAP);
    clientZusatzpruefung.setLayout(new GridLayout());
    sectionZusatzpruefung.setClient(clientZusatzpruefung);
    sectionZusatzpruefung.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createGroupFilterDefinitionenZusatzpruefung(clientZusatzpruefung);
  }
  
  /**
   * This method initializes composite01
   * 
   */
  private void createCompositeProgrammTypName(Composite parent)
  {
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalSpan = 3;
    gridData11.verticalAlignment = GridData.FILL;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    Composite compPrgName = new Composite(parent, SWT.NONE);
    compPrgName.setLayoutData(gridData);
    compPrgName.setLayout(gridLayout);

    Label labelTyp = new Label(compPrgName, SWT.NONE);
    labelTyp.setText("Programm-Typ");
    _comboProgrammTyp = new Combo(compPrgName, SWT.BORDER | SWT.READ_ONLY);
    _comboProgrammTyp.setLayoutData(gridData21);
    _comboProgrammTyp.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {

      }
    });
    _comboProgrammTyp.pack();

    _comboViewerProgrammTyp = new ComboViewer(_comboProgrammTyp);
    _comboViewerProgrammTyp.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection ss = (IStructuredSelection)event.getSelection();
        if (ss != null && !ss.isEmpty())
        {
          Object fe = ss.getFirstElement();
          if (fe != null)
          {
            if (fe instanceof String)
            {
              /*
               * Alter Mechanismus, ungenau.
               */

              String s = (String)fe;

              // Bestimmung Hand und Prioritt aus ProgrammTyp und im folgenden setzen

//              if (s.equals("HandLeer"))
//              {
//                _radioButtonHandschaltungAktiv.setSelection(true);
//                _radioButtonPrioritaet.setSelection(false);
//                _comboPrioritaet.select(0);
//              }
//              else
//              {
//                _radioButtonHandschaltungAktiv.setSelection(false);
//                _radioButtonPrioritaet.setSelection(false);
//                _comboPrioritaet.select(0);
//              }

//              CompositeSperrungDarstellung comp = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[0];
//              if (comp != null && !comp.isDisposed())
//              {
//                boolean b = DaVHelper.isProgrammTypSpurSperrung(s);
//                comp.setVisible(b);
//                if (comp.isVisible())
//                {
//                  comp.initialisiereControls();
//                }
//              }
            }
            else if (fe instanceof SswSbaProgrammTyp)
            {
              /*
               * Genauer Mechanismus.
               */
              SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)fe;

              aktionSelektionComboProgrammTyp(programmTyp);
            }
          }
        }
      }
    });
    _comboViewerProgrammTyp.setLabelProvider(new ILabelProvider() {
      @Override
      public Image getImage(Object element)
      {
        return null;
      }

      @Override
      public String getText(Object element)
      {
        String returnString = "";

        if (element instanceof String)
        {
          returnString = (String)element;
        }
        else if (element instanceof SswSbaProgrammTyp)
        {
          SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)element;
          if (programmTyp.getBezeichnung() != null)
          {
            returnString = programmTyp.getBezeichnung();
          }
        }

        return returnString;

      }

      @Override
      public void addListener(ILabelProviderListener listener)
      {
      }

      @Override
      public void dispose()
      {
      }

      @Override
      public boolean isLabelProperty(Object element, String property)
      {
        return false;
      }

      @Override
      public void removeListener(ILabelProviderListener listener)
      {
      }

    });
    _comboViewerProgrammTyp.setContentProvider(new ArrayContentProvider());

    _listProgrammTypen = ZustandsDarstellungSonderProgramm.bestimmeProgrammTypen(_connection);
    // Combo fuellen
    _comboViewerProgrammTyp.setInput(_listProgrammTypen);

//    labelName = new Label(compPrgName, SWT.NONE);
//    labelName.setText("Name");
//    _textName = new Text(compPrgName, SWT.BORDER);
//    _textName.setLayoutData(gridData1);
  }
  
  /**
   * Fhrt die Aktion hinter der Selektion auf der Combo-Box Programm-Typ durch.
   * 
   * @param programmTyp
   *          Konkreter {@link SswSbaProgrammTyp} oder <code>null</code>, falls dieser aus der aktuellen Selektion der
   *          ComboBox bestimmt werden soll.
   * @return Liefert <code>true</code> wenn erfolgreich, sonst <code>false</code>.
   */
  private boolean aktionSelektionComboProgrammTyp(SswSbaProgrammTyp programmTyp)
  {
    boolean erfolgreich = false;

    if (programmTyp == null)
    {
      if (_comboViewerProgrammTyp != null)
      {
        ISelection selection = _comboViewerProgrammTyp.getSelection();
        if ((selection != null) && (selection instanceof StructuredSelection))
        {
          StructuredSelection ss = (StructuredSelection)selection;
          Object firstElement = ss.getFirstElement();
          if ((firstElement != null) && (firstElement instanceof SswSbaProgrammTyp))
          {
            programmTyp = (SswSbaProgrammTyp)firstElement;
          }
        }
      }
    }

    if (programmTyp != null)
    {
      erfolgreich = true;
//
//      // Radiobutton Handschaltung
//      _radioButtonHandschaltungAktiv.setSelection(programmTyp.isHandprogramm());
//
//      if (programmTyp.getPrioritaet() != null)
//      {
//        // Radiobutton Prioritt
//        if (programmTyp.getPrioritaet() < 0)
//        {
//          _radioButtonPrioritaet.setSelection(false);
//          _comboPrioritaet.select(0);
//          _comboPrioritaet.setEnabled(false);
//        }
//        else
//        {
//          if (programmTyp.getPrioritaet() >= 0)
//          {
//            _radioButtonPrioritaet.setSelection(true);
//            _comboPrioritaet.setText("" + programmTyp.getPrioritaet());
//            _comboPrioritaet.setEnabled(true);
//          }
//        }
//      }
//      else
//      {
//        _radioButtonPrioritaet.setSelection(false);
//        _comboPrioritaet.select(0);
//        _comboPrioritaet.setEnabled(false);
//      }
//
//      CompositeSperrungDarstellung comp = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[0];
//      if (comp != null && !comp.isDisposed())
//      {
//        comp.setVisible(programmTyp.isSpursperrung());
//        if (comp.isVisible())
//        {
//          comp.initialisiereControls();
//        }
//      }
    }

    return erfolgreich;
  }
  
  private void createCompositeStreckenbereich(Composite parent, final EnumSonderProgrammRichtung richtung)
  {
    
    GridData gridData11 = new GridData();
    gridData11.verticalAlignment = GridData.CENTER;
    gridData11.horizontalAlignment = GridData.FILL;
    GridData gridData12 = new GridData();
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.horizontalSpan = 3;
    GridData gridData13 = new GridData();
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    gridData13.horizontalAlignment = GridData.FILL;
    
    GridData gridData15 = new GridData();
    gridData15.grabExcessHorizontalSpace = true;
    gridData15.horizontalAlignment = GridData.FILL;
    GridData gridData16 = new GridData();
    gridData16.grabExcessHorizontalSpace = true;
    gridData16.horizontalAlignment = GridData.FILL;
    
    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.grabExcessVerticalSpace = false;
    gridData18.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout13 = new GridLayout();
    gridLayout13.numColumns = 3;
    gridLayout13.makeColumnsEqualWidth = false;
    
    Composite groupStreckenbereich = new Composite(parent, SWT.NONE);
    
    groupStreckenbereich.setLayoutData(gridData18);
    groupStreckenbereich.setLayout(gridLayout13);
    Label label21 = new Label(groupStreckenbereich, SWT.NONE);
    label21.setLayoutData(gridData11);
    label21.setText("startKm");
    textStartKm = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
//    textStartKm.setLayoutData(gridData15);
    textStartKm.setTextLimit(7);
    textStartKm.setText("000,000");
    textStartKm.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            validiereTextInput(richtung.toString(), textStartKm.getText(), textStopKm.getText());
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });
    
    Button btnBereichVon = new Button(groupStreckenbereich, SWT.NONE);
    btnBereichVon.setText("ber AQ");
    btnBereichVon.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStartKm(richtung.toString());
      }
    });
    
    labelStartKm = new Label(groupStreckenbereich, SWT.NONE);
    labelStartKm.setLayoutData(gridData12);
    labelStartKm.setText("keine Ausw.");
    
    Label label22 = new Label(groupStreckenbereich, SWT.NONE);
    label22.setLayoutData(gridData11);
    label22.setText("endeKm ");
    
    textStopKm = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
//    textStopKm.setLayoutData(gridData16);
    textStopKm.setTextLimit(7);
    textStopKm.setText("000,000");
    textStopKm.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            validiereTextInput(richtung.toString(), textStartKm.getText(), textStopKm.getText());
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });
    
    Button btnBereichBis = new Button(groupStreckenbereich, SWT.NONE);
    btnBereichBis.setText("ber AQ");
    btnBereichBis.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStopKm(richtung.toString());
      }
    });
    
    labelStopKm = new Label(groupStreckenbereich, SWT.NONE);
    labelStopKm.setLayoutData(gridData12);
    labelStopKm.setText("keine Ausw.");
  }
  
  private void createCompositeStreckenbereichNeu(Composite parent)
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 3;
    gridLayout1.makeColumnsEqualWidth = false;
    
    Composite compStreckenbereich = new Composite(parent, SWT.NONE);
    
    compStreckenbereich.setLayoutData(gridData1);
    compStreckenbereich.setLayout(gridLayout1);
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.FILL;
    gd2.grabExcessHorizontalSpace = false;
    gd2.heightHint = 120;
    gd2.grabExcessVerticalSpace = false;

    _tvFahrtrichtungNord = new TableViewer(compStreckenbereich, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);     
    _tvFahrtrichtungNord.addSelectionChangedListener(new ISelectionChangedListener() {
      
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection sel = event.getSelection();
        if (sel instanceof StructuredSelection)
        {
          StructuredSelection ss = (StructuredSelection)sel;
          Iterator it = ss.iterator();
          _listSelectionFahrtrichtungNord.clear();
          while(it.hasNext()){
            Object next = it.next();
            if (next instanceof SystemObject)
            {
              SystemObject so = (SystemObject)next;             
              if (!_listSelectionFahrtrichtungNord.contains(so))
              _listSelectionFahrtrichtungNord.add(so);
            }
          }
        }
      }
    });
    
    
    _tvFahrtrichtungNord.getTable().addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        boolean b = (e.stateMask & SWT.CTRL) != 0 ? true : false;
        if (b){
          MessageBox mb = new MessageBox(getShell(), SWT.ICON_INFORMATION | SWT.OK);
          mb.setText("Aq-Auswahl");
          mb.setMessage("Nur Intervall-Selektion (kein CTRL) erlaubt.");
          mb.open();
          _tvFahrtrichtungNord.setSelection(new StructuredSelection(_listSelectionFahrtrichtungNord));
          return;          
        }
      }
    });
    
    TableViewerColumn column1 = new TableViewerColumn(_tvFahrtrichtungNord, SWT.NONE);
    column1.getColumn().setWidth(250);
    column1.getColumn().setText("AnzeigeQuerschnitte");
    
    _tvFahrtrichtungNord.setContentProvider(new ArrayContentProvider());
    _tvFahrtrichtungNord.setLabelProvider(new ListLabelProvider());
    
    _tvFahrtrichtungNord.getTable().setLayoutData(gd2);
    
    

  }
  
  private void createCompositeStreckenbereich2Neu(Composite parent)
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 3;
    gridLayout1.makeColumnsEqualWidth = false;
    
    Composite compStreckenbereich = new Composite(parent, SWT.NONE);
    
    compStreckenbereich.setLayoutData(gridData1);
    compStreckenbereich.setLayout(gridLayout1);
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.FILL;
    gd2.grabExcessHorizontalSpace = false;
    gd2.heightHint = 120;
    gd2.grabExcessVerticalSpace = false;
    
    _tvFahrtrichtungSued = new TableViewer(compStreckenbereich, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);        
    _tvFahrtrichtungSued.addSelectionChangedListener(new ISelectionChangedListener() {
      
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection sel = event.getSelection();
        if (sel instanceof StructuredSelection)
        {
          StructuredSelection ss = (StructuredSelection)sel;
          Iterator it = ss.iterator();
          _listSelectionFahrtrichtungSued.clear();
          while(it.hasNext()){
            Object next = it.next();
            if (next instanceof SystemObject)
            {
              SystemObject so = (SystemObject)next;             
              if (!_listSelectionFahrtrichtungSued.contains(so))
                _listSelectionFahrtrichtungSued.add(so);
            }
          }
        }
      }
    });
    
    
    _tvFahrtrichtungSued.getTable().addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        boolean b = (e.stateMask & SWT.CTRL) != 0 ? true : false;
        if (b){
          MessageBox mb = new MessageBox(getShell(), SWT.ICON_INFORMATION | SWT.OK);
          mb.setText("Aq-Auswahl");
          mb.setMessage("Nur Intervall-Selektion (kein CTRL) erlaubt.");
          mb.open();
          _tvFahrtrichtungSued.setSelection(new StructuredSelection(_listSelectionFahrtrichtungSued));
          return;          
        }
      }
    });
    
    
    TableViewerColumn column1 = new TableViewerColumn(_tvFahrtrichtungSued, SWT.NONE);
    column1.getColumn().setWidth(250);
    column1.getColumn().setText("AnzeigeQuerschnitte");
    
    _tvFahrtrichtungSued.setContentProvider(new ArrayContentProvider());
    _tvFahrtrichtungSued.setLabelProvider(new ListLabelProvider());
    
    _tvFahrtrichtungSued.getTable().setLayoutData(gd2);
  }



  /**
   * @param richtung
   * @param viewer
   */
  private void setInputTabelleAq(EnumSonderProgrammRichtung richtung, TableViewer viewer, boolean gedreht)
  {
    int iFr = 0;
    if (gedreht){
      iFr = 1;
    }      
    
    boolean absteigend = ermittleKilometrierungAufsteigendAbsteigend(richtung, gedreht);
    SortedMap<Integer, SystemObject> map = DaVHelper.holeAlleAnzeigeQuerschnittModelTypAbstractVonFahrtRichtung(getGraphicalViewer(), iFr, absteigend);    
    viewer.setInput(map.values());
//    viewer.getTable().pack();
  }



  private boolean ermittleKilometrierungAufsteigendAbsteigend(EnumSonderProgrammRichtung richtung, boolean gedreht)
  {
    
    boolean absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(richtung.toString());
    if (richtung.equals(EnumSonderProgrammRichtung.NO) && gedreht || richtung.equals(EnumSonderProgrammRichtung.SW) && !gedreht){
//      String strRichtung = richtung.toString();
//      if (strRichtung.equals(EnumSonderProgrammRichtung.SW))
//        strRichtung = EnumSonderProgrammRichtung.NO.toString();
//      else if (strRichtung.equals(EnumSonderProgrammRichtung.NO)){
//        strRichtung = EnumSonderProgrammRichtung.SW.toString();        
//      }else{
//        
//      }
//      absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(strRichtung);
      absteigend = !absteigend;
    }
    return absteigend;
  }
  
  private void createCompositeStreckenbereich2(Composite parent, final  EnumSonderProgrammRichtung richtung)
  {

    GridData gridData11 = new GridData();
    gridData11.verticalAlignment = GridData.CENTER;
    gridData11.horizontalAlignment = GridData.FILL;
    GridData gridData12 = new GridData();
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.horizontalSpan = 3;
    GridData gridData13 = new GridData();
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    gridData13.horizontalAlignment = GridData.FILL;

    GridData gridData15 = new GridData();
    gridData15.grabExcessHorizontalSpace = true;
    gridData15.horizontalAlignment = GridData.FILL;
    GridData gridData16 = new GridData();
    gridData16.grabExcessHorizontalSpace = true;
    gridData16.horizontalAlignment = GridData.FILL;

    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.grabExcessVerticalSpace = false;
    gridData18.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout13 = new GridLayout();
    gridLayout13.numColumns = 3;
    gridLayout13.makeColumnsEqualWidth = false;

    Composite groupStreckenbereich = new Composite(parent, SWT.NONE);

    groupStreckenbereich.setLayoutData(gridData18);
    groupStreckenbereich.setLayout(gridLayout13);
    Label label21 = new Label(groupStreckenbereich, SWT.NONE);
    label21.setLayoutData(gridData11);
    label21.setText("startKm");
    textStartKm2 = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
//    textStartKm2.setLayoutData(gridData15);
    textStartKm2.setTextLimit(7);
    textStartKm2.setText("000,000");
    textStartKm2.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            validiereTextInput(richtung.toString(), textStartKm2.getText(), textStopKm2.getText());
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });

    Button btnBereichVon = new Button(groupStreckenbereich, SWT.NONE);
    btnBereichVon.setText("ber AQ");
    btnBereichVon.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStartKm2(richtung.toString());
      }
    });

    labelStartKm2 = new Label(groupStreckenbereich, SWT.NONE);
    labelStartKm2.setLayoutData(gridData12);
    labelStartKm2.setText("keine Ausw.");

    Label label22 = new Label(groupStreckenbereich, SWT.NONE);
    label22.setLayoutData(gridData11);
    label22.setText("endeKm ");

    textStopKm2 = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
//    textStopKm2.setLayoutData(gridData16);
    textStopKm2.setTextLimit(7);
    textStopKm2.setText("000,000");
    textStopKm2.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            validiereTextInput(richtung.toString(), textStartKm2.getText(), textStopKm2.getText());
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });

    Button btnBereichBis = new Button(groupStreckenbereich, SWT.NONE);
    btnBereichBis.setText("ber AQ");
    btnBereichBis.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStopKm2(richtung.toString());
      }
    });

    labelStopKm2 = new Label(groupStreckenbereich, SWT.NONE);
    labelStopKm2.setLayoutData(gridData12);
    labelStopKm2.setText("keine Ausw.");
  }
  
  private void aktionAuswahlAqIntervall(String fr)
  {
    boolean absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(fr);
    
    int iFr = 0;
    if (fr.equals("SW"))
    {
      iFr = 1;
    }
    SortedMap<Integer, SystemObject> map = DaVHelper.holeAlleAnzeigeQuerschnittModelTypAbstractVonFahrtRichtung(getGraphicalViewer(), iFr, absteigend);
    
    ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
      protected int getTableStyle()
      {
        return SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER  | SWT.MULTI;
      }

      @Override
      protected void configureShell(Shell shell)
      {
        // Fr die F1 Taste
        PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".auswahlAnzeigequerschnitt");
        super.configureShell(shell);
      }
    };
    listDialog.setTitle("Auswahl AQ");
    listDialog.setMessage("Folgende AQ stehen zur Auswahl:");
    listDialog.setContentProvider(new ArrayContentProvider());
    listDialog.setLabelProvider(new ListLabelProvider());     
    
    listDialog.setInput(map.values());
    
  }
  
  
  private void aktionStartKm(String fr)
  {
//    SystemObject so = null;
    SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, fr);
    if (_connection != null && so != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
      Data cd = so.getConfigurationData(atg);
      double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
      if (dv > 0.0)
        dv /= 1000.0;

      if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
      {
        dv = 208.43;
      }

      String s = new Formatter().format("%1$7.3f", dv).toString();
      
      if (validiereTextInput(fr, s, textStopKm.getText()))
      {
        textStartKm.setText(s);
        labelStartKm.setText(so.getName());
        labelStartKm.setToolTipText(so.getName());  
        _richtung = EnumSonderProgrammRichtung.valueOf(fr);
        
      }
    }
  }
  
  private void aktionStartKm2(String fr)
  {
//    SystemObject so = null;
    SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, fr);
    if (_connection != null && so != null)
    {
      
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
      Data cd = so.getConfigurationData(atg);
      double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
      if (dv > 0.0)
        dv /= 1000.0;
      
      if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
      {
        dv = 208.43;
      }
      
      String s = new Formatter().format("%1$7.3f", dv).toString();
      
      
      if (validiereTextInput(fr, s, textStopKm2.getText()))
      {
        textStartKm2.setText(s);
        labelStartKm2.setText(so.getName());
        labelStartKm2.setToolTipText(so.getName());
        _richtung = EnumSonderProgrammRichtung.valueOf(fr);
      }
    }
  }

  private void aktionStopKm(String fr)
  {
//    SystemObject so = null;
    SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, fr);
    if (_connection != null && so != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
      Data cd = so.getConfigurationData(atg);
      double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
      if (dv > 0.0)
        dv /= 1000.0;

      if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
      {
        dv = 208.43;
      }

      String s = new Formatter().format("%1$7.3f", dv).toString();

      if (validiereTextInput(fr, textStartKm.getText(), s))
      {
        textStopKm.setText(s);
        labelStopKm.setText(so.getName());
        labelStopKm.setToolTipText(so.getName());
      }
    }
  }
  
  private void aktionStopKm2(String fr)
  {
//    SystemObject so = null;
    SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, fr);
    if (_connection != null && so != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
      Data cd = so.getConfigurationData(atg);
      double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
      if (dv > 0.0)
        dv /= 1000.0;
      
      if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
      {
        dv = 208.43;
      }
      
      String s = new Formatter().format("%1$7.3f", dv).toString();
      
      if (validiereTextInput(fr, textStartKm2.getText(), s))
      {
        textStopKm2.setText(s);
        labelStopKm2.setText(so.getName());
        labelStopKm2.setToolTipText(so.getName());
      }
    }
  }
  
  /**
   * This method initializes composite10
   * 
   */
  private void createCompositeCtrl(Composite parent)
  {
    GridData gdButtonReset = new GridData();
    gdButtonReset.horizontalAlignment = GridData.FILL;
    gdButtonReset.verticalAlignment = GridData.CENTER;
    GridData gdButtonReset2 = new GridData();
    gdButtonReset2.horizontalAlignment = GridData.FILL;
    gdButtonReset2.verticalAlignment = GridData.CENTER;
    GridData gdButtonOk = new GridData();
    gdButtonOk.horizontalAlignment = GridData.FILL;
    gdButtonOk.verticalAlignment = GridData.CENTER;
    
    Composite groupButtonCtrl = new Composite(parent, SWT.NONE);
    groupButtonCtrl.setLayout(new GridLayout(1, true));
    groupButtonCtrl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

    _buttonOk = new Button(groupButtonCtrl, SWT.NONE);
    _buttonOk.setText("OK");
    _buttonOk.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        SonderProgramm sonderProgramm = ueberpruefeErfassteDaten();
        if (sonderProgramm != null)
        {
          long zeitstempel = System.currentTimeMillis();
          
          
          boolean versendet = false;
          
          if (_sectionStreckeFrNo.isExpanded()){
            if (_listSelectionFahrtrichtungNord.size() > 0){
              SystemObject first = _listSelectionFahrtrichtungNord.getFirst();
              SystemObject sbaAq = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, first);
              String s = DaVHelper.holeRichtungVonSbaAq(_connection, sbaAq);
              sonderProgramm.setGewaehlteSonderProgrammRichtung(EnumSonderProgrammRichtung.valueOf(s));  
              
            }
            versendeDaten(_connection, sonderProgramm, _richtungVonUntenNachOben, false, zeitstempel);
            versendet = true;
          }
          if (_sectionStreckeFrSw.isExpanded()){
            if (_listSelectionFahrtrichtungSued.size() > 0){
              SystemObject first = _listSelectionFahrtrichtungSued.getFirst();
              SystemObject sbaAq = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, first);
              String s = DaVHelper.holeRichtungVonSbaAq(_connection, sbaAq);
              sonderProgramm.setGewaehlteSonderProgrammRichtung(EnumSonderProgrammRichtung.valueOf(s));                                       
            }
            if (versendet)
              zeitstempel+=1;
            versendeDaten(_connection, sonderProgramm, _richtungVonObenNachUnten, true, zeitstempel);
          }
        }
      }
    });
    _buttonOk.setLayoutData(gdButtonOk);

    _buttonResetFrNo = new Button(groupButtonCtrl, SWT.NONE);
    _buttonResetFrNo.setText("Reset FR von u. nach o.");
    _buttonResetFrNo.setLayoutData(gdButtonReset);
    _buttonResetFrNo.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {

        MessageBox mb = new MessageBox(Display.getCurrent().getActiveShell(), SWT.ICON_QUESTION | SWT.YES
            | SWT.NO);
        mb.setText("Reset-Schaltbild");
        mb.setMessage("Soll das Schaltbild zurckgesetzt werden\n (alte Daten gehen somit verloren)?");
        if (mb.open() == SWT.YES)
        {
          setzeAlleAnzeigen(AnzeigeInhaltTyp.BELIEBIG_BI, EnumSonderProgrammRichtung.NO, true); 
          
          TreeItem[] tis = _tree.getSelection();
          if (tis != null && tis.length > 0){                  
            Data ziel = ((Data)tis[0].getData()).getItem("AnzeigenDatenstze");                        
            Array arrayZiel = ziel.asArray();                        
            arrayZiel.setLength(0);
          }
        }
      }
    });
    
    _buttonResetFrSw = new Button(groupButtonCtrl, SWT.NONE);
    _buttonResetFrSw.setText("Reset FR von o. nach u.");
    _buttonResetFrSw.setLayoutData(gdButtonReset2);
    _buttonResetFrSw.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        MessageBox mb = new MessageBox(Display.getCurrent().getActiveShell(), SWT.ICON_QUESTION | SWT.YES
            | SWT.NO);
        mb.setText("Reset-Schaltbild");
        mb.setMessage("Soll das Schaltbild zurckgesetzt werden\n (alte Daten gehen somit verloren)?");
        if (mb.open() == SWT.YES)
        {          
          setzeAlleAnzeigen(AnzeigeInhaltTyp.BELIEBIG_BI, EnumSonderProgrammRichtung.SW, true);                     
        }
      }
    });
  }
  
  public boolean versendeDaten(ClientDavInterface connection, SonderProgramm sonderProgramm, EnumSonderProgrammRichtung richtung, boolean gedreht, long zeitstempel)
  {
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaManahmeVorschlagAnforderung");
    Aspect asp = connection.getDataModel().getAspect("asp.sswSbaAnforderung");
    Data data = connection.createData(atg);
    
    schreibeData(connection, sonderProgramm, false, data, zeitstempel, sonderProgramm.getGewaehlteSonderProgrammRichtung(), gedreht);

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = connection.getDataModel().getObject(pidSba);
      if (object != null)
      {          
        new DaVSender(_connection, SenderRole.sender()).sende(object, atg, asp, data, (short)0);
                  
        
//        boolean absteigend = ermittleKilometrierungAufsteigendAbsteigend(richtung, gedreht);
        
        
        if (!gedreht)
          _anfrageZeitstempelFrNo = ""  + zeitstempel;
        else
          _anfrageZeitstempelFrSw = ""  + zeitstempel;
      }
    }
    return true;

  }
 
 
  /**
   * @return
   */
  private SonderProgramm ueberpruefeErfassteDaten()
  {
    boolean bBezeichnung = false;
    boolean bDefinition = false;
    boolean bAktion = false;
    boolean bZustand = true;
    boolean bRichtung = false;
    boolean bKilometer = false;
    boolean bKilometer2 = false;
    boolean bAnzeigeQuerschnitt = false;
    boolean bHandschaltung = false;
    boolean bPrioHandschaltung = false;
    boolean bQuerLaengsabgleich = false;
    boolean bZusatzpruefung = false;
    boolean bSchaltOption = false;
    boolean bBetriebsArt = false;
    boolean bVorgabenSchaltungAq = false;
    boolean bZeitBereichVon = false;
    boolean bZeitBereichBis = false;
    boolean bAuswahlStunden = false;
    boolean bAuswahlTage = false;
    boolean bAuswahlMonate = false;

    SonderProgramm sonderProgramm = new SonderProgramm();
    
    if (_so != null)
      sonderProgramm.setBezeichnung(_so.getNameOrPidOrId());
 

    /*
     * Definition des Grundprogramms
     */
    ISelection selection = _comboViewerProgrammTyp.getSelection();
    if (selection != null)
    {
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;
        Object element = ss.getFirstElement();
        if (element != null)
        {
          if (element instanceof String)
          {
            String s = (String)element;
            if (s != null)
            {
              sonderProgramm.setTypBezeichnung(s);
              bDefinition = true;
            }
          }
          else if (element instanceof SswSbaProgrammTyp)
          {
            SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)element;
            String bezeichnung = programmTyp.getBezeichnung();
            if (bezeichnung != null)
            {
              sonderProgramm.setTypBezeichnung(bezeichnung);
              bDefinition = true;
            }
          }

        }
      }
    }

    /*
     * Aktion SonderProgramm
     */
    {
      if (sonderProgramm.getGewaehlteSonderProgrammAktion() == null)
      {
        bAktion = false;
      }
    }

    /*
     * Zustand SonderProgramm
     */
    {
      if (sonderProgramm.getGewaehlterSonderProgrammZustand() == null)
      {
        sonderProgramm.setGewaehlterSonderProgrammZustand(EnumSonderProgrammZustand.inaktiv);
        bZustand = true;
      }
    }

    /*
     * Richtung SonderProgramm
     */
    {
      if (sonderProgramm.getGewaehlteSonderProgrammRichtung() == null)
      {        
        if (_richtung != null){
          sonderProgramm.setGewaehlteSonderProgrammRichtung(_richtung);
          bRichtung = true;
        }
        
      }
    }
    
    
    if(_listSelectionFahrtrichtungNord != null && _listSelectionFahrtrichtungNord.size() > 0){  
      SystemObject so1 = _listSelectionFahrtrichtungNord.getFirst();
      SystemObject so2 = _listSelectionFahrtrichtungNord.getLast();
      SystemObject soSbaAq1 = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, so1);
      if (soSbaAq1 != null){
        double d = DaVHelper.holeBetriebKilometerVonSbaAq(_connection, soSbaAq1);
        sonderProgramm.setAuswahlVonKm(d);
      }
      SystemObject soSbaAq2 = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, so2);
      if (soSbaAq2 != null){
        double d = DaVHelper.holeBetriebKilometerVonSbaAq(_connection, soSbaAq2);
        sonderProgramm.setAuswahlBisKm(d);
      } 
      bKilometer = true;
    }
    
    if(_listSelectionFahrtrichtungSued != null && _listSelectionFahrtrichtungSued.size() > 0){  
      SystemObject so1 = _listSelectionFahrtrichtungSued.getFirst();
      SystemObject so2 = _listSelectionFahrtrichtungSued.getLast();
      SystemObject soSbaAq1 = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, so1);
      if (soSbaAq1 != null){
        double d = DaVHelper.holeBetriebKilometerVonSbaAq(_connection, soSbaAq1);
        sonderProgramm.setAuswahlVonKm2(d);
      }
      SystemObject soSbaAq2 = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, so2);
      if (soSbaAq2 != null){
        double d = DaVHelper.holeBetriebKilometerVonSbaAq(_connection, soSbaAq2);
        sonderProgramm.setAuswahlBisKm2(d);
      } 
      
      bKilometer = true;
    }
    
    
    
    
//    /*
//     * Kilometrierung
//     */
//    if (textStartKm != null && textStopKm != null)
//    {
//      try
//      {
//        String t1 = textStartKm.getText();
//        t1 = t1.replace(",", ".");
//        Double d1 = new Double(t1);
//        sonderProgramm.setAuswahlVonKm(d1 * 1000.0);
//
//        String t2 = textStopKm.getText();
//        t2 = t2.replace(",", ".");
//        Double d2 = new Double(t2);
//        sonderProgramm.setAuswahlBisKm(d2 * 1000.0);
//
//        EnumSonderProgrammRichtung richtung = sonderProgramm.getGewaehlteSonderProgrammRichtung();
//        if (richtung.equals(EnumSonderProgrammRichtung.NO))
//        {
//          // if (d2 >= d1)
//          bKilometer = true;
//        }
//        else if (richtung.equals(EnumSonderProgrammRichtung.SW))
//        {
//          // if (d1 >= d2)
//          bKilometer = true;
//        }
//      }
//      catch (NumberFormatException e)
//      {
//        e.printStackTrace();
//      }
//    }
//    
//    /*
//     * Kilometrierung
//     */
//    if (textStartKm2 != null && textStopKm2 != null)
//    {
//      try
//      {
//        String t1 = textStartKm2.getText();
//        t1 = t1.replace(",", ".");
//        Double d1 = new Double(t1);
//        sonderProgramm.setAuswahlVonKm2(d1 * 1000.0);
//        
//        String t2 = textStopKm2.getText();
//        t2 = t2.replace(",", ".");
//        Double d2 = new Double(t2);
//        sonderProgramm.setAuswahlBisKm2(d2 * 1000.0);
//        
//        EnumSonderProgrammRichtung richtung = sonderProgramm.getGewaehlteSonderProgrammRichtung();
//        if (richtung.equals(EnumSonderProgrammRichtung.NO))
//        {
//          // if (d2 >= d1)
//          bKilometer = true;
//        }
//        else if (richtung.equals(EnumSonderProgrammRichtung.SW))
//        {
//          // if (d1 >= d2)
//          bKilometer = true;
//        }
//      }
//      catch (NumberFormatException e)
//      {
//        e.printStackTrace();
//      }
//    }
    
    /*
     * QuerLaengsabgleich
     */
    if (_radioButtonQuerLaengsabgleich != null)
    {
      sonderProgramm.setQuerLaengsabgleich(_radioButtonQuerLaengsabgleich.getSelection());

      bQuerLaengsabgleich = true;
    }

    /*
     * Zusatzpruefung
     */
    if (_radioButtonAufhebung != null)
    {
      sonderProgramm.getZusatzPruefung()[2] = _radioButtonAufhebung.getSelection();
      bZusatzpruefung = true;
    }

     boolean returnValue = true;

    if (!bDefinition)
    {
      fehlerDialog("Programm ungltig", "Es muss ein Programm ausgewhlt werden !");
      returnValue = false;
    }
    // Werden nicht ausgewertet
    if (!bAktion)
    {
    }
    if (!bRichtung)
    {
    }
    if (!bZustand)
    {
    }
//    if (!bAnzeigeQuerschnitt)
//    {
//      fehlerDialog("Auswahl ungltig", "Es muss ein Anzeigequerschnitt ausgewhlt werden!");
//      returnValue = false;
//    }

//    if (!bHandschaltung)
//    {
//      fehlerDialog("Handschaltung ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bPrioHandschaltung)
//    {
//      fehlerDialog("Prioritt Handschaltung ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }

//    if (!bQuerLaengsabgleich)
//    {
//      fehlerDialog("QuerLngsabgleich ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bZusatzpruefung)
//    {
//      fehlerDialog("Zusatzprfung ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bBetriebsArt)
//    {
//      fehlerDialog("Betriebsart ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
    
    
    
//    // Wird nicht ausgewertet
//    if (!bVorgabenSchaltungAq)
//    {
//    }
//    if (!bZeitBereichVon)
//    {
//      fehlerDialog("Zeitbereich Von ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bZeitBereichBis)
//    {
//      fehlerDialog("Zeitbereich Bis ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bAuswahlStunden)
//    {
//      fehlerDialog("Auswahl Stunden ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bAuswahlTage)
//    {
//      fehlerDialog("Auswahl Wochentag ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//    }
//    if (!bAuswahlMonate)
//    {
//      fehlerDialog("Auswahl Monate ungltig", "Auswahl ist ungltig");
//      returnValue = false;
//
//    }

//    boolean bilden = false;
//
//    if (_sonderProgrammDatensatz != null)
//    {
//
//      if (sonderProgramm.getTypBezeichnung() != null)
//      {
//        if (!sonderProgramm.getTypBezeichnung().equals(_sonderProgrammDatensatz.getTypBezeichnung()))
//        {
//          bilden = true;
//        }
//      }
//
//      if (sonderProgramm.getBezeichnung() != null)
//      {
//        if (!sonderProgramm.getBezeichnung().equals(_sonderProgrammDatensatz.getBezeichnung()))
//        {
//          bilden = true;
//        }
//      }
//
//      Double d1 = sonderProgramm.getAuswahlVonKm();
//      Double d2 = _sonderProgrammDatensatz.getAuswahlVonKm();
//
//      if (!d1.equals(d2))
//      {
//        bilden = true;
//      }
//
//      d1 = sonderProgramm.getAuswahlBisKm();
//      d2 = _sonderProgrammDatensatz.getAuswahlBisKm();
//
//      if (!d1.equals(d2))
//      {
//        bilden = true;
//      }
//
//      Sperrung[] sp1 = sonderProgramm.getSperrungen();
//      Sperrung[] sp2 = _sonderProgrammDatensatz.getSperrungen();
//
//      if (sp1 != null && sp2 != null)
//      {
//        if (sp1.length != sp2.length)
//        {
//          bilden = true;
//        }
//        else
//        {
//          for (int i = 0; i < sp1.length; i++)
//          {
//            Sperrung s1 = sp1[i];
//            Sperrung s2 = sp2[i];
//
//            if (s1 != null && s2 != null)
//            {
//              Double d3 = s1.getSperrungVonKm();
//              Double d4 = s2.getSperrungVonKm();
//
//              if (d3 != null && d4 != null)
//              {
//                if (!d3.equals(d4))
//                {
//                  bilden = true;
//                }
//              }
//              Double d5 = s1.getSperrungBisKm();
//              Double d6 = s2.getSperrungBisKm();
//
//              if (d5 != null && d6 != null)
//              {
//                if (!d5.equals(d6))
//                {
//                  bilden = true;
//                }
//              }
//
//              Boolean[] sperrungFahrspur = s1.getSperrungFahrspur();
//              for (int j = 0; j < sperrungFahrspur.length; j++)
//              {
//                Boolean bb1 = sperrungFahrspur[j];
//                Boolean bb2 = s2.getSperrungFahrspur()[j];
//
//                if (bb1 != null && bb2 != null && !bb1.equals(bb2))
//                {
//                  bilden = true;
//                }
//              }
//            }
//            else
//            {
//              bilden = true;
//            }
//          }
//        }
//      }
//      else
//      {
//        bilden = true;
//      }
//
//      Boolean[] schaltOptionen = sonderProgramm.getSchaltOptionen();
//      if (schaltOptionen != null)
//      {
//        for (int i = 0; i < schaltOptionen.length; i++)
//        {
//          Boolean b = schaltOptionen[i];
//          if (b != null && !b.equals(_sonderProgrammDatensatz.getSchaltOptionen()[i]))
//          {
//            bilden = true;
//          }
//        }
//      }
//    }
//    else
//      bilden = true;
//
//    if (bilden)
//    {
//      sonderProgramm.setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion.Bilden);
//    }
//    else
//    {
//      sonderProgramm.setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion.ndern);
//    }
    sonderProgramm.setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion.Parametrieren);

    if (returnValue)
      return sonderProgramm;
    else
      return null;
  }
  
  private void fehlerDialog(String titel, String meldung)
  {
    MessageBox mb = new MessageBox(Display.getDefault().getActiveShell(), SWT.ICON_ERROR | SWT.OK);
    mb.setText(titel);
    mb.setMessage(meldung);
    mb.open();
  }
  
  /**
   * This method initializes groupFilterDefinitionen180
   * 
   */
  private void createGroupFilterDefinitionenZusatzpruefung(Composite parent)
  {
    GridData gridDataGroupZusatzpruefung = new GridData();
    gridDataGroupZusatzpruefung.horizontalAlignment = GridData.FILL;
    gridDataGroupZusatzpruefung.grabExcessHorizontalSpace = true;
    gridDataGroupZusatzpruefung.verticalAlignment = GridData.FILL;
    GridLayout gridLayoutGroupZusatzpruefung = new GridLayout(2, true);

    Composite groupFilterDefinitionenZusatzpruefung = new Composite(parent, SWT.NONE);
    groupFilterDefinitionenZusatzpruefung.setLayout(gridLayoutGroupZusatzpruefung);
    groupFilterDefinitionenZusatzpruefung.setLayoutData(gridDataGroupZusatzpruefung);

    _radioButtonQuerLaengsabgleich = new Button(groupFilterDefinitionenZusatzpruefung, SWT.CHECK);
    _radioButtonQuerLaengsabgleich.setText("QuerLngsabgleich");

    _radioButtonAufhebung = new Button(groupFilterDefinitionenZusatzpruefung, SWT.CHECK);
    _radioButtonAufhebung.setText("Aufhebung");
  }

  /**
   * This method initializes _compositeButton1
   * 
   */
  private void createCompositeButton()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    rowLayout.marginTop = 5;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.BEGINNING;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.BEGINNING;
    _compositeButton1 = new Composite(_compositeTree, SWT.NONE);
    _compositeButton1.setLayout(rowLayout);
    _compositeButton1.setLayoutData(gridData);
    _buttonHinzufuegen = new Button(_compositeButton1, SWT.NONE);
    _buttonHinzufuegen.setText("Hinzufgen");
    _buttonHinzufuegen.setEnabled(false);
    _buttonHinzufuegen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHinzufuegen();
      }
    });
    _buttonLoeschen = new Button(_compositeButton1, SWT.NONE);
    _buttonLoeschen.setText("Lschen");
    _buttonLoeschen.setEnabled(false);
    _buttonLoeschen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionLoeschen();
      }
    });
    _buttonHoch = new Button(_compositeButton1, SWT.NONE);
    _buttonHoch.setText("Hoch");
    _buttonHoch.setEnabled(false);
    _buttonHoch.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHoch();
      }
    });
    _buttonRunter = new Button(_compositeButton1, SWT.NONE);
    _buttonRunter.setText("Runter");
    _buttonRunter.setEnabled(false);
    _buttonRunter.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionRunter();
      }
    });
    _buttonKopieren = new Button(_compositeButton1, SWT.NONE);
    _buttonKopieren.setText("Kopieren");
    _buttonKopieren.setEnabled(false);
    _buttonKopieren.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionKopieren();
      }
    });
  }

  /**
   * This method initializes _composite2
   * 
   */
  private void createComposite2()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite2 = new Composite(_sf1, SWT.NONE);
    _composite2.setLayout(new GridLayout());
    _composite2.setLayoutData(gridData);

    createGraphicalViewer();
  }

  /**
   * GraphicalViewer fr die Darstellung
   */
  private void createGraphicalViewer()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite21 = new Composite(_composite2, SWT.BORDER);
    _composite21.setLayout(new FillLayout());
    _composite21.setLayoutData(gridData);

    // Ansicht in einen extra Viewer packen.
    graphicalViewer = new ScrollingGraphicalViewer();
    graphicalViewer.createControl(_composite21);
    // XXX: Wichtig - Damit wei der Viewer welchen EditPart er fr ein
    // Modell Objekt benutzen soll.
    graphicalViewer.setEditDomain(new EditDomain());
    graphicalViewer.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewer.setEditPartFactory(new DarstellungEditPartFactory());
    
    // Hier wird der Viewer global mit dem Kontext fr die Parameterdarstellung versorgt, da in der Legende der Kontext 
    // fr Fahrstreifen und Detektor bentigt wird, diese aber vin der Emf-Modelldefinition keinen Kontext enthalten. 
    graphicalViewer.getControl().setData("kontext", 5);
    
//    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
//    String name = store.getString(DefsPreference.P_DARSTELLUNG_PARAMETER);    
//    if (name != null && name.length() > 0){
//      graphicalViewer.setContents(EcoreUtil.copy(SbaEinstellungen.INSTANCE.getAnsicht(name)));    
//    }
    
    
    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(DefsPreference.P_DARSTELLUNG_PARAMETER);
    if (ansicht != null)
      graphicalViewer.setContents(ansicht);    
    
    getCommandStack().setUndoLimit(1);
    
    final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewer.getRootEditPart();

//    final ZoomManager zoomManager = rootEditPart.getZoomManager();
//    zoomManager.setZoomLevels(new double[] { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 });
//    graphicalViewer.setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.NONE), MouseWheelZoomHandler.SINGLETON);

    if (DaVKommunikationDobj.getInstanz().holeDaVVerbindung() != null)
    {

      AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
//      Ansicht ansicht = contents.getModel();

      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
      darstellung.setSpaltenBorder(true);

      contents = (AnsichtEditPart)graphicalViewer.getContents();
      ansicht = contents.getModel();
      darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
      
      pruefeFahrtRichtung(darstellung);          
    }
    
    erzeugeKontextMenue();

    graphicalViewer.getControl().addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        // TODO Auto-generated method stub
        ISelection selection = graphicalViewer.getSelection();

        TreeItem[] tree_selection = _tree.getSelection();
        if (tree_selection != null && tree_selection.length > 0)
        {
          Data data = (Data)tree_selection[0].getData();
          DoModelHelper.holeDoppelKlickAktionAusViewerSelection(selection, data);
        }

      }
    });
  }

  
  /**
   * Vorbelegung aller Anzeigen mit einer Inhalt
   * @param inhalt
   */
  public void setzeAlleAnzeigen(AnzeigeInhaltTyp  inhalt)
  {
    setzeAlleAnzeigen(inhalt, null, false);
  }
  
  /**
   * Vorbelegung aller Anzeigen mit einer Inhalt
   * @param inhalt
   */
  public void setzeAlleAnzeigen(AnzeigeInhaltTyp  inhalt, EnumSonderProgrammRichtung richtung, boolean scharf)
  {
    AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> dos = ebene.getDoObjekte();
      for (DoModel dom : dos)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
          
          EnumSonderProgrammRichtung tmp = null;
          if (richtung != null)
          {            
            tmp = EnumSonderProgrammRichtung.NO;
            if (model.getFahrtrichtung() > 0)
              tmp = EnumSonderProgrammRichtung.SW;
          }
          
          boolean doit = false;
          
          if (richtung == null){
            doit = true;
          }
          else if (tmp != null && tmp.equals(richtung)){
            doit = true;
          }else{
            
          }
          
          if (doit)
          {
            EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
            for (AnzeigeModelTypAbstract anzeige : komponenten)
            {
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(anzeige, feature, inhalt);
              getCommandStack().execute(cmd);
              EStructuralFeature feature2 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKEN;
              final SetCommand cmd2 = new SetCommand(anzeige, feature2, 0);
              getCommandStack().execute(cmd2);
//              EStructuralFeature feature3 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKTAKT;
//              final SetCommand cmd3 = new SetCommand(anzeige, feature3, takt);
//              getCommandStack().execute(cmd3);
                            
              SystemObject so = anzeige.getSystemObject();
              
              if (scharf){
                List<Data> list = new ArrayList<Data>();
                int loc = 0;
                TreeItem[] tis = _tree.getSelection();
                if (tis != null && tis.length > 0){
                  Data data2 = (Data)tis[0].getData();
                  if (data2!= null){
                    Data data = (data2).getItem("AnzeigenDatenstze");                        
                    Array array = data.asArray();
                    for (int i = 0; i < array.getLength(); i++)
                    {
                      Data item = array.getItem(i);
                      SystemObject anzeige2 = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                      if (anzeige2 != null && so != null && anzeige2.equals(so))
                      {                     
                        loc = i;                     
                      }
                      // Liste wird vorbelegt
                      list.add(item);                  
                    }
                    if (!list.isEmpty() && list.size() - 1 >= loc)
                    {
                      list.remove(loc);
                    }
                    array.setLength(list.size());
                    for (int i = 0; i < array.getLength(); i++)
                    {
                      Data ziel = array.getItem(i);
                      Data quelle = list.get(i);
                      // ziel = quelle;
                      DataHelper.kopiereRekursiv(quelle, ziel);
                    }                  
                  }
                }               
              }
            }            
          }
        }
      }
    }
  }
  
  
//  /**
//   * Vorbelegung aller Anzeigen mit einer Inhalt
//   * @param inhalt
//   */
//  public void setzeAlleAnzeigen(AnzeigeInhaltTyp  inhalt, EnumSonderProgrammRichtung richtung)
//  {
//    setzeAlleAnzeigen(inhalt, richtung, false);
//  }

  /**
   * Aktualisert die Darstellung im {@code GraphicalViewer} mit dem Daten des uebergebenen {@code SystemObject}
   * 
   * @param die
   *          Ursacheneinheit
   */
  protected void aktualisiereBaum(SystemObject so)
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
    Aspect asp = _connection.getDataModel().getAspect(_aspekt);
    ResultData result = _connection.getData(so, new DataDescription(atg, asp), 1500);

    _tree.removeAll();

    if (result.hasData())
    {
      if (result != null && result.hasData())
      {
        Data data = result.getData().createModifiableCopy();
        TreeItem itemAktiv = new TreeItem(_tree, SWT.NONE);
        itemAktiv.setText("Aktiv");
        Array array = data.getItem("Aktiv").asArray();
        for (int i = 0; i < array.getLength(); i++)
        {
          TreeItem subItem = new TreeItem(itemAktiv, SWT.NONE);
          subItem.setText(new String("Alternative: " + i));
          Data item = array.getItem(i);
          subItem.setData(item);
        }
        
        
        boolean selected = false;
        
        if (itemAktiv.getItems() != null && itemAktiv.getItems().length > 0){
          setzeAlleAnzeigen(AnzeigeInhaltTyp.BELIEBIG_BI);
          aktualisiereDarstellung((Data)itemAktiv.getItem(0).getData());          
          _tree.select(itemAktiv.getItem(0));
          selected = true;
        }
        
        TreeItem itemInaktiv = new TreeItem(_tree, SWT.NONE);
        itemInaktiv.setText("Inaktiv");
        array = data.getItem("Inaktiv").asArray();
        for (int i = 0; i < array.getLength(); i++)
        {
          TreeItem subItem = new TreeItem(itemInaktiv, SWT.NONE);
          subItem.setText(new String("Alternative: " + i));
          Data item = array.getItem(i);
          subItem.setData(item);
        }
        
        if (itemInaktiv.getItems() != null && itemInaktiv.getItems().length > 0){
          if (!selected){
            setzeAlleAnzeigen(AnzeigeInhaltTyp.BELIEBIG_BI);
            aktualisiereDarstellung((Data)itemAktiv.getItem(0).getData());
            _tree.select(itemInaktiv.getItem(0));
            selected = true;
          }
        }
        
        if (!selected){
          setzeAlleAnzeigen(AnzeigeInhaltTyp.INAKTIV_BI);
          _tree.select(itemAktiv);
        }
        
      }
    }
  }

  /**
   * Aktivieren bzw. Deaktivieren der Buttons anhanf der Selektion im Baum
   */
  private void updateButtons()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      String text = ti.getText();
      if (text.equals("Aktiv") || text.equals("Inaktiv"))
      {
        _buttonHinzufuegen.setEnabled(true);
        _buttonLoeschen.setEnabled(false);
        _buttonHoch.setEnabled(false);
        _buttonRunter.setEnabled(false);
        _buttonKopieren.setEnabled(false);
      }
      else
      {
        _buttonHinzufuegen.setEnabled(true);
        _buttonLoeschen.setEnabled(true);

        TreeItem parentItem = ti.getParentItem();
        int cnt = parentItem.getItemCount();
        int cntNeu = cnt;
        if (cnt > 0)
          cntNeu -= 1;
        if (cnt == 1)
        {
          _buttonHoch.setEnabled(false);
          _buttonRunter.setEnabled(false);
        }
        else if (text.equals("Alternative: 0"))
        {
          _buttonHoch.setEnabled(false);
          _buttonRunter.setEnabled(true);
        }
        else if (text.equals("Alternative: " + cntNeu))
        {
          _buttonHoch.setEnabled(true);
          _buttonRunter.setEnabled(false);
        }
        else
        {
          _buttonHoch.setEnabled(true);
          _buttonRunter.setEnabled(true);
        }

        _buttonKopieren.setEnabled(true);
      }
    }
    else
    {
      _buttonHinzufuegen.setEnabled(false);
      _buttonLoeschen.setEnabled(false);
      _buttonHoch.setEnabled(false);
      _buttonRunter.setEnabled(false);
      _buttonKopieren.setEnabled(false);
    }
  }
  
  /**
   * Aktivieren bzw. Deaktivieren der Kontextmenues anhand der Selektion im Baum
   */
  private void updateKontextMenu()
  {
    
    IAction actionHinzu = _actionRegistry.getAction("Hinzufuegen");
    IAction actionLoesch = _actionRegistry.getAction("Loeschen");
    IAction actionHoch = _actionRegistry.getAction("Hoch");
    IAction actionRunter = _actionRegistry.getAction("Runter");
    IAction actionKopie = _actionRegistry.getAction("Kopieren");
    
    
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      String text = ti.getText();
      if (text.equals("Aktiv") || text.equals("Inaktiv"))
      {
        
        actionHinzu.setEnabled(true);
        actionLoesch.setEnabled(false);
        actionHoch.setEnabled(false);
        actionRunter.setEnabled(false);
        actionKopie.setEnabled(false);
//        _buttonHinzufuegen.setEnabled(true);
//        _buttonLoeschen.setEnabled(false);
//        _buttonHoch.setEnabled(false);
//        _buttonRunter.setEnabled(false);
//        _buttonKopieren.setEnabled(false);
      }
      else
      {
        actionHinzu.setEnabled(true);
        actionLoesch.setEnabled(true);
//        _buttonHinzufuegen.setEnabled(true);
//        _buttonLoeschen.setEnabled(true);
        
        TreeItem parentItem = ti.getParentItem();
        int cnt = parentItem.getItemCount();
        int cntNeu = cnt;
        if (cnt > 0)
          cntNeu -= 1;
        if (cnt == 1)
        {
          actionHoch.setEnabled(false);
          actionRunter.setEnabled(false);
//          _buttonHoch.setEnabled(false);
//          _buttonRunter.setEnabled(false);
        }
        else if (text.equals("Alternative: 0"))
        {
          actionHoch.setEnabled(false);
          actionRunter.setEnabled(true);
//          _buttonHoch.setEnabled(false);
//          _buttonRunter.setEnabled(true);
        }
        else if (text.equals("Alternative: " + cntNeu))
        {
          actionHoch.setEnabled(true);
          actionRunter.setEnabled(false);
//          _buttonHoch.setEnabled(true);
//          _buttonRunter.setEnabled(false);
        }
        else
        {
          actionHoch.setEnabled(true);
          actionRunter.setEnabled(true);
//          _buttonHoch.setEnabled(true);
//          _buttonRunter.setEnabled(true);
        }
        
        actionKopie.setEnabled(true);
//        _buttonKopieren.setEnabled(true);
      }
    }
    else
    {
      actionHinzu.setEnabled(false);
      actionLoesch.setEnabled(false);
      actionHoch.setEnabled(false);
      actionRunter.setEnabled(false);
      actionKopie.setEnabled(false);
//      _buttonHinzufuegen.setEnabled(false);
//      _buttonLoeschen.setEnabled(false);
    }
  }

  /**
   * Erzeugt eine neue Alternative und fgt diese am Ende der Alternativen ein
   */
  protected void aktionHinzufuegen()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem selectedItem = selection[0];
      String text = selectedItem.getText();

      TreeItem parent = null;

      if (text.equals("Aktiv") || text.equals("Inaktiv"))
      {
        parent = selectedItem;
      }
      else
      {
        parent = selectedItem.getParentItem();
      }

      if (parent != null)
      {
        TreeItem subItem = new TreeItem(parent, SWT.NONE);
        int index = parent.getItemCount() - 1;
        subItem.setText(new String("Alternative: " + index));

        if (_connection != null)
        {
          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaEinzelMassnahme");
          Data data = _connection.createData(atg).createModifiableCopy();
          Array array = data.getItem(parent.getText()).asArray();
          array.setLength(parent.getItemCount());
          Data item = array.getItem(index);
          subItem.setData(item);

        }

        _tree.setSelection(subItem);
        Event event = new Event();
        event.item = subItem;
        _tree.notifyListeners(SWT.Selection, event);

      }
    }
  }

  /**
   * Loescht die selektierte Alternative
   */
  protected void aktionLoeschen()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      if (ti != null && !ti.isDisposed())
        ti.dispose();
      TreeItem[] items = parent.getItems();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];
        item.setText("Alternative: " + i);
      }
    }

  }

  /**
   * Verschiebt die Alternative um eine Position nach oben
   */
  protected void aktionHoch()
  {
    // TODO Auto-generated method stub
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      TreeItem[] items = parent.getItems();

      List<Data> list = new ArrayList<Data>();

      int index = 0;

      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];

        if (item.equals(ti))
        {
          index = i;
        }

        Data dta = (Data)item.getData();
        list.add(dta);
      }

      Collections.swap(list, index - 1, index);

      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];
        item.setData(list.get(i));
      }
      _tree.select(parent.getItem(index - 1));

    }

  }

  /**
   * Verschiebt die Alternative um eine Position nach unten
   */
  protected void aktionRunter()
  {
    // TODO Auto-generated method stub
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      TreeItem[] items = parent.getItems();

      List<Data> list = new ArrayList<Data>();

      int index = 0;

      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];

        if (item.equals(ti))
        {
          index = i;
        }

        Data dta = (Data)item.getData();
        list.add(dta);
      }

      Collections.swap(list, index + 1, index);

      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];
        item.setData(list.get(i));
      }
      _tree.select(parent.getItem(index + 1));
    }

  }

  /**
   * Kopiert die selektierte Alternative mit ihren Daten und fgt diese am Ende der Alternativen ein
   */
  protected void aktionKopieren()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem selectedItem = selection[0];
      String text = selectedItem.getText();

      TreeItem parent = null;

      if (text.equals("Aktiv") || text.equals("Inaktiv"))
      {
        parent = selectedItem;
      }
      else
      {
        parent = selectedItem.getParentItem();
      }

      if (parent != null)
      {
        TreeItem subItem = new TreeItem(parent, SWT.NONE);
        int index = parent.getItemCount() - 1;
        subItem.setText(new String("Alternative: " + index));

        Data data = (Data)selectedItem.getData();

        if (data != null)
        {
          subItem.setData(data);
        }

        _tree.setSelection(subItem);
        Event event = new Event();
        event.item = subItem;
        _tree.notifyListeners(SWT.Selection, event);

      }
    }

  }

  /**
   * Holt das {@code DoModel} aus dem Parameter atg.sswSbaEinzelMassnahme und benachrichtigt den {@code GraphicalViewer}
   * zur Darstellung der Aenderung
   * 
   * @param die
   *          Daten aus der Atg
   */
  private void aktualisiereDarstellung(Data item)
  {
//    System.out.println("EinzelmassnahmenViewParameterComposite.aktualisiereDarstellung(): " + item);
    
    if (item != null)
    {

      Array array = item.getItem("AnzeigenDatenstze").asArray();
      for (int i = 0; i < array.getLength(); i++)
      {
        Data item2 = array.getItem(i);
        SystemObject soAnzeige = item2.getItem("Anzeige").asReferenceValue().getSystemObject();
        if (soAnzeige != null)
        {
          AnzeigeModelTypAbstract doModel = DoModelHelper.holeAnzeigeModelTypAbstractVonSystemObject(
              (AnsichtEditPart)graphicalViewer.getContents(), soAnzeige,
              DoModelHelper.ENUM_DARSTELLUNG_KONTEXT.PARAMETER);
          
          SystemObject soAnzeigeInhalt = item2.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue()
              .getSystemObject();
          String blinken = item2.getItem("WVZInhalt").getItem("Blinken").asTextValue().getValueText();
          String takt = item2.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().getValueText();
          String prioritaet = item2.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().getValueText();
          int notwendig = item2.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().getValueText().equals("Ja") ? 1 : 0;
          int absolutNotwendig = item2.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().getValueText().equals("Ja") ? 1: 0;
          
          Integer bl = 0;
          if (blinken.equals("Ja"))
            bl = 1;          
          
          Data item3 = item2.getItem("WVZInhalt").getItem("DynamischerWechseltext");
          
          if (soAnzeigeInhalt != null){ // Anzeigeinhalt ist WVZ-Inhalt
            AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
            Data cd = soAnzeigeInhalt.getConfigurationData(atg);
            String bi = cd.getItem("Bildinhalt").asTextValue().getText();
            AnzeigeInhaltTyp inhalt = AnzeigeInhaltTyp.setBildInhalt(bi, null);            
           
            if (doModel != null)
            {
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(doModel, feature, inhalt);
              getCommandStack().execute(cmd);
//              System.out.println(doModel.getSystemObject() + "aktualisiereDarstellung(notify): " + inhalt.getType() + " -> " + bl + " -> " + takt);
              
            }            
          }else if (item3 != null){ // Anzeigeinhalt ist Wechseltext
            VisuAnzeigeKapsel kapsel = DaVHelper.holeDynamischerWechselTextAusData(item3);           
            if (doModel != null)
            {
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(doModel, feature, kapsel);
              getCommandStack().execute(cmd);
            }           
          }

          EStructuralFeature feature2 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKEN;
          final SetCommand cmd2 = new SetCommand(doModel, feature2, bl);
          getCommandStack().execute(cmd2);
          EStructuralFeature feature3 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKTAKT;
          final SetCommand cmd3 = new SetCommand(doModel, feature3, takt);
          getCommandStack().execute(cmd3);
          EStructuralFeature feature4 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__PRIORITAET;
          final SetCommand cmd4 = new SetCommand(doModel, feature4, prioritaet);
          getCommandStack().execute(cmd4);
          EStructuralFeature feature5 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__NOTWENDIG;
          final SetCommand cmd5 = new SetCommand(doModel, feature5, notwendig);
          getCommandStack().execute(cmd5);
          EStructuralFeature feature6 = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__ABSOLUT_NOTWENDIG;
          final SetCommand cmd6 = new SetCommand(doModel, feature6, absolutNotwendig);
          getCommandStack().execute(cmd6);
        }
      }
    }
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {

    CommandStack stack = graphicalViewer.getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * Holt den GEF-Viewer
   * 
   * @return viewer
   */
  public GraphicalViewer getGraphicalViewer()
  {
    return graphicalViewer;
  }

  /**
   * Setzt den GEF-Viewer
   * 
   * @param graphicalViewer
   */
  public void setGraphicalViewer(GraphicalViewer graphicalViewer)
  {
    // TODO Auto-generated method stub
    this.graphicalViewer = graphicalViewer;
  }

  /**
   * Holt das Baum-Objekt
   * 
   * @return
   */
  public Tree getTree()
  {
    return _tree;
  }
  
  
  /**
   * Liefert das KontextMenue zum Baum
   */
  private void erzeugeKontextMenue()
  {  
    _kontextMenuMgr = new MenuManager("#EinzelmassnahmenMenu");
    erzeugeAktionen();
    
    addtoKontextMenue((Action)_actionRegistry.getAction("Hinzufuegen"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Loeschen"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Hoch"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Runter"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Kopieren"));
    
    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr.createContextMenu(_tree);
    // Das Men dem Baum bekannt machen
    _tree.setMenu( menu );        
  }
  
  /**
   * Aktion im Kontextmen setzen
   * 
   * @param action  Aktion die im Kontextmen erscheinen soll
   */
  public void addtoKontextMenue(Action action)
  {
    
    if ( action != null )
      _kontextMenuMgr.add(action);
  }
  
  /**
   * Erzeugt die Aktionen zum Verwalten der Hierarchieobjekte
   */
  private void erzeugeAktionen()
  {
    _actionRegistry = new ActionRegistry();

    Action action1 = new Action() {
      public void run()
      {
        aktionHinzufuegen();
      }
    };
    action1.setId("Hinzufuegen");
    action1.setText("Hinzufgen");
    action1.setToolTipText("Erstellt eine neue Alternative");
    _actionRegistry.registerAction(action1);
    
    Action action2 = new Action() {
      public void run()
      {
        aktionLoeschen();
      }
    };
    action2.setId("Loeschen");
    action2.setText("Lschen");
    action2.setToolTipText("Lscht eine bestehende Alternative");
    _actionRegistry.registerAction(action2);
    
    Action action3 = new Action() {
      public void run()
      {
        aktionHoch();
      }
    };
    action3.setId("Hoch");
    action3.setText("Hoch");
    action3.setToolTipText("Verschiebt die Alternative um eine Position nach oben");
    _actionRegistry.registerAction(action3);
    
    Action action4 = new Action() {
      public void run()
      {
        aktionRunter();
      }
    };
    action4.setId("Runter");
    action4.setText("Runter");
    action4.setToolTipText("Verschiebt die Alternative um eine Position nach unten");
    _actionRegistry.registerAction(action4);
    
    Action action5 = new Action() {
      public void run()
      {
        aktionKopieren();
      }
    };
    action5.setId("Kopieren");
    action5.setText("Kopieren");
    action5.setToolTipText("Kopiert die Alternative und fgt diese am Ende der Alternativen ein");
    _actionRegistry.registerAction(action5);
  }
  

  @Override
  public void daVDataUpdate(de.bsvrz.buv.plugin.ursacheneinheit.daVDataUpdateListener.DaVDataUpdateEvent e)
  {
    // TODO Auto-generated method stub
    if (e.getResult() != null)
      verarbeiteDaVDaten(e.getResult());
  }
  
  
  /**
   * @param result
   */
  private void verarbeiteDaVDaten(ResultData result)
  {
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();    
    if (atg.getPid().equals("atg.sswSbaManahmeVorschlagAntwort"))
    {
      if (result.hasData()){
        Data data = result.getData();                
        String bezeichnung = data.getItem("Bezeichnung").asTextValue().getValueText();
        SystemObject absenderId = data.getItem("absenderId").asReferenceValue().getSystemObject();
        SystemObject app = (SystemObject)_connection.getLocalApplicationObject();
        
        String absenderZeichen = data.getItem("absenderZeichen").asTextValue().getValueText();
        if (absenderZeichen != null && absenderZeichen.contains("#")){
          String[] split = absenderZeichen.split("#");
          if (split != null && split.length > 0){
            String fr = split[0];
            String zeitstempel = split[1];
 
            if (_so.getName().equals(bezeichnung) && app.equals(absenderId)){
              
              boolean neueDaten = false;
              if (fr.equals(_richtungVonUntenNachOben.toString()) && _anfrageZeitstempelFrNo != null && _anfrageZeitstempelFrNo.equals(zeitstempel)){
                neueDaten = true;
              }
              else if (fr.equals(_richtungVonObenNachUnten.toString()) && _anfrageZeitstempelFrSw != null && _anfrageZeitstempelFrSw.equals(zeitstempel)){
                neueDaten = true;
              }
              
//              System.out.println("EinzelmassnahmenViewParameterComposite.verarbeiteDaVDaten(): " + neueDaten);
              
              if (neueDaten){
                aktualisiereDarstellung(data);
                
                if (_tree != null && !_tree.isDisposed()){
                  TreeItem[] tis = _tree.getSelection();
                  if (tis != null && tis.length > 0){
                    
                    Data data2 = (Data)tis[0].getData();                                        
                    Data ziel = data2.getItem("AnzeigenDatenstze");                    
                    Data quelle = data.getItem("AnzeigenDatenstze");
                    
                    Array arrayZiel = ziel.asArray();
                    Array arrayQuelle = quelle.asArray();
                    int start = arrayZiel.getLength(); 
                    int len = start + arrayQuelle.getLength();
                    
                    arrayZiel.setLength(len);
                    for (int i = 0; i < arrayQuelle.getLength(); i++)
                    {
                      Data itemZiel = arrayZiel.getItem(start + i);
                      Data itemQuelle = arrayQuelle.getItem(i);
                      DataHelper.kopiereRekursiv(itemQuelle, itemZiel);                     
                    }   
                    
//                    tis[0].setData(data2);
                  }                                  
                }
              }                
            }                   
          }
        }
      }
    }
  }
  
  /**
   * Hilfsmethode, uebernimmt die Daten der internen Datenstruktur {@link SonderProgramm} 
   * in den Datensatz {@link Data} zum Versenden ueber den Datenverteiler
   *  
   * @param connection
   * @param sonderProgramm
   * @param loeschen
   * @param data
   * @return
   */
  public static void schreibeData(ClientDavInterface connection, SonderProgramm sonderProgramm, boolean loeschen,
      Data data, long zeitstempel, EnumSonderProgrammRichtung richtung, boolean absteigend)
  {
    SystemObject app = (SystemObject)connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject(app);
    
    
//    long absenderZeichen = System.currentTimeMillis();
    
    data.getItem("absenderZeichen").asTextValue().setText(richtung.toString() + "#" + zeitstempel);
    data.getItem("Bezeichnung").asTextValue().setText(sonderProgramm.getBezeichnung());

    String typBezeichnung = sonderProgramm.getTypBezeichnung();
    data.getItem("ProgrammTyp").asTextValue().setText(typBezeichnung);

    // Gltigkeit Ereignisse
    SystemObject[] ereignisse = sonderProgramm.getEreignisse();
    if (ereignisse != null)
    {
      ReferenceArray refs1 = data.getItem("Gltigkeit").getItem("Ereignis").asReferenceArray();
      refs1.setLength(ereignisse.length);
      for (int i = 0; i < ereignisse.length; i++)
      {
        refs1.getReferenceValue(i).setSystemObject(ereignisse[i]);
      }
    }

    // Gltigkeit Syskals
    SystemObject[] syskals = sonderProgramm.getSystemKalenderEintraege();
    if (syskals != null)
    {
      ReferenceArray refs2 = data.getItem("Gltigkeit").getItem("SystemKalenderEintrag").asReferenceArray();
      refs2.setLength(syskals.length);
      for (int i = 0; i < syskals.length; i++)
      {
        refs2.getReferenceValue(i).setSystemObject(syskals[i]);
      }
    }

    // Streckenabschnitt
    data.getItem("Fahrtrichtung").asTextValue().setText(sonderProgramm.getGewaehlteSonderProgrammRichtung().toString());

    schreibeDataStreckenAbschnitt(sonderProgramm, data, 0, absteigend);

    // EnumSonderProgrammAktion enumAktion = sonderProgramm.getGewaehlteSonderProgrammAktion();
    if (!loeschen)
    {
      data.getItem("Aktion").asTextValue().setText(sonderProgramm.getGewaehlteSonderProgrammAktion().toString());
    }
    else
    {
      data.getItem("Aktion").asTextValue().setText(EnumSonderProgrammAktion.Lschen.toString());
    }

    Boolean b00 = sonderProgramm.getHandSchaltung();
    String s00 = "Nein";
    if (b00 != null && b00)
      s00 = "Ja";
    data.getItem("Handprogramm").getItem("Handprogramm").asTextValue().setText(s00);

    String p0 = "undefiniert";
    String p = sonderProgramm.getPrioritaetHandschaltung();
    if (p != null)
      p0 = p;

    data.getItem("Handprogramm").getItem("Prioritt").asTextValue().setText(p0);

    Boolean b0 = sonderProgramm.getQuerLaengsabgleich();
    String s0 = "Nein";
    if (b0 != null && b0)
      s0 = "Ja";
    data.getItem("QuerLngsabgleich").asTextValue().setText(s0);

    Boolean b1 = sonderProgramm.getZusatzPruefung()[0];
    String s1 = "Nein";
    if (b1 != null && b1)
      s1 = "Ja";
    data.getItem("Zusatzprfung").getItem("Ausfallbehandlung").asTextValue().setText(s1);
    b1 = sonderProgramm.getZusatzPruefung()[1];
    s1 = "Nein";
    if (b1 != null && b1)
      s1 = "Ja";
    data.getItem("Zusatzprfung").getItem("Verschiebung").asTextValue().setText(s1);
    b1 = sonderProgramm.getZusatzPruefung()[2];
    s1 = "Nein";
    if (b1 != null && b1)
      s1 = "Ja";
    data.getItem("Zusatzprfung").getItem("Aufhebung").asTextValue().setText(s1);

    data.getItem("Schaltoptionen").getItem("Aufhebung").asTextValue().setText("Nein");
    data.getItem("Schaltoptionen").getItem("Trichter").asTextValue().setText("Nein");

    Boolean b3 = sonderProgramm.getBetriebsArt()[0];
    String s3 = "Nein";
    if (b3 != null && b3)
      s3 = "Ja";
    data.getItem("Betriebsart").getItem("Simulation").asTextValue().setText(s3);
    b3 = sonderProgramm.getBetriebsArt()[1];
    s3 = "Nein";
    if (b3 != null && b3)
      s3 = "Ja";
    data.getItem("Betriebsart").getItem("Schaltung").asTextValue().setText(s3);

    // Das Array wird hier erst einmal mit einem Feldelement voreingestellt.
    Array array = data.getItem("Zeitbereich").asArray();
    array.setLength(0);
    {
      Array array4 = data.getItem("AnzeigenDatenstze").asArray();
      array4.setLength(0);
    }
    
//    return absenderZeichen;
  }
  
  /**
   * @param sonderProgramm
   * @param data
   * @param len
   */
  public static void schreibeDataStreckenAbschnitt(SonderProgramm sonderProgramm, Data data, int len, boolean absteigend)
  {    
    Double d1 = sonderProgramm.getAuswahlVonKm();
    Double d2 = sonderProgramm.getAuswahlBisKm();
           
    if (absteigend){
      d1 = sonderProgramm.getAuswahlVonKm2();
      d2 = sonderProgramm.getAuswahlBisKm2();      
    }
    long l1 = d1.longValue();
    long l2 = d2.longValue();
    

    Array array0 = data.getItem("Streckenabschnitt").asArray();
    array0.setLength(1);

    array0.getItem(0).getItem("startKm").asUnscaledValue().set(l1);
    array0.getItem(0).getItem("endeKm").asUnscaledValue().set(l2);

    Array array01 = array0.getItem(0).getItem("FahrstreifenSperrung").asArray();
    Boolean bx = sonderProgramm.isSpurSperrung();

    Sperrung[] sperrungen = sonderProgramm.getSperrungen();

    array01.setLength(len);

    for (int i = 0; i < len; i++)
    {
      Sperrung sperrung = sperrungen[i];
      Double d3 = sperrung.getSperrungVonKm();
      long l3 = d3.longValue();
      Double d4 = sperrung.getSperrungBisKm();
      long l4 = d4.longValue();

      array01.getItem(i).getItem("startKm").asUnscaledValue().set(l3);
      array01.getItem(i).getItem("endeKm").asUnscaledValue().set(l4);

      Boolean[] bb = sperrung.getSperrungFahrspur();

      Array array02 = array01.getItem(i).getItem("gesperrteFahrstreifen").asArray();
      if (bb != null && bb.length > 0)
      {
        List<String> list = new ArrayList<String>();

        for (int j = 0; j < bb.length; j++)
        {
          if (bb[j])
          {
            switch (j)
            {
              case 0:
                list.add("HFS");
                break;
              case 1:
              case 2:
              case 3:
              case 4:
              case 5:
              case 6:
                list.add(j + "FS");
                break;
              case 7:
                list.add("STNDS");
                break;
              default:
            }
          }

        }
        array02.setLength(list.size());
        for (int j = 0; j < list.size(); j++)
        {
          array02.getItem(j).asTextValue().setText(list.get(j));
        }
      }
      else
      {
        array02.setLength(0);
      }

    }
    if (bx != null && bx)
    {

    }
    else
    {
      // array01.setLength(0);
    }
  }
  
  
  /**
   * @param secId
   * @param s1
   * @param s2
   * @return
   */
  public static boolean validiereTextInput(String secId, String s1, String s2)
  {
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    String pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
    String pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

    if (secId.equals("SW"))
    {
      pref2 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
      pref1 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
    }

    double d3 = store.getDouble(pref1);
    double d4 = store.getDouble(pref2);

    return DaVHelper.validiereTextInput(secId, s1, s2, d3, d4);
  }

}

