/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.externetrigger;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.plugin.ursacheneinheit.SpinnerAddOns;
import de.bsvrz.buv.plugin.ursacheneinheit.erweiterterstaualgorithmus.TableModel;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Fenster zum Versorgen der Algorithmus abhngigen Parameter
 * @author ObertM
 */
public class CompositeParameterAlgorithmus extends Composite
{
	/**
	 * Debug
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Einheiten des DAV fr die Spinner
   */
  private static enum Einheit { KEIN };
  
  /**
   * ParameterElemente des DAV, die als Spinner angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemSpin = 
  {
    
  };
  
  /**
   * ParameterElemente des DAV, die als ComboBox angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static String[] _parItemCombo =
  {
  	"TriggerObjekt", "Aspekt", "sofortigeNeuberechnung", "ManahmeLschen"
  };
  
  /**
   * Ergebnis vom DaV
   */
  private ResultData _result;

  /**
   * Daten vom DaV
   */
  private Data _data;

  /**
   * Gesmatcomposite
   */
  private Composite _composite;

  /**
   * Composite (Gruppe) fr die Parameter
   */
  private Group _groupParameter;

  /**
   * Composite mit der Tabelle
   */
  private Composite _compositeTabelle = null;

  /**
   * Compoiste fr die Tabelle
   */
  private Composite _composite3 = null;

  /**
   * Table Viewer
   */
  private TableViewer _tvParameter;;

  /**
   * Spinner
   */
  private Spinner[] _spinnerPar;

  /**
   * ComboBoxen
   */
  private Combo[] _comboPar;

  /**
   * Liste mglicher Trigger
   */
	private List<String> _listMoeglicheTrigger;

  /**
   * Liste mglicher Aspekte
   */
	private List<String> _listMoeglicheAspekte;

  /**
   * Konstruktor der Klasse
   * @param parent Aufrufendes Composite
   * @param style Stil
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
	 * @param listeMoeglicheTrigger Liste mglicher Trigger
	 * @param listeMoeglicheAspekte Liste mglicher Aspekte
   */
  CompositeParameterAlgorithmus(Composite parent, int style, ResultData result, Data data, List<String> listeMoeglicheTrigger, List<String> listeMoeglicheAspekte)
  {
    super(parent, style);
    this._result = result;
    this._data = data;

    _listMoeglicheTrigger = listeMoeglicheTrigger;

    _listMoeglicheAspekte = listeMoeglicheAspekte;

    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    createGroupParameter();

    Table table = _tvParameter.getTable();
    TableModel[] input = createModel(table);
    _tvParameter.setInput(input);

    updateData();
  }

  /**
   * Updatefunktion fr Aufruf von aussen
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  public void update (ResultData result, Data data)
  {
    _result  = result;
    this._data = data;
    
    updateData();
    
  }
  
  /**
   * Updatefunktion
   */
  private void updateData ()
  {
    if ( _result != null )
    {
      if ( _result.hasData() )
      {
      	/*
      	 * ComboBoxen aktualisieren
      	 */
      	for (int i=0; i<_parItemCombo.length; ++i)
      	{
      		if ( i== 0 || i == 1)
      		{
      			String selected = Tools._undefiniert;
      		  SystemObject so = _data.getReferenceValue(_parItemCombo [i]).getSystemObject();
      		  if (so != null)
      		  	selected = Tools.getName(so);
      		  
  	      	Combo cb = (Combo) _comboPar [i];
  	        for (int k = 0; k < cb.getItemCount(); k++)
  	        {
  	          if (selected.equals(cb.getItem( k )))
  	            cb.select(k);      
  	        }
      		}

      		if (i == 2 || i == 3)
      		{
      		  String kennung= _data.getTextValue(_parItemCombo [i]).getText();
	         	Combo cb = (Combo) _comboPar [i];
	      	
	          for (int k = 0; k < cb.getItemCount(); k++)
	          {
	            if (kennung.equals(cb.getItem( k )))
	              cb.select(k);      
	          }
      		}
      	}

        /* 
         * AttributeGroup fr die Range der Spinner auswerten
         */
      	
        AttributeGroup attG = _result.getDataDescription().getAttributeGroup();
        
        Data itemPar = _data;
        for (int j = 0; j < _parItemSpin.length; j++)
        {
//          SpinnerAddOns.setSpinnerMinMax( attG, new String[]{"1Fahrstreifen"}, _parItemSpin[j], _spinnerPar[j] );
          SpinnerAddOns.setSpinnerTooltip( _spinnerPar[j], null );
          updateSpinner( itemPar, _parItemSpin[j], _spinnerPar[j] );          
        }
      }
    }

    getTvParameter().refresh();
    layout(true);
  }

  /**
   * Spinner aus dem DAV-Datensatz aktualisieren
   * 
   * @param itemPar DAV-Datensatz
   * @param item    Parameter-Item aus dem DAV-Datensatz
   * @param spin    Spinner
   */
  private void updateSpinner( Data itemPar, String item, Spinner spin )
  {
    if (spin.getDigits() != 0)
    	updateSpinnerFloat ( itemPar, item, spin);
    else
    	updateSpinnerInt ( itemPar, item, spin);
  }

  /**
   * Spinner aus dem DAV-Datensatz aktualisieren
   * 
   * @param itemPar DAV-Datensatz
   * @param item    Parameter-Item aus dem DAV-Datensatz
   * @param spin    Spinner
   */
  private void updateSpinnerFloat ( Data itemPar, String item, Spinner spin )
  {
  	float f0 = itemPar.getScaledValue(item).floatValue();
  	
    int digits = spin.getDigits();
	  int iWert = (int) (f0 * (int) Math.pow(10, digits));
	  spin.setSelection(iWert);
  }

  /**
   * Spinner aus dem DAV-Datensatz aktualisieren
   * 
   * @param itemPar DAV-Datensatz
   * @param item    Parameter-Item aus dem DAV-Datensatz
   * @param spin    Spinner
   */
  private void updateSpinnerInt( Data itemPar, String item, Spinner spin )
  {
    String t0 = itemPar.getItem( item ).asTextValue().getValueText();
    
    int i0;
    try
    {
      i0 = new Integer( t0 );
    }
    catch (Exception e)
    {
      i0 = 0;
    }
    spin.setSelection( i0 );
  }


  /**
   * Methode zum Anlegen des Composite mit den Parametern
   */
  private void createGroupParameter()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(fillLayout2);
    _groupParameter.setLayoutData(gridData2);
    
    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    _compositeTabelle = new Composite(_groupParameter, SWT.NONE);
    _compositeTabelle.setLayout(gridLayout);

    erzeugeTabelle();
  }

  /**
   * Tabelle fr die Parameter erzeugen
   */
  private void erzeugeTabelle()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL; 

    _composite3 = new Composite(_compositeTabelle, SWT.None);
    _composite3.setLayout(fillLayout);
    _composite3.setLayoutData( gridData );
    
    _tvParameter = new TableViewer(_composite3, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvParameter.setContentProvider(new TableContentProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column1.getColumn().setWidth(150);
    column1.getColumn().setText("Attribut");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column2.getColumn().setWidth(250);
    column2.getColumn().setText("Wert");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column3.getColumn().setWidth(100);
    column3.getColumn().setText("Einheit");
    column3.setLabelProvider(new MyCellLabelProvider());

    /*
     * Zeilenhhe anpassen, nachdem die Daten eingegangen sind
     */
    _tvParameter.getTable().addListener(SWT.MeasureItem, new Listener() 
    {
      @Override
      public void handleEvent(Event event)
      {
        event.height = 29;
      }
    });

    _tvParameter.getTable().setLinesVisible(true);
    _tvParameter.getTable().setHeaderVisible(true);
  }

  /**
   * Contentprovider fr die Tabelle
   * @author ObertM
   *
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    /*
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;
    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {

    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Tabellenmodell fr die Parameter erzeugen
   * @param table Tabelle
   * @return TabellenModell
   */
  private TableModel[] createModel( Table table )
  {
    TableModel[] elements = new TableModel[_parItemSpin.length + _parItemCombo.length];

    /*
     * Richtungen bestimmen
     */
    List<String> jaNein = Arrays.asList("Ja", "Nein");

    /*
     * ComboBoxen erzeugen
     */
    _comboPar = new Combo [_parItemCombo.length];

    elements[0] = createComboElement(table, 0, _parItemCombo[0], Einheit.KEIN, _listMoeglicheTrigger );
    elements[1] = createComboElement(table, 1, _parItemCombo[1], Einheit.KEIN, _listMoeglicheAspekte );
    elements[2] = createComboElement(table, 2, _parItemCombo[2], Einheit.KEIN, jaNein );
    elements[3] = createComboElement(table, 3, _parItemCombo[3], Einheit.KEIN, jaNein );

    /*
     * Spinner anlegen
     */
    _spinnerPar = new Spinner[_parItemSpin.length];
    
//    elements[3] = createModelElement( table, 0, _parItemSpin[0], Einheit.MS );
//    elements[4] = createModelElement( table, 1, _parItemSpin[1], Einheit.MS );

    return elements;
  }

  /**
   * ein Parameter-Element des TabellenModells erzeugen
   * 
   * @param table       Tabelle
   * @param intCombo    Index der ComboBox
   * @param item        Bezeichnung des Parameters
   * @param einheit     Einheit des Parameters
   * @param anzeigeQuerschnitte       mgliche Werte der ComboBox
   * @return
   */
  private TableModel createComboElement( Table table, int intCombo, String item, Einheit einheit, List<String> anzeigeQuerschnitte )
  {
  	/*
  	 * Composite 0: Beschriftung 
  	 */
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    
    Label label = new Label(comp0, SWT.NONE);
    label.setText( item );

    /*
     * Composite 1: Combobox
     */
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    _comboPar[intCombo] = new Combo( comp1, SWT.BORDER );
    _comboPar[intCombo].setLayoutData( gdTableItem );
    
    String[] t = new String [anzeigeQuerschnitte.size()];
    for (int i=0; i<anzeigeQuerschnitte.size(); ++i)
    	t[i] = anzeigeQuerschnitte.get(i);
    	
    _comboPar[intCombo].setItems(t);
    _comboPar[intCombo].select(0);

    /*
     *  Composite 2: Einheit
     */
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessVerticalSpace = true;

    Composite comp2 = new Composite(table, SWT.NONE);
    comp2.setBackground(table.getBackground());
    comp2.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp2.setLayoutData(gd2);
    comp2.setLayout(new GridLayout());
    Label label2 = new Label(comp2, SWT.NONE);
    label2.setText("");
    
    return new TableModel(comp0, comp1, comp2);
  }

  /**
   * ein Parameter-Element des TabellenModells erzeugen
   * 
   * @param table       Tabelle
   * @param intSpinner  Index des Spinners
   * @param item        Bezeichnung des Parameters
   * @param einheit     Einheit des Parameters
   * @return
   */
  private TableModel createModelElement( Table table, int intSpinner, String item, Einheit einheit )
  {
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    Label label = new Label(comp0, SWT.NONE);
    label.setText( item );
    
    Composite comp1 = new Composite(table, SWT.NONE);
    comp1.setBackground(table.getBackground());
    comp1.setBackgroundMode(SWT.INHERIT_DEFAULT);
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;
    comp1.setLayoutData(gd1);
    comp1.setLayout(new GridLayout());
    _spinnerPar[intSpinner] = new Spinner(comp1, SWT.BORDER);
    // zum Ausfllen der Zelle
    GridData gdSpinner = new GridData();
    gdSpinner.horizontalAlignment = GridData.FILL;
    gdSpinner.grabExcessHorizontalSpace = true;
    gdSpinner.verticalAlignment = GridData.CENTER;
    _spinnerPar[intSpinner].setLayoutData(gdSpinner);
    
    Composite comp2 = new Composite(table, SWT.NONE);
    comp2.setBackground(table.getBackground());
    comp2.setBackgroundMode(SWT.INHERIT_DEFAULT);
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessVerticalSpace = true;
    comp2.setLayoutData(gd2);
    comp2.setLayout(new GridLayout());
    Label label2 = new Label(comp2, SWT.NONE);

    //Erstinitialisierung, wird aus udateData heraus mit den aktuellen Daten aus dem DAV berschrieben
    switch ( einheit )
    {
//	    case MS:
//	      _spinnerPar[intSpinner].setMinimum(0);
//	      _spinnerPar[intSpinner].setMaximum(1000);
//	      _spinnerPar[intSpinner].setDigits(1);
//	      
//	      label2.setText( "Km/h" );      
//	      break;
	      
	    default:
	      break;
    }
    
    SpinnerAddOns.setSpinnerTooltip( _spinnerPar[intSpinner], null );
    
    return new TableModel(comp0, comp1, comp2);
  }

  /**
   * Methode liefert den TableViewer
   * @return TableViewer
   */
  public TableViewer getTvParameter()
  {
    return _tvParameter;
  }

  /**
   * Methode liefert den Spinner zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Spinner getSpinner(int index)
  {
    return _spinnerPar [index];
  }

  /**
   * Methode liefert die ComboBox zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Combo getComboBox (int index)
  {
    return _comboPar [index];
  }

  /**
   * Methode liefert den Namen eines Items eines Spinners zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemSpinnerName (int index)
  {
  	return _parItemSpin [index];
  }

  /**
   * Methode liefert den Namen eines Items einer ComboBox zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemComboBoxName (int index)
  {
  	return _parItemCombo [index];
  }

  /**
   * Labelprovider
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }


      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getBezeichnung();
          break;
        case 1:
          comp = model.getAttribut();
          break;
        case 2:
          comp = model.getEinheit();
          break;
        default:
      }
      TableEditor editor = new TableEditor(item.getParent());
      
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);
    }
  }
  
  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlSpinnerItems ()
  {
  	return _parItemSpin.length;
  }

  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlComboBoxItems ()
  {
  	return _parItemCombo.length;
  }
}
