/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.naessewarnung;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Messquerschnitte
   */
  private ResultData _resultDataMQ;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Umfelddatenmesstellen
   */
  private ResultData _resultDataUFD;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherMQ;

  /**
   * Liste mit den mglichen Umfelddatenmessstellen
   */
  private List<String> _listeMoeglicherUFD;

	/**
	 * Liste mit den mglichen Tag/Nacht Sensoren
	 */
	private List<String> _listeMoeglicherTNS;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit den Reitern
   */
  private CTabFolder _tabFolder = null;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmus[] _compositeAlg;

	/**
	 * Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
	private boolean _auswahlMQ;

	/**
	 * Instanz zur Auswahl der Messquerschnitte
	 */
	private AuswahlObjekte _auswahlObjekteMQ;

	/**
	 * Instanz zur Auswahl der Umfelddatenmessstellen
	 */
	private AuswahlObjekte _auswahlObjekteUFD;

	/**
	 * Instanz zur Auswahl des Tag/Nacht Sensors
	 */
	private AuswahlObjekt _auswahlTNS;

	private Group _groupParameter;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataUFD Ergebnis Umfelddatenmesstellen 
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheMQ Liste mit den mglichen Messquerschnitten
	 * @param listeMoeglicherUFD Liste mit den mglichen Umfelddatenmessstellen
	 * @param listeMoeglicherTNS Liste mit den mglichen Tag/Nacht Sensoren
	 * @param auswahlMQ Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataMq, ResultData resultDataUFD, ResultData resultDataAlg, List<String> listMoeglicheMQ, List<String> listeMoeglicherUFD, List<String> listeMoeglicherTNS, boolean auswahlMQ)
  {
    super(parent, style);
    
    this._resultDataMQ  = resultDataMq;
    this._resultDataUFD = resultDataUFD;
    this._resultDataAlg = resultDataAlg;
    
    this._auswahlMQ          = auswahlMQ;
    
    this._listeMoeglicherMQ  = listMoeglicheMQ;
    
    this._listeMoeglicherUFD = listeMoeglicherUFD;
    
    this._listeMoeglicherTNS = listeMoeglicherTNS;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    if (_auswahlMQ)
    {
    	_auswahlObjekteMQ  = new AuswahlObjekte(_composite, "Messquerschnitte");
    	_auswahlObjekteUFD = new AuswahlObjekte(_composite, "Umfelddatenmessstellen");
    	_auswahlTNS        = new AuswahlObjekt (_composite, "Tag/Nacht Sensor");
    }
    
    createCompositeTab();
    
  	_compositeAlg = new CompositeParameterAlgorithmus [1];

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   * @param resultData 
   */
  public synchronized void update (ResultData resultDataMq, ResultData resultDataUFD, ResultData resultDataAlg)
  {
    this._resultDataMQ  = resultDataMq;
    this._resultDataUFD = resultDataUFD;
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
   * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void updateDataAlg (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updataDataAlgorithmus ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updataDataAlgorithmus ();
    }

    if (_resultDataMQ != null)
    {
    	if (_auswahlMQ)
        updateDataMessQuerschnitt();
    }
    
    if (_resultDataUFD != null)
    {
    	if (_auswahlMQ)
        updateDataUFD();
    }
  }
  
  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    Composite c = _groupParameter;
    Control[] children = c.getChildren();
    CompositeParameterAlgorithmus compPar = null;

    Data data = null;
    if (_resultDataAlg.hasData())
      data = _resultDataAlg.getData();
    
    if ( children.length == 0 )
    {
      compPar = new CompositeParameterAlgorithmus ( c, SWT.None, _resultDataAlg, data );
      _compositeAlg [0] = compPar;
      
      compPar.layout( true );
      c.layout( true );
      
      compPar.getTvParameter().refresh();
    }
    else
    {
      compPar = (CompositeParameterAlgorithmus) children[0];
      compPar.update ( _resultDataAlg, data );
      
      compPar.layout( true );
      c.layout( true );

      compPar.getTvParameter().refresh();
    }
  }

  /**
   * Updatefunktion ber die Messquerschnitte
   */
	private void updateDataMessQuerschnitt()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
    if ( _resultDataMQ != null)
    {
      if ( _resultDataMQ.hasData() )
      {
      	Data data = _resultDataMQ.getData();
      	
        Array arrayMQ = data.getArray( "MessQuerschnitt" );
        for (int k = 0; k < arrayMQ.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayMQ.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());
          
          angewaehlteObjekte.add(text);
        }
      }
    }
    
    _auswahlObjekteMQ.setAngewaehlteObjekte(_listeMoeglicherMQ, angewaehlteObjekte);
  }

  /**
   * Updatefunktion ber die Umfelddaten
   */
	private void updateDataUFD()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
		String tagNachtSensor = Tools._undefiniert;
		
    if ( _resultDataUFD != null)
    {
      if ( _resultDataUFD.hasData() )
      {
      	Data data = _resultDataUFD.getData();
      	
        Array arrayUFD = data.getArray( "UmfeldDatenMessStelle" );
        for (int k = 0; k < arrayUFD.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayUFD.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());
          
          angewaehlteObjekte.add(text);
        }
        
        SystemObject so = data.getReferenceValue("UmfeldDatenSensorTagNacht").getSystemObject();
        if (so != null)
        	tagNachtSensor = Tools.getName(so);
      }
    }
    
    _auswahlObjekteUFD.setAngewaehlteObjekte(_listeMoeglicherUFD, angewaehlteObjekte);
    
    _auswahlTNS.setAngewaehltesObjekt(_listeMoeglicherTNS, tagNachtSensor);
  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(new GridLayout(1, false));
    _groupParameter.setLayoutData(gridDataGroup);
  }

  /**
   * Methode liefert die Liste der angewhlten Messquerschnitte
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteMQ()
  {
    return _auswahlObjekteMQ.getZugeordneteObjekte();
  }

  /**
   * Methode liefert die Liste der angewhlten Umfelddatenmessstellen
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteUFD()
  {
    return _auswahlObjekteUFD.getZugeordneteObjekte();
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmus[] getCompositeAlg()
  {
  	return _compositeAlg;
  }

  /**
   * Methode liefert den Tag/Nach Sensor
   * @return Tag/Nach Sensor
   */
	public String getZugeordneterTNS()
  {
	  return _auswahlTNS.getAusgewaehltesObjekt();
  }
  
}
