/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class ProjektSpezifischeEinstellungen
{
	/**
	 * Enum zum Kodieren der unterschiedlichen SBA
	 * @author ObertM
	 *
	 */
	public enum SBA
	{
		/**
		 * bisher nicht bercksichtigte SBA
		 */
	  UNBEKANNT,
	  
	  /**
	   * SBA A14
	   */
	  A14,
	  
	  /**
	   * SBA B27
	   */
	  B27,

	  /**
	   * SBA A8
	   */
	  A8,
	};
	
	/**
	 * Pid der SBA B27
	 */
	private static String _pidSBAB27 = "bw.uz.B27.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Pid der SBA A14
	 */
	private static String _pidSBAA14 = "st.sba.uz.a14.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Pid der SBA A8
	 */
	private static String _pidSBAA8 = "bw.uz.A8.sba_stgt.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Aktuelle Streckenbeeinflussungsanlage
	 */
	private SBA _sba;
	
	/**
	 * Pid der SBA fr die das Plugin arbeitet
	 */
	private String _pidSba;
	
	/**
	 * Einzige Instanz der Klasse
	 */
	private static ProjektSpezifischeEinstellungen _instance = null;
	
	/**
	 * Privater Konstruktor der Klasse
	 */
	private ProjektSpezifischeEinstellungen()
  {
		bestimmeSBA ();
  }
	
	/**
	 * Methode zum Bestimmen der SBA
	 */
	private void bestimmeSBA()
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    List<SystemObject> imDaVVorhandeneSBA = new ArrayList<SystemObject>();
    List<SystemObject> tmp = connection.getDataModel().getType( "typ.sswSbaStreckenBeeinflussungsAnlage" ).getElements();
    for (SystemObject so : tmp)
    	if (!so.getPid().startsWith("swarco.pua"))
    		imDaVVorhandeneSBA.add(so);
    
    /*
     * Prfen ob mehrere SBA im DaV versorgt sind
     */
    if (imDaVVorhandeneSBA.size() == 1)
    	_pidSba = imDaVVorhandeneSBA.get(0).getPid();
    else
    {
    	/*
    	 * Falls mehrere vorhanden sind, prfen ob eine der vordefinierten SBA
    	 * vorhanden ist, falls ja wird die erste bekannte SBA verwendet.
    	 */

      for (SystemObject so : imDaVVorhandeneSBA)
      {
        if ( so.getPid().equals(_pidSBAA14) ||
        		 so.getPid().equals(_pidSBAB27) ||
        		 so.getPid().equals(_pidSBAA8))
        {
        	_pidSba = so.getPid();
        	break;
        }
      }
    }
    
    _sba = SBA.UNBEKANNT;
    
    if (_pidSba.equals(_pidSBAA14))
     	_sba = SBA.A14;

    if (_pidSba.equals(_pidSBAB27))
    	_sba = SBA.B27;

    if (_pidSba.equals(_pidSBAA8))
    	_sba = SBA.A8;

    System.out.println("ProjektSpezifischeEinstellungen.bestimmeSBA() Aktuelle SBA: " + _sba);
  }

	/**
	 * Methode liefert die einzige Instanz der Klasse
	 * @return
	 */
	public static ProjektSpezifischeEinstellungen getInstance ()
	{
  	if (_instance == null)
  	{
  		synchronized (ProjektSpezifischeEinstellungen.class)
      {
  			if (_instance == null)
  			{
  				_instance = new ProjektSpezifischeEinstellungen();
  			}
      }
  	}
    return _instance;
	}
	
	/**
	 * Methode liefert den Praefix fr die Pids der Ursacheneinheiten 
	 * @return
	 */
	public String getPraefixUrsachenEinheit ()
	{
	  String praefix = "sbaUrsachenEinheit";
	  
	  if (_sba == SBA.B27)
	  	praefix = "sbaB27UrsachenEinheit";

	  if (_sba == SBA.A14)
	  	praefix = "sbaA14UrsachenEinheit";

	  if (_sba == SBA.A8)
	  	praefix = "sbaA8StgtUrsachenEinheit";

	  return praefix;
	}

	/**
   * Methode liefert pidSba zurck
   * @return pidSba
   */
  public String getPidSba()
  {
  	return _pidSba;
  }

	/**
   * Methode liefert sba zurck
   * @return sba
   */
  public SBA getSba()
  {
  	return _sba;
  }

}
