/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.stau;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Darstellung des Fensters zur Parameternderung der Parameter Staukriterium 3 (Strfallerkennung)
 * @author ObertM
 */
public class CompositeParameterStauStoerfall extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Bewertung
   */
  private ResultData _resultDataBewertung;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit den Reitern fr die Fahrstreifenspezifischen Parameter
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterStauStoerfallProFahrstreifen[] _compositeAlg;

  /**
   * Composite Bewertung
   */
	private CompositeBewertung _bewertung;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param resultDataBewertung Ergebnis Bewertung
	 */
  CompositeParameterStauStoerfall(Composite parent, int style, ResultData resultDataAlg, ResultData resultDataBewertung)
  {
    super(parent, style);
    
    this._resultDataAlg = resultDataAlg;
    
    this._resultDataBewertung = resultDataBewertung;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    createCompositeTab();
    
    _bewertung = new CompositeBewertung();
    _bewertung.createGroupBewertung(_composite);

    _list = new ArrayList<String>();
    _list.add("2 Fahrstreifen");
    _list.add("3 Fahrstreifen");

    _compositeAlg = new CompositeParameterStauStoerfallProFahrstreifen [_list.size()];
    
    erzeugeTabItems(_list);

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   * @param resultDataBewertung 
   */
  public synchronized void update (ResultData resultDataAlg, ResultData resultDataBewertung)
  {
    this._resultDataAlg = resultDataAlg;
    
    this._resultDataBewertung = resultDataBewertung;
    
    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    	updataDataAlgorithmus ();

    if (_resultDataBewertung != null)
    	updataDataBewertung ();
  }

  /**
   * Updatefunktion fr die Bewertung
   */
  private void updataDataBewertung ()
  {
    Data data = null;
    if (_resultDataBewertung.hasData())
    {
      data = _resultDataBewertung.getData();
      
      Data d1 = data.getItem("Staukriterium3");
      _bewertung.update (d1);
    }
  }

  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    Data data = null;
    if (_resultDataAlg.hasData())
    {
      data = _resultDataAlg.getData();

	    /*
	     * Loop ber die Fahrstreifen	
	     */
	  
	  	int index = 0;
	  	for (String s : _list)
	  	{
	      /*
	       * Reiter ermitten
	       */
	
	  		CTabItem[] items = _tabFolder.getItems();
	      CTabItem item = null;
	      
        for (CTabItem ti : items)
	      {
	        String pid = (String) ti.getData( ti.getText() );
	        if ( pid.equals(s) )
	       	 item = ti;
	      }
	
	      if ( item != null )
	      {
          Composite c = (Composite) item.getControl();
	        Control[] children = c.getChildren();
	        CompositeParameterStauStoerfallProFahrstreifen compPar = null;

	        String kennung = item.getText().replace(" Fahrstreifen", "Fahrstreifen");
          Data aktuell = data.getItem(kennung);
          
          if ( children.length == 0 )
          {
            compPar = new CompositeParameterStauStoerfallProFahrstreifen ( c, SWT.None, _resultDataAlg, aktuell );
            _compositeAlg [index] = compPar;
            
            compPar.layout( true );
            c.layout( true );
            
            compPar.getTvParameter().refresh();
          }
          else
          {
            compPar = (CompositeParameterStauStoerfallProFahrstreifen) children[0];
            compPar.update ( _resultDataAlg, aktuell );
            
            compPar.layout( true );
            c.layout( true );

            compPar.getTvParameter().refresh();
          }
          
          index++;
        }
      }
  	
  	  /*
  	   * Gte
  	   */
  	
   	  float f = data.getScaledValue("Gte").floatValue();
	  	_bewertung.setGuete(f);
    }
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 300;

    Group groupParameter = new Group(_composite, SWT.NONE);
    groupParameter.setText("Parameter");
    groupParameter.setLayout(new GridLayout(1, false));
    groupParameter.setLayoutData(gridDataGroup);
  
    _tabFolder = new CTabFolder(groupParameter, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        if (((Composite)cSel).getChildren().length > 0)
        {
        	CompositeParameterStauStoerfallProFahrstreifen compParSel = (CompositeParameterStauStoerfallProFahrstreifen)cSel.getChildren()[0];
        
          TableViewer tvParameter = compParSel.getTvParameter();
          tvParameter.refresh();
        }
      }
    });
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeStauKriterium ()
  {
    new CompositeParameterStauStoerfallProFahrstreifen ( _composite, SWT.None, _resultDataAlg, null );
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  	{
      String so = list.get(i);
     
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = so;
      tabItem.setText( tabName );
      tabItem.setData( tabName, so );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterStauStoerfallProFahrstreifen[] getCompositeParameterStauBeleg ()
  {
  	return _compositeAlg;
  }

	/**
	 * Methode zum bernehmen der Daten in den DaV
	 * @param data Datenstruktur in die die Daten geschrieben werden sollen
	 */
	public void getData(Data data)
  {
		CTabItem[] items = _tabFolder.getItems();
		for (int i=0; i<items.length; ++i)
		{
	    CTabItem cTab = items [i];

	    String kennung = cTab.getText().replace(" Fahrstreifen", "Fahrstreifen");
	    Data aktuell = data.getItem(kennung);
	    
	    Composite c = (Composite) cTab.getControl();
	    Control[] children = c.getChildren();
	    
	    CompositeParameterStauStoerfallProFahrstreifen cAlg = (CompositeParameterStauStoerfallProFahrstreifen) children[0];

	    if (cAlg != null)
	    {
	      for (int j=0; j<cAlg.getAnzahlItems(); ++j)
	      {
	        String item = cAlg.getItemName(j);
	  
	        Spinner sp = cAlg.getSpinner(j);
	        	
	        Integer i1 = sp.getSelection();
	        
	        if (cAlg.getSpinner(j).getDigits() == 0)
	        {
	          aktuell.getItem( item ).asScaledValue().set( i1 );
	        }
	        else
	        {
	        	float f1 = getWert(sp);
	        	aktuell.getScaledValue(item).set(f1);
	        }
	      }
	    }
		}
		
		/*
		 * Gte
		 */

		float guete = _bewertung.getGuete();
  	data.getScaledValue("Gte").set(guete);
  }
	
	/**
	 * Methode zum Aktualisieren der Folder
	 */
	public void refreshTabelle()
  {
    /*
     * Loop ber die Fahrstreifen	
     */
  
  	for (String s : _list)
  	{
      /*
       * Reiter ermitten
       */

  		CTabItem[] items = _tabFolder.getItems();
      CTabItem item = null;
      
      for (CTabItem ti : items)
      {
        String pid = (String) ti.getData( ti.getText() );
        if ( pid.equals(s) )
       	 item = ti;
      }

      if ( item != null )
      {
        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterStauStoerfallProFahrstreifen compPar = null;

        if ( children.length != 0 )
        {
          compPar = (CompositeParameterStauStoerfallProFahrstreifen) children[0];
          
          compPar.layout( true );
          c.layout( true );

          compPar.getTvParameter().refresh();
        }
      }
    }
  }

	/**
   * Methode liefert bewertung zurck
   * @return bewertung
   */
  public CompositeBewertung getBewertung()
  {
  	return _bewertung;
  }
  
  /**
   * Methode liefert den Wert einers Spinners
   * @param spinner Spinner
   * @return eingestellter Wert
   */
  private float getWert (Spinner spinner)
  {
    Integer i1 = spinner.getSelection();
    int digits = spinner.getDigits();
    
  	double f1 = (i1 / Math.pow(10, digits));
  	
  	return (float) f1;
  }

  /**
   * Dispose
   */
  @Override
  public void dispose ()
  {
		CTabItem[] items = _tabFolder.getItems();
    
     for (CTabItem item : items)
     {
    	 if (item.isDisposed())
    		 continue;
    	 
       Composite c = (Composite) item.getControl();
       if (c.isDisposed())
      	 continue;
       
       Control[] children = c.getChildren();

       if ( children.length != 0 )
       {
      	 CompositeParameterStauStoerfallProFahrstreifen compPar = (CompositeParameterStauStoerfallProFahrstreifen) children[0];
        if (!compPar.isDisposed())
          compPar.dispose ();
      }
    }
  	
  	super.dispose();
  }

}
