/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.Perspective;
import org.eclipse.ui.internal.WorkbenchPage;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.autarkprogramm.ViewSchaltbildVersorgungAutarkProgramm;
import de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen.EinzelmassnahmenViewParameter;
import de.bsvrz.buv.plugin.ursacheneinheit.messquerschnitte.ViewParameterMessQuerschnitte;
import de.bsvrz.buv.plugin.ursacheneinheit.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.buv.plugin.ursacheneinheit.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen.SBA;
import de.bsvrz.buv.plugin.ursacheneinheit.sbaueparam.SbaUrsachenEinheitViewParameter;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.buv.plugin.ursacheneinheit.umfelddaten.ViewParameterUmfeldDaten;
import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * In diesem Dialog sind die aktuell definierten Eintrge zu den Automatikprogrammen dargestellt. 
 * Fr jeden EintragUrsachenEinheit ist die Eigenschaft Name und Pid dargestellt.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @author Krause
 * 
 */
public class VerwaltungsTabelleComp extends Composite
{
  public static final String ID = "de.bsvrz.buv.plugin.automatikprogramme.verwaltung.verwaltungsTabelleComp";
  
  private final String _alle = "Alle Ursacheneinheiten";
  
  private final String _defaultText = "Suchkriterium eingeben";
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private ConfigurationObject _konfigObj = null;
  
  /**
   * Verbindung zum DaV
   */
  private DaVKommunikation _daVKommunikation = null;

  /**
   * TableViewer
   */
  private TableViewer _tvUrsachenEinheit = null;

  /**
   * Gibt den selektierten EintragUrsachenEinheit an.
   */
  private EintragUrsachenEinheit[] _eintragUrsachenEinheit = null;

  /**
   * Speichert die UrsachenEinheitDatenstze fr die tabellarische Anzeige.
   */
  private List<EintragUrsachenEinheit> _ursachenEinheitDatensaetze = null;

  private Composite _composite = null;

  private Composite _compositeTop = null; 

  private Composite _compositeRight = null;

  private Composite _compositeTable = null;

  private Button buttonLoeschen = null;

  private Button buttonKopieren = null;

  private Button buttonUEMQAnlegen = null;

  private Button buttonUEUFDAnlegen = null;

  private Button buttonParameterBearbeiten = null;

  private Button buttonParameterBearbeitenInEinemReiter = null;

  private Button buttonZuordnungMessquerschnitte = null;

  private Button buttonZuordnungUmfelddaten = null;

  private Button buttonParameterUEBearbeiten = null;

  private Button buttonSchaltBild = null;

  private Button buttonNeu = null;

  /**
   * Der Listener fr die Berechtigungen
   */
  private IOberflaechenBerechtigungZuhoerer _berechtigungsZuhoerer = null;

  /**
   * Liste der GUI-Elemente fr das Berechtigungskonzept
   */
  private Map<String, Object> _oberflaechenElemente = new HashMap<String, Object>();
  
  /**
   * Instanz der Kommandozeilenparameter
   */
  private Kommandozeilenparameter _kommandoZeilenParameter = Kommandozeilenparameter.getInstanz();

	private Composite _compositeFilter;

	private Combo _comboObjTypFilter;

	private Text _textNamen;

	/**
	 * Liste mit den Pid der Typen der Ursacheneinheiten fr die eine automatische Generierung der
	 * Ursacheneinheiten MQ mglich ist
	 */
	private List<String> _ursachenEinheitenMQAutomatisch = Arrays.asList
	(
			"typ.sswSbaUeHarmoBelastungUnruhe120",
			"typ.sswSbaUeHarmoBelastung100",
			"typ.sswSbaUeHarmoBelastung80",
			"typ.sswSbaUeHarmoBelastung60",
			"typ.sswSbaUeHarmoGeschwDiff120",
			"typ.sswSbaUeHarmoGeschwDiff100",
			"typ.sswSbaUeHarmoGeschwDiff80",
			"typ.sswSbaUeStau",
			"typ.sswSbaUeErweiterterStaualgorithmus",
			"typ.sswSbaUeLkwberholverbotBelastung",
			"typ.sswSbaUeLkwberholverbotNsse",
			"typ.sswSbaUeLkwberholverbotSicht",
			"typ.sswSbaUeAbstandswarnung1",
			"typ.sswSbaUeAbstandswarnung2"
	);

	/**
	 * Liste mit den Pid der Typen der Ursacheneinheiten fr die eine automatische Generierung der
	 * Ursacheneinheiten UFD mglich ist
	 */
	private List<String> _ursachenEinheitenUFDAutomatisch = Arrays.asList
	(
			"typ.sswSbaUeNssewarnung",
			"typ.sswSbaUeNebelwarnung",
			"typ.sswSbaUeGlttewarnung"
	);

	/**
	 * Liste mit den Pid der Typen der Ursacheneinheiten fr die Versorgung mehrerer Ursacheneinheiten
	 * in einem Reiter implementiert ist 
	 */
	String [] _ursachenEinheitenFuerVersorgungInEinemReiter = new String[] 
  {
			"typ.sswSbaUeStau",
			"typ.sswSbaUeHarmoBelastungUnruhe120",
			"typ.sswSbaUeHarmoBelastung100",
			"typ.sswSbaUeHarmoBelastung80",
			"typ.sswSbaUeHarmoBelastung60",
			
			"typ.sswSbaUeHarmoGeschwDiff120",
			"typ.sswSbaUeHarmoGeschwDiff100",
			"typ.sswSbaUeHarmoGeschwDiff80",
			
			"typ.sswSbaUeLkwberholverbotBelastung",
			"typ.sswSbaUeLkwberholverbotNsse",
			"typ.sswSbaUeLkwberholverbotSicht",
			"typ.sswSbaUeAbstandswarnung1",
			"typ.sswSbaUeAbstandswarnung2",
			
			"typ.sswSbaUeNssewarnung",
			"typ.sswSbaUeNebelwarnung",
			"typ.sswSbaUeLrm",
			
			"typ.sswSbaUeErweiterterStaualgorithmus",
			"typ.sswSbaUeTemposplit",
			
			"typ.sswSbaUeGlttewarnung",
	};

	/**
	 * Liste mit den Pid der Typen der Ursacheneinheiten die eine Zuordnung Messquerschnitte besitzen  
	 */
	String [] _ursachenEinheitenMitMQZuordnung= new String[] 
  {
			"typ.sswSbaUeStau",
			"typ.sswSbaUeHarmoBelastungUnruhe120",
			"typ.sswSbaUeHarmoBelastung100",
			"typ.sswSbaUeHarmoBelastung80",
			"typ.sswSbaUeHarmoBelastung60",
			
			"typ.sswSbaUeHarmoGeschwDiff120",
			"typ.sswSbaUeHarmoGeschwDiff100",
			"typ.sswSbaUeHarmoGeschwDiff80",
			
			"typ.sswSbaUeLkwberholverbotBelastung",
			"typ.sswSbaUeLkwberholverbotNsse",
			"typ.sswSbaUeLkwberholverbotSicht",
			
			"typ.sswSbaUeAbstandswarnung1",
			"typ.sswSbaUeAbstandswarnung2",
			
			"typ.sswSbaUeNssewarnung",
			"typ.sswSbaUeLangsameFahrzeuge",
			"typ.sswSbaUeSeitenstreifenFreigabeBelegung",
			"typ.sswSbaUeSeitenstreifenFreigabeBelastung",
	};

	/**
	 * Liste mit den Pid der Typen der Ursacheneinheiten die eine Zuordnung Umfelddaten besitzen  
	 */
	String [] _ursachenEinheitenMitUFDZuordnung= new String[] 
  {
			"typ.sswSbaUeNssewarnung",
			"typ.sswSbaUeNebelwarnung",
			
			"typ.sswSbaUeLkwberholverbotNsse",
	};

  public VerwaltungsTabelleComp( Composite parent, int style, SystemObject obj )
  {
    super( parent, style );
    _composite = this;
    _konfigObj = (ConfigurationObject)obj;

    init();

  }

  /**
   * This method initializes _composite
   */
  private void init()
  {
    _composite.setLayout( new GridLayout() );

    // Darstellung aufbauen

//    createCompositeFilter();

    createCompositeTop();
    createCompositeTable();
    createCompositeRight();

    KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter( "-berechtigungsKonzept" );
    if ( kp.getWert().equals( "true" ) )
    {
      erzeugeOberflaechenBerechtigung();
    }

    _daVKommunikation = DaVKommunikation.getInstanz();
    
    _daVKommunikation.anmeldenNeu(_konfigObj);

    _ursachenEinheitDatensaetze = _daVKommunikation.getUrsachenEinheitenEintraege();
    
    /*
     * Sortieren der Ursacheneinheiten nach Namen
     */
	  TreeMap<String, EintragUrsachenEinheit> treeMap = new TreeMap<String, EintragUrsachenEinheit>();
	  for (EintragUrsachenEinheit eue : _ursachenEinheitDatensaetze)
	    treeMap.put(eue.getName(), eue);
	
	  _ursachenEinheitDatensaetze.clear();
	  _ursachenEinheitDatensaetze.addAll(treeMap.values());

    bestimmeGefilterteUrsachenEinheiten();

  }

  /**
   * Definiert einen neuen EintragUrsachenEinheit. Es ffnet sich der Definitionsdialg ({@link UrsachenEinheitDefinitionsdialog})
   * zum Anlegen des Eintrags.
   * 
   */
  private void aktionNeu()
  {
  	ParaVerwaltungDynamischerObjekte.getInstance();
  	
  	int index = _comboObjTypFilter.getSelectionIndex();
  	String nameTyp = _comboObjTypFilter.getItem(index);
  	
  	UrsachenEinheitDefinitionsdialog ueinheitDefinitionsdialog = new UrsachenEinheitDefinitionsdialog(PlatformUI
          .getWorkbench().getDisplay().getActiveShell());
  	
    _eintragUrsachenEinheit = new EintragUrsachenEinheit [1]; 

    if (nameTyp.equals(_alle))
      _eintragUrsachenEinheit [0] = ueinheitDefinitionsdialog.oeffne();
    else
      _eintragUrsachenEinheit [0] = ueinheitDefinitionsdialog.oeffne(nameTyp);
    
    bestimmeZustandButtons();

  }

  /**
   * Bearbeitet der Parameter zum EintragUrsachenEinheit. Es ffnet sich die entsprechende ParameterView 
   * zum Bearbeiten der Parameter.
   * 
   */
  private void aktionBearbeiten(int typ)
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length > 0)
    {
    	String pidObjTyp = _eintragUrsachenEinheit[0].getPidObjTyp();
    	
      /*
       * ParameterView ffnen
       */
      
      if ( pidObjTyp == null || pidObjTyp.length() == 0 )
        Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "Kein Objekttyp zur Parametrisierung vorhanden!") );
      else
      {
        String viewID = Activator.PLUGIN_ID + "." + Activator.ID_PARAMETER + "." + pidObjTyp;
        
        StringBuffer sb = new StringBuffer();
        
        if (typ != 0)
        	sb.append("Typ" + typ + ";");
        
        for (int i = 0; i < _eintragUrsachenEinheit.length; i++)
        {
          String s = _eintragUrsachenEinheit [i].getPid();
          if (i<_eintragUrsachenEinheit.length-1)
            sb.append(s + ";");
          else
            sb.append(s);
        }
        
	      /*
 	       * Sonderfall Autarkprogramm:
	       * Die Parameter des Autarkprogrammes hngen nicht an der Ursacheneinheit sondern an der
	       * Streckenbeeinflussungsanlage
	       */
	     
	      if (pidObjTyp.equals("typ.sswSbaUeAutarkprogramm"))
	      {
	     	 sb = new StringBuffer();
	     	 
	     	 ClientDavInterface connection = DaVKommunikation.getInstanz().getConnection();
	        List<SystemObject> elements = connection.getDataModel().getType( "typ.sswSbaStreckenBeeinflussungsAnlage" ).getElements();
	        
	        if (elements.size() > 0)
	       	 sb.append(elements.get(0).getPid()); 
	      }


        String uePid = sb.toString();

        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();

        try
        {
          
          System.out.println(viewID + " : " + uePid);
          
          page.showView( viewID, uePid, IWorkbenchPage.VIEW_CREATE );
          
          IViewReference ref = page.findViewReference( viewID, uePid );
          Perspective ap = ((WorkbenchPage) page).getActivePerspective();
          if (ap != null)
          {
            if (ref != null)
            {
              ap.getPresentation().detachPart( ref );
              
              IWorkbenchPart wbPart = ref.getPart( false );
              Shell shell = wbPart.getSite().getShell();
              shell.setSize( 650, 730 );
              IViewPart vPart = ref.getView( true );              
            }
          }
          else
          {
            System.out.println("Perspective ist null!");
          }
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
          Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "ParameterView zur Ursacheneinheit " + pidObjTyp + " nicht vorhanden!") );
        }
      }
    }
    else
    {
      MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb1.setText( "Fehler" );
      mb1.setMessage( "Es muss ein Eintrag Automatikporgramm selektiert werden!" );
      mb1.open();

    }

    bestimmeZustandButtons();
  }

  /**
   * Aktion zum Versorgen der Messquerschnittzuordnung
   */
  private void aktionZuordnungMessquerschnitte ()
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length > 0)
    {
    	String pidObjTyp = _eintragUrsachenEinheit[0].getPidObjTyp();
    	
      /*
       * ParameterView ffnen
       */
      
      if ( pidObjTyp == null || pidObjTyp.length() == 0 )
        Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "Kein Objekttyp zur Parametrisierung vorhanden!") );
      else
      {
        String viewID = ViewParameterMessQuerschnitte.ID; 
        
        StringBuffer sb = new StringBuffer();
       
      	sb.append("Typ1;");

        for (int i = 0; i < _eintragUrsachenEinheit.length; i++)
        {
          String s = _eintragUrsachenEinheit [i].getPid();
          if (i<_eintragUrsachenEinheit.length-1)
            sb.append(s + ";");
          else
            sb.append(s);
        }
        
        String uePid = sb.toString();

        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();

        try
        {
          System.out.println(viewID + " : " + uePid);
          
          page.showView( viewID, uePid, IWorkbenchPage.VIEW_CREATE );
          
          IViewReference ref = page.findViewReference( viewID, uePid );
          Perspective ap = ((WorkbenchPage) page).getActivePerspective();
          if (ap != null)
          {
            if (ref != null)
            {
              ap.getPresentation().detachPart( ref );
              
              IWorkbenchPart wbPart = ref.getPart( false );
              Shell shell = wbPart.getSite().getShell();
              shell.setSize( 650, 730 );
              IViewPart vPart = ref.getView( true );              
            }
          }
          else
          {
            System.out.println("Perspective ist null!");
          }
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
          Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "ParameterView zur Ursacheneinheit " + pidObjTyp + " nicht vorhanden!") );
        }
      }
    }
    else
    {
      MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb1.setText( "Fehler" );
      mb1.setMessage( "Es muss ein Eintrag Automatikporgramm selektiert werden!" );
      mb1.open();

    }

    bestimmeZustandButtons();
  }

  /**
   * Aktion zum Versorgen der Umfelddaten Zuordnung
   */
  private void aktionZuordnungUmfelddaten ()
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length > 0)
    {
    	String pidObjTyp = _eintragUrsachenEinheit[0].getPidObjTyp();
    	
      /*
       * ParameterView ffnen
       */
      
      if ( pidObjTyp == null || pidObjTyp.length() == 0 )
        Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "Kein Objekttyp zur Parametrisierung vorhanden!") );
      else
      {
        String viewID = ViewParameterUmfeldDaten.ID; 
        
        StringBuffer sb = new StringBuffer();
       
      	sb.append("Typ1;");

        for (int i = 0; i < _eintragUrsachenEinheit.length; i++)
        {
          String s = _eintragUrsachenEinheit [i].getPid();
          if (i<_eintragUrsachenEinheit.length-1)
            sb.append(s + ";");
          else
            sb.append(s);
        }
        
        String uePid = sb.toString();

        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();

        try
        {
          System.out.println(viewID + " : " + uePid);
          
          page.showView( viewID, uePid, IWorkbenchPage.VIEW_CREATE );
          
          IViewReference ref = page.findViewReference( viewID, uePid );
          Perspective ap = ((WorkbenchPage) page).getActivePerspective();
          if (ap != null)
          {
            if (ref != null)
            {
              ap.getPresentation().detachPart( ref );
              
              IWorkbenchPart wbPart = ref.getPart( false );
              Shell shell = wbPart.getSite().getShell();
              shell.setSize( 650, 730 );
              IViewPart vPart = ref.getView( true );              
            }
          }
          else
          {
            System.out.println("Perspective ist null!");
          }
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
          Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "ParameterView zur Ursacheneinheit " + pidObjTyp + " nicht vorhanden!") );
        }
      }
    }
    else
    {
      MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb1.setText( "Fehler" );
      mb1.setMessage( "Es muss ein Eintrag Automatikporgramm selektiert werden!" );
      mb1.open();

    }

    bestimmeZustandButtons();
  }

  /**
   * Bearbeitet der Parameter zum EintragUrsachenEinheit. Es ffnet sich die entsprechende ParameterView 
   * zum Bearbeiten der Parameter.
   * 
   */
  private void aktionUEBearbeiten()
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length > 0)
    {
    	String pidObjTyp = _eintragUrsachenEinheit[0].getPidObjTyp();
    	
      /*
       * ParameterView ffnen
       */
      
      if ( pidObjTyp == null || pidObjTyp.length() == 0 )
        Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "Kein Objekttyp zur Parametrisierung vorhanden!") );
      else
      {
        String viewID = SbaUrsachenEinheitViewParameter.ID; 
        	
//        	Activator.PLUGIN_ID + "." + Activator.ID_PARAMETER + "." + pidObjTyp;
        
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < _eintragUrsachenEinheit.length; i++)
        {
          String s = _eintragUrsachenEinheit [i].getPid();
          if (i<_eintragUrsachenEinheit.length-1)
            sb.append(s + ";");
          else
            sb.append(s);
        }

        String uePid = sb.toString();

        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();

        try
        {
          
          System.out.println(viewID + " : " + uePid);
          
          page.showView( viewID, uePid, IWorkbenchPage.VIEW_CREATE );
          
          IViewReference ref = page.findViewReference( viewID, uePid );
          Perspective ap = ((WorkbenchPage) page).getActivePerspective();
          if (ap != null)
          {
            if (ref != null)
            {
              ap.getPresentation().detachPart( ref );
              
              IWorkbenchPart wbPart = ref.getPart( false );
              Shell shell = wbPart.getSite().getShell();
              shell.setSize( 600, 700 );
              IViewPart vPart = ref.getView( true );              
            }
          }
          else
          {
            System.out.println("Perspective ist null!");
          }
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
          Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "ParameterView zur Ursacheneinheit " + pidObjTyp + " nicht vorhanden!") );
        }
      }
    }
    else
    {
      MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb1.setText( "Fehler" );
      mb1.setMessage( "Es muss ein Eintrag Automatikporgramm selektiert werden!" );
      mb1.open();

    }

    bestimmeZustandButtons();
  }

  /**
   * Kopiert einen EintragUrsachenEinheit. Es ffnet sich der Definitionsdialg ({@link UrsachenEinheitDefinitionsdialog})
   * zum Bearbeiten des Eintrags.
   * 
   */
  private void aktionKopieren()
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length == 1)
    {
      UrsachenEinheitDefinitionsdialog ueinheitDefinitionsdialog = new UrsachenEinheitDefinitionsdialog( PlatformUI
          .getWorkbench().getDisplay().getActiveShell() );

      EintragUrsachenEinheit clone = _eintragUrsachenEinheit[0].clone();
      clone.setPid(_eintragUrsachenEinheit[0].getPid());

      _eintragUrsachenEinheit[0] = ueinheitDefinitionsdialog.oeffne( clone, true );
    }
    else
    {
      MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb1.setText( "Fehler" );
      mb1.setMessage( "Es muss ein Eintrag Automatikprogramm selektiert werden!" );
      mb1.open();
    }

    bestimmeZustandButtons();
  }

  /**
   * Lscht einen selektierten EintragUrsachenEinheit nach vorhergehender "Sind sie sicher?"-Abfrage und dem Urlasser-Dialog.
   * 
   */
  private void aktionLoeschen()
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length > 0 )
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL );
      mb.setText( "Automatikprogramm EintragUrsachenEinheit lschen" );
      
      if (_eintragUrsachenEinheit.length == 1)
        mb.setMessage( "Wollen Sie die Ursacheneinheit: " + _eintragUrsachenEinheit[0].getName()
            + " wirklich lschen?" );
      else
        mb.setMessage( "Wollen Sie die Ursacheneinheiten wirklich lschen?" );
      	
      if ( mb.open() == SWT.OK )
      {
        UrlasserDialog urlasserDialog = new UrlasserDialog( PlatformUI.getWorkbench().getDisplay().getActiveShell() );

        if ( urlasserDialog.anzeigen( null ).getEingabeOk() )
        {
        	for (EintragUrsachenEinheit eue : _eintragUrsachenEinheit)
            _daVKommunikation.loescheUrsachenEinheitEintrag( eue.getPid() );
          _eintragUrsachenEinheit = null;
        }
      }
    }
    else
    {
      MessageBox mb1 = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb1.setText("Fehler");
      mb1.setMessage("Es muss ein Eintrag Automatikprogramm selektiert werden!");
      mb1.open();
    }

    bestimmeZustandButtons();
  }

  /**
   * This method initializes _compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.grabExcessVerticalSpace = true;
    gridData5.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    _compositeTop = new Composite(_composite, SWT.NONE);
    _compositeTop.setLayoutData(gridData3);
    _compositeTop.setLayout(gridLayout);

  }

  /**
   * This method initializes _compositeTable
   * 
   */
  private void createCompositeTable()
  {
    GridData gridData1 = new GridData();
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalAlignment = GridData.FILL;
    
    _compositeTable = new Composite(_compositeTop, SWT.NONE);
    _compositeTable.setLayout(new GridLayout(1, false));
    _compositeTable.setLayoutData(gridData1);

    erzeugeTabelle();

  }

  /**
   * This method initializes _compositeBottom
   * 
   */
  private void createCompositeRight()
  {
    RowLayout rowLayout2 = new RowLayout();
    rowLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout2.wrap = false;
    rowLayout2.pack = false;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.CENTER;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    
    _compositeRight = new Composite(_compositeTop, SWT.NONE);
    _compositeRight.setLayoutData(gridData6);
    _compositeRight.setLayout(rowLayout2);
    
    buttonNeu = new Button(_compositeRight, SWT.NONE);
    buttonNeu.setText("Neu...");
    buttonNeu.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionNeu();
      }

    });

    buttonUEMQAnlegen = new Button(_compositeRight, SWT.NONE);
    buttonUEMQAnlegen.setText("Ursacheneinheiten MQ automatisch anlegen");
    buttonUEMQAnlegen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionErzeugeUEMQ ();
      }
    });

    buttonUEUFDAnlegen = new Button(_compositeRight, SWT.NONE);
    buttonUEUFDAnlegen.setText("Ursacheneinheiten UFD automatisch anlegen");
    buttonUEUFDAnlegen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionErzeugeUEUFD();
      }
    });

    buttonKopieren = new Button(_compositeRight, SWT.NONE);
    buttonKopieren.setText("Kopieren...");
    buttonKopieren.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionKopieren();
      }

    });

    buttonLoeschen = new Button(_compositeRight, SWT.NONE);
    buttonLoeschen.setText("Lschen");
    buttonLoeschen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionLoeschen();
      }

    });

    buttonParameterBearbeiten = new Button(_compositeRight, SWT.NONE);
    buttonParameterBearbeiten.setText("Parameter Algorithmus (getrennt)");
    buttonParameterBearbeiten.setToolTipText(
    		"Parametrieren der Ursacheneinheiten spezifischen Algorithmen.\n" + 
    		"Pro Ursacheneinheit wird dabei ein Reiter angelegt, so dass die\n" + 
    		"Parameter der einzelnen Ursacheneinheiten verglichen werden knnen."
    		);
    buttonParameterBearbeiten.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionBearbeiten(0);
      }

    });

    buttonParameterBearbeitenInEinemReiter = new Button(_compositeRight, SWT.NONE);
    buttonParameterBearbeitenInEinemReiter.setText("Parameter Algorithmus (gemeinsam)");
    buttonParameterBearbeitenInEinemReiter.setToolTipText(
    		"Parametrieren der Ursacheneinheiten spezifischen Algorithmen.\n" + 
    		"In diesem Dialog wird eine Eingaebmaske fr alle Ursacheneinheiten\n" +
    		"zur Verfgung gestellt."
    		);

    buttonParameterBearbeitenInEinemReiter.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionBearbeiten(1);
      }

    });

    buttonZuordnungMessquerschnitte = new Button(_compositeRight, SWT.NONE);
    buttonZuordnungMessquerschnitte.setText("Parameter Zuordnung Messquerschnitte");
    buttonZuordnungMessquerschnitte.setToolTipText(
    		"Parametrieren der Messquerschnitt Zuordnung zu den Ursacheneinheiten\n" + 
    		"Es knnen gleichzeitig Ursacheneinheiten unterschiedlicher Typen versorgt werden");
    
    buttonZuordnungMessquerschnitte.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionZuordnungMessquerschnitte();
      }

    });

    buttonZuordnungUmfelddaten = new Button(_compositeRight, SWT.NONE);
    buttonZuordnungUmfelddaten.setText("Parameter Zuordnung Umfelddaten");
    buttonZuordnungUmfelddaten.setToolTipText(
    		"Parametrieren der Umfelddaten Zuordnung zu den Ursacheneinheiten\n" + 
    		"Es knnen gleichzeitig Ursacheneinheiten unterschiedlicher Typen versorgt werden");
    
    buttonZuordnungUmfelddaten.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionZuordnungUmfelddaten();
      }

    });

    buttonParameterUEBearbeiten = new Button(_compositeRight, SWT.NONE);
    buttonParameterUEBearbeiten.setText("Parameter Ursacheneinheit");
    buttonParameterUEBearbeiten.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionUEBearbeiten();
      }

    });

    buttonSchaltBild = new Button(_compositeRight, SWT.NONE);
    buttonSchaltBild.setText("Schaltbildversorgung");
    buttonSchaltBild.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionSchaltBildVersorung();
      }

    });

    bestimmeZustandButtons();
  }

  private void erzeugeFilterTypen (Composite parent)
  {
  	GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;
    
    _comboObjTypFilter = new Combo( parent, SWT.BORDER );
    _comboObjTypFilter.setLayoutData( gdTableItem );

    holeSystemObjekte();
    
    _comboObjTypFilter.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				bestimmeGefilterteUrsachenEinheiten();
				bestimmeZustandButtons();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
  }

  private void erzeugeFilterNamen (Composite parent)
  {
  	GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;
    
    _textNamen = new Text (parent, SWT.BORDER);
    _textNamen.setLayoutData(gdTableItem);
    
    _textNamen.setText(_defaultText);
    
    _textNamen.addFocusListener(new FocusListener()
		{
			@Override
			public void focusLost(FocusEvent e)
			{
				if (_textNamen.getText().isEmpty())
					_textNamen.setText(_defaultText);
			}
			
			@Override
			public void focusGained(FocusEvent e)
			{
				if (_textNamen.getText().equals(_defaultText))
					_textNamen.setText("");
			}
		});
    
    _textNamen.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent e)
			{
				bestimmeGefilterteUrsachenEinheiten();
			}
		});
  }

  /**
   * Holt die SystemObjekte zum Objekttyp
   */
  private void __holeSystemObjekte()
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    List<SystemObjectType> liste = connection.getDataModel().getType("typ.sswSbaUrsachenEinheit").getSubTypes();   
    
    Map<String, SystemObjectType> treeMap = new TreeMap<String, SystemObjectType>();
    for ( SystemObjectType sot : liste )
    {
      if ( sot.getSubTypes().size() == 0 )
      {
      	if (verwende(sot))
          treeMap.put(sot.getName(), sot);
      }
      else
      {
        List<SystemObjectType> subTypes = sot.getSubTypes();
        for ( SystemObjectType  subSot : subTypes )
        {
        	if (verwende(subSot))
            treeMap.put(subSot.getName(), subSot);
        }
      }
    }
    
    String [] sortList = treeMap.keySet().toArray(new String [treeMap.keySet().size()]); 
    String [] listMitAlle = new String [sortList.length + 1];
    
    listMitAlle [0] = _alle;
    for (int i=0; i<sortList.length; ++i)
    	listMitAlle [i + 1] = sortList [i];
    
    _comboObjTypFilter.setItems ( listMitAlle );
    _comboObjTypFilter.select(0);
    
  }

  /**
   * Holt die SystemObjekte zum Objekttyp
   */
  private void holeSystemObjekte()
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    List<SystemObjectType> liste = connection.getDataModel().getType("typ.sswSbaUrsachenEinheit").getSubTypes();

    List<SystemObjectType> listeTmp = new ArrayList<SystemObjectType>();

    for (SystemObjectType sot : liste)
    	bestimmeObjektTypen (sot, listeTmp);
    
    Map<String, SystemObjectType> treeMap = new TreeMap<String, SystemObjectType>();
    for (SystemObjectType sot : listeTmp)
    {
    	if (verwende (sot))
        treeMap.put(sot.getName(), sot);
    }

    String [] sortList = treeMap.keySet().toArray(new String [treeMap.keySet().size()]); 
    String [] listMitAlle = new String [sortList.length + 1];
    
    listMitAlle [0] = _alle;
    for (int i=0; i<sortList.length; ++i)
    	listMitAlle [i + 1] = sortList [i];
    
    _comboObjTypFilter.setItems ( listMitAlle );
    _comboObjTypFilter.select(0);
  }

  /**
   * Methode bestimmt rekursiv die vorhandenen Objekttypen
   * @param sot ausgehendes Objekt
   * @param liste Liste in die die Objekte eingetragen werden
   */
  private void bestimmeObjektTypen(SystemObjectType sot, List<SystemObjectType> liste)
  {
    if ( sot.getSubTypes().size() == 0 )
    {
    	liste.add(sot);
    }
    else
    {
      for (SystemObjectType subTyp : sot.getSubTypes())
      	bestimmeObjektTypen(subTyp, liste);
    }
  }

  private boolean verwende(SystemObjectType sot)
  {
  	if (sot.getPid().equals("typ.sswSbaUeSonderprogramm") ||
  			sot.getPid().equals("typ.sswSbaUeSonderUndHandprogramme"))
  		return false;
  	
  	return true;
  }

  private void erzeugeTabelle()
  {
  	erzeugeFilterTypen(_compositeTable);
  	erzeugeFilterNamen(_compositeTable);

  	GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.FILL;
    gdTableItem.grabExcessVerticalSpace = true;

    _tvUrsachenEinheit = new TableViewer(_compositeTable, SWT.MULTI | SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);

    _tvUrsachenEinheit.setContentProvider(new TabelleInhaltsVersorger());
    _tvUrsachenEinheit.setLabelProvider(new TabelleDarstellungsVersorger());
    
    _tvUrsachenEinheit.getTable().setLayoutData(gdTableItem);
    
    String[] colNames = new String[] {"Name", "Typ", "Pid"};
    int i = 0;
    for (String colName : colNames)
    {
      final int colIdx = i;
      TableColumn column = new TableColumn(_tvUrsachenEinheit.getTable(), SWT.LEFT | SWT.Move);
      column.setText(colName);
      column.setWidth(300);
      column.setMoveable(true);

      /*
       * Sortierung nach der Spalte
       */
      TabelleSpaltenSortierer colSorter = new TabelleSpaltenSortierer(_tvUrsachenEinheit, column)
      {
        protected int doCompare(Viewer v, Object e1, Object e2)
        {
          TableViewer tv = (TableViewer)v;
          ITableLabelProvider lp = (ITableLabelProvider)tv.getLabelProvider();

          String t1 = lp.getColumnText( e1, colIdx );
          String t2 = lp.getColumnText( e2, colIdx );
          
          /*
           * String-Vergleich
           */
          return t1.compareTo( t2 );
          
//          //Zahlen-Vergleich
//          return numberComparator( t1, t2 );
         
//          //Zeit-Vergleich
//          return timeComparator( t1, t2 );
          
        }
        
      };
      
      i++;
    }
       
    _tvUrsachenEinheit.getTable().setHeaderVisible(true);
    _tvUrsachenEinheit.getTable().setLinesVisible(true);
    
    _tvUrsachenEinheit.addSelectionChangedListener(new ISelectionChangedListener() 
    {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof EintragUrsachenEinheit)
        {
          Object[] array = selection.toArray();

          _eintragUrsachenEinheit = new EintragUrsachenEinheit [array.length];
          
          for (int i=0; i<array.length; ++i)
          {
            Object object = array [i];
            _eintragUrsachenEinheit [i] = (EintragUrsachenEinheit) object;
          }

          bestimmeZustandButtons ();
          
          KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
          if (kp.getWert().equals("true")){
          
            ArrayList<SystemObject> liste = new ArrayList<SystemObject>();

            for (EintragUrsachenEinheit eue : _eintragUrsachenEinheit)
              liste.add(_daVKommunikation.holeDaVVerbindung().getDataModel().getObject(eue.getPid()));

            pruefeBerechtigung(liste);
          }

        }
        else
          _eintragUrsachenEinheit = null;

      }

    });
    
    _tvUrsachenEinheit.addDoubleClickListener(new IDoubleClickListener() {

      @Override
      public void doubleClick(DoubleClickEvent event)
      {
//        aktionBearbeiten();
        aktionSchaltBildVersorung();
      }
    });

  }

  /**
   * Klasse fr die Sortierung der Tabelle im TableViewer nach einer Spalte
   * 
   * @author Krause
   *
   */
  private abstract class TabelleSpaltenSortierer extends ViewerComparator
  {
    public static final int ASC = 1;
    public static final int NONE = 0;
    public static final int DESC = -1;
    
    private int _direction = 0;
    private TableColumn _column;
    private TableViewer _viewer;
    
    public TabelleSpaltenSortierer( TableViewer viewer, TableColumn colunm )
    {
      _column = colunm;
      _viewer = viewer;
      
      _column.addSelectionListener( new SelectionAdapter()
      {
        public void widgetSelected( SelectionEvent event )
        {
          TabelleSpaltenSortierer tabelleSpaltenSortierer = TabelleSpaltenSortierer.this;
          int newDirection = ASC;
          if ( tabelleSpaltenSortierer._viewer.getComparator() != null )
          {
            if ( tabelleSpaltenSortierer._viewer.getComparator() == tabelleSpaltenSortierer )
            {
              int tcsDirection = tabelleSpaltenSortierer._direction;
              if ( tcsDirection == ASC )
                newDirection = DESC ;
//              else if ( tcsDirection == DESC )
//                newDirection = NONE ;
            }
          }
          setSorter( tabelleSpaltenSortierer, newDirection );
        }

      });
    }
    
    public void setSorter(TabelleSpaltenSortierer sorter, int direction)
    {
      Table table = _column.getParent();
      if (direction == NONE)
      {
        table.setSortColumn( null );
//        table.setSortDirection( SWT.NONE );
        _viewer.setComparator( null );
      }
      else
      {
        table.setSortColumn( _column );
        sorter._direction = direction;
//        if (direction == ASC)
//          table.setSortDirection( SWT.UP );
//        else
//          table.setSortDirection( SWT.DOWN );
        if (_viewer.getComparator()== sorter)
          _viewer.refresh();
        else
          _viewer.setComparator( sorter );
      }
    }
    

    /**
     * Vergleichen von Numerischen Werten...
     * 
     */
    public int numberComparator( String t1, String t2 )
    {
      Long l1 = Long.parseLong(t1);
      Long l2 = Long.parseLong(t2);
      return l1.compareTo( l2 );
    }
    
    /**
     * Vergleichen von Datumsangaben...
     * 
     */
    public int timeComparator( String t1, String t2 )
    {
      Date d1 = null;
      Date d2 = null;
      try
      {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
        d1 = sdf.parse(t1);
        d2 = sdf.parse(t2);
      }
      catch (Exception e)
      {
        // TODO: handle exception
        e.printStackTrace();
      }
      return d1.compareTo( d2 );
    }
    
    public int compare(Viewer viewer, Object e1, Object e2)
    {
      return _direction * doCompare(viewer, e1, e2);
    }
    
    protected abstract int doCompare(Viewer tableViewer, Object e1, Object e2);
    
  }
  
  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    @SuppressWarnings("unchecked")
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((EintragUrsachenEinheit)object).getName();
          break;

        case 1:
        	if (((EintragUrsachenEinheit)object).getObjTyp() != null)
            returnString = ((EintragUrsachenEinheit)object).getObjTyp().getName();
        	else
        		returnString = "";
        	break;

        case 2:
          returnString = ((EintragUrsachenEinheit)object).getPid();
          break;

        	

        default:
          returnString = "";

      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Fllt die Liste der UrsachenEinheiten des Dialogs.
   */
  public void bestimmeGefilterteUrsachenEinheiten()
  {
  	int index = _comboObjTypFilter.getSelectionIndex();
  	String nameTyp = _comboObjTypFilter.getItem(index);
  	
  	List<EintragUrsachenEinheit> gefilterteDatensaetze = new ArrayList<EintragUrsachenEinheit>();
  	
  	/*
  	 * Nach Typen filtern
  	 */
  	if (nameTyp.equals(_alle))
  	{
  		for (EintragUrsachenEinheit eue : _ursachenEinheitDatensaetze)
  			gefilterteDatensaetze.add(eue);
  	}
  	else
  	{
  		for (EintragUrsachenEinheit eue : _ursachenEinheitDatensaetze)
  		{
  			if (eue.getObjTyp().getName().equals(nameTyp))
  			  gefilterteDatensaetze.add(eue);
  		}
  	}

  	/*
  	 * Nach Namen filtern
  	 */

  	List<EintragUrsachenEinheit> gefilterteDatensaetzeNamen = new ArrayList<EintragUrsachenEinheit>();

  	String filterName = _textNamen.getText();
  	if (filterName.isEmpty() || filterName.equals(_defaultText))
  	{
  		for (EintragUrsachenEinheit eue : gefilterteDatensaetze)
  			gefilterteDatensaetzeNamen.add(eue);
  	}
  	else
  	{
  		for (EintragUrsachenEinheit eue : gefilterteDatensaetze)
  		{
  			try
  			{
  			  if (eue.getName().contains(filterName))
  			  	gefilterteDatensaetzeNamen.add(eue);
  			  else
  			  	if (eue.getName().matches(filterName))
   			      gefilterteDatensaetzeNamen.add(eue);
  			}
  			catch (Exception e) 
  			{
				}
  		}
  	}
  	
    _tvUrsachenEinheit.setInput(gefilterteDatensaetzeNamen);
  }

  /**
   * Prft alle SystemObjekte der Liste ob eine Freigabe oder Sperrung vorliegt 
   * @param liste
   */
  private void pruefeBerechtigung(List<SystemObject> liste){
    for (Map.Entry<String, Object> me : _oberflaechenElemente.entrySet())
    {
      OberflaechenBerechtigung.getInstanz().pruefeBerechtigungElement(
          liste, me.getKey());
      
    }
  }

  /**
   * Erzeugt die Oberflchen-Berechtigung
   */
  private void erzeugeOberflaechenBerechtigung()
  {
    
    _berechtigungsZuhoerer = new IOberflaechenBerechtigungZuhoerer() {
      @Override
      public void freigabe(OberflaechenBerechtigungEreignis e)
      {
        _debug.fine("Info Freigabe erhalten: " + e.hatFreigabe());

        List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
        List<SystemObject> liste2 = e.getSystemObjekteSperrung();
        for (SystemObject so : liste1)
        {
          _debug.fine("Objekte Freigabe: " + so);
        }
        for (SystemObject so : liste2)
        {
          _debug.fine("Objekte Sperrung: " + so);
        }

        List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
        List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
        for (OberflaechenFunktion of : liste3)
        {
          _debug.fine("Funktion Freigabe: " + of.getFunktionsName());
          if (_oberflaechenElemente.containsKey(of.getFunktionsName()))
          {
            Object object = _oberflaechenElemente.get(of.getFunktionsName());
            if (object instanceof Button)
            {
              Button button = (Button)object;
              button.setEnabled(true);
              
            }
            
          }

        }
        for (OberflaechenFunktion of : liste4)
        {
          _debug.fine("Funktion Sperrung: " + of.getFunktionsName());
        }

      }

      @Override
      public void sperrung(OberflaechenBerechtigungEreignis e)
      {
        _debug.fine("Info Sperrung erhalten: " + e.hatFreigabe());

        List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
        List<SystemObject> liste2 = e.getSystemObjekteSperrung();
        for (SystemObject so : liste1)
        {
          _debug.fine("Objekte Freigabe: " + so);
        }
        for (SystemObject so : liste2)
        {
          _debug.fine("Objekte Sperrung: " + so);
        }

        List<OberflaechenFunktion> liste3 = e.getOberflaechenFunktionFreigabe();
        List<OberflaechenFunktion> liste4 = e.getOberflaechenFunktionSperrung();
        for (OberflaechenFunktion of : liste3)
        {
          _debug.fine("Funktion Freigabe: " + of.getFunktionsName());
        }
        for (OberflaechenFunktion of : liste4)
        {
          _debug.fine("Funktion Sperrung: " + of.getFunktionsName());
          if (_oberflaechenElemente.containsKey(of.getFunktionsName()))
          {
            Object object = _oberflaechenElemente.get(of.getFunktionsName());
            if (object instanceof Button)
            {
              Button button = (Button)object;
              button.setEnabled(false);
              
            }
           
          }

        }

      }
    };

    de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz().hinzufuegenOberflaechenBenutzerZuhoerer(
        _berechtigungsZuhoerer);
    

    OberflaechenFunktion funktion1 = new OberflaechenFunktion(
        VerwaltungsTabelleComp.ID + ".schaltflaeche.neu", _berechtigungsZuhoerer,
        VerwaltungsTabelleComp.ID, "anlegen");
    OberflaechenFunktion funktion2 = new OberflaechenFunktion(
        VerwaltungsTabelleComp.ID + ".schaltflaeche.bearbeiten", _berechtigungsZuhoerer,
        VerwaltungsTabelleComp.ID, "bearbeiten");
    OberflaechenFunktion funktion3 = new OberflaechenFunktion(
        VerwaltungsTabelleComp.ID + ".schaltflaeche.kopieren", _berechtigungsZuhoerer,
        VerwaltungsTabelleComp.ID, "kopieren");
    OberflaechenFunktion funktion4 = new OberflaechenFunktion(
        VerwaltungsTabelleComp.ID + ".schaltflaeche.loeschen", _berechtigungsZuhoerer,
        VerwaltungsTabelleComp.ID, "loeschen");

    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion1);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion2);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion3);
    OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(funktion4);

    _oberflaechenElemente.put(funktion1.getFunktionsName(), buttonNeu);
    _oberflaechenElemente.put(funktion2.getFunktionsName(), buttonParameterBearbeiten);
    _oberflaechenElemente.put(funktion2.getFunktionsName(), buttonParameterBearbeitenInEinemReiter);
    _oberflaechenElemente.put(funktion3.getFunktionsName(), buttonKopieren);
    _oberflaechenElemente.put(funktion4.getFunktionsName(), buttonLoeschen);

  }
  
  public TableViewer getTvUrsachenEinheit()
  {

    return _tvUrsachenEinheit;
  }
  
  @Override
  public void dispose()
  {
    KmdParameter kp = _kommandoZeilenParameter.getKommandoZeilenParameter("-berechtigungsKonzept");
    if (kp.getWert().equals("true")){
      de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
          _berechtigungsZuhoerer);
    }
    super.dispose();
  }

  /**
   * Methode bestimmt den Zustand der Buttons je nach Selektion der Ursacheneinheiten
   */
	private void bestimmeZustandButtons()
  {
  	buttonUEMQAnlegen.setEnabled(isUrsachenEinheitenMQErstellenMoeglich());

  	buttonUEUFDAnlegen.setEnabled(isUrsachenEinheitenUFDErstellenMoeglich());

    buttonZuordnungMessquerschnitte.setEnabled(isButtonZuordnungMessquerschnitteMoeglich ());

    buttonZuordnungUmfelddaten.setEnabled(isButtonZuordnungUmfeldDatenMoeglich ());

		/*
		 * Keine Ursacheneinheit angewhlt --> alle Buttons deaktivieren
		 */
		if (_eintragUrsachenEinheit == null || (_eintragUrsachenEinheit.length == 0))
		{
			buttonParameterBearbeiten.setEnabled(false);
			buttonParameterBearbeitenInEinemReiter.setEnabled(false);
			buttonParameterUEBearbeiten.setEnabled(false);
			buttonSchaltBild.setEnabled(false);
			
			buttonLoeschen.setEnabled(false);
			buttonKopieren.setEnabled(false);
			
			buttonZuordnungMessquerschnitte.setEnabled(false);
		}
		else
		{
			/*
			 * Mindestens eine Ursacheneinheit angewhlt:
			 * --> Parameter Ursacheneinheit
			 * --> Parameter Schaltbild
			 * --> Lschen
			 */
			buttonParameterUEBearbeiten.setEnabled(true);
			buttonSchaltBild.setEnabled(true);
			buttonLoeschen.setEnabled(true);
			
		  if (_eintragUrsachenEinheit.length == 1)
		  {
		  	/*
		  	 * Genaue eine Ursacheneinheit angewhlt
		  	 * --> Kopiere
		  	 * --> Parameter Algorithmus
		  	 */
				buttonKopieren.setEnabled(true);
				buttonParameterBearbeiten.setEnabled(true);
		    buttonParameterBearbeitenInEinemReiter.setEnabled(false);    
		  }
		  else
		  {
		  	/*
		  	 * Mehrere Ursacheneinheiten angewhlt
		  	 * --> Parameter Algorithmus nur wenn alle vom gleichen Typ sind 
		  	 */
				buttonKopieren.setEnabled(false);
				
				/*
				 * Die Parameter mehrerer Ursacheneinheiten knnen gleichzeitig nur versorgt werden, 
				 * wenn die Ursacheneinheiten vom selben Typ sind, sonst nicht
				 */
				boolean gleicherTyp = true;
				String pid = null;
				for (EintragUrsachenEinheit eue : _eintragUrsachenEinheit)
				{
					if (pid == null)
						pid = eue.getPidObjTyp();
					else
						if (!pid.equals(eue.getPidObjTyp()))
							gleicherTyp = false;
				}
				
				buttonParameterBearbeiten.setEnabled(gleicherTyp);
				
				System.out.println("VerwaltungsTabelleComp.bestimmeZustandButtons() " + pid);
				
				/*
				 * Nur Algorithmen zulassen fr die diese Art der Versorgung implementiert ist
				 */
				
				boolean bearbeiten2 = gleicherTyp;
				if (bearbeiten2)
				{
					bearbeiten2 = false;
					for (String s : _ursachenEinheitenFuerVersorgungInEinemReiter)
					  if (pid.equals(s))
					  	bearbeiten2 = true;
				}
				
		    buttonParameterBearbeitenInEinemReiter.setEnabled(bearbeiten2);
		  }
		}
  }

	/**
	 * Methode prft ob der Button "Zuordnung Umfelddaten" auf Basis der aktuell selektierten
	 * Ursacheneinheiten angewhlt werden kann oder nicht
	 * @return Info ob Button angewhlt werden kann
	 */
	private boolean isButtonZuordnungUmfeldDatenMoeglich()
  {
		if (_eintragUrsachenEinheit == null || (_eintragUrsachenEinheit.length == 0))
			return false;

		for (EintragUrsachenEinheit eue : _eintragUrsachenEinheit)
		{
  		String pid = eue.getPidObjTyp();
  		
  		boolean gefunden = false;
  		for (String s : _ursachenEinheitenMitUFDZuordnung)
  			if (pid.equals(s))
  				gefunden = true;
  		
  		if (!gefunden)
  			return false;
		}

	  return true;
  }

	/**
	 * Methode prft ob der Button "Zuordnung Messquerschnitte" auf Basis der aktuell selektierten
	 * Ursacheneinheiten angewhlt werden kann oder nicht
	 * @return Info ob Button angewhlt werden kann
	 */
	private boolean isButtonZuordnungMessquerschnitteMoeglich()
  {
		if (_eintragUrsachenEinheit == null || (_eintragUrsachenEinheit.length == 0))
			return false;

		for (EintragUrsachenEinheit eue : _eintragUrsachenEinheit)
		{
  		String pid = eue.getPidObjTyp();
  		
  		boolean gefunden = false;
  		for (String s : _ursachenEinheitenMitMQZuordnung)
  			if (pid.equals(s))
  				gefunden = true;
  		
  		if (!gefunden)
  			return false;
		}

	  return true;
  }

	/**
	 * Methode prft ob aufgrund der derzeit eingestellten Filtermglichkeiten
	 * der Button buttonUEAnlegen aktiviert werden kann oder nicht
	 * @return
	 */
	private boolean isUrsachenEinheitenMQErstellenMoeglich ()
	{
		boolean anwaehlbar = false;
		
  	int index = _comboObjTypFilter.getSelectionIndex();
  	String nameTyp = _comboObjTypFilter.getItem(index);
  	
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = connection.getDataModel();

  	for (String pid: _ursachenEinheitenMQAutomatisch)
  		if (dataModel.getObject(pid).getName().equals(nameTyp))
  			anwaehlbar = true;
  				
  	return anwaehlbar;
	}

	/**
	 * Methode prft ob aufgrund der derzeit eingestellten Filtermglichkeiten
	 * der Button buttonUEUFDAnlegen aktiviert werden kann oder nicht
	 * @return
	 */
	private boolean isUrsachenEinheitenUFDErstellenMoeglich ()
	{
		boolean anwaehlbar = false;
		
  	int index = _comboObjTypFilter.getSelectionIndex();
  	String nameTyp = _comboObjTypFilter.getItem(index);
  	
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = connection.getDataModel();

  	for (String pid: _ursachenEinheitenUFDAutomatisch)
  		if (dataModel.getObject(pid).getName().equals(nameTyp))
  			anwaehlbar = true;
  				
  	return anwaehlbar;
	}

	/**
	 * Methode ffnet die View zum Versorgen der Schaltbildversorgung
	 */
  private void aktionSchaltBildVersorung ()
  {
    if ( _eintragUrsachenEinheit != null && _eintragUrsachenEinheit.length > 0)
    {
    	String pidObjTyp = _eintragUrsachenEinheit[0].getPidObjTyp();
    	
      /*
       * ParameterView ffnen
       */
      
      if ( pidObjTyp == null || pidObjTyp.length() == 0 )
        Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "Kein Objekttyp zur Parametrisierung vorhanden!") );
      else
      {
        String viewID = EinzelmassnahmenViewParameter.ID; 
        	
//        	Activator.PLUGIN_ID + "." + Activator.ID_PARAMETER + "." + pidObjTyp;
        
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < _eintragUrsachenEinheit.length; i++)
        {
          String s = _eintragUrsachenEinheit [i].getPid();
          if (i<_eintragUrsachenEinheit.length-1)
            sb.append(s + ";");
          else
            sb.append(s);
        }

	      /*
 	       * Sonderfall Autarkprogramm:
	       * Die Parameter des Autarkprogrammes hngen nicht an der Ursacheneinheit sondern an der
	       * Streckenbeeinflussungsanlage
	       */
	     
	      if (pidObjTyp.equals("typ.sswSbaUeAutarkprogramm"))
	      {
	      	viewID = ViewSchaltbildVersorgungAutarkProgramm.ID;
	     	  sb = new StringBuffer();
	     	 
	     	  ClientDavInterface connection = DaVKommunikation.getInstanz().getConnection();
	        List<SystemObject> elements = connection.getDataModel().getType( "typ.sswSbaStreckenBeeinflussungsAnlage" ).getElements();
	        
	        if (elements.size() > 0)
	       	  sb.append(elements.get(0).getPid()); 
	      }

        String uePid = sb.toString();

        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();

        try
        {
          
          System.out.println(viewID + " : " + uePid);
          
          page.showView( viewID, uePid, IWorkbenchPage.VIEW_CREATE );
          
          IViewReference ref = page.findViewReference( viewID, uePid );
          Perspective ap = ((WorkbenchPage) page).getActivePerspective();
          if (ap != null)
          {
            if (ref != null)
            {
              ap.getPresentation().detachPart( ref );
              
              IWorkbenchPart wbPart = ref.getPart( false );
              Shell shell = wbPart.getSite().getShell();
              shell.setSize( 600, 700 );
              IViewPart vPart = ref.getView( true );              
            }
          }
          else
          {
            System.out.println("Perspective ist null!");
          }
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
          Activator.getDefault().getLog().log( new Status(Status.ERROR, VerwaltungsTabelleComp.ID, "ParameterView zur Ursacheneinheit " + pidObjTyp + " nicht vorhanden!") );
        }
      }
    }
    else
    {
      MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb1.setText( "Fehler" );
      mb1.setMessage( "Es muss ein Eintrag Automatikporgramm selektiert werden!" );
      mb1.open();

    }

    bestimmeZustandButtons();
  }

  /**
   * Methode erzeugt automatisch Ursacheneinheiten fr die Messquerschnitte 
   */
	private void aktionErzeugeUEMQ()
  {
		List <String> objekte = new ArrayList<String>();
		
		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A14)
			objekte = bestimmeMqSbaA14();

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.B27)
			objekte = bestimmeMqSbaB27();

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A8)
			objekte = bestimmeMqSbaA8();

		/*
		 * Systemobjekt fr Ursacheneinheit bestimmen
		 */
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = connection.getDataModel();
    
  	int index = _comboObjTypFilter.getSelectionIndex();
  	String nameTyp = _comboObjTypFilter.getItem(index);

    SystemObject soUE = null;
  	for (String pid: _ursachenEinheitenMQAutomatisch)
  		if (dataModel.getObject(pid).getName().equals(nameTyp))
  			soUE = dataModel.getObject(pid);

  	/*
  	 * Prfen ob Konfigurationsbereich fr die Ursacheneinheit bereits definiert ist,
  	 * wenn nicht wird der entsprechende Parameter angelegt und bei der Passwortabfrage an den
  	 * DaV gesendet
  	 */
    String pidTyp = soUE.getPid();
    
    ParaVerwaltungDynamischerObjekte v = ParaVerwaltungDynamischerObjekte.getInstance();
    
    List<ResultData> paraDynObjekte = null;
    if (!v.isKonfigurationsBereichDefiniert(pidTyp))
    {
      System.out.println("VerwaltungsTabelleComp.aktionErzeugeUEMQ() Erweitere Parameter fr " + pidTyp );    	
    	v.erzeugeZuordnungZuDefaultBereich(pidTyp);
    	paraDynObjekte = v.getDatenFuerDaV();
    }

    /*
     * Passwortabfrage (mit u.U. bertragung der Parameter Dynamischer Objekte
     */
    UrlasserDialog urlasserDialog = new UrlasserDialog( PlatformUI.getWorkbench().getDisplay().getActiveShell() );
    if ( !urlasserDialog.anzeigen( paraDynObjekte ).getEingabeOk() )
    {
    	/*
    	 * Falsche Zugangsdaten
    	 * --> Zuordnung zum Defaultbereich intern wieder lschen da dieser ja nicht an den DaV 
    	 *     bertragen wurde. 
    	 */
    	if (paraDynObjekte != null)
    		v.loescheZuordnungZuDefaultBereich(pidTyp);
    	return;
    }

    /*
     * Ursacheneinheiten anlegen und an den DaV bertragen
     */
  	String praefix = nameTyp.replace("UrsachenEinheit", "");
  	for (String postfix : objekte)
  	{
  		String nameUE = praefix + postfix;
  		
      EintragUrsachenEinheit eintragUrsachenEinheit = new EintragUrsachenEinheit();

      String pidName = nameUE.substring( 0, nameUE.length() ).replaceAll(" ", "");
      
      eintragUrsachenEinheit.setName(nameUE);
      String praefixProjekt = ProjektSpezifischeEinstellungen.getInstance().getPraefixUrsachenEinheit();
      
      eintragUrsachenEinheit.setPid(praefixProjekt + pidName );

      eintragUrsachenEinheit.setObjTyp( (SystemObjectType) soUE );

      if (!_daVKommunikation.isObjektVorhanden(eintragUrsachenEinheit.getPid()))
      {
        _daVKommunikation.erzeugeUrsachenEinheitEintrag ( eintragUrsachenEinheit );
      }
  	}
  }

	/**
	 * Methode bestimmt die Namen der Messquerschnitte die fr die automatische Erzeugung der
	 * Ursacheneinheiten verwendet werden sollen
	 * @return Namen der Messquerschnitte
	 */
	private List<String> bestimmeMqSbaA14 ()
	{
		return Arrays.asList(
				"_180-60_NO_MQ_702_HFB1",
				"_180-61_SW_MQ_180_HFB1",
				"_181-40_NO_MQ_175_HFB1",
				"_181-40_SW_MQ_175_HFB1",
				"_182-17_NO_MQ_710_HFB1",
				"_182-41_SW_MQ_170_HFB1",
				"_183-75_NO_MQ_718_HFB1",
				"_183-75_SW_MQ_718_HFB1",
				"_184-30_NO_MQ_160_HFB1",
				"_184-30_SW_MQ_160_HFB1",
				"_185-20_NO_MQ_720_HFB1",
				"_185-23_SW_MQ_720_HFB1",
				"_186-03_SW_MQ_158_HFB1",
				"_186-23_NO_MQ_158_HFB1",
				"_186-80_NO_MQ_730_HFB1",
				"_186-85_SW_MQ_730_HFB1",
				"_187-85_NO_MQ_150_HFB1",
				"_187-85_SW_MQ_150_HFB2",
				"_188-22_SW_MQ_150_HFB1",
				"_188-67_NO_MQ_740_HFB1",
				"_190-56_NO_MQ_750_HFB1",
				"_190-56_SW_MQ_750_HFB1",
				"_191-28_NO_MQ_130_HFB1",
				"_191-28_SW_MQ_130_HFB1",
				"_192-20_NO_MQ_760_HFB1",
				"_192-69_SW_MQ_125_HFB1",
				"_193-46_NO_MQ_770_HFB1",
				"_193-47_SW_MQ_770_HFB1",
				"_194-00_SW_MQ_120_HFB1",
				"_194-95_NO_MQ_780_HFB1",
				"_194-95_SW_MQ_780_HFB1",
				"_195-67_NO_MQ_110_HFB1",
				"_195-67_SW_MQ_110_HFB1"
		);
	}
	
	/**
	 * Methode bestimmt die Namen der Messquerschnitte die fr die automatische Erzeugung der
	 * Ursacheneinheiten verwendet werden sollen
	 * @return Namen der Messquerschnitte
	 */
	private List<String> bestimmeMqSbaB27 ()
	{
		return Arrays.asList(
				
				"_MA_Verkehr_AQ_1T",
				"_MA_Verkehr_AQ_2T",
				"_MA_Verkehr_AQ_3T",
				"_MA_Verkehr_AQ_4T",
				"_MA_Verkehr_AQ_5T",
				"_MA_Verkehr_AQ_6T",
				"_MA_Verkehr_AQ_7T",
				"_MA_Verkehr_AQ_8T",
				"_MA_Verkehr_AQ_9T",
				"_MA_Verkehr_AQ_10T",
				"_MA_Verkehr_AQ_11T",
				"_MA_Verkehr_AQ_12T",
				"_MA_Verkehr_AQ_13T",
				"_MA_Verkehr_AQ_14T",
				"_MA_Verkehr_AQ_15T",
				"_MA_Verkehr_AQ_1S",
				"_MA_Verkehr_AQ_2S",
				"_MA_Verkehr_AQ_3S",
				"_MA_Verkehr_AQ_4S",
				"_MA_Verkehr_AQ_5S",
				"_MA_Verkehr_AQ_6S",
				"_MA_Verkehr_AQ_7S",
				"_MA_Verkehr_AQ_8S",
				"_MA_Verkehr_AQ_9S",
				"_MA_Verkehr_AQ_10S",
				"_MA_Verkehr_AQ_11S",
				"_MA_Verkehr_AQ_12S",
				"_MA_Verkehr_AQ_13S",
				"_MA_Verkehr_AQ_14S",
				"_MA_Verkehr_AQ_15S",
				"_MA_Verkehr_AQ_16S",
				"_MA_Verkehr_AQ_17S",
				"_MA_Verkehr_AQ_18S"
		);
	}

	/**
	 * Methode bestimmt die Namen der Messquerschnitte die fr die automatische Erzeugung der
	 * Ursacheneinheiten verwendet werden sollen
	 * @return Namen der Messquerschnitte
	 */
	private List<String> bestimmeMqSbaA8 ()
	{
		List<String> namen = new ArrayList<String>();
		
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.messQuerschnitt");
		for (SystemObject so : objekte)
		{
			String name = so.getName().replace("MQ SBA A8 Stgt ", "");

			if (name.contains(" FR Karlsruhe"))
				name = "_FR-KA_MQ-" + name.replace(" FR Karlsruhe", "");
			else if (name.contains(" FR Mnchen"))
				name = "_FR-M_MQ-" + name.replace(" FR Mnchen", "");

			namen.add(name);
//			System.out.println("VerwaltungsTabelleComp.bestimmeMqSbaA8() " + name);
		}
		
		return namen;
	}

  /**
   * Methode erzeugt automatisch Ursacheneinheiten fr die Messquerschnitte 
   */
	private void aktionErzeugeUEUFD()
  {
		/*
		 * Systemobjekt fr Ursacheneinheit bestimmen
		 */
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = connection.getDataModel();
    
  	int index = _comboObjTypFilter.getSelectionIndex();
  	String nameTyp = _comboObjTypFilter.getItem(index);

    SystemObject soUE = null;
  	for (String pid: _ursachenEinheitenUFDAutomatisch)
  		if (dataModel.getObject(pid).getName().equals(nameTyp))
  			soUE = dataModel.getObject(pid);

  	/*
  	 * Ursacheneinheiten bestimmen
  	 */
		List <String> objekte = new ArrayList<String>();
		
		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A14)
			objekte = bestimmeUFDSbaA14();

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.B27)
			objekte = bestimmeUFDSbaB27();

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A8)
			objekte = bestimmeUFDSbaA8(soUE.getPid());

  	/*
  	 * Prfen ob Konfigurationsbereich fr die Ursacheneinheit bereits definiert ist,
  	 * wenn nicht wird der entsprechende Parameter angelegt und bei der Passwortabfrage an den
  	 * DaV gesendet
  	 */
    String pidTyp = soUE.getPid();
    
    ParaVerwaltungDynamischerObjekte v = ParaVerwaltungDynamischerObjekte.getInstance();
    
    List<ResultData> paraDynObjekte = null;
    if (!v.isKonfigurationsBereichDefiniert(pidTyp))
    {
      System.out.println("VerwaltungsTabelleComp.aktionErzeugeUEUFD() Erweitere Parameter fr " + pidTyp );
      
    	v.erzeugeZuordnungZuDefaultBereich(pidTyp);
    	paraDynObjekte = v.getDatenFuerDaV();
    }

    /*
     * Passwortabfrage (mit u.U. bertragung der Parameter Dynamischer Objekte
     */
    UrlasserDialog urlasserDialog = new UrlasserDialog( PlatformUI.getWorkbench().getDisplay().getActiveShell() );
    if ( !urlasserDialog.anzeigen( paraDynObjekte ).getEingabeOk() )
    {
    	/*
    	 * Falsche Zugangsdaten
    	 * --> Zuordnung zum Defaultbereich intern wieder lschen da dieser ja nicht an den DaV 
    	 *     bertragen wurde. 
    	 */
    	if (paraDynObjekte != null)
    		v.loescheZuordnungZuDefaultBereich(pidTyp);
    	return;
    }

    /*
     * Ursacheneinheiten anlegen und an den DaV bertragen
     */
  	for (String nameUE : objekte)
  	{
      EintragUrsachenEinheit eintragUrsachenEinheit = new EintragUrsachenEinheit();

      String pidName = nameUE.substring( 0, nameUE.length() ).replaceAll(" ", "");
      
      eintragUrsachenEinheit.setName(nameUE);
      String praefixProjekt = ProjektSpezifischeEinstellungen.getInstance().getPraefixUrsachenEinheit();
      
      eintragUrsachenEinheit.setPid(praefixProjekt + pidName );

      eintragUrsachenEinheit.setObjTyp( (SystemObjectType) soUE );

      if (!_daVKommunikation.isObjektVorhanden(eintragUrsachenEinheit.getPid()))
      {
        _daVKommunikation.erzeugeUrsachenEinheitEintrag ( eintragUrsachenEinheit );
      }
  	}
  }

	/**
	 * Methode bestimmt die Namen der Messquerschnitte die fr die automatische Erzeugung der
	 * Ursacheneinheiten verwendet werden sollen
	 * @return Namen der Messquerschnitte
	 */
	private List<String> bestimmeUFDSbaA14 ()
	{
		return Arrays.asList(
		);
	}
	
	/**
	 * Methode bestimmt die Namen der Messquerschnitte die fr die automatische Erzeugung der
	 * Ursacheneinheiten verwendet werden sollen
	 * @return Namen der Messquerschnitte
	 */
	private List<String> bestimmeUFDSbaB27 ()
	{
		return Arrays.asList(
				
		);
	}

	/**
	 * Methode bestimmt die Namen der Messquerschnitte die fr die automatische Erzeugung der
	 * Ursacheneinheiten verwendet werden sollen
	 * @return Namen der Messquerschnitte
	 */
	private List<String> bestimmeUFDSbaA8 (String pid)
	{
		System.out.println("VerwaltungsTabelleComp.bestimmeUFDSbaA8() " + pid);
		
		List<String> namen = new ArrayList<String>();
		
		if (pid.equals ("typ.sswSbaUeNssewarnung"))
		{
			for (int i=1; i<=11; ++i)
			{
			  String s = String.format("UDE%02d", i );
			  
			  namen.add("Nsse120-Nass1_" + s + "_Tag");
			  namen.add("Nsse120-Nass1_" + s + "_Nacht");

			  namen.add("Nsse100-Nass2_" + s + "_Tag");
			  namen.add("Nsse100-Nass2_" + s + "_Nacht");
			  
			  namen.add("Nsse80-Nass2-v_" + s + "_Tag");
			  namen.add("Nsse80-Nass2-v_" + s + "_Nacht");

			  namen.add("Nsse80-Nass3_" + s + "_Tag");
			  namen.add("Nsse80-Nass3_" + s + "_Nacht");

			  namen.add("Nsse60-Nass4-5_" + s + "_Tag");
			  namen.add("Nsse60-Nass4-5_" + s + "_Nacht");

			  namen.add("Nsse60-Nass2-3-v_" + s + "_Tag");
			  namen.add("Nsse60-Nass2-3-v_" + s + "_Nacht");

			}
		}

		if (pid.equals ("typ.sswSbaUeNebelwarnung"))
		{
			for (int i=1; i<=11; ++i)
			{
			  String s = String.format("UDE%02d", i );
			  
			  namen.add("Sicht100-Stufe1_" + s + "_Tag");
			  namen.add("Sicht100-Stufe1_" + s + "_Nacht");

			  namen.add("Sicht100-Stufe2_" + s + "_Tag");
			  namen.add("Sicht100-Stufe2_" + s + "_Nacht");

			  namen.add("Sicht80-Stufe3_" + s + "_Tag");
			  namen.add("Sicht80-Stufe3_" + s + "_Nacht");

			  namen.add("Sicht60-Stufe4_" + s + "_Tag");
			  namen.add("Sicht60-Stufe4_" + s + "_Nacht");

			  namen.add("Sicht40-Stufe5_" + s + "_Tag");
			  namen.add("Sicht40-Stufe5_" + s + "_Nacht");

			}
		}

		if (pid.equals("typ.sswSbaUeGlttewarnung"))
		{
			for (int i=1; i<=11; ++i)
			{
			  String s = String.format("UDE%02d", i );
			  
			  namen.add("Gltte_" + s);
			}
		}
		
		return namen;
	}
}
