/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceStore;

import de.bsvrz.sba.plugin.dynobj.Activator;

/**
 * Mit Hilfe dieser Klasse ist es mglich, direkt auf die im
 * {@link PreferenceStore} hinterlegten Zuordnungen von Objekttypen zu
 * Konfigurationsbereichen zuzugreifen.
 * 
 * @author HofmannJ
 * 
 */
public class ConfigurationAreaDirectAccessPreferenceStore
{
	/**
	 * PreferenceStore
	 */
	private IPreferenceStore _preferenceStore = null;

	/**
	 * Controller
	 */
	private ConfigurationAreaControler _controller = null;

	/**
	 * Privater Konstruktor. Holt den {@link PreferenceStore} des Plug-Ins und
	 * initialisiert einen {@link ConfigurationAreaControler}.
	 */
	private ConfigurationAreaDirectAccessPreferenceStore()
	{
		_preferenceStore = Activator.getDefault().getPreferenceStore();
		_controller = new ConfigurationAreaControler();
	}

	/**
	 * Private, innere Klasse. Speichert die Singleton-Instanz der Klasse
	 * {@link ConfigurationAreaDirectAccessPreferenceStore}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Inner
	{
		private static ConfigurationAreaDirectAccessPreferenceStore INSTANCE = new ConfigurationAreaDirectAccessPreferenceStore();
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse
	 */
	public static ConfigurationAreaDirectAccessPreferenceStore getInstance()
	{
		return Inner.INSTANCE;
	}

	/**
	 * Liefert das im {@link PreferenceStore} hinterlegte
	 * {@link ConfigurationAreaModel} zurck.
	 * 
	 * @return Im {@link PreferenceStore} hinterlegtes
	 *         {@link ConfigurationAreaModel}
	 */
	public ConfigurationAreaModel getModel()
	{
		_controller.readFromPreferenceStore(_preferenceStore);
		ConfigurationAreaModel model = _controller.getModel();
		return model;
	}

	/**
	 * Liefert zu einem ObjektTyp die Pid des zugehrigen Konfigurationbereiches
	 * an Hand der im {@link PreferenceStore} hinterlegten
	 * {@link ConfigurationAreaModel}s.
	 * 
	 * @param pidObjectType
	 *          Pid des ObjektTyps, dessen Konfigurationsbereich bestimmt werden
	 *          soll.
	 * @return Pid des zugehrigen Konfigurationsbereiches oder <code>null</code>,
	 *         wenn zu diesem Objekttyp noch kein Konfigurationsbereich zugeordnet
	 *         worden ist.
	 */
	public String getPidConfigurationAreaFromObjectType(String pidObjectType)
	{
		return getModel().getPidConfigurationArea(pidObjectType);
	}

}
