/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorModell;


import java.util.ArrayList;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Interface fr die Elemente des Navigators
 * 
 * @author Krause
 * 
 */
public interface INavigatorElement extends ClientReceiverInterface
{
  /**
   * mgliche Stati des NavigatorElements
   */
  public enum STATUS
    {
      unbekannt, IO, teilweiseIO, teilweiseStoerung, TeilStoerung, Stoerung,
      ungueltig, zeitlich_gueltig, verkehrlich_gueltig, zeitverkehrlich_gueltig,
      inaktiv_automatik, aktiv_automatik, inaktiv_manuell, aktiv_manuell
    };

  /**
   * Liefert das Flag fr Ausgeklappt
   * 
   * @return Ausgeklappt
   */
  public boolean isAusgeklappt();

  /**
   * Setzt das Flag fr Ausgeklappt
   * 
   * @param b
   */
  public void setAusgeklappt( boolean b );

  /**
   * Liefert den Status
   * 
   * @return Status
   */
  public STATUS getStatus();

  /**
   * Setzt den Status und das entsprechende Bild
   * sowie den Status des Elternteils und die Aktualisierung des Teilbaums
   * 
   * @param status
   */
  public boolean setStatus( STATUS status );

  /**
   * Setzt den Status automatisch, entsprechend den Stati der Kinder
   */
  public boolean setAutoStatus();

  /**
   * Liefert die Kinder
   * 
   * @return Children
   */
  public Object[] getChildren();

  /**
   * Setzt die Kinder
   * 
   * @param children
   */
  public void setChildren( ArrayList<Object> children );

  /**
   * Setzt ein Kind
   * 
   * @param child
   */
  public void addChild( INavigatorElement child );

  /**
   * Liefert den Elternteil
   * 
   * @return Parent
   */
  public Object getParent();

  /**
   * Setzt den Namen
   * 
   * @param name
   */
  public void setName( String name );

  /**
   * Setzt den Elternteil
   * 
   * @param parent
   */
  public void setParent( INavigatorElement parent );

  /**
   * Liefert den Bildpfad
   * 
   * @return Image
   */
  public String getImagePath();

  /**
   * Setzt den Bildpfad
   * 
   * @param bild
   */
  public void setImagePath( String bildPfad );

  /**
   * Liefert den Text
   * 
   * @return Text
   */
  public String getName();

  /**
   * Liefert die Objekte zum Element
   * 
   * @return SystemObjekt
   */
  public LinkedList<SystemObject> getObjekte();

  /**
   * Besitzt Kinder
   * 
   * @return boolean
   */
  public boolean hasParent();

  /**
   * Besitzt Elternteil
   * 
   * @return boolean
   */
  public boolean hasChildren();

  /**
   * Besitzt das Objekt
   * 
   * @return boolean
   */
  public boolean hasObjekt( SystemObject objekt );

  //------------------------------------------------------------
  // Methoden fr nderungen im DatenModell
  //------------------------------------------------------------

  /**
   * Listener setzen
   */
  public void addListener( IDeltaListener listener );

  /**
   * Listener entfernen
   */
  public void removeListener( IDeltaListener listener );

  /**
   * Event beim Listener updaten
   */
  public void upd( INavigatorElement toUpd );

  /**
   * Kind zum Element hinzufgen
   */
  public void add( INavigatorElement toAdd );

  /**
   * Kind vom Element entfernen
   */
  public void remove( INavigatorElement toRemove );

}
