package de.bsvrz.sba.plugin.parameter.steuerungskern.dialog;

import java.util.List;
import java.util.Timer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;

/**
 * Definitionsdialog zur Auswahl der verfgbaren Anzeigeinhalte. <br>
 * 
 * @author Krause
 *
 */
public class AnzeigeInhaltAuswahlDialog extends Dialog
{

  /**
   * Singleton-Instanz
   */
  private static AnzeigeInhaltAuswahlDialog _instanz = null;

  private Shell shell = null;

  private AnzeigeElementTyp _typ = null;

  private List<AnzeigeInhaltTyp> _inhalte = null;

  private AnzeigeInhaltTyp _inhalt = null;

  private Label[] _labels;

  private MouseAdapter _mouseAdapter;

  private MouseTrackListener _mouseTrackListener;

  private FocusAdapter _focusAdapter;

  private boolean _schalten = true;

  /**
   * Das Timer Objekt
   */
  private Timer timer = new Timer();

  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link AnzeigeInhaltAuswahlDialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public AnzeigeInhaltAuswahlDialog(Shell parent)
  {
    this(parent, true);
  }

  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link AnzeigeInhaltAuswahlDialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   * @param schalten  
   *          Auswahl mglich
   */
  public AnzeigeInhaltAuswahlDialog(Shell parent, boolean schalten)
  {
    super(parent);
    _instanz = this;
    _schalten = schalten;
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public AnzeigeInhaltTyp oeffne(AnzeigeElementTyp typ)
  {
    return oeffne( typ, AnzeigeElementFigure.createAnzeigeInhalte(typ) );
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public AnzeigeInhaltTyp oeffne(AnzeigeElementTyp typ, boolean mitDontCare, boolean mitBeliebig, boolean mitNegiertenInhalten )
  {
    return oeffne( typ, AnzeigeElementFigure.createAnzeigeInhalte(typ, mitBeliebig, mitDontCare, mitNegiertenInhalten) );
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public AnzeigeInhaltTyp oeffneMitBeliebig (AnzeigeElementTyp typ)
  {
    return oeffne( typ, AnzeigeElementFigure.createAnzeigeInhalteMitBeliebig(typ) );
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public AnzeigeInhaltTyp oeffneMitBeliebigUndNegiertenInhalten (AnzeigeElementTyp typ)
  {
    return oeffne( typ, AnzeigeElementFigure.createAnzeigeInhalteMitBeliebigundNegiertenInhalten(typ) );
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public AnzeigeInhaltTyp oeffneMitDontCare (AnzeigeElementTyp typ)
  {
    return oeffne( typ, AnzeigeElementFigure.createAnzeigeInhalteMitDontCare(typ) );
  }

  /**
   * ffnet den Dialog {@link AnzeigeInhaltAuswahlDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public AnzeigeInhaltTyp oeffne(AnzeigeElementTyp typ, List<AnzeigeInhaltTyp> inhalte)
  {
    _typ = typ;
    _inhalte = inhalte;

    shell = new Shell(getParent(), SWT.NONE);

    _focusAdapter = new FocusAdapter() {

      @Override
      public void focusLost(FocusEvent e)
      {
        dispose();
      }
    };

    shell.addFocusListener(_focusAdapter);

    _mouseTrackListener = new MouseTrackAdapter() {

      @Override
      public void mouseExit(MouseEvent e)
      {
        if (e.getSource() instanceof Shell)
        {

          Shell s = (Shell)e.getSource();
          Rectangle ca = s.getClientArea();

          if (true) // Dialog bei Verlassen mit Mauszeiger schliessen
          {
            if (e.x < 0 || e.x >= ca.width || e.y < 0 || e.y >= ca.height)
            {
              dispose();
            }
          }
          else
          // Mauszeiger am Verlassen des Dialogbereiches hindern (schlechtes Verhalten bei Eclipse !!!)
          {
            if (e.x < 0 || e.x >= ca.width || e.y < 0 || e.y >= ca.height)
            {
              int x = e.x;
              int y = e.y;

              if (e.x <= 0)
              {
                x = 2;
              }

              if (e.x >= ca.width)
              {
                x = ca.width - 3;
              }

              if (e.y <= 0)
              {
                y = 2;
              }

              if (e.y >= ca.height)
              {
                y = ca.height - 3;
              }

              Point p = shell.toDisplay(x, y);

              Display.getCurrent().setCursorLocation(p);
            }
          }

        }
        else if (e.getSource() instanceof Label)
        {
          Point pt = shell.toDisplay(e.x, e.y);
          timer.schedule(new SchliesseZeitGesteuert(_instanz, pt), 100);

        }
      }

    };

    shell.addMouseTrackListener(_mouseTrackListener);

    RowLayout rowLayout = new RowLayout();
    rowLayout.type = org.eclipse.swt.SWT.HORIZONTAL;

    rowLayout.wrap = true;

    shell.setLayout(rowLayout);

    createCompositeImage();

    Display display = shell.getDisplay();
    Point p = display.getCursorLocation();

    /*
     * Gre des Dialogs in Abhngigkeit der anzuzeigenden Bilder bestimmen
     */
    Image image = AnzeigeElementFigure.getImage(typ, typ.getDefaultBild());
    int h = image.getBounds().height;
    int b = image.getBounds().width;
    
    int anzahlBilderProZeile = 6;
    if (inhalte.size() < anzahlBilderProZeile)
    	anzahlBilderProZeile = inhalte.size();
    
    int anzahlZeilen = inhalte.size() / anzahlBilderProZeile;
    if ((inhalte.size() % anzahlBilderProZeile) != 0)
    	anzahlZeilen++;

    int breite = 211;
    int hoehe  = 120;
    
    breite = anzahlBilderProZeile * (b + 5);
    hoehe  = anzahlZeilen         * (h + 5);
    
    if (anzahlZeilen == 1)
    	hoehe += 5;
    
//    if (typ == AnzeigeElementTyp.WZG_A)
//    {
//    	breite = 300;
//    	hoehe  = 120;
//    }
    
    shell.setBounds(new Rectangle(p.x - 10, p.y - 10, breite, hoehe));

    shell.open();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _inhalt;

  }
  
  /**
   * This method initializes compositeImage
   * 
   */
  private void createCompositeImage()
  {
    _mouseAdapter = new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        if (e.button == 1)
        {
          if (_schalten){
            Label l = (Label)e.widget;
            AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)l.getData();
            _inhalt = inhalt;            
          }
          dispose();
        }
      }
    };

    _labels = new Label[_inhalte.size()];

    int i = 0;

    for (AnzeigeInhaltTyp inhalt : _inhalte)
    {
      _labels[i] = new Label(shell, SWT.None);      
      _labels[i].addMouseListener(_mouseAdapter);      
      _labels[i].addMouseTrackListener(_mouseTrackListener);

      Image image = AnzeigeElementFigure.getImage(_typ, inhalt);
      
      if (image != null)
      {
        _labels[i].setData(inhalt);
        _labels[i].setImage(image);

      }
      else
      {
        System.out.println("Path ist null");
      }

      i++;

    }

  }

  public void dispose()
  {

    // _label.removeMouseListener(_adapter);
    // _label.dispose();
    if (!shell.isDisposed()){
      shell.removeFocusListener(_focusAdapter);
      shell.removeMouseTrackListener(_mouseTrackListener);
      for (Label l : _labels)
      {
        if (l != null)
        {
          l.removeMouseListener(_mouseAdapter);
          l.removeMouseTrackListener(_mouseTrackListener);
        }
      }      
      shell.dispose();
    }
  }

  public Shell getShell()
  {
    return shell;
  }

  public void setShell(Shell shell)
  {
    this.shell = shell;
  }

}
