package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert die betigten Informationen eines Objekts von Typ 'typ.sbaAnzeige'
 * @author ObertM
 *
 */
public class SBAAnzeige
{
	/**
	 * Systemobjekt im DaV
	 */
  SystemObject _systemObjekt;

  /**
   * Anzeige auf die die SBA Anzeige verweist
   */
  SystemObject _anzeige;
  
  /**
   * WZG Typ der SBA Anzeige
   */
  SystemObject _wzgTyp;
  
  /**
   * Verbindung zum DaV
   */
	private ClientDavInterface _connection;
  
	/**
	 * Konstruktor der Klasse
	 * @param so Systemobjekt im DaV
	 */
  public SBAAnzeige (SystemObject so)
  {
  	_connection = DaVVerbindung.getInstanz().getDaVVerbindung();
  	
  	_systemObjekt = so;
  	
  	bestimmeKonfigurierendeEigenschaften();
  }

	/**
   * Methode liefert systemObjekt zurck
   * @return systemObjekt
   */
  public SystemObject getSystemObjekt()
  {
  	return _systemObjekt;
  }

	/**
   * Methode setzt die Variable systemObjekt
   * @param systemObjekt zu bernehmender Wert fr systemObjekt
   */
  public void setSystemObjekt(SystemObject systemObjekt)
  {
  	_systemObjekt = systemObjekt;
  }
  
  /**
   * Methode bestimmt die konfigurierenden Eigenschaften der SbaAnzeige
   */
  private void bestimmeKonfigurierendeEigenschaften ()
  {
    // Datenmodell bestimmen
    
    DataModel dataModel = _connection.getDataModel();
    
    // Konfiguration auslesen
    
    AttributeGroup atg  = dataModel.getAttributeGroup("atg.sswSbaAnzeige");
    
    if (atg != null)
    {
      Data confData = _systemObjekt.getConfigurationData(atg);
    
      if (confData != null)
      {
        // Referenz auf ein Objekt vom Typ 'typ.anzeige'. 
        
        _anzeige = confData.getReferenceValue("Anzeige").getSystemObject();
        
        // Optionale Referenz auf einen Anzeigetyp (WZG Typ A, B, C, Blinker usw.). 
        
        _wzgTyp = confData.getReferenceValue("Typ").getSystemObject();
      }
    }
  }

	/**
   * Methode liefert anzeige zurck
   * @return anzeige
   */
  public SystemObject getAnzeige()
  {
  	return _anzeige;
  }

	/**
   * Methode liefert wzgTyp zurck
   * @return wzgTyp
   */
  public SystemObject getWzgTyp()
  {
  	return _wzgTyp;
  }
}
