/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.SendeParameter;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger.ParameterVorgaengerNachfolger;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger.VerwaltungParaVorgaengerNachfolger;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeQuerschnittTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse beinhaltet die Parameter mit den Anzeigequerschnitten die in den einzelnen Regeln verwendet
 * werden knnen. 
 * @author ObertM
 *
 */
public class VerwaltungMoeglicheAnzeigeQuerschnitte
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungMoeglicheAnzeigeQuerschnitte _instance;

  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);
  
  /**
   * Pid der angeforderten Attributgruppe
   */
  public static String _pidAtg = "atg.sswSbaMoeglicheAnzeigeQuerschnitte";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Hashmap mit den Zuordnungen
   */
  private static HashMap<SystemObject, AnzeigeQuerschnittMitTyp> _zuordnung = new HashMap<SystemObject, AnzeigeQuerschnittMitTyp>();
  
  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungMoeglicheAnzeigeQuerschnitte ()
  {
  	_pidSBA = ProjektSpezifischeEinstellungen.getPidSba();
    
    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
		
		pruefeAufDefaultWerte ();
		
//		pruefeQuerBeziehung ();
  }
  
  /**
   * Methode prft ob die Querbeziehungend der AQs aus der Vorgnger- Nachfolgerbeziehung
   * mit den Defaultvorgaben bereinstimmt (nur fr Test)
   */
  private void pruefeQuerBeziehung()
  {
	  for (SystemObject so : getAnzeigeQuerschnitte())
	  {
	  	try
	  	{
	  		pruefeQuerBeziehung(so);
	  	}
	  	catch (Exception e) 
	  	{
				e.printStackTrace();
			}
	  }
  }

  /**
   * Methode prft ob die Querbeziehungend eines AQ aus der Vorgnger- Nachfolgerbeziehung
   * mit den Defaultvorgaben bereinstimmt (nur fr Test)
   * @param soAQ Systemobjekt des zu prfenden AQ
   */
	private void pruefeQuerBeziehung(SystemObject soAQ)
  {
  	VerwaltungParaVorgaengerNachfolger verw = VerwaltungParaVorgaengerNachfolger.getInstance();
  	
  	ParameterVorgaengerNachfolger para = verw.getParameter();
  	
  	SystemObject[] anzeigenAusQL = para.getQuerbeziehung(soAQ);
  	
  	SystemObject[] anzeigenAusDefault = DefaultEinstellungen.getDefaultQuerBeziehung(soAQ);

  	boolean fehler = false;
  	
  	int max = anzeigenAusDefault.length > anzeigenAusQL.length ? anzeigenAusDefault.length : anzeigenAusQL.length; 

		String s = "";
  	
  	for (int i=0; i<max; ++i)
  	{
  		if ((i < anzeigenAusDefault.length) && (i < anzeigenAusQL.length))
  		{
  			if (anzeigenAusDefault [i] != anzeigenAusQL [i])
  			{
  				if (!s.equals(""))
  					s += "\n";
  				
  				s += "- Unterschied in Anzeige " + i + ": Default: ";
  				
  				if (anzeigenAusDefault [i] != null)
  				  s += anzeigenAusDefault [i].getName();
  				else
  					s += "<null>";
  				
  				s += ": QL: ";

  				if (anzeigenAusQL [i] != null)
  				  s += anzeigenAusQL [i].getName();
  				else
  					s += "<null>";
  		
  				fehler = true;
  			}
  		}
  		else
  		{
  			if (!s.equals(""))
  				s +="\n";
  			
        if (i < anzeigenAusDefault.length)
        {
        	s += "- Default: " + anzeigenAusDefault [i].getName() + " --- QL: keine Anzeige";
        	fehler = true;
        }
        else if (i < anzeigenAusDefault.length)
        {
        	s += "- Default: keien Anzeige --- QL: " + anzeigenAusQL [i].getName();
        	fehler = true;
        }
  		}
  	}
  	
  	if (fehler)
  	{
    	System.out.println("Prfe " + soAQ.getName() + " " + getAnzeigeQuerschnittTyp(soAQ).getName());
    	System.out.println(s);
  	}
  }

	/**
   * Methode prft ob der Parameter im DaV vorhanden und versorgt ist. Wenn nicht werden, sofern
   * vorhanden, Defaultdaten fr den Parametersatz erzeugt und in den DaV geschrieben. 
   */
  private void pruefeAufDefaultWerte()
  {
  	if (_zuordnung.size() == 0)
  	{
  		ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
  		DataModel model = connection.getDataModel();
  		
      System.err.println("Parameter " + _pidAtg + " fr " + _pidSBA + " nicht definiert. Es wird ein" +
                     		 "Defaultdatensatz erzeugt ! " + ProjektSpezifischeEinstellungen.getInstance().getSba());
     
      HashMap<String, EAnzeigeQuerschnittTyp> querschnitte = DefaultEinstellungen.getDefaultAnzeigeQuerschnitte();
      
      Iterator<Entry<String, EAnzeigeQuerschnittTyp>> it = querschnitte.entrySet().iterator();
      while (it.hasNext())
      {
        Entry<String, EAnzeigeQuerschnittTyp> entry = it.next();	
      	SystemObject so = model.getObject(entry.getKey());
      	if (so != null)
      		_zuordnung.put(so, new AnzeigeQuerschnittMitTyp(so,entry.getValue()));
      }
    
 			speichereInDaV();
  	}
  }

  /**
   * Methode zum Speichern der Parameter im DaV
   */
  private void speichereInDaV ()
  {
	  Data data = getData();
	  
    if (data != null)
    {
      SendeParameter sende = new SendeParameter(_pidAtg);
      sende.sendeParameter(data);
    }
  }
  
	/**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungMoeglicheAnzeigeQuerschnitte getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungMoeglicheAnzeigeQuerschnitte.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungMoeglicheAnzeigeQuerschnitte ();
        }
      }
    }
    
    return _instance;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaZuordnungWVZInhalte'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData result )
  {
    if (result.hasData())
    {
      Data data = result.getData();
      
      Array a = data.getArray("AnzeigeQuerschnitte");
      
      for (int i=0; i<a.getLength(); ++i)
      {
      	Data aktuell = a.getItem(i);
      	
      	SystemObject so = aktuell.getReferenceValue("AnzeigeQuerschnitte").getSystemObject();
      	int kodierung = aktuell.getUnscaledValue("Typ").intValue();
      	
        EAnzeigeQuerschnittTyp typ = EAnzeigeQuerschnittTyp.getTyp(kodierung); 	
        
        _zuordnung.put(so, new AnzeigeQuerschnittMitTyp (so, typ));
      }
    }
  }
	
  /**
   * Methode zum Speichern der Parameter im DaV
   * @param abhaengigkeiten Abhngigkeiten
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(_pidAtg);
    Data data = connection.createData(atg );
    
    Array a = data.getArray("AnzeigeQuerschnitte");
    a.setLength(_zuordnung.size());
    
    int i = 0;
    
    Iterator<Entry<SystemObject, AnzeigeQuerschnittMitTyp>> it = _zuordnung.entrySet().iterator();
    while (it.hasNext())
    {
    	Entry<SystemObject, AnzeigeQuerschnittMitTyp> entry = it.next();
    	
    	SystemObject so = entry.getKey();
    	int kodierung = entry.getValue().getTyp().getKodierung();
    	
    	Data aktuell = a.getItem(i++);
    	
    	aktuell.getReferenceValue("AnzeigeQuerschnitte").setSystemObject(so);
    	aktuell.getUnscaledValue("Typ").set(kodierung);
    }
    
    return data;
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

	/**
   * Methode liefert zuordnung zurck
   * @return zuordnung
   */
  public HashMap<SystemObject, AnzeigeQuerschnittMitTyp> getZuordnung()
  {
  	return _zuordnung;
  }

  /**
   * Methode liefert eine Liste mit den mglichen Anzeigequerschnitten
   * @return Liste mit Anzeigequerschnitten
   */
  public List<SystemObject> getAnzeigeQuerschnitte ()
  {
  	List<SystemObject> l = new ArrayList<SystemObject>();
  	
  	l.addAll (_zuordnung.keySet());
  	
  	return l;
  }

  /**
   * Methode liefert eine Liste mit den mglichen Anzeigequerschnitten
   * @return Liste mit Anzeigequerschnitten
   */
  public List<AnzeigeQuerschnittMitTyp> getMoeglicheAnzeigeQuerschnitte ()
  {
  	List<AnzeigeQuerschnittMitTyp> l = new ArrayList<AnzeigeQuerschnittMitTyp>();
  	
  	l.addAll (_zuordnung.values());
  	
  	return l;
  }

  /**
   * Methode liefert den Anzeigequerschnitttyp zu einem AQ
   * @param soAQ Systemobjekt des AQ
   * @return Typ
   */
  public static EAnzeigeQuerschnittTyp getAnzeigeQuerschnittTyp (SystemObject soAQ)
  {
  	if (_zuordnung.containsKey(soAQ))
  	  return _zuordnung.get(soAQ).getTyp();
  	
  	return null;
  }

  /**
   * Methode zum Setzen der Anzeigequerschnitte
   * @param moeglicheAnzeigeQuerschnitte mgliche Anzeigequerschnitte
   */
	public void setMoeglicheAnzeigeQuerschnitte(
      List<AnzeigeQuerschnittMitTyp> moeglicheAnzeigeQuerschnitte)
  {
		_zuordnung.clear();
		
		for (AnzeigeQuerschnittMitTyp aqmt : moeglicheAnzeigeQuerschnitte)
	   _zuordnung.put(aqmt.getAnzeigeQuerschnitt(), aqmt);
  }
}
