package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.aufhebungen.aufhebung;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter Aufhebung
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Parameter X (zu prfender Anzeigeinhalt)
   */
  private Composite _x;
  
  /**
   * Parameter t1 (Zeitpunkt t1) 
   */
  private Composite _t1;

  /**
   * Parameter t2 (Zeiptpunkt t2)
   */
  private Composite _t2;

  /**
   * Parameter A (zu schaltender Anzeigeninhalt)
   */
  private Composite _a;

  /**
   * Konstruktor der Klasse
   * @param loeschen Lschen
   * @param schaltgrund Schaltgrund
   * @param x zu prfender Anzeigeninhalt
   * @param t1 Zeitpunkt t1
   * @param t2 Zeitpunkt t2
   * @param a zu schaltender Anzeigeninhalt
   */
  public TableModel(Composite loeschen, Composite schaltgrund, Composite x, Composite a, Composite t1, Composite t2)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._x  = x;    
    this._t1 = t1;    
    this._t2 = t2;    
    this._a  = a;
  }

  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }

  public String getSelectedT1 ()
  {
    return getSelectedWert(_t1);    
  }

  public String getSelectedT2 ()
  {
    return getSelectedWert(_t2);    
  }

  public String getSelectedA ()
  {
    return getSelectedWert(_a);    
  }

  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }

  public void setSelectedA(Image selImage)
  {
    setSelectedWert(_a, null, selImage);
  }

  public String getSelectedWert(Composite c)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode setzt die Variable x
   * @param x zu bernehmender Wert fr x
   */
  public void setX(Composite x)
  {
  	_x = x;
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode liefert t1 zurck
   * @return t1
   */
  public Composite getT1()
  {
  	return _t1;
  }

	/**
   * Methode setzt die Variable t1
   * @param t1 zu bernehmender Wert fr t1
   */
  public void setT1(Composite t1)
  {
  	_t1 = t1;
  }

	/**
   * Methode liefert t2 zurck
   * @return t2
   */
  public Composite getT2()
  {
  	return _t2;
  }

	/**
   * Methode setzt die Variable t2
   * @param t2 zu bernehmender Wert fr t2
   */
  public void setT2(Composite t2)
  {
  	_t2 = t2;
  }

	/**
   * Methode liefert a zurck
   * @return a
   */
  public Composite getA()
  {
  	return _a;
  }

	/**
   * Methode setzt die Variable a
   * @param a zu bernehmender Wert fr a
   */
  public void setA(Composite a)
  {
  	_a = a;
  }

}
