/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Ausfallbehandlung", Regel "Ausfall anderes Zeichen".<br>
 * Regel:<br>
 * Wenn an einer Anzeige vom Typ B der Anzeigeinhalt = X ist und<br>
 * Inhalt X an dieser Anzeige nicht darstellbar ist und <br>
 * Inhalt X an keiner anderen Anzeige dieses AQ dargestellt wird (dieser Regelteil muss jeweils abschaltbar sein)<br>
 * dann aus anfordern (entweder an keiner Anzeige, an dieser Anzeige oder an allen Anzeigen des gleichen Typs des AQ)
 * und an allen Anzeigen vom Typ A (wahlweise an diesem AQ oder in Lngsvorgnger / Lngsnachfolgerichtung) 
 * das erste Zeichen aus der Reihe  X, Y, Z anzeigen, welches dort darstellbar ist. 
 * @version $Revision: 1.1 $ / $Date: 2013/06/25 09:27:12 $ / ($Author: ObertM $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class VerwaltungParaAusfallAnderesZeichen
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaAusfallAnderesZeichen _instance;

  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;
  
  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaAusfallAnderesZeichen";

  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz fr die Anlage
   */
  private RegelnProAnzeigeTyp m_parameterAnlage = new RegelnProAnzeigeTyp ();
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Parametersatz pro Anzeige
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeige = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der Anlage
   */
  private String m_anlage;
  
  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaAusfallAnderesZeichen";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaAusfallAnderesZeichen ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    
    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaAusfallAnderesZeichen getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaAusfallAnderesZeichen.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaAusfallAnderesZeichen ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(this.toString());
  }

  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaTrichterung'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_parameterAnlage.clear();
        m_parameterProAnzeigeQuerschnitt.clear();
        m_parameterProAnzeige.clear();
        
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
        
        a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
        
        a = data.getArray( "Anzeige");
        auswerteAttributgruppeAnzeige (a);
      }
      
      m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    }
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
    m_parameterAnlage.auswerteAttributgruppe( array, getSchaltgrundAllgemein() );
  }

  /**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeQuerschnittRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeigeQuerschnitt.put (anzeigeQuerschnitt, regeln);
      }
    }
  }

  /**
   * Methode zum Auswerten der Anzeige spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeige
   */
  private void auswerteAttributgruppeAnzeige (Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeige)
    {
      m_parameterProAnzeige.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeige = d1.getReferenceValue( "Anzeige" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeige.put (anzeige, regeln);
      }
    }
  }

  /**
   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
   * @param wvzTyp WVZ Typ fr den die Regel gelten soll
   * @param richtung Richtung in die die Regel arbeitet
   * @param anzeigeInhalt zu prfender Anzeigeinhalt ("X")
   * @param angeforderterAnzeigeInhalt angeforderter Anzeigeinhalt ("aus")
   * @param auswahlAnzeige Welche Anzeige soll geschaltet werden
   * @param pruefeAndereZeichen Sollen die anderen Zeichen geprft werden
   * @param anzeigeTypZusaetzlich zustzlicher Anzeigetyp ("Typ A")
   * @param auswahlAnzeigeQuerschnitt Welche Anzeigequerschnitte sollen geschaltet werden
   * @param angeforderterAnzeigenInhaltZusaetzlich angeforderter Anzeigeninhalte ("X", "Y", "Z");
   * @param anzeigeTypVerschiebung Anzeigetyp Verschiebung ("Typ B")
   * @param angeforderterAnzeigenInhaltVerschiebung angeforderter Anzeigeninhalte ("X", "Y", "Z");
   * @param ebene Ebene bis zu der verschoben wird
   * @param schaltgrund Schaltgrund unter dem die Regel arbeitet
   */
  public void addRegelAllgemein ( SystemObject wvzTyp,
                                  ArbeitsRichtungVN richtung,      
                                  WVZInhalt anzeigeInhalt, 
                                  WVZInhalt angeforderterAnzeigeInhalt,
                                  SelektionAnzeige auswahlAnzeige,
                                  boolean pruefeAndereZeichen,
                                  SystemObject anzeigeTypZusaetzlich,
                                  SelektionAnzeigeQuerschnitt auswahlAnzeigeQuerschnitt,
                                  Vector<WVZInhalt> angeforderterAnzeigenInhaltZusaetzlich,
                                  SystemObject anzeigeTypVerschiebung,
                                  Vector<WVZInhalt> angeforderterAnzeigenInhaltVerschiebung,
                                  int ebene,
                                  Aspect schaltgrund )
  {
    ListeMitRegeln regeln = m_parameterAnlage.getParameter( wvzTyp );
    regeln.addRegel( richtung,
                     anzeigeInhalt, 
                     angeforderterAnzeigeInhalt, 
                     auswahlAnzeige, 
                     pruefeAndereZeichen, 
                     anzeigeTypZusaetzlich, 
                     auswahlAnzeigeQuerschnitt, 
                     angeforderterAnzeigenInhaltZusaetzlich, 
                     anzeigeTypVerschiebung, 
                     angeforderterAnzeigenInhaltVerschiebung, 
                     ebene,
                     schaltgrund );
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "ParaAusfallAnderesZeichen:\n";
    
    s += "Parameter Ausfall Anderes Zeichen\n";
    s += "=================================\n";
    
    synchronized (m_mutex)
    {
      s += "Parameter Anlage:\n";
      s += "=================\n";
      s += m_parameterAnlage.toString();
      
      s += "Parameter pro Anzeigequerschnitt:\n";
      s += "=================================\n";
      Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
        
        SystemObject anzeigeQuerschnitt = (SystemObject) entry.getKey();
        RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
        
        s += Tools.getObjektName( anzeigeQuerschnitt )  + "\n";
        s += regeln.toString();
      }
  
      s += "Parameter pro Anzeige:\n";
      s += "======================\n";
      it = m_parameterProAnzeige.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
        
        SystemObject anzeige = (SystemObject) entry.getKey();
        RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
        
        s += Tools.getObjektName( anzeige )  + "\n";
        s += regeln.toString();
      }
    }

    return s;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    bereinigeParameter();

    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    Array anzeigeQuerschnitt = data.getArray( "AnzeigeQuerschnitt" );
    erzeugeAnzeigeQuerschnittRegeln (anzeigeQuerschnitt);
    
    Array anzeige = data.getArray( "Anzeige" );
    erzeugeAnzeigeRegeln (anzeige);
    
    return data;
  }

  /**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    RegelnProAnzeigeTyp rpat = getParameterAnlage();
    erzeugeRegeln( array, rpat );
  }

  /**
   * Methode schreibt die Anzeigequerschnittregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeQuerschnittRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAq = getParameterProAnzeigeQuerschnitt();
    int anzahlAQ = paraAq.size();
    
    array.setLength( anzahlAQ );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeQuerschnittRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Anzeigeregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAnzeige = getParameterProAnzeige();
    int anzahlAnzeigen = paraAnzeige.size();
    
    array.setLength( anzahlAnzeigen );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Regeln pro Anzeigetyp in ein Array
   * @param array Array in das die Regeln geschrieben werden sollen
   * @param rpat Regeln
   */
  private void erzeugeRegeln( Array array, RegelnProAnzeigeTyp rpat )
  {
    HashMap<SystemObject, ListeMitRegeln> p = rpat.getParameter();
    int anzahl = p.size();
    
    array.setLength( anzahl );
    
    int i=0;
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = p.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, ListeMitRegeln> entry = it.next();
      
      SystemObject anzeigeTyp = entry.getKey();
      ListeMitRegeln regeln = entry.getValue();
      
      Data aktuell = array.getItem( i++ );
      
      aktuell.getReferenceValue( "AnzeigeTyp" ).setSystemObject( anzeigeTyp );
      Array r = aktuell.getArray( "Regeln" );
      
      Vector<RegelAusfallAnderesZeichen> parameter = regeln.getParameter();
      r.setLength( parameter.size() );
      
      for (int j=0; j<parameter.size(); ++j)
      {
        Data d = r.getItem( j );
        RegelAusfallAnderesZeichen rv = parameter.get( j );
        
        d.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rv.getSchaltgrundRegel() );
        d.getUnscaledValue ( "VorgngerNachfolger" ).set (rv.getRichtung().getKodierung());
        
        d.getTextValue ( "PruefeAndereAnzeigen" ).setText( rv.isPruefeAndereZeichen() ? "Ja" : "Nein" );

        SelektionAnzeige aA = rv.getAuswahlAnzeige();
        d.getUnscaledValue( "AuswahlAnzeige" ).set( aA.getKodierung() );
        
        d.getReferenceValue( "AnzeigeTypZustzlich" ).setSystemObject( rv.getAnzeigeTypZusaetzlich() );
        
        SelektionAnzeigeQuerschnitt aAQ = rv.getAuswahlAnzeigeQuerschnitt();
        d.getUnscaledValue( "AuswahlAnzeigeQuerschnitt" ).set( aAQ.getKodierung() );
        
        d.getReferenceValue( "AnzeigeTypVerschiebung" ).setSystemObject( rv.getAnzeigeTypVerschiebung() );
        
        d.getUnscaledValue( "Ebene" ).set (rv.getEbene());
        
        WVZInhalt inhalt = rv.getAnzeigenInhalt();
        inhalt.erzeugeAttributgruppe( d.getItem( "Anzeigeinhalt" ));

        WVZInhalt inhaltAngefordert = rv.getAngeforderterAnzeigenInhalt();
        inhaltAngefordert.erzeugeAttributgruppe( d.getItem( "AngeforderterAnzeigeInhalt" ));

        Vector<WVZInhalt> inhaltZusaetzlich = rv.getAngeforderterAnzeigenInhaltZusaetzlich();
        Array a1 = d.getArray( "AngeforderterAnzeigeInhaltZustzlich" );
        a1.setLength( inhaltZusaetzlich.size() );
        for (int k=0; k<inhaltZusaetzlich.size(); ++k)
          inhaltZusaetzlich.get( k ).erzeugeAttributgruppe( a1.getItem( k ) );

        Vector<WVZInhalt> inhaltVerschiebung = rv.getAngeforderterAnzeigenInhaltVerschiebung();
        Array a2 = d.getArray( "AngeforderterAnzeigeInhaltVerschiebung" );
        a2.setLength( inhaltVerschiebung.size() );
        for (int k=0; k<inhaltVerschiebung.size(); ++k)
          inhaltVerschiebung.get( k ).erzeugeAttributgruppe( a2.getItem( k ) );
      }
    }
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public RegelnProAnzeigeTyp getParameterAnlage()
  {
    return m_parameterAnlage;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeigeQuerschnitt zurck
   * @return Variable m_parameterProAnzeigeQuerschnitt
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeigeQuerschnitt()
  {
    return m_parameterProAnzeigeQuerschnitt;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeige zurck
   * @return Variable m_parameterProAnzeige
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeige()
  {
    return m_parameterProAnzeige;
  }

  /**
   * Methode lscht leere Parameterfelder. Dies ist notwendig, da sonst die Steuerung Problem
   * mit der Bestimmung der richtigen Parameterstze hat
   */
  private void bereinigeParameter()
  {
  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeigeQuerschnitt());

  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeige());
  }
  
  /**
   * Methode bereinigt die Parameter. Dabei werden Anzeigetypen fr die keine Regeln definiert sind
   * aus dem Parametersatz entfern.
   * @param regelnProAnzeigeTyp zu bereinigende Regeln
   */
  private void bereinigeRegelProAnzeigeTyp (HashMap<SystemObject, RegelnProAnzeigeTyp> regelnProAnzeigeTyp)
  {
  	List<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
  	
	  Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = regelnProAnzeigeTyp.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<SystemObject,RegelnProAnzeigeTyp> entry = it.next();
	  	
	  	SystemObject anzeigeQuerschnitt = entry.getKey();
	  	RegelnProAnzeigeTyp regeln = entry.getValue();
	  	
	  	regeln.bereinigeParameter();
	  	if (regeln.getParameter().size() == 0)
	  		zuLoeschen.add (anzeigeQuerschnitt);
	  }

	  for (SystemObject so : zuLoeschen)
	  	regelnProAnzeigeTyp.remove(so);
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }
  
  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
	public RegelnProAnzeigeTyp getParameter(String pid)
  {
    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
      return m_parameterAnlage;
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parameter Anzeige
     */
    for (SystemObject so : m_parameterProAnzeige.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeige.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return new RegelnProAnzeigeTyp();
  }
	
  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, RegelnProAnzeigeTyp regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    			m_parameterProAnzeigeQuerschnitt.put(so, regeln);

    		if (so.getType().getPid().equals("typ.anzeige"))
    			m_parameterProAnzeige.put(so, regeln);
    	}
    }
    
    benachrichtigeListener();
  }

}
