package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter Ausschaltregel
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Parameter X (zu prfender Inhalt)
   */
  private Composite _x;
  
  /**
   * Parameter A (WZG Typ der Nachbaranzeigen) 
   */
  private Composite _a;

  /**
   * Parameter Y (zu schaltende Anzeige)
   */
  private Composite _y;

  /**
   * Parameter Z (Prioritt)
   */
  private Composite _z;

  /**
   * Konstruktor der Klasse
   * @param loeschen
   * @param schaltgrund
   * @param x zu prfender Inhalt X
   * @param a WZG Typ der Nachbaranzeigen A
   * @param y zu schaltende Anzeige Y
   * @param z Prioritt Z
   */
  public TableModel(Composite loeschen, Composite schaltgrund, Composite x, Composite a, Composite y, Composite z)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._x = x;    
    this._a = a;    
    this._y = y;    
    this._z = z;    
  }

  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  
  
  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  
  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }
  
  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }

  
  public String getSelectedA ()
  {
    return getSelectedWert(_a);    
  }

  public void setSelectedA(String selWert)
  {
    setSelectedWert(_a, selWert, null);
  }

  
  public String getSelectedY ()
  {
    return getSelectedWert(_y);    
  }

  public void setSelectedY(Image selImage)
  {
    setSelectedWert(_y, null, selImage);
  }

  
  public void setSelectedZ (String selWert)
  {
    setSelectedWert(_z, selWert, null);
  }

  public int getSelectedZ()
  {
    Control control = _z.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int intPrio = sp.getSelection();
      return intPrio;
    }
    else
      return 0;
  }


  public String getSelectedWert(Composite c)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode setzt die Variable x
   * @param x zu bernehmender Wert fr x
   */
  public void setX(Composite x)
  {
  	_x = x;
  }

	/**
   * Methode liefert a zurck
   * @return a
   */
  public Composite getA()
  {
  	return _a;
  }

	/**
   * Methode setzt die Variable a
   * @param a zu bernehmender Wert fr a
   */
  public void setA(Composite a)
  {
  	_a = a;
  }

	/**
   * Methode liefert y zurck
   * @return y
   */
  public Composite getY()
  {
  	return _y;
  }

	/**
   * Methode setzt die Variable y
   * @param y zu bernehmender Wert fr y
   */
  public void setY(Composite y)
  {
  	_y = y;
  }

	/**
   * Methode liefert z zurck
   * @return z
   */
  public Composite getZ()
  {
  	return _z;
  }

	/**
   * Methode setzt die Variable z
   * @param z zu bernehmender Wert fr z
   */
  public void setZ(Composite z)
  {
  	_z = z;
  }

}
