package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.verdopplung;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private List<RegelVerdopplung> m_parameter = new ArrayList<RegelVerdopplung> ();

  /**
   * Parameterloser Konstruktor
   */
  public ListeMitRegeln ()
  {
    
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaVerdopplungProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelVerdopplung r = new RegelVerdopplung ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein);
        
        m_parameter.add( r );
      }
    }
  }

  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelVerdopplung> getParameter()
  {
  	Vector<RegelVerdopplung> v = new Vector<RegelVerdopplung>();
  	
  	synchronized (m_parameter)
    {
	    for (RegelVerdopplung r : m_parameter)
	    	v.add (r);
    }
  	
    return v;
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param x Anzeigeinhalt der aktuell betrachteten Anzeige 
   * @param richtung Richtung in der die Regel arbeitet
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (WVZInhalt x, ArbeitsRichtungVN richtung, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelVerdopplung r = new RegelVerdopplung ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setAnzeigenInhalt( x );
      r.setRichtung( richtung );
      
      m_parameter.add( r );
    }
  }
  
  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelVerdopplung r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode zum bernehmen der Parameter
   * @param parameter Zu bernehmende Parameter 
   */
	public void setParameter(Vector<RegelVerdopplung> parameter)
  {
	  synchronized (m_parameter)
    {
	    m_parameter.clear();
	    
	    for (RegelVerdopplung r : parameter)
	    	m_parameter.add(r);
    }
  }

}
