package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.verdopplung;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", 
 * Regel "Verdopplung". 
 * @author Swarco traffic System GmbH
 */
public class RegelVerdopplung
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;

  /**
   * arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   */
  private ArbeitsRichtungVN m_richtung = ArbeitsRichtungVN.VORGAENGER;

  /**
   * Anzeigeninhalt (in der Regel "X")
   */
  private WVZInhalt m_anzeigenInhalt = new WVZInhalt();

  /**
   * Parameterloser Konstruktor
   */
  public RegelVerdopplung ()
  {
  }

  /**
   * Konstruktor der Klasse
   * @param a Schaltgrund
   * @param r Arbeitsrichtung
   * @param x Anzeigeninhalt X
   */
  public RegelVerdopplung(Aspect a, ArbeitsRichtungVN r, WVZInhalt x)
  {
	  setSchaltgrundRegel(a);
	  setRichtung(r);
	  setAnzeigenInhalt(x);
  }

	/**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelVerdopplung'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    String vorgaengerNachfolger = data.getItem( "VorgngerNachfolger" ).asTextValue().getText();
    
    if (vorgaengerNachfolger.equals( "Vorgnger" ))
      setRichtung (ArbeitsRichtungVN.VORGAENGER);
    else
      setRichtung (ArbeitsRichtungVN.NACHFOLGER);
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
  }

  /**
   * Methode liefert die Variable anzeigenInhalt zurck
   * @return Variable anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }

  /**
   * Methode setzt die Variable anzeigenInhalt
   * @param anzeigenInhalt Wert fr anzeigenInhalt
   */
  public void setAnzeigenInhalt(WVZInhalt anzeigenInhalt)
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_richtung zurck
   * @return Variable m_richtung
   */
  public ArbeitsRichtungVN getRichtung()
  {
    return m_richtung;
  }

  /**
   * Methode setzt die Variable m_richtung
   * @param richtung Wert auf den die Variable m_richtung gesetzt werden soll
   */
  public void setRichtung( ArbeitsRichtungVN richtung )
  {
    m_richtung = richtung;
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel )
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Richtung: " + m_richtung + "\n";
    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    
    return s;
  }

}
