package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassen;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;
/**
 * Klasse enthlt eine Regel fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", 
 * Regel "Zusammenfassen".<br> 
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class RegelZusammenfassen
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * Anzeigeninhalt (in der Regel: "X")
   */
  private WVZInhalt  m_anzeigenInhalt = new WVZInhalt();
  
  /**
   * Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: "Y")
   */
  private WVZInhalt m_anzeigenInhaltVorgaengerNachfolger = new WVZInhalt();
  
  /**
   * angeforderter Anzeigeninhalt (in der Regel: "Z")
   */
  private WVZInhalt m_angeforderterAnzeigenInhalt = new WVZInhalt();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelZusammenfassen ()
  {
  }
  
  /**
   * Konstruktor der Klasse
   * @param a Schaltgrund
   * @param x Anzeigeinhalt X
   * @param y Anzeigeninhalt Vorgnger/Nachfolger Y
   * @param z angeforderter Anzeigeninhalt Z
   */
  public RegelZusammenfassen (Aspect a, WVZInhalt x,  WVZInhalt y, WVZInhalt z)
  {
  	setSchaltgrundRegel(a);
  	setAnzeigenInhalt(x);
  	setAnzeigenInhaltVorgaengerNachfolger(y);
  	setAngeforderterAnzeigenInhalt(z);
  }

	/**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }


  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }


  /**
   * Methode liefert die Variable m_anzeigenInhaltVorgaengerNachfolger zurck
   * @return Variable m_anzeigenInhaltVorgaengerNachfolger
   */
  public WVZInhalt getAnzeigenInhaltVorgaengerNachfolger()
  {
    return m_anzeigenInhaltVorgaengerNachfolger;
  }


  /**
   * Methode setzt die Variable m_anzeigenInhaltVorgaengerNachfolger
   * @param anzeigenInhaltVorgaengerNachfolger Wert auf den die Variable m_anzeigenInhaltVorgaengerNachfolger gesetzt werden soll
   */
  public void setAnzeigenInhaltVorgaengerNachfolger( WVZInhalt anzeigenInhaltVorgaengerNachfolger )
  {
    m_anzeigenInhaltVorgaengerNachfolger = anzeigenInhaltVorgaengerNachfolger;
  }


  /**
   * Methode liefert die Variable m_angeforderterAnzeigenInhalt zurck
   * @return Variable m_angeforderterAnzeigenInhalt
   */
  public WVZInhalt getAngeforderterAnzeigenInhalt()
  {
    return m_angeforderterAnzeigenInhalt;
  }


  /**
   * Methode setzt die Variable m_angeforderterAnzeigenInhalt
   * @param angeforderterAnzeigenInhalt Wert auf den die Variable m_angeforderterAnzeigenInhalt gesetzt werden soll
   */
  public void setAngeforderterAnzeigenInhalt( WVZInhalt angeforderterAnzeigenInhalt )
  {
    m_angeforderterAnzeigenInhalt = angeforderterAnzeigenInhalt;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaZusammenfassenProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
    
    Data anzeigenInhaltVorgaengerNachfolger = data.getItem( "AnzeigeinhaltVorgngerNachfolger" );
    m_anzeigenInhaltVorgaengerNachfolger.auswerteAttributgruppe( anzeigenInhaltVorgaengerNachfolger );
    
    Data angeforderterAnzeigenInhalt = data.getItem( "AngeforderterAnzeigeInhalt" );
    m_angeforderterAnzeigenInhalt.auswerteAttributgruppe( angeforderterAnzeigenInhalt );
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    s += "Y = " + Tools.getWVZInhalt( m_anzeigenInhaltVorgaengerNachfolger ) + "\n";
    s += "Z = " + Tools.getWVZInhalt( m_angeforderterAnzeigenInhalt ) + "\n";
    
    return s;
  }
}
