package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassungEnde;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

public class RegelZusammenfassungEnde
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;

  /**
   * Anzeigetyp A
   */
  private SystemObject m_anzeigeTypA;

  /**
   * Anzeigetyp B
   */
  private SystemObject m_anzeigeTypB;

  /**
   * Prioritt A
   */
  private int m_prioritaetA;

  /**
   * Prioritt B
   */
  private int m_prioritaetB;

  /**
   * Anzeigeninhalt (in der Regel: "A")
   */
  private List<WVZInhalt> m_anzeigenInhaltA = new ArrayList<WVZInhalt>();

  /**
   * Anzeigeninhalt (in der Regel: "B")
   */
  private List<WVZInhalt> m_anzeigenInhaltB = new ArrayList<WVZInhalt>();

  /**
   * Angeforderter Inhalt A
   */
  private WVZInhalt m_angeforderterInhaltA;

  /**
   * Angeforderter Inhalt B
   */
  private WVZInhalt m_angeforderterInhaltB;

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelZusammenfassungEnde()
  {
		m_anzeigeTypA = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel().getObject(AnzeigeElementTyp.WZG_A.getPid());
		
		m_anzeigeTypB = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel().getObject(AnzeigeElementTyp.WZG_B.getPid());
  }

  /**
   * Konstruktor der Klasse
   * @param schaltgrund
   * @param anzeigenInhaltA
   * @param anzeigenInhaltB
   * @param angeforderterInhaltA
   * @param angeforderterInhaltB
   * @param prioritaetA
   * @param prioritaetB
   */
  public RegelZusammenfassungEnde(Aspect schaltgrund, List<WVZInhalt> anzeigenInhaltA, List<WVZInhalt> anzeigenInhaltB,
  		WVZInhalt angeforderterInhaltA, WVZInhalt angeforderterInhaltB, int prioritaetA, int prioritaetB)
  {
		m_anzeigeTypA = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel().getObject(AnzeigeElementTyp.WZG_A.getPid());
		
		m_anzeigeTypB = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel().getObject(AnzeigeElementTyp.WZG_B.getPid());
		
  	m_schaltgrundRegel = schaltgrund;
  	
  	m_anzeigenInhaltA = anzeigenInhaltA;
  	m_anzeigenInhaltB = anzeigenInhaltB;

  	m_angeforderterInhaltA = angeforderterInhaltA;
  	m_angeforderterInhaltB = angeforderterInhaltB;
  	
  	m_prioritaetA = prioritaetA;
  	m_prioritaetB = prioritaetB;
  }

  /**
   * Methode zum Auswerten einer Regel der Attributliste
   * 'atl.sswSbaRegelZusammenfassungAufhebung'
   * 
   * @param data
   *          Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data , Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    if ( m_schaltgrundRegel == null )
      m_schaltgrundRegel = schaltgrundAllgemein;

//    m_anzeigeTypA = data.getReferenceValue( "AnzeigeTypA" ).getSystemObject();
//
//    m_anzeigeTypB = data.getReferenceValue( "AnzeigeTypB" ).getSystemObject();

    m_prioritaetA = data.getUnscaledValue( "PriorittA" ).intValue();

    m_prioritaetB = data.getUnscaledValue( "PriorittB" ).intValue();

    m_anzeigenInhaltA.clear();
    Array a1 = data.getArray( "AnzeigeinhaltTypA" );
    for (int i = 0; i < a1.getLength(); ++i)
    {
      WVZInhalt wi = new WVZInhalt();
      wi.auswerteAttributgruppe( a1.getItem( i ) );
      m_anzeigenInhaltA.add( wi );
    }

    m_anzeigenInhaltB.clear();
    Array a2 = data.getArray( "AnzeigeinhaltTypB" );
    for (int i = 0; i < a2.getLength(); ++i)
    {
      WVZInhalt wi = new WVZInhalt();
      wi.auswerteAttributgruppe( a2.getItem( i ) );
      m_anzeigenInhaltB.add( wi );
    }

    m_angeforderterInhaltA = new WVZInhalt();
    m_angeforderterInhaltA.auswerteAttributgruppe( data.getItem( "AngeforderterAnzeigeInhaltA" ) );

    m_angeforderterInhaltB = new WVZInhalt();
    m_angeforderterInhaltB.auswerteAttributgruppe( data.getItem( "AngeforderterAnzeigeInhaltB" ) );
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString()
  {
    String s = "";

    s += "AnzeigeTypA: " + m_anzeigeTypA + "\n";
    s += "AnzeigeTypB: " + m_anzeigeTypB + "\n";
    s += "PriorittA:  " + getPrioritaetA() + "\n";
    s += "PriorittB:  " + getPrioritaetB() + "\n";

    s += "AnzeigeinhaltA:" + "\n";
    for (WVZInhalt wi : m_anzeigenInhaltA)
      s += "-" + Tools.getWVZInhalt( wi ) + "\n";
    s += "\n";

    s += "AnzeigeinhaltB:" + "\n";
    for (WVZInhalt wi : m_anzeigenInhaltB)
      s += "-" + Tools.getWVZInhalt( wi ) + "\n";
    s += "\n";

    s += "AngefordertA:\n" + Tools.getWVZInhalt( getAngeforderterInhaltA() ) + "\n";
    s += "AngefordertB:\n" + Tools.getWVZInhalt( getAngeforderterInhaltB() ) + "\n";

    return s;
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * 
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * 
   * @param schaltgrundRegel
   *          Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel )
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }

  /**
   * Methode liefert die Variable m_anzeigeTypA zurck
   * 
   * @return Variable m_anzeigeTypA
   */
  public SystemObject getAnzeigeTypA()
  {
    return m_anzeigeTypA;
  }

//  /**
//   * Methode setzt die Variable m_anzeigeTypA
//   * 
//   * @param anzeigeTypA
//   *          Wert auf den die Variable m_anzeigeTypA gesetzt werden soll
//   */
//  public void setAnzeigeTypA( SystemObject anzeigeTypA )
//  {
//    m_anzeigeTypA = anzeigeTypA;
//  }

  /**
   * Methode liefert die Variable m_anzeigeTypB zurck
   * 
   * @return Variable m_anzeigeTypB
   */
  public SystemObject getAnzeigeTypB()
  {
    return m_anzeigeTypB;
  }

//  /**
//   * Methode setzt die Variable m_anzeigeTypB
//   * 
//   * @param anzeigeTypB
//   *          Wert auf den die Variable m_anzeigeTypB gesetzt werden soll
//   */
//  public void setAnzeigeTypB( SystemObject anzeigeTypB )
//  {
//    m_anzeigeTypB = anzeigeTypB;
//  }

  /**
   * Methode liefert die Variable m_prioritaetA zurck
   * 
   * @return Variable m_prioritaetA
   */
  public int getPrioritaetA()
  {
    return m_prioritaetA;
  }

  /**
   * Methode setzt die Variable m_prioritaetA
   * 
   * @param prioritaetA
   *          Wert auf den die Variable m_prioritaetA gesetzt werden soll
   */
  public void setPrioritaetA( int prioritaetA )
  {
    m_prioritaetA = prioritaetA;
  }

  /**
   * Methode liefert die Variable m_prioritaetB zurck
   * 
   * @return Variable m_prioritaetB
   */
  public int getPrioritaetB()
  {
    return m_prioritaetB;
  }

  /**
   * Methode setzt die Variable m_prioritaetB
   * 
   * @param prioritaetB
   *          Wert auf den die Variable m_prioritaetB gesetzt werden soll
   */
  public void setPrioritaetB( int prioritaetB )
  {
    m_prioritaetB = prioritaetB;
  }

  /**
   * Methode liefert die Variable m_anzeigenInhaltA zurck
   * 
   * @return Variable m_anzeigenInhaltA
   */
  public List<WVZInhalt> getAnzeigenInhaltA()
  {
    return m_anzeigenInhaltA;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhaltA
   * 
   * @param anzeigenInhaltA
   *          Wert auf den die Variable m_anzeigenInhaltA gesetzt werden soll
   */
  public void setAnzeigenInhaltA( List<WVZInhalt> anzeigenInhaltA )
  {
    m_anzeigenInhaltA = anzeigenInhaltA;
  }

  /**
   * Methode liefert die Variable m_anzeigenInhaltB zurck
   * 
   * @return Variable m_anzeigenInhaltB
   */
  public List<WVZInhalt> getAnzeigenInhaltB()
  {
    return m_anzeigenInhaltB;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhaltB
   * 
   * @param anzeigenInhaltB
   *          Wert auf den die Variable m_anzeigenInhaltB gesetzt werden soll
   */
  public void setAnzeigenInhaltB( List<WVZInhalt> anzeigenInhaltB )
  {
    m_anzeigenInhaltB = anzeigenInhaltB;
  }

  /**
   * Methode liefert die Variable m_angeforderterInhaltA zurck
   * 
   * @return Variable m_angeforderterInhaltA
   */
  public WVZInhalt getAngeforderterInhaltA()
  {
    return m_angeforderterInhaltA;
  }

  /**
   * Methode setzt die Variable m_angeforderterInhaltA
   * 
   * @param angeforderterInhaltA
   *          Wert auf den die Variable m_angeforderterInhaltA gesetzt werden
   *          soll
   */
  public void setAngeforderterInhaltA( WVZInhalt angeforderterInhaltA )
  {
    m_angeforderterInhaltA = angeforderterInhaltA;
  }

  /**
   * Methode liefert die Variable m_angeforderterInhaltB zurck
   * 
   * @return Variable m_angeforderterInhaltB
   */
  public WVZInhalt getAngeforderterInhaltB()
  {
    return m_angeforderterInhaltB;
  }

  /**
   * Methode setzt die Variable m_angeforderterInhaltB
   * 
   * @param angeforderterInhaltB
   *          Wert auf den die Variable m_angeforderterInhaltB gesetzt werden
   *          soll
   */
  public void setAngeforderterInhaltB( WVZInhalt angeforderterInhaltB )
  {
    m_angeforderterInhaltB = angeforderterInhaltB;
  }

}
