package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import java.util.Vector;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;

/**
 * Klasse liefert die Defaultwerte unabhngig von der SBA
 * @author ObertM
 *
 */
public class DefaultWerte
{
	/**
	 * Pid des Schaltgunds "Stau"
	 */
	public static String _pidStau = "asp.sswSbaStau";
	
	/**
	 * Schaltgrund "Stau"
	 */
	private Aspect _schaltgrundStau;

	/**
	 * Pid des Schaltgunds "Hand"
	 */
	public static String _pidHand = "asp.sswSbaHandschaltung";

	/**
	 * Schaltgrund "Hand"
	 */
	private Aspect _schaltgrundHand;

	/**
	 * Pid des Schaltgunds "Nsse"
	 */
	public static String _pidNaesse = "asp.sswSbaNssewarnung";

	/**
	 * Schaltgrund "Nsse"
	 */
	private Aspect _schaltgrundNaesse;
	
	public DefaultWerte () 
	{
    _schaltgrundHand   = Konfig.getInstance().getAspekt(_pidHand);

    _schaltgrundStau   = Konfig.getInstance().getAspekt(_pidStau);

    _schaltgrundNaesse = Konfig.getInstance().getAspekt(_pidNaesse);
	}
	
	/**
	 * Methode setzt die Defaultwerte
	 * @return
	 */
	public Prio [] getDefaultWerte (AnzeigeElementTyp typ)
  {
		Prio [] tmp = null;
		
		if (typ == AnzeigeElementTyp.WZG_A)
		  tmp = getDefaultWertTypA();

		if (typ == AnzeigeElementTyp.WZG_B)
		  tmp = getDefaultWerteTypB();

		if (typ == AnzeigeElementTyp.WZG_C)
		  tmp = getDefaultWerteTypC();

		if (typ == AnzeigeElementTyp.WZG_D)
		  tmp = getDefaultWertTypD();

		if (typ == AnzeigeElementTyp.BLINKER)
		  tmp = getDefaultWerteBlinker();

		if (typ == AnzeigeElementTyp.WZG_BAQ)
		  tmp = getDefaultWerteBAQ();

		if (typ == AnzeigeElementTyp.WZG_TAQ)
		  tmp = getDefaultWerteTAQ();

		if (typ == AnzeigeElementTyp.WZG_WAQ)
		  tmp = getDefaultWerteWAQ();

		if (typ == AnzeigeElementTyp.WZG_BAG)
		  tmp = getDefaultWerteBAG();

		Vector<Prio> v = new Vector<Prio>();
		for (Prio p : tmp)
		{
			AnzeigeInhaltTyp inhalt = p.getInhalt();
			if (VerwaltungWvzInhalte.getInstance().isVersorgt (typ, inhalt))
			  v.add(p);
		}
		
		return v.toArray(new Prio [v.size()]);
  }

	/**
	 * Methode setzt die Parameter von Typ A auf Defaultwerte
	 */
	private Prio [] getDefaultWertTypA()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_STAU, _schaltgrundHand),

				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40,       _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_50,       _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60,       _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80,       _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100,      _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120,      _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_130,      _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE,  _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_40,  _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_50,  _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_60,  _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_80,  _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_100, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_120, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_130, _schaltgrundHand),

				new Prio (AnzeigeInhaltTyp.DUNKEL,      _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL_STAU, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL_HP,   _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT),
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS),
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS),
				
				new Prio (AnzeigeInhaltTyp.DUNKEL,      _schaltgrundStau),
				new Prio (AnzeigeInhaltTyp.DUNKEL_STAU),
				new Prio (AnzeigeInhaltTyp.DUNKEL_HP),
				
				new Prio (AnzeigeInhaltTyp.TLS_STAU),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_50),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120),
				new Prio (AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_130),
				
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_40),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_50),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_60),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_80),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_100),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_120),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_130),
				new Prio (AnzeigeInhaltTyp.DUNKEL),
    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ B auf Defaultwerte
	 */
	private Prio [] getDefaultWerteTypB()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.TLS_GLAETTEGEFAHR, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_STAU, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_BAUSTELLE, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_SCHLEUDERGEFAHR, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_LKW, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.BAG_LKW, _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.DUNKEL,      _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL_STAU, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL_HP,   _schaltgrundHand),

				new Prio (AnzeigeInhaltTyp.DUNKEL,      _schaltgrundStau),
				new Prio (AnzeigeInhaltTyp.DUNKEL_STAU),
				new Prio (AnzeigeInhaltTyp.DUNKEL_HP),

				new Prio (AnzeigeInhaltTyp.TLS_GLAETTEGEFAHR),
				new Prio (AnzeigeInhaltTyp.TLS_STAU),
				new Prio (AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE, _schaltgrundStau),
				new Prio (AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE),
				new Prio (AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW, _schaltgrundNaesse),
				
				new Prio (AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN),
				new Prio (AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW),
				new Prio (AnzeigeInhaltTyp.TLS_BAUSTELLE),
				new Prio (AnzeigeInhaltTyp.TLS_SCHLEUDERGEFAHR),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN),
				new Prio (AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_LKW),
				new Prio (AnzeigeInhaltTyp.BAG_LKW),
				new Prio (AnzeigeInhaltTyp.DUNKEL),

    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ C auf Defaultwerte
	 */
	private Prio [] getDefaultWerteTypC()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.TLS_STAU,          _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_UNFALL,        _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_STAUGEFAHR,    _schaltgrundHand),

				new Prio (AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN, _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_NEBEL,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_NAESSE,        _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_GLAETTE,       _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_SMOG,          _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_SICHT,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_TIERE,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_KONTROLLE,     _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_OELSPUR,       _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_500M,          _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_1000M,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_1KM,           _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_1500M,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_1KOMMA5KM,     _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_2000M,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_2KM,           _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_2500M,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_2KOMMA5KM,     _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_3000M,         _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_3KM,           _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_AUF_2500M, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_2000M, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_1500M, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_1000M, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_500M, _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.DUNKEL,     _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL_STAU,_schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL_HP,  _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.DUNKEL,     _schaltgrundStau),
				new Prio (AnzeigeInhaltTyp.DUNKEL,     _schaltgrundNaesse),

				new Prio (AnzeigeInhaltTyp.DUNKEL_STAU),
				new Prio (AnzeigeInhaltTyp.DUNKEL_HP),

				new Prio (AnzeigeInhaltTyp.TLS_STAU),
				new Prio (AnzeigeInhaltTyp.TLS_UNFALL),
				new Prio (AnzeigeInhaltTyp.TLS_STAUGEFAHR),
				new Prio (AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN),
				
				new Prio (AnzeigeInhaltTyp.TLS_NEBEL),
				new Prio (AnzeigeInhaltTyp.TLS_NAESSE),
				new Prio (AnzeigeInhaltTyp.TLS_GLAETTE),
				new Prio (AnzeigeInhaltTyp.TLS_SMOG),

				new Prio (AnzeigeInhaltTyp.TLS_SICHT),
				new Prio (AnzeigeInhaltTyp.TLS_TIERE),
				new Prio (AnzeigeInhaltTyp.TLS_KONTROLLE),
				new Prio (AnzeigeInhaltTyp.TLS_OELSPUR),
				
				new Prio (AnzeigeInhaltTyp.TLS_500M),
				new Prio (AnzeigeInhaltTyp.TLS_1000M),
				new Prio (AnzeigeInhaltTyp.TLS_1KM),
				new Prio (AnzeigeInhaltTyp.TLS_1500M),
				new Prio (AnzeigeInhaltTyp.TLS_1KOMMA5KM),
				new Prio (AnzeigeInhaltTyp.TLS_2000M),
				new Prio (AnzeigeInhaltTyp.TLS_2KM),
				new Prio (AnzeigeInhaltTyp.TLS_2500M),
				new Prio (AnzeigeInhaltTyp.TLS_2KOMMA5KM),
				new Prio (AnzeigeInhaltTyp.TLS_3000M),
				new Prio (AnzeigeInhaltTyp.TLS_3KM),
				
				new Prio (AnzeigeInhaltTyp.TLS_AUF_2500M),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_2000M),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_1500M),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_1000M),
				new Prio (AnzeigeInhaltTyp.TLS_AUF_500M),

				new Prio (AnzeigeInhaltTyp.DUNKEL),
    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ Blinker auf Defaultwerte
	 */
	private Prio [] getDefaultWerteBlinker()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT,           _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_SYNCHRON,  _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_ASYNCHRON, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL, _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT),
				new Prio (AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_SYNCHRON),
				new Prio (AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_ASYNCHRON),
				new Prio (AnzeigeInhaltTyp.DUNKEL),
    };
		
		return prio;
  }
	
	/**
	 * Methode setzt die Parameter von Typ BAQ auf Defaultwerte
	 */
	private Prio [] getDefaultWerteBAQ()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.BAQ_BA_LKW_STOP, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.BAQ_BA_LKW,      _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL,          _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.BAQ_BA_LKW_STOP),
				new Prio (AnzeigeInhaltTyp.BAQ_BA_LKW),
				new Prio (AnzeigeInhaltTyp.DUNKEL),
    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ TAQ auf Defaultwerte
	 */
	private Prio [] getDefaultWerteTAQ()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.TAQ_NICHT_BEFAHREN, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TAQ_RAEUMEN,        _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TAQ_BEFAHREN,       _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TAQ_NEUTRAL,        _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.TAQ_NEUTRAL_S3,     _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TAQ_NICHT_BEFAHREN),
				new Prio (AnzeigeInhaltTyp.TAQ_RAEUMEN),
				new Prio (AnzeigeInhaltTyp.TAQ_BEFAHREN),
				new Prio (AnzeigeInhaltTyp.TAQ_NEUTRAL),
				new Prio (AnzeigeInhaltTyp.TAQ_NEUTRAL_S3),
    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ WAQ auf Defaultwerte
	 */
	private Prio [] getDefaultWerteWAQ()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_1, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_2, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_3, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_4, _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_1),
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_2),
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_3),
				new Prio (AnzeigeInhaltTyp.WAQ_SEITE_4),
    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ D2 auf Defaultwerte
	 */
	private Prio [] getDefaultWertTypD ()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_HALTEN, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL,                  _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_HALTEN),
				new Prio (AnzeigeInhaltTyp.DUNKEL),
    };
		
		return prio;
  }

	/**
	 * Methode setzt die Parameter von Typ BAG auf Defaultwerte
	 */
	private Prio [] getDefaultWerteBAG()
  {
		Prio [] prio = new Prio []
    {
				new Prio (AnzeigeInhaltTyp.BAG_LKW, _schaltgrundHand),
				new Prio (AnzeigeInhaltTyp.DUNKEL,  _schaltgrundHand),
				
				new Prio (AnzeigeInhaltTyp.BAG_LKW),
				new Prio (AnzeigeInhaltTyp.DUNKEL),
    };
		
		return prio;
  }
}
