package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Parameternderung ohne ButtonLeiste
 * 
 * @author Krause
 *
 */
public class CompositeAspekte extends Composite
{
	/**
	 * Text keine Diagnosedaten
	 */
	private final String _keineDiagnose = "  - keine Diagnosedaten -";
	
  /**
   * Liste mit den moeglichen Aspekten
   */
  private Vector<Aspect> _moeglicheAspekte = new Vector<Aspect>();  

  /**
   * Liste mit den angewaehlten Aspekten
   */
  private Vector<Aspect> _zugewieseneAspekte = new Vector<Aspect>();  

  /**
   * Moegliche Aspekte
   */
  private List _listeMoeglicherAspekte;

  /**
   * Zugewiesene Aspekte
   */
  private List _listeZugewiesenerAspekte;

  /**
   * Button nach rechts
   */
  private Button buttonNachRechts;

  /**
   * Button nach links
   */
  private Button buttonNachLinks;

  /**
   * Viewer mgliche Aspekte
   */
	private ListViewer _lvMoeglicheAspekte;

	/**
	 * Viewer zugewiesene Aspekte
	 */
	private ListViewer _lvZugewieseneAspekte;

	/**
	 * Combobox Ausgangsaspekt
	 */
	private Combo _comboAusgangsAspekt;

	/**
	 * Viewer Ausgangsaspekt
	 */
	private ComboViewer _cvAusgangsAspekt;

	/**
	 * Vektor mit den mglichen Aspekten
	 */
	private Vector<Aspect> _aspekte = new Vector<Aspect>();

	/**
	 * Combobox Diagnoseaspekt
	 */
	private Combo _comboDiagnoseAspekt;

	/**
	 * Viewer Diagnoseaspekt
	 */
	private ComboViewer _cvDiagnoseAspekt;

	/**
	 * Aktuelle Parameter
	 */
	private ParaAspekte _parameter;
	
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil
   * @param parameter aktuelle Parameter
   */
	CompositeAspekte(Composite parent, int style, ParameterSteuerungsKern parameter) 
  {
    super(parent, style);
    
    _parameter = parameter.getParaAspekte();
   
    initialize();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;

    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    bestimmeAspekte ();
    
    if (_parameter.getAspekteEingang() != null)
    {
      _zugewieseneAspekte.clear();
      for (String as : _parameter.getAspekteEingang())
      	_zugewieseneAspekte.add(Konfig.getInstance().getAspekt(as));
    }
    
    createCompositeEingangsAspekte(this);
    createCompositeAusgangsAspekte(this);
    createCompositeDiagnoseAspekte(this);
   
    initialisiereAspekte();
    
    aktualisiereRueckmeldungen ();
  }

  /**
   * Methode aktualisiert die Rckmeldungen
   */
  private void aktualisiereRueckmeldungen()
  {
  	Aspect ausgang = Konfig.getInstance().getAspekt(_parameter.getAspektAusgang());
  	if (ausgang != null)
  	{
		  String name = getBezeichnung(ausgang);
	    StructuredSelection ss = new StructuredSelection(name);
	    _cvAusgangsAspekt.setSelection(ss, true);
  	}

  	Aspect diagnose = Konfig.getInstance().getAspekt(_parameter.getAspektDiagnosedaten());
  	String name = _keineDiagnose;
  	if (diagnose != null)
		  name = getBezeichnung(diagnose);
  	
	  StructuredSelection ss = new StructuredSelection(name);
	  _cvDiagnoseAspekt.setSelection(ss, true);
  }

  /**
   * Methode bestimmt die im DaV definierten Aspekte
   */
	private void bestimmeAspekte()
  {
		TreeMap<String, Aspect> sorter = new TreeMap<String, Aspect>();

		java.util.List<SystemObject> objekte = Tools.getSystemObjekte("typ.aspekt");
    for (SystemObject so : objekte)
    {
    	Aspect a = (Aspect) so;
    	sorter.put(getBezeichnung(a), a);
    }
    
    for (Aspect a : sorter.values())
		  _aspekte.add(a);
  }

	/**
   * Methode fgt die Aspekte hinzu
   */
  private void initialisiereAspekte ()
  {
  	int c = 0;

    for (Aspect a : _aspekte)
    {
    	if (!_moeglicheAspekte.contains(a) && !_zugewieseneAspekte.contains(a))
    	{
    		if (true) // c < 20)
    		{
    		  _moeglicheAspekte.add(a);
    		  c++;
    		}
    	}
    }
  	
    _lvMoeglicheAspekte.refresh();
  }  

  /**
   * Methode legt ein Composite an in dem die Aspekte "Eingang" versorgt werden knnen
   * @param parent Parent Composite
   */
  private void createCompositeEingangsAspekte (Composite parent)
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    Group eingang = new Group(parent, SWT.BORDER);
  	eingang.setLayout(fillLayout1);
  	eingang.setLayoutData(gridData2);
  	eingang.setText("Eingangsaspekte:");

//
    // Zeile 0
    
    GridData gdZeile0 = new GridData ();
    gdZeile0.horizontalAlignment = GridData.FILL;
    gdZeile0.verticalAlignment = GridData.FILL;
    gdZeile0.grabExcessHorizontalSpace = true;
    gdZeile0.grabExcessVerticalSpace = false;

    Group zeile0 = new Group (eingang, SWT.NONE);
    zeile0.setLayout(new GridLayout (3, false));
//    zeile0.setLayoutData(gdZeile0);
//  	zeile0.setText("Eingangsaspekte:");
  	
    // Zeile 0 Spalte 1
    
    GridData gdZeile0Spalte1 = new GridData ();
    gdZeile0Spalte1.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte1.verticalAlignment = GridData.FILL;
    gdZeile0Spalte1.grabExcessHorizontalSpace = true;
    gdZeile0Spalte1.grabExcessVerticalSpace = true;
    gdZeile0Spalte1.minimumWidth = 200;

    Composite zeile0Spalte1 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte1.setLayout(new FillLayout ());
    zeile0Spalte1.setLayoutData(gdZeile0Spalte1);

    // Zeile 0 Spalte 2

    GridData gdZeile0Spalte2 = new GridData ();
    gdZeile0Spalte2.horizontalAlignment = GridData.CENTER;
    gdZeile0Spalte2.grabExcessHorizontalSpace = false;
    gdZeile0Spalte2.minimumWidth = 80;

    Composite zeile0Spalte2 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte2.setLayout(new GridLayout (1, true));
    zeile0Spalte2.setLayoutData(gdZeile0Spalte2);

    // Zeile 0 Spalte 3

    GridData gdZeile0Spalte3 = new GridData ();
    gdZeile0Spalte3.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte3.verticalAlignment = GridData.FILL;
    gdZeile0Spalte3.grabExcessHorizontalSpace = true;
    gdZeile0Spalte3.grabExcessVerticalSpace = true;
    gdZeile0Spalte3.minimumWidth = 200;
//
    Composite zeile0Spalte3 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte3.setLayout(new GridLayout(1, false));
    zeile0Spalte3.setLayoutData(gdZeile0Spalte3);

    addMoeglicheAusloeser(zeile0Spalte1);
    
    addKopierenButtons(zeile0Spalte2);
    
    addZugewieseneAusloeser(zeile0Spalte3);
  }

  /**
   * Methode legt ein Composite an in dem die Aspekte "Ausgang" versorgt werden knnen
   * @param parent Parent Composite
   */
  private void createCompositeAusgangsAspekte (Composite parent)
  {
    // Zeile 0
    
    GridData gdZeile0 = new GridData ();
    gdZeile0.horizontalAlignment = GridData.FILL;
    gdZeile0.verticalAlignment = GridData.BEGINNING;
    gdZeile0.grabExcessHorizontalSpace = true;
    gdZeile0.grabExcessVerticalSpace = false;

    Group zeile0 = new Group (parent, SWT.NONE);
    zeile0.setLayout(new GridLayout (2, false));
    zeile0.setLayoutData(gdZeile0);
  	zeile0.setText("Ausgangsaspekt:");
  	
  	Label l = new Label(zeile0, SWT.NONE);
  	l.setText("Ausgangsaspekt:");
  	
	  _comboAusgangsAspekt = new Combo(zeile0, SWT.NONE);
		_cvAusgangsAspekt= new ComboViewer(_comboAusgangsAspekt);
		
		String[] namen = new String [_aspekte.size()];
		for (int x = 0; x < _aspekte.size(); ++x)
			namen [x] = getBezeichnung(_aspekte.get(x));
		
//		_cvAusgangsAspekt.setSorter(new ViewerSorter());
		_cvAusgangsAspekt.add(namen);
		if (_comboAusgangsAspekt.getItemCount() > 0)
		{
			_cvAusgangsAspekt.setSelection(new StructuredSelection(_cvAusgangsAspekt.getElementAt(0)), true);
		}

  }

  /**
   * Methode legt ein Composite an in dem die Aspekte "Diagnose" versorgt werden knnen
   * @param parent Parent Composite
   */
  private void createCompositeDiagnoseAspekte (Composite parent)
  {
    // Zeile 0
    
    GridData gdZeile0 = new GridData ();
    gdZeile0.horizontalAlignment = GridData.FILL;
    gdZeile0.verticalAlignment = GridData.BEGINNING;
    gdZeile0.grabExcessHorizontalSpace = true;
    gdZeile0.grabExcessVerticalSpace = false;

    Group zeile0 = new Group (parent, SWT.NONE);
    zeile0.setLayout(new GridLayout (2, false));
    zeile0.setLayoutData(gdZeile0);
  	zeile0.setText("Diagnoseaspekt:");

  	Label l = new Label(zeile0, SWT.NONE);
  	l.setText("Diagnoseaspekt:");
  	
	  _comboDiagnoseAspekt = new Combo(zeile0, SWT.NONE);
		_cvDiagnoseAspekt= new ComboViewer(_comboDiagnoseAspekt);
		
		String[] namen = new String [_aspekte.size() + 1];
		
		namen [0] = _keineDiagnose;
		for (int x = 0; x < _aspekte.size(); ++x)
			namen [x + 1] = getBezeichnung(_aspekte.get(x));
		
//		_cvDiagnoseAspekt.setSorter(new ViewerSorter());
		_cvDiagnoseAspekt.add(namen);
		if (_comboDiagnoseAspekt.getItemCount() > 0)
		{
			_cvDiagnoseAspekt.setSelection(new StructuredSelection(_cvDiagnoseAspekt.getElementAt(0)), true);
		}
  }

  /**
   * Fgt die Liste mit den mglichen Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addMoeglicheAusloeser (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Mgliche Aspekte");
    group.setLayout(new FillLayout(SWT.VERTICAL));

    _listeMoeglicherAspekte = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _listeMoeglicherAspekte.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
      	boolean anwahlMoeglich = true;
      	
     		anwahlMoeglich = _listeMoeglicherAspekte.getSelectionCount() != 0;
      	
        buttonNachRechts.setEnabled(anwahlMoeglich);
      }
    });
    
    _listeMoeglicherAspekte.addMouseListener(new MouseListener()
		{
			
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
        actionNachRechts();		
			}
		});
    
    _lvMoeglicheAspekte = new ListViewer(_listeMoeglicherAspekte);
    _lvMoeglicheAspekte.setContentProvider(new IStructuredContentProvider()
		{
			@Override
			public void dispose()
			{
			}
			
			@Override
			public Object[] getElements(Object inputElement)
			{
        Vector v = (Vector)inputElement;
        return v.toArray();
			}

			@Override
      public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
      {
      }
		});
    
    _lvMoeglicheAspekte.setInput(_moeglicheAspekte);
    
    _lvMoeglicheAspekte.setLabelProvider(new LabelProvider() 
    {
      public Image getImage(Object element) 
      {
        return null;
      }

      public String getText(Object element) 
      {
        return (getBezeichnung ((Aspect) element));
      }
    });
    
    _lvMoeglicheAspekte.setSorter(new ViewerSorter()
    {
      public int compare(Viewer viewer, Object e1, Object e2) 
      {
        return getBezeichnung ((Aspect)e1).compareTo(getBezeichnung((Aspect)e2));
      }
    });
  }

  /**
   * Fgt die Liste mit den zugewiesenden Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addZugewieseneAusloeser (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Zugeordnete Aspekte");
    group.setLayout(new FillLayout(SWT.VERTICAL));
    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    group.setLayoutData(gd);
   
    _listeZugewiesenerAspekte = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _listeZugewiesenerAspekte.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
        if (_listeZugewiesenerAspekte.getSelectionCount() != 0)
          buttonNachLinks.setEnabled(true);
        else
          buttonNachLinks.setEnabled(false);
      }
    });

    _listeZugewiesenerAspekte.addMouseListener(new MouseListener()
		{
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				actionNachLinks();
			}
		});
		
    _lvZugewieseneAspekte = new ListViewer(_listeZugewiesenerAspekte);
    _lvZugewieseneAspekte.setContentProvider(new IStructuredContentProvider()
		{
			@Override
			public void inputChanged (Viewer viewer, Object oldInput, Object newInput)
			{
			}
			
			@Override
			public void dispose()
			{
			}
			
			@Override
			public Object[] getElements(Object inputElement)
			{
        Vector v = (Vector)inputElement;
        return v.toArray();
			}
		});
    
    _lvZugewieseneAspekte.setInput(_zugewieseneAspekte);
    
    _lvZugewieseneAspekte.setLabelProvider(new LabelProvider() 
    {
      public Image getImage(Object element) 
      {
        return null;
      }

      public String getText(Object element) 
      {
        return (getBezeichnung ((Aspect) element));
      }
    });
    
    _lvZugewieseneAspekte.setSorter(new ViewerSorter()
    {
      public int compare(Viewer viewer, Object e1, Object e2) 
      {
        return getBezeichnung ((Aspect)e1).compareTo(getBezeichnung((Aspect)e2));
      }
    });

    if (_zugewieseneAspekte != null)
	    for (Aspect inhalt : _zugewieseneAspekte)
	    	if (_moeglicheAspekte.contains(inhalt))
	        _moeglicheAspekte.remove(inhalt);
    
    _lvMoeglicheAspekte.refresh();
    _lvZugewieseneAspekte.refresh();
  }

  /**
   * Fgt die Kopierbuttons hinzu
   * @param parent Composite in das die Buttons eingefgt werden sollen
   */
  private void addKopierenButtons (Composite parent)
  {
    Composite composite = new Composite (parent, SWT.NONE);
    
    buttonNachRechts = new Button(composite, SWT.PUSH);
    buttonNachRechts.setText(" Hinzufgen >>");
    buttonNachRechts.setEnabled(false);
    buttonNachRechts.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent e)
      {
        actionNachRechts();
      }
    });

    buttonNachLinks = new Button(composite, SWT.PUSH);
    buttonNachLinks.setText(" << Entfernen ");
    buttonNachLinks.setEnabled(false);
    buttonNachLinks.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent e)
      {
        actionNachLinks();
      }
    });

    /*
     * Layout
     */
    RowLayout rlUnten = new RowLayout(SWT.VERTICAL);
    rlUnten.pack = false;
    composite.setLayout(rlUnten);
  }

  /**
   * Kopiert die selektierten Eintrge von links nach rechts
   */
  private void actionNachRechts()
  {
    IStructuredSelection selection = (IStructuredSelection) _lvMoeglicheAspekte.getSelection();
    StructuredSelection ss = (StructuredSelection) selection;

  	for (int x = 0; x < ss.size(); ++x)
    {
      Object obj1 = ss.toArray()[x];

      if (obj1 instanceof Aspect)
      {
      	Aspect as = (Aspect) obj1;
      	
      	if (!_zugewieseneAspekte.contains(as))
      		_zugewieseneAspekte.add(as);
      	
      	_moeglicheAspekte.remove(as);
      }
    }

  	_lvMoeglicheAspekte.refresh();
  	_lvZugewieseneAspekte.refresh();
  	
    buttonNachRechts.setEnabled(false);
  }

  /**
   * Kopiert die selektierten Eintrge von rechts nach links
   */
  private void actionNachLinks()
  {
    IStructuredSelection selection = (IStructuredSelection) _lvZugewieseneAspekte.getSelection();
    StructuredSelection ss = (StructuredSelection) selection;

  	for (int x = 0; x < ss.size(); ++x)
    {
      Object obj1 = ss.toArray()[x];

      if (obj1 instanceof Aspect)
      {
      	Aspect as = (Aspect) obj1;
      	
      	if (!_moeglicheAspekte.contains(as))
          _moeglicheAspekte.add(as);
        
        _zugewieseneAspekte.remove(as);
      }
    }

    _lvMoeglicheAspekte.refresh();  
    _lvZugewieseneAspekte.refresh();
    	
    buttonNachLinks.setEnabled(false);
  }

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public ParaAspekte getParameter()
  {
	  return leseAktuelleEinstellungen();
  }
 
	private ParaAspekte leseAktuelleEinstellungen ()
	{
    ParaAspekte para = new ParaAspekte();
    
    /*
     * Eingang
     */
    Vector<String> pidsEingang = new Vector<String>();
    for (Aspect as : _zugewieseneAspekte)
    	pidsEingang.add(as.getPid());
    para.setAspekteEingang(pidsEingang);
    
    /*
     * Ausgang
     */
		ISelection selection = _cvAusgangsAspekt.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			if (ss.size() == 1)
			{
				Object firstElement = ss.getFirstElement();
				if (firstElement instanceof String)
				{
					String name = (String) firstElement;
					para.setAspektAusgang(getAspektByBezeichnung (name).getPid());
				}
			}
		}

    /*
     * Diagnosedaten
     */
		selection = _cvDiagnoseAspekt.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			if (ss.size() == 1)
			{
				Object firstElement = ss.getFirstElement();
				if (firstElement instanceof String)
				{
					String name = (String) firstElement;
					
					if (name.equals(_keineDiagnose))
					  para.setAspektDiagnosedaten(null);
					else
					  para.setAspektDiagnosedaten(getAspektByBezeichnung (name).getPid());
				}
			}
		}

  	return para;
  }

	/**
	 * Methode liefert einen Aspekt der ber eine Bezeichnung referenziert ist 
	 * @param bezeichnung Beszeichnung des Aspekts (siehe auch Methode getBezeichnung ())
	 * @return Aspekt
	 */
	private Aspect getAspektByBezeichnung (String bezeichnung)
  {
		if (bezeichnung.equals(_keineDiagnose))
			return null;
		
		for (Aspect a : _aspekte)
			if (getBezeichnung(a).equals(bezeichnung))
				return a;
		
	  return null;
  }

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
  {
  }
	
	/**
	 * Methode liefert eine Bezeichnung fr einen Aspekt. Die Bezeichnung setzt sich
	 * zusammen aus Name + Pid
	 * @param objekt Systemobjekt des Aspekts
	 * @return Bezeichnung 
	 */
	private String getBezeichnung (SystemObject objekt)
	{
		return objekt.getName() + " (" + objekt.getPid() + ")";
	}
}
