/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeQuerschnittTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Verriegelungsmatrix", Regel "Allgemeine Verriegelungsmatrix".<br>
 * Regel:<br>
 * Tritt von einer Anzeige aus in Vorgnger / Nachfolgerrichtung die Zeichenfolge "X", "Y", "Z",  auf 
 * (Lnge der Sequenz ist in der Parametrierung festlegbar), so sind an allen Vorgngern und Nachfolgern
 * der Anzeige in Querrichtung (ber beliebig viele Ebenen) alle Anforderungen als entfernt zu kennzeichnen 
 * und es ist eine Betriebsmeldung abzusetzen. 
 * @version $Revision: 1.2 $ / $Date: 2013/06/27 13:36:26 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaAllgemeineVerriegelungsMatrix
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaAllgemeineVerriegelungsMatrix _instance;

  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaAllgemeineVerriegelungsMatrix";
  
  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz fr die Anlage
   */
  private HashMap<EAnzeigeQuerschnittTyp, RegelnProAnzeigeTyp> m_parameterAnlage = new HashMap<EAnzeigeQuerschnittTyp, RegelnProAnzeigeTyp>();
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Parametersatz pro Anzeige
   */
  private HashMap<SystemObject, RegelnProAnzeigeTyp> m_parameterProAnzeige = new HashMap<SystemObject, RegelnProAnzeigeTyp>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der Anlage
   */
  private String m_anlage;
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaVerriegelung";

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaAllgemeineVerriegelungsMatrix ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaAllgemeineVerriegelungsMatrix getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaAllgemeineVerriegelungsMatrix.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaAllgemeineVerriegelungsMatrix ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(toString());
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaAllgemeineVerriegelungsMatrix'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_parameterAnlage.clear();
        m_parameterProAnzeigeQuerschnitt.clear();
        m_parameterProAnzeige.clear();
        
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
        
        a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
        
        a = data.getArray( "Anzeige");
        auswerteAttributgruppeAnzeige (a);
      }
    }
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
  	RegelnProAnzeigeTyp alleRegeln = new RegelnProAnzeigeTyp();
  	alleRegeln.auswerteAttributgruppe( array , getSchaltgrundAllgemein());
  	
  	bestimmeRegelnProQuerschnittTyp (alleRegeln);
  	
//    m_parameterAnlage.auswerteAttributgruppe( array , getSchaltgrundAllgemein());
  }

	/**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeQuerschnittRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeigeQuerschnitt.put (anzeigeQuerschnitt, regeln);
      }
    }
  }

  /**
   * Methode zum Auswerten der Anzeige spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeige
   */
  private void auswerteAttributgruppeAnzeige (Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeige)
    {
      m_parameterProAnzeige.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeige = d1.getReferenceValue( "Anzeige" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeRegeln" );
        
        RegelnProAnzeigeTyp regeln  = new RegelnProAnzeigeTyp ();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );
        
        m_parameterProAnzeige.put (anzeige, regeln);
      }
    }
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Parameter Allgemeine Verriegelungsmatrix\n";
    s += "========================================\n";
    
    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterAnlage.toString();
    
    s += "Parameter pro Anzeigequerschnitt:\n";
    s += "=================================\n";
    Iterator<Entry<SystemObject,RegelnProAnzeigeTyp>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject,RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = (SystemObject) entry.getKey();
      RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
      
      s += Tools.getObjektName( anzeigeQuerschnitt )  + "\n";
      s += regeln.toString();
    }

    s += "Parameter pro Anzeige:\n";
    s += "======================\n";
    it = m_parameterProAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject,RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = (SystemObject) entry.getKey();
      RegelnProAnzeigeTyp regeln = (RegelnProAnzeigeTyp) entry.getValue();
      
      s += Tools.getObjektName( anzeige )  + "\n";
      s += regeln.toString();
    }

    return s;
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public RegelnProAnzeigeTyp getParameterAnlage(EAnzeigeQuerschnittTyp typ)
  {
    return m_parameterAnlage.get(typ);
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeigeQuerschnitt zurck
   * @return Variable m_parameterProAnzeigeQuerschnitt
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeigeQuerschnitt()
  {
    return m_parameterProAnzeigeQuerschnitt;
  }

  /**
   * Methode setzt die Variable m_parameterProAnzeigeQuerschnitt
   * @param parameterProAnzeigeQuerschnitt Wert auf den die Variable m_parameterProAnzeigeQuerschnitt gesetzt werden soll
   */
  public void setParameterProAnzeigeQuerschnitt( HashMap<SystemObject, RegelnProAnzeigeTyp> parameterProAnzeigeQuerschnitt )
  {
    m_parameterProAnzeigeQuerschnitt = parameterProAnzeigeQuerschnitt;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeige zurck
   * @return Variable m_parameterProAnzeige
   */
  public HashMap<SystemObject, RegelnProAnzeigeTyp> getParameterProAnzeige()
  {
    return m_parameterProAnzeige;
  }

  /**
   * Methode setzt die Variable m_parameterProAnzeige
   * @param parameterProAnzeige Wert auf den die Variable m_parameterProAnzeige gesetzt werden soll
   */
  public void setParameterProAnzeige( HashMap<SystemObject, RegelnProAnzeigeTyp> parameterProAnzeige )
  {
    m_parameterProAnzeige = parameterProAnzeige;
  }

  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, RegelnProAnzeigeTyp regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
    	return;
//      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    			m_parameterProAnzeigeQuerschnitt.put(so, regeln);

    		if (so.getType().getPid().equals("typ.anzeige"))
    			m_parameterProAnzeige.put(so, regeln);
    	}
    }
    
    benachrichtigeListener();
  }

  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(EAnzeigeQuerschnittTyp typ, RegelnProAnzeigeTyp regeln)
  {
    m_parameterAnlage.put(typ, regeln);
    benachrichtigeListener();
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }
  
  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    bereinigeParameter();
    
    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    Array anzeigeQuerschnitt = data.getArray( "AnzeigeQuerschnitt" );
    erzeugeAnzeigeQuerschnittRegeln (anzeigeQuerschnitt);
    
    Array anzeige = data.getArray( "Anzeige" );
    erzeugeAnzeigeRegeln (anzeige);
    
    return data;
  }

  /**
   * Methode lscht leere Parameterfelder. Dies ist notwendig, da sonst die Steuerung Problem
   * mit der Bestimmung der richtigen Parameterstze hat
   */
  private void bereinigeParameter()
  {
  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeigeQuerschnitt());

  	bereinigeRegelProAnzeigeTyp(getParameterProAnzeige());
  }
  
  /**
   * Methode bereinigt die Parameter. Dabei werden Anzeigetypen fr die keine Regeln definiert sind
   * aus dem Parametersatz entfern.
   * @param regelnProAnzeigeTyp zu bereinigende Regeln
   */
  private void bereinigeRegelProAnzeigeTyp (HashMap<SystemObject, RegelnProAnzeigeTyp> regelnProAnzeigeTyp)
  {
  	List<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
  	
	  Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = regelnProAnzeigeTyp.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<SystemObject,RegelnProAnzeigeTyp> entry = it.next();
	  	
	  	SystemObject anzeigeQuerschnitt = entry.getKey();
	  	RegelnProAnzeigeTyp regeln = entry.getValue();
	  	
	  	regeln.bereinigeParameter();
	  	if (regeln.getParameter().size() == 0)
	  		zuLoeschen.add (anzeigeQuerschnitt);
	  }

	  for (SystemObject so : zuLoeschen)
	  	regelnProAnzeigeTyp.remove(so);
  }

  /**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    SystemObject wzgTyp = Konfig.getInstance().getWZGTypByPid(AnzeigeElementTyp.WZG_A.getPid());

  	RegelnProAnzeigeTyp tmp = new RegelnProAnzeigeTyp();
  	ListeMitRegeln paraTmp = tmp.getParameter(wzgTyp);
  	
  	Iterator<Entry<EAnzeigeQuerschnittTyp, RegelnProAnzeigeTyp>> it = m_parameterAnlage.entrySet().iterator();
  	while (it.hasNext())
  	{
  		Entry<EAnzeigeQuerschnittTyp, RegelnProAnzeigeTyp> entry = it.next();
  		
  		EAnzeigeQuerschnittTyp typ = entry.getKey();
  		RegelnProAnzeigeTyp rpat = entry.getValue();

  		ListeMitRegeln para = rpat.getParameter(wzgTyp);
  		if (para != null)
  		{
  		  Vector<RegelAllgemeineVerriegelungsMatrix> regeln = para.getParameter();
  		  if (regeln != null)
  		  {
  		  	for (RegelAllgemeineVerriegelungsMatrix regel : regeln)
  		  	{
  		  		regel.setKennung(typ.getName());
  		  		paraTmp.addRegel(regel);
  		  	}
  		  }
  		}
  	}
  	
    erzeugeRegeln( array, tmp );
  }

  /**
   * Methode schreibt die Anzeigequerschnittregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeQuerschnittRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAq = getParameterProAnzeigeQuerschnitt();
    int anzahlAQ = paraAq.size();
    
    array.setLength( anzahlAQ );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAq.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeigeQuerschnitt = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeQuerschnittRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Anzeigeregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnzeigeRegeln( Array array )
  {
    HashMap<SystemObject, RegelnProAnzeigeTyp> paraAnzeige = getParameterProAnzeige();
    int anzahlAnzeigen = paraAnzeige.size();
    
    array.setLength( anzahlAnzeigen );
    
    int i=0;
    Iterator<Entry<SystemObject, RegelnProAnzeigeTyp>> it = paraAnzeige.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, RegelnProAnzeigeTyp> entry = it.next();
      
      SystemObject anzeige = entry.getKey();
      RegelnProAnzeigeTyp rpat = entry.getValue();
      
      Data dataAQ = array.getItem( i++ );
      
      dataAQ.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeRegeln" );
      
      erzeugeRegeln (arrayRegeln, rpat);
    }
  }

  /**
   * Methode schreibt die Regeln pro Anzeigetyp in ein Array
   * @param array Array in das die Regeln geschrieben werden solle
   * @param rpat Regeln
   */
  private void erzeugeRegeln( Array array, RegelnProAnzeigeTyp rpat )
  {
    HashMap<SystemObject, ListeMitRegeln> p = rpat.getParameter();
    int anzahl = p.size();
    
    array.setLength( anzahl );
    
    int i=0;
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = p.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, ListeMitRegeln> entry = it.next();
      
      SystemObject anzeigeTyp = entry.getKey();
      ListeMitRegeln regeln = entry.getValue();
      
      Data aktuell = array.getItem( i++ );
      
      aktuell.getReferenceValue( "AnzeigeTyp" ).setSystemObject( anzeigeTyp );
      Array r = aktuell.getArray( "Regeln" );
      
      Vector<RegelAllgemeineVerriegelungsMatrix> parameter = regeln.getParameter();
      r.setLength( parameter.size() );
      
      for (int j=0; j<parameter.size(); ++j)
      {
        Data d = r.getItem( j );
        RegelAllgemeineVerriegelungsMatrix rv = parameter.get( j );
        
        d.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rv.getSchaltgrundRegel() );
        d.getUnscaledValue ( "VorgngerNachfolger" ).set (rv.getRichtung().getKodierung());
        d.getTextValue("Kennung").setText(rv.getKennung());
        
        Vector<WVZInhalt> inhalte = rv.getZuPruefendeZeichenFolge();
        Array a = d.getArray("Zeichenfolge");
        a.setLength(inhalte.size());
        
        for (int k=0; k<inhalte.size(); ++k)
        {
          WVZInhalt x = inhalte.get(k);
          x.erzeugeAttributgruppe( a.getItem(k) );
        }
      }
    }
  }

  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
	public RegelnProAnzeigeTyp getParameter(String pid)
  {
//    /*
//     * Parameter Anlage
//     */
//    if (pid.equals(_pidSBA))
//      return getParameterAnlage();
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parameter Anzeige
     */
    for (SystemObject so : m_parameterProAnzeige.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeige.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return new RegelnProAnzeigeTyp();
  }

	public RegelnProAnzeigeTyp getParameter(EAnzeigeQuerschnittTyp typ)
  {
		if (m_parameterAnlage.containsKey(typ))
			return m_parameterAnlage.get(typ);
		
	  return new RegelnProAnzeigeTyp();
  }

  private void bestimmeRegelnProQuerschnittTyp(RegelnProAnzeigeTyp alleRegeln)
  {
    SystemObject wzgTyp = Konfig.getInstance().getWZGTypByPid(AnzeigeElementTyp.WZG_A.getPid());

  	synchronized (m_parameterAnlage)
    {
    	m_parameterAnlage.clear();
    	
    	ListeMitRegeln para = alleRegeln.getParameter(wzgTyp);
    	for (RegelAllgemeineVerriegelungsMatrix p : para.getParameter())
    	{
    		String kennung = p.getKennung();
    		EAnzeigeQuerschnittTyp typ = EAnzeigeQuerschnittTyp.getTyp(kennung);
    		
    		if (typ != null)
    		{
	    		if (!m_parameterAnlage.containsKey(typ))
	    			m_parameterAnlage.put(typ, new RegelnProAnzeigeTyp());
	    		
	    		RegelnProAnzeigeTyp regeln = m_parameterAnlage.get(typ);
	    		ListeMitRegeln x = regeln.getParameter(wzgTyp);
	    		x.addRegel(p);
    		}
    	}
    }
  }
}
