package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse speichert den direkten Nachbarn und den nchsten Nachbarn vom selben Anzeigetyp
 * @author       Dambach Werke GmbH
 */
public class NachbarAnzeigen
{
  /**
   * Direkter Vorgnger der Anzeige
   */
  private SystemObject m_direkterVorgaenger;

  /**
   * Direkter Nachfolger der Anzeige 
   * (Achtung: eine Anzeige kann mehrere Nachfolger haben)
   */
  private SystemObject[] m_direkterNachfolger = new SystemObject [0];

  /**
   * Vorgnger vom gleichen Anzeigetyp der Anzeige
   */
  private SystemObject m_vorgaengerGleicherTyp;

  /**
   * Nachfolger vom gleichen Anzeigetyp der Anzeige
   * (Achtung: eine Anzeige kann mehrere Nachfolger haben)
   */
  private SystemObject[] m_nachfolgerGleichenTyps = new SystemObject [0];

  /**
   * Methode liefert die Variable direkterVorgaenger zurck
   * @return Variable direkterVorgaenger
   */
  public SystemObject getDirekterVorgaenger()
  {
    return m_direkterVorgaenger;
  }

  /**
   * Methode setzt die Variable direkterVorgaenger
   * @param direkterVorgaenger Wert fr direkterVorgaenger
   */
  public void setDirekterVorgaenger(SystemObject direkterVorgaenger)
  {
    m_direkterVorgaenger = direkterVorgaenger;
  }

  /**
   * Methode liefert die Variable direkterNachfolger zurck
   * @return Variable direkterNachfolger
   */
  public SystemObject[] getDirekterNachfolger()
  {
    return m_direkterNachfolger;
  }

  /**
   * Methode setzt die Variable direkterNachfolger
   * @param direkterNachfolger Wert fr direkterNachfolger
   */
  public void setDirekterNachfolger(SystemObject[] direkterNachfolger)
  {
    m_direkterNachfolger = direkterNachfolger;
  }

  /**
   * Methode liefert die Variable vorgaengerGleicherTyp zurck
   * @return Variable vorgaengerGleicherTyp
   */
  public SystemObject getVorgaengerGleicherTyp()
  {
    return m_vorgaengerGleicherTyp;
  }

  /**
   * Methode setzt die Variable vorgaengerGleicherTyp
   * @param vorgaengerGleicherTyp Wert fr vorgaengerGleicherTyp
   */
  public void setVorgaengerGleicherTyp(SystemObject vorgaengerGleicherTyp)
  {
    m_vorgaengerGleicherTyp = vorgaengerGleicherTyp;
  }

  /**
   * Methode liefert die Variable nachfolgerGleicherTyp zurck
   * @return Variable nachfolgerGleicherTyp
   */
  public SystemObject[] getNachfolgerGleicherTyp()
  {
    return m_nachfolgerGleichenTyps;
  }

  /**
   * Methode setzt die Variable nachfolgerGleicherTyp
   * @param nachfolgerGleicherTyp Wert fr nachfolgerGleicherTyp
   */
  public void setNachfolgerGleicherTyp(SystemObject[] nachfolgerGleicherTyp)
  {
    m_nachfolgerGleichenTyps = nachfolgerGleicherTyp;
  }

  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaVorgngerNachfolgerBeziehung'
   * @param data empfangender Datensatz
   */
  public void auswerteAttributgruppe( Data data )
  {
    m_direkterVorgaenger = data.getReferenceValue( "DirekterVorgnger" ).getSystemObject();
    
    ReferenceArray direkteNachfolger = data.getReferenceArray( "DirekteNachfolger" );
    int anzahl = direkteNachfolger.getLength();
    
    m_direkterNachfolger = new SystemObject [anzahl];
    for (int x = 0; x < anzahl; ++x)
      m_direkterNachfolger [x] = direkteNachfolger.getSystemObject( x );
      
    m_vorgaengerGleicherTyp = data.getReferenceValue( "VorgngerGleichenTyps" ).getSystemObject();
    
    ReferenceArray nachfolgerGleichentyps = data.getReferenceArray( "NachfolgerGleichenTyps" );
    anzahl = nachfolgerGleichentyps.getLength();
    
    m_nachfolgerGleichenTyps = new SystemObject [anzahl];
    for (int x = 0; x < anzahl; ++x)
      m_nachfolgerGleichenTyps [x] = nachfolgerGleichentyps.getSystemObject( x );
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Direkter Vorgnger: " + Tools.getObjektName( m_direkterVorgaenger ) + "\n";
    s += "Direkte Nachfolger:\n";
    if (m_direkterNachfolger.length == 0)
    {
      s += "-keine\n";
    }
    else
    {
      for (SystemObject nf : m_direkterNachfolger)
        s += "-" + Tools.getObjektName( nf ) + "\n";
    }

    s += "Vorgnger gleichen Typs: " + Tools.getObjektName( m_vorgaengerGleicherTyp ) + "\n";
    s += "Nachfolger gleichen Typs:\n";
    if (m_nachfolgerGleichenTyps.length == 0)
    {
      s += "-keine\n";
    }
    else
    {
      for (SystemObject nf : m_nachfolgerGleichenTyps)
        s += "-" + Tools.getObjektName( nf ) + "\n";
    }
    
    return s;
  }
  
  public NachbarAnzeigen getKopie ()
  {
    NachbarAnzeigen kopie = new NachbarAnzeigen();

    kopie.setDirekterVorgaenger(getDirekterVorgaenger());
    kopie.setVorgaengerGleicherTyp(getVorgaengerGleicherTyp());

    SystemObject[] nfgt = getNachfolgerGleicherTyp();
    SystemObject[] nfgtKopie = new SystemObject [nfgt.length];
    for (int i=0; i<nfgt.length; ++i)
    	nfgtKopie [i] = nfgt [i];
    
    kopie.setNachfolgerGleicherTyp(nfgtKopie);

    SystemObject[] dnf = getDirekterNachfolger();
    SystemObject[] dnfKopie = new SystemObject [dnf.length];
    for (int i=0; i<dnf.length; ++i)
    	dnfKopie [i] = dnf [i];
    
    kopie.setDirekterNachfolger(dnfKopie);

    return kopie;
  }
}
