package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;

/**
 * Klasse realisiert die verwendeten Anzeigetypen
 * @author ObertM
 *
 */
public class AnzeigeElementTyp
{
  private static List<AnzeigeElementTyp> elementTypen = new ArrayList<AnzeigeElementTyp>();

  public static AnzeigeElementTyp WZG_A       = new AnzeigeElementTyp(EAnzeigeTypen.TYP_WZG_A,  "WZG A");
  public static AnzeigeElementTyp WZG_B       = new AnzeigeElementTyp(EAnzeigeTypen.TYP_WZG_B,  "WZG B");
  public static AnzeigeElementTyp WZG_C       = new AnzeigeElementTyp(EAnzeigeTypen.TYP_WZG_C,  "WZG C");
  public static AnzeigeElementTyp WZG_D       = new AnzeigeElementTyp(EAnzeigeTypen.TYP_WZG_D,  "WZG D");
  public static AnzeigeElementTyp WZG_D2      = new AnzeigeElementTyp(EAnzeigeTypen.TYP_WZG_D2, "WZG D2");
  
  public static AnzeigeElementTyp WZG_BAQ     = new AnzeigeElementTyp(EAnzeigeTypen.TYP_BAQ,    "BAQ");
  public static AnzeigeElementTyp WZG_TAQ     = new AnzeigeElementTyp(EAnzeigeTypen.TYP_TAQ,    "TAQ");
  public static AnzeigeElementTyp WZG_WAQ     = new AnzeigeElementTyp(EAnzeigeTypen.TYP_WAQ,    "WAQ");
    
  public static AnzeigeElementTyp BLINKER     = new AnzeigeElementTyp(EAnzeigeTypen.TYP_BLINKER,     "Blinker");
  public static AnzeigeElementTyp FAIL_SAFE_A = new AnzeigeElementTyp(EAnzeigeTypen.TYP_FAIL_SAFE_A, "FailSave A");
  public static AnzeigeElementTyp INFO_A      = new AnzeigeElementTyp(EAnzeigeTypen.TYP_INFO_A,      "Info A");

  public static AnzeigeElementTyp WZG_BAG     = new AnzeigeElementTyp(EAnzeigeTypen.TYP_BAG,     "BAG");

  /**
   * Typ
   */
  private EAnzeigeTypen _typ;
  
  /**
   * Pid des Typs im DaV
   */
	private String _pid;

	/**
	 * Defaultbild das fr neue Regeln verwendet wird
	 */
	private AnzeigeInhaltTyp _defaultBild;

	/**
	 * Name des Typs
	 */
	private String _name;
	
	
	private AnzeigeElementTyp(EAnzeigeTypen typ, String name)
	{
		try
		{
//			System.out.println("AnzeigeElementTyp.AnzeigeElementTyp() " + typ + " " + name);
			_typ = typ;
			
			_name = name;
			
		  _pid = VerwaltungWvzInhalte.getInstance().getPidAnzeigeTyp (typ);

		  if (_pid == null)
		  	System.err.println("**** Keine Pid zu Anzeigetyp " + typ + " definiert ! ***");
		  
		  _defaultBild = VerwaltungWvzInhalte.getInstance().getDefaultBild (typ);
	
		  if (_defaultBild == null)
		  	System.err.println("**** Kein Defaultbild zu Anzeigetyp " + typ + " definiert ! ***");
//		  else
//		    System.out.println("AnzeigeElementTyp.AnzeigeElementTyp() " + typ + " Default: " + _defaultBild.getBildInhalt());
		  
		  elementTypen.add(this);
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}

	public static AnzeigeElementTyp getTypByPid (String pid)
	{
		for (AnzeigeElementTyp aet: elementTypen)
		{
			if (aet.getPid() != null)
				if (aet.getPid().equals(pid))
					return aet;
		}
		
		return null;
	}

	public static AnzeigeElementTyp getTypBySystemObject (SystemObject so)
	{
		if (so != null)
			return getTypByPid(so.getPid());
		
		return null;
	}

	public static AnzeigeElementTyp getTypByName (String name)
	{
		for (AnzeigeElementTyp aet: elementTypen)
		{
			if (aet.getName() != null)
				if (aet.getName().equals(name))
					return aet;
		}
		
		return null;
	}


	/**
   * Methode liefert pid zurck
   * @return pid
   */
  public String getPid()
  {
  	return _pid;
  }

	/**
   * Methode liefert defaultBild zurck
   * @return defaultBild
   */
  public AnzeigeInhaltTyp getDefaultBild()
  {
  	return _defaultBild;
  }

	/**
   * Methode liefert typ zurck
   * @return typ
   */
  public EAnzeigeTypen getTyp()
  {
  	return _typ;
  }
	
  /**
   * Methode liefert eine Liste mit den versorgten Typen
   * @return Liste
   */
  public static List<AnzeigeElementTyp> getVersorgteTypen ()
  {
  	List<AnzeigeElementTyp> l = new ArrayList<AnzeigeElementTyp>();
  	
  	for (AnzeigeElementTyp aet : elementTypen)
  		if (aet.getPid() != null)
  			l.add(aet);
  	
  	return l;
  }

	/**
   * Methode liefert name zurck
   * @return name
   */
  public String getName()
  {
  	return _name;
  }

	/* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
	  return _name;
  }
}