package de.bsvrz.sba.plugin.parameter.steuerungskern.tools;

import java.awt.FlowLayout;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;

import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;

/**
 * Klasse enthlt allgemeine Methoden um die Composites aufzubauen, die fr die Versorgung
 * der Parameter bentigt werden.
 * @author ObertM
 *
 */
public class CompositeTools
{
  /**
   * Methode erzeugt ein Composite um die Arbeitsrichtung anzuzeigen
   * @param table Tabelle in der das Composite erzeugt werden soll
   * @return erzeugtes Composite
   */
  public static Composite erzeugeCompositeCombo (Table table, String [] auswahl)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.CENTER;
    gd.verticalAlignment = GridData.CENTER;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    Composite comp = new Composite( table, SWT.NONE );
    comp.setBackground( table.getBackground() );
    comp.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp.setLayoutData( gd );
    comp.setLayout( new GridLayout() );

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;
    
    Combo combo = new Combo( comp, SWT.BORDER );
    combo.setLayoutData( gdTableItem );
    
    combo.setItems(auswahl);
    combo.select(0);
    
    return comp;
  }

  /**
   * Methode erzeugt ein Composite um die WVZ-Inhalte anzuzeigen
   * @param table Tabelle in der das Composite erzeugt werden soll
   * @param typ Anzeige Typ
   * @return erzeugtes Composite 
   */
  public static Composite erzeugeCompositeWVZ (Table table, final AnzeigeElementTyp typ)
  {
  	return erzeugeCompositeWVZ(table, typ, 1, 1);
  }
  
  /**
   * Methode erzeugt ein Composite um die WVZ-Inhalte anzuzeigen
   * @param table Tabelle in der das Composite erzeugt werden soll
   * @param typ Anzeige Typ
   * @param anzBilder Anzahl Bilder
   * @param anzBilderProZeile Anzahl Bilder pro Zeile
   * @return erzeugtes Composite 
   */
  public static Composite _erzeugeCompositeWVZ (Table table, final AnzeigeElementTyp typ, int anzBilder, int anzBilderProZeile)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.CENTER;
    gd.verticalAlignment = GridData.CENTER;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    Composite comp = new Composite( table, SWT.NONE );
    comp.setBackground( table.getBackground() );
    comp.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp.setLayoutData( gd );
    comp.setLayout( new FillLayout()) ;

    GridData gdWVZItem = new GridData();
    gdWVZItem.horizontalAlignment = GridData.FILL;
    gdWVZItem.grabExcessHorizontalSpace = true;
    gdWVZItem.grabExcessVerticalSpace = true;
    gdWVZItem.verticalAlignment = GridData.CENTER;

    for (int i=0; i<anzBilder; ++i)
    {
	    Label lbImage = new Label( comp, SWT.CENTER );
//	    lbImage.setLayoutData(gdWVZItem);
	    
	    lbImage.addMouseListener( new MouseAdapter()
	    {
				@Override
	      public void mouseDown( MouseEvent e )
	      {
	      	Tools.waehleZuPruefendesBildMitDontCare(e, typ);
	      }         
	    });
    }
    
    return comp;
  }

  /**
   * Methode erzeugt ein Composite um die WVZ-Inhalte anzuzeigen
   * @param table Tabelle in der das Composite erzeugt werden soll
   * @param typ Anzeige Typ
   * @param anzBilder Anzahl Bilder
   * @param anzBilderProZeile Anzahl Bilder pro Zeile
   * @return erzeugtes Composite 
   */
  public static Composite erzeugeCompositeWVZ (Table table, final AnzeigeElementTyp typ, int anzBilder, int anzBilderProZeile)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.CENTER;
    gd.verticalAlignment = GridData.CENTER;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    Composite comp = new Composite( table, SWT.NONE );
    comp.setBackground( table.getBackground() );
    comp.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp.setLayoutData( gd );
    comp.setLayout( new GridLayout(anzBilderProZeile, false) );

    GridData gdWVZItem = new GridData();
    gdWVZItem.horizontalAlignment = GridData.FILL;
    gdWVZItem.grabExcessHorizontalSpace = true;
    gdWVZItem.grabExcessVerticalSpace = true;
    gdWVZItem.verticalAlignment = GridData.CENTER;

    for (int i=0; i<anzBilder; ++i)
    {
	    Label lbImage = new Label( comp, SWT.CENTER );
	    lbImage.setLayoutData(gdWVZItem);
	    
	    lbImage.addMouseListener( new MouseAdapter()
	    {
				@Override
	      public void mouseDown( MouseEvent e )
	      {
	      	Tools.waehleZuPruefendesBildMitDontCare(e, typ);
	      }         
	    });
    }
    
    return comp;
  }

  /**
   * Methode erzeugt ein Composite mit dem Lschenbutton
   * @param table Tabelle in der das Composite erzeugt werden soll
   * @return erzeugtes Composite
   */
  public static Composite erzeugeCompositeLoeschen (Table table)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.CENTER;
    gd.verticalAlignment = GridData.CENTER;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    Composite comp = new Composite( table, SWT.NONE );
    comp.setBackground( table.getBackground() );
    comp.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp.setLayoutData( gd );
    comp.setLayout( new GridLayout() );
    
    Button b = new Button( comp, SWT.CHECK );
    
    return comp;
  }
  
  public static Composite erzeugeCompositeWVZmitPrio (Table table, final AnzeigeElementTyp typ)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.CENTER;
    gd.verticalAlignment = GridData.CENTER;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    Composite comp = new Composite( table, SWT.NONE );
    comp.setBackground( table.getBackground() );
    comp.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp.setLayoutData( gd );
    comp.setLayout( new GridLayout(1, false) );

    GridData gdWVZItem = new GridData();
    gdWVZItem.horizontalAlignment = GridData.FILL;
    gdWVZItem.grabExcessHorizontalSpace = true;
//    gdWVZItem.grabExcessVerticalSpace = true;
    gdWVZItem.verticalAlignment = GridData.CENTER;

    GridData gdSpinnerItem = new GridData();
    gdSpinnerItem.horizontalAlignment = GridData.FILL;
    gdSpinnerItem.grabExcessHorizontalSpace = true;
//    gdSpinnerItem.grabExcessVerticalSpace = true;
    gdSpinnerItem.verticalAlignment = GridData.CENTER;

    // Label mit dem WVZ-Inhalt
    
    Label lbImage = new Label( comp, SWT.CENTER );
    lbImage.setLayoutData(gdWVZItem);

    lbImage.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
      	Tools.waehleZuPruefendesBildMitDontCare(e, typ);
      }         
    });

    Label dummy = new Label(comp, SWT.CENTER);
    dummy.setText("Prioritt:");
    dummy.setLayoutData(gdSpinnerItem);
    
    // Spinner fr die Prioritt
    
    Spinner spinner = new Spinner( comp, SWT.BORDER );

    spinner.setMinimum(-1);
    spinner.setMaximum(Integer.MAX_VALUE);
    spinner.setDigits(0);
      
    String toolTip = "Prioritt des Anzeigeinhalts. Zulssig von  " + spinner.getMinimum() 
                     + " bis " + spinner.getMaximum();
     
    spinner.setToolTipText(toolTip);
    spinner.setLayoutData( gdSpinnerItem );
    
    
    return comp;
  }
}
