/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.awt.Frame;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.pat.sysbed.dataEditor.DataEditorPanel;

public class CompositeParameterEditor extends Composite
{

	private Group _groupAusgewaehlteObjekte = null;
	private Composite _compositeDarstellung = null;
	private Combo _comboObjekte = null;
	private ComboViewer _comboViewer = null;
	private ClientDavInterface _connection = null;

	public CompositeParameterEditor(Composite parent, int style, ClientDavInterface connection)
	{
		super(parent, style);

		this._connection = connection;

		initialize();
	}

	private void initialize()
	{
		createGroupAusgewaehlteObjekte();
		createCompositeDarstellung();
		setSize(new Point(300, 200));
		setLayout(new GridLayout());
	}

	/**
	 * This method initializes groupAusgewaehlteObjekte
	 * 
	 */
	private void createGroupAusgewaehlteObjekte()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		_groupAusgewaehlteObjekte = new Group(this, SWT.NONE);
		_groupAusgewaehlteObjekte.setLayout(new GridLayout());
		_groupAusgewaehlteObjekte.setLayoutData(gridData);
		createComboObjekte();
		_groupAusgewaehlteObjekte.setText("Ausgewhlte Objekte:");
	}

	/**
	 * This method initializes compositeDarstellung
	 * 
	 */
	private void createCompositeDarstellung()
	{
		GridData gridDataDarstellung = new GridData();
		gridDataDarstellung.horizontalAlignment = GridData.FILL;
		gridDataDarstellung.grabExcessHorizontalSpace = true;
		gridDataDarstellung.grabExcessVerticalSpace = true;
		gridDataDarstellung.verticalAlignment = GridData.FILL;
		// _compositeDarstellung = new Composite(this, SWT.BORDER);
		// _compositeDarstellung.setLayout(new GridLayout());
		// _compositeDarstellung.setLayoutData(gridDataDarstellung);
		_compositeDarstellung = new Composite(this, SWT.EMBEDDED
				| SWT.NO_BACKGROUND);
		Frame frame = SWT_AWT.new_Frame(_compositeDarstellung);
		_compositeDarstellung.setLayout(new GridLayout());
		_compositeDarstellung.setLayoutData(gridDataDarstellung);

		java.awt.Panel panel = new java.awt.Panel(new java.awt.BorderLayout());
		frame.add(panel);

		if (_connection != null)
		{
			DataEditorPanel dataEditorPanel = new DataEditorPanel(_connection);
			panel.add(dataEditorPanel);
			
			AttributeGroup atg = _connection.getDataModel().getAttributeGroup(
					"atg.anteilLkwFuzzy");
			Data data = _connection.createData(atg);
			dataEditorPanel.setData(data);
		}
	}

	/**
	 * This method initializes comboObjekte
	 * 
	 */
	private void createComboObjekte()
	{
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		_comboObjekte = new Combo(_groupAusgewaehlteObjekte, SWT.NONE);
		_comboObjekte.setLayoutData(gridData2);
		_comboViewer = new ComboViewer(_comboObjekte);
	}

}
