/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.pat.sysbed.dataEditor.DataEditorPanel;

public class CompositeParameterEditorNeu extends Composite
{
	/**
	 * Bereich fr die Darstellung. Wird bei Verwendung des
	 * {@link ScrolledComposite} nicht benstigt.
	 */
	private Composite _compositeDarstellung = null;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;
	// private DataEditorPanel _dataEditorPanel = null; // Bereich fr den JPanel
	// basierten Editor
	private DataEditorComposite _dataEditorPanel = null; // Bereich fr den
	// Composite basierten
	// Editor
	/**
	 * Gibt an, ob die Felder der der {@link CompositeParameterEditorNeu}
	 * editierbar sind, oder nicht.
	 */
	private boolean _editable = true;

	/**
	 * Klasse zur Kommunikation mit dem DaV.
	 */
	private DaVKommunikation _daVKommunikation = null;

	/**
	 * Kapsel mit den zu Bearbeitenden Daten
	 */
	private ParameterEditorKapsel _kapsel = null;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *          Basis-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum Dav
	 */
	public CompositeParameterEditorNeu(Composite parent, int style,
			ClientDavInterface connection)
	{
		super(parent, style);

		this._connection = connection;
		this._daVKommunikation = new DaVKommunikation(
				(ClientDavConnection) _connection);

		initialize();
	}

	private void initialize()
	{
		// createGroupAusgewaehlteObjekte();
		createCompositeDarstellung();
		setSize(new Point(300, 200));
		// setLayout(new GridLayout());
		setLayout(new FillLayout());
	}

	/**
	 * This method initializes compositeDarstellung
	 * 
	 */
	private void createCompositeDarstellung()
	{
		ScrolledComposite scrolledComposite = new ScrolledComposite(this,
				SWT.H_SCROLL | SWT.V_SCROLL);

		scrolledComposite.getVerticalBar().setIncrement(20);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);

		// GridData gridDataScrolled = new GridData();
		// gridDataScrolled.horizontalAlignment = GridData.FILL;
		// gridDataScrolled.grabExcessHorizontalSpace = true;
		// gridDataScrolled.grabExcessVerticalSpace = true;
		// gridDataScrolled.verticalAlignment = GridData.FILL;
		// scrolledComposite.setLayout(new GridLayout(1, false));
		// scrolledComposite.setLayoutData(gridDataScrolled);

		GridData gridDataDarstellung = new GridData();
		gridDataDarstellung.horizontalAlignment = GridData.FILL;
		gridDataDarstellung.grabExcessHorizontalSpace = true;
		gridDataDarstellung.grabExcessVerticalSpace = true;
		gridDataDarstellung.verticalAlignment = GridData.FILL;

		// _compositeDarstellung = new Composite(this, SWT.EMBEDDED
		// | SWT.NO_BACKGROUND);
		_compositeDarstellung = new Composite(scrolledComposite, SWT.EMBEDDED
				| SWT.NO_BACKGROUND);
		// _compositeDarstellung.setLayout(new GridLayout());
		// _compositeDarstellung.setLayoutData(gridDataDarstellung);

		// Frame frame = SWT_AWT.new_Frame(_compositeDarstellung);
		// java.awt.Panel panel = new java.awt.Panel(new java.awt.BorderLayout());
		// frame.add(panel);

		if (_connection != null)
		{
			// _dataEditorPanel = new DataEditorPanel(_connection);
			// panel.add(_dataEditorPanel);
			// _dataEditorPanel = new DataEditorComposite(_compositeDarstellung,
			// SWT.NONE, _connection);
			// _dataEditorPanel.setLayoutData(gridDataDarstellung);

			_dataEditorPanel = new DataEditorComposite(scrolledComposite, SWT.NONE,
					_connection);

//			AttributeGroup atg = _connection.getDataModel().getAttributeGroup(
//					"atg.anteilLkwFuzzy");
//			Data data = _connection.createData(atg);
//			_dataEditorPanel.setDataDav(data);
		}

		scrolledComposite.setContent(_dataEditorPanel);
		scrolledComposite.setMinSize(_dataEditorPanel.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));
	}

	/**
	 * Setzt die {@link ParameterEditorKapsel} mit den Infos zu den zu
	 * bearbeitenden Daten.
	 * 
	 * @param kapsel
	 *          {@link ParameterEditorKapsel}
	 */
	protected void setKapsel(ParameterEditorKapsel kapsel)
	{
		this._kapsel = kapsel;

		if (_kapsel != null)
		{			
			if (_kapsel.getDataEmpfangen() == null)
			{
				_kapsel.setDataEmpfangen(_daVKommunikation.empfangeDaten(_kapsel));
			}

			if (_kapsel.getDataEmpfangen() != null)
			{
				if (_dataEditorPanel != null)
				{
					_dataEditorPanel.setEditable(_editable);					
					_dataEditorPanel.setDataDav(_kapsel.getDataEmpfangen()
							.createModifiableCopy());
				}
			}

		}
	}

	/**
	 * Liefert die {@link ParameterEditorKapsel} mit den bearbeiteten Daten.
	 * 
	 * @return
	 */
	protected ParameterEditorKapsel getKapsel()
	{
		return _kapsel;
	}

	/**
	 * Sichert die durch die {@link ParameterEditorKapsel} definieren Daten,
	 * welche vom {@link DataEditorPanel} bearbeitet wurden, auf dem DaV. Zuvor
	 * sollte die Methode
	 * {@link CompositeParameterEditorNeu#synchronisiereBearbeitung()} aufgerufen
	 * worden sein!
	 */
	protected void sichereBearbeiteteDatenAufDaV()
	{
		if (_kapsel != null)
		{
			if (_daVKommunikation != null)
			{
				_daVKommunikation.sendeDaten(_kapsel);
			}
		}
	}

	/**
	 * Fhrt die Synchronisierung zwischen den Daten des Editors und der Kapsel
	 * durch.
	 */
	protected void synchronisiereBearbeitung()
	{
		if ((_kapsel != null) && (_dataEditorPanel != null))
		{
			_kapsel.setDataSenden(_dataEditorPanel.getDataDav());
		}
	}
	
	protected void setEditable(boolean editable)
	{
		this._editable = editable;
	}
	
	protected boolean isEditable()
	{
		return _editable;
	}

}
