/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.sonstiges.statusleiste;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.archive.ArchiveAvailabilityListener;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;

/**
 * Diese Klasse liefert ein {@link ContributionItem}, welches eine Uhr
 * beinhaltet und in der Statusleiste positioniert werden kann.
 * 
 * @author Pittner
 * 
 */
public class SymbolleisteArchivItem extends ContributionItem implements IDaVVerbindungZuhoerer
{
  /**
   * Textfeld zur Darstellung des Archivzustandes.
   */
  private Text _textArchiv = null;
  
  /**
   * Der Archiv-Listener
   */
  private ArchiveAvailabilityListener _archiveAvailabilityListener = null;
  
  /**
   * Das Parent-Composite
   */
  private Composite _parent;
  
  /**
   * Farbe Grn fr Archiv verfgbar
   */
  private final static Color farbeArchivOk = new Color(null, 96, 255, 96);
  
  /**
   * Farbe Rot fr Archiv nicht verfgbar
   */
  private final static Color farbeArchivGestoert = new Color(null, 255, 0, 0); 
  
  /**
   * Farbe Rot fr Archiv nicht verfgbar
   */
  private final static SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss"); 
  
  
  /**
   * Bildet den Archiv-Zustand ab
   * 
   * @author Pittner
   *
   */
  private static enum ArchivZustand{
    ARCHIV_VERFUEGBAR, 
    ARCHIV_NICHT_VERFUEGBAR
  };
 

  /**
   * Konstruktor. 
   * 
   * @param id
   *          ID des {@link ContributionItem}.
   */
  public SymbolleisteArchivItem(String id)
  {
    super(id);
    
    DaVVerbindung.getInstanz().hinzufuegenZuhoerer(this);
    
    _archiveAvailabilityListener = new ArchiveAvailabilityListener()
    {
      @Override
      public void archiveAvailabilityChanged( ArchiveRequestManager archive )
      {
        
        boolean available = archive.isArchiveAvailable();
        final ArchivZustand zustand = available ? ArchivZustand.ARCHIV_VERFUEGBAR :  ArchivZustand.ARCHIV_NICHT_VERFUEGBAR;        
        if (_parent != null){
          Display display = _parent.getDisplay();          
          if (display != null && !display.isDisposed()){
            display.asyncExec(new Runnable() {
              
              @Override
              public void run()
              {
                 aktualisiereStatusItem(zustand);
              }
            });          
          }          
        }        
      }
    };
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      ArchiveRequestManager archive = connection.getArchive();
      if (archive != null){        
        archive.addArchiveAvailabilityListener(_archiveAvailabilityListener);
      }
    }
  }
  
  
  

  /* (non-Javadoc)
   * @see org.eclipse.jface.action.ContributionItem#fill(org.eclipse.swt.widgets.Composite)
   */
  public void fill(final Composite parent)
  {
    _parent = parent;    
    _textArchiv = new Text(parent, SWT.BORDER);
    _textArchiv.setEditable(false);    
    aktualisiereArchivInformationen();
  }
 

  /* (non-Javadoc)
   * @see org.eclipse.jface.action.ContributionItem#dispose()
   */
  public void dispose()
  {    
    super.dispose();
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      ArchiveRequestManager archive = connection.getArchive();
      if (archive != null){        
        archive.removeArchiveAvailabilityListener(_archiveAvailabilityListener);
      }
    }
    DaVVerbindung.getInstanz().entferneZuhoerer(this);
  }

  @Override
  public void verbindungAufgebaut(DaVVerbindungEreignis e)
  {
    aktualisiereArchivInformationen();             
  }

  @Override
  public void verbindungAbgebaut(DaVVerbindungEreignis e)
  {
    if (_parent != null)
    {
      Display display = _parent.getDisplay();      
      if (display != null && !display.isDisposed()){
        display.asyncExec(new Runnable() {
          
          @Override
          public void run()
          {
            aktualisiereStatusItem(ArchivZustand.ARCHIV_NICHT_VERFUEGBAR);             
          }
        });  
      }
    }
  }
  
  
  /**
   * Holt die Informationen aus dem DaV und benachrichtigt den Archiv-Listener 
   */
  private void aktualisiereArchivInformationen(){
        
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();  
    if (connection != null){
      ArchiveRequestManager archive = connection.getArchive();
      if (archive != null){                
        _archiveAvailabilityListener.archiveAvailabilityChanged(archive);
      }      
    }
  }
  
  /**
   * Fhrt die nderung der Darstellung durch.
   * 
   * @param zustand
   *            der Zustand des Archivs
   */
  private void aktualisiereStatusItem(ArchivZustand zustand){
    if ((_textArchiv != null) && (!_textArchiv.isDisposed()))
    {
      String text = "";
      
      if (zustand.equals(ArchivZustand.ARCHIV_VERFUEGBAR)){
        text = "Archiv verfgbar";
        _textArchiv.setText(text); 
        _textArchiv.setBackground(farbeArchivOk);
      }else{
        text = "Archiv nicht verfgbar";
        _textArchiv.setText(text); 
        _textArchiv.setBackground(farbeArchivGestoert);      
      }
      
      _textArchiv.setToolTipText(text + " : " + _sdf.format(new Date()));
      
      _textArchiv.pack();
      _parent.layout();      
    }   
  }
}
