/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;

/**
 * Kommunikation fr folgende Anmeldung:
 * <ul>
 * <li>TYP: <code>typ.tsfAbschnitt</code></li>
 * <li>ATG: <code>atg.tsfZustandSteuerung</code></li>
 * <li>ASP: <code>asp.tsfZustand</code></li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFSteuerung implements ClientReceiverInterface
{
	private static DaVKomTSFSteuerung _instance = null;

	/**
	 * Verbindung DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * {@link DataDescription} fr die Lese-Anmeldung
	 */
	private LinkedList<DataDescription> _lDdLeseAnmeldungen = new LinkedList<DataDescription>();

	/**
	 * {@link SystemObject}s, auf welche die Leseanmeldung erfolgte.
	 */
	private LinkedList<SystemObject> _lSoLeseAnmeldung = new LinkedList<SystemObject>();

	/**
	 * Modell.
	 */
	private GUIModelTSFSteuerung _modelVorgabe = new GUIModelTSFSteuerung(
			Defs.ASP_TSF_VORGABE_ZUSTAND);
	private GUIModelTSFSteuerung _modelZustand = new GUIModelTSFSteuerung(
			Defs.ASP_TSF_ZUSTAND);

	private DaVKomTSFSteuerung()
	{
		initKommunikation();
	}

	private DaVKomTSFSteuerung(ClientDavInterface connection)
	{
		this._connection = connection;
		initKommunikation();
	}

	public static DaVKomTSFSteuerung getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFSteuerung.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFSteuerung();
				}
			}
		}

		return _instance;
	}

	public static DaVKomTSFSteuerung getInstance(ClientDavInterface connection)
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFSteuerung.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFSteuerung(connection);
				}
			}
		}

		return _instance;
	}

	private void initKommunikation()
	{
		_modelZustand.addListenerZustandsSammlungReset(_modelVorgabe);

		if (_connection == null)
		{
			this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		}

		if (_connection != null)
		{
			anmeldenEmpfang();
		} else
		{
			System.err
					.println("Es ist keine Kommunikation mglich, dar keine Verbindung zum DaV existent ist.");
		}
	}

	private void anmeldenEmpfang()
	{
		if (_lDdLeseAnmeldungen.size() == 0)
		{
			DataModel dataModel = _connection.getDataModel();
			SystemObjectType sot = dataModel.getType(Defs.TYP_TSF_ABSCHNITT);
			if (sot != null)
			{
				List<SystemObject> lSOTs = sot.getObjects();

				if (lSOTs.size() > 0)
				{
					Aspect asp = dataModel.getAspect(Defs.ASP_TSF_ZUSTAND);

					// TSFZustandArgoscan
					AttributeGroup atgTSFZustandArgoscan = dataModel
							.getAttributeGroup(Defs.ATG_TSF_ZUSTAND_ARGOSCAN);
					if ((atgTSFZustandArgoscan != null) && (asp != null))
					{
						DataDescription dd = new DataDescription(atgTSFZustandArgoscan, asp);
						_lDdLeseAnmeldungen.add(dd);
						_lSoLeseAnmeldung.addAll(lSOTs);

						_connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions
								.delta(), ReceiverRole.receiver());

					} else
					{
						System.err.println("ATG und ASP fr Anmeldung TSFZustandArgoscan");
					}

					// TSFZustandSteuerung
					AttributeGroup atgTSFZustandSteuerung = dataModel
							.getAttributeGroup(Defs.ATG_TSF_ZUSTAND_STEUERUNG);

					if ((atgTSFZustandSteuerung != null) && (asp != null))
					{
						DataDescription dd = new DataDescription(atgTSFZustandSteuerung,
								asp);
						_lDdLeseAnmeldungen.add(dd);
						_lSoLeseAnmeldung.addAll(lSOTs);

						_connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions
								.delta(), ReceiverRole.receiver());

					} else
					{
						System.err.println("ATG und ASP konnten nicht bestimmt werden.");
					}
					
					// TSFUrsachenEinheitInputAnalyse
//					AttributeGroup atgTSFUrsachenEinheitInputAnalyse = dataModel
//					.getAttributeGroup(Defs.ATG_TSF_URSACHENEINHEIT_INPUT_ANALYSE);
//					
//					asp = dataModel.getAspect(Defs.ASP_ANALYSE);
//					
//					if ((atgTSFZustandSteuerung != null) && (asp != null))
//					{
//					  SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaUeSeitenstreifenFreigabe");
//					  List<SystemObject> elements = type.getElements();
//					  
//					  DataDescription dd = new DataDescription(atgTSFUrsachenEinheitInputAnalyse,
//					      asp);
//					  _lDdLeseAnmeldungen.add(dd);
//					  _lSoLeseAnmeldung.addAll(elements);
//					  
//					  _connection.subscribeReceiver(this, elements, dd, ReceiveOptions
//					      .delta(), ReceiverRole.receiver());
//					  
//					} else
//					{
//					  System.err.println("ATG und ASP konnten nicht bestimmt werden.");
//					}

				} else
				{
					System.err.println("Es muss mindestens ein Objekt vom Typ "
							+ Defs.TYP_TSF_ABSCHNITT
							+ " existieren. Dies ist z.Zt. nicht der Fall!");
				}
			} else
			{
				System.err.println("Es konnten z.Zt. keine Objekte vom Typ "
						+ Defs.TYP_TSF_ABSCHNITT
						+ " bestimmt werden. Es ist keine Empfangsanmeldung mglich.");
			}
		} else
		{
			System.err.println("Es wurde bereits eine Leseanmeldung durchgefhrt.");
		}
	}

	private void abmeldenEmpfang()
	{
		if ((_connection != null) && (_lDdLeseAnmeldungen.size() > 0))
		{
			if (_lSoLeseAnmeldung.size() > 0)
			{

				for (DataDescription dd : _lDdLeseAnmeldungen)
				{
					_connection.unsubscribeReceiver(this, _lSoLeseAnmeldung, dd);
				}

			}
			this._lDdLeseAnmeldungen.clear();
			this._lSoLeseAnmeldung.clear();
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		if ((results != null) && (results.length > 0))
		{

			for (ResultData resultData : results)
			{

				if (resultData.hasData())
				{

					if (resultData.getDataDescription().getAttributeGroup().getPid()
							.equals(Defs.ATG_TSF_ZUSTAND_STEUERUNG))
					{
						Data data = resultData.getData();
						int iProgramm = data.getUnscaledValue("Programm").intValue();
						long lStufe = data.getUnscaledValue("Stufe").longValue();
						int iZustand = data.getUnscaledValue("Zustand").intValue();

						M_TSFZustandSteuerung zustandSteuerung = new M_TSFZustandSteuerung();
						zustandSteuerung.setSo(resultData.getObject());
						zustandSteuerung.setProgramm(E_TSFBefehl.getBefehl(iProgramm));
						zustandSteuerung.setStufe(lStufe);
						zustandSteuerung.setZustand(E_TSFSteuerungZustand
								.getBefehl(iZustand));
						zustandSteuerung.setTimestamp(new Date(resultData.getDataTime()));
						
//						System.out.println("update1" + zustandSteuerung.getSo().getName() + " : " + zustandSteuerung.getTimestamp());
						
						getModelVorgabe().putZustand(zustandSteuerung);
						getModelZustand().putZustand(zustandSteuerung);
					}

					if (resultData.getDataDescription().getAttributeGroup().getPid()
							.equals(Defs.ATG_TSF_ZUSTAND_ARGOSCAN))
					{
						M_TSFZustandArgoscan zustandArgoscan = new M_TSFZustandArgoscan(
								resultData);
						getModelVorgabe().putZustand(zustandArgoscan);
						getModelZustand().putZustand(zustandArgoscan);

						GUIModelArgoscanRuhezustand.getInstance().setZustandArgoscan(
								zustandArgoscan);
					}
					
//					if (resultData.getDataDescription().getAttributeGroup().getPid()
//					    .equals(Defs.ATG_TSF_URSACHENEINHEIT_INPUT_ANALYSE))
//					{					  
//					  SystemObject soAbs = holeAbschnittVonUrsachenEinheit(resultData.getObject(), _connection);					  
//					  AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup(Defs.ATG_TSF_ZUSTAND_STEUERUNG);
//					  Aspect asp2 = _connection.getDataModel().getAspect(Defs.ASP_TSF_ZUSTAND);
//					  ResultData resultData2 = _connection.getData(soAbs, new DataDescription(atg2, asp2), 1000);
//					  if (resultData2 != null){
//					    Data data2 = resultData2.getData();
//	            int iProgramm = data2.getUnscaledValue("Programm").intValue();
//	            long lStufe = data2.getUnscaledValue("Stufe").longValue();
//	            int iZustand = data2.getUnscaledValue("Zustand").intValue();
//					    
//					    M_TSFZustandSteuerung zustandSteuerung = new M_TSFZustandSteuerung();
//	            zustandSteuerung.setSo(resultData2.getObject());
//	            zustandSteuerung.setProgramm(E_TSFBefehl.getBefehl(iProgramm));
//	            zustandSteuerung.setStufe(lStufe);
//	            zustandSteuerung.setZustand(E_TSFSteuerungZustand
//	                .getBefehl(iZustand));
//	            zustandSteuerung.setTimestamp(new Date(resultData.getDataTime()));
//	            
////	            System.out.println("update2" + zustandSteuerung.getSo().getName() + " : " + zustandSteuerung.getTimestamp());
//	            
//	            getModelVorgabe().putZustand(zustandSteuerung);					  					  
//	            getModelZustand().putZustand(zustandSteuerung);					  
//					  }
//					}
				}
			}
		}
	}

	public void dispose()
	{
		abmeldenEmpfang();

		if (getModelVorgabe() != null)
		{
			getModelVorgabe().dispose();
		}

		if (getModelZustand() != null)
		{
			getModelZustand().dispose();
		}
	}

	public GUIModelTSFSteuerung getModelVorgabe()
	{
		return _modelVorgabe;
	}

	public GUIModelTSFSteuerung getModelZustand()
	{
		return _modelZustand;
	}
	
  public static SystemObject holeUrsachenEinheitVonAbschnitt(SystemObject so, ClientDavInterface connection)
  {
    SystemObjectType type = connection.getDataModel().getType("typ.sswSbaUeSeitenstreifenFreigabe");
    List<SystemObject> elements = type.getElements();

    SystemObject ue = null;

    for (SystemObject tmp : elements)
    {
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaTsfAbschnitte");
      Data cd = tmp.getConfigurationData(atg);
      ReferenceArray refArray = cd.getItem("Abschnitte").asReferenceArray();
      SystemObject[] systemObjectArray = refArray.getSystemObjectArray();
      for (SystemObject systemObject : systemObjectArray)
      {
        if (systemObject.equals(so))
        {
          ue = tmp;
        }
      }

    }
    return ue;
  }
  
  public static SystemObject holeAbschnittVonUrsachenEinheit(SystemObject so, ClientDavInterface connection)
  {
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaTsfAbschnitte");
    Data cd = so.getConfigurationData(atg);
    ReferenceArray refArray = cd.getItem("Abschnitte").asReferenceArray();
    SystemObject[] systemObjectArray = refArray.getSystemObjectArray();
    if (systemObjectArray.length > 0)
      return systemObjectArray[0];
    return null;
  }
  
  
  public ZustandSammlung[] holeZustandSammlungInitial(){
    
    ZustandSammlung[] re = null;
    
    SystemObjectType sot = _connection.getDataModel().getType("typ.sswSbaTsfAbschnitt");
    if (sot != null){
      List<SystemObject> elements = sot.getElements();
      List<ZustandSammlung> list = new ArrayList<ZustandSammlung>();
      for (SystemObject so : elements)
      {
        ZustandSammlung zustandSammlung = getModelZustand().getZustandSteuerungSammlung(so);
//        System.out.println("add->: " + zustandSammlung);
        list.add(zustandSammlung);
      }
            
      if (list.size() > 0)
        re = list.toArray(new ZustandSammlung[list.size()]);      
    }
    
    return re;
    
  }
  

}
