/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.Timer;
import java.util.TimerTask;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.PreferenceInit;
import de.sts.buv.plugin.workflow.tsf.dlg.DialogWFEinfach;
import de.sts.buv.plugin.workflow.tsf.dlg.IWiedervorlage;
import de.sts.buv.plugin.workflow.tsf.dlg.RegistryWiedervorlagen;

/**
 * Diese Klasse liefert einen Timer, der die Wiedervorlage des {@link DialogWFEinfach} fr einen TSFAbschnitt auslst.
 * 
 * 
 * 
 * @author HofmannJ
 * 
 */
public class WiedervorlageTSFWorkflowAbschnitt implements IWiedervorlage
{
  private M_TSFWorkflowAbschnitt _tsfWorkflowAbschnitt = null;

  private Timer _timer;

  private Long _startzeitpunkt;

  private Long _alternateDelayWiedervorlage = null;

  /**
   * 
   * @param tsfWorkflowAbschnitt
   * @param alternateDelayWiedervorlage
   *          Alternative Wiedervorlage in Sekunden. Wenn nicht gesetzt wird die aus den Prferenzen verwendet.
   */
  public WiedervorlageTSFWorkflowAbschnitt(M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt, Long alternateDelayWiedervorlage)
  {
    this._tsfWorkflowAbschnitt = tsfWorkflowAbschnitt;
    this._alternateDelayWiedervorlage = alternateDelayWiedervorlage;
    startTimer();
  }

  public M_TSFWorkflowAbschnitt getTsfWorkflowAbschnitt()
  {
    return _tsfWorkflowAbschnitt;
  }

  private void startTimer()
  {

    if ((getTsfWorkflowAbschnitt() != null) && (getTsfWorkflowAbschnitt().getAbschnitt() != null))
    {
      TimerTask task = new TimerTask() {

        @Override
        public void run()
        {
          aktionWiedervorlegen();
        }
      };
      _timer = new Timer("Timer Wiedervorlage Workflow Abschnitt "
          + getTsfWorkflowAbschnitt().getAbschnitt().getPidOrNameOrId());
      long delayWiedervorlageSchaltempfehlung = PreferenceInit.getDelayWiedervorlageSchaltempfehlung() * 1000l;

      _startzeitpunkt = new Long(System.currentTimeMillis());

      if (_alternateDelayWiedervorlage == null)
      {
        _timer.schedule(task, delayWiedervorlageSchaltempfehlung);
      }
      else
      {
        _timer.schedule(task, _alternateDelayWiedervorlage * 1000L);
      }

      RegistryWiedervorlagen.getInstance().addWiedervorlage(this);
    }
  }

  private void aktionWiedervorlegen()
  {
    if (_timer != null)
    {
      _timer.cancel();
      _timer = null;
    }

    /*
     * Prfen ob der WorkflowAbschnitt zu einem aktuell existierenden Workflow zugeordnet ist.
     */
    if ((getTsfWorkflowAbschnitt() != null) && (getTsfWorkflowAbschnitt().getTsfWorkflow() != null))
    {
      M_TSFWorkflow tsfWorkflow = getTsfWorkflowAbschnitt().getTsfWorkflow();

      SystemObject soWorkflow = tsfWorkflow.getSo();
      if (soWorkflow != null)
      {
        M_TSFWorkflow wfVorhanden = DaVKomTSFWorkflow.getInstance().getModel().getWorkflow(soWorkflow);
        if (wfVorhanden != null)
        {
          /*
           * Erneut darstellen. Zur Zeit nur der komplette Workflow mglich.
           */
          // DaVKomTSFWorkflow.getInstance().getModel().putWorkflow(
          // tsfWorkflow.duplicate());
          DaVKomTSFWorkflow.getInstance().getModel().putWorkflow(wfVorhanden.duplicate());
        }
      }
    }

    dispose();
  }

  private boolean pruefeObWfNochExitiert(M_TSFWorkflow alt)
  {
    boolean gleich = false;

    GUIModelTSFWorkflow model = DaVKomTSFWorkflow.getInstance().getModel();
    M_TSFWorkflow wfNeu = model.getWorkflow(alt.getSo());

    if (wfNeu != null)
    {
      gleich = wfNeu.isGleich(alt);
      System.err.println("=> ALTER UND NEUER WORKFLOW GLEICH/EXISTENT -> " + gleich);
    }

    return gleich;
  }

  private void stopTimer()
  {
    if (_timer != null)
    {
      _timer.cancel();
      _timer = null;
    }
  }

  private boolean isDisposed = false;

  public void dispose()
  {
    if (!isDisposed)
    {
      stopTimer();
      RegistryWiedervorlagen.getInstance().removeWiedervorlage(this);
      isDisposed = true;
    }
  }

  @Override
  public String getBeschreibung()
  {
    String re = null;

    if ((_tsfWorkflowAbschnitt != null) && (_tsfWorkflowAbschnitt.getAbschnitt() != null))
    {
      // re = "Wiedervorlage Schaltempfehlung Abschnitt "
      re = "Wiedervorlage Schaltempfehlung " + _tsfWorkflowAbschnitt.getAbschnitt().getNameOrPidOrId();
    }

    if (re == null)
    {
      re = "";
    }
    return re;
  }

  @Override
  public long getTimeout()
  {
    /*
     * Wird u.
     */
    long re = 0;

    re = PreferenceInit.getDelayWiedervorlageSchaltempfehlung();

    return re;
  }

  @Override
  public String getId()
  {
    String re = null;

    if ((_tsfWorkflowAbschnitt != null) && (_tsfWorkflowAbschnitt.getAbschnitt() != null))
    {
      re = "Schaltempfehlung " + _tsfWorkflowAbschnitt.getAbschnitt().getPid();
    }

    if (re == null)
    {
      re = "";
    }
    return re;
  }

  @Override
  public void vorzeitigeWiedervorlage()
  {
    if (_timer != null)
    {
      // _timer.cancel();

      System.out.println("vorzeitigeWiedervorlage(): " + getBeschreibung());

      aktionWiedervorlegen();
    }
  }

  @Override
  public Long getStartZeitpunkt()
  {
    return _startzeitpunkt;
  }
}
